//{$INCLUDE CadreDessinBGRARedessinEcran.inc}
// Include contenant la procédure TCadreDessinBGRA.RedessinEcran
// 20/08/2015: La procédure TCadreDessinBGRA.RedessinEcran(const DoGenererImage: boolean),
//             qui est très volumineuse, est déportée dans le fichier d'inclusion
//             CadreDessinBGRARedessinEcran.inc
// 25/08/2015: Echelle de couleurs pour les pentes
// 30/09/2015: Ajout du mode 'croix' pour les quadrillages

// Ce fichier est une dépendance de cdrdessincanvasBGRA.pas





// redessin écran
procedure TCadreDessinBGRA.RedessinEcran(const DoGenererImage: boolean);
var
  FDrwCurrentMode   : TModeSelectionEntites;
  FDrwCurrentIdxObj : integer;


  ii: integer;
  TmpBuffer: TGHCaveDrawDrawingContext;
  RF       : TRect;
  QEWE: TSymbole;
  QStyleCourbe: TStyleCourbe;
  NbGroupes: Integer;
  NumGroupe: Integer;
  MyGroupe: TGroupeEntites;
  NbCourbes: Integer;
  NbScraps: Integer;
  NbPolyLines: Integer;
  NbPolygones: Integer;
  NbSymboles: Integer;
  NbTextes: Integer;
  NbSimpleLignes: Integer;
  NbImages: Integer;
  MyImage: TImageObject;
  EWE: String;

  // dessin de la station courante
  procedure DessinerCurrentBaseStation();
  const
    RD = 8;
  begin
    TmpBuffer.DrawShape(FCurrentBasePoint.PosStation.X, FCurrentBasePoint.PosStation.Y, 0, RD, RD, clYellow, clRED);
  end;
  //----------------------------------------------------------------------------
  // dernière station topo trouvée (peut être différente de la station par défaut)
  procedure DrawLastStation(const BS: TBaseStation);
  const
    R = 8;
  var
    PM : TPoint2Df;
    PP : TPoint;
  begin
    AfficherMessage(self.ClassName, Format('DrawLastStation(%d)', [BS.IDStation]));
    if (BS.IDStation > 0) then
    begin
      try
        PM.X := BS.PosStation.X;
        PM.Y := BS.PosStation.Y;
        PP:= TmpBuffer.QGetCoordsPlan(PM);
        TmpBuffer.CanvasBGRA.Pen.Style   := psSolid;
        TmpBuffer.CanvasBGRA.Pen.Color   := clBlue;
        TmpBuffer.CanvasBGRA.Brush.Color := clAqua;
        TmpBuffer.CanvasBGRA.EllipseC(PP.X, PP.Y, R, R);
      except
        ;
      end;
    end;
  end;
  // dessin des objets temporaires (directement sur le canevas)
  procedure DrawTmpObj_FreeHandPoints;
  var
    i, Nb: Integer;
    PM: TPoint2Df;
    PP: TPoint;
  begin
    Nb := FCourbeFreehand.GetNbPoints();
    if (Nb > 0) then
    begin
      Vue.Canvas.Pen.Color := clGray;
      for i := 0 to Nb -1 do
      begin
        PM := FCourbeFreehand.GetPoint2d(i);
        PP := TmpBuffer.QGetCoordsPlan(PM);
        Vue.Canvas.EllipseC(PP.X, PP.Y, 2, 2);
      end;
    end;
  end;
  procedure DrawTmpObj_CourbeProvisoire;
  begin
    DessinerCourbeProvisoire(vue.Canvas);
  end;
  procedure DrawBoundingBoxGroupe(const GRP: TGroupeEntites);
  begin
    if (not GRP.Displayed) then Exit;
    try
      if (IsValidBoundingBox(GRP.BoundingBox)) then
      begin
        TmpBuffer.DefineCrayon(psSolid, 0, clMaroon, 255);
        TmpBuffer.DefineBrosse(bsClear, clWhite, 255);
        TmpBuffer.DrawRectangle(GRP.BoundingBox.C1, GRP.BoundingBox.C2);
        TmpBuffer.RestoreBrosse();
        TmpBuffer.RestoreCrayon()
      end;
    except
    end;
  end;

begin
  // sémaphore 'FRedessinInProcess' est armé ? = On quitte (le dessin est en cours)
  lbMessages.Caption := IIF(FRedessinInProcess, 'REDESS', 'DONE');
  if (FRedessinInProcess) then
  begin
    AfficherMessageErreur(Format('[FRedessinInProcess = %s] Redessin en cours', [BoolToStr(FRedessinInProcess)]));
    exit;
  end;
  // Armement du sémaphore
  FRedessinInProcess := True;
  try
    TmpBuffer:= TGHCaveDrawDrawingContext.Create(Vue.Width, Vue.Height, BGRA(Red(FBackGroundColor),
                                                 Green(FBackGroundColor),
                                                 Blue(FBackGroundColor),
                                                 255));
    //RF.Left:=Vue.Left;
    //RF.Top :=Vue.Top;
    //RF.Bottom:=Vue.Left+Vue.Height;
    //RF.Right :=Vue.Top+Vue.Width;
    begin
      try
        if (Not FDoDraw) then Exit;
        // paramétrage
        TmpBuffer.SetDocuTopo(FDocumentDessin);
        TmpBuffer.SetParamDessin(FElementsDessines, FPenteLimiteDisp, FTailleEchelle,
                                 FParametresGrilles,
                                 FDoDrawScrapsTransparents,
                                 FColorScrapTransparent);
        AfficherMessageErreur('RedessinEcran' + IntToHex(FColorScrapTransparent, 6));
        TmpBuffer.SetTextures(FTexturesPolygones);
        TmpBuffer.SetBounds(FRXMini, FRYMini, FRXMaxi, FRYMaxi);
        // début conventionnel
        TmpBuffer.BeginDrawing();
        AfficherMessage(ClassName, 'Redessiner: ' + IIF(FDoDraw, '', '** Pas de document chargé **'));
        //************************
        // le dessin de la cavité
        NbGroupes   := FDocumentDessin.GetNbGroupes();
        NbScraps    := FDocumentDessin.GetNbScraps();
        NbCourbes   := FDocumentDessin.GetNbCourbes();
        NbPolyLines := FDocumentDessin.GetNbPolylignes();
        NbPolygones := FDocumentDessin.GetNbPolygones();
        NbSimpleLignes := FDocumentDessin.GetNbSimpleLignes();
        NbSymboles  := FDocumentDessin.GetNbSymboles();
        NbTextes    := FDocumentDessin.GetNbTextes();
        // les images de fond
        if (tedIMAGES in FElementsDessines) then
        begin
          NbImages := FDocumentDessin.GetNbImages();
          if (NbImages > 0) then
          begin
            for ii := 0 to NbImages - 1 do TmpBuffer.DrawImage(FDocumentDessin.GetImage(ii));
          end;
        end;

        for NumGroupe := 0 to NbGroupes - 1 do
        begin
          FDrwCurrentMode := mseNONE;
          MyGroupe := FDocumentDessin.GetGroupeByIdx(NumGroupe);
          //AfficherMessage(Format('Groupe %d: %d - %s', [NumGroupe, MyGroupe.IDGroupeEntites, MyGroupe.NomGroupe]));
          // tracer le BBX du groupe
          // on teste si le groupe est partiellement visible dans la fenêtre
          // avant de le tracer (forte accélération du dessin)
          if (GroupeIsInView(MyGroupe) and (MyGroupe.Displayed)) then
          begin
            //AfficherMessage('-- Scraps');
            FDrwCurrentMode := mseSCRAPS;
            if (NbScraps > 0) then
            begin
              for ii:=0 to NbScraps - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerScrap(FDocumentDessin.GetScrap(ii), ii, False, MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := msePOLYGONES;
            if (NbPolygones > 0) then
            begin
              for ii:=0 to NbPolygones - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerPolygone(FDocumentDessin.GetPolygone(ii), ii, False, MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := mseCOURBES;
            if (NbCourbes > 0) then
            begin
              for ii:=0 to NbCourbes - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerCourbe(FDocumentDessin.GetCourbe(ii), ii, MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := msePOLYLIGNES;
            if (NbPolyLines > 0) then
            begin
              for ii:=0 to NbPolyLines - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerPolyLigne(FDocumentDessin.GetPolyligne(ii), ii, MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := mseLIGNES;
            if (NbSimpleLignes > 0) then
            begin
              for ii := 0 to NbSimpleLignes - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerSimpleLigne(FDocumentDessin.GetSimpleLigne(ii), false, MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := mseSYMBOLES;
            if (NbSymboles > 0) then   // Symboles OK
            begin
              for ii := 0 to NbSymboles - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerSymbole(FDocumentDessin.GetSymbole(ii), MyGroupe.IDGroupeEntites);
              end;
            end;
            FDrwCurrentMode := mseTEXTES;
            if ((tedTEXTES in FElementsDessines) and (NbTextes > 0)) then
            begin
              for ii := 0 to NbTextes - 1 do
              begin
                FDrwCurrentIdxObj := ii;
                if (MyGroupe.Displayed) then TmpBuffer.DessinerTexte(FDocumentDessin.GetTexte(ii), False, MyGroupe.IDGroupeEntites);
              end;
            end;
          end;
        end; // for NoGroupe ...
        //************************
        //AfficherMessage('-- Quadrillage');
        if (tedQUADRILLES in FElementsDessines) then
        begin
          TmpBuffer.DrawQuadrillage(FParametresGrilles, true)
        end;
        //AfficherMessage('-- centerlines');
        if (tedCENTERLINES in FElementsDessines) then
        begin
          for ii:= 0 to FDocumentDessin.GetNbBaseStations() - 1 do
            TmpBuffer.DessinerBaseStation(FDocumentDessin.GetBaseStation(ii), 12);
        end;
        // dessiner station courante
        DessinerCurrentBaseStation();
        // dessiner dernière station trouvée (peut être différente de la station courante)
        DrawLastStation(FLastStationFound);
        // et enfin l'échelle
        if (tedECHELLE_NORD in FElementsDessines) then TmpBuffer.DrawEchelleNord(Vue.Width, Vue.Height);
        // fin conventionnelle
        TmpBuffer.EndDrawing()
      except
        EWE := ChooseString(Ord(FDrwCurrentMode), [
                              'mseNONE',
                              'mseSCRAPS',
                              'mseCOURBES',
                              'msePOLYLIGNES',
                              'msePOLYGONES',
                              'mseLIGNES',
                              'mseSYMBOLES',
                              'mseTEXTES',
                              'mseIMAGES' ]);

        AfficherMessageErreur(Format('Mode: %s - Objet: %d', [EWE, FDrwCurrentIdxObj]));
      end;
    end;
    TmpBuffer.Draw(Vue.Canvas, 0, 0, True);

    if (DoGenererImage) then TmpBuffer.SaveToFile(FFichierImage);
  finally
    TmpBuffer.Free;
    FRedessinInProcess := False; // libération du sémaphore
    lbMessages.Caption := IIF(FRedessinInProcess, 'REDESS', 'DONE');
    // ne pas mettre cette ligne avant la libération du sémaphore
    Application.ProcessMessages;
  end;
  // on dessine ici les objets temporaires
  try
    DrawTmpObj_FreeHandPoints;   // dessin des points main levée
    DrawTmpObj_CourbeProvisoire;
  except
  end;
end;

//******************************************************************************

