unit CadreEntrance;
// Date: 16/05/2013
// Statut: OK
// 04/10/2013: Conversions UTF8 <> ANSI fixed
// 19/04/2018: Simplification et corrections
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Common,
  StructuresDonnees,
  CallDialogsStdVersion,
  ToporobotClasses2012,
  UnitEntitesExtended,
  Classes, SysUtils, FileUtil, Forms, Dialogs, Controls, StdCtrls, curredit;

type

  { TCdrEntreeCavites }

  TCdrEntreeCavites = class(TFrame)
    btnCallCalculette: TButton;
    btnGetCoordsFromIDTerrain: TButton;
    btnGetCoordsFromSerSt: TButton;
    editIDTerrain: TEdit;
    editCommentaires: TEdit;
    editNomEntree: TEdit;
    editPoint: TCurrencyEdit;
    editSerie: TCurrencyEdit;
    editXEntree: TCurrencyEdit;
    editYEntree: TCurrencyEdit;
    editZEntree: TCurrencyEdit;
    Label1: TLabel;
    lbIDTerrainEntree: TLabel;
    lbStationOfEntrance: TLabel;
    lbEntranceCoordinates: TLabel;
    lbEntranceName: TLabel;
    lbComments: TLabel;
    lbNumEntrance: TStaticText;
		lbCurrentEPSG: TStaticText;
    procedure btnGetCoordsFromIDTerrainClick(Sender: TObject);
    procedure btnCallCalculetteClick(Sender: TObject);
    procedure btnGetCoordsFromSerStClick(Sender: TObject);
  private
    FIdx     : integer;
    FDocTopo :  TToporobotStructure2012;
    FBDDEntites: TBDDEntites;
    procedure InitCaptions();
    procedure PutEntranceInForm(const E: TEntrance; const DoInitCaptions: boolean);
    { private declarations }
  public
    { public declarations }
    function Initialiser(const FD: TToporobotStructure2012; const FB: TBDDEntites; const QEntrance: TEntrance; const QIdxEntrance: integer): boolean;
    function GetEntrance(): TEntrance;
  end;

implementation

{$R *.lfm}


procedure TCdrEntreeCavites.InitCaptions();
begin
  lbEntranceName.Caption               := GetResourceString(rsCDR_ENTR_ENTRNAME);
  lbIDTerrainEntree.Caption            := GetResourceString(rsCDR_ENTR_ENTR_ID);

  lbEntranceCoordinates.Caption        := GetResourceString(rsCDR_ENTR_COORDINATES);
  lbStationOfEntrance.Caption          := GetResourceString(rsCDR_ENTR_STATOFENTR);
  btnCallCalculette.Caption            := GetResourceString(rsCDR_ENTR_CALLCALCULETTE);
  lbComments.Caption                   := GetResourceString(rsLBL_COMMENTS);
  btnGetCoordsFromIDTerrain.Caption    := GetResourceString(rsCDR_ENTR_BTN_GET_COORD_FROM_IDTERRAIN);
  btnGetCoordsFromSerSt.Caption        := GetResourceString(rsCDR_ENTR_BTN_GET_COORD_FROM_SER_ST);
end;


procedure TCdrEntreeCavites.btnCallCalculetteClick(Sender: TObject);
var
  QCoords: TPoint2Df;
  QResultatCalcul, QDeclimag: double;
begin
  QResultatCalcul := 0.00;
  QDeclimag       := 0.00;
  QCoords := MakeTPoint2Df(0.00, 0.00);
  if (CallCalculette(FDocTopo, '', 1, QCoords, QResultatCalcul, QDeclimag)) then
  begin
    editXEntree.Value := QCoords.X;
    editYEntree.Value := QCoords.Y;
  end;
end;

procedure TCdrEntreeCavites.btnGetCoordsFromSerStClick(Sender: TObject);
var
  EWE: TEntiteEtendue;
begin
  if (FBDDEntites.GetEntiteViseeFromSerSt(editSerie.AsInteger, editPoint.AsInteger, EWE)) then
  begin
    editXEntree.Value  := EWE.Une_Station_2_X;
    editYEntree.Value  := EWE.Une_Station_2_Y;
    editZEntree.Value  := EWE.Une_Station_2_Z;
    editIDTerrain.Text := EWE.oIDLitteral;
    editSerie.AsInteger:= EWE.Entite_Serie;
    editPoint.AsInteger:= EWE.Entite_Station;
  end
  else ShowMessage(GetResourceString(rsMATCHNOTFOUND));
end;

procedure TCdrEntreeCavites.btnGetCoordsFromIDTerrainClick(Sender: TObject);
var
  EWE: TEntiteEtendue;
begin
  if (FBDDEntites.FindStationByCle(true, trim(editIDTerrain.Text), EWE)) then
  begin
    editXEntree.Value  := EWE.Une_Station_2_X;
    editYEntree.Value  := EWE.Une_Station_2_Y;
    editZEntree.Value  := EWE.Une_Station_2_Z;
    editIDTerrain.Text := EWE.oIDLitteral;
    editSerie.AsInteger:= EWE.Entite_Serie;
    editPoint.AsInteger:= EWE.Entite_Station;
  end
  else
  begin
    ShowMessage(GetResourceString(rsMATCHNOTFOUND));
  end;
end;




procedure TCdrEntreeCavites.PutEntranceInForm(const E: TEntrance; const DoInitCaptions: boolean);
var
	EWE: TLabelSystemesCoordsEPSG;
begin;
  if (DoInitCaptions) then InitCaptions;

  lbNumEntrance.Caption := Format('%d', [FIdx]);
  editNomEntree.Text := _AnsiToLCLStr(E.eNomEntree);
  editIDTerrain.Text := _AnsiToLCLStr(E.eIDTerrain);

  editXEntree.Value  := E.eXEntree;
  editYEntree.Value  := E.eYEntree;
  editZEntree.Value  := E.eZEntree;
  editSerie.AsInteger:= E.eRefSer;
  editPoint.AsInteger:= E.eRefSt;
  editCommentaires.Text:=_AnsiToLCLStr(E.eObserv);
  EWE := FDocTopo.GetCodeEPSGSystemeCoordonnees();
  lbCurrentEPSG.Caption := Format('EPSG:%d (%s)', [EWE.CodeEPSG, EWE.NomEPSG]);
end;

function TCdrEntreeCavites.Initialiser(const FD: TToporobotStructure2012; const FB: TBDDEntites; const QEntrance: TEntrance; const QIdxEntrance: integer): boolean;

begin
  btnGetCoordsFromIDTerrain.Visible := true;
  btnGetCoordsFromSerSt.Visible     := true;
  result := False;
  FDocTopo    := FD;
  FBDDEntites := FB;
  PutEntranceInForm(QEntrance, True);
  btnGetCoordsFromIDTerrain.Enabled := Assigned(FBDDEntites);
  btnGetCoordsFromSerSt.Enabled     := Assigned(FBDDEntites);
  result := True;
end;

function TCdrEntreeCavites.GetEntrance(): TEntrance;
var
  E: TEntrance;
begin
  //E.eNumEntree   := StrToIntDef(editNumEntree.Text,0);
  E.eNomEntree   := _LCLStrToAnsi(Trim(editNomEntree.Text));
  e.eIDTerrain   := _LCLStrToAnsi(Trim(editIDTerrain.Text));
  E.eXEntree     := editXEntree.Value;
  E.eYEntree     := editYEntree.Value;
  E.eZEntree     := editZEntree.Value;
  E.eRefSer      := editSerie.AsInteger;
  E.eRefSt       := editPoint.AsInteger;
  E.eObserv      := _LCLStrToAnsi(trim(editCommentaires.Text));
  Result:=E;
end;

end.

