unit CadreExpe;
// Date: 16/05/2013
// Statut: OK
// 04/10/2013: Conversions UTF8 <> ANSI fixed
// 06/05/2016: Les dates sont saisies par un TDatePicker
// 02/01/2017: Les dates sont saisies via un TMaskdit. TDatePicker est une bouse.
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Common,
  StructuresDonnees,
  CallDialogsStdVersion,
  UnitClassPalette,
  ToporobotClasses2012,

  // pour la déclinaison magnétique
  {$IFDEF MSWINDOWS}
    UnitWrapperDeclimag,
  {$ENDIF}
  {$IFDEF LINUX}
    UnitWrapperDeclimagLinux,
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
    UnitWrapperDeclimagMacOSX,
  {$ENDIF}
  math, Classes, SysUtils, FileUtil, Forms, Dialogs, Controls, StdCtrls,
  Buttons, EditBtn, MaskEdit, curredit;

type

  { TCdrExpe }

  TCdrExpe = class(TFrame)
    btnCalcDeclimag: TButton;
    Button1: TButton;
    Button2: TButton;
    cmbModeDecl: TComboBox;
    cmbUnitDeclimag: TComboBox;
    editDateSeance: TDateEdit;
    editCouleur: TCurrencyEdit;
    editDeclimag: TCurrencyEdit;
    editSpeleometre: TEdit;
    editSpeleographe: TEdit;
    editNoExpe: TCurrencyEdit;
    lblColor: TLabel;
    lblObs: TLabel;
    lblDeclimag: TLabel;
    lblSpeleometre: TLabel;
    lblSpeleographe: TLabel;
    lblDate: TLabel;
    lblSeance: TLabel;
    editComment: TMemo;
    lbColorSeance: TStaticText;
    lbInfosDiverses: TStaticText;
    procedure btnCalcDeclimagClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure cmbModeDeclChange(Sender: TObject);
    procedure cmbUnitDeclimagChange(Sender: TObject);
    procedure editCouleurKeyPress(Sender: TObject; var Key: char);
    procedure editDateSeanceChange(Sender: TObject);
    procedure lbColorSeanceClick(Sender: TObject);
  private
    FMyDocTopo: TToporobotStructure2012;
    FExpe   : TExpe;
    procedure InitCaptions;
    procedure PutExpeInForm;
    procedure SetLbColor(const IdxColor: integer);

  public
    procedure SetDocTopo(const D: TToporobotStructure2012);
    procedure SetExpe(const E: TExpe; const DoInitCaptions: boolean);
    function  GetExpeFromForm(): TExpe;
  end;
  // TODO: Afficher dans editDeclimag la valeur de la déclinaison
  // calculée par WMM Geomag
  // TODO: Activer la possibilité d'utiliser le mode manuel pour la déclimag


implementation

{$R *.lfm}
// appelée une seule fois dans leurs feuilles respectives
// --> initialisation du cadre effectuée en même temps
procedure TCdrExpe.SetExpe(const E: TExpe; const DoInitCaptions: boolean);
begin
  FExpe := E;
  if (DoInitCaptions) then InitCaptions;
  PutExpeInForm;
end;

procedure TCdrExpe.lbColorSeanceClick(Sender: TObject);
var
  EWE: Integer;
begin
  EWE := SelectionCouleurToporobot(editCouleur.AsInteger);
  SetLbColor(EWE);
end;

procedure TCdrExpe.editCouleurKeyPress(Sender: TObject; var Key: char);
begin
  if (Key = #13) then SetLbColor(editCouleur.AsInteger);
end;

procedure TCdrExpe.editDateSeanceChange(Sender: TObject);
begin

end;

procedure TCdrExpe.btnCalcDeclimagClick(Sender: TObject);
var
  PT: TPoint2Df;
  ResultatCalcul,
  QLat, QLon, QDeclimag: double;
  DescEPSG: TLabelSystemesCoordsEPSG;
  MyEntrance: TEntrance;
  EWE: String;
begin
  QDeclimag := 0.00;
  DescEPSG := FMyDocTopo.GetCodeEPSGSystemeCoordonnees();
  // extraction du point zéro
  MyEntrance := FMyDocTopo.GetEntree(0);
  if (ConversionCoordonneesIsoleesEPSG(DescEPSG.CodeEPSG, 4326, MyEntrance.eXEntree, MyEntrance.eYEntree, QLat, QLon)) then
  begin
    EWE := format('%.2f, %.2f, %.2f - Lat = %.6f, Lon = %.6f - EPSG:%d',
                  [MyEntrance.eXEntree, MyEntrance.eYEntree, MyEntrance.eZEntree,
                   QLat, QLon,
                   DescEPSG.CodeEPSG]);
    lbInfosDiverses.Caption := EWE;
    if (CalcDeclimagIsolee(QLon, QLat, MyEntrance.eZEntree, editDateSeance.Date, QDeclimag)) then
    begin
      case cmbUnitDeclimag.ItemIndex of
        0: editDeclimag.Value := QDeclimag;
        1: editDeclimag.Value := degtograd(QDeclimag);
      end;
    end
    else
      lbInfosDiverses.Caption := 'Echec du calcul de la déclinaison magnétique';
  end
  else
  begin
    lbInfosDiverses.Caption := 'Calcul de la déclinaison impossible: réseau non géoréférencé';
  end;
end;

procedure TCdrExpe.Button1Click(Sender: TObject);
var
  EWE: string;
begin
  EWE := editSpeleometre.Text;
  if (SelectTopographes(EWE)) then editSpeleometre.Text := EWE;
end;

procedure TCdrExpe.Button2Click(Sender: TObject);
var
  EWE: string;
begin
  EWE := editSpeleographe.Text;
  if (SelectTopographes(EWE)) then editSpeleographe.Text := EWE;
end;

procedure TCdrExpe.cmbModeDeclChange(Sender: TObject);
begin
  btnCalcDeclimag.Enabled := (cmbModeDecl.ItemIndex = 0);
end;

procedure TCdrExpe.cmbUnitDeclimagChange(Sender: TObject);
begin

end;

procedure TCdrExpe.InitCaptions;
begin
 lblSeance.Caption       := GetResourceString(rsCDR_EXPE_SEANCE);
 lblDate.Caption         := GetResourceString(rsCDR_EXPE_DATE);
 lblSpeleometre.Caption  := GetResourceString(rsCDR_EXPE_SPELEOMETRE);
 lblSpeleographe.Caption := GetResourceString(rsCDR_EXPE_SPELEOGRAPHE);
 lblDeclimag.Caption     := GetResourceString(rsCDR_EXPE_DECLIMAG);
 //lblInclin.Caption       := GetResourceString(rsCDR_EXPE_INCLIN);
 lblColor.Caption        := GetResourceString(rsCOLOR);
 lblObs.Caption          := GetResourceString(rsLBL_COMMENTS);
 btnCalcDeclimag.Caption := GetResourceString(rsDLG_CALC_DOCALC);
 cmbUnitDeclimag.Enabled := false;
 cmbModeDecl.Enabled     := True;
 cmbUnitDeclimag.Clear;
 cmbUnitDeclimag.Items.Add(GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES));
 cmbUnitDeclimag.Items.Add(GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES));
 cmbUnitDeclimag.ItemIndex := 0;
end;

procedure TCdrExpe.PutExpeInForm;
var
  WU: TDateTime;
begin
  AfficherMessage('PutExpeInForm: 001');
  editNoExpe.AsInteger    := FExpe.IDExpe;
  AfficherMessage('PutExpeInForm: 002');
  WU := GetSecuredDate(FExpe.AnneeExpe, FExpe.MoisExpe, FExpe.JourExpe);
  AfficherMessage('PutExpeInForm: 003');
  editDateSeance.Text     := DateToStr(WU, DefaultFormatSettings);
  editSpeleometre.Text    := GetResourceString(FExpe.Operateur);
  editSpeleographe.Text   := GetResourceString(FExpe.ClubSpeleo);
  editDeclimag.Value   := FExpe.Declinaison;
  //editInclinaison.Value   := FExpe.Inclinaison;
  SetLbColor(FExpe.Couleur);
  cmbModeDecl.ItemIndex   := 0; // FExpe.ModeDecl; // volontairement non supporté par GHTopo
  editComment.Text        := _AnsiToLCLStr(Trim(FExpe.Commentaire));
end;
function TCdrExpe.GetExpeFromForm(): TExpe;
var
  E: TExpe;
  YYYY, MM, DD: word;
  WU: TDateTime;
begin
  with E do
  begin
    IDExpe      := editNoExpe.AsInteger;
    WU := StrToDate(Trim(editDateSeance.Text), DefaultFormatSettings);
    DecodeDate(WU, YYYY, MM, DD);
    JourExpe    := DD;
    MoisExpe    := MM;
    AnneeExpe   := YYYY;
    Operateur   := _LCLStrToAnsi(editSpeleometre.Text);
    ClubSpeleo  := _LCLStrToAnsi(editSpeleographe.Text);
    //Inclinaison := editInclinaison.Value;
    Declinaison := editDeclimag.Value;
    Couleur     := editCouleur.AsInteger;
    ModeDecl    := 0; // cmbModeDecl.ItemIndex;    // volontairement non supporté par GHTopo
    Commentaire := _LCLStrToAnsi(editComment.Text);
  end;
  Result:=E;
end;
procedure TCdrExpe.SetLbColor(const IdxColor: integer);
var
  P: TPalette256;
begin
  editCouleur.AsInteger:=IdxColor;
  P := TPalette256.Create;
  try
    P.GenerateTOPOROBOTPalette();
    lbColorSeance.Color   := P.GetColorByIndex(IdxColor);
    editCouleur.AsInteger := IdxColor;
  finally
    P.Free;
  end;
end;

procedure TCdrExpe.SetDocTopo(const D: TToporobotStructure2012);
begin
  FMyDocTopo := D;
end;

end.

