// Afficheur de listes incorporées ds les viewers
// (réseaux, secteurs, dates, profondeurs)
// Date: 03/05/2013
// 29/05/2015: Interaction entre ce cadre et les autres modules
// 27/07/2017: Possibilité de surligner par réseaux, secteurs, codes, expés
//             en plus du surlignage par séries
{$INCLUDE CompilationParameters.inc}
unit CadreListesPourVueGraphique;
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Classes, SysUtils, FileUtil, Graphics,
  Forms, Controls, CheckLst, ComCtrls, StdCtrls, Grids, LCLType,
  StructuresDonnees,
  Common,
  CallDialogsStdVersion,
  UnitClassPalette,
  UnitEntitesExtended,
  CadreGHTopoContext2D, // pour la localisation
  ToporobotClasses2012,
  unitobjetserie, types;

type TIndexItem = record
  aIdx     : integer;
  aChecked : boolean;
end;

type

  { TCdrListesPourVisualisateurs }

  TCdrListesPourVisualisateurs = class(TFrame)
    btnLocaliser: TButton;
    btnSupprimerSerie: TButton;
    chkDoHighLightCurrentItem: TCheckBox;
    lsbListeElements: TCheckListBox;
    cmbModeListe: TComboBox;
    hcColsTitres: THeaderControl;
    lbTitreListe: TLabel;
    lbRegenerationListe: TStaticText;
    procedure btnLocaliserClick(Sender: TObject);
    procedure btnSupprimerSerieClick(Sender: TObject);
    procedure chkDoHighLightCurrentItemChange(Sender: TObject);
    procedure cmbModeListeChange(Sender: TObject);
    procedure hcColsTitresResize(Sender: TObject);
    procedure hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl;
      Section: THeaderSection);
    procedure lsbListeElementsClick(Sender: TObject);
    procedure lsbListeElementsDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
  private
    { private declarations }
    FDocToporobot: TToporobotStructure2012;
    FTableEntites: TBDDEntites; // pour le mode 'Stations'
    FCadreVue2D  : TGHTopoContext2DA;
    FNumeroElement: integer;
    FUseInternalIndex: boolean;
    FModeSelection: TModeSelectionListe;
    FNbItems: LongInt;
    FMyPalette: TPalette256;
    //FProcChangeListItem: TProcedureOfObject;
    // tableau d'index
    FTableauIndex: array of TIndexItem;
    function SetTIndexItem(const QIdx: integer; const QChecked: boolean): TIndexItem;
    function FindInternalIdx(const Idx: integer): integer;
    procedure RefreshListe();
    procedure SurlignerItem();

  public
    { public declarations }
    function InitialiseListeSelection(const DT: TToporobotStructure2012;
                                      const DE: TBDDEntites;
                                      const GHVue2D: TGHTopoContext2DA;
                                      const MS: TModeSelectionListe;
                                      const QIndex: integer;
                                      const UseInternalIndex: boolean;
                                      //const ProcChangeListItem: TProcedureOfObject;
                                      const DoReinitFull: boolean): boolean;
    function GetModeSelection(): TModeSelectionListe;
  end;

implementation

{$R *.lfm}

{ TCdrListesPourVisualisateurs }


function TCdrListesPourVisualisateurs.InitialiseListeSelection(const DT: TToporobotStructure2012;
                                                               const DE: TBDDEntites;
                                                               const GHVue2D: TGHTopoContext2DA;
                                                               const MS: TModeSelectionListe;
                                                               const QIndex: integer;
                                                               const UseInternalIndex: boolean;
                                                               //const ProcChangeListItem: TProcedureOfObject;
                                                               const DoReinitFull: boolean): boolean;
var
  i: integer;
  miou: string;
  ht:  THeaderSection;
  //MyIndex: LongInt;
  CC: TCode;
  EE: TExpe;
  SSR: TObjSerie;
  RR: TReseau;
  SS: TSecteur;

  //R: TReseau;
  procedure AjouterTitreColonne(const Titre: string; const LG: integer);
  begin
    ht := hcColsTitres.Sections.Add;
    ht.Text := GetResourceString(Titre);
    ht.MinWidth := LG;
  end;
begin
  Result := False;
  {$IFDEF GROS_MINET}
  btnSupprimerSerie.Visible := false;
  {$ELSE}
  btnSupprimerSerie.Visible := true;
  {$ENDIF}
  FUseInternalIndex := UseInternalIndex;
  if (DoReinitFull) then
  begin
    FDocToporobot  := DT;
    FTableEntites  := DE;
    FModeSelection := MS;
    FCadreVue2D    := GHVue2D;
    cmbModeListe.Clear;
      cmbModeListe.Items.Add(GetResourceString('Entrees'));           // mslENTRANCES
      cmbModeListe.Items.Add(GetResourceString('Reseaux'));           // mslRESEAUX
      cmbModeListe.Items.Add(GetResourceString('Secteurs'));          // mslSECTEURS
      cmbModeListe.Items.Add(GetResourceString('Codes'));             // mslCODE
      cmbModeListe.Items.Add(GetResourceString('Expes'));             // mslEXPE
      cmbModeListe.Items.Add(GetResourceString('Series'));            // mslSERIE
      cmbModeListe.Items.Add(GetResourceString('Types visees'));      // mslTYPE_GALERIE
      cmbModeListe.Items.Add(GetResourceString('Dates'));             // mslDATE
      //cmbModeListe.Items.Add(GetResourceString('Stations'));          // mslSTATIONS_CALCULEES
    cmbModeListe.ItemIndex := Ord(FModeSelection);
  end;
  // purge des titres des headers
  hcColsTitres.Sections.Clear;
  AjouterTitreColonne('V' , 20);
  // purge des listes
  lsbListeElements.Clear;
  lsbListeElements.ItemHeight := 22;
  // titres
  case FModeSelection of
    mslENTRANCES: begin
                  miou := GetResourceString(rsSELECT_LISTE_ENTREES);
                  FNbItems          := FDocToporobot.GetNbEntrees;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_ENTRANCE);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_ID     , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_NOM    , 300);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_REF    , 80);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_X      , 100);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_Y      , 100);
                  AjouterTitreColonne(rsSELECT_LISTE_ENTREES_Z      , 100);
                end;
    mslRESEAUX: begin
                  miou := GetResourceString(rsSELECT_LISTE_RESEAUX);
                  FNbItems := FDocToporobot.GetNbReseaux;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_RESEAU);
                  AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_ID     , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_COLOR  , 60);
                  AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_NOM    , 300);
                end;
    mslSECTEURS:begin
                  miou := GetResourceString(rsSELECT_LISTE_SECTEURS);
                  FNbItems := FDocToporobot.GetNbSecteurs;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_SECTEUR);
                  AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_ID    , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_COLOR , 60);
                  AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_NOM   , 300);
                end;
    mslCODE:    begin
                  miou := GetResourceString(rsSELECT_LISTE_CODES);
                  FNbItems := FDocToporobot.GetNbCodes;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_CODE);
                  AjouterTitreColonne(rsSELECT_LISTE_CODES_ID       , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_CODES_AZIMUTS  , 80);
                  AjouterTitreColonne(rsSELECT_LISTE_CODES_PENTES   , 80);
                  AjouterTitreColonne(rsSELECT_LISTE_CODES_OBS      , 500);
                end;
    mslEXPE:   begin
                  // préparation de la palette
                  FMyPalette := TPalette256.Create;
                  FMyPalette.GenerateTOPOROBOTPalette;
                  // -------------------------
                  miou := GetResourceString(rsSELECT_LISTE_EXPES);
                  FNbItems := FDocToporobot.GetNbExpes;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_EXPE);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_ID             , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_COULEUR        , 80);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_DATE           , 90);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_OPERATEUR_TOPO , 140);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_CLUB_SPELEO    , 140);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_DECLINAISON    , 70);
                  AjouterTitreColonne(rsSELECT_LISTE_EXPES_OBS            , 500);
                end;
    mslSERIE:  begin
                  miou := GetResourceString(rsSELECT_LISTE_SERIES);
                  FNbItems := FDocToporobot.GetNbSeries;
                  self.Caption      := GetResourceString(rsSELECT_LISTE_SERIE);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_ID            , 40);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_DEPART        , 70);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_ARRIVEE       , 70);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_NOM           , 400);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_RESEAU        , 300);
                  AjouterTitreColonne(rsSELECT_LISTE_SERIES_NBPOINTS      , 80);
                end;
    mslDATE:
               begin
                  miou := GetResourceString(rsSELECT_LISTE_DATES);
                  FNbItems      := FTableEntites.GetNbDates;
                  self.Caption  := GetResourceString(rsSELECT_LISTE_DATES);
                  AjouterTitreColonne(rsSELECT_LISTE_UNE_DATE, 120);
               end;

    mslSTATIONS_CALCULEES:
                begin
                  miou := GetResourceString(rsSELECT_LISTE_STATIONS);
                  FNbItems      := FTableEntites.GetNbEntitesVisees();
                  self.Caption  := GetResourceString(rsSELECT_LISTE_STATIONS);
                  AjouterTitreColonne(rsSELECT_LISTE_STATIONS_ID          , 60);
                  AjouterTitreColonne(rsSELECT_LISTE_STATIONS_LABEL       , 100);
                  AjouterTitreColonne(rsSELECT_LISTE_STATIONS_X           , 90);
                  AjouterTitreColonne(rsSELECT_LISTE_STATIONS_Y           , 90);
                  AjouterTitreColonne(rsSELECT_LISTE_STATIONS_Z           , 70);
                end;

  end;
  lbTitreListe.Caption := Format(GetResourceString(rsSELECT_LISTE_NB_ELEMENTS), [FNbItems, miou]);
  lbRegenerationListe.Caption := GetResourceString('Regen liste');
  Result:=False;
  // liste
  try
   try
    lsbListeElements.Visible     := False;
    lbRegenerationListe.Visible  := True;
    with FDocToporobot do
    begin
      SetLength(FTableauIndex, 0);
      if FNbItems = 0 then Exit;
      SetLength(FTableauIndex, FNbItems);
      for i:=0 to FNbItems - 1 do
      begin
        //R:=GetReseau(i);
        //lsbReseaux.Items.Add(Format('%000d - %s',  [R.IdxReseau, R.NomReseau]));
        case FModeSelection of
          mslRESEAUX:
            begin
              RR := FDocToporobot.GetReseau(i);
              FTableauIndex[i] := SetTIndexItem(i, True);
            end;
          mslSECTEURS:
            begin
              SS := FDocToporobot.GetSecteur(i);
              FTableauIndex[i] := SetTIndexItem(i, True);
            end;
          mslCODE :
            begin
              CC := FDocToporobot.GetCode(i);
              FTableauIndex[i] := SetTIndexItem(CC.IDCode, True);
            end;
          mslEXPE :
            begin
              EE := FDocToporobot.GetExpe(i);
              FTableauIndex[i] := SetTIndexItem(EE.IDExpe, True);
            end;
          mslSERIE :
            begin
              SSR := FDocToporobot.GetSerie(i);
              FTableauIndex[i] := SetTIndexItem(SSR.GetNumeroDeSerie(), True);
            end;
          else
            FTableauIndex[i] := SetTIndexItem(i, True);
        end;
        lsbListeElements.Items.Add('');
      end;
    end;
    if (FUseInternalIndex) then FNumeroElement := QIndex
                           else FNumeroElement := FindInternalIdx(QIndex);

    lsbListeElements.ItemIndex := FNumeroElement;
    chkDoHighLightCurrentItem.Caption := 'Surligner';
    Result:=True;
  except
  end;
 finally
   lsbListeElements.Visible := True;
   lbRegenerationListe.Visible  := False;

 end;
end;



function TCdrListesPourVisualisateurs.GetModeSelection: TModeSelectionListe;
begin
  Result := FModeSelection;
end;

procedure TCdrListesPourVisualisateurs.lsbListeElementsDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  Q4  = 4; // compensation du décalage entre le header et la liste
  mg  = 1;
  // lgr = 25;
var
  r: TRect;
  rs: TReseau;
  es: TEntrance;
  cs: TCode;
  ss: TExpe;
  sr: TObjSerie;
  miou: string;
  sc: TSecteur;
  dt: TDateTime;
  et: TEntiteEtendue;
  procedure DessineCoche(const IsChecked: boolean);
  var
    QR: TRect;
    H: Integer;
  begin
    with lsbListeElements.Canvas do
    begin
      Pen.Style    := psSolid;
      Brush.Style  := bsSolid;
      Pen.Color    := clBlack;
      QR.Left      := ARect.Left   + 1;
      QR.Top       := ARect.Top    + 1;
      QR.Bottom    := ARect.Bottom - 1;
      H            := QR.Bottom - QR.Top;
      QR.Right     := QR.Left + H;
      Brush.Color  := IIF(IsChecked, clBlue, clWhite);
      Rectangle(QR);
    end;
  end;
  procedure DessineFiletColonne(const TB: integer); inline;
  begin
    lsbListeElements.Canvas.Line(TB, ARect.Top, TB, ARect.Bottom);
  end;
  procedure DessineRectangleColore(const QBG, PC, BC: TColor; const L, R, T, B: integer);
  var
    QR: TRect;
  begin
    lsbListeElements.canvas.Pen.Color   := PC;
    lsbListeElements.canvas.Brush.Color := BC;
    QR.Left     := L;
    QR.Right    := R;
    QR.Top      := T;
    QR.Bottom   := B;
    lsbListeElements.canvas.Rectangle(QR);
    // et on restaure l'ancienne couleur
    lsbListeElements.Canvas.Brush.Color := QBG;
    lsbListeElements.canvas.Pen.Color   := clSilver;
  end;

  // bg = couleur de fond; tc= couleur du texte
  procedure DessineItem(const bg,tc: TColor; const QChecked: boolean);
  VAR
    //TB: integer;
    HS: THeaderSection;
  begin
    with lsbListeElements do
    begin
      case FModeSelection of
        mslENTRANCES:
          begin
            Canvas.FillRect(ARect);
            Canvas.Brush.Color:=bg;
            Canvas.Font.Color :=tc;
            Canvas.Pen.Color  :=clSilver; // pour les filets
            //TB := hcColsTitres.hea
            HS := hcColsTitres.Sections.Items[1];  // ID entrée
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[2];  // Nom entrée
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(es.eNomEntree));
            HS := hcColsTitres.Sections.Items[3];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%d.%d',[es.eRefSer, es.eRefSt]));
            HS := hcColsTitres.Sections.Items[4];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eXEntree));
            HS := hcColsTitres.Sections.Items[5];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eYEntree));
            HS := hcColsTitres.Sections.Items[6];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eZEntree));
            DessineFiletColonne(HS.Right);
          end;
        mslRESEAUX:
          begin
            // offset:=Rect.Left + 30 + lgr;
            //Canvas.Brush.Color:=clWhite;
            Canvas.FillRect(ARect);
            canvas.Pen.Color:=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[1];  // ID réseau
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[2];  // Couleur réseau
            DessineFiletColonne(HS.Left - Q4);
            DessineRectangleColore(bg, clBlack, rs.ColorReseau, HS.Left, HS.Left - 8, ARect.Top + mg, ARect.Bottom - mg);
            HS := hcColsTitres.Sections.Items[3];  // Nom réseau
            DessineFiletColonne(HS.Left - Q4);
            Canvas.Font.Color :=tc;
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(rs.NomReseau));
          end;
        mslSECTEURS:
          begin
            // offset:=Rect.Left + 30 + lgr;
            //Canvas.Brush.Color:=clWhite;
            Canvas.FillRect (ARect);
            canvas.Pen.Color  := clSilver;
            Canvas.Font.Color := tc;

            HS := hcColsTitres.Sections.Items[1];  // ID secteur
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[2];  // Couleur secteur
            DessineFiletColonne(HS.Left - Q4);
            DessineRectangleColore(bg, clBlack, sc.CouleurSecteur, HS.Left, HS.Left - 8, ARect.Top + mg, ARect.Bottom - mg);
            HS := hcColsTitres.Sections.Items[3];  // Nom secteur
            DessineFiletColonne(HS.Left - Q4);
            //Canvas.Font.Color :=clBlack;
            Canvas.Font.Color :=tc;
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(sc.NomSecteur));
          end;
        mslCODE:
          begin
            Canvas.FillRect(ARect);
            HS := hcColsTitres.Sections.Items[1];  // Nom réseau

            Canvas.Brush.Color:=bg;
            Canvas.Font.Color :=tc;
            canvas.Pen.Color  :=clSilver;

            canvas.TextOut(HS.Left + 4  , ARect.Top+1, Format('%d', [cs.IDCode]));

            HS := hcColsTitres.Sections.Items[2];  // az
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4  , ARect.Top+1, Format('%.0f', [cs.GradAz]));
            HS := hcColsTitres.Sections.Items[3];  // p
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.0f', [cs.Gradinc]));
            HS := hcColsTitres.Sections.Items[4];  // Commentaires
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(cs.Commentaire));
          end;
        mslEXPE:
          begin
            try
              miou := DateToStr(GetSecuredDate(ss.AnneeExpe,ss.MoisExpe,ss.JourExpe));
            except
              miou := '01/01/2000';
            end;

            Canvas.FillRect(ARect);
            canvas.Pen.Color  :=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[1];  // ID expé
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[ss.IDExpe]));
            HS := hcColsTitres.Sections.Items[2];  // Couleur expé
            DessineFiletColonne(HS.Left - Q4);
            // couleur de l'expé
            DessineRectangleColore(bg, clBlack, FMyPalette.GetColorByIndex(ss.Couleur), HS.Left, HS.Left + 32, ARect.Top + mg, ARect.Bottom - mg);
            Canvas.TextOut(HS.Left + 32 + 4, ARect.Top+1, Format('%d',[ss.Couleur]));
            HS := hcColsTitres.Sections.Items[3];  // date
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, miou);

            HS := hcColsTitres.Sections.Items[4];  // spéléomètre
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.Operateur));

            HS := hcColsTitres.Sections.Items[5];  // spéléographe
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.ClubSpeleo));

            HS := hcColsTitres.Sections.Items[6];  // déclinaison
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[ss.Declinaison]));


            HS := hcColsTitres.Sections.Items[7];  // commentaires
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.Commentaire));
          end;
        mslSERIE:
          begin
            with lsbListeElements do
            begin
              Canvas.FillRect(ARect);
              canvas.Pen.Color:=clSilver;
              Canvas.Font.Color :=tc;

              HS := hcColsTitres.Sections.Items[1];  // ID série
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNumeroDeSerie()]));
              HS := hcColsTitres.Sections.Items[2];  // Départ
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieDep, sr.GetNoPointDep]));
              HS := hcColsTitres.Sections.Items[3];  // Arrivée
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieArr, sr.GetNoPointArr]));
              HS := hcColsTitres.Sections.Items[4];  // nom
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(sr.GetNomSerie));
              // réseau
              HS := hcColsTitres.Sections.Items[5];  // nom
              DessineFiletColonne(HS.Left - Q4);
              rs := FDocToporobot.GetReseau(sr.GetNumeroReseau());
              DessineRectangleColore(bg, clBlack, rs.ColorReseau, HS.Left, HS.Left + 32, ARect.Top + mg, ARect.Bottom - mg);
              Canvas.TextOut(HS.Left + 32 + 4, ARect.Top+1, _AnsiToLCLStr(rs.NomReseau));

              HS := hcColsTitres.Sections.Items[6];  // Nb points
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNbVisees]));
            end;
          end;
        mslDATE:
          begin
            Canvas.FillRect(ARect);
            canvas.Pen.Color:=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[1];  // rsSELECT_LISTE_STATIONS_ID
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%s',[DateToStr(dt)]));
          end;
        (*
        mslSTATIONS_CALCULEES:
          begin
            Canvas.FillRect(ARect);
            canvas.Pen.Color:=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[1];  // rsSELECT_LISTE_STATIONS_ID
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[et.Entite_Serie, et.Entite_Station]));
            HS := hcColsTitres.Sections.Items[2];  // rsSELECT_LISTE_STATIONS_LABEL
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  et.oIDLitteral);

            HS := hcColsTitres.Sections.Items[3];  // rsSELECT_LISTE_STATIONS_X
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(et.Une_Station_2_X));

            HS := hcColsTitres.Sections.Items[4];  // rsSELECT_LISTE_STATIONS_Y
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(et.Une_Station_2_Y));

            HS := hcColsTitres.Sections.Items[5];  // rsSELECT_LISTE_STATIONS_X
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(et.Une_Station_2_Z));

          end;
          //*)
      end; //case FModeSelection of
      DessineCoche(QChecked);
    end;
  end;
begin
  try
    case FModeSelection of
      mslENTRANCES : es := FDocToporobot.GetEntree(Index);
      mslRESEAUX   : rs := FDocToporobot.GetReseau(Index);
      mslSECTEURS  : sc := FDocToporobot.GetSecteur(Index);
      mslCODE      : cs := FDocToporobot.GetCode(Index);
      mslEXPE      : ss := FDocToporobot.getExpe(Index);
      mslSERIE     : sr := FDocToporobot.GetSerie(Index);
      mslDATE      : dt := FTableEntites.GetUneDate(Index);
      mslSTATIONS_CALCULEES : et := FTableEntites.GetEntiteVisee(Index);
    end; //case FModeSelection of
    //ShowMessageFmt('%d',[Index]);
    if (odSelected in State) then DessineItem(clBlue , clWhite, lsbListeElements.Checked[Index])
                             else DessineItem(clwhite, clBlack, lsbListeElements.Checked[Index]);
  except
    AfficherMessageErreur('Warn: Liste fiches vide');
  end;
end;

procedure TCdrListesPourVisualisateurs.cmbModeListeChange(Sender: TObject);
begin
  FModeSelection := TModeSelectionListe(cmbModeListe.ItemIndex);
  FCadreVue2D.SetModeSelectionListe(FModeSelection);
  RefreshListe();
end;

procedure TCdrListesPourVisualisateurs.hcColsTitresResize(Sender: TObject);
begin

end;

procedure TCdrListesPourVisualisateurs.hcColsTitresSectionResize(
  HeaderControl: TCustomHeaderControl; Section: THeaderSection);
begin
  lsbListeElements.Invalidate;
end;

procedure TCdrListesPourVisualisateurs.lsbListeElementsClick(Sender: TObject);
begin
  //if (assigned(FProcChangeListItem)) then FProcChangeListItem;
  SurlignerItem();
end;

procedure TCdrListesPourVisualisateurs.btnLocaliserClick(Sender: TObject);
const
  MARGE = 100;
var
  MF: String;
  RS: TReseau;
  SC: TSecteur;
  EX: TExpe;
  i : Integer;
  C1: TPoint3Df;
  C2: TPoint3Df;
  SR: TObjSerie;
  procedure zdar;
  var
    DbgTag: String;
    QDevelViseesVisibles: double;
  begin
    FTableEntites.Metafiltre(MF, QDevelViseesVisibles);
    FTableEntites.SetMinMax(True);
    C1 := FTableEntites.GetMetafilteredCoinBasGauche;
    C2 := FTableEntites.GetMetafilteredCoinHautDroit;
    DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
    FCadreVue2D.SetViewLimits(C1.X - MARGE, C1.Y - MARGE,
                              C2.X + MARGE, C2.Y + MARGE,
                              DbgTag);
    FCadreVue2D.ApplyMetaFiltre(true, MF);
  end;
begin
  // codé à l'arrache
  case FModeSelection  of
    mslRESEAUX:
      begin
        MF := '';
        for i := 0 to FDocToporobot.GetNbReseaux - 1 do
        begin
          RS := FDocToporobot.GetReseau(i);
          if (lsbListeElements.Checked[i]) then MF += Format('RESEAU=%d;', [i]);
        end;
        zdar;
      end;
    mslSECTEURS:
      begin
        MF := '';
        for i := 0 to FDocToporobot.GetNbSecteurs - 1 do
        begin
          SC := FDocToporobot.GetSecteur(i);
          if (lsbListeElements.Checked[i]) then MF += Format('SECTEUR=%d;', [i]);
        end;
        zdar;
      end;
    mslEXPE:
      begin
        MF := '';
        for i := 0 to FDocToporobot.GetNbExpes - 1 do
        begin
          EX := FDocToporobot.GetExpe(i);
          if (lsbListeElements.Checked[i]) then MF += Format('SEANCE=%d;', [EX.IDExpe]);
        end;
        zdar;
      end;
    mslSERIE:
      begin
        MF := '';
        for i := 0 to FDocToporobot.GetNbExpes - 1 do
        begin
          SR := FDocToporobot.GetSerie(i);
          if (lsbListeElements.Checked[i]) then MF += Format('SERIE=%d;', [SR.GetNumeroDeSerie()]);
        end;
        zdar;
      end;
    otherwise
  end;
end;

procedure TCdrListesPourVisualisateurs.btnSupprimerSerieClick(Sender: TObject);
var
  n: Integer;
begin
  if (not chkDoHighLightCurrentItem.Checked) then exit;
  n := lsbListeElements.ItemIndex;
  if (GHTopoQuestionOuiNon(Format(rsMSG_WARN_DELETE_ITEM, [n]))) then
  begin
    FDocToporobot.RemoveSerie(n);
    RefreshListe;
  end;

end;

procedure TCdrListesPourVisualisateurs.chkDoHighLightCurrentItemChange(Sender: TObject);
begin
  FCadreVue2D.SetHighLightCurrentItem(chkDoHighLightCurrentItem.Checked);
end;

function TCdrListesPourVisualisateurs.SetTIndexItem(const QIdx: integer; const QChecked: boolean): TIndexItem;
begin
  Result.aIdx     := QIdx;
  Result.aChecked := QChecked;
end;

function TCdrListesPourVisualisateurs.FindInternalIdx(const Idx: integer): integer;
var
  i: Integer;
begin
  Result := -1;
  if (SizeOf(FTableauIndex) = 0) then Exit(-1);
  try
    for i:= 0 to High(FTableauIndex) do
    begin
      if (FTableauIndex[i].aIdx = Idx) then Exit(i);
    end;
  except
  end;
end;

procedure TCdrListesPourVisualisateurs.RefreshListe;
begin
  InitialiseListeSelection(FDocToporobot,
                           FTableEntites,
                           FCadreVue2D,
                           FModeSelection,
                           0,
                           FUseInternalIndex,
                           //FProcChangeListItem,
                           False);
end;
procedure TCdrListesPourVisualisateurs.SurlignerItem;
var
  WU: Integer;
  SR: TObjSerie;
  CD: TCode;
  CE: TExpe;
  RS: TReseau;
  SC: TSecteur;
begin

  WU := self.lsbListeElements.ItemIndex;
  //ShowMessage(IntToStr(WU));
  case FModeSelection of
    mslSERIE:
      begin
        if (chkDoHighLightCurrentItem.Checked) then
        begin
          SR := FDocToporobot.GetSerie(WU);
          FCadreVue2D.SetModeSelectionListe(FModeSelection);
          FCadreVue2D.SetCurrentNumeroSerieAndPoint(SR.GetNumeroDeSerie(), 0);
        end;
      end;
    mslCODE:
      begin
        if (chkDoHighLightCurrentItem.Checked) then
        begin
          CD := FDocToporobot.GetCode(WU);
          FCadreVue2D.SetModeSelectionListe(FModeSelection);
          FCadreVue2D.SetCurrentNumeroCode(CD.IDCode);
        end;
      end;
    mslEXPE:
      begin
        if (chkDoHighLightCurrentItem.Checked) then
        begin
          CE := FDocToporobot.GetExpe(WU);
          FCadreVue2D.SetModeSelectionListe(FModeSelection);
          FCadreVue2D.SetCurrentNumeroExpe(CE.IDExpe);
        end;
      end;
    mslRESEAUX:
      begin
        if (chkDoHighLightCurrentItem.Checked) then
        begin
          RS := FDocToporobot.GetReseau(WU);
          FCadreVue2D.SetModeSelectionListe(FModeSelection);
          FCadreVue2D.SetCurrentNumeroReseau(WU);
        end;
      end;
    mslSECTEURS:
      begin
        if (chkDoHighLightCurrentItem.Checked) then
        begin
          SC := FDocToporobot.GetSecteur(WU);
          FCadreVue2D.SetModeSelectionListe(FModeSelection);
          FCadreVue2D.SetCurrentNumeroSecteur(WU);
        end;
      end;
  otherwise
    ;
  end;

end;
end.


//******************************************************************************


