unit CadreProfilsTopo;

{$INCLUDE CompilationParameters.inc}

interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  UnitClasseMaillage,
  ToporobotClasses2012,
  unitProfilTopo,
  Classes, SysUtils, FileUtil,
  FastGEO, UnitObjetSerie,
  TAGraph,
  TATypes,
  TASeries,
  TAChartUtils,
  Graphics,
  curredit, Forms, Controls, ComCtrls, ExtCtrls, StdCtrls, Grids, Dialogs, Types, LCLType;

type

  { TCdrProfilTopo }

  TCdrProfilTopo = class(TFrame)
    btnCopierTableau: TButton;
    btnProfilColor: TColorButton;
    btnRecalculer: TButton;
    Chart1: TChart;
    editNomProfil: TEdit;
    editX1: TCurrencyEdit;
    editX2: TCurrencyEdit;
    editY1: TCurrencyEdit;
    editY2: TCurrencyEdit;
    grdValeursProfil: TStringGrid;
    HeaderControl1: THeaderControl;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    lbNbConduitsRecoupes: TLabel;
    lbCoordsProfil: TStaticText;
    lbNbPointsProfil: TLabel;
    lsbGaleriesRecoupees: TListBox;
    PageControl1: TPageControl;
    Panel1: TPanel;
    pnlProfil: TPanel;
    pnlProfil1: TPanel;
    tabShtConduits: TTabSheet;
    tabShtProfil: TTabSheet;
    tabShtValeurs: TTabSheet;
    procedure btnCopierTableauClick(Sender: TObject);
    procedure Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure lsbGaleriesRecoupeesDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
  private
    FMaillage    : TMaillage;
    FDocTopo     : TToporobotStructure2012; // pour descro des conduits traversés
    FProfilTopo  : TProfilTopo;
    FProfilReady : boolean;
    FCourbeProfil: TLineSeries;
    FPtsConduits : TLineSeries;
    procedure DispProfil(const P: TProfilTopo);
    procedure InitCaptions();
    procedure ListerValeursProfil(const P: TProfilTopo);

  public
    function  Initialiser(const DT: TToporobotStructure2012; const QMaillage: TMaillage; const ProfilTopo: TProfilTopo): boolean;
    procedure Finaliser();
    function  GetProfilTopo(): TProfilTopo;
  end;

implementation

{$R *.lfm}

{ TCdrProfilTopo }

function TCdrProfilTopo.Initialiser(const DT: TToporobotStructure2012; const QMaillage: TMaillage; const ProfilTopo: TProfilTopo): boolean;
begin
  result := false;
  FProfilReady := false;

  try
    InitCaptions();
    FDocTopo   := DT;
    FMaillage  := QMaillage;

    if (FMaillage.GetNbProfilsTopo() = 0) then exit(false);
    Chart1.ClearSeries;

    DispProfil(ProfilTopo);
    ListerValeursProfil(ProfilTopo);
    PageControl1.ActivePageIndex := 0;
    result    := true;
  finally
  end;
end;

procedure TCdrProfilTopo.Finaliser();
begin
  try
    FCourbeProfil.Clear;
  finally
    FCourbeProfil.Free;
  end;
end;

procedure TCdrProfilTopo.InitCaptions();
var
  c: Integer;
begin
  btnCopierTableau.Caption := GetResourceString(rsBTN_COPIER_TABLEAU);
  grdValeursProfil.RowCount := 0;
  grdValeursProfil.ColCount := 5;
  for c := 1 to 4 do grdValeursProfil.ColWidths[c] := 100;
  grdValeursProfil.Cells[0, 0] := GetResourceString(rsCDR_PROFIL_COL_NO_PT);
  grdValeursProfil.Cells[1, 0] := GetResourceString(rsCDR_PROFIL_COL_ABSCISSE);
  grdValeursProfil.Cells[2, 0] := GetResourceString(rsCDR_PROFIL_COL_COTE);
  grdValeursProfil.Cells[3, 0] := GetResourceString(rsCDR_PROFIL_COL_X);
  grdValeursProfil.Cells[4, 0] := GetResourceString(rsCDR_PROFIL_COL_Y);
end;

procedure TCdrProfilTopo.Chart1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  PM: TDoublePoint;
  WU, PZ: Double;
begin
  if (FProfilReady) then
  begin
    PM := Chart1.ImageToGraph(MakeTPoint(X, Y));
    PZ := FProfilTopo.GetAltitudeProfil(PM.X);
    lbCoordsProfil.Caption := format('%.2f, %.2f', [PM.X, PZ]);
  end;
end;

procedure TCdrProfilTopo.lsbGaleriesRecoupeesDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  Q4  = 4; // compensation du décalage entre le header et la liste
  mg  = 1;
var
  GC: TEntiteEtendue;
  procedure DessineFiletColonne(const TB: integer); inline;
   begin
     lsbGaleriesRecoupees.Canvas.Line(TB, ARect.Top, TB, ARect.Bottom);
   end;
   procedure DessineItem(const bg,tc: TColor);
   VAR
     HS: THeaderSection;
     R : TRect;
     QIdx: integer;
     SR: TObjSerie;
     MyReseau: TReseau;
   begin
     with lsbGaleriesRecoupees do
     begin
       Canvas.FillRect(ARect);
       canvas.Pen.Color:=clSilver;
       Canvas.Font.Color :=tc;

       HS := HeaderControl1.Sections.Items[0];  // No
       canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
       DessineFiletColonne(HS.Left - Q4);
       HS := HeaderControl1.Sections.Items[1];  // Station
       canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%d.%d', [GC.Entite_Serie, GC.Entite_Station]));

       HS := HeaderControl1.Sections.Items[2];  // Couleur réseau
       DessineFiletColonne(HS.Left - Q4);
       canvas.Pen.Color   := clBlack;
       MyReseau := FDocTopo.GetReseau(GC.eReseau);
       canvas.Brush.Color := MyReseau.ColorReseau; // couleur du profil
       r.Left   := HS.Left;
       r.Right  := HS.Right - 8;
       r.Top    := ARect.Top + mg;
       r.Bottom := ARect.Bottom - mg;
       canvas.Rectangle(R);
       Canvas.Brush.Color := bg;
       canvas.Pen.Color   := clSilver;

       HS := HeaderControl1.Sections.Items[3];  // Nom profil
       DessineFiletColonne(HS.Left - Q4);
       Canvas.Font.Color := tc;
       FDocTopo.GetSerieByNumeroSerie(GC.Entite_Serie, SR, QIdx);
       canvas.TextOut(HS.Left + 4, ARect.Top+1, SR.GetNomSerie());
     end;
   end;

 begin
   try
     GC := FProfilTopo.GetGalerieRecoupee(Index);
     with lsbGaleriesRecoupees do
     begin
       canvas.brush.color := clWhite;
       DessineItem(clWhite, clBlack);
       //affichage lorsque la ligne est sélectionnée
       if (odSelected in state) then
       begin
         canvas.brush.color:=clBlue;
         DessineItem(clBlue, clWhite);
       end;
     end;
   except
     pass;
   end;
end;

procedure TCdrProfilTopo.btnCopierTableauClick(Sender: TObject);
begin
  grdValeursProfil.CopyToClipboard();
end;



procedure TCdrProfilTopo.DispProfil(const P: TProfilTopo);
var

  LongCumuleeProfil: Extended;
  Nb, i: Integer;
  Pt: TPoint3DfOrderedByP;
  PT0, PT1: TPoint2Df;
  QIX, QIY: double;
  MyVisee: TEntiteEtendue;
begin
  Chart1.ClearSeries;

  FProfilTopo  := P;
  FProfilReady := false;
  self.Caption := FProfilTopo.ProfilName;
  btnProfilColor.ButtonColor := FProfilTopo.ProfilColor;
  editNomProfil.Text         := FProfilTopo.ProfilName;
  PT0 := FProfilTopo.GetExtremite1();
  PT1 := FProfilTopo.GetExtremite2();

  editX1.Value  := PT0.X;
  editY1.Value  := PT0.Y;
  editX2.Value  := PT1.X;
  editY2.Value  := PT1.Y;

  // Le tracé du profil
  LongCumuleeProfil := 0.00;
  // on supprime la série précédente s'il y en a
  try
    FCourbeProfil := TLineSeries.Create(Chart1);
    FCourbeProfil.Clear;
    FCourbeProfil.ShowPoints     := false;
    FCourbeProfil.LinePen.Color  := clRed;
    FCourbeProfil.LinePen.Width  := 3;
    Nb := P.GetNbElements();
    for i := 0 to Nb - 1 do
    begin
      Pt := P.GetElement(i);
      FCourbeProfil.AddXY(Pt.P, Pt.Z, '', clBlue);
    end;
    Chart1.AddSeries(FCourbeProfil);
    FProfilReady  := True;
  except
  ;
  end;
  // Les conduits
  Nb := P.GetNbGaleriesRecoupees();
  if (Nb > 0) then
  begin
    try
      FPtsConduits := TLineSeries.Create(Chart1);
      FPtsConduits.Clear;
      FPtsConduits.ShowPoints     := True;
      FPtsConduits.LineType       := ltNone;
      FPtsConduits.Pointer.Brush.Color := clAqua;
      FPtsConduits.Pointer.Pen.Color   := clBlue;

      FPtsConduits.Pointer.Pen.Width   := 0;

      FPtsConduits.Pointer.Style       := psCircle;
      FPtsConduits.Marks.Style         := smsLabel;
      for i := 0 to Nb - 1 do
      begin
        MyVisee := P.GetGalerieRecoupee(i);
        // calcul de l'intersection
        Intersect(PT0.X, PT0.Y, PT1.X, PT1.Y, MyVisee.Une_Station_1_X, MyVisee.Une_Station_1_Y, MyVisee.Une_Station_2_X, MyVisee.Une_Station_2_Y, QIX, QIY);
        Pt.P := Hypot2D(QIX - PT0.X, QIY - PT0.Y);
        Pt.Z := 0.50 * (MyVisee.Une_Station_2_Z + MyVisee.Une_Station_1_Z);
        FPtsConduits.AddXY(Pt.P, Pt.Z,
                           Format('%d.%d', [MyVisee.Entite_Serie, MyVisee.Entite_Station]),
                           clBlue);
      end;
      Chart1.AddSeries(FPtsConduits);
    except
      pass;
    end; // if (Nb > 0) then
  end;

end;

function TCdrProfilTopo.GetProfilTopo(): TProfilTopo;
begin
  FProfilTopo.ProfilColor      := btnProfilColor.ButtonColor;
  FProfilTopo.ProfilName       := Trim(editNomProfil.Text);
  Result                       := FProfilTopo;
end;

procedure TCdrProfilTopo.ListerValeursProfil(const P: TProfilTopo);
var
  i, Nb, q, QInternalIdx: Integer;
  PP: TPoint3DfOrderedByP;
  PT: TEntiteEtendue;
  OMG: TObjSerie;
begin
  Nb := P.GetNbElements();
  lbNbPointsProfil.Caption := format(GetResourceString(rsCDR_PROFIL_NB_POINTS), [Nb]);
  if (Nb = 0) then Exit;
  grdValeursProfil.RowCount := 1 + Nb;
  for i := 0 to Nb - 1 do
  begin
    PP := P.GetElement(i);
    q := i+1;
    grdValeursProfil.Cells[0, q] := Format('%d', [i]);
    grdValeursProfil.Cells[1, q] := FormatterNombreOOo(PP.P, 3, false);
    grdValeursProfil.Cells[2, q] := FormatterNombreOOo(PP.Z, 3, false);
    grdValeursProfil.Cells[3, q] := FormatterNombreOOo(PP.X, 3, false);
    grdValeursProfil.Cells[4, q] := FormatterNombreOOo(PP.Y, 3, false);
  end;

  // les conduits recoupés
  Nb := P.GetNbGaleriesRecoupees();
  lbNbConduitsRecoupes.Caption := format(GetResourceString(rsCDR_PROFIL_NB_CONDUITS), [Nb]);
  lsbGaleriesRecoupees.Clear;
  if (Nb = 0) then Exit;
  for i := 0 to Nb - 1 do
  begin
    PT := P.GetGalerieRecoupee(i);
    FDocTopo.GetSerieByNumeroSerie(PT.Entite_Serie, OMG, QInternalIdx);
    lsbGaleriesRecoupees.Items.Add(format('%d: %d.%d (%s)', [i, Pt.Entite_Serie, Pt.Entite_Station, OMG.GetNomSerie()]));
  end;
end;

end.

