unit CadreViseesAntenne;
// Visées en antenne
// Date: 19/08/2012
// Statut: Opérationnel
// 04/10/2013: Améliorations dans la grille
// 06/02/2014: Remplacement de l'évaluateur d'expressions
// 14/02/2014: Implémentation du champ secteur
// 29/10/2014: Mise à 3 décimales des valeurs L, Az, P (DistoX)
// 26/09/2017: Suppression de la grille et de sa gestion: elle était
//             inutilisée, mobilisait beaucoup de mémoire
//             et s'est avérée la cause de plantages
// 27/09/2017: Incorporé dans un dialogue modal (n'est plus une fenêtre permanente)

{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Common,
  evaluateur_expressions,
  StructuresDonnees,
  ToporobotClasses2012, Dialogs,
  Classes, SysUtils, FileUtil, curredit, Forms, Controls, StdCtrls, Graphics,
  ActnList, Menus, Buttons, LCLType, ExtCtrls, ComCtrls;

type

  { TCdrAntennes }

  TCdrAntennes = class(TFrame)
    acSauvegrd: TAction;
    acSelectCode: TAction;
    acSelectExpe: TAction;
    acFindStationDepart: TAction;
    acSelectReseau: TAction;
    acSelectSecteur: TAction;
    acArmerDistoX: TAction;
    acRefreshListeAntennes: TAction;
    acNettoyerAntennes: TAction;
    acOuvrirFichierTexte: TAction;
    acChangerPointAccrochageAntennes: TAction;
    acSortBySerieSt: TAction;
    acAttribCodesExpes: TAction;
    acViderListeAntennes: TAction;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    BitBtn5: TBitBtn;
    CdrAntennesActionList: TActionList;
    CdrAntennesImagesList: TImageList;
    editLongueurMaxAntenne: TCurrencyEdit;
    lbEtapeTraitement: TLabel;
    lbLengthMaxAntenne: TLabel;
    lbNbAntennes: TLabel;
    lbStatutReattribAntennes: TStaticText;
    lbStatutNettoyageAntennes: TStaticText;
    lbStatutDestructionAntennes: TStaticText;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    lbStatutTriAntennes: TStaticText;
    Panel1: TPanel;
    ProgressBar1: TProgressBar;
    procedure acAttribCodesExpesExecute(Sender: TObject);
    procedure acChangerPointAccrochageAntennesExecute(Sender: TObject);
    procedure acFindStationDepartExecute(Sender: TObject);
    procedure acNettoyerAntennesExecute(Sender: TObject);
    procedure acSortBySerieStExecute(Sender: TObject);
    procedure acViderListeAntennesExecute(Sender: TObject);
    procedure lbNbAntennesClick(Sender: TObject);
  private
    { private declarations }
    FMyDocToporobot: TToporobotStructure2012;
    procedure InitCaptions();
    procedure SetVoyantStatut(const LB: TStaticText; const DoneOK: boolean; const MsgOK: string; const MsgKO: string = 'Echec');
    procedure DisplayProgression(const Etape: string; const Done, Starting, Ending: integer);
  public
    { public declarations }
    function Initialise(const DT: TToporobotStructure2012) : boolean;
  end;

implementation
uses
  CallDialogsStdVersion;


{$R *.lfm}

function TCdrAntennes.Initialise(const DT: TToporobotStructure2012) : boolean;
begin
  Result := False;
  try
    FMyDocToporobot := DT;
    InitCaptions();
    lbNbAntennes.Caption := format(GetResourceString(rsCDR_ANTENNES_LB_NB_ANTENNES), [FMyDocToporobot.GetNbAntennes()]);
    Result := True;
  except

  end;
end;

procedure TCdrAntennes.lbNbAntennesClick(Sender: TObject);
begin
  lbNbAntennes.Caption := format('%d antennes', [FMyDocToporobot.GetNbAntennes()]);
end;

procedure TCdrAntennes.SetVoyantStatut(const LB: TStaticText;
                                       const DoneOK: boolean;
                                       const MsgOK: string;
                                       const MsgKO: string = 'Echec');
begin
  LB.Color := IIF(DoneOK, clGreen, clRed);
  LB.Caption := IIF(DoneOK, MsgOK, MsgKO);
end;








procedure TCdrAntennes.acFindStationDepartExecute(Sender: TObject);
var
  IDL: String;
  S: integer;
  P: integer;
  QDoMatchExact: boolean;
begin
  QDoMatchExact := false;
  (*
  if (GHTopoInputQuery(GetResourceString(rsDLG_FIND_PT_BY_ID_TITLE),
                       GetResourceString(rsDLG_FIND_PT_BY_ID_PROMPT),
                       IDL)) then
  begin
    if (FMyDocToporobot.FindPtTopoByCle(QDoMatchExact, IDL, S, P)) then
    begin
      grdViseesEnAntenne.Cells[GRD_ANTENNES_COL_SER_DEP, FCurrRow] := Format('%d', [S]);
      grdViseesEnAntenne.Cells[GRD_ANTENNES_COL_PT_DEP , FCurrRow] := Format('%d', [P]);
    end
    else
      ShowMessage(GetResourceString(rsMATCHNOTFOUND));
  end;
  //*)
end;

procedure TCdrAntennes.acNettoyerAntennesExecute(Sender: TObject);
var
  WU: Integer;
  BF: TProcDisplayProgression;
begin
  acNettoyerAntennes.Enabled := false;
  try
    if (GHTopoQuestionOuiNon(GetResourceString(rsCDR_ANTENNES_CONFIRM_NETTOYAGE))) then
    begin
      BF := FMyDocToporobot.GetProcDisplayProgression();
      FMyDocToporobot.SetProcDisplayProgression(DisplayProgression);
      WU  := FMyDocToporobot.NettoyerAntennes(editLongueurMaxAntenne.Value, -1, -1);
      SetVoyantStatut(lbStatutNettoyageAntennes, WU >= 0, Format('%d antennes supprimées', [WU]));

    end;
  finally
    acNettoyerAntennes.Enabled := True;
    FMyDocToporobot.SetProcDisplayProgression(BF);
  end;
end;





procedure TCdrAntennes.acSortBySerieStExecute(Sender: TObject);
var
  WU: integer;
begin
  WU := FMyDocToporobot.SortAntennesBySerieStation();
  SetVoyantStatut(lbStatutTriAntennes, WU >= 0, Format('%d antennes triées', [WU]));
end;

procedure TCdrAntennes.acViderListeAntennesExecute(Sender: TObject);
begin
  if (GHTopoQuestionOuiNon('Supprimer TOUTES les visées en antenne')) then
  begin
    FMyDocToporobot.ViderListeViseesAntennes;
  end;
end;

procedure TCdrAntennes.DisplayProgression(const Etape: string; const Done, Starting, Ending: integer);
begin
  lbEtapeTraitement.Caption := Format('%s: %d of %d', [Etape, Done, Ending]);
  ProgressBar1.Min      := Starting;
  ProgressBar1.Max      := Ending;
  ProgressBar1.Position := Done;
  lbNbAntennes.Caption  := Format(GetResourceString(rsCDR_ANTENNES_LB_NB_ANTENNES), [FMyDocToporobot.GetNbAntennes()]);
  Application.ProcessMessages;
end;


procedure TCdrAntennes.acAttribCodesExpesExecute(Sender: TObject);
var
  WU: Boolean;
begin
  WU := FMyDocToporobot.ReattribuerCodesExpesViseesAntennes();
  SetVoyantStatut(lbStatutReattribAntennes, WU, 'Réattribution effectuée');
end;

procedure TCdrAntennes.acChangerPointAccrochageAntennesExecute(Sender: TObject);
var
  MyAntenne: TViseeAntenne;
  EWE: String;
  WU: String;
  QSer: integer;
  QPt: integer;
  Nb: Integer;
  QDoMatchExact: boolean;
begin
  (*
  QDoMatchExact := false;
  if (grdViseesEnAntenne.Row < 1) then Exit;
  MyAntenne := FMyDocToporobot.GetViseeAntenne(grdViseesEnAntenne.Row);
  EWE := Format('Deplacer Antenne: %d.%d - %.2f, %.2f, %.2f', [MyAntenne.SerieDepart, MyAntenne.PtDepart, MyAntenne.Longueur, MyAntenne.Azimut,  MyAntenne.Pente]);
  WU  := Format('%d.%d', [MyAntenne.SerieDepart, MyAntenne.PtDepart]);
  if (InputQuery('Changement de point de base', 'Nouveau point de base (couple serie et station ss.pp)', WU)) then
  begin
    if (FMyDocToporobot.FindPtTopoByCle(QDoMatchExact, WU, QSer, QPt)) then
    begin
      Nb := FMyDocToporobot.DeplacerOrigineViseesRayonnantes(MyAntenne.SerieDepart, MyAntenne.PtDepart, QSer, QPt);
      ShowMessageFmt('%d antennes deplacees', [Nb]);
      RemplirTableau;
    end
    else
      ShowMessageFmt('Point de base %s introuvable', [WU]);
  end;
  //*)
end;

procedure TCdrAntennes.InitCaptions;
  procedure EWE(const WU: TAction; const S: string);
  begin
    WU.Caption  := GetResourceString(S);
    WU.Hint     := WU.Caption;
  end;
begin
  EWE(acOuvrirFichierTexte    , rsCDR_ANTENNES_AC_IMPORT_CSV);
  EWE(acSauvegrd              , rsCDR_ANTENNES_AC_SAVEGRD);
  EWE(acAttribCodesExpes      , rsCDR_ANTENNES_ATTRIB_CODES_EXPES);
  EWE(acNettoyerAntennes      , rsCDR_ANTENNES_NETTOYER_ANTENNES);
  EWE(acRefreshListeAntennes  , rsCDR_ANTENNES_REFRESH_LISTE);
  EWE(acSelectReseau          , rsCDR_ANTENNES_SELECT_RESEAU);
  EWE(acSelectCode            , rsCDR_ANTENNES_SELECT_CODE);
  EWE(acSelectExpe            , rsCDR_ANTENNES_SELECT_EXPE);
  EWE(acSelectSecteur         , rsCDR_ANTENNES_SELECT_SECTEUR);
  EWE(acViderListeAntennes    , rsCDR_ANTENNES_VIDER_LISTE_ANTENNES);
  lbLengthMaxAntenne.Caption := GetResourceString(rsCDR_ANTENNES_LB_MAX_LENGTH_ANTENNE);
end;


end.

