unit CallDialogs;
{$INCLUDE CompilationParameters.inc}
interface
uses
  {$IFDEF LANGUAGE_FRENCH}
    UnitMessages_fr,
  {$ENDIF}
  Math,
  GHCD_Types,
  GeneralFunctions,
  UnitDocDessin,
  unitSectionsTransversales,
  unitGenererFichePointTopo,
  SysUtils, Classes, Graphics,
  Controls,
  Forms,
  stdctrls,
  Dialogs,
  extdlgs,
  FileUtil;

  // infos base station
  procedure DisplayInfosBaseStation(const BP: TBaseStation);
  // Question Oui-Non
  function QuestionOuiNon(const Msg: string): boolean;
  // choix de fichiers
  // choix d'une photo
  function ChoosePhotoFileName(var PhotoFileName: string): boolean;

  function DoDialogOpenFilename(const Filters: string; var QFileName: TStringDirectoryFileName): boolean;
  function DoDialogSaveFilename(const QFilters, QDefaultExt: string; var QFileName: TStringDirectoryFileName): boolean;
  function DoDialogSelectDossier(const QInitialDir: TStringDirectoryFileName; var QDirectorySelected: TStringDirectoryFileName): boolean;
  // paramétrage de la grille de dessin
  function ChooseParamsGrid(const PrmsGrd: TParamsGrilleDessin): TParamsGrilleDessin;
  // choix d'une couleur
  function ChooseColor(const OldColor: TColor): TColor;
  (*
  // remplir les combobox de style et de groupes
  procedure RemplirCmbGroupes(const MyDocDessin: TDocumentDessin;
                              var   cmbDest    : TComboBox;
                              const Idx: integer);
  //*)
  // édition d'un objet symbole
  procedure  EditerSymboleObject(const MyDocuDessin: TDocumentDessin;
                                 const CurrentIdxSymbole: integer;
                                 const ModeEdition: TModeEdition;
                                 const IdxGroupeCourant:  TIDGroupeEntites;
                                 const TypeDeSymbole   : TNatureObjetSymbole;
                                 const OP: TSymbole;
                                 const AbsolutePos: TPoint3Df;
                                 const ProcRepercuter: TProcRefreshVues);

  // édition d'un objet texte
  procedure  EditerTextObject(const MyDocuDessin    : TDocumentDessin;
                             const CurrentIdxTexte: integer;
                             const ModeEdition     : TModeEdition;
                             const IdxGroupeCourant: TIDGroupeEntites;
                             const TypeDeTexte     : TNatureObjetTexte;
                             var   OT              : TTextObject;
                             const AbsolutePos     : TPoint3Df;
                             const ProcRepercute   : TProcRefreshVues);
  // édition d'un scrap
  function EditerScrap(const MyDocDessin:TDocumentDessin;
                       var   MyScrap: TScrap): boolean;

  // éditeur graphique de sections transversales
  function  EditeurSectionTransversale(const ST: TBaseStation; const QIDGroupe: TIDGroupeEntites): integer;
  // édition de supergroupe
  function EditerSuperGroupe(var MySuperGroupe: TSuperGroupe): boolean;

  // dialogue d'édition de groupe
  function EditerGroupe(var MyGroupe: TGroupeEntites; const ME: TModeEdition; const QFiltres: string; out DoSetThisGroupeAsCurrent: boolean): boolean;

  // dialogue d'édition de style d'objets
  function EditerStylesObjets(const MyDocDessin: TDocumentDessin; const PF: TProcRefreshStyles): boolean;

  // sélection des zooms
  function GetPresetedZoom(const MyTableauZooms: TTableauZooms; const NoZoom: integer): integer;
  // garnir une combobox avec les types de symboles
  procedure RemplirComboTypesSymboles(const CB: TComboBox; const Idx: integer);
  procedure RemplirComboTypesCourbes(const CB: TComboBox; const Idx: integer);
  procedure RemplirComboTypesSimplesLignes(const CB: TComboBox; const Idx: integer);
  procedure RemplirComboTypesPolygones(const CB: TComboBox; const Idx: integer);
  procedure RemplirComboTypesTextes(const CB: TComboBox; const Idx: integer);

  // boite A propos
  procedure DisplayHelpSystem(const MyDocDessin:TDocumentDessin;
                              const Topics: string);
  // hiérarchie des objets
  procedure DisplayHierarchieObjets(const MyDocDessin:TDocumentDessin);
  // saisie d'une valeur numérique isolée
  function  InputNumValue(const ACaption, APrompt: string; const ValMini, ValMax: double; var Value: double): boolean;
  // insertion / modification d'une image TImageObject
  function DoDialogTImageObject(const FD: TDocumentDessin;
                                var EWE: TImageObject;
                                const QInitialDir: string;
                                const ImgPosition: TPoint3Df;
                                const DoCreateImage: boolean): boolean;
  // éditeur de texte
  procedure DisplayEditeurTexte(const QText: TStringList; const QTitreDlg: string; const QM: TModeSelectionEntites); overload;

  // notifier un message d'erreur et afficher la console
  procedure NotifyError(const Msg: string);
  function  EditerSectionTransversale(const FD: TDocumentDessin; const BP: TBaseStation; const QOffsetX, QOffsetY: double): boolean;
  procedure DisplayDlgExportToTherion(const FD: TDocumentDessin);
  procedure DisplayDlgExportGIS(const FD: TDocumentDessin);

  function  SelectIndexGroupe(const FD: TDocumentDessin; const IdxGroupe: TIDGroupeEntites; out IdxSelected: TIDGroupeEntites): boolean;
  // création d'une fiche point topo avec un extrait de plan
  function CreateFichePointTopo(const FD: TDocumentDessin;
                                const QTextures: TTexturesPolygonObject;
                                const Station: TBaseStation): boolean;
implementation
uses
  frmGroupes               // pour dialogue Groupe;
 ,frmJournal               // console
 ,frmTextObject            // pour EditerTextObject()
 ,frmSymbolObject          // pour EditerSymbolObject()
 ,dlgHelpSystem            // pour le système d'aide
 ,frmEditStylesObjet       // pour EditerStylesObjets()
 ,frmEditScraps            // pour EditerScrap()
 ,frmHierarchieObjets      // pour la hiérarchie d'objets
 ,frmTImageObject          // pour DoDialogTImageObject()
 ,frmEditeurTexte          // pour l'éditeur de texte intégré
 ,dlgEditeurSectionTransversale // éditeur de sections
 ,frmExportTherion         // pour export vers Therion
 ,frmExportToSIG           // pour export GIS
 ,frmListeGroupes          // pour liste des groupes
 ;

procedure DisplayHierarchieObjets(const MyDocDessin:TDocumentDessin);
var
  DLG: TdlgHierarchieObjets;
begin
  DLG := TdlgHierarchieObjets.Create(Application);
  try
    if (DLG.Initialise(MyDocDessin)) then DLG.ShowModal;
    DLG.Finalise();
  finally
    DLG.Release;
  end;

end;

function InputNumValue(const ACaption, APrompt: string; const ValMini, ValMax: double; var Value: double): boolean;
var
  WU: String;
  EWE: Extended;
begin
  Result := false;
  WU := FloatToStr(Value);
  if (InputQuery(ACaption, APrompt, WU)) then
  begin
    EWE := StrToFloatDef(WU, 0.00);
    Value := EWE;
    Result := IsInRange(EWE, ValMini, ValMax);
  end;
end;

procedure DisplayHelpSystem(const MyDocDessin:TDocumentDessin; const Topics: string);
var
  s: string;
begin
  with TfrmHelpSystem.Create(Application) do
  begin
    try
      {$IFDEF FRENCH_MESSAGES}   s:= GetGHTopoDirectory() + 'HelpFile_fr.txt'; {$ENDIF}
      {$IFDEF ENGLISH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_en.txt'; {$ENDIF}
      {$IFDEF SPANISH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_es.txt'; {$ENDIF}
      {$IFDEF ITALIAN_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_it.txt'; {$ENDIF}
      {$IFDEF DEUTSCH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_de.txt'; {$ENDIF}
      if (Init(MyDocDessin, s, Topics)) then ShowModal
                                        else ShowMessage(Application.ExeName + ' - version: ' + GetGHCaveDrawVersion());
    finally
      Release;
    end;
  end;
end;


function EditerStylesObjets(const MyDocDessin: TDocumentDessin; const PF: TProcRefreshStyles): boolean;
var
  TD: TdlgEditStylesObjets;
begin
  Result := False;
  TD := TdlgEditStylesObjets.Create(Application);
  try
    TD.Initialise(MyDocDessin, PF);
    TD.ShowModal;
  finally
    TD.Release;
  end;
end;

// sélection des zooms
function GetPresetedZoom(const MyTableauZooms: TTableauZooms; const NoZoom: integer): integer;
begin
  Result := -1;
  if (NoZoom = -1) then
  begin
    ShowMessage('Pas de niveau de zoom sauvegardé');
    Exit;
  end
  else
  begin
    (*
    with TdlgSelectZooms.Create(Application) do begin
      try
        TableauZooms := MyTableauZooms;
        QInit;
        ShowModal;
        if (ModalResult = mrOK) then Result := lsbZooms.ItemIndex;
      finally
        Release;
      end;
    end;
    //*)
  end;
end;
// infos base station
procedure DisplayInfosBaseStation(const BP: TBaseStation);
begin
  (*
  with TdlgInfosBaseStation.Create(Application) do begin
    try
      MyBaseStation := BP;
      ShowModal;
    finally
      Release;
    end;
  end;
  //*)
end;
// Question Oui-Non
function QuestionOuiNon(const Msg: string): boolean;
begin
  Result := (MessageDlg(Msg, mtConfirmation,[mbYES, mbNO], 0) = mrYES);
end;
// choix d'une photo
function ChoosePhotoFileName(var PhotoFileName: string): boolean;
begin
  Result := False;
  with TOpenPictureDialog.Create(Application) do begin
    try
      InitialDir := GetGHCaveDrawDirectory();
      if (Execute) then begin
         PhotoFileName := ExtractFileName(Filename);
         Result        := True;
      end;
    finally
      Free;
    end;
  end;
end;
// choix de fichiers
function DoDialogOpenFilename(const Filters: string; var QFileName: TStringDirectoryFileName): boolean;
begin
  Result := False;
  with TOpenDialog.Create(Application) do
  begin
    try
      InitialDir   := ExtractFilePath(QFileName); // Paramstr(0));
      FileName     := QFileName;
      Filter       := Filters;
      if execute then
      begin
        QFileName := UTF8ToSys(FileName);
        Result := True;
      end;
    finally
      Free
    end;
  end;
end;
function DoDialogSaveFilename(const QFilters, QDefaultExt: string; var QFileName: TStringDirectoryFileName): boolean;
begin
  Result := False;
  with TSaveDialog.Create(Application) do begin
    try
      Options      := Options + [ofOverWritePrompt];
      InitialDir   := ExtractFilePath(QFileName);
      FileName     := ExtractFileName(QFileName);
      Filter       := QFilters;
      DefaultExt   := QDefaultExt;
      if (Execute) then
      begin
        QFileName := UTF8ToSys(FileName);
        Result := True;
      end;
    finally
      Free;
    end;
  end;
end;
// paramétrage de la grille de dessin
function ChooseParamsGrid(const PrmsGrd: TParamsGrilleDessin): TParamsGrilleDessin;
begin
  Result := PrmsGrd;
  (*
  with TdlgSetupDrawCtxt.Create(Application) do begin
    try
      ParamsGrilleDessin := PrmsGrd;
      ShowModal;
      if (ModalResult = mrOK) then Result := ParamsGrilleDessin;
    finally
      release;
    end;
  end;
  //*)
end;
// choix d'un dossier
function DoDialogSelectDossier(const QInitialDir: TStringDirectoryFileName; var QDirectorySelected: TStringDirectoryFileName): boolean;
var
  DS: TSelectDirectoryDialog;
begin
  Result := false;
  DS := TSelectDirectoryDialog.Create(Application);
  try
    DS.InitialDir := QInitialDir;
    if (DS.Execute) then
    begin
       QDirectorySelected := DS.FileName;
       result := true;
    end;
  finally
    DS.Free;
  end;

end;

// choix d'une couleur
function ChooseColor(const OldColor: TColor): TColor;
begin
  Result := OldColor;
  with TColorDialog.Create(Application) do begin
   try
     Color := OldColor;
     if Execute then begin
       Result := Color;
     end;
   finally
     Free;
   end;
  end;
end;

// remplir les combobox de style et de groupes
(*
procedure RemplirCmbGroupes(const MyDocDessin: TDocumentDessin;
                            var   cmbDest    : TComboBox;
                            const Idx: Integer);
var
  i: integer;
  GRP: TGroupeEntites;
begin
   // liste des groupes
  with cmbDest do begin
    Clear;
    for i:=0 to MyDocDessin.GetNbGroupes - 1 do
    begin
      GRP := MyDocDessin.GetGroupeByIdx(i);
      Items.Add(Format('%d - %s',[GRP.IDGroupeEntites, TxtGHCAnsiToUTF8(GRP.NomGroupe)]));
    end;
    ItemIndex := Idx;
  end;
end;
//*)
procedure EditerSymboleObject(const MyDocuDessin: TDocumentDessin;
                              const CurrentIdxSymbole: integer;
                              const ModeEdition: TModeEdition;
                              const IdxGroupeCourant: TIDGroupeEntites;
                              const TypeDeSymbole   : TNatureObjetSymbole;
                              const OP: TSymbole;
                              const AbsolutePos: TPoint3Df;
                              const ProcRepercuter: TProcRefreshVues);
begin
  with TdlgSymbolObject.Create(Application) do
  begin
    try
      Initialiser(MyDocuDessin,
                  CurrentIdxSymbole,
                  ModeEdition,
                  IdxGroupeCourant,
                  TypeDeSymbole,
                  OP,
                  AbsolutePos,
                  ProcRepercuter);
      ShowModal;
    finally
      Release;
    end;
  end;
end;

procedure EditerTextObject(const MyDocuDessin: TDocumentDessin;
                           const CurrentIdxTexte: integer;
                           const ModeEdition: TModeEdition;
                           const IdxGroupeCourant: TIDGroupeEntites;
                           const TypeDeTexte     : TNatureObjetTexte;
                           var   OT: TTextObject;
                           const AbsolutePos: TPoint3Df;
                           const ProcRepercute   : TProcRefreshVues);

begin
  with TdlgTextObject.Create(Application) do
  begin
    try
      Initialiser(MyDocuDessin, CurrentIdxTexte, ModeEdition, IdxGroupeCourant, TypeDeTexte, OT, AbsolutePos, ProcRepercute);
      ShowModal;
    finally
      Release;
    end;
  end;
  //*)
end;

// éditeur graphique de sections transversales
function  EditeurSectionTransversale(const ST: TBaseStation; const QIDGroupe: TIDGroupeEntites): integer;
begin
  Result := -1;
  AfficherMessage(Format('Editeur de section pour %s (ID = %d)',[ST.IDTerrain, ST.IDStation]));
  if (ST.IDStation = -1) then begin
    ShowMessage('Sélectionnez un point topo');
    Exit;
  end;
  (*
  with TdlgEditeurSection.Create(Application) do begin
    try
      MyStation := St;
      MyGroupeID := QIDGroupe;

      if Initialize then begin
        ShowModal;
      end else
        ShowMessage('Erreur de démarrage de l''éditeur de sections');
    finally
      Release;
    end;
  end;
  /:*)
end;

// dialogue d'édition de groupe
function EditerGroupe(var MyGroupe: TGroupeEntites; const ME: TModeEdition; const QFiltres: string; out DoSetThisGroupeAsCurrent: boolean): boolean;
var
  TD: TdlgGroupe;
begin
  Result := False;
  TD := TdlgGroupe.Create(Application);
  try
    TD.Initialise(MyGroupe, ME, QFiltres);
    TD.ShowModal;
    if (TD.ModalResult = mrOK) then
    begin
      MyGroupe := TD.GetGroupe();
      DoSetThisGroupeAsCurrent := TD.GetDoSetGrpAsCurrent();
      Result := True;
    end;
  finally
    TD.Release;
  end;
end;


function EditerSuperGroupe(var MySuperGroupe: TSuperGroupe): boolean;
begin

  result := False;
end;
// garnir une combobox avec les types de symboles
procedure RemplirComboTypesSymboles(const CB: TComboBox; const Idx: integer);
var
  i: Integer;
begin
  CB.Clear;
  for i := 0 to High(ListeNatureSymboles) do
    CB.Items.Add(ListeNatureSymboles[i]);
  CB.DropDownCount := CB.Items.Count;
  CB.ItemIndex     := Idx;
end;

// garnir combobox avec types de courbes
procedure RemplirComboTypesCourbes(const CB: TComboBox; const Idx: integer);
var
  i: Integer;
begin
  CB.Clear;
  for i := 0 to High(ListeStylesCourbes) do
    CB.Items.Add(ListeStylesCourbes[i]);
  CB.DropDownCount := CB.Items.Count;
  CB.ItemIndex     := Idx;
end;
// garnir combobox avec types de polygones
procedure RemplirComboTypesPolygones(const CB: TComboBox; const Idx: integer);
var
  i: Integer;
begin
  CB.Clear;
  for i := 0 to High(ListeStylesPolygones) do
    CB.Items.Add(ListeStylesPolygones[i]);
  CB.DropDownCount := CB.Items.Count;
  CB.ItemIndex     := Idx;
end;
// garnir combobox avec types de lignes
procedure RemplirComboTypesSimplesLignes(const CB: TComboBox; const Idx: integer);
var
  i: Integer;
begin
  CB.Clear;
  for i := 0 to High(ListeStylesLignes) do
    CB.Items.Add(ListeStylesLignes[i]);
  CB.DropDownCount := CB.Items.Count;
  CB.ItemIndex     := Idx;
end;

// garnir combobox avec types de polygones
procedure RemplirComboTypesTextes(const CB: TComboBox; const Idx: integer);
var
  i: Integer;
begin
  CB.Clear;
  for i := 0 to High(ListeStylesTextes) do
    CB.Items.Add(ListeStylesTextes[i]);
  CB.DropDownCount := CB.Items.Count;
  CB.ItemIndex     := Idx;
end;


function EditerScrap(const MyDocDessin: TDocumentDessin; var MyScrap: TScrap): boolean;
begin
  Result := False;
  with TdlgEditScrap.Create(Application) do
  begin
    try
      Initialise(MyDocDessin, MyScrap);
      ShowModal;
      if (ModalResult = mrOK) then
      begin
        MyScrap := GetScrap();
        Result := True;
      end;
    finally
      Release;
    end;
  end;
end;

// ajout d'images
function DoDialogTImageObject(const FD: TDocumentDessin;
                              var EWE: TImageObject;
                              const QInitialDir: string;
                              const ImgPosition: TPoint3Df;
                              const DoCreateImage: boolean): boolean;
var
  DLG: TdlgTImageObject;
begin
  Result := false;
  DLG := TdlgTImageObject.Create(Application);
  try
    DLG.Initialiser(FD, QInitialDir, DoCreateImage);
    //DLG.setPositionPtRefImage(EWE.PositionXY);
    DLG.setPositionPtRefImage(MakeTPoint2Df(ImgPosition.X, ImgPosition.Y), 50.00);
    DLG.ShowModal;
    if (DLG.ModalResult = mrOK) then
    begin
      EWE := DLG.getImageObject();
      Result := True;
    end;
    DLG.Finaliser();
  finally
    DLG.Free;
  end;
end;
// affichage d'un petit éditeur de texte intégré

procedure DisplayEditeurTexte(const QText: TStringList; const QTitreDlg: string; const QM: TModeSelectionEntites); overload;
var
  DLG: TdlgEditeurTexte;
begin
  DLG := TdlgEditeurTexte.Create(Application);
  try
    DLG.SetText(QText);
    DLG.Caption := QTitreDlg;
    DLG.ShowModal;
  finally
    DLG.Release;
  end;
end;
// affiche le message d'erreur et met les consoles en avant plan
procedure NotifyError(const Msg: string);
begin
  ShowMessage(Msg);
  dlgProcessing.SetFocus;
end;

function EditerSectionTransversale(const FD: TDocumentDessin; const BP: TBaseStation; const QOffsetX, QOffsetY: double): boolean;
var
  DLG: TfrmEditeurSections;
begin
  result := false;
  DLG := TfrmEditeurSections.Create(Application);
  try
    if (DLG.Initialiser(FD, BP, QOffsetX, QOffsetY)) then
    begin
      DLG.ShowModal;
      DLG.Finaliser();
      result := True;
    end;
  finally
    DLG.Release;
  end;

end;

procedure DisplayDlgExportToTherion(const FD: TDocumentDessin);
var
  TD: TdlgExportToTherion;
begin
  TD := TdlgExportToTherion.Create(Application);
  try
    if (TD.Initialiser(FD)) then
    begin
      TD.ShowModal;
    end;
  finally
    TD.Release;
  end;
end;

procedure DisplayDlgExportGIS(const FD: TDocumentDessin);
var
  TD: TdlgExportSIG;
begin
  TD := TdlgExportSIG.Create(Application);
  try
    if (TD.Initialiser(FD)) then
    begin
      TD.ShowModal;
    end;
  finally
    TD.Release;
  end;
end;

function SelectIndexGroupe(const FD: TDocumentDessin;
                           const IdxGroupe: TIDGroupeEntites;
                           out IdxSelected: TIDGroupeEntites): boolean;
var
  DG: TdlgListeGroupes;
begin
  Result := false;
  DG := TdlgListeGroupes.Create(Application);
  try
    if (DG.Initialiser(FD, IdxGroupe)) then
    begin
      DG.ShowModal;
    end;
  finally
    DG.Release;
  end;

end;

function CreateFichePointTopo(const FD: TDocumentDessin;
                              const QTextures: TTexturesPolygonObject;
                              const Station: TBaseStation): boolean;
var
  LesFichesPointTopo: TFichesPointTopo;
  EWE: RawByteString;
begin
  ShowMessageFmt('%s', [GetToporobotIDStationAsString(Station, True)]);
  LesFichesPointTopo := TFichesPointTopo.Create;
  EWE := ExtractFilePath(paramstr(0)) + '\' + 'toto.pdf';
  try
    if (LesFichesPointTopo.Initialiser(FD, QTextures, EWE)) then
    begin
      LesFichesPointTopo.AddStation(Station);
      LesFichesPointTopo.CreerLesFiches();
    end;
    LesFichesPointTopo.Finaliser();
  finally
    LesFichesPointTopo.Free;
  end;


end;

end.

