unit CallDialogsStdVersion;
// CallDialogs pour la version std de GHTopo

// Date: 19/04/2012
// Statut: Fonctionnel
// Ajouts en cours au fil du portage
// 17/01/2014: Nouveau dialogue Distance
// 26/04/2014: Centralisation du TSaveDialog
// 22/05/2015: Ajout d'un dialogue de recherche de IDTerrain
// 13/10/2015: Fixation du bug des chemins d'accès Unicode (caractères accentués)
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  {$IFDEF MSWINDOWS}
    Windows,
  {$ENDIF}
  StructuresDonnees,
  Common, Classes, SysUtils, FileUtil, process,
  ToporobotClasses2012,
  ConvertisseurJPC,
  UnitObjetSerie,
  UnitEntitesExtended,
  UnitClassPalette,
  UnitClasseMaillage,
  UnitListesSimplesWithGeneriques,
  unitProfilTopo
  , Forms, Dialogs, Controls, Graphics, StdCtrls
  , Grids
  , Printers      //, OSPrinters, // OSPrinters est INDISPENSABLE ici !!
  , PrintersDlgs
  , Printer4Lazarus
  ;
// appel de l'outil Calculette
function CallCalculette(const FD: TToporobotStructure2012; const Expression: string; const QTabIndex: integer; var Coords: TPoint2Df; out ResultatCalcul: double; out Declimag: double): boolean;

// éditeur de texte interne
procedure DisplayTextEditor(const FileTexte: TStringDirectoryFilename; const EditMode: boolean);
// sélection expé, code, ... dans liste
function  SelectionDansListe(const DT: TToporobotStructure2012; const Mode: TModeSelectionListe; const UseInternalIndex: boolean; out SelectedIndex: integer): boolean;


// sélection de couleur Toporobot
function SelectionCouleurToporobot(const OldIdx: integer): Integer;
// choix d'une couleur
function ChooseColor(const OldColor: TColor): TColor;
// choix d'un type de galerie
function ChooseTypeVisee(const T: integer): integer;

// paramétrage d'un onglet de vue 2D
function ParametrerOngletVue2D(var O: TVue2DParams): boolean;
// paramétrage d'un onglet de vue 3D
function ParametrerOngletVue3D(var O: TVue3DParams): boolean;
// paramétrage du visualisateur coupe développée
function ParametrerOngletCoupeDeveloppee(var O: TCoupeDeveloppeeParams): boolean;

// définition d'attributs de texte pour imprimante
function SelectAttributsTexte(const AT: TTexteAttributs): TTexteAttributs;

// système d'aide
procedure DisplayHelpSystem(const Topics: string);
// fenêtres avec passage d'un TBDDEntites
// -- centre d'impression
function DisplayCentreImpressionExt(const QBDDTopo     : TToporobotStructure2012;
                                    const QVue2DParams : TVue2DParams;
                                    const QBDDEntites  : TBDDEntites): boolean;
// -- export SIG
function DisplayExportSIG(const QBDDEntites: TBDDEntites): boolean;
// -- statistiques
function DisplayStatistiquesExt(const QDocTopo: TToporobotStructure2012; const QBDDEntites: TBDDEntites): boolean;
// vues 3D GDI
procedure DisplayVue3DGDIExt(const QBDDEntites: TBDDEntites; const DT: TToporobotStructure2012; const QMaillage: TMaillage; const QFiltres: string);
// vue 3D OpenGL
{$IFDEF USE_VIEWER_OPENGL}
procedure DisplayVue3DOpenGLExt(const QBDDEntites: TBDDEntites;
                                const QMaillage: TMaillage;
                                const QFiltres: string);
{$ENDIF}
// affichage d'une station (détails + QR)
procedure AfficherInfosQRCodeStation(const FD: TBDDEntites;
                                     const ST: TEntiteEtendue);
// Capture d'écran
procedure CapturerVueVersPNG(const QHandle: THandle; const QFileName: string);

// Enregistrement de fichiers
function DoDialogSaveFile(const QFilter: string; const QDefaultExt: string; var QFileName: TStringDirectoryFilename; out idxFilter: integer): boolean;
function DoDialogOpenFile(const QFilter: string; const QDefaultExt: string; var QFileName: TStringDirectoryFilename): boolean; overload;
function DoDialogOpenFile(const QInitialdir:TStringDirectoryFilename;
                          const QFilter: string;
                          const QDefaultExt:
                          string; var QFileName: TStringDirectoryFilename): boolean; overload;
function DoDialogOpenFileGHTopo(const QListeRecents: TStringList;
                                var   QFileName: TStringDirectoryFilename): boolean; overload;
// Editeur de coupe développée
procedure AfficherEditeurCoupeDeveloppee(const FT: TToporobotStructure2012);
// Wrapper pour InputQuery
function GHTopoInputQuery(const ACaption, APrompt: string; var QValeur: string): boolean; inline;
// Question Oui-Non
function GHTopoQuestionOuiNon(const Msg: string): boolean;

// liste les stations les plus proches d'une station et retourne le numéro série/station de la station choisie
function ChooseNearStationAtPoint(const BTC: TBDDEntites;
                                  const QNumeroSerie, QNumeroStation: integer;
                                  const RayonDeCapture: double;
                                  out EntiteSelected: TEntiteEtendue): boolean;
// choisir un ID Terrain
function SelectIDTerrain(const DC: TToporobotStructure2012; var QIDxTerrain: integer): boolean;
function DispDlgFindIDTerrain(const QDC: TToporobotStructure2012;
                              out   QDoMatchExact: boolean;
                              out   QFindWhat: string
                              ): boolean;


//function DoDialogOpenFilename(const Filters: string; var QFileName: TStringDirectoryFilename): boolean;
//function DoDialogSaveFilename(const QFilters, QDefaultExt: string; var QFileName: TStringDirectoryFilename): boolean;
//******************************************************************************
// FONCTIONS SPECIFIQUES A LA VERSION TABLETTE
//******************************************************************************


// édition de codes, expés, secteurs, réseaux
// Ces fonctions modifient un record et retournent une autorisation (O/N). Point
// Le paramètre FD ne sert que pour les opérations de contrôle.
function EditerEntrance(const FD: TToporobotStructure2012;  const QIdx: integer; const B: TBDDEntites; var EE: TEntrance): boolean;
function EditerReseau(const FD: TToporobotStructure2012; const QIdx: integer; var RS: TReseau): boolean;
function EditerSecteur(const FD: TToporobotStructure2012; const QIdx: integer; var SS: TSecteur): boolean;
function EditerCode(const FD: TToporobotStructure2012; const QIdx: integer; var CC: TCode): boolean;
function EditerExpe(const FD: TToporobotStructure2012; const QIdx: integer; var EX: TExpe): boolean;
function EditerUneSerie(const FD: TToporobotStructure2012; const PD: TBDDEntites; const QIdx: integer; var EX: TObjSerie): boolean;
function EditerAnnotation(const FD: TToporobotStructure2012; const QIdx: integer; var AN: TAnnotation): boolean;

// recopie vers le bas dans une grille
procedure GridUndocopy(const MyGrid: TStringGrid; const AutoIncremented: boolean);
// suppression de lignes
procedure GridEraseLines(const MyGrid: TStringGrid);
// insertion de lignes
procedure GridInsertLignes(const MyGrid: TStringGrid);
// se positionner sur une cellule de la grilles
procedure GridSetCurrentRow(const MyGrid: TStringGrid; const R, C: integer);

// afficher les stations au voisinage du point topo courant
function  DisplayStationsNearToCurrStation(const FBDDEntites: TBDDEntites;
                                          const BasePt: TEntiteEtendue): boolean;
// sélection d'un topographe
function SelectTopographes(out Topographes: string): boolean;

// fusionner des topographies
procedure DispFusionTopographies(const FD: TToporobotStructure2012;
                                 const CV: TConversionSysteme);
// imprimante
function SelectionnerImprimante(var MyPrinter: TPrinter): boolean;
// choix de code, d'expé, secteur
function SelectionnerSecteurCodeExpe(const FD: TToporobotStructure2012;
                                     const MyRow: integer;
                                     const DoUseLastItems: boolean;
                                     var MyIdxSecteur: integer;
                                     var MyIdxCode: TNumeroCode;
                                     var MyIdxExpe: TNumeroExpe): boolean;

// gestion des visées rayonnantes
function DispGestionDesViseesRayonnantes(const FD: TToporobotStructure2012;
                                     const ED: TBDDEntites): boolean;
// lister les dépendances d'une série
function ListerDependancesDeUneSerie(const FD: TToporobotStructure2012;
                                     const IdxSerie: integer): boolean;
// maintenance du document
procedure DispMaintenanceDocument(const FD: TToporobotStructure2012;
                                  const FE: TBDDEntites;
                                  const FP: TPalette256;
                                  const FC: TConversionSysteme;
                                  const PostAction: TProcedureOfObject);
// affichage de la distance entre deux stations
function DispDistanceEntreStations(const FD: TToporobotStructure2012;
                                   const FE: TBDDEntites;
                                   const S1, S2: TEntiteEtendue;
                                   const DescriptionAction: string;
                                   const CaptionAdditionalAction: string): integer;

procedure RunExternalProgram(const ProgName: string; const Params: string; const FileToOpen: string);
// affichage d'un profil topo
function  DisplayProfilTopo(const DT: TToporobotStructure2012; const QMaillage: TMaillage; var ProfilTopo: TProfilTopo): TModalResult;
// gestion des filtres
function DisplayAndEditFiltres(const FD: TToporobotStructure2012; var F: string): boolean;
// afficher les profils topo
function DisplayListeProfils(const DT: TToporobotStructure2012; const MyMaillage: TMaillage): boolean;


// afficher un clavier virtuel (pour Raspberry)
{$IFDEF RASPBERRY_PI}
function  DisplayClavierVirtuel(const QTextBox: TEdit; const ModeClavier: TModeClavierVirtuel): boolean;
function  DisplayPaveNumVirtuel(var Value: double): boolean;
{$ENDIF RASPBERRY_PI}
implementation
uses
  // Les virgules de séparation d'unités sont placées devant elles
  // ce qui facilite la compilation conditionnelle
  dlgToporobotColorNew   // sélecteur de couleur
  , dlgSelectDansListes // sélection dans une liste
  , dlgSelectionTypeVisee // type visée
  // CentreImpression,      // outil d'impression (obsolète)
  , dlgHelpSystem         // système d'aide
  , dlgCalculette         // calculette
  , dlgParametrerOngletVue2D // dialogue de configuration d'une vue 2D
  , MiniTextEditor        // éditeur de texte incorporé
  , dlgSetTexteAttributs  // selecteur de fontes imprimante
  //dlgSibertConvertisseur, // pilote pour le convertisseur Sibert
  // fenêtres utilisant un TBDDEntites
  , CentreImpressionExt   // centre d'impression
  , VueEn3DExt            // visualisateur 3D GDI
  , frmEditSerie          // éditeur de série isolée

  {$IFDEF USE_VIEWER_OPENGL}
  , VueEn3DOpenGLExt         // visualisateur 3D OpenGL
  {$ENDIF}
  , dlgExportVersSIGExt   // export vers SIG
  , dlgStatistiquesExt    // stats (nouvelle version)
  //dlgStatistiquesExt2,    // stats (nouvelle version)

  , frmInfosDetailleesStation // Infos station + QRCode
  {$IFDEF USE_COUPES_DEVELOPPEES}
  , dlgCoupeDeveloppee      // éditeur de coupe développée
  {$ELSE}
  {$ENDIF}


  , dlgListeNearViseesTo    // sélecteur de stations voisines
  , frmSelectIDTerrain      // sélection d'IDTerrain
  , frmFindAStation

  // dialogues d'édition réseau, secteur, expés, codes
  , frmEditEntrance
  , frmEditReseau
  , frmEditSecteur
  , frmEditCode
  , frmEditExpe
  , frmEditAnnotation
  , frmClavierVirtuel
  , frmPaveNumVirtuel
  , frmSelectTopographes // topographes
  , frmFusionnerTopos
  , frmDependancesDeUneSerie
  , frmMaintenanceDocument
  , frmSelectGHTopoFiles
  , frmDistanceEntreDeuxPoints
  , frmEditFiltres
  , frmSelectSecteurCodeExpe
  , frmLesViseesRayonnantes
  , frmProfilTopo
  , frmListeProfilsTopo
  ; // et ce point-virgule est indispensable
//******************************************************************************
// calculette et convertisseur de coordonnées
function CallCalculette(const FD: TToporobotStructure2012; const Expression: string; const QTabIndex: integer; var Coords: TPoint2Df; out ResultatCalcul: double; out Declimag: double): boolean;
var
  TD: TfrmCalculette;
begin
  Result := False;
  ResultatCalcul := 0.00;

  TD := TfrmCalculette.Create(Application);

  try
    TD.SetDefaultExpression(Expression);
    TD.SetCoordonnees(Coords);
    TD.SetTabIndex(QTabIndex);
    TD.SetDocuTopo(FD);
    TD.ShowModal;

    if (TD.ModalResult = mrOK) then
    begin
      ResultatCalcul:= TD.GetResultatCalcul;
      Declimag      := TD.GetDeclimag;
      Coords        := TD.GetCoordonnees;
      Result := True;
    end;
  finally
    TD.Release;
  end;
end;

// Editeur de texte interne
// TODO: L'implémenter (fonction vide)
//EditMode=0 : Consultation; 1 = Edition;
procedure DisplayTextEditor(const FileTexte: TStringDirectoryFilename; const EditMode: boolean);
begin
  with TdlgEditeur.Create(Application) do begin
    try
      if (InitialiseEditor(FileTexte, EditMode)) then
        ShowModal;
    finally
      Free;
    end;
  end;
  //*)
end;

// Question Oui-Non
function GHTopoQuestionOuiNon(const Msg: string): boolean;
begin
  Result := (MessageDlg(GetResourceString(Msg), mtConfirmation,[mbYES, mbNO], 0) = mrYES);
end;

// sélection expé, code, ... dans liste
// DONE: Traiter les index - Retourne désormais les index TOPOROBOT (pouvant être <=> des index internes)
function SelectionDansListe(const DT: TToporobotStructure2012; const Mode: TModeSelectionListe; const UseInternalIndex: boolean; out SelectedIndex: integer): boolean;
var
  TD: TdlgSelectElement;
begin
  // ancien index = en cas d'abandon, reprend cet index
  Result := false;
  TD := TdlgSelectElement.Create(Application);
  SelectedIndex := 0;
  try
    if (TD.InitialiseListeSelection(DT, Mode, SelectedIndex, UseInternalIndex)) then
    begin
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then
      begin
        SelectedIndex := TD.GetIndexElement(UseInternalIndex); //  Result := GetNumeroElement;      ;
        Result := True;
      end;
    end
    else
    begin
      ShowMessage(rsMATCHNOTFOUND);
    end;
  finally
    TD.Release;
  end
end;

// sélection de couleurs issues de palettes
// Version simplifiée: les autres palettes ne sont pas utilisées bien
// qu'opérationnelles. La palette AutoCAD est utilisée dans un autre module
function SelectionCouleurToporobot(const OldIdx: integer): integer;
var
  TD: TDialogSelTopoColor;
begin
  Result := OldIdx;
  TD := TDialogSelTopoColor.Create(Application);
  try
      TD.Initialiser(OldIdx);
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then Result  := TD.GetIdxColor;
  finally
    TD.Release;
  end;
end;


// choix d'une couleur RGB
function ChooseColor(const OldColor: TColor): TColor;
begin
  Result := OldColor;
  with TColorDialog.Create(Application) do
  begin
    try
      Color := OldColor;
      if (Execute) then Result := Color;
    finally
      Free;
    end;
  end;
end;



//******************************************************************************
// Fenêtres utilisant un TBDDEntites
function DisplayCentreImpressionExt(const QBDDTopo     : TToporobotStructure2012;
                                    const QVue2DParams : TVue2DParams;
                                    const QBDDEntites  : TBDDEntites): boolean;
var
  PT: TfrmPrintingCenterExt;
begin
  Result := False;
  PT := TfrmPrintingCenterExt.Create(Application);
  try
    if (PT.InitialiseByPointer(QBDDEntites, QVue2DParams, '')) then
    begin
      PT.Caption := EnlevePerluete(Format(GetResourceString(rsPRN_TITLE), ['']));
      PT.ShowModal;
    end;
    Result := True;
  finally
    PT.Release;
  end;
end;
// export SIG
function DisplayExportSIG(const QBDDEntites: TBDDEntites): boolean;
begin
  Result := False;
  with TfrmExportVersSIGExt.Create(Application) do
  begin
    try
      if (InitialiseByPointer(QBDDEntites, '')) then
      begin
        ShowModal;
      end;
      Result := True;
    finally
      Release;
    end;
  end;
end;

// statistiques
function DisplayStatistiquesExt(const QDocTopo   : TToporobotStructure2012;
                                const QBDDEntites: TBDDEntites): boolean;
var
  FS : TfrmStatistiquesExt;
  QDevelViseesVisibles: double;
begin
  AfficherMessage(Format('DisplayStatistiquesExt: %d - %s entites', [QBDDEntites.GetNbEntitesVisees(), Application.ExeName]));
  Result := False;
  FS := TfrmStatistiquesExt.Create(Application);
  try
    if (FS.InitialiseByPointer(QDocTopo, QBDDEntites, '')) then
    begin
      FS.Caption := EnlevePerluete(GetResourceString(rsSTATISTIQUES));
      FS.ShowModal;
      QBDDEntites.MetaFiltre('', QDevelViseesVisibles);
    end;
    Result := True;
  finally
    FS.Release;
  end;
end;
// vue 3D GDI
procedure DisplayVue3DGDIExt(const QBDDEntites: TBDDEntites; const DT: TToporobotStructure2012; const QMaillage: TMaillage; const QFiltres: string);
var
  TD: TfrmVue3DExt;
begin
  AfficherMemoryUsage();
  TD := TfrmVue3DExt.Create(Application);
  try
    if (TD.InitialiseByPointer(QBDDEntites, DT, QMaillage, QFiltres)) then
    begin
      TD.Caption := EnlevePerluete(GetResourceString(rsVUE3D));
      TD.ShowModal;
    end;
  finally
    TD.Release;
  end;
  AfficherMemoryUsage();
end;
// vue 3D OpenGL
{$IFDEF USE_VIEWER_OPENGL}
procedure DisplayVue3DOpenGLExt(const QBDDEntites: TBDDEntites;
                                const QMaillage: TMaillage;
                                const QFiltres: string);
begin
  AfficherMemoryUsage();
  with TfrmVueEn3DOpenGLExt.Create(Application) do
  begin
    try
      if (InitialiseByPointer(QBDDEntites,
                              QFiltres)) then
      begin
        Caption := EnlevePerluete(GetResourceString(rsRENDU3D));
        ShowModal;
      end;
    finally
      Release;
    end;
  end;
  AfficherMemoryUsage();
end;
{$ENDIF}

//******************************************************************************
// choix d'un type de galerie
function ChooseTypeVisee(const T: integer): integer;
begin
  Result := T;
  with TdlgSelectTypeVisee.Create(Application) do
  begin
    try
      SetTypeVisee(T);
      ShowModal;
      if (ModalResult = mrOK) then Result := GetTypeVisee();
    finally
      Release;
    end;
  end;
end;

function ParametrerOngletVue2D(var O: TVue2DParams): boolean;
begin
  Result := false;
  with TfrmParametrerOngletVue2D.Create(Application) do
  begin
    try
      SetModeParametrageDialog(mpdVUE2D);
      SetValuesOnglet2D(O);
      ShowModal;
      if (ModalResult = mrOK) then
      begin
        O := GetValuesOnglet2D();
        result := True;
      end;
    finally
      Release;
    end;
  end;
end;

function ParametrerOngletVue3D(var O: TVue3DParams): boolean;
begin
  Result := false;
  with TfrmParametrerOngletVue2D.Create(Application) do
  begin
    try
      SetModeParametrageDialog(mpdVUE3D);
      SetValuesOnglet3D(O);
      ShowModal;
      if (ModalResult = mrOK) then
      begin
        O := GetValuesOnglet3D();
        Result := True;

      end;
    finally
      Release;
    end;
  end;
end;
function ParametrerOngletCoupeDeveloppee(var O: TCoupeDeveloppeeParams): boolean;
begin
  Result := false;
  with TfrmParametrerOngletVue2D.Create(Application) do
  begin
    try
      SetModeParametrageDialog(mpdCOUPE_DEVELOPPEE);
      SetValuesCoupeDeveloppee(O);
      ShowModal;
      if (ModalResult = mrOK) then
      begin
        O := GetValuesCoupeDeveloppee();
        Result := True;
      end;
    finally
      Release;
    end;
  end;
end;

function SelectAttributsTexte(const AT: TTexteAttributs): TTexteAttributs;
begin
  Result := AT;
  with TfrmSelectTexteAttributs.Create(Application) do
  begin
    try
      SetTexteAttributs(AT);
      ShowModal;
      if (ModalResult = mrOK) then Result := GetTexteAttributs;
    finally
      Release;
    end;
  end;
end;


procedure DisplayHelpSystem(const Topics: string);
var
  s: string;
  TD: TfrmHelpSystem;
begin
  {$IFDEF FRENCH_MESSAGES}   s:= GetGHTopoDirectory() + 'HelpFile_fr.xml'; {$ENDIF}
  {$IFDEF ENGLISH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_en.txt'; {$ENDIF}
  {$IFDEF SPANISH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_de.txt'; {$ENDIF}
  {$IFDEF ITALIAN_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_it.txt'; {$ENDIF}
  {$IFDEF DEUTSCH_MESSAGES}  s:= GetGHTopoDirectory() + 'HelpFile_de.txt'; {$ENDIF}
  TD := TfrmHelpSystem.Create(Application);
  try
    if (not TD.Initialiser(s, Topics)) then AfficherMessageErreur(Format(GetResourceString(rsMSG_FILENOTFOUND),[s]));
    TD.ShowModal;
  finally
    TD.Release;
  end;
end;


procedure AfficherInfosQRCodeStation(const FD: TBDDEntites;
                                     const ST: TEntiteEtendue);
var
  TD: TdlgInfosQRStation;
begin
  TD := TdlgInfosQRStation.Create(Application);
  try
    TD.Initialise(FD, ST);
    TD.ShowModal;
  finally
    TD.Release;
  end;
end;

//******************************************************************************
// Capture d'écran
{$IFDEF MSWINDOWS}

procedure CapturerVueVersPNG(const QHandle: THandle; const QFileName: string);
var
  Bitmap: TBitmap; //TFPImageBitmap;
  DC    : HDC;
begin
  AfficherMessage('CapturerVueVersPNG');
  try
    Bitmap := TBitmap.Create;
    DC     := GetDC(QHandle); // Unité windows indispensable
    Bitmap.LoadFromDevice(DC);
    ReleaseDC(QHandle, DC);
    Bitmap.SaveToFile(QFileName);
  finally
    Bitmap.Free;
  end;
end;
{$ENDIF}
{$IFDEF LINUX}
procedure CapturerVueVersPNG(const QHandle: THandle; const QFileName: string);
begin
  ;
end;
{$ENDIF}
{$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
procedure CapturerVueVersPNG(const QHandle: THandle; const QFileName: string);
begin
  ;
end;
{$ENDIF}
// Enregistrement de fichiers
function DoDialogSaveFile(const QFilter: string;
                          const QDefaultExt: string;
                          var QFileName: TStringDirectoryFilename;
                          out idxFilter: integer): boolean;
begin
  Result := False;
  with TSaveDialog.Create(Application) do
  begin
    try
      Options      := Options + [ofOverWritePrompt];
      InitialDir   := ExtractFilePath(QFileName);
      FileName     := ExtractFileName(QFileName);
      Filter       := QFilter + '|Tous (*.*)|*.*';
      DefaultExt   := QDefaultExt;
      if (Execute) then
      begin
        QFileName := FileName;
        idxFilter := FilterIndex;
        Result := True;
      end;
    finally
      Free;
    end;
  end;
end;
// Ouverture de fichiers
function DoDialogOpenFile(const QFilter: string; const QDefaultExt: string; var QFileName: TStringDirectoryFilename): boolean;
begin
  Result := False;
  with TOpenDialog.Create(Application) do
  begin
    try
      FileName   := QFileName;
      InitialDir := GetGHTopoDirectory;
      Filter     := QFilter + '|Tous (*.*)|*.*';
      DefaultExt := QDefaultExt;
      if (Execute) then
      begin
        QFileName := FileName;
        Result    := True;
      end;
    finally
      Free;
    end;
  end;
end;
function DoDialogOpenFile(const QInitialdir:TStringDirectoryFilename;
                          const QFilter: string;
                          const QDefaultExt:
                          string; var QFileName: TStringDirectoryFilename): boolean; overload;
begin
  Result := False;
  with TOpenDialog.Create(Application) do
  begin
    try
      FileName   := QFileName;
      InitialDir := QInitialdir;
      Filter     := QFilter + '|Tous (*.*)|*.*';
      DefaultExt := QDefaultExt;
      if (Execute) then
      begin
        QFileName := FileName;
        Result    := True;
      end;
    finally
      Free;
    end;
  end;

end;
// nouveau dialogue d'ouverture de fichiers GHTopo
function DoDialogOpenFileGHTopo(const QListeRecents: TStringList;
                                var QFileName: TStringDirectoryFilename): boolean; overload;
var
  TD: TdlgSelectGHTopoFiles;
begin
  Result := False;
  TD := TdlgSelectGHTopoFiles.Create(Application);
  try
    if (TD.Initialiser(QListeRecents)) then
    begin
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then
      begin
        QFileName := TD.GetFileName();
        exit(True);
      end;
    end;
  finally
    TD.Finaliser();
    TD.Release;
  end;
end;


// Editeur de coupe développée
procedure AfficherEditeurCoupeDeveloppee(const FT: TToporobotStructure2012);
var
  TD: TfrmCoupeDeveloppee;
begin
  {$IFDEF USE_COUPES_DEVELOPPEES}
  TD := TfrmCoupeDeveloppee.Create(Application);
  try
    TD.Initialiser(FT);
    TD.ShowModal;
  finally
    TD.Release;
  end;
  //*)
  {$ELSE}
  ShowMessage('Fonctionnalité non disponible dans cette version' + #13#10 +
              '(adaptations en cours)');
  {$ENDIF}
end;




function GHTopoInputQuery(const ACaption, APrompt: string; var QValeur: string): boolean;
begin
  Result := InputQuery(ACaption, APrompt, QValeur);
end;
// liste les stations les plus proches d'une station et retourne le numéro série/station de la station choisie
function ChooseNearStationAtPoint(const BTC: TBDDEntites;
                                  const QNumeroSerie, QNumeroStation: integer;
                                  const RayonDeCapture: double;
                                  out   EntiteSelected: TEntiteEtendue): boolean;
var
  EWE: TEntiteEtendue;
  BB : TfrmListeNearViseesTo;
begin
  Result := false;
  // d'abord, on recherche si la station (S, P) existe
  if (not BTC.GetEntiteViseeFromSerSt(QNumeroSerie, QNumeroStation, EWE)) then
  begin
    ShowMessageFmt('Station %d.%d introuvable', [QNumeroSerie, QNumeroStation]);
    Exit; // si introuvable, on sort direct
  end;
  // ensuite on crée le dialogue
  BB := TfrmListeNearViseesTo.Create(Application);
  try
    if (BB.Initialiser(BTC, QNumeroSerie, QNumeroStation, RayonDeCapture)) then
    begin
      BB.ShowModal;
      if (BB.ModalResult = mrOK) then
      begin
        EntiteSelected := BB.GetResultEntite;
        if ((EntiteSelected.Entite_Serie = -1) and (EntiteSelected.Entite_Station = -1)) then Exit;
        Result := True;
      end;
      BB.Finaliser();
    end;
  finally
    BB.Release;
  end;
end;

function SelectIDTerrain(const DC: TToporobotStructure2012; var QIDxTerrain: integer): boolean;
var
  BB: TdlgSelectIDTerrain;
begin
  Result := False;
  BB := TdlgSelectIDTerrain.Create(Application);
  try
    if (BB.Initialiser(DC)) then
    begin
      BB.ShowModal;
      if (BB.ModalResult = mrOK) then
      begin
        QIDxTerrain := BB.GetIDXTerrainFound;
        Result := True;
      end;
    end;
  finally
    BB.Release;
  end;
end;


function DispDlgFindIDTerrain(const QDC: TToporobotStructure2012;
                              out   QDoMatchExact: boolean;
                              out   QFindWhat: string): boolean;
var
  BB: TdlgFindAStation;
begin

  Result := False;
  BB := TdlgFindAStation.Create(Application);
  try
    if (BB.Initialiser(QDC)) then
    begin
      BB.ShowModal;
      if (BB.ModalResult = mrOK) then
      begin
        QFindWhat := BB.GetFindWhat();
        QDoMatchExact := BB.GetDoMatchExact();
        //ShowMessage(QFindWhat + ' - ' + BoolToStr(QDoMatchExact, 'Recherche exacte', 'Rechercher une partie'));
        Result := True;
      end;
    end;
    //*)
  finally
    BB.Release;
  end;
  //*)
end;

//------------------------------------------------------------------------------
function EditerEntrance(const FD: TToporobotStructure2012;
                        const QIdx: integer;
                        const B: TBDDEntites;
                        var EE: TEntrance): boolean;
var
  QDlg: TdlgEditEntrance;
begin
  Result := False;
  try
    QDlg := TdlgEditEntrance.Create(Application);
    QDlg.PrepareDialog(FD, QIdx, EE, 'Entrée', B);
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      EE := QDlg.GetAEntrance();
      Result := True;
    end;
  finally
    QDlg.Release;
  end;
end;
function EditerReseau(const FD: TToporobotStructure2012; const QIdx: integer; var RS: TReseau): boolean;
var
  QDlg: TdlgEditReseau;
begin
  Result := False;
  try
    QDlg := TdlgEditReseau.Create(Application);
    QDlg.PrepareDialog(FD, QIdx, RS, 'Réseau');
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      RS := QDlg.GetAReseau();
      Result := True;
    end;
  finally
    QDlg.Release;
  end;
end;

function EditerSecteur(const FD: TToporobotStructure2012; const QIdx: integer; var SS: TSecteur): boolean;
var
  QDlg: TdlgEditSecteur;
begin
  Result := False;
  try
    QDlg := TdlgEditSecteur.Create(Application);
    QDlg.PrepareDialog(FD, QIdx, SS, 'Secteur');
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      SS := QDlg.GetASecteur();
      Result := True;
    end;
  finally
    QDlg.Release;
  end;
end;

function EditerCode(const FD: TToporobotStructure2012; const QIdx: integer; var CC: TCode): boolean;
var
  QDlg: TdlgEditCode;
begin
  Result := False;
  try
    QDlg := TdlgEditCode.Create(Application);
    QDlg.PrepareDialog(FD, QIdx, CC, 'Instruments topo');
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      CC := QDlg.GetACode();
      Result := True;
    end;
  finally
    QDlg.Release;
  end;
end;

function EditerExpe(const FD: TToporobotStructure2012; const QIdx: integer; var EX: TExpe): boolean;
var
  QDlg: TdlgEditExpe;
begin
  Result := False;
  try
    QDlg := TdlgEditExpe.Create(Application);

    QDlg.PrepareDialog(FD, QIdx, EX, 'Séance topo');
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      EX := QDlg.GetAExpe();
      Result := True;
    end;
  finally
    QDlg.Release;
  end;
end;
// édition d'une série isolée
function EditerUneSerie(const FD: TToporobotStructure2012;
                        const PD: TBDDEntites;
                        const QIdx: integer; var EX: TObjSerie): boolean;
var
  QDlg: TdlgEditSerie;
begin
  Result := False;
  QDlg := TdlgEditSerie.Create(Application);
  try
    QDlg.PrepareDialog(FD, PD, QIdx);
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      Result := QDlg.AppliquerModifsSerie();
    end;
  finally
    QDlg.Release;
  end;
end;

function EditerAnnotation(const FD: TToporobotStructure2012; const QIdx: integer; var AN: TAnnotation): boolean;
var
  QDlg: TdlgEditAnnotation;
begin
  Result := False;
  try
    QDlg := TdlgEditAnnotation.Create(Application);
    QDlg.PrepareDialog(FD, QIdx, AN, 'Annotation');
    QDlg.ShowModal;
    if (QDlg.ModalResult = mrOK) then
    begin
      AN := QDlg.GetAnnotation();
      // avant de valider, on vérifie si le texte n'est pas vide
      Result := (Trim(AN.Texte) <> '');
    end;
  finally
    QDlg.Release;
  end;

end;

//------------------------------------------------------------------------------
// recopie vers le bas
procedure GridUndocopy(const MyGrid: TStringGrid; const AutoIncremented: boolean);
var
  i, j, Nb: integer;
  WU: integer;
  EWE: string;
begin
  Nb := MyGrid.Selection.Bottom - MyGrid.Selection.Top;
  if (Nb = 0) then
  begin
    EWE  := Format('%d', [MyGrid.RowCount - 1]);
    if (GHTopoInputQuery('Recopier vers le bas', 'Nombre de lignes', EWE)) then
      Nb := StrToIntDef(EWE, MyGrid.RowCount - 1)
    else
      Exit;
  end;
  WU := MyGrid.Selection.Top + Nb;
  if (WU > (MyGrid.RowCount -1)) then WU := MyGrid.RowCount - 1;
  for i := MyGrid.Selection.Top + 1 to WU do
  begin
    for j := MyGrid.Selection.Left to MyGrid.Selection.Right do
      MyGrid.Cells[j, i] := MyGrid.Cells[j, MyGrid.Selection.Top];
  end;
end;
// effacement de lignes
procedure GridEraseLines(const MyGrid: TStringGrid);
var
  Nb, WU, i, L: LongInt;
begin
  L := MyGrid.Selection.Top;
  Nb := MyGrid.Selection.Bottom - L;
  // Demande de confirmation
  if (not GHTopoQuestionOuiNon(Format(GetResourceString(rsCDR_SERIES_FLAT_MSG_REMOVE_LINE), [Nb + 1, L]))) then Exit;

  WU := L + Nb;
  if (WU > (MyGrid.RowCount -1)) then WU := MyGrid.RowCount - 1;
  for i := WU downto L do  MyGrid.DeleteRow(i);
end;

procedure GridInsertLignes(const MyGrid: TStringGrid);
begin
  MyGrid.InsertColRow(false, MyGrid.Row);
end;
procedure GridSetCurrentRow(const MyGrid: TStringGrid; const R, C: integer);
begin
  MyGrid.Row    := R;
  MyGrid.TopRow := R;
  if (c > 0) then MyGrid.Col := c;
  MyGrid.SetFocus;
end;

// afficher les stations au voisinage du point topo courant
function DisplayStationsNearToCurrStation(const FBDDEntites: TBDDEntites; const BasePt: TEntiteEtendue): boolean;
var
  QEntiteSelected: TEntiteEtendue;
begin
  Result := ChooseNearStationAtPoint(FBDDEntites,
                                    BasePt.Entite_Serie, BasePt.Entite_Station,
                                    22.00,
                                    QEntiteSelected);
end;

function SelectTopographes(out Topographes: string): boolean;
var
  TD: TdlgSelectTopographes;
begin
  Topographes := '';
  Result := false;
  TD := TdlgSelectTopographes.Create(Application);
  try
    TD.Initialise();
    TD.ShowModal;
    if (TD.ModalResult = mrOK) then
    begin
      Topographes := TD.GetTopographe();
      Result := True;
    end;
  finally
    TD.Release;
  end;
end;



// fusionner des topographies
procedure DispFusionTopographies(const FD: TToporobotStructure2012;
                                 const CV: TConversionSysteme);
var
  TD: TdlgFusionTopos;
begin
  TD := TdlgFusionTopos.Create(Application);
  try
    if (TD.Initialiser(FD, CV)) then
    begin
      TD.ShowModal;
    end;
  finally
    TD.Release;
  end;
end;


function SelectionnerImprimante(var MyPrinter: TPrinter): boolean;
var
  PC: TPrinterSetupDialog;
begin
  result := false;
  PC := TPrinterSetupDialog.Create(Application);
  try
    result := PC.Execute;
  finally
    PC.Free;
  end;
end;

// choix de code, d'expé, secteur
function SelectionnerSecteurCodeExpe(const FD: TToporobotStructure2012;
                                     const MyRow: integer;
                                     const DoUseLastItems: boolean;
                                     var MyIdxSecteur: integer;
                                     var MyIdxCode: TNumeroCode;
                                     var MyIdxExpe: TNumeroExpe): boolean;
var
  TD: TdlgSelectSecteurCodeExpe;
begin
  result := false;
  TD := TdlgSelectSecteurCodeExpe.Create(Application);
  try
    if (TD.Initialiser(FD, MyRow, MyIdxSecteur, MyIdxCode, MyIdxExpe, DoUseLastItems)) then
    begin
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then
      begin
        MyIdxSecteur := TD.GetIdxSecteur();
        MyIdxCode    := TD.GetIdxCode();
        MyIdxExpe    := TD.GetIdxExpe();
        Result := True;
      end;
    end;
  finally
    TD.Release;
  end;
end;

// gestion des visées rayonnantes
function DispGestionDesViseesRayonnantes(const FD: TToporobotStructure2012;
                                         const ED: TBDDEntites): boolean;
var
  TD: TfrmGestionViseesRayonnantes;
begin
  Result := false;
  TD := TfrmGestionViseesRayonnantes.Create(Application);
  try
    if (TD.Initialiser(FD, ED)) then TD.ShowModal;
  finally
    TD.Release;
  end;
end;

// lister les dépendances d'une série
function ListerDependancesDeUneSerie(const FD: TToporobotStructure2012;
                                     const IdxSerie: integer): boolean;
var
  TD: TdlgDependancesSerie;
begin
  result := false;
  TD := TdlgDependancesSerie.Create(application);
  try
    if (TD.Initialiser(FD, IdxSerie)) then
    begin
      TD.ShowModal;
      Result := (TD.ModalResult = mrYes);
    end;
  finally
    TD.Release;
  end;

end;

// maintenance du document
procedure DispMaintenanceDocument(const FD: TToporobotStructure2012;
                                  const FE: TBDDEntites;
                                  const FP: TPalette256;
                                  const FC: TConversionSysteme;
                                  const PostAction: TProcedureOfObject);
var
  TD: TdlgMaintenanceDocTopo;
begin
  TD := TdlgMaintenanceDocTopo.Create(Application);
  try
    if (TD.Initialiser(FD, FE, FP, FC)) then
    begin
      TD.ShowModal;
      if (Assigned(PostAction) and TD.DoPostAction()) then PostAction();
    end;
  finally
    TD.Release;
  end;
end;
// affichage de la distance entre deux stations
function DispDistanceEntreStations(const FD: TToporobotStructure2012;
                                   const FE: TBDDEntites;
                                   const S1, S2: TEntiteEtendue;
                                   const DescriptionAction: string;
                                   const CaptionAdditionalAction: string): integer;
var
  TD: TdlgDistanceBetweenTwoStations;
begin
  result := 0;
  TD := TdlgDistanceBetweenTwoStations.Create(Application);
  try
    if (TD.Initialiser(FD, FE, S1, S2, DescriptionAction, CaptionAdditionalAction)) then
    begin
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then
      begin
        Result += 1;
        if (TD.DoAllowAdditionalAction()) then Result += 1;
      end;
    end;
  finally
    TD.Release;
  end;
end;

// affichage d'un profil topo
function DisplayProfilTopo(const DT: TToporobotStructure2012; const QMaillage: TMaillage; var ProfilTopo: TProfilTopo): TModalResult;
var
  TD: TdlgProfilTopo;
begin
  TD := TdlgProfilTopo.Create(Application);
  try
    if (TD.Initialiser(DT, QMaillage, ProfilTopo)) then
    begin
      TD.ShowModal;
      Result := TD.ModalResult;
      if (Result = mrOK) then
      begin
        ProfilTopo := TD.GetProfilTopo();
      end;
      TD.Finaliser();
    end;
  finally
    TD.Release;
  end;
end;

// gestion des filtres
function DisplayAndEditFiltres(const FD: TToporobotStructure2012; var F: string): boolean;
var
  TD: TdlgEditFiltres;
  WU: Boolean;
begin
  result := false;
  TD := TdlgEditFiltres.Create(Application);
  try
    WU := ('' <> Trim(F));
    if (TD.Initialiser(FD, F, WU)) then
    begin
      TD.ShowModal;
      if (TD.ModalResult = mrOK) then
      begin
        F := TD.GetFiltre();
        Result := True;
      end;
    end;
  finally
    TD.Release;
  end;
end;
// afficher les profils topo
function DisplayListeProfils(const DT: TToporobotStructure2012; const MyMaillage: TMaillage): boolean;
var
  EWE: Boolean;
  TD: TdlgLesProfilsTopo;
begin
  TD := TdlgLesProfilsTopo.Create(Application);
  try
    if (TD.Initialiser(DT, MyMaillage)) then
    begin
      TD.ShowModal;
      TD.Finaliser();
    end;
  finally
    TD.Release;
  end;
end;

procedure RunExternalProgram(const ProgName: string; const Params: string; const FileToOpen: string);
var
  AProcess: TProcess;
   // This is where our program starts to run
begin
// Now we will create the TProcess object, and
  AProcess := TProcess.Create(nil);
  try
    try
      // Tell the new AProcess what the command to execute is.
      // Let's use the FreePascal compiler
      AProcess.CommandLine := ProgName + ' ' + Params + ' ' + FileToOpen;
      // We will define an option for when the program
      // is run. This option will make sure that our program
      // does not continue until the program we will launch
      // has stopped running.
      //AProcess.Options := AProcess.Options + [poWaitOnExit];

      // Now that AProcess knows what the commandline is
      // we will run it.
      AProcess.Execute;
    except
      AfficherMessageErreur('*** Error calling external program ***');
    end;
  finally
    // This is not reached until ppc386 stops running.
    AProcess.Free;
  end;
end;

//******************************************************************************
// Spécifique Raspberry
//******************************************************************************
{$IFDEF RASPBERRY_PI}
function DisplayClavierVirtuel(const QTextBox: TEdit; const ModeClavier: TModeClavierVirtuel): boolean;
var
  KBDV: TdlgClavierVirtuel;
begin
  result := false;
  //type TModeClavierVirtuel = (mkbdvALPHA_MINUSCULES, mkbdvALPHA_MAJUSCULES, mkbdvNUMERIC);
  KBDV := TdlgClavierVirtuel.Create(application);
  try
    KBDV.Initialiser(QTextBox.Text, ModeClavier);
    KBDV.ShowModal;
    if (KBDV.ModalResult = mrOK) then
    begin
      QTextBox.Text := KBDV.GetValue();
    end;
  finally
    KBDV.Release;
  end;
end;

function DisplayPaveNumVirtuel(var Value: double): boolean;
var
  DG: TdlgPaveNumVirtuel;
begin
  result := false;
  DG := TdlgPaveNumVirtuel.create(Application);
  try
    DG.Initialiser(Value);
    DG.ShowModal;
    if (DG.ModalResult = mrOK) then result := true;
  finally
    DG.Release;
  end;
end;
{$ENDIF RASPBERRY_PI}
end.

