unit Common;
{$INCLUDE CompilationParameters.inc}
// Fonctions communes
// Date: 19/04/2013
// Statut: Fonctionnel
// Des dtails  revoir
// 04/10/2013: Centralisation de la conversion de chaines ANSI <> UTF8

// 18/07/2014: Ajout de ConvertirEnNombreReel(), qui supporte infiffremment le point ou la virgule
// 15/12/2014: Ajout de fonctions pour le DistoX
// 28/01/2015: Ajout de qques fonctions dont GetDescTypeVisee()
// 11/04/2015: Adaptation pour Linux : OK
// 21/05/2015: Ajout de la fonction ExtractIDTerrainFromComment()
// 06/06/2016: Corrections. Fonction DegMinSec -> DegDec
// 08/08/2016: Nettoyage de code dont les fonctions de couleurs
// 23/05/2017: TStringArray est un type de donnes de la FCL 3.0.2. Remplac par TGHStringArray
// 20/06/2017: Correction dans function CalculerVisee()
interface
uses
  {$INCLUDE SelectionLangues.inc} // insre les units en fonction de la langue
  StructuresDonnees
  , CalculMatriciel3x3
  , SysUtils
  , Classes
  , Grids, Clipbrd, LCLType, types // pour les grilles et le presse-papiers
  , Process
  , math
  , FileUtil
  , Graphics
  , Forms
  , unitUtilsCPU
  {$IFDEF MSWINDOWS}
  , windows
  {$ENDIF}
  //typinfo
  ;
//type TTabStopArray  = array of const; // pour lignes  colonnage fixe
const ArrayOfFilters: array[0..21] of string = (
      rsMETAFILTRE_NIL,         // 0
      rsMETAFILTRE_ALL,         // 1
      rsMETAFILTRE_ID,          // 2
      rsMETAFILTRE_LENGTH,      // 3
      rsMETAFILTRE_AZIMUTH,     // 4
      rsMETAFILTRE_PENTE,       // 5
      rsMETAFILTRE_DATE,        // 6
      rsMETAFILTRE_COLOR,       // 7
      rsMETAFILTRE_X,           // 8
      rsMETAFILTRE_Y,           // 9
      rsMETAFILTRE_Z,           // 10
      rsMETAFILTRE_LARGEUR,     // 11
      rsMETAFILTRE_HAUTEUR,     // 12
      rsMETAFILTRE_DATES,       // 13
      rsMETAFILTRE_COLORS,      // 14
      rsMETAFILTRE_SERIE,       // 15
      rsMETAFILTRE_RESEAU,      // 16
      rsMETAFILTRE_CODE,        // 17
      rsMETAFILTRE_EXPE,        // 18
      rsMETAFILTRE_TYPEVISEE,   // 19
      rsMETAFILTRE_SECTEUR,     // 20
      rsMETAFILTRE_ENTREE_RATT  // 21
      );

// pseudo-mot cl ne faisant rien, quivalent de l'instruction pass du Python
// utilis dans les embranchements sans action  faire de structures de contrle
procedure pass; inline;

function GetGHTopoVersion(): string;
procedure AfficherMemoryUsage();

// affectation rapide de valeurs (pour viter des affectations multiples ds le code)
function MakeTPoint(const QX, QY: integer): TPoint;
function MakeTPoint2Df(const QX, QY: double): TPoint2Df;
function MakeTPoint3Df(const QX, QY, QZ: double): TPoint3Df;
function MakeTRect2Df(const QX1, QY1, QX2, QY2: double): TRect2Df;

function MakeTPointCoupeDeveloppee(const QP, QZ: double): TPointCoupeDeveloppee;
function MakeTMarker(const D: boolean; const QX, QY: double; const C: TColor; const S: string): TMarker;


function CheckGHTopoPath(const Chemin: string): boolean;
// attraper le dossier de GHTopo (remplace ExtractFilePath)
function GetGHTopoDirectory(): TStringDirectoryFilename;
function GetLastDirectoryUsed(): TStringDirectoryFilename; // attraper le dossier du dernier doc utilis
// extraction du dossier contenant le doc courant
function ExtractDirectoryOfGHTopoDocument(const FN: TStringDirectoryFilename): TStringDirectoryFilename; inline;

// fonction d'affichage vers la console
procedure AfficherMessage(const Msg: string);
procedure AfficherMessageErreur(const Msg: string);

// conversions de chaines
function _AnsiToLCLStr(const S: string): string; inline;
function _LCLStrToAnsi(const S: string): string; inline;
// fonction scurise d'utilisation de resoursestrings
function GetResourceString(const RS: string): string; inline;
// obtenir l'anne std depuis anne abrges
function GetAnneeISO(const QAnnee: word): word;
// calcul d'une vise
procedure CalculerVisee(var MaVisee: TUneVisee;
                        const CC: TCode; const EE: TExpe;
                        var AX, AY: double;
                        const Sens: double;
                        var AZ, AP: double);

// retourne le type de vise
function GetTypeDeVisee(const T: integer): TTypeDeVisee;
function GetDescTypeVisee(const T: TTypeDeVisee): string;

// La vise est naturelle ?
function IsViseetInNaturalCave(const E: TEntiteEtendue): boolean;
// La vise passe dans un vide souterrain ?
function IsViseetInCaveOrTunnel(const E: TEntiteEtendue): boolean;
// La vise appartien  une srie ?
function IsViseeInSerie(const E: TEntiteEtendue): boolean;

// fonction d'incrmentation lexicographique
function IncrementString(const S0: string): string;

//****************************
// fonctions communes
//****************************
// encodage/Dcodage scuris des dates
function  GetSecuredDate(Year, Month, Day: Word): TDateTime;
procedure SafeDecodeDate(Date: TDateTime; var Year, Month, Day: Word);

procedure Swap(var V1, V2: integer); overload;
procedure Swap(var V1, V2: Double); overload;
procedure Swap(var V1, V2: string); overload;

// bornes comprises
function IsInRange(const Value, MinValue, MaxValue: Extended): Boolean; inline; overload;
function IsInRange(const Value, MinValue, MaxValue: integer): Boolean; inline; overload;

function IIF(const Condition: boolean; const V1, V2: boolean): boolean; inline; overload;
function IIF(const Condition: boolean; const V1, V2: integer): integer; inline; overload;
function IIF(const Condition: boolean; const V1, V2: Extended): Extended; inline; overload;
function IIF(const Condition: boolean; const V1, V2: TColor): TColor; inline; overload;
function IIF(const Condition: boolean; const V1, V2: String): String; inline; overload;
function IIF(const Condition: boolean; const V1, V2: char): char; inline; overload;



//----------------------------------
function CalculerAngleBissecteur(const dX1, dY1, dX2, dY2: double): double;

//--------------------------------------------------------
function Hypot2D(const DX, DY: Double): Double; inline;
function Hypot3D(const DX, DY, DZ: Double): Double; inline;

function GetAzimut(const dx, dy: Double; const Unite: double): double;

procedure GetBearingInc(const dx, dy, dz: double;
                        var Dist, Az, Inc: double;
                        const fUB, fUC: Double);
//*****************************************
//-------------------------------------
function GetFloatRValue(const C: TColor): double; inline;
function GetFloatGValue(const C: TColor): double; inline;
function GetFloatBValue(const C: TColor): double; inline;
function Acad2RGB(const n : integer) : tColor; // palette par dfaut d'Autocad
function RGB2Acad(const C: TColor) : Byte;
function GetSVGColor(const C: TColor): string;
function GetColorRGBA(const Coul: TColor; const Alpha: byte): TColorRGBA;

//-------------------------------------
// routines de couleurs
function GetPCColor(const MC: TMacintoshColor): TColor; overload; inline;
function GetPCColor(const mR, mG, mB: word):TColor; overload; inline;
function GetMacColor(const PCC: TColor): TMacintoshColor;
function GetBYTEColor(const Coul: TColor): TColor3b;

function GetColorDegrade(const z: Double;
                         const zmin, zmax: Double;
                         const Coul1, Coul2: TColor): TColor;
function GetNTSCGrayScale(const C: TColor): Byte;
// couleur RGBA au format KML
function KMLColor(const R, G, B, A: byte): string;
// couleur RGBA au format HTML
function ColorToHTMLColor(const Couleur: TColor): string;
function ColorHTMLToColor(const CouleurHTML: string): TColor;



//*************
// routines de texte
function EnsureMakeFilename(const S: TStringDirectoryFilename): TStringDirectoryFilename;
function EnlevePerluete(const s: string): string; inline;
function IndexOfString(const S: string; const IsEqual: boolean; const Strs: array of string): integer;
function ChooseString(const Index: integer; const Strs: array of string): string;
function SafeTruncateString(const Str: string; const L: integer): String;


//function GetDeclimag(const InitialDate, CurrentDate: TDateTime; const InitialDeclimag, Variation: Double):Double;

// Purge les accents d'un texte
function PurgerAccents(const S: string): string;


// Convertir un fichier texte vers le format dsir
function ConvertTextFile(const InputFileName, OutputFilename: TStringDirectoryFilename;
                         const InputFormat, OutputFormat: TTextFileFormat): boolean;

// extraction de paramtres d'une chaine
function Split(const Str: string; const Sep: string):TGHStringArray;
function SplitStr(const Str: string; const Sep: string; var ArrValues: TStringList): boolean;

function ExtractStringsToTStringArray(Separators, WhiteSpace: TSysCharSet;
                                      const MyStr: string): TGHStringArray;
function DecoupeChaineEspacesGuillemets(const S: string; const DoRemoveQuotes: boolean): TGHStringArray;
function DecoupeChaine(const S: string;
                       const Separators, WhiteSpace: TSysCharSet;
                       const DoRemoveQuotes: boolean): TGHStringArray;

// extraction et dpilage du premier lment d'une string
function ExtractFirstElementOfStr(var Str: string; const Sep: string): string;
// splitter une ligne  positions fixes
function SplitFixedColLine(const MyLine: string; const Tabs: array of integer): TGHStringArray;
// obtenir un tableau 2D  partir d'une string
function GetStringArray2D(const S: string; const SepCol: string): TGHStringArray2D;


// formater un filtre de fichier
// Arguments pairs du tableau: Nom du filtre
// Arguments impairs: Filtre
//function FormatFileFilters(const FF: array of string; const WithAll: boolean): string;

// proposer une quidistance en fonction de l'tendue du rseau
function ProposerEquidistanceDef(const C1, C2: TPoint3Df; const Defaut: double): Double;

// test d'intersection de deux rectangle
function IntersectRectangles(const R1, R2: TRect2Df): boolean;


// fonctions de rappel
//type TCallbackProc = procedure of object;



// description d'un systme de coordonnes
function DescribeEPSGSystem(const CodeEPSG: integer): string; deprecated;

// formatter nombre avec sparateurs de milliers
function FormatterNombreAvecSepMilliers(const X: double): string; inline;



//
function GetIDStation(const N: integer; const St: TStation): string;
// calcul localis de coordonnes
function ConversionTableauCoordonneesIsoleesEPSG(const Src, Tgt: integer; const ArrCoordsInput: TArrayPoints2Df; out ArrPointsConverted: TArrayPoints2Df): boolean;
function ConversionCoupleCoordonneesIsoleesEPSG(const Src, Tgt: integer; const PointIn: TPoint2Df; out PointOut: TPoint2Df): boolean;
function ConversionCoordonneesIsoleesEPSG(const Src, Tgt: integer; const PointInX, PointInY: double; out PointOutX, PointOutY: double): boolean;

// calcul d'une dclinaison magntique isole
function CalcDeclimagIsolee(const Lon, Lat, Alt: double; const MyDate: TDate; out DecliMagValue: double): boolean;


// sauvegarde et copie de grilles
// NOTA: Le composant TStringGrid contient un srialiseur SaveToFile mais
//       il sauvegarde uniquement au format XML

function GRDCCopierUnTableau(const Grille: TStringGrid): boolean;
function GRDCollerDepuisClipBoard(const Grille: TStringGrid; const HasTitres: boolean; const QSeparateur: string): boolean;
function GRDChargerDepuisFichier(const Grille: TStringGrid; const HasTitres: boolean; const FileName: string; const QSeparateur: string): boolean;
function GRDSauverUnTableauEnCSV(const Grille: TStringGrid; const FichierCSV: TStringDirectoryFilename): boolean; inline;
procedure GRDRemplirDepuisStringList(const Grille: TStringGrid; const LesLignes: TStringList; const HasTitres: boolean; const Separateur: string);
// excution d'une programme externe
procedure ExecuteExternalProgram(const MyCommandLine: string);
// dates SQL et vice-versa
function DatePascalToDateSQL(const MyDate: TDateTime): string;
function DateSQLToDatePascal(const MyDateSQL: string): TDateTime;
function DateTimeSQLToDateTimePascal(const MyDateSQL: string): TDateTime;
function DateTimePascalToDateTimeSQL(const MyDate: TDateTime; const WithMilliseconds: boolean = true): string;
// Dates condenses AAAAMMJJ
function DatePascalToDateCondensee(const QDate: TDateTime): string;
// Dates condenses AAAAMMJJ_HHMNSSMS
function DatePascalToDateHeureCondensee(const QDate: TDateTime): string;

// version du XML et encodage
function FormatterVersionEncodageXML(const version: string; const encodage: string): string; inline;
// entit tendue vide
function EmptyEntiteEtendue: TEntiteEtendue;
// pour mtafiltre
function ChooseFilter(const s90: string): integer;

// wrapper pour codes EPSG
//function GetCodeEPSGFrom(const

// convertir un  nombre Pascal en nombre OOo Calc
function  PasNumberToOOoCalcFormula(const V: double): string;
// formatage OOo des nombres
function FormatterNombreOOo(const Value: extended; const NbDecs: integer; const IgnoreNulls: boolean): string; overload;
function FormatterNombreOOo(const Value: extended): string; overload;
function ensureConvertLongOrLatToXML(const Value: extended): string;
// formatage des nombres pour KML et OSM
function ensureConvertAltitudeToXML(const Value: extended): string;
// convertir en nombre des valeurs de type 123 456 789.10 ou 1245-45785
function ConvertirEnNombreReel(const S: string; const Default: Extended): Extended;
// convertir une dclinaison en deg dc en dclinaison Toporobot (unit et signe)
function makeToporobotDeclimag(const decl: double): double;
// type littral de mesure DistoX
function getLitteralTypeMesureDistoX(const T: TTypeMesureDistoX): string;

// sortie formate pour colonnages fixes
function FormatterFixedColumn(const S: string; const W: integer; const Aligment: TAlignment): string;
// date au format JJ/MM/AA HH:MM:SS
function FormatterDateHeureEnFrancais(const D: TDateTime): string;
// date au format JJ/MM/AA
function FormatterDateEnFrancais(const D: TDateTime): string;
// date complte JJ/MM/AAAA HH:MM:SS.MS avec heures et millisecondes
function FormatterDateHeureAvecMiliSecEnFrancais(const D: TDateTime): string;
// extraire le libell d'une tiquette de terrain depuis un commentaire
// l'tiquette commence par un '@' et ne comporte pas d'espace
function ExtractIDTerrainFromComment(const S: string): string;
// correction angulaire
function MakeTParamFoncCorrectionAngulaire(const QCo, QErreurMax, QPosErrMax: double): TParamFoncCorrectionAngulaire;
function CalcAzimutCorrige(const AzBrut: double;
                           const UniteBoussole: double;                    // 360 = degr, 400 = grade
                           const ParamsFunc: TParamFoncCorrectionAngulaire): double;
function CalcPenteCorrigee(const PenteBrute: double;
                           const UniteClino: double;                    // 360 = degr, 400 = grade
                           const ParamsFunc: TParamFoncCorrectionAngulaire): double;
function MakeTReseau(const QColorReseau: TColor; const QTypeReseau: integer; const QNomReseau, QObsReseau: string): TReseau;
function MakeTSecteur(const QCouleurSecteur: TColor; const QNomSecteur: string): TSecteur;
function MakeTEntrance(const QNomEntree: string;
                       const QIDTerrain: string;
                       const QXEntree  : double;
                       const QYEntree  : double;
                       const QZEntree  : double;
                       const QRefSer   : integer;
                       const QRefSt    : integer;
                       const QObserv   : string): TEntrance;
function MakeTStationMatchFound(const S, P: integer; const M: string): TStationMatchFound;

function IsSameSerie(const E1,E2: TEntiteEtendue): boolean;

function DegMinSec2DegDec(const S: string): double;

function ChooseColorByTypeEntite(const TE: TTypeDeVisee): TColor;

// remplace les diacritiques
function RemoveDiacritiques(const S: string): string;

//RTTI
function GetDescTModeRepresentationGaleriesType(const Idx: TModeRepresentationGaleries): string;


function DecomposeStationToporobot(const S: string): TToporobotStation;

procedure DecomposeNumeroSerie(const QNumSerie: TNumeroSerie; out QIdxNameSpace, QNoSerie: integer);
procedure DecomposeNumeroCode(const QNumCode: TNumeroCode; out QIdxNameSpace, QNoCode: integer);
procedure DecomposeNumeroExpe(const QNumExpe: TNumeroExpe; out QIdxNameSpace, QNoExpe: integer);

function BaseNToDecimal(const binval : string; const Base:Int64) : Int64;

function MakeGHCaveDrawIDPtCenterline(const E: TEntiteEtendue; var NbViseesEnAntenne: integer): int64;
// prise en compte des casses de caractres et autres paramtres pour le ID terrain
function FormatterIDTerrainStation(const S: string): string;
function ExplainCodeAzimut(const CC: TCode): string;
function ExplainCodePente(const CC: TCode): string;

// neutralise les entits HTML (pour le support du XML)
function PasStrToXMLStr(const S: string): string;
function XMLStrToPasStr(const S: string): string;

// rcuprer le nombre de coeurs d'un processeur
function GetNbCoresOfProcessor(): integer;
// formatage des labels de station et de noeuds
function MakeLabelNoeud(const ND: TJonctionXYZ): String;
function MakeLibelleStationTopo(const E: TEntiteEtendue): string;
function MakeLibelleCode(const E: TCode): string;
function MakeLibelleExpe(const E: TExpe): string;

function MakeTUneVisee(const QNoVisee          : integer;
                       const QTypeVisee        : TTypeDeVisee;
                       const QIdSecteur        : TNumeroSecteur;
                       const QCode             : TNumeroCode;
                       const QExpe             : TNumeroExpe;
                       const QL, QAz, QP       : double;
                       const QLG, QLD, QHZ, QHN: double;
                       const QIDStation        : string = '';
                       const QCommentaire      : string = ''): TUneVisee;
function EmptyVisee(const C: string): TUneVisee;

function MakeTViseeAntenne(const QIDViseeAntenne          : integer;
                           const QEntranceRatt            : TNumeroEntrance;
                           const QReseau                  : TNumeroReseau;
                           const QSecteur                 : TNumeroSecteur;
                           const QSerieDepart             : TNumeroSerie;
                           const QPtDepart                : integer;
                           const QCode                    : TNumeroCode;
                           const QExpe                    : TNumeroExpe;
                           const QL, QAz, QP              : double;
                           const QIDStation               : string = '';
                           const QCommentaire             : string = ''): TViseeAntenne;
// dfinir l'quidistance d'une grille
function EnsureSetQuadrillageSpacing(const S: double): double; inline;
// calcul de la constante de correction des mesures verticales en fonction de la longueur du cheminement
function CalcCorrectionAngulaire(const CC: TCode; const LongDev, DeltaZReference: double): double;


function IsPointInRectangle(const P: TPoint2Df; const R: TRect2Df): boolean;
function IsSegmentInRectangle(const P1, P2: TPoint2Df; const R: TRect2Df; const PartiallyInRectangle: boolean = true): boolean;
function IsQuadInRectangle(const P1, P2, P3, P4: TPoint2Df; const R: TRect2Df): boolean;

//******************************************************************************
implementation
uses
  {$IFDEF MSWINDOWS}
     frmJournal,
  {$ENDIF}
  {$IFDEF LINUX}
  //frmJournal,
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
    frmJournal,
  {$ENDIF}
  UnitTSAGeoMag,
  ConvertisseurJPC in 'ConversionsCoords\convertisseurjpc.pas'; // API JPC
// pseudo-mot cl ne faisant rien
procedure pass; inline;
begin
  ;
end;
// extraire l'ID d'espace de noms et le numro de srie originel
procedure DecomposeNumeroSerie(const QNumSerie: TNumeroSerie; out QIdxNameSpace, QNoSerie: integer);
begin
  QIdxNameSpace := QNumSerie div NB_MAXI_SERIES_PAR_CAVITE;
  QNoSerie      := QNumSerie mod NB_MAXI_SERIES_PAR_CAVITE;
end;
procedure DecomposeNumeroCode(const QNumCode: TNumeroCode; out QIdxNameSpace, QNoCode: integer);
begin
  QIdxNameSpace := QNumCode div NB_MAXI_SERIES_PAR_CAVITE;
  QNoCode       := QNumCode mod NB_MAXI_SERIES_PAR_CAVITE;
end;
procedure DecomposeNumeroExpe(const QNumExpe: TNumeroExpe; out QIdxNameSpace, QNoExpe: integer);
begin
  QIdxNameSpace := QNumExpe div NB_MAXI_SERIES_PAR_CAVITE;
  QNoExpe       := QNumExpe mod NB_MAXI_SERIES_PAR_CAVITE;
end;
// extraire le libell d'une tiquette de terrain depuis un commentaire
// l'tiquette commence par un '@' et ne comporte pas d'espace
// L'tiquette est passe en majuscules
function ExtractIDTerrainFromComment(const S: string): string;
var
  p0: SizeInt;
begin
  Result := '';
  p0 := Pos('@', S);
  if (p0 = 0) then Exit;
  Result := Trim(Copy(S, p0+1, Length(S) - p0));
  Result := UpperCase(Result);
  p0 := Pos(' ', Result);
  if (p0 = 0) then Exit;
  Result := Copy(Result, 1, p0 - 1);
end;

// conversions de chaines
function _AnsiToLCLStr(const S: string): string; inline;
begin
  Result := system.AnsiToUtf8(S); // pour Lazarus < 1.6
end;
function _LCLStrToAnsi(const S: string): string; inline;
begin
  Result := System.Utf8ToAnsi(S); // pour Lazarus < 1.6
end;

// fonction scurise d'utilisation de resoursestrings
function GetResourceString(const RS: string): string; inline;
begin
  Result := _AnsiToLCLStr(RS);
end;

// scuris avec AnsiToUTF8
function FormatterNombreAvecSepMilliers(const X: double): string; inline;
begin
  Result := _AnsiToLCLStr(Format('%.2n',[X])); // pour Lazarus < 1.6
end;

function KMLColor(const R, G, B, A: byte): string;
begin
  Result := Format('<color>%.2X%.2X%.2X%.2X</color>',[A, B, G, R]);
end;

function ColorToHTMLColor(const Couleur: TColor): string;
var
  R, V, B: byte;
begin
  R := Red(Couleur);
  V := Green(Couleur);
  B := Blue(Couleur);
  Result := Format('#%.2X%.2X%.2X', [R, V, B]);
end;
function ColorHTMLToColor(const CouleurHTML: string): TColor;
var
  WU: String;
begin
  WU      := Trim(CouleurHTML);
  WU[1]   := '$';
  Result  := StringToColor(WU);
end;
//fonction de split
function Split(const Str: string; const Sep: string): TGHStringArray;
var
  pn   : integer;
  ps   : integer;
  S    : string;
begin
  for pn:=0 to High(Result) do Result[pn]:='';
  S:=Str;
  ps:=0;
  try
    pn:=0;
    repeat
     if pn>High(Result) then Break;
     ps:=Pos(Sep, S);
     //s:=Copy(s,0, ps-1);
     Result[pn]:=Trim(Copy(s,0, ps-1));
     Inc(pn);
     s:=Copy(s, 1+ps, Length(s));
    until ps=0;
    Result[pn-1]:=Trim(s);
  except
  end;
end;

function SplitStr(const Str: string; const Sep: string; var ArrValues: TStringList): boolean;
var
  S: String;
  ps, pn: Integer;
begin
  result := false;
  ArrValues.Clear;
  S:=Str;
  ps:=0;
  //try
    pn:=0;
    repeat
      ps:=Pos(Sep, S);
      //s:=Copy(s,0, ps-1);
      //SetLength(ArrValues, pn+1);
      ArrValues.Add(Trim(Copy(s,0, ps-1)));
      Inc(pn);
      s:=Copy(s, 1+ps, Length(s));
    until ps=0;
    ArrValues.Add(Trim(s));
    result := true;
  //except
  //end;

end;

// extraction et dpilage du premier lment d'une string
function ExtractFirstElementOfStr(var Str: string; const Sep: string): string;
var
  ps, n: Integer;
begin
  Result := '';
  try
    Str := Trim(Str) + Sep;
    ps:=Pos(Sep, Str);
    if (ps > 0) then
    begin
      Result := Copy(Str, 1, ps - 1);
      Str := Copy(Str, ps+1, Length(Str));
    end;
    // et on vire le sparateur de fin s'il existe
    n := length(Str);
    if ((n > 0) and (Str[n] = Sep)) then System.Delete(Str, n, 1);
  except
  end;
end;

// splitter une ligne  positions fixes
// Fonctionnement OK
function SplitFixedColLine(const MyLine: string; const Tabs: array of integer): TGHStringArray;
var
  pn   : integer;
  ps   : integer;
  S    : string;
begin
  for pn:=0 to High(Result) do Result[pn]:='';
  for pn := 0 to High(Tabs)-1 do begin
    Result[pn] := Copy(MyLine, Tabs[pn], (Tabs[pn+1] - Tabs[pn]));
  end;
end;

// Fonctions d'change
procedure Swap(var V1, V2: integer); overload;
var Tmp: integer;
begin
  Tmp  := V1;
  V1   := V2;
  V2   := Tmp;
end;
procedure Swap(var V1, V2: Double); overload;
var Tmp: Double;
begin
  Tmp  := V1;
  V1   := V2;
  V2   := Tmp;
end;
procedure Swap(var V1, V2: string); overload;
var Tmp: string;
begin
  Tmp  := V1;
  V1   := V2;
  V2   := Tmp;
end;
function GetGHTopoVersion(): string;
begin
  Result := Format('%.9f', [pi]);
end;

function MakeTPoint(const QX, QY: integer): TPoint;
begin
 Result.X := QX;
 Result.Y := QY;
end;

function MakeTPoint2Df(const QX, QY: double): TPoint2Df;
begin
  Result.X := QX;
  Result.Y := QY;
end;

function MakeTPoint3Df(const QX, QY, QZ: double): TPoint3Df;
begin
  Result.X := QX;
  Result.Y := QY;
  Result.Z := QZ;
end;

function MakeTRect2Df(const QX1, QY1, QX2, QY2: double): TRect2Df;
begin
  Result.X1:= QX1; Result.X2 := QX2;
  Result.Y1:= QY1; Result.Y2 := QY2;
end;

function MakeTPointCoupeDeveloppee(const QP, QZ: double): TPointCoupeDeveloppee;
begin
  Result.P := QP;
  Result.Z := QZ;
end;
function MakeTMarker(const D: boolean; const QX, QY: double; const C: TColor; const S: string): TMarker;
begin
  Result.Displayed := D;
  Result.X := QX;
  Result.Y := QY;
  Result.Couleur := C;
  Result.Caption := Trim(S);
end;

// vrifier si le dossier de GHTopo ne comporte pas de caractres accentus
function CheckGHTopoPath(const Chemin: string): boolean;
const
  CaracteresAutorises = [':', '0' .. '9', 'A' .. 'Z', 'a' .. 'z', '_', '-', '+', ' ', '.', DirectorySeparator];
var
  C: Char;
  i: Integer;
begin
  Result := True;
  for i := 1 to Length(Chemin) - 1 do
  begin
    C := Chemin[i];
    Result := Result and (C in CaracteresAutorises);
  end;
end;

// Les chemins avec caractres accentus ne sont pas supports.
function GetGHTopoDirectory(): TStringDirectoryFilename;
begin
  Result := ExtractFilePath(Application.ExeName);
end;

function GetLastDirectoryUsed(): TStringDirectoryFilename;
begin
  //TODO: A coder
  Result := ExtractFilePath(Application.ExeName);
end;

function ExtractDirectoryOfGHTopoDocument(const FN: TStringDirectoryFilename): TStringDirectoryFilename; inline;
begin
  Result := ExtractFileDir(FN);
end;

// non factorisation de ce code pour viter une dpendance
procedure AfficherMessage(const Msg: string);
var
  WU: String;
begin
  {$IFDEF MSWINDOWS}
    {$IFDEF DISP_FRM_JOURNAL}
       try
         if (not assigned(dlgProcessing)) then Exit;
         if (not Assigned(dlgProcessing.lsbJournal.Items)) then Exit;
         dlgProcessing.lsbJournal.Items.Add(TimeToStr(Now()) +' | ' + AnsiToUtf8(Msg));
         if (dlgProcessing.lsbJournal.Items.Count > MAX_LINES_LOG) then  dlgProcessing.lsbJournal.Items.Delete(0);
         dlgProcessing.lsbJournal.ItemIndex := dlgProcessing.lsbJournal.Items.Count-1;
       finally
         //  Application.ProcessMessages dsactiv:
         // si la fentre Journal est recouverte, une erreur 'Indice hors limites' se produit
         // Ce bug ne se produit pas si cette fentre est partiellement visible
         //Application.ProcessMessages;
       end;
    {$ENDIF}
  {$ENDIF}

  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
  try
     with dlgProcessing.lsbJournal do
     begin
        if (Items.Count > MAX_LINES_LOG) then
          Items.Delete(0);
        Items.Add(TimeToStr(Now()) +' | ' + AnsiToUtf8(Msg));
        ItemIndex := dlgProcessing.lsbJournal.Items.Count-1;
        Refresh;
      end;
   except
   end;
  {$ENDIF}
  {$IFDEF LINUX}
    WriteLn(AnsiToUtf8(Msg));
  {$ENDIF}
  //*)
end;

// non factorisation de ce code pour viter une dpendance
procedure AfficherMessageErreur(const Msg: string);
begin

  {$IFDEF MSWINDOWS}
     {$IFDEF DISP_FRM_JOURNAL}
     try
       if (not assigned(dlgProcessing)) then Exit;
       if (not Assigned(dlgProcessing.memoErreurs.Lines)) then Exit;
       dlgProcessing.memoErreurs.Lines.Add(Msg);
       Application.ProcessMessages;
     except
     end;
    {$ENDIF}
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
    dlgProcessing.memoErreurs.Lines.Add(Msg);
  {$ENDIF}
  {$IFDEF LINUX}
    WriteLn(AnsiToUtf8(msg));
  {$ENDIF}
  //*)
end;
procedure AfficherMemoryUsage();
var
  M: TMEMORYSTATUS;
  EWE: THeapStatus;
begin
   {$IFDEF MSWINDOWS}
     {$IFDEF DISP_FRM_JOURNAL}
     try
       if (not assigned(dlgProcessing)) then Exit;
       M.dwLength := sizeof(TMEMORYSTATUS);
       GlobalMemoryStatus(M);
       EWE := GetHeapStatus;
       // M1 = EWE.TotalAddrSpace = EWE.TotalAllocated + EWE.TotalFree,
       dlgProcessing.lbMemoryUsage.Caption := Format(GetResourceString(rsGHTOPO_MEMORY_USAGE),
                                                                       [EWE.TotalAddrSpace / 1.0,
                                                                        EWE.TotalAllocated / 1.0,
                                                                        EWE.TotalFree / 1.0]);
       Application.ProcessMessages;
     except
     end;
    {$ENDIF}
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
    dlgProcessing.memoErreurs.Lines.Add(Msg);
  {$ENDIF}
  {$IFDEF LINUX}
    WriteLn(AnsiToUtf8(msg));
  {$ENDIF}

end;


// choisir une chaine en fonction d'une valeur
function ChooseString(const Index: integer; const Strs: array of string): string;
begin
  try
    if (Index<0) or (Index>High(Strs)) then begin
      Result:=Format('** Erroneous index: %d **',[Index]);
      Exit;
    end;
    Result := Strs[Index];
  finally
  end;
end;
function IndexOfString(const S: string; const IsEqual: boolean; const Strs: array of string): integer;
var
  i: integer;
  C: boolean;
begin
  Result:=-1;
  try
    for i:=Low(Strs) to High(Strs) do
    begin
      if (IsEqual) then C := (LowerCase(S) = LowerCase(Strs[i]))     // Strs[i] est identique  S
                   else C := (Pos(S, Strs[i]) > 0);                  // Strs[i] contient S
      if (C) then
      begin
        Result:=i;
        Exit;
      end;
    end;
  except
    Result:=-1;
  end;
end;
function Hypot3D(const DX, DY, DZ: Double): Double;
begin
  Result:=Sqrt(dx*dx + dy*dy +dz*dz);
end;


function Hypot2D(const DX, DY: Double): Double;
begin
  Result:=Sqrt(dx*dx + dy*dy);
end;
// retourne un azimut
function GetAzimut(const dx, dy: Double; const Unite: double): double;
var
  a: double;
begin
  a:=ArcTan2(dy, dx+1e-12);
  if a<0 then a:=a+2*PI;
  a:=0.50*PI-a;
  if a<0 then a:=a+2*PI;
  Result:=a*0.50*Unite/pi;
end;


// retourne la longueur, direction et pente pour dx, dy, dz
procedure GetBearingInc(const dx, dy, dz: double;
                        var Dist, Az, Inc: double;
                        const fUB, fUC: Double);
var
  dp: Double;
begin;
  dp   := Hypot2D(dx, dy);
  Dist := Hypot2D(dp,dz);
  Inc  := ArcTan2(dz, dp) * 0.5 * fUC / pi;
  Az   := GetAzimut(dx,dy, fUB);
end;


// enlve les perluettes
function EnlevePerluete(const s: string): string;
begin
  result := StringReplace(s, '&', '', [rfReplaceAll, rfIgnoreCase]);
end;

// calcul de l'angle bissecteur de deux segments
function CalculerAngleBissecteur(const dX1, dY1, dX2, dY2: double): double;
var
  V1, V2, W: TPoint3Df;
begin
  // vecteur V1           vecteur V2        vecteur w
  V1.X := dX1;           V2.X := dX2;         W.X  := 0.0;
  V1.Y := dY1;           V2.Y := dY2;         W.Y  := 0.0;
  V1.Z := 0.0;           V2.Z := 0.0;         W.Z  := 1.0;
  // produits vectoriels
  v1:=ProduitVectoriel(v1,w,True);
  v2:=ProduitVectoriel(v2,w,True);
  //composition vectorielle
  w.x := v1.x + v2.X ;
  w.y := v1.y + v2.Y;
  w.z := v1.z + v2.z;
  // angles
  Result := ArcTan2(w.y+1e-12, w.x +1e-12);
end;
//****************************************************************************
// fonctions de couleurs recadres sur intervalle [0.00; 1.00]
function GetFloatRValue(const C: TColor): double;
begin
  Result := Red(C) / 256;
end;
function GetFloatGValue(const C: TColor): double;
begin
  Result := Green(C) / 256;
end;
function GetFloatBValue(const C: TColor): double;
begin
  Result := Blue(C) / 256;
end;
//*****************************************
// Conversions couleurs PC<>Mac
function GetPCColor(const MC: TMacintoshColor): TColor; overload;
begin
  Result := RGBToColor(MC.R shr 8, MC.G shr 8, MC.B shr 8);
end;
function GetPCColor(const mR, mG, mB: word):TColor; overload;
begin
  Result := RGBToColor(mR shr 8, mG shr 8, mB shr 8);
end;

function GetMacColor(const PCC: TColor): TMacintoshColor;
begin
  Result.R:=Red(PCC) * 256;
  Result.G:=Green(PCC) * 256;
  Result.B:=Blue(PCC) * 256;
end;
//-------------------------------------
//-------------------------------------
function GetBYTEColor(const Coul: TColor): TColor3b;
begin
  Result.R := Red(Coul);
  Result.G := Green(Coul);
  Result.B := Blue(Coul);
end;
//----------------------------------------------------------------
// dgrad de couleurs
function GetColorDegrade(const z: Double;
                         const zmin, zmax: Double;
                         const Coul1, Coul2: TColor): TColor;
var
  D: Double;
  H: Double;
  C1, C2, C: TColor3b;
  //DC       : TColor3b;
  DR, DG, DB : SmallInt;
begin
  D:=zmax-zmin;
  if Abs(D)<1e-8 then Exit(Coul1);
  H:=(z-zmin)/D;
  c1:=GetBYTEColor(Coul1);
  c2:=GetBYTEColor(Coul2);
  DR:=C2.R-C1.R;
  DG:=C2.G-C1.G;
  DB:=C2.B-C1.B;

  C.R:=Round(C1.R + H * DR);
  C.G:=Round(C1.G + H * DG);
  C.B:=Round(C1.B + H * DB);

  Result := RGBToColor(C.R, C.G, C.B);
end;
//-------------------------------------------------------
//Convertit une couleur en niveaux de gris; mthode NTSC.
function GetNTSCGrayScale(const C: TColor): Byte;
var
  cl: TColor3b;
begin
  cl:=GetBYTEColor(C);
  Result := round(0.30 * cl.R + 0.59 * cl.G + 0.11 * cl.B);
end;

function RGB2Acad(const C: TColor): Byte;
var drmin,dgmin,dbmin,
    CouleurR,
    CouleurG,
    CouleurB,
    ACcolor,
    ACcolorR,
    ACcolorG,
    ACcolorB,
    dColor,
    res    : integer;
begin
    Result:=0;

    dRmin := 99999999;
    dGmin := 99999999;
    dBmin := 99999999;

    CouleurR := Red(C);
    CouleurG := Green(C);
    CouleurB := Blue(C);
    for res := 1 to 255 do
    begin
      ACcolor := Acad2RGB(res);
      ACcolorR := Red(ACcolor);
      ACcolorG := Red(ACcolor);
      ACcolorB := Blue(ACcolor);
      dColor := abs(ACcolorR - CouleurR)+
                abs(ACcolorG - CouleurG)+
                abs(ACcolorB - CouleurB);
      if (dColor < dRmin) then begin
          dRmin := dColor;
          result := res;
      end;
    end;
end;

function GetSVGColor(const C: TColor): string;
var
  R,G,B: Double;
begin
  R := 100.0 * GetFloatRValue(C);
  G := 100.0 * GetFloatGValue(C);
  B := 100.0 * GetFloatBValue(C);
  //Result:=Format('#%X%X%X',[R,G,B]);
  Result:=Format(' rgb(%.2f%%, %.2f%%, %.2f%%)', [R,G,B]);
end;
function GetColorRGBA(const Coul: TColor; const Alpha: byte): TColorRGBA;
begin
  Result.R := Red(Coul);
  Result.G := Green(Coul);
  Result.B := Blue(Coul);
  Result.A := Alpha;
end;


function Acad2RGB(const n : integer) : tColor; // palette par dfaut d'Autocad
var r,g,b,
    c,d,u : integer;
    C1 : Tcolor;  // RGB(r, g ,b)
    const StandCol : array[0..9] of tcolor =
        (clBlack ,clRed,$0001E1F3{clYellow},$0000C800{clLime},$00FDE302{clAqua},
         clBlue,clFuchsia,clBlack,clGray,clLtGray);
    const FullPalete : array[0..23] of tcolor =
        ($0000FF,$0040FF,$0080FF,$00C0FF,
         $00FFFF,$00FFC0,$00FF80,$00FF40,
         $00FF00,$40FF00,$80FF00,$C0FF00,
         $FFFF00,$FFC000,$FF8000,$FF4000,
         $FF0000,$FF0040,$FF0080,$FF00C0,
         $FF00FF,$C000FF,$8000FF,$4000FF);//..$0000FF (retour au dbut)
begin
    c := n mod 256; // au cas ou ?
    if c<10 then C1 := StandCol[c]
    else begin
        d := ((c-10) div 10);// 0..23
        // Couleur de base  corriger
        C1 :=FullPalete[d mod 24];
        // Correction:--------------------------------
        d := c div 10; // dizaines
        u := c-d*10; // units
        // sparation des couleurs RGB
        b := (C1 and $FF0000) shr 16;
        g := (C1 and $00FF00) shr 8;
        r := (C1 and $0000FF);
        //Plus clair pour les impairs
        if ((u div 2)*2<>u) then begin
            b := b + ((255-b) div 2);
            g := g + ((255-g) div 2);
            r := r + ((255-r) div 2);
        end;
        // Plus fonc si u grand
        b := b*4 div (u+4);
        g := g*4 div (u+4);
        r := r*4 div (u+4);
        // Couleur corrige:---------------------------
        C1 := RGBToColor(r,g,b);
    end;
    result := C1;
end;

//-------------------------------------
// bornes comprises
function IsInRange(const Value, MinValue, MaxValue: Extended): Boolean; overload;
begin
  Result := ((Value >= MinValue) and (Value <= MaxValue));
end;
function IsInRange(const Value, MinValue, MaxValue: integer): Boolean; overload;
begin
  Result := ((Value >= MinValue) and (Value <= MaxValue));
end;


//-------------------------------------
function IIF(const Condition: boolean; const V1, V2: integer): integer; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;
function IIF(const Condition: boolean; const V1, V2: Extended): Extended; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;
function IIF(const Condition: boolean; const V1, V2: boolean): boolean; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;

function IIF(const Condition: boolean; const V1, V2: TColor): TColor; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;

function IIF(const Condition: boolean; const V1, V2: String): String; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;

function IIF(const Condition: boolean; const V1, V2: char): char; overload;
begin
  if (Condition) then Result := V1 else Result := V2;
end;



// construire un nom de fichier valide en virant les accents et caractres spciaux
function EnsureMakeFilename(const S: TStringDirectoryFilename): TStringDirectoryFilename;
var
  Lin: String;
  L: Integer;
  procedure RemplacerChar(const Caracts: string);
  var
    w: Integer;
    C: Integer;
    QC: Char;
    Nb: Integer;
  begin
    Nb := Length(Caracts);
    for C := 1 to Nb do
    begin
      QC := Caracts[C];
      for w := 1 to L do if (Lin[w] = QC) then Lin[w] := '_';
    end;
  end;
begin
  Lin := Trim(PurgerAccents(S)); //
  L   := Length(Lin);
  if (L = 0) then begin Result :=''; Exit; end;
  // on dquille certains caractres spciaux en remplaant par des '_'
  RemplacerChar('^& #\"''(=+*/<>)[:,;!.%?]{~|`@}$');
  Result := Lin;
end;

// Purge les accents d'un texte
function PurgerAccents(const S: string): string;
var
  w1 : integer;
  L  : integer;
  Lin: string;
begin
  Lin := S;
  L   := Length(Lin);
  if L = 0 then begin Result :=''; Exit; end;
  // convertir accentus
    for w1:=0 to L do if Lin[w1]=#136 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#142 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#143 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#148 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#137 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#144 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#144 then Lin[w1]:='';
    for w1:=0 to L do if Lin[w1]=#141 then Lin[w1]:='';
    //*)
    // virer les accents, ce qui pose bcp de problmes
    // avec la LCL de WinCE
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='e';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='e';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='e';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='e';

    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='a';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='a';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='a';

    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='i';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='i';

    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='o';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='o';

    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='u';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='u';
    for w1:=0 to L do if Lin[w1]=''  then Lin[w1]:='u';
  Result := Lin;
end;
//*)

// Convertir un fichier texte vers le format dsir
// cette variante utilise un TStringList
function ConvertTextFile(const InputFileName, OutputFilename: TStringDirectoryFilename;
                         const InputFormat, OutputFormat: TTextFileFormat): boolean;
var
  FO  : TextFile;
  ENDL: string;   // fin de ligne
  ALine: string;
  i: integer;
begin
  Result:=False;
  if Not(FileExists(InputFileName)) then Exit;
  case OutputFormat of
    tfWINDOWS: ENDL:=#13+#10;
    tfUNIX   : ENDL:=#10;
    tfMAC    : ENDL:=#13;
  end;
  with TStringList.Create do
  begin
    try
      Clear;
      LoadFromFile(InputFileName);
      AssignFile(FO, OutputFilename);
      ReWrite(FO);
      try
        try
          for i:=0 to Count-1 do begin
            ALine := Strings[i]+ENDL; // ne PAS purger la chane !!!
            Write(FO, ALine);
          end;
          Result:=True;
        except
        end;
      finally
        CloseFile(FO);
      end;
      Clear;
    finally // with TStringList
      Free;
    end;
  end;
end;
// troncature scurise d'un texte  la longueur L
function SafeTruncateString(const Str: string; const L: integer): String;
begin
 if Length(Str)>L then
   Result := Trim(Copy(Str, 1, L))
 else
   Result := Trim(Str);
end;

function GetAnneeISO(const QAnnee: word): word;
const
  PIVOT = 50; // anne 1950 = pivot pour les annes sur deux chiffres
begin
  try
  Result := QAnnee;
    if (QAnnee >  1900 + PIVOT) then Exit; // anne sur quatre chiffres -->[]
    if (QAnnee <= PIVOT) then Result := 2000 + QAnnee
                         else Result := 1900 + QAnnee;

  except
    Result := 1899;
  end;
end;
// calcul des accroissements pour une vise
procedure CalculerVisee(var MaVisee: TUneVisee;
                        const CC: TCode;
                        const EE: TExpe;
                        var AX, AY: double;
                        const Sens: double;
                        var AZ, AP: double);
const
  TWOPI = 2*PI;
  PI_2  = PI/2;
  PI_200 = PI/200;
var
  LeCode: TCode;
  LaExpe: TExpe;
  ubb   : integer;
  ucc   : integer;
  udd   : double;
  RX, RY, RZ, RP: double;
  UB, UC, LP: double;
  Az1: double;
  Pente1, QCorrectionPente, QLongueurVisee: double;
  procedure CalcDXDY(const QLP, QAz1: double; out QRx, QRy: double);
  begin
    QRX := QLP * Sin(QAz1);
    QRY := QLP * Cos(QAz1);
  end;
begin
  LeCode := CC;
  LaExpe := EE;
  // DONE: Support de FactLong valid au niveau du code de calcul
  QLongueurVisee := MaVisee.Longueur * CC.FactLong;
  ubb := Round(LeCode.GradAz);
  ucc := Round(LeCode.GradInc);
  //AfficherMessageErreur(Format('%d - %d - %.2f, %.3f, %.3f', [ubb, udd, MaVisee.Longueur, MaVisee.Azimut, MaVisee.Pente]));
  udd := 400.00;
  if (ubb = 0) then LeCode.GradAz  := 360.00;
  if (ucc = 0) then LeCode.GradInc := 360.00;
  //WriteLn(Format('LeCode.GradAz = %f ucc = %f',[ucc]));
  case ubb of
    359, 360: begin // vises directes en degrs
       UB := TWOPI/360.00;
       Az1 := CalcAzimutCorrige(MaVisee.Azimut,
                                360.00,
                                LeCode.ParamsFuncCorrAz);
       Az1 := Az1 * UB + LaExpe.Declinaison  * PI_200;
    end;
    399, 400: begin // vises directes en grades
       UB := TWOPI/400.00;
       Az1 := CalcAzimutCorrige(MaVisee.Azimut,
                                400.00,
                                LeCode.ParamsFuncCorrAz);
       Az1:= Az1 * UB + LaExpe.Declinaison  * PI_200;
    end;
    349, 350: begin  // vises inverses en degrs
       UB := TWOPI/360.00;
       Az1 := CalcAzimutCorrige(MaVisee.Azimut,
                                360.00,
                                LeCode.ParamsFuncCorrAz);
       Az1 := Az1 * UB + LaExpe.Declinaison  * PI_200;
       Az1 := PI+Az1;
    end;
    389, 390: begin // vises inverses en grades
       UB  := TWOPI/400.00;
       Az1 := CalcAzimutCorrige(MaVisee.Azimut,
                                400.00,
                                LeCode.ParamsFuncCorrAz);
       Az1 := Az1 * UB + LaExpe.Declinaison  * PI_200;
       Az1 := PI+Az1;
    end;
  end;
  (* Dterminer si on travaille en znithal*)
  ucc := Round(LeCode.GradInc);
  // Correction erreurs systmatiques des pentes
  QCorrectionPente := LeCode.ParamsFuncCorrInc.Co;
  case ucc of
    360, 400: begin // zro  l'horizontale
      UC := TWOPI/ucc;     // DONE: FIXED: 'ubb' remplac par 'ucc'
      // corrections d'erreurs
      Pente1 := CalcPenteCorrigee(MaVisee.Pente, ucc, LeCode.ParamsFuncCorrInc);
      Pente1 := Pente1 * UC;
      LP := QLongueurVisee * Cos(Pente1);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(Pente1);
    end;
    361, 401: begin // zro znithal
      UC := TWOPI/(ucc - 1);
      Pente1 := PI_2 - ((QCorrectionPente + MaVisee.Pente) * UC);
      LP := QLongueurVisee * Cos(Pente1);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(Pente1);
    end;
    359, 399: begin // zro nadiral
      UC := TWOPI/(ucc + 1);
      Pente1 := -(PI_2 - ((QCorrectionPente + MaVisee.Pente) * UC));
      LP := QLongueurVisee * Cos(Pente1);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(Pente1); // Fix le 31/05.
    end;
    370: begin // DONE: Pourcentages sont exprims sous la forme 59.11 pour 59.11%
      udd:= ArcTan(MaVisee.Pente / 100.0);
      LP := QLongueurVisee * Cos(udd);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(udd);
    end;
    380: // dans ce mode, les donnes sont 'Longueur VISEE et DENIVELEE
    begin // diffrences d'altitude, stockes ds Visee.Pente
      try
        LP := sqrt(QLongueurVisee * QLongueurVisee - MaVisee.Pente * MaVisee.Pente);
        CalcDXDY(LP, Az1, RX, RY);
        RZ := MaVisee.Pente;
        AfficherMessageErreur(Format('Shot: %s: L=%.2f AZ=%.2f P=%.2f - RX = %.2f - RY = %.2f - RZ = %.2f - LP = %.2f',
                               [MaVisee.IDTerrainStation,
                                QLongueurVisee, MaVisee.Azimut, MaVisee.Pente,
                                RX, RY, RZ, LP]));
      except
        AfficherMessage(Format('** ERROR ** Shot: %s: L=%.2f AZ=%.2f P=%.2f - RX = %.2f - RY = %.2f - LP = %.2f',
                               [MaVisee.IDTerrainStation,
                                QLongueurVisee, MaVisee.Azimut, MaVisee.Pente,
                                RX, RY, LP]));
      end;
    end;
    350, 390:
    begin // vises inverses en degrs/grades (ucc = UniteClino -10);
      UC := TWOPI/(ucc + 10);
      Pente1 := -(QCorrectionPente + MaVisee.Pente) * UC;
      LP := QLongueurVisee * Cos(Pente1);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(Pente1);
    end;
    UNITE_CLINO_LASERMETRE_STANLEY_TLM330_DIR,
    UNITE_CLINO_LASERMETRE_STANLEY_TLM330_INV: // support du lasermtre
    begin
      // longueur projete est dans Long
      LP := QLongueurVisee;
      CalcDXDY(LP, Az1, RX, RY);
      // dnivele est dans Pente
      RZ := MaVisee.Pente;
      // vise inverse ?
      if (ucc = UNITE_CLINO_LASERMETRE_STANLEY_TLM330_INV) then RZ := -RZ;
    end;
  else
    begin // zro horizontal par dfault
      //AfficherMessage('toto');
      UC := TWOPI/360.00;
      Pente1 := (QCorrectionPente + MaVisee.Pente) * UC;
      LP := QLongueurVisee * Cos(Pente1);
      CalcDXDY(LP, Az1, RX, RY);
      RZ := QLongueurVisee * Sin(Pente1);
    end;
  end;
  //==============
  RP := Hypot(RX, RY);
  RZ += CC.ErreurTourillon; // On applique in fine la correction de l'erreur de tourillons
  AX += RX;
  AY += RY;
  AZ += RZ;
  AP += RP * Sens;
  MaVisee.DeltaX := RX;
  MaVisee.DeltaY := RY;
  MaVisee.DeltaZ := RZ;
  MaVisee.DeltaP := RP;
  MaVisee.AccroissP := AP;
  MaVisee.AccroissZ := AZ;
  //AfficherMessageErreur(Format('%f, %f', [MaVisee.AccroissP, MaVisee.AccroissZ]));
end;

function GetTypeDeVisee(const T: integer): TTypeDeVisee;
begin
  try
    Result := TTypeDeVisee(T);
  except
    Result := tgDEFAULT;
  end;
end;

function GetDescTypeVisee(const T: TTypeDeVisee): string;
begin
  Result := ChooseString(Ord(T), [
                         'DEFAULT',
                         'ENTRANCE',
                         'FOSSILE',
                         'VADOSE',
                         'ENNOYABLE',
                         'SIPHON',
                         'FIXPOINT',
                         'SURFACE',
                         'TUNNEL',
                         'MINE',
                         'TYPE_ENTITES_VISEES_ANTENNE'
                        ]);
end;

function IsViseetInNaturalCave(const E: TEntiteEtendue): boolean;
begin
  Result := (E.Type_Entite in [tgDEFAULT, tgFOSSILE, tgVADOSE, tgENNOYABLE, tgSIPHON]);
end;
function IsViseetInCaveOrTunnel(const E: TEntiteEtendue): boolean;
begin
  Result := (E.Type_Entite in [tgDEFAULT, tgFOSSILE, tgVADOSE, tgENNOYABLE, tgSIPHON, tgTUNNEL, tgMINE]);
end;

function IsViseeInSerie(const E: TEntiteEtendue): boolean;
begin
  Result := (E.Type_Entite in [tgDEFAULT, tgFOSSILE, tgVADOSE, tgENNOYABLE, tgSIPHON, tgTUNNEL, tgMINE, tgSURFACE]);
  Result := Result AND (E.Entite_Serie > 0);
end;

// fonction d'incrmentation lexicographique
// Si le string ne comporte que des lettres, on ajoute un indice  la fin
// Si le string comporte un prfixe et un indice, on incrmente l'indice
function IncrementString(const S0: string): string;
var
  i,q: integer;
  Groupe: string;
  Prefix: string;
  Reste : string;
  Index : integer;
  procedure DecomposeLitteral;
  const
    Seps =':./-_,;';
  var
    a,b: integer;

  begin
    Groupe:='';
    for a:=1 to length(Seps) do begin
      b:=Pos(Seps[a], S0);
      if b>0 then begin // si le sparateur est trouv, on sort le rsultat
        Groupe:=Copy(S0, 1, b-1)+Seps[a];
        Break;
      end;
    end;
    if Groupe='' then
      Reste:=S0
    else
      Reste:=Copy(S0, b+1, Length(S0));
  end;
begin
  // Si chaine vide, on renvoie rien
  Result := '';
  if (S0 = '') then Exit;
  // rechercher un groupe (srie, etc ...)
  // spar par un ":", ".", "/", "-", "_"
  DecomposeLitteral;
  // rechercher un chiffre
  Q := -1;
  for i:=1 to Length(Reste) do begin
    if IsInRange(Ord(Reste[i]), Ord('0'), Ord('9')+1) then begin
      Q:=i;
      Break;
    end;
  end;
  // Si chiffre trouv:
  if Q>0 then begin
    Prefix:=Copy(Reste, 1, Q-1);
    Index :=StrToIntDef(Copy(Reste, Q, Length(Reste)-Q+1),0);
  end else begin
    Prefix:=Trim(Reste);
    Index:=0;
  end;
  AfficherMessage(Format('IncrementString: G=%s S=%s - P=%s - I=%d',[Groupe, Reste, Prefix, Index]));
  Result:=Format('%s%s%d',[Groupe, Prefix, Index+1]);
end;




// Suggestion d'une quidistance en fonction de l'etendue totale du rseau
function ProposerEquidistanceDef(const C1, C2: TPoint3Df; const Defaut: double): Double;
var
  d: double;
begin
  Result:=Defaut;
  try
    d:=Hypot3D(C2.X - C1.X,
               C2.Y - C1.Y,
               C2.Z - C1.Z);
    d:=d/10;
    Result:=d;
    if IsInRange(d,    0.00,   10.00) then Result:=10.00;
    if IsInRange(d,   10.00,   25.00) then Result:=25.00;
    if IsInRange(d,   25.00,   50.00) then Result:=50.00;
    if IsInRange(d,   50.00,  100.00) then Result:=100.00;
    if IsInRange(d,  100.00,  200.00) then Result:=200.00;
    if IsInRange(d,  200.00,  250.00) then Result:=250.00;
    if IsInRange(d,  250.00,  500.00) then Result:=500.00;
    if IsInRange(d,  500.00, 1000.00) then Result:=1000.00;
    if IsInRange(d, 1000.00, 2000.00) then Result:=2000.00;
    if IsInRange(d, 2000.00, 5000.00) then Result:=5000.00;
    if IsInRange(d, 5000.00,10000.00) then Result:=10000.00;
    if IsInRange(d,10000.00,20000.00) then Result:=20000.00;
    // protection 'anti-nul'
    if (Result < 5.00) then Result := 5.00;
  except
    Result:=Defaut;
  end;
end;


//******************************************************************************

// test d'intersection de deux rectangles
{

   |---------------------------------------------------------------|
   |                                    |-------------------------||
   |                                    |                         ||
   |                                    |                         ||
   ||---------------------------------------|                     ||
   ||                                   |---|---------------------||
   ||                                       |                      |
   ||                                       |                      |
   ||                                       |                      |
   ||                                       |                      |
   ||---------------------------------------|                      |
   |---------------------------------------------------------------|


}
function IntersectRectangles(const R1, R2: TRect2Df): boolean;
var
  BoundingBox: TRect2Df;  // boite englobante
  LB, HB     : double;    // taille de la boite englobante
  LM, HM     : double;    // taille du plus petit rectangle contenant
                          // les deux rectangles R1 et R2 en diagonale

begin
  Result:=False;
  BoundingBox.X1 := Math.Min(R1.X1, R2.X1);
  BoundingBox.Y1 := Math.Min(R1.Y1, R2.Y1);

  BoundingBox.X2 := Math.Max(R1.X2, R2.X2);
  BoundingBox.Y2 := Math.Max(R1.Y2, R2.Y2);

  LB:= BoundingBox.X2 - BoundingBox.X1;
  HB:= BoundingBox.Y2 - BoundingBox.Y1;

  LM:=(R1.X2 - R1.X1) + (R2.X2 - R2.X1);
  HM:=(R1.Y2 - R1.Y1) + (R2.Y2 - R2.Y1);

  Result:= (LM > LB) and (HM > HB);
end;


// obtenir un tableau 2D  partir d'une string
function GetStringArray2D(const S: string; const SepCol: string): TGHStringArray2D;
var
  i,j: integer;
  P : integer;
  S1: string;
  MesLignes: TGHStringArray;
begin
  //NbLignes := 0;
  // pour ne pas tre perturb par les dbuts de ligne
  P := Pos(SepCol,S);
  if (P>0) then S1 := '$' + SepCol + S + SepCol+'$';
  S1 := Trim(S);
  // purger
  for i:=0 to MAX_SIZE_PARAM_ARRAY do
    for j:=0 to MAX_SIZE_PARAM_ARRAY do
      Result[i,j] :='';

  // virer les retour-chariots
  P:=0;
  P:=Pos(#13, S1);
  while (P>0) do begin
    System.Delete(S1, P, 1);
    P:=Pos(#13, S1);
  end;

  MesLignes := Split(S, #10);
  for i:=0 to MAX_SIZE_PARAM_ARRAY do
    AfficherMessage(Format('>>GetStringArray2D: MesLignes[%d] = %s',[i, MesLignes[i]]));
  // dcouper les lignes
  for i:=0 to MAX_SIZE_PARAM_ARRAY do
    Result[i] := Split(MesLignes[i], SepCol);
  // contrle:
  (*
  for i:=0 to MAX_SIZE_PARAM_ARRAY do
    for j:= 0 to MAX_SIZE_PARAM_ARRAY do
      Result[i][j]
  //*)
end;


// encodage/Dcodage scuris des dates
function  GetSecuredDate(Year, Month, Day: Word): TDateTime;
begin
  try
    if (Year < 100) then
    begin
      if (Year < 50) then
        Year := 2000 + Year
      else
        Year := 1900 + Year;
    end;
    Result := EncodeDate(Year, Month, Day);
  except
    result := Now();
  end;
end;
procedure SafeDecodeDate(Date: TDateTime; var Year, Month, Day: Word);
begin
  DecodeDate(Date, Year, Month, Day);
end;



function DescribeEPSGSystem(const CodeEPSG: integer): string; deprecated;
var
  FuseauUTM: integer;
begin
  // pas de case of ici: les valeurs peuvent dpasser la limite d'un case of (65535)
  if      (CodeEPSG = 0)                                      then    Result := 'Local UCS'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_93)                   then    Result := 'Lambert 93 France'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_I_NTF_DEPRECATED)     then    Result := 'NTF (Paris) / Lambert North France'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_II_NTF_DEPRECATED)    then    Result := 'NTF (Paris) / Lambert Centre France'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_III_NTF_DEPRECATED)   then    Result := 'NTF (Paris) / Lambert South France'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_IV_NTF_DEPRECATED)    then    Result := 'NTF (Paris) / Lambert Corsica'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_I)                    then    Result := 'NTF (Paris) / Lambert zone I'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_II)                   then    Result := 'NTF (Paris) / Lambert zone II'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_III)                  then    Result := 'NTF (Paris) / Lambert zone III'
  else if (CodeEPSG = CODE_EPSG_LAMBERT_IV)                   then    Result := 'NTF (Paris) / Lambert zone IV'
  else if (CodeEPSG = CODE_EPSG_GOOGLE)                       then    Result := 'Google Projection'
  // UTM
  else if ((CodeEPSG div 100) = 326) then
  begin
    FuseauUTM := CodeEPSG MOD 100;
    Result := Format('UTM zone %d', [FuseauUTM]);
  end
  else Result := 'Unknown projection';

end;

//-----------------
function GetIDStation(const N: integer; const St: TStation): string;
var
  miaou: string;
begin
  miaou := Trim(St.IDTerrainStation);
  Result := IIF(miaou = '', '', miaou);
end;
// conversion pour un tableau de coordonnes
function ConversionTableauCoordonneesIsoleesEPSG(const Src, Tgt: integer; const ArrCoordsInput: TArrayPoints2Df; out ArrPointsConverted: TArrayPoints2Df): boolean; deprecated;
var
  i, Nb: integer;
  WU, EWE: TProjUV;
  TC: TConversionSysteme;
begin
  Result := False;
  Nb := High(ArrCoordsInput) + 1;
  if (Nb < 0) then Exit;
  SetLength(ArrPointsConverted, Nb);
  TC := TConversionSysteme.Create;
  try
    if (TC.Initialiser) then
    begin
      for i := 0 to Nb - 1 do
      begin
        WU.U := ArrCoordsInput[i].X;
        WU.V := ArrCoordsInput[i].Y;
        EWE := TC.ConversionSyst1ToSyst2EPSG(Src, Tgt, WU);
        ArrPointsConverted[i].X := EWE.U;
        ArrPointsConverted[i].Y := EWE.V;
      end;
    end;
    TC.Finaliser;
    Result := True;
  finally
    TC.Free;
  end;
end;
// conversion pour un tableaude coordonnes isoles (version EPSG)
function ConversionCoupleCoordonneesIsolees(const Src, Tgt: integer; const PointIn: TPoint2Df; out PointOut: TPoint2Df): boolean; deprecated;
var
  WU, EWE: TProjUV;
  TC: TConversionSysteme;
begin
  Result := False;
  TC := TConversionSysteme.Create;
  try
    if (TC.Initialiser) then
    begin
      WU.U := PointIn.X;
      WU.V := PointIn.Y;

      EWE := TC.ConversionSyst1ToSyst2EPSG(Src, Tgt, WU);
      PointOut.X := EWE.U;
      PointOut.Y := EWE.V;
    end;
    TC.Finaliser;
    Result := True;
  finally
    TC.Free;
  end;
end;

// conversion pour un couple de coordonnes isoles (version EPSG)
function ConversionCoupleCoordonneesIsoleesEPSG(const Src, Tgt: integer; const PointIn: TPoint2Df; out PointOut: TPoint2Df): boolean;
var
  WU, EWE: TProjUV;
  TC: TConversionSysteme;
begin
  Result := False;
  TC := TConversionSysteme.Create;
  try
    if (TC.Initialiser) then
    begin
      WU.U := PointIn.X;
      WU.V := PointIn.Y;

      EWE := TC.ConversionSyst1ToSyst2EPSG(Src, Tgt, WU);
      PointOut.X := EWE.U;
      PointOut.Y := EWE.V;
    end;
    TC.Finaliser;
    Result := True;
  finally
    TC.Free;
  end;
end;

function ConversionCoordonneesIsoleesEPSG(const Src, Tgt: integer; const PointInX, PointInY: double; out PointOutX, PointOutY: double): boolean;
var
  PIN, POUT: TPoint2Df;
begin
  PIN  := MakeTPoint2Df(PointInX, PointInY);
  POUT := MakeTPoint2Df(0.00, 0.00);
  Result := ConversionCoupleCoordonneesIsoleesEPSG(src, Tgt, PIN, POUT);
  PointOutX := POUT.X;
  PointOutY := POUT.Y;
end;

// calcul d'une dclinaison magntique isole
function CalcDeclimagIsolee(const Lon, Lat, Alt: double; const MyDate: TDate; out DecliMagValue: double): boolean;
var
  FCalculateurDeclimag: TSAGeoMag;
begin

  result := false;
  FCalculateurDeclimag := TSAGeoMag.Create;
  try
    FCalculateurDeclimag.Initialiser('');
      DecliMagValue := FCalculateurDeclimag.CalculerDeclinaison(Lon, Lat, Alt, MyDate);
    FCalculateurDeclimag.Finaliser();
    result := True;
  finally
    FCalculateurDeclimag.Free;
  end;

end;

// version du XML et encodage
function FormatterVersionEncodageXML(const version: string; const encodage: string): string; inline;
begin
  Result := Format('<?xml version="%s" encoding="%s"?>', [version, encodage]);
end;

function EmptyEntiteEtendue: TEntiteEtendue;
begin
  with Result do
  begin
    eCode       := 0;
    eExpe       := 0;
    eSecteur    := 0;
    eReseau     := 0;
    eEntrance   := 0;
    Type_Entite := tgSURFACE; //2
    DateLeve    := Now();

    Entite_Serie    := 0;
    Entite_Station  := 0;
    // drapeau
    Drawn           := false;
    // donnes originales
    oLongueur       := 0.00;
    oAzimut         := 0.00;
    oPente          := 0.00;
    oLG             := 0.00;
    oLD             := 0.00;
    oHZ             := 0.00;
    oHN             := 0.00;
    // centerline
    Une_Station_1_X := 0.00;
    Une_Station_1_Y := 0.00;
    Une_Station_1_Z := 0.00;
    Une_Station_2_X := 0.00;
    Une_Station_2_Y := 0.00;
    Une_Station_2_Z := 0.00;
    // habillage
    X1PD := 0.00;
    Y1PD := 0.00;
    X1PG := 0.00;
    Y1PG := 0.00;
    Z1PH := 0.00;
    Z1PB := 0.00;

    X2PD := 0.00;
    Y2PD := 0.00;
    X2PG := 0.00;
    Y2PG := 0.00;
    Z2PH := 0.00;
    Z2PB := 0.00;
    ColourByDepth := clWhite;
    IsPOI:= false;
    // commentaires
    oIDLitteral   := '';
    oCommentaires := '';
  end;
end;




//******************************************************************************
// copie et sauvegarde de tableaux
//******************************************************************************

procedure ExecuteExternalProgram(const MyCommandLine: string);
begin
  (* Cration du processus *)
  with TProcess.Create(Application) do
  begin
    try
      (* Dfinition de la commande  excuter *)
      CommandLine := MyCommandLine;
      (* On Attendra la fin du programme externe avant de continuer *)
      Options := Options + [poWaitOnExit];
      (* Tout est prt pour l'excution *)
      Execute;
    finally
      Free;
    end;
  end;
end;

function GRDCCopierUnTableau(const Grille: TStringGrid): boolean;
var
  WU: String;
  l, c: Integer;
  ClipBoard: TClipboard;
begin
  result := False;
  WU := '';
  ClipBoard := TClipboard.Create(ctClipboard);
  try
    ClipBoard.Clear;
    // contenu
    for l := 0 to Grille.RowCount - 1 do
    begin
      for c := 0 to Grille.ColCount - 1 do
      begin
        WU := WU + Grille.Cells[c, l] + #9;
      end;
      WU := WU + #13#10;
    end;
    ClipBoard.AsText := WU;
    result := True;
  finally
    ClipBoard.Free;
  end;
end;

function GRDCollerDepuisClipBoard(const Grille: TStringGrid; const HasTitres: boolean; const QSeparateur: string): boolean;
var
  LesLignes: TStringList;
  EWE: String;
  P: SizeInt;
  Lin : String;
begin
  AfficherMessageErreur('GRDCollerDepuisClipBoard');
  Result := False;
  LesLignes := TStringList.Create;
  try
    LesLignes.Clear;
    try
      EWE := Trim(ClipBoard.AsText) + #13+#10;
      // dcoupage du texte
      repeat
        P := Pos(#10, EWE);
        Lin := Trim(Copy(EWE, 0, P-1));
        EWE := Copy(EWE, P+1, Length(EWE));
        if (Lin <> '') then LesLignes.Add(Lin);  // on zappe les lignes vides
      until (P = 0);
      // traitement des lignes
      if (LesLignes.Count = 0) then Exit;
      // remplissage du tableau
      GRDRemplirDepuisStringList(Grille, LesLignes, HasTitres, QSeparateur);
    except
    end;
    LesLignes.Clear;
  finally
    LesLignes.Free;
    //ClipBoard.Free; // ne pas librer le presse-papiers
  end;
end;
function GRDChargerDepuisFichier(const Grille: TStringGrid; const HasTitres: boolean; const FileName: string; const QSeparateur: string): boolean;
var
  LesLignes: TStringList;
begin
  Result := False;
  LesLignes := TStringList.Create;
  try
    LesLignes.Clear;
    LesLignes.LoadFromFile(FileName);
    // traitement des lignes
    if (LesLignes.Count = 0) then Exit;
    // remplissage du tableau
    GRDRemplirDepuisStringList(Grille, LesLignes, HasTitres, QSeparateur);
    LesLignes.Clear;
    Result := True;
  finally
    LesLignes.Free;
  end;
end;

procedure GRDRemplirDepuisStringList(const Grille: TStringGrid; const LesLignes: TStringList; const HasTitres: boolean; const Separateur: string);
var
  i, j: Integer;
  ValeursColonnes: TGHStringArray;
  R: Integer;
begin

  Grille.ColCount := 1 + MAX_SIZE_PARAM_ARRAY + 2; // nb de colonnes = taille d'un TStringArray
  Grille.RowCount := IIF(HasTitres, LesLignes.Count, LesLignes.Count + 1);
  // on n'implmente pas un tableur => grille brute
  Grille.FixedCols   := 1;
  Grille.FixedRows   := 1;
  for i:= 0 to LesLignes.Count -1 do
  begin
    ValeursColonnes := Split(LesLignes[i], Separateur);
    if (HasTitres) then R := i else R := i+1;
    // renumrotation des lignes
    Grille.Cells[0, R] := IntToStr(i);
    for j := 0 to High(ValeursColonnes) do Grille.Cells[j+1, R] := ValeursColonnes[j];
  end;

end;

function GRDSauverUnTableauEnCSV(const Grille: TStringGrid; const FichierCSV: TStringDirectoryFilename): boolean;
begin
 Grille.SaveToCSVFile(FichierCSV, #9);
end;



// Dates SQL
function DatePascalToDateSQL(const MyDate: TDateTime): string;
var
  YYYY, MM, DD: word;
begin
  DecodeDate(MyDate, YYYY, MM, DD);
  Result := Format('%.4d-%.2d-%.2d', [YYYY, MM, DD]);
end;
function DateSQLToDatePascal(const MyDateSQL: string): TDateTime;
var
  YYYY, MM, DD: word;
  QDateSQL: String;
begin
  QDateSQL := Trim(MyDateSQL);
  try
    YYYY := StrToInt(Copy(QDateSQL, 1, 4));
    // annes sur deux chiffres
    if (YYYY < 100) then
    begin
      if (YYYY < 50) then  YYYY := 2000 + YYYY
                     else  YYYY := 1900 + YYYY;
    end;
    MM   := StrToInt(Copy(QDateSQL, 6, 2));
    DD   := StrToInt(Copy(QDateSQL, 9, 2));
    Result := EncodeDate(YYYY, MM, DD);
  except
    Result := TDateTime(0.0);
  end;
end;
function DateTimeSQLToDateTimePascal(const MyDateSQL: string): TDateTime;
var
  YYYY, MM, DD: word;
  HH, MN, SS, MS: word;
  QDate, QTime: TDateTime;
  QDateSQL: String;
begin
  QDateSQL := Trim(MyDateSQL);
  try
    YYYY := StrToInt(Copy(QDateSQL, 1, 4));
    // annes sur deux chiffres
    if (YYYY < 100) then
    begin
      if (YYYY < 50) then  YYYY := 2000 + YYYY
                     else  YYYY := 1900 + YYYY;
    end;
    MM   := StrToInt(Copy(QDateSQL, 6, 2));
    DD   := StrToInt(Copy(QDateSQL, 9, 2));
    // 0        1111111112
    // 1234567890123456789
    // 2017-01-02 12:34:56

    HH   := StrToInt(Copy(QDateSQL, 12, 2));
    MN   := StrToInt(Copy(QDateSQL, 15, 2));
    SS   := StrToInt(Copy(QDateSQL, 18, 2));
    MS   := 0;
    QDate := EncodeDate(YYYY, MM, DD);
    QTime := EncodeTime(HH, MN, SS, MS);
    Result := ComposeDateTime(QDate,QTime);
  except
    Result := TDateTime(0.0);
  end;
end;

function DateTimePascalToDateTimeSQL(const MyDate: TDateTime; const WithMilliseconds: boolean = true): string;
var
  YYYY, MM, DD: word;
  HH, MN, SS, MS: word;
begin
  DecodeDate(MyDate, YYYY, MM, DD);
  DecodeTime(MyDate, HH, MN, SS, MS);
  Result := Format('%.4d-%.2d-%.2d %.2d:%.2d:%.2d', [YYYY, MM, DD, HH, MN, SS]);
  if (WithMilliseconds) then Result += Format('.%.3d', [MS]);
end;

// Dates condenses AAAAMMJJ
function DatePascalToDateCondensee(const QDate: TDateTime): string;
var
  YYYY, MM, DD: word;
begin
  DecodeDate(QDate, YYYY, MM, DD);
  Result := Format('%.4d-%.2d-%.2d', [YYYY, MM, DD]);
end;
// Dates condenses AAAAMMJJ_HHMNSSMS
function DatePascalToDateHeureCondensee(const QDate: TDateTime): string;
var
  YYYY, MM, DD: word;
  HH, MN, SS, MS: word;
begin
  DecodeDate(QDate, YYYY, MM, DD);
  DecodeTime(QDate, HH, MN, SS, MS);
  Result := Format('%.4d-%.2d-%.2d_%.2dh%.2dm%.2d,%.3ds', [YYYY, MM, DD, HH, MN, SS, MS]);
end;


// Dtermine le type de filtre:
//    -1 = Filtre erronn
//     0 = Valeur simple
//     1 = Intervalle
function ChooseFilter(const s90: string): integer;
var
  i: Integer;
  C: Boolean;
begin

  Result:=-1;
  Result := IndexOfString(s90, False, ArrayOfFilters);
  (*Result := IndexOfString(s90, False, [rsMETAFILTRE_NIL,         // 0
    rsMETAFILTRE_ALL,         // 1
    rsMETAFILTRE_ID,          // 2
    rsMETAFILTRE_LENGTH,      // 3
    rsMETAFILTRE_AZIMUTH,     // 4
    rsMETAFILTRE_PENTE,       // 5
    rsMETAFILTRE_DATE,        // 6
    rsMETAFILTRE_COLOR,       // 7
    rsMETAFILTRE_X,           // 8
    rsMETAFILTRE_Y,           // 9
    rsMETAFILTRE_Z,           // 10
    rsMETAFILTRE_LARGEUR,     // 11
    rsMETAFILTRE_HAUTEUR,     // 12
    rsMETAFILTRE_DATES,       // 13
    rsMETAFILTRE_COLORS,      // 14
    rsMETAFILTRE_SERIE,       // 15
    rsMETAFILTRE_RESEAU,      // 16
    rsMETAFILTRE_CODE,        // 17
    rsMETAFILTRE_EXPE,        // 18
    rsMETAFILTRE_TYPEVISEE,   // 19
    rsMETAFILTRE_SECTEUR,     // 20
    rsMETAFILTRE_ENTREE_RATT  // 21
                                      ]); //*)

end;
// convertir un  nombre Pascal en nombre OOo Calc
function  PasNumberToOOoCalcFormula(const V: double): string;
var
  EWE: String;
begin
  EWE := '=' + FloatToStr(V);
  Result := StringReplace(EWE, DefaultFormatSettings.DecimalSeparator, ',', [rfReplaceAll, rfIgnoreCase]);
end;
// formatage OOo des nombres
function FormatterNombreOOo(const Value: extended; const NbDecs: integer; const IgnoreNulls: boolean): string; overload;
var
  EWE: String;
begin
  EWE := Format('%%.%df', [NbDecs]);
  // remplacer les points par des virgules
  Result := Format(EWE, [Value]);
  Result := StringReplace(Result, DefaultFormatSettings.DecimalSeparator, ',', [rfReplaceAll, rfIgnoreCase]);
  if (IgnoreNulls) then
  begin
    if (SameValue(Value, 0)) then Result := '';
  end;
end;

function FormatterNombreOOo(const Value: extended): string; overload;
begin
  Result := FormatterNombreOOo(Value, 2, true);
end;

// formatage des nombres pour KML et OSM
function ensureConvertLongOrLatToXML(const Value: extended): string;
begin
  Result := Format('%.10f', [Value]);
  Result := StringReplace(Result, DefaultFormatSettings.DecimalSeparator, '.', [rfReplaceAll, rfIgnoreCase]);
end;
// formatage des nombres pour KML et OSM
function ensureConvertAltitudeToXML(const Value: extended): string;
begin
  Result := Format('%.2f', [Value]);
  Result := StringReplace(Result, DefaultFormatSettings.DecimalSeparator, '.', [rfReplaceAll, rfIgnoreCase]);
end;

// convertir en nombre des valeurs de type 123 456 789.10 ou 1245-45785
function ConvertirEnNombreReel(const S: string; const Default: Extended): Extended;
var
  st: String;
  m1: extended;
  i : integer;
  c : char;
  procedure S666(const QC: char);
  begin
    if (st[1] = '-') then m1 := -1.00;
    if (st[1] = QC) then system.Delete(st, 1, 1);
  end;

begin
  try
    Result := Default;
    m1 := 1.00;
    // on nettoie la chane
    st := Trim(s);
    if (st = '') then Exit(Default);
    // on vire les guillemets
    st := StringReplace(st, '"', ' ', [rfIgnoreCase, rfReplaceAll]);

    S666('''');       // on vire l'apostrophe de dbut
    S666('-');        // sauvegarde du signe
    S666('+');        // on vire le '+' de tte

    // balayage avec nettoyage et remplacement du point dcimal
    ST := StringReplace(ST, '.', DefaultFormatSettings.DecimalSeparator, [rfIgnoreCase, rfReplaceAll]);
    ST := StringReplace(ST, ',', DefaultFormatSettings.DecimalSeparator, [rfIgnoreCase, rfReplaceAll]);
    // on ne retient que les chiffres
    result := StrToFloat(ST) * m1;
  except
    Result := Default;
  end;
end;
// convertir une dclinaison en deg dc en dclinaison Toporobot (unit et signe)
// TODO: La dclinaison est-elle bien en GRADES et de signe OPPOSE
function makeToporobotDeclimag(const decl: double): double;
begin
  Result := -degtograd(decl);
end;

function BinToInt(const BinValue: string): integer;
var
  i, Nb: Integer;
begin
  Result := 0;
  Nb := length(BinValue);
  for i := Nb-1 downto 0 do
  begin
    Result := Result + 2 ** i * IIF(BinValue[i] = '1', 1, 0);
  end;
end;

//TTypeMesureDistoX = (tmdxUNKNOWN, tmdxCHEMINEMENT, tmdxLRUD, tmdxANTENNE, tmdxSIGNALISATION);
function getLitteralTypeMesureDistoX(const T: TTypeMesureDistoX): string;
begin
  Result := ChooseString(Ord(T),
                         ['Inconnu',
                          'Cheminement',
                          'LRUD data',
                          'Antenne',
                          'Signalisation'
                         ]);
end;
// sortie formate pour colonnages fixes
function FormatterFixedColumn(const S: string; const W: integer; const Aligment: TAlignment): string;
var
  L, LL: Integer;
  G, D: Integer;
  EWE: String;
begin
  EWE := Trim(S);
  LL := Length(EWE);
  L := W - LL;
  if (LL > W) then EWE := Copy(EWE, 1, W); // tronque si S est plus grand que la colonne
  case Aligment of
    taLeftJustify   : G := 0;
    taRightJustify  : G := L;
    taCenter        : G := L div 2;
  end;
  case Aligment of
    taLeftJustify   : D := L;
    taRightJustify  : D := 0;
    taCenter        : D := (L div 2) - 1;
  end;
  Result := StringOfChar(#32, G) + EWE + StringOfChar(#32, D);
end;

// date au format JJ/MM/AA HH:MM:SS
function FormatterDateHeureEnFrancais(const D: TDateTime): string;
var
  JJ, MM, AA, HH, MN, SS, MS: word;
begin
  DecodeDate(D, AA, MM, JJ);
  AA := AA mod 100; // ne retenir que les deux derniers chiffres
  DecodeTime(D, HH, MN, SS, MS);
  Result := Format('%.2d/%.2d/%.2d %.2d:%.2d:%.2d', [JJ, MM, AA, HH, MN, SS]);
end;
// date au format JJ/MM/AA
function FormatterDateEnFrancais(const D: TDateTime): string;
var
  JJ, MM, AA: word;
begin
  DecodeDate(D, AA, MM, JJ);
  AA := AA mod 100;
  Result := Format('%.2d/%.2d/%.2d', [JJ, MM, AA]);
end;
// date au format JJ/MM/AAAA HH:MM:SS:MS
function FormatterDateHeureAvecMiliSecEnFrancais(const D: TDateTime): string;
var
  JJ, MM, AA, HH, MN, SS, MS: word;
begin
  DecodeDate(D, AA, MM, JJ);
  AA := AA mod 100; // ne retenir que les deux derniers chiffres
  DecodeTime(D, HH, MN, SS, MS);
  Result := Format('%.2d/%.2d/%.2d %.2d:%.2d:%.2d.%.3d', [JJ, MM, AA, HH, MN, SS, MS]);
end;
//*******************************************************************************
function MakeTParamFoncCorrectionAngulaire(const QCo, QErreurMax, QPosErrMax: double): TParamFoncCorrectionAngulaire;
begin
  Result.Co        := QCo;
  Result.ErreurMax := QErreurMax;
  Result.PosErrMax := QPosErrMax;
end;
// calcul d'un azimut corrig par une fonction trigonomtrique
// Le terme de correction correspont  l'angle form par le Nord Magntique et le Nord indiqu par l'instrument.
// Correction sur azimuts             = Cb = 0,00 degrs (angle entre le barreau et le 0 de la graduation)
// Terme de correction maxi           = Ct = 0.437 degrs (par rapport au Nord Magntique)
// Azimut du terme de correction maxi = AzCt = 90,00
// Formule:
// Result = (AzB + Cb) + Ct * sin(90 - (AzB + Cb + AzCt))

function CalcAzimutCorrige(const AzBrut: double;
                           const UniteBoussole: double;                    // 360 = degr, 400 = grade
                           const ParamsFunc: TParamFoncCorrectionAngulaire): double;
const
  TWO_PI = 2 * PI;
var
  AzC: Extended;
  UnitToRad: Extended;
  Quadrant : Extended;
  TermeTrigo: Extended;
begin
  //AfficherMessageErreur(Format('%.3f - %.2f - %.6f, %.6f, %.6f', [AzBrut, UniteBoussole, ParamsFunc.Co, ParamsFunc.ErreurMax, ParamsFunc.PosErrMax]));
  Result     := AzBrut;
  // erreur systmatique constante: mauvais positionnement de l'aiguille aimante
  AzC        := AzBrut + ParamsFunc.Co;
  UnitToRad  := TWO_PI / UniteBoussole;
  Quadrant   := UniteBoussole / 4;
  // prise en compte de l'erreur d'excentrement de l'axe de la boussole
  // PosErreuMax: Angle sous lequel est observe l'erreur la plus importante
  // ErreurMax: Valeur de l'erreur maximale
  TermeTrigo := UnitToRad * (Quadrant - (AzC - ParamsFunc.PosErrMax));
  Result     := AzC + ParamsFunc.ErreurMax * sin(TermeTrigo);
  if (Result > UniteBoussole) then Result := Result - UniteBoussole;
end;

function CalcPenteCorrigee(const PenteBrute: double;
                           const UniteClino: double;                    // 360 = degr, 400 = grade
                           const ParamsFunc: TParamFoncCorrectionAngulaire): double;
const
  TWO_PI = 2 * PI;
var
  UnitToRad: Extended;
  TermeTrigo: Extended;
  PenteC: Extended;
begin
  Result     := PenteBrute;
  // erreur systmatique constante: mauvais positionnement du limbe sur la couronne du clinomtre
  PenteC     := PenteBrute + ParamsFunc.Co;
  UnitToRad  := TWO_PI / UniteClino;
  // prise en compte de l'erreur d'excentrement de l'axe de la couronne du clinomtre
  // PosErreuMax: Angle sous lequel est observe l'erreur la plus importante
  // ErreurMax: Valeur de l'erreur maximale
  TermeTrigo := UnitToRad * (PenteC - ParamsFunc.PosErrMax);
  // projection de l'erreur max sur la perpendiculaire  la vise
  // Ex: Sachant PosErrMax = 0, on considre que ErrMax est maximum pour une pente nulle
  //     et son influence varie avec le cos de (PenteC - ParamsFunc.PosErrMax), donc de la pente.
  Result     := PenteC + ParamsFunc.ErreurMax * abs(cos(TermeTrigo));
  //AfficherMessageErreur(Format('CalcPenteCorrigee:  Co = %.3f, CMax = %.3f - Incl = %.3f - PenteC = %.3f', [ParamsFunc.Co, ParamsFunc.ErreurMax, PenteBrute, Result]));
end;

function DecomposeStationToporobot(const S: string): TToporobotStation;
var
  EWE: TGHStringArray;
begin
  result.eSerie:= -1;
  result.ePoint:= -1;
  if (Pos('.', S) = 0) then Exit;
  EWE := Split(S, '.');
  try
    Result.eSerie := StrToIntDef(EWE[0], -1);
    Result.ePoint := StrToIntDef(EWE[1], -1);
  except
  end;
end;

//==============================================================================
// version de la fonction ExtractStrings() mais envoie le rsultat dans un TStringArray au lieu d'un TStringList
function ExtractStringsToTStringArray(Separators, WhiteSpace: TSysCharSet;
                                      const MyStr: string): TGHStringArray;
var
  b, c : pchar;
  quoted : char;
  IdxResultArray: Integer;
  procedure SkipWhitespace;
  begin
    while (c^ in Whitespace) do inc (c);
  end;
  procedure AddString;
  var
    l : integer;
    s : string;
  begin
    l := c-b;
    if (l > 0) then
    begin
      setlength(s, l);
      move (b^, s[1],l);
      //Strings.Add (s);
      Result[IdxResultArray] := s;
      IdxResultArray += 1;
    end;
  end;
begin
  // vider le tableau rsultat
  for IdxResultArray := 0 to High(Result) do Result[IdxResultArray] := '';
  IdxResultArray := 0;
  c := PChar(MyStr);
  Quoted := #0;
  //Separators := Separators + [#13, #10] - ['''','"'];
  Separators := Separators + [#13, #10] - ['"'];
  SkipWhitespace;
  b := c;
  while (c^ <> #0) do
  begin
    if (c^ = Quoted) then
    begin
      if ((c+1)^ = Quoted) then
        inc (c)
      else
        Quoted := #0
    end
    //else if (Quoted = #0) and (c^ in ['''','"']) then
    else if (Quoted = #0) and (c^ in ['"']) then
      Quoted := c^;
    if (Quoted = #0) and (c^ in Separators) then
    begin
      AddString;
      inc (c);
      SkipWhitespace;
      b := c;
    end
    else
      inc (c);
  end;
  if (c <> b) then AddString;
end;



function DecoupeChaine(const S: string;
                       const Separators, WhiteSpace: TSysCharSet;
                       const DoRemoveQuotes: boolean): TGHStringArray;
var
  i: Integer;
  function RemoveQuote(const S: string): string;
  begin
    Result := StringReplace(S, '"', '', [rfReplaceAll]);
  end;
begin
  Result := ExtractStringsToTStringArray(Separators, WhiteSpace, S);
  if (DoRemoveQuotes) then
  begin
    for i := 0 to high(Result) do Result[i] := RemoveQuote(Result[i]);
  end;
end;

function DecoupeChaineEspacesGuillemets(const S: string; const DoRemoveQuotes: boolean): TGHStringArray;
var
  i: Integer;
  function RemoveQuote(const S: string): string;
  begin
    Result := StringReplace(S, '"', '', [rfReplaceAll]);
  end;
begin
  Result := ExtractStringsToTStringArray([' '], [], S);
  if (DoRemoveQuotes) then
  begin
    for i := 0 to high(Result) do Result[i] := RemoveQuote(Result[i]);
  end;
end;

function MakeTReseau(const QColorReseau: TColor; const QTypeReseau: integer; const QNomReseau, QObsReseau: string): TReseau;
begin
  //result.IdxReseau       := QIdxReseau;
  result.ColorReseau     := QColorReseau;
  result.TypeReseau      := QTypeReseau;
  result.NomReseau       := QNomReseau;
  result.ObsReseau       := QObsReseau;
end;
function MakeTSecteur(const QCouleurSecteur: TColor; const QNomSecteur: string): TSecteur;
begin
//  result.IdxSecteur      := QIdxSecteur;
  result.CouleurSecteur  := QCouleurSecteur;
  Result.NomSecteur      := QNomSecteur;
end;
function MakeTEntrance(const QNomEntree: string;
                       const QIDTerrain: string;
                       const QXEntree  : double;
                       const QYEntree  : double;
                       const QZEntree  : double;
                       const QRefSer   : integer;
                       const QRefSt    : integer;
                       const QObserv   : string): TEntrance;
begin
  //result.eNumEntree := QNumEntree;
  result.eNomEntree := QNomEntree;
  result.eIDTerrain := QIDTerrain;

  result.eXEntree   := QXEntree;
  result.eYEntree   := QYEntree;
  result.eZEntree   := QZEntree;
  result.eRefSer    := QRefSer;
  result.eRefSt     := QRefSt;
  result.eObserv    := QObserv;
end;
function MakeTStationMatchFound(const S, P: integer; const M: string): TStationMatchFound;
begin
  Result.Serie   := S;
  Result.Station := P;
  Result.Match   := M;
end;

function IsSameSerie(const E1,E2: TEntiteEtendue): boolean;
begin
  Result:= ((E1.Entite_Serie   = E2.Entite_Serie) and
            (E1.Entite_Station = (E2.Entite_Station + 1)));

end;

// RTTI
function GetDescTModeRepresentationGaleriesType(const Idx: TModeRepresentationGaleries): string;
begin
  Result := ''; //GetEnumName(TypeInfo(TModeRepresentationGaleries), Ord(Idx));
end;

function DegMinSec2DegDec(const S: string): double;
var
  WU: String;
  EWE: TGHStringArray;
  SigneLongitude, i: Integer;
  dd, mn, ss, Signe: Extended;
  QAT: Boolean;
begin
  try
    WU := Trim(UpperCase(S));
    // extraction du signe
    QAT := (Pos('-', WU) > 0) or
           (Pos('S', WU) > 0) or
           (Pos('W', WU) > 0) or
           (Pos('O', WU) > 0);
    Signe := IIF(QAT, -1.0, 1.0);
    // nettoyage
    for i := length(WU) downto 1 do
    begin
      if (not(WU[i] in ['0' .. '9', chr(194), '-', '.', ',', '''', '"', 'N', 'M', 'S', 'O', 'W', 'E', 'D'])) then
        system.Delete(WU, i, 1);
    end;

    //SigneLongitude := IIF((Pos('O', WU) > 0) or (Pos('W', WU) > 0) or (Pos('S', WU) > 0), -1, 1);
    WU := StringReplace(WU, ',', DefaultFormatSettings.DecimalSeparator, [rfReplaceAll, rfIgnoreCase]);

    WU := StringReplace(WU, chr(194), 'd', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'd', ':', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, '''', ':', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, '"', ':', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, '''''', ':', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'd', ':', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'N', '', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'S', '', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'E', '', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'O', '', [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, 'W', '', [rfReplaceAll, rfIgnoreCase]);
    EWE := Split(WU, ':');

    AfficherMessageErreur(Format('"%s"', [EWE[0]]));
    AfficherMessageErreur(Format('"%s"', [EWE[1]]));
    AfficherMessageErreur(Format('"%s"', [EWE[2]]));
    //WU := '';
    //for i := 1 to Length(EWE[0]) do WU += Format('%.4d,', [Ord(EWE[0][i])]);
    //AfficherMessageErreur(WU);
    dd := Abs(ConvertirEnNombreReel(EWE[0], 0.00));
    mn := Abs(ConvertirEnNombreReel(EWE[1], 0.00));
    ss := Abs(ConvertirEnNombreReel(EWE[2], 0.00));
    Result := Signe * (dd + mn/60 + ss/3600);

  except
    Result := -361.00;
  end;
end;

function ChooseColorByTypeEntite(const TE: TTypeDeVisee): TColor;
begin
  case TE of
    tgDEFAULT,
    tgFOSSILE   : result := clRed;
    tgVADOSE    : result := clBlue;
    tgENNOYABLE : result := clNavy;
    tgSIPHON    : result := clFuchsia;
    tgSURFACE   : result := clLime;
    tgTUNNEL    : result := clMaroon;
    tgMINE      : result := clTeal;
    tgVISEE_RADIANTE: result := clGray;
  else
    result := clBlack;
  end;
end;

function RemoveDiacritiques(const S: string): string;
begin
  //Result := AnsiToUtf8(S);
  Result := S;
  // pour le franais
  Result := StringReplace(Result, #195#169, 'e', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#168, 'e', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#170, 'e', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#171, 'e', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#160, 'a', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#162, 'a', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#164, 'a', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#174, 'i', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#175, 'i', [rfIgnoreCase, rfReplaceAll]);   //
  Result := StringReplace(Result, #195#180, 'o', [rfIgnoreCase, rfReplaceAll]);   // 
  Result := StringReplace(Result, #195#182, 'o', [rfIgnoreCase, rfReplaceAll]);   // 
  Result := StringReplace(Result, #195#185, 'u', [rfIgnoreCase, rfReplaceAll]);       //''
  Result := StringReplace(Result, #195#187, 'u', [rfIgnoreCase, rfReplaceAll]);    // ''
  Result := StringReplace(Result, #195#188, 'u', [rfIgnoreCase, rfReplaceAll]);  //''
  Result := StringReplace(Result, #195#167, 'c', [rfIgnoreCase, rfReplaceAll]);      // 
end;

function BaseNToDecimal(const binval : string; const Base:Int64) : Int64;
var
  i : integer;
begin
  result:=0;
  for i := 1 to Length(binval) do
  begin
    Case BinVal[i] Of
      '0'..'9': Result := Result * base + Ord(BinVal[i]) - Ord('0');
      'A'..'Z': Result := Result * base + Ord(BinVal[i]) - Ord('A') + 10;
    End;
  end;
end;

function MakeGHCaveDrawIDPtCenterline(const E: TEntiteEtendue; var NbViseesEnAntenne: integer): int64;
begin
  if (E.Type_Entite = tgVISEE_RADIANTE) then
  begin
    NbViseesEnAntenne += 1;
    Result := NB_MAXI_SERIES_PAR_CAVITE * Abs(E.Entite_Serie) + 10 * Abs(E.Entite_Station);
    Result := 0 - Result;
  end
  else
  begin
    Result := NB_MAXI_SERIES_PAR_CAVITE * E.Entite_Serie + 10 * E.Entite_Station;
  end;
end;

function FormatterIDTerrainStation(const S: string): string;
begin
  Result := GetResourceString(Trim(S));
end;

function ExplainCodeAzimut(const CC: TCode): string;
var
  ubb: Int64;
  DescUBB, SensVisee: String;
begin
  ubb := Round(CC.GradAz);
  Result := Format('%d: ', [ubb]);
  case ubb of
    359, 360: begin // vises directes en degrs
       DescUBB   := GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES);
       SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);

    end;
    399, 400: begin // vises directes en grades
       DescUBB   := GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES);
       SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
    end;
    349, 350: begin  // vises inverses en degrs
       DescUBB   := GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES);
       SensVisee := GetResourceString(rsDESC_VISEE_INVERSE);
    end;
    389, 390: begin // vises inverses en grades
       DescUBB   := GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES);
       SensVisee := GetResourceString(rsDESC_VISEE_INVERSE);
    end;
  else
    begin
      DescUBB   := Format('%.2f', [CC.GradAz]);
      SensVisee := GetResourceString('??');
    end;
  end;
  result += DescUBB + ', ' + SensVisee;
end;
//******************************************************************************
{
  (* Dterminer si on travaille en znithal*)

//*}
//******************************************************************************
function ExplainCodePente(const CC: TCode): string;
var
  ucc: Int64;
  DescUCC, PosZero, SensVisee: String;
begin
  result := '';
  ucc := Trunc(CC.GradInc);
  Result := Format('%d: ', [ucc]);
  case ucc of
    360:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    370:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_PENTES_PERCENT);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    380: // mode plonge: Longueur, Dnivel
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_PENTE_DENIVELE);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    400:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    359:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_NADIRAL);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    399:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_NADIRAL);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    361:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_DEGRES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_ZENITHAL);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    401:
      begin
        DescUCC   := GetResourceString(rsDESC_UNITE_ANGULAIRE_GRADES);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_ZENITHAL);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    UNITE_CLINO_LASERMETRE_STANLEY_TLM330_DIR:
      begin
        DescUCC   := GetResourceString(rsDESC_DEVICE_TLM330);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_DIRECTE);
      end;
    UNITE_CLINO_LASERMETRE_STANLEY_TLM330_INV:
      begin
        DescUCC   := GetResourceString(rsDESC_DEVICE_TLM330);
        PosZero   := GetResourceString(rsDESC_POS_ZERO_HZ);
        SensVisee := GetResourceString(rsDESC_VISEE_INVERSE);
      end;
    else
      begin
        DescUCC   := Format('%.2f', [CC.GradAz]);
        PosZero   := '??';
        SensVisee := '??';
      end;
  end;
  result += DescUCC + ', ' + SensVisee + ', ' + PosZero;
end;
function PasStrToXMLStr(const S: string): string;
begin
  Result := S; // nothing
end;

function XMLStrToPasStr(const S: string): string;
begin
  Result := S; // nothing
end;

// neutralise les entits HTML (pour le support du XML)
(*
function EscapeXMLEntities(const S: string): string;
begin
  Result := StringReplace(Result, '&', XML_CHAR_AMP, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(S, '<', XML_CHAR_LT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '<', XML_CHAR_LT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '>', XML_CHAR_GT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '"', XML_CHAR_QUOT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '''', XML_CHAR_APOS, [rfIgnoreCase, rfReplaceAll]);
end;
function EscapeXMLEntities(const S: string): string;
begin
  Result := StringReplace(Result, '&', XML_CHAR_AMP, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(S, '<', XML_CHAR_LT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '<', XML_CHAR_LT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '>', XML_CHAR_GT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '"', XML_CHAR_QUOT, [rfIgnoreCase, rfReplaceAll]);
  Result := StringReplace(Result, '''', XML_CHAR_APOS, [rfIgnoreCase, rfReplaceAll]);
end;
//*)

// rcuprer le nombre de coeurs d'un processeur
function GetNbCoresOfProcessor(): integer;
var
  MyCPU: TCPU;
begin
  Result := 1;
  MyCPU := TCPU.Create;
  try
    Result := MyCPU.Count;
  finally
    MyCPU.Free;
  end;

end;




function MakeLabelNoeud(const ND: TJonctionXYZ): String;
var
  QIdxNameSpace, QNoSerie: TNumeroSerie;
  WU, EWE: String;
begin
  //Result := Format('%d.%d', [ND.NoSer, ND.NoSt]);
  DecomposeNumeroSerie(ND.NoSer, QIdxNameSpace, QNoSerie);
  WU := IIF(QIdxNameSpace = 0, '', Format('@%d', [QIdxNameSpace]));
  EWE := IIF(ND.IDJonction = '', '', ' - ' + ND.IDJonction);
  //Result := Format('%d.%d%s%s', [QNoSerie, ND.oNoSt, WU, EWE]);
  Result := Format('%d.%d%s', [QNoSerie, ND.NoSt, WU]);
end;

function MakeLibelleStationTopo(const E: TEntiteEtendue): string;
var
  QIdxNameSpace, QNoSerie: TNumeroSerie;
  WU, EWE: String;
begin
  if (E.Type_Entite = tgVISEE_RADIANTE) then Exit('');
  DecomposeNumeroSerie(E.Entite_Serie, QIdxNameSpace, QNoSerie);
  WU := IIF(QIdxNameSpace = 0, '', Format('@%d', [QIdxNameSpace]));
  EWE := IIF(E.oIDLitteral='', '', ' - ' + E.oIDLitteral);
  Result := Format('%d.%d%s%s', [QNoSerie, E.Entite_Station, WU, EWE]);
end;

function MakeLibelleExpe(const E: TExpe): string;
var
  QIdxNameSpace, QNoExpe: TNumeroExpe;
  WU: String;
begin
  DecomposeNumeroExpe(E.IDExpe, QIdxNameSpace, QNoExpe);

  WU := IIF(QIdxNameSpace = 0, '', Format('@%d', [QIdxNameSpace]));
  Result := Format('%d%s - %s', [QNoExpe, WU, '']);
end;

function MakeLibelleCode(const E: TCode): string;
var
  QIdxNameSpace, QNoCode: TNumeroCode;
  WU: String;
begin
  DecomposeNumeroCode(E.IDCode, QIdxNameSpace, QNoCode);
  WU := IIF(QIdxNameSpace = 0, '', Format('@%d', [QIdxNameSpace]));
  Result := Format('%d%s - %s', [QNoCode, WU, '']);
end;

function MakeTUneVisee(const QNoVisee          : integer;
                       const QTypeVisee        : TTypeDeVisee;
                       const QIdSecteur        : TNumeroSecteur;
                       const QCode             : TNumeroCode;
                       const QExpe             : TNumeroExpe;
                       const QL, QAz, QP       : double;
                       const QLG, QLD, QHZ, QHN: double;
                       const QIDStation        : string = '';
                       const QCommentaire      : string = ''): TUneVisee;
begin
  Result.NoVisee   := QNoVisee;
  result.TypeVisee := QTypeVisee;
  Result.IDSecteur := QIdSecteur;
  Result.Code      := QCode;
  Result.Expe      := QExpe;

  Result.Longueur  := QL;
  Result.Azimut    := QAz;
  Result.Pente     := QP;
  Result.LG        := QLG;
  Result.LD        := QLD;
  Result.HZ        := QHZ;
  Result.HN        := QHN;
  result.IDTerrainStation  := QIDStation;
  Result.Commentaires      := QCommentaire;
  Result.DeltaX    := 0.00;
  Result.DeltaY    := 0.00;
  Result.DeltaY    := 0.00;
  Result.DeltaP    := 0.00;
  Result.AccroissP := 0.00;
  Result.AccroissZ := 0.00;
end;
// vise vide
function EmptyVisee(const C: string): TUneVisee;
begin
  result := MakeTUneVisee(0, tgDEFAULT, 0, 1, 1, 0.001, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, '', C);
end;
function MakeTViseeAntenne(const QIDViseeAntenne          : integer;
                           const QEntranceRatt            : TNumeroEntrance;
                           const QReseau                  : TNumeroReseau;
                           const QSecteur                 : TNumeroSecteur;
                           const QSerieDepart             : TNumeroSerie;
                           const QPtDepart                : integer;
                           const QCode             : TNumeroCode;
                           const QExpe             : TNumeroExpe;
                           const QL, QAz, QP       : double;
                           const QIDStation        : string = '';
                           const QCommentaire      : string = ''): TViseeAntenne;
begin
  Result.IDViseeAntenne   := QIDViseeAntenne;
  Result.EntranceRatt     := QEntranceRatt;
  Result.Reseau           := QReseau;
  Result.Secteur          := QSecteur;
  Result.SerieDepart      := QSerieDepart;
  Result.PtDepart         := QPtDepart;
  Result.Code             := QCode;
  Result.Expe             := QExpe;

  Result.Longueur         := QL;
  Result.Azimut           := QAz;
  Result.Pente            := QP;
  result.IDTerrainStation  := QIDStation;
  Result.Commentaires      := QCommentaire;
end;

// dfinir l'quidistance d'une grille
function EnsureSetQuadrillageSpacing(const S: double): double; inline;
const
  EQUIDISTANCE_MINI = 10.00;
begin
  Result := IIF(S < EQUIDISTANCE_MINI, EQUIDISTANCE_MINI, S);
end;



// calcul de la constante de correction des mesures verticales en fonction de la longueur du cheminement
function CalcCorrectionAngulaire(const CC: TCode; const LongDev, DeltaZReference: double): double;
var
  dz: Double;
  WU: float;
begin
  if (IsZero(LongDev)) then Exit(0.00);
  dz := DeltaZReference + CC.ErreurTourillon;
  WU := arcsin(dz / LongDev);
  case Trunc(CC.GradInc) of
    360: Result := radtodeg(WU);
    400: Result := radtograd(WU);
  otherwise
    Result := 0.00; // en attente du support d'autres units
  end;
end;


function IsPointInRectangle(const P: TPoint2Df; const R: TRect2Df): boolean;
begin
  Result := IsInRange(P.X, R.X1, R.X2) and (IsInRange(P.Y, R.Y1, R.Y2));
end;

function IsSegmentInRectangle(const P1, P2: TPoint2Df; const R: TRect2Df; const PartiallyInRectangle: boolean = true): boolean;
begin
  if (PartiallyInRectangle) then Result := IsPointInRectangle(P1, R) or  IsPointInRectangle(P2, R)
                            else Result := IsPointInRectangle(P1, R) and IsPointInRectangle(P2, R);
end;
function IsQuadInRectangle(const P1, P2, P3, P4: TPoint2Df; const R: TRect2Df): boolean;
begin
  Result := IsPointInRectangle(P1, R) or IsPointInRectangle(P2, R) or IsPointInRectangle(P3, R) or IsPointInRectangle(P4, R);
end;

end.
