//{$INCLUDE DocDessinExporterScrapsToKMLorOSM.inc}

// Ce fichier est une dépendance de UnitDocDessin.pas

function TDocumentDessin.ExporterScrapsToKMLorOSM(const QFileName: string;
                                                  const QMode: TModeExportGIS;
                                                  const DoUseDefaultStyle: boolean;
                                                  const DefaultColor: TColor;
                                                  const DefaultOpacity: byte): boolean;
var
  fp: TextFile;
  CC: TConversionSysteme;
  i, NbGroupes, NbScraps: Integer;
  MyGrp: TGroupeEntites;
   // ouverture et fermeture des contextes
  procedure OpenOutputFile(); inline;
  begin
    AssignFile(fp, QFileName);
    ReWrite(fp);
  end;
  function FormatterVersionEncodageXML(const version: string; const encodage: string): string; inline;
  begin
    Result := Format('<?xml version="%s" encoding="%s"?>', [version, encodage]);
  end;
  // encodage du style
  procedure KML_WritePolygonStyle(const IDStyle: string;
                                  const LineColor, FillColor: TColor;
                                  const Opacity: byte;
                                  const LineWidth: double);
  begin
    Writeln(fp, Format('   <Style id="%s">', [IDStyle]));
    WriteLn(fp,        '     <LineStyle>');
    WriteLn(fp, Format('       %s', [KMLColor(GetRValue(LineColor), GetGValue(LineColor), GetBValue(LineColor), Opacity)]));
    WriteLn(fp, Format('       <width>%.1f</width>', [LineWidth]));
    WriteLn(fp,        '     </LineStyle>');
    WriteLn(fp,        '     <PolyStyle>');
    WriteLn(fp, Format('       %s', [KMLColor(GetRValue(FillColor), GetGValue(FillColor), GetBValue(FillColor), Opacity)]));
    WriteLn(fp,        '     </PolyStyle>');
    WriteLn(fp,        '   </Style>');
  end;
  procedure CloseOutputFile(); inline;
  begin
    try
      ;
    finally
      CloseFile(fp);
    end;
  end;
  procedure WriteHeader();
  var
    s: Integer;
    MyScrap: TScrap;
  begin
    //AfficherMessageErreur('WriteHeader');
    case QMode of
      gisGOOGLE_EARTH:
      begin
        WriteLn(fp, FormatterVersionEncodageXML('1.0', 'UTF-8'));
        WriteLn(fp, Format('<kml xmlns="%s" xmlns:gx="%s" xmlns:kml="%s" xmlns:atom="%s">',
                    [KML_OPENGIS_WEBSITE,
                     KML_GOOGLE_KML_WEBSITE,
                     KML_OPENGIS_WEBSITE,
                     W3C_W3_WEBSITE
                    ]));
        WriteLn(fp, '  <Document>');
        // styles de scraps
        KML_WritePolygonStyle('ScrapDefault', clRed, clRed, 255, 1.0);
        for s := 0 to GetNbScraps() - 1 do
        begin
          MyScrap := GetScrap(s);
          KML_WritePolygonStyle(Format('Scrap%d', [s]), MyScrap.Couleur, MyScrap.Couleur, MyScrap.Opacite, 1.0);
        end;
        // et on ajoute un Style de silhouette par défaut
        KML_WritePolygonStyle(SILHOUETTE_STYLE_SCRAP,
                              DefaultColor, DefaultColor,
                              DefaultOpacity, 1.0);

        // dossier principal = la cavité
        WriteLn(fp, '    <Folder>');
        WriteLn(fp, Format('    <name>%s</name>', ['MyCavite001']));
        WriteLn(fp, Format('    <open>%d</open>', [1]));
      end;
      gisOPEN_STREET_MAP:
      begin
        WriteLn(fp, FormatterVersionEncodageXML('1.0', 'UTF-8'));
        WriteLn(fp, Format('<osm version=''%s'' upload=''%s'' generator=''%s''>', ['0.6', 'true', 'GHCaveDraw']));
      end;
    end;
  end;

  procedure WriteFooter();
  begin
    case QMode of
      gisGOOGLE_EARTH:
      begin
        // dossier principal = la cavité
        WriteLn(fp, '    </Folder>');
        WriteLn(fp, '  </Document>');
        WriteLn(fp, '</kml>');
      end;
      gisOPEN_STREET_MAP:
      begin
        WriteLn(fp, ' </osm>');
      end;
    end;
  end;
  // données OSM
  procedure WriteOSMNodeRef(const Idx: Int64);
  begin
    WriteLn(fp, Format('      <nd ref="%d"/>', [-Idx]));
  end;
  // version courte de l'objet noeud
  procedure WriteOSMNode(const Idx: Int64; const Action: string; const Visible: boolean; const QOSMLon, QOSMLat: double);
  begin
    WriteLn(fp, Format('  <node id="%d" action="%s" visible="%s" lat="%s" lon="%s"/>',
                       [-Idx,       // ajout ds OSM ==> IDX négatifs
                        Action,
                        BoolToStr(Visible, 'true', 'false'),
                        ensureConvertLongOrLatToXML(QOSMLat),
                        ensureConvertLongOrLatToXML(QOSMLon)
                       ]));
  end;
  // version longue. Mettre les tags entre BeginOSMNode() et EndOSMNode()
  procedure BeginOSMNode(const Idx: Int64; const Action: string; const Visible: boolean; const QOSMLon, QOSMLat: double);
  begin
    WriteLn(fp, Format('  <node id="%d" action="%s" visible="%s" lat="%s" lon="%s">',
                       [-Idx,       // ajout ds OSM ==> IDX négatifs
                        Action,
                        BoolToStr(Visible, 'true', 'false'),
                        ensureConvertLongOrLatToXML(QOSMLat),
                        ensureConvertLongOrLatToXML(QOSMLon)
                       ]));
  end;
  // le paramètre Idx n'est utilisé que pour le programmeur
  // pour assurer la concordance des blocs
  procedure EndOSMNode(const Idx: Int64);
  begin
    WriteLn(fp, '  </node>');
  end;
  procedure WriteOSMTag(const K, V: string);
  begin
    WriteLn(fp, Format('   <tag k="%s" v="%s" />',[K, XML_real_escape_string(V)]));
  end;

  //****************************************************************************
  // dessiner le polygone de silhouette
  procedure DessinerScrap(const IdxScrap: int64;
                          const QScrap: TScrap;
                          const TagString: string;
                          const TagNum: Int64);
  const
    DELTA_Z_SURFACE = 10.00;
  var
    ii : integer;
    PP: TPoint2Df;
    P1, P2: TProjUV;
    Nb: LongInt;
    fred_mignon: String;  // le Fred Mignon est en string :lol:
    NomScrap: String;
    V: TVertexPolygon;
    BS: TBaseStation;
    QStart: Integer;
    QStartWay: Integer;
  begin
    Nb := 1 + High(QScrap.Sommets);
    AfficherMessageErreur(inttostr(Nb));
    // première passe: Contrôle des BP pour éviter des erreurs de formation du XML
    for ii := 0 to Nb - 1 do
    begin
      V := QScrap.Sommets[ii];
      if (Not GetBasePointByIndex(V.IDStation, BS)) then Exit;
    end;
    // construction du polygone
    case QMode of
       gisGOOGLE_EARTH:
       begin // KML
         NomScrap := IIF(DoUseDefaultStyle, SILHOUETTE_STYLE_SCRAP, mysqli_real_escape_string(TagString));
         WriteLn(fp, '      <Placemark>');
         WriteLn(fp, Format('       <!-- %s (%d sommets) -->',[NomScrap, Nb]));
         WriteLn(fp, Format('         <name>%s</name>', [XML_real_escape_string(TagString)]));
         WriteLn(fp, Format('         <styleUrl>#%s</styleUrl>', [NomScrap]));
         WriteLn(fp,        '         <Polygon>');
         WriteLn(fp, Format('           <tesselate>%d</tesselate>', [1]));
         WriteLn(fp, '           <outerBoundaryIs>');
         WriteLn(fp, '             <LinearRing>');
         WriteLn(fp, '               <coordinates>');
         for ii := 0 to Nb - 1 do
         begin
           V := QScrap.Sommets[ii];
           GetBasePointByIndex(V.IDStation, BS);
           PP := MakeTPoint2Df(BS.PosStation.X + V.Offset.X,
                                 BS.PosStation.Y + V.Offset.Y);
           P1.U := PP.X;
           P1.V := PP.Y;
           //AfficherMessageErreur(Format('EPSG:%d -> EPSG:%d', [FCodeEPSG, CODE_EPSG_WGS84]));
           P2 := CC.ConversionSyst1ToSyst2EPSG(FCodeEPSG, CODE_EPSG_WGS84, P1);
           WriteLn(fp, Format('             %s,%s,%s', [
                              ensureConvertLongOrLatToXML(P2.V),
                              ensureConvertLongOrLatToXML(P2.U),
                              ensureConvertAltitudeToXML(DELTA_Z_SURFACE)]));   //Lon, Lat


         end;
         WriteLn(fp, '               </coordinates>');
         WriteLn(fp, '             </LinearRing>');
         WriteLn(fp, '           </outerBoundaryIs>');
         WriteLn(fp, '         </Polygon>');
  	     WriteLn(fp, '      </Placemark>');
      end;
      gisOPEN_STREET_MAP:
        begin // OSM
          // première passe: Liste des sommets
          for ii := 0 to Nb - 1 do
          begin
            QStart := 10000 * (1 + TagNum);
            V := QScrap.Sommets[ii];
            GetBasePointByIndex(V.IDStation, BS);
            PP := MakeTPoint2Df(BS.PosStation.X + V.Offset.X,
                               BS.PosStation.Y + V.Offset.Y);
            P1.U := PP.X;
            P1.V := PP.Y;
            P2 := CC.ConversionSyst1ToSyst2EPSG(FCodeEPSG, CODE_EPSG_WGS84, P1);
            WriteOSMNode(QStart + (ii + 1), 'modify', True, P2.V, P2.U);        //QLon, QLat
          end;
          // seconde passe: Vertex
          QStartWay := (1 + TagNum) * MULTIPLICATEUR_SERIE; //100000;
          WriteLn(fp, Format('   <way id=''%d'' action=''%s'' visible=''%s''>', [-QStartWay, 'modify', 'true']));
          for ii := 0 to Nb - 1 do  WriteOSMNodeRef(QStart + (ii + 1));
          // cloture du polygone
          WriteOSMNodeRef(QStart + 1);
          WriteOSMTag('leisure', 'common');
          WriteLn(fp, '   </way>');
        end;
    end;
  end;
  // génération des scraps
  procedure WriteScrapsOfGroupe(const MyGroupe: TGroupeEntites);
  var
    s: Integer;
    MyScrap: TScrap;
  begin
    for s := 0 to NbScraps - 1 do
    begin
      MyScrap := GetScrap(s);
      if (MyGroupe.IDGroupeEntites = MyScrap.IDGroupe) then
        DessinerScrap(s, MyScrap, MyGroupe.NomGroupe, s);
    end;
  end;
begin
  Result := false;
  AfficherMessageErreur(Format('%s.ExporterScrapsToKMLorOSM: EPSG:%d; %s (mode %s)', [
                         ClassName,
                         FCodeEPSG,
                         QFileName,
                         ChooseString(Ord(QMode), ['Google Earth', 'OpenStreetMap'])
                         ]));
  NbScraps := self.GetNbScraps();
  if (NbScraps = 0) then Exit;
  NbGroupes := self.GetNbGroupes();
  CC := TConversionSysteme.Create;
  try
    if (CC.Initialiser) then
    begin
      OpenOutputFile();
      WriteHeader();
      for i := 0 to NbGroupes - 1 do
      begin
        MyGrp := self.GetGroupeByIdx(i);
        WriteScrapsOfGroupe(MyGrp);
        if (assigned(FProcAfficherProgression)) then
           FProcAfficherProgression(Format('Groupe %d / %d: %s', [i+1, NbGroupes, MyGrp.NomGroupe]), 0, NbGroupes - 1, i);
      end;
      WriteFooter();
      CC.Finaliser();
      CloseOutputFile();
      result := true;
    end;
  finally
    CC.Free;
  end;
end;

function TDocumentDessin.GetDossierContenantDoc(): string;
begin
  Result := FDossierDuDocument;
end;
//------------------------------------------------------------------------------
function TDocumentDessin.GenererCarteLeaflet(const QFileName: string;
                                             const MapWidth, MapHeight: integer;
                                             const DoUseDefaultStyle: boolean;
                                             const DefaultColor: TColor;
                                             const DefaultOpacity: byte): boolean;

var
  fp: TextFile;
  i, NbGroupes, NbScraps: Integer;
  CC: TConversionSysteme;
  Coin1, Coin2, Centroide: TPoint3Df;
  QCentroideXY, QCentroideLonLat: TProjUV;
  MyGrp: TGroupeEntites;
  procedure QWriteText(const s: string);
  begin
    Writeln(fp, S);
  end;
  procedure WriteHeader();
  const
    URL_LEAFLET_JS  = 'http://cdn.leafletjs.com/leaflet/v0.7.7/leaflet.js';
    URL_LEAFLET_CSS = 'http://cdn.leafletjs.com/leaflet/v0.7.7/leaflet.css';
    URL_OSM_TILES   = 'http://{s}.tile.osm.org/{z}/{x}/{y}.png';
    MAP_NAMEREF    =  'map';
  var
    URL_OSM_Contributors: String;
  begin
    QWriteText('<!DOCTYPE html>');
    QWriteText(Format('<!-- %s generated by %s %s - Date: %s -->', [GetDocumentName(),
                                                                            GetGHCaveDrawApplicationName(),
                                                                            GetGHCaveDrawVersionAndDateBuild(),
                                                                            DateTimeToStr(Now())
                                                                           ]));
    QWriteText('<HTML>');
    QWriteText('<HEAD>');
    QWriteText(Format('  <TITLE>%s</TITLE>', ['Report sur OpenStreetMap: ' + GetDocumentName()]));
    QWriteText(Format('  <META charset="%s" />', ['utf-8']));
    QWriteText(Format('  <META name="%s" content="width=%s, initial-scale=%.1f">', ['viewport', 'device-width', 1.0]));
  	//<meta name="viewport" content="width=device-width, initial-scale=1.0">
    QWriteText(Format('  <link rel="%s" href="%s" />', ['stylesheet', URL_LEAFLET_CSS]));
    QWriteText('</HEAD>');
    QWriteText('<BODY>');
    QWriteText(Format('  <div id="%s" style="width: %dpx; height: %dpx"></div>', [MAP_NAMEREF, MapWidth, MapHeight]));
    QWriteText(Format('  <script src="%s"></script>', [URL_LEAFLET_JS]));
    QWriteText('    <SCRIPT>');
    QWriteText(Format('     var map = L.map(''%s'').setView([%.10f, %.10f], %d);', [MAP_NAMEREF, QCentroideLonLat.U, QCentroideLonLat.V, 15])); // OK
    QWriteText(Format('     L.tileLayer(''%s'',', [URL_OSM_TILES]));  // OK
    QWriteText('       {');
      URL_OSM_Contributors := Format('<a href="%s">%s</a> %s', ['http://osm.org/copyright', 'OpenStreetMap', 'contributors']);
      QWriteText(Format('         attribution: ''&copy; %s''', [URL_OSM_Contributors]));
    QWriteText('       }');
    QWriteText(Format('     ).addTo(%s);', [MAP_NAMEREF]));

    QWriteText('    //**----- BEGIN DATA -----**');
  end;
  procedure WriteFooter();  // ok
  begin
    QWriteText('        //**----- END OF DATA -----**');
    QWriteText('        // Info-bulle indiquant les coordonnées du point clique');
    QWriteText('        var popup = L.popup();');
    QWriteText('        function onMapClick(e) {');
    QWriteText(Format('         popup.setLatLng(e.latlng).setContent("%s " + e.latlng.toString()).openOn(map);', ['']));
    QWriteText('        }');
    QWriteText('        map.on(''click'', onMapClick);');
    QWriteText('  </SCRIPT>');
    QWriteText('</BODY>');
    QWriteText('</HTML>');
  end;


  procedure OpenOutputFile(); inline;
  begin
    AssignFile(fp, QFileName);
    ReWrite(fp);
  end;
  procedure CloseOutputFile(); inline;
  begin
    try
      ;
    finally
      CloseFile(fp);
    end;
  end;
  procedure DessinerScrap(const IdxScrap: int64;
                          const QScrap: TScrap;
                          const TagString: string;
                          const TagNum: Int64);
  const
    DELTA_Z_SURFACE = 10.00;
  var
    ii : integer;
    PP: TPoint2Df;
    P1, P2: TProjUV;
    Nb: LongInt;
    fred_mignon: String;  // le Fred Mignon est en string :lol:
    NomScrap: String;
    V: TVertexPolygon;
    BS: TBaseStation;
    QStart: Integer;
    QStartWay: Integer;
    WU: String;
    EWE: string;
    QScrapColor: TColor;
    QScrapOpacity: double;
  begin
    WU := Format('Scrap%d', [IdxScrap]);
    // construction du polygone
    Nb := 1 + High(QScrap.Sommets);
    if (DoUseDefaultStyle) then
    begin
      QScrapColor      := DefaultColor;
      QScrapOpacity    := 1.0; //DefaultOpacity / 256;
    end
    else
    begin
      QScrapColor      := QScrap.Couleur;
      QScrapOpacity    := 1.0; //QScrap.Opacite / 256;
    end;
    QWriteText(Format('      // Scrap: %s', [WU]));
    QWriteText('      L.polygon([');
    for ii := 0 to Nb - 1 do
    begin
      V := QScrap.Sommets[ii];
      if (GetBasePointByIndex(V.IDStation, BS)) then
      begin
        PP := MakeTPoint2Df(BS.PosStation.X + V.Offset.X,
                           BS.PosStation.Y + V.Offset.Y);
        P1.U := PP.X;
        P1.V := PP.Y;
        if (ii < (Nb-1)) then EWE := ',' else EWE := '';
        //AfficherMessageErreur(Format('EPSG:%d -> EPSG:%d', [FCodeEPSG, CODE_EPSG_WGS84]));
        P2 := CC.ConversionSyst1ToSyst2EPSG(FCodeEPSG, CODE_EPSG_WGS84, P1);
        QWriteText(Format('             [%s, %s] %s // %s', [
                          ensureConvertLongOrLatToXML(P2.U),
                          ensureConvertLongOrLatToXML(P2.V),
                          EWE,
                          ensureConvertAltitudeToXML(DELTA_Z_SURFACE)]));   //Lon, Lat
      end;
    end;
    QWriteText('       ], {');
    QWriteText(Format('          color: ''%s'', ', [ColorToHTMLColor(QScrapColor)]));
    QWriteText(Format('          weight: %d, ', [1]));
    QWriteText(Format('          opacity: %.2f, ', [1.00]));
    QWriteText(Format('          fillcolor: ''%s'', ', [ColorToHTMLColor(QScrapColor)]));
    QWriteText(Format('          fillopacity: %.2f', [QScrapOpacity])); //[QScrap.Opacite / 256.0]));
    QWriteText('        }');
    // on utilise TagString, auquel le nom du groupe a été passé
    QWriteText(Format('      ).addTo(map).bindPopup("%s");', [TagString]));
  end;
  procedure WriteScrapsOfGroupe(const MyGroupe: TGroupeEntites);
  var
    s: Integer;
    MyScrap: TScrap;
  begin
    for s := 0 to NbScraps - 1 do
    begin
      MyScrap := GetScrap(s);
      if (MyGroupe.IDGroupeEntites = MyScrap.IDGroupe) then DessinerScrap(s, MyScrap, MyGroupe.NomGroupe, s);
    end;
  end;
begin
  Result := false;
  AfficherMessageErreur(Format('%s.GenererCarteLeaflet: EPSG:%d; %s', [
                         ClassName,
                         FCodeEPSG,
                         QFileName
                         ]));
  // calcul du centroide
  Coin1 := self.GetCoordsMini();
  Coin2 := self.GetCoordsMaxi();
  Centroide := MakeTPoint3Df(0.50 * (Coin1.X + Coin2.X),
                             0.50 * (Coin1.Y + Coin2.Y),
                             0.50 * (Coin1.Z + Coin2.Z)
                            );

  //--------------------
  NbScraps := self.GetNbScraps();
  if (NbScraps = 0) then Exit;
  NbGroupes := self.GetNbGroupes();
  CC := TConversionSysteme.Create;
  try
    CC.Initialiser();
    // conversion du centroide
    QCentroideXY.U := Centroide.X;
    QCentroideXY.V := Centroide.Y;
    QCentroideLonLat := CC.ConversionSyst1ToSyst2EPSG(FCodeEPSG, CODE_EPSG_WGS84, QCentroideXY);
    OpenOutputFile();
    WriteHeader();
    QWriteText('   // ************************************************');
    QWriteText('   // SCRAPS SECTION');
    for i := 0 to NbGroupes - 1 do
    begin
      MyGrp := self.GetGroupeByIdx(i);
      WriteScrapsOfGroupe(MyGrp);
      if (assigned(FProcAfficherProgression)) then
           FProcAfficherProgression(Format('Groupe %d / %d: %s', [i+1, NbGroupes, MyGrp.NomGroupe]),
                                    0, NbGroupes - 1, i);

    end;
    QWriteText('   // END SCRAPS SECTION');
    QWriteText('   // ************************************************');

    WriteFooter();
    CC.Finaliser();
    CloseOutputFile();
    result := true;
  finally
    CC.Free;
  end;
end;


