unit CadreCalculette;
// Calculette et convertisseur de coordonnées
// 07/01/2014: Désactivation provisoire du support des déclinaisons magnétiques
// 06/02/2014: Remplacement de l'évaluateur d'expressions
// 10/09/2014: Importantes améliorations d'ergonomie.
// 10/09/2014: L'évaluateur d'expressions devient un objet
// 09/10/2014: Nombreux bugs fixés
//             Les nombres formatés sont automatiquement convertis
// 15/01/2015: Nouvel onglet Interpréteur Pascal *** SUPPRIME **
// 05/02/2015: Adaptation de l'interface pour les tablettes
// 13/12/2015: Support export KML et GPX du tableau des données converties
// 06/06/2016: Tableau: Support des conversions depuis le datum spécifié dans la ligne courante
{$INCLUDE CompilationParameters.inc}


interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Common,
  Graphics,
  StructuresDonnees,
  unitUtilsComposants,
  ConvertisseurJPC, // API JPC
  unitExportGIS,
  // calculateur de déclinaison
  UnitTSAGeoMag,
  math,
  {$IFDEF MSWINDOWS}
  //  UnitWrapperDeclimag,        // pour la déclinaison magnétique
  {$ENDIF}
  {$IFDEF LINUX}
  //  UnitWrapperDeclimagLinux,
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
  //  UnitWrapperDeclimagMacOSX,
  {$ENDIF}
  {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
  ToporobotClasses2012,
  {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}

  CallDialogsStdVersion,
  SVGCanvasUnit,
  evaluateur_expressions,
  Classes, SysUtils, FileUtil, curredit, SynEdit, SynHighlighterPas,
  //SynHighlighterPas,
  //uPSComponent,
  //uPSComponent_Default,
  //uPSComponent_Forms,
  //uPSComponent_Controls,
  //uPSComponent_StdCtrls,
  ubarcodes, Forms, Dialogs,
  Controls, ComCtrls, StdCtrls, Grids, EditBtn, ExtCtrls, PairSplitter, Menus,
  LCLType, Types, Clipbrd,
  ActnList, Buttons,
  ValEdit,
  PdfDoc,
  PdfFonts,
  PdfTypes, PReport,
  GenerationFichesPointsTopo, UnitObjetSerie,
  uPSComponent,
  uPSCompiler,
  uPSRuntime,
  uPSComponent_Default,
  uPSUtils;

// pour l'export KML
type TLigneCoordonnees = record
  EtiquetteTitle      : string;
  EtiquetteDescription: string;
  XSource        : double;
  YSource        : double;
  ZSource        : double;
  XCible         : double;
  YCible         : double;
end;



type

  { TCdrCalculette }

  TCdrCalculette = class(TFrame)
    acRemoveLigne: TAction;
    acRemoveColonne: TAction;
    acQRCodeCoordPoint: TAction;
    acReverseSystCoordonnees: TAction;
    acCalculer: TAction;
    acEchangerValeurs: TAction;
    acConvertir: TAction;
    acCopierTableau: TAction;
    acColler: TAction;
    acOuvrirCSV: TAction;
    acConvertirTableau: TAction;
    acCalculDeclimag: TAction;
    acCopyTableauDeclimag: TAction;
    acCalculDeclimagTableau: TAction;
    acExportCSV: TAction;
    acExportListePointsKML: TAction;
    acQRCodeListeCoords: TAction;
    acCopyCoordinates: TAction;
    acGenerateQRCodeFromTexte: TAction;
    acExportQRCodeToSVG: TAction;
    acExportOSMLeaflet: TAction;
    acPSExecute: TAction;
    acPSStop: TAction;
    acLoadPSScript: TAction;
    acSavePSScript: TAction;
    ActionList1: TActionList;
    btnConvertDMS2DegDec: TButton;
    btnOuvrirTXT: TButton;
    btnCopierTableau: TButton;
    btnSeparateurs: TButton;
    btnReparser: TButton;
    chkUseDatumOfColumn: TCheckBox;
    chkSepVirgule: TCheckBox;
    chkSepPeriod: TCheckBox;
    chkSepSemicolon: TCheckBox;
    chkSepOther: TCheckBox;
    chkRetirerGuillemets: TCheckBox;
    chkSepTab: TCheckBox;
    chkSepEspace: TCheckBox;
    cmbColDatum: TComboBox;
    cmbColLabelsDescros: TComboBox;
    cmbColXCible: TComboBox;
    cmbColYCible: TComboBox;
    cmbColYSource: TComboBox;
    cmbColZSource: TComboBox;
    editLatitudeDMS: TEdit;
    editLongitudeDMS: TEdit;
    editPascalScript: TSynEdit;
    editPersonalSeparator: TEdit;
    editTextToEncodeQR: TSynEdit;
    ImageList1: TImageList;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label3: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lbListeVariables: TStaticText;
    lbStatutConvertisseur: TStaticText;
    lsbPSFunctions: TListBox;
    lsbPSMessages: TListBox;
    lsbVariables: TListBox;
    memoHelp: TMemo;
    PairSplitter2: TPairSplitter;
    PairSplitter3: TPairSplitter;
    PairSplitter4: TPairSplitter;
    PairSplitter5: TPairSplitter;
    PairSplitter6: TPairSplitter;
    PairSplitter7: TPairSplitter;
    PairSplitterSide10: TPairSplitterSide;
    PairSplitterSide11: TPairSplitterSide;
    PairSplitterSide12: TPairSplitterSide;
    PairSplitterSide13: TPairSplitterSide;
    PairSplitterSide14: TPairSplitterSide;
    PairSplitterSide3: TPairSplitterSide;
    PairSplitterSide4: TPairSplitterSide;
    PairSplitterSide5: TPairSplitterSide;
    PairSplitterSide6: TPairSplitterSide;
    PairSplitterSide7: TPairSplitterSide;
    PairSplitterSide8: TPairSplitterSide;
    PairSplitterSide9: TPairSplitterSide;
    Panel4: TPanel;
    Panel5: TPanel;
    pnlListeVariables: TPanel;
    pnlSeparateurs: TPanel;
    PSCustomPlugin1: TPSCustomPlugin;
    PSImport_Classes1: TPSImport_Classes;
    PSImport_DateUtils1: TPSImport_DateUtils;
    PSScriptDebugger1: TPSScriptDebugger;
    ScrollBox2: TScrollBox;
    SpeedButton1: TSpeedButton;
    SpeedButton10: TSpeedButton;
    SpeedButton11: TSpeedButton;
    SpeedButton12: TSpeedButton;
    SpeedButton13: TSpeedButton;
    SpeedButton14: TSpeedButton;
    SpeedButton15: TSpeedButton;
    SpeedButton16: TSpeedButton;
    SpeedButton17: TSpeedButton;
    SpeedButton18: TSpeedButton;
    SpeedButton19: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton20: TSpeedButton;
    SpeedButton21: TSpeedButton;
    SpeedButton22: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SpeedButton6: TSpeedButton;
    SpeedButton7: TSpeedButton;
    SpeedButton8: TSpeedButton;
    grdDataSeparetedBySpaces: TStringGrid;
    SpeedButton9: TSpeedButton;
    editPSOutPut: TSynEdit;
    lbCopyrightNotesConvertisseur: TStaticText;
    SynPasSyn1: TSynPasSyn;
    tabShtPascalScript: TTabSheet;
    tabShtImportFichierTexte: TTabSheet;

    VueQRCode: TBarcodeQR;
    cmbSeparateur: TComboBox;
    Label1: TLabel;
    lbListeDesFonctions: TStaticText;
    lbListeCalculs: TStaticText;
    lbQRCodeGenerator: TStaticText;
    lsbFonctions: TListBox;
    lsbLastCalculs: TListBox;
    chkAutoIncrement: TCheckBox;
    chkPremLigneIsTitres: TCheckBox;
    cmbColLabelsTitres: TComboBox;
    cmbColXSource: TComboBox;
    cmbSystCibleJPC: TComboBox;
    cmbSystSourceJPC: TComboBox;
    editAltitude: TCurrencyEdit;
    editAnneeArrivee: TCurrencyEdit;
    editAnneeDepart: TCurrencyEdit;
    editDateDeclimag: TDateEdit;
    editDeclinaison: TCurrencyEdit;
    editExpression: TEdit;
    editLatitude: TCurrencyEdit;
    editLongitude: TCurrencyEdit;
    editX_Cible1JPC: TCurrencyEdit;
    editX_Source1JPC: TCurrencyEdit;
    editY_Cible1JPC: TCurrencyEdit;
    editY_Source1JPC: TCurrencyEdit;
    grdDeclinaisons: TStringGrid;
    grdDonnees: TStringGrid;
    lbPromptExpression: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    lbConversionsTableau: TStaticText;
    lbSystCible: TLabel;
    lbSystemesCoordonnees: TStaticText;
    lbDeclimagUnitaire: TStaticText;
    lbDeclimagTableau: TStaticText;
    lbHintGrdConversions: TStaticText;
    lbExpression: TStaticText;
    lbSystSource: TLabel;
    lbValeursCalculees: TLabel;
    lbValeursEntree: TLabel;
    lbValeursX: TLabel;
    lbValeursY: TLabel;
    Long: TLabel;
    Long1: TLabel;
    Long2: TLabel;
    Long3: TLabel;
    Long4: TLabel;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    PageControl1: TPageControl;
    PairSplitter1: TPairSplitter;
    PairSplitterSide1: TPairSplitterSide;
    PairSplitterSide2: TPairSplitterSide;
    Panel1: TPanel;
    lbConversionsUnitaires: TStaticText;
    Panel10: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    pnlQRCode: TPanel;
    Panel9: TPanel;
    pnlTableauDeclimag: TPanel;
    Panel7: TPanel;
    Panel8: TPanel;
    lbDescDeclimagCalculator: TStaticText;
    lbErreurCalcul: TStaticText;
    PopupMenu1: TPopupMenu;
    tabShtGenerateurQRCode: TTabSheet;
    tabShtCalculatrice: TTabSheet;
    tabShtConvertisseur: TTabSheet;
    tabShtDeclimag: TTabSheet;
    procedure acCalculDeclimagExecute(Sender: TObject);
    procedure acCalculDeclimagTableauExecute(Sender: TObject);
    procedure acCalculerExecute(Sender: TObject);
    procedure acCollerExecute(Sender: TObject);
    procedure acConvertirExecute(Sender: TObject);
    procedure acConvertirTableauExecute(Sender: TObject);
    procedure acCopierTableauExecute(Sender: TObject);
    procedure acCopyCoordinatesExecute(Sender: TObject);
    procedure acCopyTableauDeclimagExecute(Sender: TObject);
    procedure acEchangerValeursExecute(Sender: TObject);
    procedure acExportCSVExecute(Sender: TObject);
    procedure acExportListePointsKMLExecute(Sender: TObject);
    procedure acExportQRCodeToSVGExecute(Sender: TObject);
    procedure acGenerateQRCodeFromTexteExecute(Sender: TObject);
    procedure acLoadPSScriptExecute(Sender: TObject);
    procedure acOuvrirCSVExecute(Sender: TObject);
    procedure acPSExecuteExecute(Sender: TObject);
    procedure acPSStopExecute(Sender: TObject);
    procedure acQRCodeCoordPointExecute(Sender: TObject);
    procedure acQRCodeListeCoordsExecute(Sender: TObject);
    procedure acRemoveColonneExecute(Sender: TObject);
    procedure acRemoveLigneExecute(Sender: TObject);
    procedure acReverseSystCoordonneesExecute(Sender: TObject);
    procedure acSavePSScriptExecute(Sender: TObject);
    procedure btnCopierTableauClick(Sender: TObject);
    procedure btnSeparateursClick(Sender: TObject);
    procedure btnOuvrirTXTClick(Sender: TObject);
    procedure btnReparserClick(Sender: TObject);
    procedure btnConvertDMS2DegDecClick(Sender: TObject);
    procedure cmbColLabelsDescrosChange(Sender: TObject);
    procedure editLatitudeDMSChange(Sender: TObject);
    procedure editExpressionEnter(Sender: TObject);
    procedure editLatitudeDMSEnter(Sender: TObject);
    procedure editLatitudeDMSExit(Sender: TObject);
    procedure editLongitudeDMSEnter(Sender: TObject);
    procedure editLongitudeDMSExit(Sender: TObject);
    procedure editPascalScriptChange(Sender: TObject);

    procedure editTextToEncodeQRKeyPress(Sender: TObject; var Key: char);
    procedure FrameClick(Sender: TObject);
    procedure FrameResize(Sender: TObject);
    procedure lsbFonctionsClick(Sender: TObject);
    procedure lsbFonctionsDblClick(Sender: TObject);

    procedure Button2Click(Sender: TObject);
    procedure cmbColLabelsTitresChange(Sender: TObject);
    procedure cmbColXCibleChange(Sender: TObject);
    procedure cmbColXSourceChange(Sender: TObject);
    procedure cmbColYCibleChange(Sender: TObject);
    procedure cmbColYSourceChange(Sender: TObject);
    procedure cmbSystSourceJPCChange(Sender: TObject);
    procedure editExpressionKeyPress(Sender: TObject; var Key: char);
    procedure lsbLastCalculsDblClick(Sender: TObject);

    procedure lsbPSFunctionsSelectionChange(Sender: TObject; User: boolean);
    procedure PageControl1Change(Sender: TObject);
    procedure PSScriptDebugger1AfterExecute(Sender: TPSScript);
    procedure PSScriptDebugger1Compile(Sender: TPSScriptDebugger);
    procedure PSScriptDebugger1Execute(Sender: TPSScript);
    procedure PSScriptDebugger1Idle(Sender: TObject);
    procedure tabShtCalculatriceContextPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);

  private
    // Pour les fonctions fopen et fclose
    FpTextFiles: Array[0..9] of TextFile;
    // Pour traitement de fichiers textes séparés par espaces
    // Mettre cet objet en pseudo-global permet d'y faire des opérations
    // sans avoir à recharger le fichier
    {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
    FDocuTopo: TToporobotStructure2012;
    {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}
    FContenuFichierTexte  : TStringList;
    FEvaluateurExpressions: TEvaluateurExpressions;
    FConversionUtils     : TConversionSysteme;
    FCalculateurDeclimag : TSAGeoMag;
    FConvertisseurActif       : boolean;
    FCalculateurDeclimagActif : boolean;
    FColonneLabelsTitres  : integer;
    FColonneLabelsDescros : integer;

    FColonneDatum  : integer;

    FColonneXSource: integer;
    FColonneYSource: integer;
    FColonneZSource: integer;

    FColonneXCible : integer;
    FColonneYCible : integer;

    procedure AjouterDerniereExpressionValable(const Expr: string);
    function CheckLatLonInMetropole(const QLat, QLon: double): boolean;
    procedure ClearComboColonnes(const cmb: TComboBox; const Idx: integer);
    procedure DisplayHelpForProc(const MyProc: TPSRegProc);

    procedure ExporterUnPointKML(const FileName: TStringDirectoryFilename; const PlaceMark: TKMLPlaceMark; const M: TOutputFormatGIS);
    procedure ExportListePointsVersCarto(const FileName: string;
                                         const MC: TOutputFormatGIS;
                                         const FirstLineIsTitreCol: Boolean;
                                         const LabelsAutoIncrement: Boolean;
                                         const DoUseColDatum: boolean;
                                         const SystSource, SystCible: integer;
                                         const NoColLabelTitre: integer;
                                         const NoColLabelDescro: integer;
                                         const NoColXSource, NoColYSource, NoColZSource: integer);

    //*)
    procedure GenererPagesPointsTopo(const QFileName: string; const QNb: integer);
    function GetSeparateursFromPnl(): TSysCharSet;
    function InitialiseEvaluateurExpressions(): boolean;
    function InitialiseCalculateurDeclimag(): boolean;
    function InitialiseConvertisseur(): boolean;
    procedure ParserContenuTexte();
    procedure RefactorerTitresDesColonnes();
    procedure RemplirComboColonnes(const Cmb: TComboBox; const Idx: integer; const QLibColonne: string);
    procedure SetLabelsValeursDemandees(const Idx: integer);

    procedure FaireUnCalcul(const Expr: string);
    procedure ListerLesVariables();
    procedure DispPSCompilerMsg(const Msg: string);
    procedure DispPSOutput(const Msg: string);
    procedure printf(const FMT: string; const Argv: array of const);

    procedure ListerPSFunctions();

    function  fopen(const Descr: integer; const QFileName: string; const Mode: Char): boolean;
    function  feof(const Descr: integer): boolean;
    procedure fprint(const Descr: integer; const QLine: string);
    procedure fprintf(const Descr: integer; const FMT: string; const Argv: array of const);
    function  freadln(const Descr: integer): string;
    procedure fclose(const Descr: integer);
    procedure fcloseall();
    {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
    function  DT_GetNbreStationsOfSerieByIdx(const Idx: integer): integer;
    function  DT_GetNbreStationsOfSerieByNoSerie(const NoSerie: integer): integer;


    {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}
    { private declarations }
  public
    { public declarations }
    {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
    function Initialiser(const T: integer; const FD: TToporobotStructure2012): boolean;
    {$ELSE}
    function Initialiser(const T: integer): boolean;
    {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}
    procedure Finaliser();
    procedure SetTabIndex(const T: integer);

    procedure SetCoordonneesA(const P: TPoint2Df);
    procedure SetDefaultExpressionA(const S: string);
    function GetCoordonneesA(): TPoint2Df;
    function GetDeclimagA(): double;
    function GetResultatCalculA(): double;
    function GetCurrentCodeEPSGNomSysteme(): TLabelSystemesCoordsEPSG;
  end;

var
  CdrCalculette: TCdrCalculette;
implementation
{$R *.lfm}
uses
 uPSC_graphics,
 uPSC_controls,
 uPSC_classes,
 uPSR_graphics,
 uPSR_controls,
 uPSR_classes,
 psSyntaxDoc;



const
  TAB_INDEX_QRCODE = 3;

function psCompilerBeforeOutput(Sender: TPSPascalCompiler): Boolean;
  var
    s:string;
  begin
    Result:=True;
    if MakePSSyntaxDoc(Sender, s, sdmXml) then
      with TStringList.create do
      begin
        Text := s;
        SaveToFile(GetGHTopoDirectory() + 'PSMySyntax.xml');
        Free;
      end;
  end;

// wrappers pour fonctions standard
// f en préfixe des fonctions, convention C aka fabs(x)

function fabs(const X: Extended): Extended; inline;
begin
  result := abs(X);
end;

function fexp(X: Extended): extended; inline;
begin
  result := exp(X);
end;
function farctan(X: Extended): extended; inline;
begin
  result := arctan(X);
end;
function farctan2(Y, X: Extended): extended; inline;
begin
  result := arctan2(Y, X);
end;
function fln(X: Extended): extended; inline;
begin
  result := ln(X);
end;
function flog(X: Extended): extended; inline;
begin
  result := log10(X);
end;
// wrappers pour fonctions utilitaires
// (doivent figurer dans TCdrCalculette en raison de la liste des descripteurs de fichiers
procedure TCdrCalculette.fprint(const Descr: integer; const QLine: string);
begin
  WriteLn(FpTextFiles[Descr], QLine);
end;

procedure TCdrCalculette.fprintf(const Descr: integer; const FMT: string; const Argv: array of const);
var
  WU: String;
begin
  WU := Format(FMT, Argv);
  WriteLn(FpTextFiles[Descr], WU);
end;

function TCdrCalculette.feof(const Descr: integer): boolean;
begin
  Result := EOF(FpTextFiles[Descr]);
end;

function TCdrCalculette.freadln(const Descr: integer): string;
var
  QLine: string;
begin
  ReadLn(FpTextFiles[Descr], QLine);
  result := QLine;
end;

procedure TCdrCalculette.fclose(const Descr: integer);
begin
  try
    CloseFile(FpTextFiles[Descr]);
  finally
  end;
end;

procedure TCdrCalculette.fcloseall();
var
  i: Integer;
begin
  for i := 0 to high(FpTextFiles) do self.fclose(i);
end;



procedure TCdrCalculette.ListerPSFunctions();
var
  n, p: LongInt;
  i, j: Integer;
  WU: TPSRegProc;
  EWE: TPSParameterDecl;
begin
  lsbPSFunctions.Clear;
  n := PSScriptDebugger1.Comp.GetRegProcCount;
  if (n = 0) then exit;
  for i := 0 to n - 1 do
  begin
    WU := PSScriptDebugger1.Comp.GetRegProc(i);
    lsbPSFunctions.Items.Add(Format('%s', [LowerCase(WU.Name)]));
    (*
    p := WU.Decl.ParamCount;
    if (p > 0) then
    begin
      try
        for j := 0 to p - 1 do
        begin
          EWE := WU.Decl.Params[j];
          lsbPSFunctions.Items.Add(Format('--> %s: %s',[LowerCase(EWE.Name), LowerCase(EWE.aType.Name)]));
        end;
      except

      end;
    end;
    //*)
    ///*)
  end;
  //lsbPSFunctions.ItemIndex:=0;
end;

//******************************************************************************
{$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}

function TCdrCalculette.DT_GetNbreStationsOfSerieByIdx(const Idx: integer): integer;
var
  SR: TObjSerie;
begin
  SR := FDocuTopo.GetSerie(Idx);
  Result := SR.GetNbVisees();
end;

function TCdrCalculette.DT_GetNbreStationsOfSerieByNoSerie(const NoSerie: integer): integer;
var
  SR: TObjSerie;
  QIdx: integer;
begin
  if (not FDocuTopo.GetSerieByNumeroSerie(NoSerie, SR, QIdx)) then exit(-1);
  Result := SR.GetNbVisees();
end;

{$endif CALCULETTE_EMBEDDED_IN_GHTOPO}

//------------------------------------------------------------------------------


//*)
function TCdrCalculette.GetSeparateursFromPnl(): TSysCharSet;
var
  WU: String;
  PersoSep: Char;
begin
  Result := [];
  WU := Trim(editPersonalSeparator.Text) + ' ';
  PersoSep := WU[1];

  if (chkSepTab.Checked)       then Result := Result + [#9]    else Result := Result - [#9];
  if (chkSepVirgule.Checked)   then Result := Result + [',']   else Result := Result - [','];
  if (chkSepPeriod.Checked)    then Result := Result + [';']   else Result := Result - [';'];
  if (chkSepSemicolon.Checked) then Result := Result + [':']   else Result := Result - [':'];
  if (chkSepOther.Checked)     then Result := Result + [PersoSep]
                               else Result := Result - [PersoSep];
  // le séparateur espace doit être armé en dernier
  if (chkSepEspace.Checked)    then Result := Result + [' ']  else Result := Result - [' '];
end;

{$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
function TCdrCalculette.Initialiser(const T: integer; const FD: TToporobotStructure2012): boolean;
{$else}
function TCdrCalculette.Initialiser(const T: integer): boolean;
{$endif CALCULETTE_EMBEDDED_IN_GHTOPO}

var
  i: Integer;
  WU: Boolean;
  procedure S777(const AC: TAction; const CaptionHint: string);
  begin
    AC.Caption := '';
    AC.Hint    := GetResourceString(CaptionHint);
  end;
begin
  Result := False;
  try
    {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
    FDocuTopo := FD;
    {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}
    pnlSeparateurs.Visible := false;
    FContenuFichierTexte        := TStringList.Create;
    FContenuFichierTexte.Clear;
    // onglet évaluateur

    lbListeDesFonctions.Caption := GetResourceString(rsDLG_CALC_LSB_FUNCTIONS);
    lbListeCalculs.Caption      := GetResourceString(rsDLG_CALC_LSB_CALCULS);

    lbExpression.Caption        := GetResourceString(rsDLG_CALC_EXPRESSION);
    lbPromptExpression.Caption  := GetResourceString(rsDLG_CALC_EXPR);
    tabShtCalculatrice.Caption  := GetResourceString(rsDLG_CALC_TAB_CAL);
    tabShtCalculatrice.Enabled  := InitialiseEvaluateurExpressions();
    lbListeVariables.Caption    := GetResourceString(rsDLG_CALC_LSB_VARIABLES);
    PageControl1.ActivePageIndex := 0;
    // liste des variables (si utilisation de fpexprpars: activé, sinon désactivé et invisible)
    //PairSplitterSide5.Height  := PairSplitter3.Height - 60;
    //pnlListeVariables.Visible := false;
    //pnlListeVariables.Enabled := false;
    ListerLesVariables();
    // onglet convertisseur de coordonnées
    lbCopyrightNotesConvertisseur.Caption := GetResourceString(rsCOORDS_CONVERTER_AUTHOR);
    lbSystemesCoordonnees.Caption   := GetResourceString(rsDLG_CALC_SYSCOORDS);
    lbConversionsTableau.Caption    := GetResourceString(rsDLG_CALC_COORDS_TABLEAU);
    lbConversionsUnitaires.Caption  := GetResourceString(rsDLG_CALC_COORDS_UNITAIRE);
    lbSystSource.Caption            := GetResourceString(rsDLG_CALC_LB_SYST_SOURCE);
    lbSystCible.Caption             := GetResourceString(rsDLG_CALC_LB_SYST_CIBLE);
    tabShtConvertisseur.Caption     := GetResourceString(rsDLG_CALC_CDR_CONVERT);
    tabShtPascalScript.Caption      := GetResourceString(rsDLG_CALC_TAB_PASCAL_SCRIPT);
    S777(acConvertir               , rsDLG_CALC_BTN_CONVERT);
    S777(acEchangerValeurs         , rsDLG_CALC_BTN_ECHANGE);
    S777(acReverseSystCoordonnees  , rsDLG_CALC_BTN_ECHANGE);
    S777(acQRCodeListeCoords       , rsDLG_CALC_TAB_COORDS_QRCODE_LISTE);
    S777(acConvertir               , rsDLG_CALC_BTN_CONVERT);
    S777(acOuvrirCSV               , rsDLG_CALC_BTN_OPEN_CSV);
    S777(acExportCSV               , rsDLG_CALC_BTN_EXPORT_CSV);
    S777(acExportListePointsKML    , rsDLG_CALC_BTN_EXPORT_GIS);
    S777(acQRCodeCoordPoint        , rsDLG_CALC_BTN_QRCODE_PT);
    S777(acQRCodeListeCoords       , rsDLG_CALC_BTN_QRCODE_SEL);
    S777(acColler                  , rsDLG_CALC_BTN_PASTE);
    S777(acCopierTableau           , rsBTN_COPIER_TABLEAU);
    S777(acConvertirTableau        , rsDLG_CALC_BTN_CONVERT_TABLEAU);
    S777(acCopyCoordinates         , rsDLG_CALC_BTN_COPY_COORDS_ISOLEES);
    S777(acGenerateQRCodeFromTexte , rsDLG_CALC_BTN_QRCODE_FROM_TEXT);
    S777(acExportQRCodeToSVG       , rsDLG_CALC_BTN_QRCODE_TO_SVG);
    S777(acPSExecute               , rsDLG_CALC_BTN_PASCAL_SCRIPT_DO_RUN);

    WU := InitialiseConvertisseur();
    tabShtConvertisseur.Enabled := WU;
    if (WU) then lbStatutConvertisseur.Color := clGreen;

    RemplirCombosSystemesCoordonnees(FConversionUtils, cmbSystSourceJPC, False, 3);
    RemplirCombosSystemesCoordonnees(FConversionUtils, cmbSystCibleJPC, False, 0);
    SetLabelsValeursDemandees(0);
    // onglet déclinaison magnétique
    lbDeclimagUnitaire.Caption      := GetResourceString(rsDLG_CALC_DECL_UNITAIRE);
    lbDeclimagTableau.Caption       := GetResourceString(rsDLG_CALC_DECL_TABLEAU);
    tabShtDeclimag.Caption          := GetResourceString(rsDLG_CALC_CDR_DECLIMAG);
    FCalculateurDeclimagActif       := InitialiseCalculateurDeclimag();
    tabShtDeclimag.Visible          := FCalculateurDeclimagActif;
    tabShtDeclimag.Enabled          := FCalculateurDeclimagActif;
    if (FCalculateurDeclimagActif) then
    begin
      lbDescDeclimagCalculator.Caption := FCalculateurDeclimag.GetDescriptionDeclimag();
      lbDescDeclimagCalculator.Color   := clGreen;
    end;
    // onglet QRCode
    tabShtGenerateurQRCode.Caption  := GetResourceString(rsDLG_CALC_TAB_QRCODE);
    tabShtGenerateurQRCode.Visible  := True;
    // onglet Import de fichiers texte
    tabShtImportFichierTexte.Caption := GetResourceString(rsDLG_CALC_TAB_TEXTFILES);
    tabShtImportFichierTexte.Visible := true;

    //----------------------


    editDateDeclimag.Date := Now();

    // sous rubriques

    // grille de conversion en rafale
    // la grille est éditable et on peut l'utiliser pour des conversions
    grdDonnees.Options := grdDonnees.Options + [goColSizing, goEditing];
    grdDonnees.ColCount :=  MAX_SIZE_PARAM_ARRAY;
    grdDonnees.RowCount := 20;
    // proposition d'un entete par défaut
    for i := 0 to grdDonnees.ColCount - 1 do grdDonnees.Cells[i, 0] := Format('Col %d', [i]);
    grdDonnees.Cells[0, 0] := '';
    grdDonnees.Cells[1, 0] := 'Etiquette';
    grdDonnees.Cells[2, 0] := 'Datum';
    grdDonnees.Cells[3, 0] := 'X source';
    grdDonnees.Cells[4, 0] := 'Y source';
    grdDonnees.Cells[5, 0] := 'Altitude';
    grdDonnees.Cells[6, 0] := 'Latitude';
    grdDonnees.Cells[7, 0] := 'Longitude';
    for i := 0 to grdDonnees.ColCount - 1 do grdDonnees.ColWidths[i] := 120;
    for i := 1 to grdDonnees.RowCount - 1 do grdDonnees.Cells[0,i] := IntToStr(i);

    FColonneLabelsTitres    := 0;
    FColonneLabelsDescros   := 0;

    FColonneDatum      := 1;
    FColonneXSource    := 2;
    FColonneYSource    := 3;
    FColonneZSource    := 4;
    FColonneXCible     := 5;
    FColonneYCible     := 6;
    FColonneLabelsDescros := 7;
    RemplirComboColonnes(cmbColLabelsTitres ,  FColonneLabelsTitres , 'EtiquetteTitre');
    RemplirComboColonnes(cmbColLabelsDescros , FColonneLabelsDescros , 'EtiquetteDescro');

    RemplirComboColonnes(cmbColDatum  , FColonneDatum  , 'Datum');
    RemplirComboColonnes(cmbColXSource, FColonneXSource, 'X source');
    RemplirComboColonnes(cmbColYSource, FColonneYSource, 'Y source');
    RemplirComboColonnes(cmbColZSource, FColonneZSource, 'Z source');
    RemplirComboColonnes(cmbColXCible , FColonneXCible , 'Lat');
    RemplirComboColonnes(cmbColYCible , FColonneYCible , 'Lon');

    // aide pour la grille de conversion
    lbHintGrdConversions.Caption := _AnsiToLCLStr(rsDLG_CALC_HINT_GRD_CONVERSIONS);

    // interpréteur PascalScript
    //tabShtPascalScript.Caption      := GetResourceString(rsDLG_CALC_TAB_PASCAL_SCRIPT);
    //for i := 0 to High(FpTextFiles) do FpTextFiles[i] := Text(0);

    PageControl1.ActivePageIndex := T;
    if (T = 0) then editExpression.SetFocus;
    result := True;
  except

  end;
end;



procedure TCdrCalculette.ListerLesVariables();
var
  n, i: Integer;
  V: TVariable;
begin
  AfficherMessage(format('%s.ListerLesVariables', [ClassName]));

  lsbVariables.Items.Clear;
  n := FEvaluateurExpressions.GetNbVariables();
  AfficherMessage(Format('%d variables', [n]));
  if (n = 0) then Exit;
  for i := 0 to n - 1 do
  begin
    V := FEvaluateurExpressions.GetVariable(i);
    lsbVariables.Items.add(Format('$%s = %s', [V.Nom, FloatToStr(V.Valeur)]));

  end;
  lsbVariables.ItemIndex := 0;
end;

procedure TCdrCalculette.DispPSCompilerMsg(const Msg: string);
begin
  lsbPSMessages.Items.Add(msg);
  lsbPSMessages.ItemIndex := lsbPSMessages.Items.Count - 1;
end;

procedure TCdrCalculette.SetTabIndex(const T: integer);
begin
  PageControl1.ActivePageIndex := T;
end;

procedure TCdrCalculette.Finaliser();
begin
  try
    FEvaluateurExpressions.Finaliser;
    FConversionUtils.Finaliser;
    FCalculateurDeclimag.Finaliser;
    FContenuFichierTexte.Clear;
  finally
    FEvaluateurExpressions.Free;
    FConversionUtils.Free;
    FCalculateurDeclimag.Free;
    FContenuFichierTexte.Free;
  end;
end;

procedure TCdrCalculette.FrameClick(Sender: TObject);
begin

end;

procedure TCdrCalculette.FrameResize(Sender: TObject);
begin
  PairSplitter2.Position := self.Width - 300;
end;

procedure TCdrCalculette.lsbFonctionsClick(Sender: TObject);
begin

end;


function TCdrCalculette.InitialiseEvaluateurExpressions(): boolean;
var
  i, n: Integer;
  EWE: String;
begin
  result := true;
  try
    lsbFonctions.Clear;
    FEvaluateurExpressions := TEvaluateurExpressions.Create;
    Result := FEvaluateurExpressions.Initialiser();

    n := FEvaluateurExpressions.GetNbreDeFonctions();
    AfficherMessage(format('%d functions', [n]));
    if (n > 0) then
    begin
      for i := 0 to n - 1 do
      begin
        EWE := FEvaluateurExpressions.GetNomFonction(i);
        lsbFonctions.Items.Add(EWE);
      end;
    end;
  except
  end;
end;

function TCdrCalculette.InitialiseConvertisseur(): boolean;
begin
  Result := False;
  try
    FConversionUtils := TConversionSysteme.Create;
    FConvertisseurActif := FConversionUtils.Initialiser;
    Result := True;
  except
  end;
end;

function TCdrCalculette.InitialiseCalculateurDeclimag(): boolean;
begin
  AfficherMessage('-- InitialiseCalculateurDeclimag');
  Result := False;
  FCalculateurDeclimag := TSAGeoMag.Create;
  try
    Result := FCalculateurDeclimag.Initialiser('');
  finally
  end;
end;
//******************************************************************************
procedure TCdrCalculette.SetLabelsValeursDemandees(const Idx: integer);
begin
  lbValeursX.Caption := 'X (m)';
  lbValeursY.Caption := 'Y (m)';
  if (Idx = 0) then
  begin
    lbValeursX.Caption := 'Lat. (Deg. dec.)';
    lbValeursY.Caption := 'Long. (Deg. dec.)';
  end;
end;

procedure TCdrCalculette.tabShtCalculatriceContextPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);
begin

end;



procedure TCdrCalculette.FaireUnCalcul(const Expr: string);
var
  Err: Integer;
  S: String;
  R: Double;
  DescErr: string;
begin
  Err := 0;
  S := Trim(Expr);
  R := FEvaluateurExpressions.Evaluate(S, Err, DescErr);
  if (Err = 0) then
  begin
    editExpression.Text    := FloatToStr(R);
    AjouterDerniereExpressionValable(Expr);
    lbErreurCalcul.Caption := 'OK';
    ListerLesVariables();
  end
  else
  begin
    editExpression.Text    := Expr;
    lbErreurCalcul.Caption := Format('Erreur %d - %s', [Err, DescErr]);
  end;
end;

procedure TCdrCalculette.AjouterDerniereExpressionValable(const Expr: string);
const
  NB_MAX_CALCULS = 500;
var
  Nb: Integer;
begin
  // dernier calcul en tête de liste
  lsbLastCalculs.Items.Insert(0, Expr);
  Nb := lsbLastCalculs.Count;
  if (Nb > NB_MAX_CALCULS) then lsbLastCalculs.Items.Delete(Nb - 1);
end;

procedure TCdrCalculette.RemplirComboColonnes(const Cmb: TComboBox; const Idx: integer; const QLibColonne: string);
var
  qc: Integer;
  EWE: String;
begin
  ClearComboColonnes(Cmb, Idx);
  for qc := 0 to grdDonnees.ColCount - 1 do
  begin
    EWE := Trim(grdDonnees.Cells[qc, 0]);
    if (EWE <> '') then cmb.Items[qc] := EWE;
  end;
  cmb.ItemIndex := Idx;
  // rechercher les noms de colonnes contenant QLibColonne
  if (QLibColonne =  '') then Exit;
  for qc := 0 to grdDonnees.ColCount - 1 do
  begin
    EWE := Lowercase(Trim(grdDonnees.Cells[qc, 0]));
    if (Pos(Lowercase(QLibColonne), EWE) > 0) then
    begin
      cmb.ItemIndex := qc;
      break;
    end;
  end;
end;
procedure TCdrCalculette.RefactorerTitresDesColonnes();
var
  NbC: Integer;
  i: Integer;
  EWE: String;
begin
  NbC := grdDonnees.ColCount;
  for i := 1 to NbC -1 do
  begin
    EWE := Trim(grdDonnees.Cells[i, 0]);
    if (EWE = '') then grdDonnees.Cells[i, 0] := Format('Colonne%d',[i]);
  end;
end;

procedure TCdrCalculette.ClearComboColonnes(const cmb: TComboBox; const Idx: integer);
var
  q: Integer;
begin
  cmb.Clear;
  for q := 0 to grdDonnees.ColCount - 1  do cmb.Items.add(format('Colonne_%d', [q]));
  cmb.ItemIndex := Idx;
end;

//******************************************************************************
procedure TCdrCalculette.SetDefaultExpressionA(const S: string);
begin
  editExpression.Text := Trim(S);
end;

function TCdrCalculette.GetResultatCalculA(): double;
begin
  Result := ConvertirEnNombreReel(trim(editExpression.Text), 0.00);
end;


function TCdrCalculette.GetDeclimagA(): double;
begin
  Result := editDeclinaison.Value;
end;

function TCdrCalculette.GetCoordonneesA(): TPoint2Df;
begin
  Result := MakeTPoint2Df(editX_Cible1JPC.Value, editY_Cible1JPC.Value);
end;

function TCdrCalculette.GetCurrentCodeEPSGNomSysteme(): TLabelSystemesCoordsEPSG;
begin
  Result := FConversionUtils.GetCodeEPSGNomSysteme(cmbSystCibleJPC.ItemIndex);
end;

procedure TCdrCalculette.SetCoordonneesA(const P: TPoint2Df);
begin
  editX_Source1JPC.Value := P.X;
  editY_Source1JPC.Value := P.Y;
end;
//******************************************************************************

function TCdrCalculette.CheckLatLonInMetropole(const QLat, QLon: double): boolean;
begin
  // Elle est comprise entre les latitudes 42°19'46" N et 51°5'47" N, ainsi que les longitudes 4°46' O et 8°14'42" E
  AfficherMessageErreur(Format('Lat %.5f, Lon %.5f', [QLat, QLon]));
  result := InRange(QLon, -5.0, 8.50) and InRange(QLat, 42.00, 51.50);
end;

procedure TCdrCalculette.ExportListePointsVersCarto(const FileName: string;
                                                    const MC: TOutputFormatGIS;
                                                    const FirstLineIsTitreCol: Boolean;
                                                    const LabelsAutoIncrement: Boolean;
                                                    const DoUseColDatum: boolean;
                                                    const SystSource, SystCible: integer;
                                                    const NoColLabelTitre : integer;
                                                    const NoColLabelDescro: integer;
                                                    const NoColXSource, NoColYSource, NoColZSource: integer);
var
  EWE: TLigneCoordonnees;
  LigneDep: Integer;
  i, Nb: Integer;
  P1, P2: TProjUV;
  MyExportGIS: TExportGIS;
  WU: String;
  QOSMNode: Int64;
  IdxCodeWGS84: Integer;
  QQ: LongInt;
begin
  case MC of
    gisKML       : WU := 'KML';
    gisGPX       : WU := 'GPX';
    gisLEAFLET   : WU := 'Leaflet';
  end;
  AfficherMessage(Format('%s.ExportListePointsKML: Format: %s - %s: %s - %s: %d %d %d',
                         [ClassName,
                          WU,
                          FileName,
                          IIF(FirstLineIsTitreCol, 'Avec Titres', 'Sans Titres'),
                          IIF(LabelsAutoIncrement, 'Labels auto', 'Ya colonne labels'),
                          NoColLabelTitre,
                          NoColXSource, NoColYSource
                         ]));
  MyExportGIS := TExportGIS.Create;
  try
    MyExportGIS.Initialiser(MC, nil, FConversionUtils, SystSource, false, clblue,
                            Screen.Width, Screen.Height,
                            FileName);
    // première ligne
    LigneDep := IIF(FirstLineIsTitreCol, 1, 0);
    IdxCodeWGS84 := FConversionUtils.GetIndexSystemeByCodeEPSG(CODE_EPSG_WGS84);
    P1.U := ConvertirEnNombreReel(grdDonnees.Cells[NoColXSource, LigneDep], 0.00);
    P1.V := ConvertirEnNombreReel(grdDonnees.Cells[NoColYSource, LigneDep], 0.00);
    if (DoUseColDatum) then
    begin
      P2 := FConversionUtils.ConversionSyst1ToSyst2EPSG(strtointdef(grdDonnees.Cells[FColonneDatum, LigneDep], 2154), 4326, P1);
    end
    else
    begin
      P2 := FConversionUtils.ConversionSyst1ToSyst2Index(cmbSystSourceJPC.ItemIndex, IdxCodeWGS84, P1);
    end;
    MyExportGIS.WriteHeader(True, P2.U, P2.v);
    MyExportGIS.WriteCommentaire('*** BEGIN OF DATA ***');
    //try
      Nb := grdDonnees.RowCount;
      // balancer les points
      QOSMNode := 10000;
      for i := LigneDep to Nb - 1 do
      begin
        EWE.EtiquetteTitle       := IIF(LabelsAutoIncrement, Format('Label%d', [i]), Trim(grdDonnees.Cells[NoColLabelTitre, i]));
        EWE.EtiquetteDescription := Trim(grdDonnees.Cells[NoColLabelDescro, i]);

        P1.U := ConvertirEnNombreReel(grdDonnees.Cells[NoColXSource, i], 0.00);
        P1.V := ConvertirEnNombreReel(grdDonnees.Cells[NoColYSource, i], 0.00);

        if (DoUseColDatum) then
        begin
          QQ := strtointdef(grdDonnees.Cells[FColonneDatum, i], 2154);
          if (QQ = -4326) then
          begin
            P1.U := DegMinSec2DegDec(grdDonnees.Cells[NoColXSource, i]);
            P1.U := DegMinSec2DegDec(grdDonnees.Cells[NoColYSource, i]);
          end;
          P2 := FConversionUtils.ConversionSyst1ToSyst2EPSG(QQ, 4326, P1);
        end
        else
        begin
          P2 := FConversionUtils.ConversionSyst1ToSyst2Index(cmbSystSourceJPC.ItemIndex, FConversionUtils.GetIndexSystemeByCodeEPSG(4326), P1);
        end;
        if (CheckLatLonInMetropole(P2.U, P2.V)) then
        begin
          EWE.XSource := P1.U;
          EWE.YSource := P1.V;
          EWE.ZSource := ConvertirEnNombreReel(grdDonnees.Cells[NoColZSource, i], 0.00);;
          EWE.XCible  := P2.U;
          EWE.YCible  := P2.V;
          AfficherMessage(Format('%d - %s - %.2f, %.2f, %.2f > %.8f, %.8f', [i, EWE.EtiquetteTitle, EWE.XSource, EWE.YSource, EWE.ZSource, EWE.XCible, EWE.YCible]));
          MyExportGIS.WritePoint(EWE.XCible, EWE.YCible, EWE.ZSource, EWE.EtiquetteTitle, EWE.EtiquetteDescription, QOSMNode);
          QOSMNode += 1;
        end;
      end;

    //except
    //end;
    MyExportGIS.WriteCommentaire('*** END OF DATA ***');
    MyExportGIS.WriteFooter;
    MyExportGIS.Finaliser;
  finally
    MyExportGIS.Free;
  end;
end;


procedure TCdrCalculette.editExpressionKeyPress(Sender: TObject; var Key: char);
begin
  if (key = #13) then
  begin
    FaireUnCalcul(editExpression.Text);
    editExpression.SelStart := length(editExpression.Text);
    editExpression.SelLength:= 0;
  end;
end;


procedure TCdrCalculette.lsbLastCalculsDblClick(Sender: TObject);
var
  idx: Integer;
  WU: String;
  l: Integer;
  EWE: String;
begin
  // rappel d'un calcul: recopie l'expression dans la zone de texte
  idx := lsbLastCalculs.ItemIndex;
  if (idx >=0) then
  begin
    EWE := lsbLastCalculs.Items[idx];
    // possibilité d'ajout: si un underscore est ajouté en fin de la formule courante,
    // on ajoute la ligne sélectionnée à la formule
    WU := Trim(editExpression.Text);
    l  := length(WU);
    if (WU[l] = '_') then
    begin
      System.Delete(WU, l, 1); // on efface le caractère de continuation
      WU += EWE;               // on ajoute à l'expression existante
    end
    else
    begin
      WU := EWE;               // sinon, on remplace l'expression
    end;
    editExpression.Text := WU;
  end;
end;


procedure TCdrCalculette.lsbPSFunctionsSelectionChange(Sender: TObject; User: boolean);
var
  MyProc: TPSRegProc;
begin
  ShowMessage('DisplayHelpForProc()');
  try
    MyProc := PSScriptDebugger1.Comp.GetRegProc(lsbPSFunctions.ItemIndex);
    DisplayHelpForProc(MyProc);
  except
  end;
end;

procedure TCdrCalculette.PageControl1Change(Sender: TObject);
begin
  case PageControl1.ActivePageIndex of
    0: editExpression.SetFocus;
    1: editX_Source1JPC.SetFocus;
    2: editLatitude.SetFocus;
    3: editTextToEncodeQR.SetFocus;
    4: pass;
    5: editPascalScript.SetFocus;
  end;
end;

procedure TCdrCalculette.PSScriptDebugger1AfterExecute(Sender: TPSScript);
begin
  ShowMessage('Script terminé');
end;

procedure TCdrCalculette.DisplayHelpForProc(const MyProc: TPSRegProc);
var
  p: LongInt;
begin

  memoHelp.Clear;
  memoHelp.Lines.Add(MyProc.Name);
  p := MyProc.Decl.ParamCount;
  (*
    if (p > 0) then
    begin
      try
        for j := 0 to p - 1 do
        begin
          EWE := WU.Decl.Params[j];
          lsbPSFunctions.Items.Add(Format('--> %s: %s',[LowerCase(EWE.Name), LowerCase(EWE.aType.Name)]));
        end;
      except

      end;
    end;
  //*)
  //memoHelp.Lines.
end;


procedure TCdrCalculette.PSScriptDebugger1Compile(Sender: TPSScriptDebugger);
var
  MyPlugin: TPSImport_Classes;
  miaou: TPSType;
begin
  lsbPSMessages.Clear;


  //passage de la classe TToporobotStructure2012;
  //SIRegister_Classes(Sender.Comp, True);
  //SIRegister_Graphics(Sender.Comp, True);


  //AddImportedClassVariable(Sender.Comp, 'FDocuTopo', 'TToporobotStructure2012');
  //miaou := sender.comp.AddType('TStringArray', TStringArray);
  //sender.add('OutArrStr', 'TStringDynArray');
  sender.AddFunction(@pass                     , 'procedure pass;');
  sender.AddFunction(@SplitStr                 , 'function SplitStr(const Str: string; const Sep: string; var ArrValues: TStringList): boolean;');


  sender.AddFunction(@GetGHTopoVersion         , 'function GetGHTopoVersion(): string;');
  sender.AddFunction(@GetGHTopoDirectory       , 'function GetGHTopoDirectory(): string');
  sender.AddFunction(@FileExists               , 'function FileExists(const NF: string): boolean;');
  sender.AddFunction(@ExtractFileExt           , 'function ExtractFileExt(const FileName: string): string;');
  sender.AddFunction(@ExtractFileName          , 'function ExtractFileName(const FileName: string): string;');

  //sender.AddFunction(@AfficherMessage        , 'procedure AfficherMessage(const msg: string);');
  sender.AddFunction(@Format                   , 'function format(Const Fmt : String; const Args : Array of const) : String;');
  sender.AddFunction(@Format                   , 'function sprintf(Const Fmt : String; const Args : Array of const) : String;');
  sender.AddFunction(@KMLColor                 , 'function KMLColor(const R, G, B, A: byte): string;');
  sender.AddFunction(@ConversionCoordonneesIsoleesEPSG, 'function ConversionCoordonneesIsoleesEPSG(const Src, Tgt: integer; const PointInX, PointInY: double; out PointOutX, PointOutY: double): boolean;');
  sender.AddFunction(@ConvertirEnNombreReel    , 'function StrToFloatDef(const S: string; const Default: double): double;');
  sender.AddFunction(@FormatterIDTerrainStation, 'function FormatterIDTerrainStation(const S: string): string;');

  // boites de dialogue
  sender.AddFunction(@ShowMessage              , 'procedure ShowMessage(const msg: string);');
  sender.AddFunction(@ShowMessage              , 'procedure MsgBox(const msg: string);');
  sender.AddFunction(@GHTopoQuestionOuiNon     , 'function  QuestionOuiNon(const Msg: string): boolean;');
  sender.AddFunction(@GHTopoInputQuery         , 'function  InputQuery(const ACaption, APrompt: string; var QValeur: string): boolean;');
  sender.AddFunction(@DoDialogSaveFile         , 'function  DialogSaveFile(const QFilter: string; const QDefaultExt: string; var QFileName: string; out idxFilter: integer): boolean;');
  sender.AddFunction(@DoDialogOpenFile         , 'function  DialogOpenFile(const QFilter: string; const QDefaultExt: string; var QFileName: string): boolean;');
  sender.AddFunction(@SelectionCouleurToporobot, 'function  SelectionCouleurToporobot(const OldIdx: integer): Integer;');

  // fonctions math additionnelles : certaines fonctions ne sont pas acceptées telles quelles
  sender.AddFunction(@tan                      , 'function tan(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@arcsin                   , 'function arcsin(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@arccos                   , 'function arccos(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@farctan                  , 'Function arctan(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@farctan2                 , 'Function arctan2(y, x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@sinh                     , 'Function sinh(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@cosh                     , 'Function cosh(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@tanh                     , 'Function tanh(x : Extended) : Extended;'); // added by JPC
  sender.AddFunction(@fexp                     , 'function exp(x : Extended) : Extended;');
  sender.AddFunction(@fln                      , 'function ln(x : Extended) : Extended;');
  sender.AddFunction(@flog                     , 'function log(x : Extended) : Extended;');
  sender.AddFunction(@floor                    , 'function floor(x : Extended) : integer;');
  sender.AddFunction(@ceil                     , 'function ceil(x : Extended) : integer;');
  sender.AddFunction(@fabs                     , 'function fabs(x : Extended) : Extended;');

  sender.AddFunction(@radtodeg                 , 'function radtodeg(x : Extended) : Extended;');
  sender.AddFunction(@degtorad                 , 'function degtorad(x : Extended) : Extended;');
  sender.AddFunction(@radtograd                , 'function radtograd(x : Extended) : Extended;');
  sender.AddFunction(@gradtorad                , 'function gradtorad(x : Extended) : Extended;');
  sender.AddFunction(@degtograd                , 'function degtograd(x : Extended) : Extended;');
  sender.AddFunction(@gradtodeg                , 'function gradtodeg(x : Extended) : Extended;');
  sender.AddFunction(@DegMinSec2DegDec         , 'function DegMinSec2DegDec(const S: string): double;');

  sender.AddFunction(@Now                      , 'function Now(): double;');
  sender.AddFunction(@Now                      , 'function Maintenant(): double;');

  sender.AddFunction(@DateTimePascalToDateTimeSQL , 'function DateTimePascalToDateTimeSQL(const MyDate: double): string;');
  sender.AddFunction(@DateTimeSQLToDateTimePascal , 'function DateTimeSQLToDateTimePascal(const MyDateSQL: string): double;');

  sender.AddFunction(@Hypot2D                     , 'function Hypot2D(const DX, DY: Double): Double;');
  sender.AddFunction(@Hypot3D                     , 'function Hypot3D(const DX, DY, DZ: Double): Double;');
  sender.AddFunction(@GetAzimut                   , 'function GetAzimut(const dx, dy: double; const Unite: double): double;');


  //sender.AddFunction(@ , '');
  //sender.AddFunction(@ , '');
  //sender.AddFunction(@ , '');
  //sender.AddFunction(@ , '');


  // ces fonctions utilisent des variables privées de TCdrCalculette;
  sender.AddMethod(self, @TCdrCalculette.DispPSOutput, 'procedure print(const Msg: string);');
  sender.AddMethod(self, @TCdrCalculette.printf      , 'procedure printf(const Fmt : String; const Args : Array of const): String;');
  sender.AddMethod(self, @TCdrCalculette.feof        , 'function  feof(const Descr: integer): boolean;');
  sender.AddMethod(self, @TCdrCalculette.fopen       , 'function  fopen(const Descr: integer; const QFileName: string; const Mode: Char): boolean;');
  sender.AddMethod(self, @TCdrCalculette.fprint      , 'procedure fwriteln(const Descr: integer; const QLine: string);');
  sender.AddMethod(self, @TCdrCalculette.fprintf     , 'procedure fprintf(const Descr: integer; const FMT: string; const Args: array of const);');
  sender.AddMethod(self, @TCdrCalculette.freadln     , 'function  freadln(const Descr: integer): string;');
  sender.AddMethod(self, @TCdrCalculette.fclose      , 'procedure fclose(const Descr: integer)');
  sender.AddMethod(self, @TCdrCalculette.fcloseall   , 'procedure fcloseall()');

  {$ifdef CALCULETTE_EMBEDDED_IN_GHTOPO}
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNomEtude           , 'function GetNomEtude(): string;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbSeries           , 'function GetNbSeries(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbEntrees          , 'function GetNbEntrees(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbReseaux          , 'function GetNbReseaux(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbExpes            , 'function GetNbExpes(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbCodes            , 'function GetNbCodes(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbSecteurs         , 'function GetNbSecteurs(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.GetNbAntennes         , 'function GetNbAntennes(): integer;');
    sender.AddMethod(FDocuTopo, @TToporobotStructure2012.CheckerLesDonneesTopo , 'procedure CheckerLesDonneesTopo();');
    //*)
    sender.AddMethod(self, @TCdrCalculette.DT_GetNbreStationsOfSerieByIdx      , 'function GetNbStationsOfSerieByIdx(const Idx: integer): integer;');
    sender.AddMethod(self, @TCdrCalculette.DT_GetNbreStationsOfSerieByNoSerie  , 'function GetNbStationsOfSerieByNoSerie(const NumeroSerie: integer): integer;');
  {$endif CALCULETTE_EMBEDDED_IN_GHTOPO}
   ListerPSFunctions();
end;

procedure TCdrCalculette.PSScriptDebugger1Execute(Sender: TPSScript);
begin
  Application.ProcessMessages;
end;

procedure TCdrCalculette.PSScriptDebugger1Idle(Sender: TObject);
begin
  //DispPSOutput('miaou');
end;

procedure TCdrCalculette.acRemoveLigneExecute(Sender: TObject);
begin
  if (GHTopoQuestionOuiNon('Supprimer cette ligne')) then
  begin
    try
      grdDonnees.DeleteRow(grdDonnees.Row);
      //grdDonnees++;
    except
    end;
  end;
end;



procedure TCdrCalculette.acReverseSystCoordonneesExecute(Sender: TObject);
var
  toto: Integer;
begin
  toto := cmbSystSourceJPC.ItemIndex;
  cmbSystSourceJPC.ItemIndex := cmbSystCibleJPC.ItemIndex;
  cmbSystCibleJPC.ItemIndex  := toto;
end;

procedure TCdrCalculette.acSavePSScriptExecute(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QIdx: integer;
begin
  QFileName := 'Script001.pas';
  if (DoDialogSaveFile('Fichiers PascalScript (*.pas)|*.pas', '.pas', QFileName, QIdx)) then
  begin
    editPascalScript.Lines.SaveToFile(QFileName);
  end;
end;



procedure TCdrCalculette.btnCopierTableauClick(Sender: TObject);
begin
  grdDataSeparetedBySpaces.CopyToClipboard();
end;


procedure TCdrCalculette.acRemoveColonneExecute(Sender: TObject);
begin
  grdDonnees.DeleteCol(grdDonnees.Col);
end;


procedure TCdrCalculette.acCalculerExecute(Sender: TObject);
begin
  FaireUnCalcul(editExpression.Text);
end;

procedure TCdrCalculette.acCalculDeclimagExecute(Sender: TObject);
begin
  editDeclinaison.Value := FCalculateurDeclimag.CalculerDeclinaison(editLongitude.Value,
                                                                    editLatitude.Value,
                                                                    editAltitude.Value,
                                                                    editDateDeclimag.Date);
end;

procedure TCdrCalculette.acCalculDeclimagTableauExecute(Sender: TObject);
var
  NbValeurs: Integer;
  AnneeCourante: LongInt;
  i: Integer;
  EWE: TDateTime;
  Decl: Double;
begin

  if (not FCalculateurDeclimagActif) then Exit;
  try
    i := editAnneeArrivee.AsInteger - editAnneeDepart.AsInteger;
    NbValeurs := i + 1;
    AnneeCourante := editAnneeDepart.AsInteger;
    if (i < 0) then
    begin
       ShowMessage('L''année de fin doit être supérieure à l''année de début');
       Exit;
    end;
    with grdDeclinaisons do
    begin
      ColCount     := 2;
      RowCount     := 1 + NbValeurs;
      Cells[0, 0]  := 'Année';
      Cells[1, 0]  := 'Déclinaison';
      ColWidths[0] := 70;
      ColWidths[1] := 100;
      for i := 1 to NbValeurs do
      begin
        EWE := GetSecuredDate(AnneeCourante, 1, 1);
        Cells[0,i] := Format('%d', [AnneeCourante]);
        Decl := FCalculateurDeclimag.CalculerDeclinaison(editLongitude.Value,
                                                         editLatitude.Value,
                                                         editAltitude.Value,
                                                         EWE);
        Cells[1, i] := Format('%.4f', [Decl]);
        Inc(AnneeCourante, 1);
      end;
    end;

  except

  end;
  //*)
end;

procedure TCdrCalculette.acCollerExecute(Sender: TObject);
var
  EWE: String;
begin
  AfficherMessage(Format('%s.Coller', [self.ClassName]));
  //try
    EWE := ChooseString(cmbSeparateur.ItemIndex, [#9, ';', ',', ':', '|']);
    //AfficherMessage(Format('%d', [000]));
    // \t Tabulation; Point virgule, Virgule: Deux points| Barre
    GRDCollerDepuisClipBoard(grdDonnees, chkPremLigneIsTitres.Checked, EWE);
    //AfficherMessage(Format('%d', [001]));
    // A partir d'ici, on n'a plus besoin du presse-papier ni de la liste provisoire
    // On travaille directement sur la grille
    // garnir les combobox
    RemplirComboColonnes(cmbColLabelsTitres  , FColonneLabelsTitres , 'Nom');
    RemplirComboColonnes(cmbColLabelsDescros , FColonneLabelsDescros, 'Description');
    //AfficherMessage(Format('%d', [002]));
    RemplirComboColonnes(cmbColXSource, FColonneXSource, 'X');
    RemplirComboColonnes(cmbColYSource, FColonneYSource, 'Y');
    RemplirComboColonnes(cmbColZSource, FColonneZSource, 'Z');

    RemplirComboColonnes(cmbColXCible , FColonneXCible, 'Lat');
    RemplirComboColonnes(cmbColYCible , FColonneYCible, 'Lon');
    //AfficherMessage(Format('%d', [003]));
    RefactorerTitresDesColonnes;
    //AfficherMessage(Format('%d', [004]));
  //except
  //end;
end;

procedure TCdrCalculette.acConvertirExecute(Sender: TObject);
var
  P1, P2: TProjUV;
begin
  // si syst source = syst cible --> []
  if (cmbSystSourceJPC.ItemIndex = cmbSystCibleJPC.ItemIndex) then Exit;
  try
    P1.U := editX_Source1JPC.Value;
    P1.V := editY_Source1JPC.Value;
  except
    P1.U := DegMinSec2DegDec(editX_Source1JPC.Text);
    P1.V := DegMinSec2DegDec(editY_Source1JPC.Text);
  end;
  P2 := FConversionUtils.ConversionSyst1ToSyst2Index(cmbSystSourceJPC.ItemIndex,
                                                     cmbSystCibleJPC.ItemIndex,
                                                     P1);
  editX_Cible1JPC.Value := P2.U;
  editY_Cible1JPC.Value := P2.V;
  // pour les cases Long et Lat du calcul de déclinaison
  editLongitude.Value:= P2.U;
  editLatitude.Value := P2.V;
end;

procedure TCdrCalculette.acConvertirTableauExecute(Sender: TObject);
var
  P1, P2: TProjUV;
  LigneDep: Integer;
  i: Integer;
  WU: Boolean;
begin
  if (cmbSystSourceJPC.ItemIndex = cmbSystCibleJPC.ItemIndex) then
  begin
    ShowMessage('Les systèmes de coordonnées source et cible sont identiques');
    Exit;
  end;

  // récupération des index de colonnes
  FColonneDatum   := cmbColDatum.ItemIndex;
  FColonneXSource := cmbColXSource.ItemIndex;
  FColonneYSource := cmbColYSource.ItemIndex;
  FColonneZSource := cmbColZSource.ItemIndex;


  FColonneXCible  := cmbColXCible.ItemIndex;
  FColonneYCible  := cmbColYCible.ItemIndex;
  WU := (FColonneXSource = FColonneXCible) OR // colonne sources et destination identiques
        (FColonneYSource = FColonneYCible) OR // colonne sources et destination identiques
        (FColonneXSource = FColonneYCible) OR
        (FColonneYSource = FColonneXCible) OR
        (FColonneXSource = FColonneYSource) OR // colonne XY sources identiques
        (FColonneXCible  = FColonneYCible);     // colonne XY sources identiques
  if (WU) then
  begin
    ShowMessageFmt('Les colonnes %d, %d, %d, %d se marchent dessus', [FColonneXSource, FColonneXCible,FColonneYSource,FColonneYCible]);
    Exit;
  end;
  if (chkPremLigneIsTitres.Checked) then LigneDep := 1 else LigneDep := 0;
  for i := LigneDep to grdDonnees.RowCount - 1 do
  begin
    try
      P1.U := ConvertirEnNombreReel(grdDonnees.Cells[FColonneXSource, i], 0.00);
      P1.V := ConvertirEnNombreReel(grdDonnees.Cells[FColonneYSource, i], 0.00);
      if (chkUseDatumOfColumn.Checked) then
      begin
        P2 := FConversionUtils.ConversionSyst1ToSyst2EPSG(strtointdef(grdDonnees.Cells[FColonneDatum, i], 2154), 4326, P1);
      end
      else
      begin
        P2 := FConversionUtils.ConversionSyst1ToSyst2Index(cmbSystSourceJPC.ItemIndex, cmbSystCibleJPC.ItemIndex, P1);
      end;
      grdDonnees.Cells[FColonneXCible, i] := FormatterNombreOOo(P2.U, 8, false);
      grdDonnees.Cells[FColonneYCible, i] := FormatterNombreOOo(P2.V, 8, false);
    except
      grdDonnees.Cells[FColonneXCible, i] := 'Err. X';
      grdDonnees.Cells[FColonneYCible, i] := 'Err. Y';
    end;
  end;
end;

procedure TCdrCalculette.acCopierTableauExecute(Sender: TObject);
begin
  GRDCCopierUnTableau(grdDonnees);
end;

procedure TCdrCalculette.acCopyCoordinatesExecute(Sender: TObject);
const
  QSep = ';';
var
  EWE: String;
  MyClipBoard: TClipboard;
begin
  MyClipBoard := TClipboard.Create(ctClipboard);
  try
    MyClipBoard.Clear;
    // contenu
    EWE := editX_Source1JPC.Text + QSep +
           editY_Source1JPC.Text + QSep +
           editX_Cible1JPC.Text + QSep +
           editY_Cible1JPC.Text;
    MyClipBoard.AsText := EWE;
  finally
    MyClipBoard.Free;
  end;
end;

procedure TCdrCalculette.acCopyTableauDeclimagExecute(Sender: TObject);
begin
  GRDCCopierUnTableau(grdDeclinaisons);
end;

procedure TCdrCalculette.acEchangerValeursExecute(Sender: TObject);
var
  EWE: Extended;
begin
  EWE := editX_Source1JPC.Value;
  editX_Source1JPC.Value := editX_Cible1JPC.Value;
  editX_Cible1JPC.Value  := EWE;
  EWE := editY_Source1JPC.Value;
  editY_Source1JPC.Value := editY_Cible1JPC.Value;
  editY_Cible1JPC.Value  := EWE;
end;

procedure TCdrCalculette.acExportCSVExecute(Sender: TObject);
var
  QFilename: TStringDirectoryFilename;
  QFilterIndex: integer;
begin
  QFilename := 'Tableau1.csv';
  if (DoDialogSaveFile('Fichier CSV (*.csv)|*.csv', '.csv', QFilename, QFilterIndex)) then
  begin
    GRDSauverUnTableauEnCSV(grdDonnees, QFilename);
  end;
end;



procedure TCdrCalculette.acExportListePointsKMLExecute(Sender: TObject);
var
  QFilters: String;
  QFileName: TStringDirectoryFilename;
  QIdxFilter: integer;
begin
  if (not GHTopoQuestionOuiNon('Utilisation: Sélectionner les colonnes contenant l''étiquette et les coordonnnées' + #13#10 +
                         Format('Colonne pour X: %s', [cmbColXSource.Text]) + #13#10 +
                         Format('Colonne pour Y: %s', [cmbColYSource.Text]) + #13#10 +
                         Format('Colonne pour Z: %s', [cmbColZSource.Text]) + #13#10 +
                         Format('Colonne pour étiquette: %s', [cmbColLabelsTitres.Text]) + #13#10 +
                         'Continuer')) then Exit;
  QIdxFilter := 0;
  QFilters := 'Fichiers Google Earth (*.kml)|*.kml|' +
              'Fichiers GPS (*.gpx)|*.gpx|' +
              'Fichiers OpenStreetMap (*.osm)|*.osm|' +
              'Cartes OSM Leaflet (*.htm)|*.htm';

  if (DoDialogSaveFile(QFilters, '*.kml', QFileName, QIdxFilter)) then
  begin
    ExportListePointsVersCarto(QFileName,
                               TOutputFormatGIS(QIdxFilter - 1),  // rappel: TSaveDialog.FilterIndex démarre à 1
                               chkPremLigneIsTitres.Checked,
                               chkAutoIncrement.Checked,
                               chkUseDatumOfColumn.Checked,
                               cmbSystSourceJPC.ItemIndex,
                               CODE_EPSG_WGS84,
                               cmbColLabelsTitres.ItemIndex,
                               cmbColLabelsDescros.ItemIndex,
                               cmbColXSource.ItemIndex,
                               cmbColYSource.ItemIndex,
                               cmbColZSource.ItemIndex);
  end;
end;


procedure TCdrCalculette.acExportQRCodeToSVGExecute(Sender: TObject);
const
  CoteCarre = 05;
  NOM_STYLE_CADRE       = 'Cadre';
  NOM_STYLE_CARRE_NOIR  = 'CarreNoir';
  NOM_STYLE_CARRE_BLANC = 'CarreBlanc';
var
  SVGCanvas: TSVGCanvas;
  QFileName: TStringDirectoryFilename;
  NbCarres: Integer;
  L, H: Integer;
  QRMatrix: TMatriceResultat;
  i, j: Integer;
  EWE: String;
  QFilterIndex: integer;
begin
  if (Not DoDialogSaveFile('Fichiers SVG |*.svg', '.svg', QFileName, QFilterIndex)) then Exit;
  VueQRCode.DoMonochrome := FALSE;  // ne pas utiliser le mode monochrome
  VueQRCode.Text := editTextToEncodeQR.Text;
  VueQRCode.Generate;
  SVGCanvas := TSVGCanvas.Create;
  try
    NbCarres := VueQRCode.GetNbQRTaille;
    // en-tetes
    SVGCanvas.Commentaire := '';
    SVGCanvas.Scale       := 1.00;
    // maxi et mini
    L := CoteCarre * NbCarres;
    H := L;
    if (Not SVGCanvas.InitializeDocument(QFileName,
                                         False,
                                         'TextEncoded ' + VueQRCode.Text,
                                         Format('%s: QRCode', [ExtractFileName(QFileName)]),
                                         0, 0, L, H)) then Exit;
    // patterns
    SVGCanvas.BeginPatternsSection;
      // les patterns ici
    SVGCanvas.EndPatternsSection;
    // section de styles
    SVGCanvas.BeginStylesSection;
      // styles de cube englobant
      SVGCanvas.WriteStyleSolidLine(NOM_STYLE_CADRE, clGray, 0.01, 'cadre');
      // style par défaut: gris

      SVGCanvas.WriteStyleSolidPolygone('Carre_00' , clWhite, clWhite, 255, 0.0, 'data: Blanc');
      SVGCanvas.WriteStyleSolidPolygone('Carre_01' , clBlack, clBlack, 255, 0.0, 'data: Noir');
      SVGCanvas.WriteStyleSolidPolygone('Carre_16' , clWhite, clWhite, 255, 0.0, 'carres de controle (zones blanches)');
      SVGCanvas.WriteStyleSolidPolygone('Carre_17' , clBlack, clBlack, 255, 0.0, 'carres de controle (zones noires)');
      SVGCanvas.WriteStyleSolidPolygone('Carre_20' , clBlack, clBlack, 255, 0.0, 'carre en M[8,8]: noir');
      SVGCanvas.WriteStyleSolidPolygone('Carre_21' , clBlack, clBlack, 255, 0.0, 'carre en M[8,8]: noir');
      SVGCanvas.WriteStyleSolidPolygone('Carre_32' , clWhite, clWhite, 255, 0.0, 'zones de detrompage: blanc');
      SVGCanvas.WriteStyleSolidPolygone('Carre_33' , clBlack, clBlack, 255, 0.0, 'zones de detrompage: noir');
      SVGCanvas.WriteStyleSolidPolygone('Carre_65' , clBlack, clBlack, 255, 0.0, 'carres en M[6, 8]');
    SVGCanvas.EndStylesSection;
    AfficherMessage('----ksdlf');
    // dessin
    SVGCanvas.BeginLayer('Principal');
      SVGCanvas.BeginGroupe('QRCode', '', 10.00, 10.00);
        QRMatrix := VueQRCode.GetMatrixQRResultat;
        for i := 0 to NbCarres -1 do
          for j := 0 to NbCarres -1 do
          begin
            SVGCanvas.DrawRectangle(Format('Carre_%.2d', [QRMatrix[i,j]]),
                                    CoteCarre * i    , H - CoteCarre * j,
                                    CoteCarre * (i+1), H - CoteCarre * (j+1));
          end;
      SVGCanvas.EndGroupe('QRCode');
    SVGCanvas.EndLayer('Principal');


    // cloture
    SVGCanvas.FinalizeDocument;
  finally
    SVGCanvas.Free;
  end;
end;

procedure TCdrCalculette.acGenerateQRCodeFromTexteExecute(Sender: TObject);
begin
  VueQRCode.Text := Trim(editTextToEncodeQR.Text);
  VueQRCode.Generate;
end;

procedure TCdrCalculette.acLoadPSScriptExecute(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
begin
  if (DoDialogOpenFile('Pascal Script (*.pas)|*.pas', '.pas', QFileName)) then
  begin
    editPascalScript.Lines.LoadFromFile(QFileName);

  end;
end;



procedure TCdrCalculette.acOuvrirCSVExecute(Sender: TObject);
var
  EWE: String;
  QFileName: TStringDirectoryFilename;
begin
  if (DoDialogOpenFile('Fichiers CSV(*.csv)|*.csv', '.csv', QFileName)) then
  begin
    EWE := ChooseString(cmbSeparateur.ItemIndex, [#9, ';', ',', ':', '|']);
    // \t Tabulation; Point virgule, Virgule: Deux points| Barre
    if (GRDChargerDepuisFichier(grdDonnees, True, QFileName, EWE)) then
    begin
       RemplirComboColonnes(cmbColLabelsTitres , FColonneLabelsTitres , '');
       RemplirComboColonnes(cmbColLabelsDescros , FColonneLabelsDescros, '');
       RemplirComboColonnes(cmbColXSource, FColonneXSource, 'X');
       RemplirComboColonnes(cmbColYSource, FColonneYSource, 'Y');
       RemplirComboColonnes(cmbColXCible , FColonneXCible, '');
       RemplirComboColonnes(cmbColYCible , FColonneYCible, '');
       RefactorerTitresDesColonnes;
     end;
  end;
end;



procedure TCdrCalculette.acQRCodeCoordPointExecute(Sender: TObject);
begin
  editTextToEncodeQR.Lines.Add(editX_Cible1JPC.Text);
  editTextToEncodeQR.Lines.Add(editY_Cible1JPC.Text);
  VueQRCode.Text := Trim(editTextToEncodeQR.Text);
  VueQRCode.Generate;
  PageControl1.ActivePageIndex := TAB_INDEX_QRCODE;
end;

procedure TCdrCalculette.acQRCodeListeCoordsExecute(Sender: TObject);
var
  i, j, Nb: Integer;
  MySelection: TGridRect;
  EWE: String;
begin
  Nb := grdDonnees.RowCount;
  MySelection := grdDonnees.Selection;
  for i := MySelection.Top to MySelection.Bottom do
  begin
    EWE := '';
    for j := MySelection.Left to MySelection.Right do
    begin
      EWE := EWE + Trim(grdDonnees.Cells[j, i]) + '; ';
    end;
    editTextToEncodeQR.Lines.Add(EWE);
  end;
  VueQRCode.Text := Trim(editTextToEncodeQR.Text);
  VueQRCode.Generate;
  PageControl1.ActivePageIndex := TAB_INDEX_QRCODE;
end;

procedure TCdrCalculette.btnSeparateursClick(Sender: TObject);
begin
  pnlSeparateurs.Visible := not pnlSeparateurs.Visible;
end;

procedure TCdrCalculette.btnConvertDMS2DegDecClick(Sender: TObject);
begin
  editX_Source1JPC.Value := DegMinSec2DegDec(editLatitudeDMS.Text);
  editY_Source1JPC.Value := DegMinSec2DegDec(editLongitudeDMS.Text);
end;



procedure TCdrCalculette.btnOuvrirTXTClick(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
begin
  if (DoDialogOpenFile('Fichiers texte |*.*', '.txt', QFileName)) then
  begin
    FContenuFichierTexte.Clear;
    FContenuFichierTexte.LoadFromFile(QFileName);
    ParserContenuTexte;
  end;
end;


procedure TCdrCalculette.ParserContenuTexte();
var
  WU: String;
  i, j, Nb: Integer;
  MySeparateurs: TSysCharSet;
  EWE: TGHStringArray;
begin
  Nb := FContenuFichierTexte.Count;
  grdDataSeparetedBySpaces.RowCount := Nb + 1;
  grdDataSeparetedBySpaces.ColCount := 1 + High(EWE);
  MySeparateurs := GetSeparateursFromPnl();
  for i := 0 to Nb - 1 do
  begin
    WU := StringReplace(FContenuFichierTexte.Strings[i], '\t', #9, [rfReplaceAll, rfIgnoreCase]);
    WU := StringReplace(WU, '\r', ' ', [rfReplaceAll, rfIgnoreCase]);
    EWE := DecoupeChaine(WU,
                         MySeparateurs, [],
                         chkRetirerGuillemets.Checked);
    for j := 0 to High(EWE) do grdDataSeparetedBySpaces.Cells[j, i+1] := EWE[j];
  end;
end;


procedure TCdrCalculette.editExpressionEnter(Sender: TObject);
begin
  editExpression.SelStart := length(editExpression.Text);
  editExpression.SelLength:= 0;
end;

procedure TCdrCalculette.editLatitudeDMSEnter(Sender: TObject);
begin
  editLatitude.Color := clYellow;
end;

procedure TCdrCalculette.editLatitudeDMSExit(Sender: TObject);
begin
  editLatitude.Color := clWhite;
end;

procedure TCdrCalculette.editLongitudeDMSEnter(Sender: TObject);
begin
  editLatitude.Color := clYellow;
end;

procedure TCdrCalculette.editLongitudeDMSExit(Sender: TObject);
begin
  editLatitude.Color := clWhite;
end;

procedure TCdrCalculette.editPascalScriptChange(Sender: TObject);
begin

end;


procedure TCdrCalculette.editTextToEncodeQRKeyPress(Sender: TObject;
  var Key: char);
begin
  if (Key = #13) then
  begin
    VueQRCode.Text := Trim(editTextToEncodeQR.Text);
    VueQRCode.Generate;
  end;
end;

procedure TCdrCalculette.lsbFonctionsDblClick(Sender: TObject);
var
  Expr: TCaption;
  EWE: String;
begin
  Expr := editExpression.Text;                               // récupère l'expression en cours d'écriture
  EWE  := Trim(lsbFonctions.Items[lsbFonctions.ItemIndex]);  // attrappe la fonction
  EWE += '(';                                                // ajoute la parenthèse ouvrante de la fonction
  Expr += EWE;                                               // poursuit la composition
  editExpression.Text := Expr;                               // met à jour la textbox
  editExpression.SetFocus;                                   // et s'y rend
end;




procedure TCdrCalculette.Button2Click(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QFilterIndex: integer;
begin
  (*
  QFileName := 'CartesPtsTopo.pdf';
  if (DoDialogSaveFile('Fichiers PDF|*.pdf', '.pdf', QFileName, QFilterIndex)) then
  begin
    GenererPagesPointsTopo(QFileName, editNbCartesQRAGenerer.AsInteger);
  end;
  //*)
end;



(*
procedure TCdrCalculette.Button3Click(Sender: TObject);
var
  i, n: Integer;
  P1 , P2: TProjUV;
begin
  n := grdDonnees.RowCount;
  for i := 1 to n-1 do
  begin
    P1.U := DegMinSec2DegDec(grdDonnees.Cells[cmbColXSource.ItemIndex, i]);
    P1.V := DegMinSec2DegDec(grdDonnees.Cells[cmbColYSource.ItemIndex, i]);

    P2 := FConversionUtils.ConversionSyst1ToSyst2EPSG(4326, 27573, P1);
    grdDonnees.Cells[cmbColXCible.ItemIndex, i] := FormatterNombreOOo(P2.U);
    grdDonnees.Cells[cmbColYCible.ItemIndex, i] := FormatterNombreOOo(P2.V);
  end;
end;
//*)

procedure TCdrCalculette.btnReparserClick(Sender: TObject);
begin
  ParserContenuTexte();
end;



procedure TCdrCalculette.cmbColLabelsTitresChange(Sender: TObject);
begin
  FColonneLabelsTitres  := cmbColLabelsTitres.ItemIndex;
end;
procedure TCdrCalculette.cmbColLabelsDescrosChange(Sender: TObject);
begin
  FColonneLabelsDescros := cmbColLabelsTitres.ItemIndex;
end;

procedure TCdrCalculette.cmbColXCibleChange(Sender: TObject);
begin
  FColonneXCible := cmbColXCible.ItemIndex;
end;

procedure TCdrCalculette.cmbColXSourceChange(Sender: TObject);
begin
  FColonneXSource := cmbColXSource.ItemIndex;
end;

procedure TCdrCalculette.cmbColYCibleChange(Sender: TObject);
begin
  FColonneYCible := cmbColYCible.ItemIndex;
end;

procedure TCdrCalculette.cmbColYSourceChange(Sender: TObject);
begin
  FColonneYSource := cmbColYSource.ItemIndex;
end;

procedure TCdrCalculette.cmbSystSourceJPCChange(Sender: TObject);
begin
  SetLabelsValeursDemandees(cmbSystSourceJPC.ItemIndex);
end;

procedure TCdrCalculette.editLatitudeDMSChange(Sender: TObject);
begin

end;

//******************************************************************************

procedure TCdrCalculette.GenererPagesPointsTopo(const QFileName: string; const QNb: integer);
var
  LSD: TEnsembleFichesPointsTopo;
begin
  AfficherMessage(Format('%s.GenererPagesPointsTopo: %s - %d', [ClassName, QFileName, QNb]));
  LSD := TEnsembleFichesPointsTopo.Create;
  try
    LSD.Initialise();
    LSD.CreerNouvellesFiches(QNB, QFileName);
  finally
    LSD.Free;
  end;
end;
//------------------------------------------------------------------------------

procedure TCdrCalculette.ExporterUnPointKML(const FileName: TStringDirectoryFilename; const PlaceMark: TKMLPlaceMark; const M: TOutputFormatGIS);
var
  fp: TextFile;
begin
  AfficherMessage(Format('%s.ExporterUnPointKML: Lat: %.6f, Lon=%.6f, Lbl=%s - Filename: %s',
                        [ClassName, PlaceMark.Latitude, PlaceMark.Longitude,
                         PasStrToXMLStr(PlaceMark.Etiquette),
                         PasStrToXMLStr(FileName)]));
end;

///*****************************************************************************
// Interpréteur PascalScript

procedure TCdrCalculette.acPSExecuteExecute(Sender: TObject);
var
  i, n: Integer;
  CompilationOK, ResExec: Boolean;
  P: Cardinal;
  EWE: String;
begin
  // récupérer le script
  n := editPascalScript.Lines.Count;
  if (n = 0) then Exit;
  PSScriptDebugger1.Script.Clear;
  PSScriptDebugger1.Comp.OnBeforeOutput := psCompilerBeforeOutput;
  for i := 0 to n - 1 do  PSScriptDebugger1.Script.Add(editPascalScript.Lines[i]);


  CompilationOK := PSScriptDebugger1.Compile;
  for i:= 0 to PSScriptDebugger1.CompilerMessageCount - 1 do
    DispPSCompilerMsg(PSScriptDebugger1.CompilerMessages[i].MessageToString);
  if (CompilationOK) then
  begin
    //ResExec := PSScriptDebugger1.Execute;
    ResExec := (PSScriptDebugger1.Exec.RunScript and (PSScriptDebugger1.Exec.ExceptionCode = erNoError));
    if (not ResExec) then
    begin
      EWE := Format('Run-time error: %d - %s', [
                    PSScriptDebugger1.ExecErrorCode,
                    PSScriptDebugger1.ExecErrorToString
                    ]);
      lsbPSMessages.Items.Add(EWE);
    end;

  end
  else
  begin
    ShowMessage('Erreur dans le script');
    if (PSScriptDebugger1.CompilerMessageCount > 0) then
    begin
      for i:= 0 to PSScriptDebugger1.CompilerMessageCount - 1 do
          DispPSCompilerMsg(PSScriptDebugger1.CompilerErrorToStr(i));

    end;
    lsbPSMessages.ItemIndex := lsbPSMessages.Count - 1;
    // se positionner au droit de l'erreur
    editPascalScript.CaretY := PSScriptDebugger1.CompilerMessages[0].Row;
    editPascalScript.CaretX := PSScriptDebugger1.CompilerMessages[0].Col;
    editPascalScript.SetFocus;
  end;
end;

procedure TCdrCalculette.acPSStopExecute(Sender: TObject);
begin
  try
  // if (PSScriptDebugger1.Exec.) then PSScriptDebugger1.Stop;
    PSScriptDebugger1.Exec.Stop;
  except
    pass;
  end;
end;

//------------------------------------------------------------------------------
procedure TCdrCalculette.DispPSOutput(const Msg: string);
begin
  self.editPSOutPut.Lines.add(msg);
  self.editPSOutPut.CaretY := self.editPSOutPut.Lines.Count - 1;
  self.editPSOutPut.MoveCaretToVisibleArea;
end;

procedure TCdrCalculette.printf(const FMT: string; const Argv: array of const);
begin
  self.editPSOutPut.Lines.Add(Format(FMT, Argv));
end;



function TCdrCalculette.fopen(const Descr: integer; const QFileName: string; const Mode: Char): boolean;
begin
  result := false;
  try
    if (not InRange(Descr, Low(FpTextFiles), high(FpTextFiles))) then exit;
    AssignFile(FpTextFiles[Descr], QFileName);
    case Mode of
      'a': Append(FpTextFiles[Descr]);
      'r': Reset(FpTextFiles[Descr]);
      'w': ReWrite(FpTextFiles[Descr]);
    end;
    result := true;
  except

  end;
end;

//******************************************************************************



end.

