unit CadreGHTopoContext2D;
// Cadre de dessin 2D pour la nouvelle version des données compîlées
// 05/11/2013: Modes de représentation OK (secteurs, réseaux, expés)
// 28/11/2013: Gestion des dégradés dans les onglets
// 17/01/2014: Nouveau dialogue Distance
// 23/01/2014: Calcul altitude d'un point de surface si maillage MNT OK
// 14/04/2014: Implémentation du support des cotes "spéléo" (-268, -140)
// 04/08/2014: Nouvelle procédure de dessin des entrées
// 20/11/2014: Support des largeurs de trait pour les polygonales (lié aux visées en antenne)
// 20/11/2014: Les visées en antenne deviennent une catégorie d'éléments affichés
// 13/05/2015: Possibilité optionnelle de TBGRABitmap
// 28/05/2015: Surbrillance de la série courante
// 19/06/2015: Noms des entrées sont affichés
// 22/02/2015: Recherche de stations OK. Si trouvée, un marqueur est affiché
// 06/05/2016: La méthode de zoom et pan est unifiée et alignée sur celle de GHCaveDraw (AutoCAD-like)
// 14/06/2016: Ajout du mode de visu 'Type de visées'
// 07/04/2017: Suppression du cadre 'Filtres', trop complexe
// 21/04/2017: Optimisations de l'affichage par tests IsPointInRect() et IsSegmentInRect()
// 12/11/2017: La barre de statut/infos du viewer passe en haut. Motif: Le bas de la fenêtre est souvent masqué
// 20/09/2018: Fix de l'affichage des cotes: les visées radiantes étaient également cotées



{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  ToporobotClasses2012,
  UnitEntitesExtended,
  unitobjetserie,
  UnitClasseMaillage,
  math,
  types,
  dateutils,
  Graphics,
  Clipbrd,

  BGRABitmap,
  Common,
  CallDialogsStdVersion,
  unitUtilsComposants,
  unitdistox,
  Classes, SysUtils, FileUtil, curredit, Forms, Dialogs, Controls,
  ExtCtrls, StdCtrls, ActnList, Menus, Buttons;

type

  { TGHTopoContext2DA }

  TGHTopoContext2DA = class(TFrame)
    acIsovaleurSurZStation: TAction;
    acMetaFiltreCode: TAction;
    acMetaFiltreExpe: TAction;
    acMetaFiltreReseau: TAction;
    acMetaFiltreSerie: TAction;
    acMetaFiltreTopoAnnee: TAction;
    acMetaFiltreTopoDuJour: TAction;
    acMetaFiltreVueCourante: TAction;
    acMetaFiltreZone: TAction;
    acMetaFiltreSecteur: TAction;
    acAddSerieHere: TAction;
    acSetCurrentIdxSerieStation: TAction;
    acPanVue: TAction;
    acDistanceP2P: TAction;
    acMetaFiltreEntree: TAction;
    acDisplayVoisinageCurrStation: TAction;
    acCreerEntreeFromThisPoint: TAction;
    acAddserieEntreCesPoints: TAction;
    acLocaliserUneStation: TAction;
    acAddAnnotationHere: TAction;
    acSupprimerSerie: TAction;
    acSetCurrentSerie: TAction;
    acAddStationAtThisSerie: TAction;
    acZoomFenetre: TAction;
    acZoomTout: TAction;
    ActListCdrVue2D: TActionList;
    btnApplyMetaFiltre: TButton;
    btnHelpMetaFiltre: TButton;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    btnMoreInfos: TButton;
    btnCopyCoords: TButton;
    btnListeProfils: TButton;
    chkDrawMaillage: TCheckBox;
    btnCouleurCourbeNiveau: TColorButton;
    chkMetaFiltreActif: TCheckBox;
    editObservations: TEdit;
    editIDTerrain: TEdit;
    editQLongueur: TCurrencyEdit;
    editQAzimut: TCurrencyEdit;
    editQPente: TCurrencyEdit;
    editQLG: TCurrencyEdit;
    editQLD: TCurrencyEdit;
    editQHZ: TCurrencyEdit;
    editQHN: TCurrencyEdit;
    editIsoValeur: TCurrencyEdit;
    editMetaFiltre: TEdit;
    imgLstCtxtV2D: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lbNumSerieCouranteDistoX: TLabel;
    lbFiltres: TLabel;
    lbColorReseau: TStaticText;
    lbCouleurVisee: TStaticText;
    lbColorSecteur: TStaticText;
    lbColorSeance: TStaticText;
    lbCoordonnees: TStaticText;
    lbGCSMouse: TStaticText;
    lbInfos: TStaticText;
    lbMesures: TStaticText;
    lbAdditionalInfos: TStaticText;
    lbNomReseau: TStaticText;
    lbNomSecteur: TStaticText;
    lbModeTravail: TStaticText;
    lbStation: TStaticText;
    lbInfosExpe: TStaticText;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItem25: TMenuItem;
    MenuItem28: TMenuItem;
    MenuItem29: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem30: TMenuItem;
    MenuItem33: TMenuItem;
    MenuItem34: TMenuItem;
    MenuItem35: TMenuItem;
    MenuItem37: TMenuItem;
    mnuDistoX: TMenuItem;
    MenuItem36: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    mnuMetaFiltreStation: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    pnlQuickAddVisee: TPanel;
    pnlFiltres: TPanel;
    pnlFullInfos: TPanel;
    pnlMaillage: TPanel;
    PopUpCdrVue2D: TPopupMenu;
    sclDeplX: TScrollBar;
    sclDeplY: TScrollBar;
    lbCode: TStaticText;
    lbSerie: TStaticText;
    lbEntreeRattachement: TStaticText;
    lbCurrentIdxSerie: TStaticText;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    lbSerieCourante: TStaticText;
    lbMiscMaillage: TStaticText;
    lbLongViseesVisibles: TStaticText;
    Vue: TPaintBox;
    procedure acAddAnnotationHereExecute(Sender: TObject);
    procedure acAddserieEntreCesPointsExecute(Sender: TObject);
    procedure acAddSerieHereExecute(Sender: TObject);
    procedure acAddStationAtThisSerieExecute(Sender: TObject);
    procedure acAddViseeRadianteHereExecute(Sender: TObject);
    procedure acCreerEntreeFromThisPointExecute(Sender: TObject);
    procedure acDisplayVoisinageCurrStationExecute(Sender: TObject);
    procedure acDistanceP2PExecute(Sender: TObject);
    procedure acIsovaleurSurZStationExecute(Sender: TObject);
    procedure acLocaliserUneStationExecute(Sender: TObject);
    procedure acMetaFiltreCodeExecute(Sender: TObject);
    procedure acMetaFiltreEntreeExecute(Sender: TObject);
    procedure acMetaFiltreExpeExecute(Sender: TObject);
    procedure acMetaFiltreReseauExecute(Sender: TObject);
    procedure acMetaFiltreSecteurExecute(Sender: TObject);
    procedure acMetaFiltreSerieExecute(Sender: TObject);
    procedure acMetaFiltreTopoAnneeExecute(Sender: TObject);
    procedure acMetaFiltreTopoDuJourExecute(Sender: TObject);
    procedure acMetaFiltreVueCouranteExecute(Sender: TObject);
    procedure acMetaFiltreZoneExecute(Sender: TObject);
    procedure acPanVueExecute(Sender: TObject);
    procedure acSetCurrentIdxSerieStationExecute(Sender: TObject);
    procedure acSetCurrentSerieExecute(Sender: TObject);
    procedure acSupprimerSerieExecute(Sender: TObject);
    procedure acZoomFenetreExecute(Sender: TObject);
    procedure acZoomToutExecute(Sender: TObject);
    procedure btnApplyMetaFiltreClick(Sender: TObject);
    procedure btnCopyCoordsClick(Sender: TObject);
    procedure btnHelpMetaFiltreClick(Sender: TObject);
    procedure btnListeProfilsClick(Sender: TObject);
    //procedure btnApplyClick(Sender: TObject);
    procedure btnMoreInfosClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure chkDrawMaillageChange(Sender: TObject);
    procedure chkMetaFiltreActifChange(Sender: TObject);
    procedure lbInfosClick(Sender: TObject);

    procedure PopUpCdrVue2DPopup(Sender: TObject);
    procedure sclDeplXChange(Sender: TObject);
    procedure sclDeplYChange(Sender: TObject);
    procedure SetStationInfo(const S: TEntiteEtendue);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure VueMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure VueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure VueMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure VuePaint(Sender: TObject);
  private
    { private declarations }

    // callback pour actualisation de la liste des séries après un ajout sur le plan
    FProcActualiserAffichagesBDD: TProcOfObjectWithOneBoolParameter;
    // callback pour recalcul du réseau
    FProcPerformActionRecalcul: TProcedureOfObject;
    // callback pour transmission du contenu du Métafiltre
    FProcUseMetaFiltre: TProcedureOfObject;
    // sémaphore indiquant si le dessin est terminé
    FDessinEnCours: boolean;
    // interaction avec le document GHTopo
    FDocuTopo: TToporobotStructure2012;
    // surlignage de visées
    FModeSelectionListe : TModeSelectionListe;
    FDoHighLightCurrentItem: boolean; // mettre la série courante en surbrillance

    // BDD entités
    FBDDEntites: TBDDEntites;

    // entité courante
    FCurrentEntite: TEntiteEtendue;
    FCurrentInternalIdxEntite : integer;
    // marqueurs
    FCurrentMarker: TMarker;
    // drapeaux
    FCanDraw                : Boolean;


    // onglets
    FArrOngletsParams : TArrOngletsParams;
    FCurrIdxOnglet    : byte;

    // modes de travail
    FModesTravail : TModesTravail;
    FModeRepresentationGaleries: TModeRepresentationGaleries;
    // coordonnées souris
    //FSCRMouse: TPoint;
    FMyPos        : TPoint2Df;
    // gestion des zooms
    FAttenduSecondPoint       : boolean; // zoom, pan et ligne
    FZC1, FZC2    :   TPoint2Df;
    FZP1, FZP2    :   TPoint;

    FRegionXMini  :double;
    FRegionXMaxi  :double;
    FRegionYMini  :double;
    FRegionYMaxi  :double;
    // paramètres de vues
    FRappHLVue      : double;
    FRappScrReal    : double;
    FInvRappScrReal : double;

    // Maillages

    FMyMaillage     : TMaillage;
    FMaillageReady  : boolean;
    // Spécifique DistoX
    //******************************************************
    // Pointeur sur le DistoX
    FPtrDistoX                        : TPilotageDistoX;

    FCurrentNoSerieForDistoX          : TNumeroSerie;     // série courante recevant les data du DistoX
    FProcTraiterUneMesureDeDistoX     : TProcTransmitMesureVisee;


    //--------------------------------------
    // Méthodes
    //--------------------------------------
    // vue
    (*
    procedure AjouterUneViseeALaSerieCourante(const QIndexSerie: integer;
                                              const QLong, QAz, QPente, QG, QD, QH, QN: double;
                                              const QIDTerrain, QCommentaire: string);
    //*)


    function  AjouterUneAntenneALaStationCourante(const QLongueur, QAzimut, QPente: double; const DoRefreshPlan: boolean): boolean;
    procedure AppliquerLeFiltre(const FF: string);
    procedure DisplayFullInfosStation(const S: TEntiteEtendue);
    procedure DisplayQuickAddVisee(const OS: TObjSerie);
    procedure DrawTexteVectoriel(const Position: TPoint2Df;
                                 const HauteurChar, LargeurChar: double;
                                 const DistToLigneAttache: double;
                                 const Alignment: Byte;
                                 const OrientationInRadians: double; const Texte: string);


    // stations
    // conversion de coordonnées écran<>générales
    function GetCoordsMonde(const PP: TPoint): TPoint2Df;
    function GetCoordsPlan(const PM: TPoint2Df): TPoint;
    function GetRYMaxi(): double;
    // onglets

    function  GetNomOnglet(): string;
    procedure InitCaptions();
    procedure QSetPen(const QPM: TPenMode; const QPS: TPenStyle; const QPW: integer; const QPC: TColor);
    procedure Rearmer();
    function ReplaceEntitesCoords(const QE: TEntiteEtendue; const QQX, QQY, QQZ: double): TEntiteEtendue;

    procedure SetMarker(const M: TMarker);
    procedure SetNomOnglet(const S: string);
    function  GetBackGrdColorOnglet(const QIdxOnglet: integer): TColor;
    procedure SetBackGrdColorOnglet(const QIdxOnglet: integer; const Col: TColor);
    function  GetLargeurViseesOnglet(const QIdxOnglet: integer): integer;
    procedure SetLargeurViseesOnglet(const QIdxOnglet: integer; const qL: integer);


    // quadrillage
    function  GetQuadrillageColor(): TColor;
    function  GetQuadrillageSpc(): double;
    procedure SetQuadrillageColor(const Col: TColor);
    procedure SetQuadrillageSpc(const Spacing: double);

    procedure QDrawTexte(const Cnv: TBGRABitmap; const X, Y: double; const Attr: TTexteAttributs; const Texte: string);
    // provisoire
    {
    procedure AjouterUneVisee(const QStation: TToporobotStation;
                              const QSecteur, QCode, QExpe: integer;
                              const QType: TTypeDeVisee;
                              const QLong, QAz, QPente, QG, QD, QH, QN: double);
    //*}

    procedure TraiterViseeRecueDuDistoX(const V: TMesureViseeDistoX);
  public
    { public declarations }
    property  CanDraw: boolean         read FCanDraw;

    // assignation des callbacks
    procedure SetProcUseMetaFiltre(const P: TProcedureOfObject);


    // affectation des callbacks
    procedure DeplacerVue(const QOffsetX, QOffsetY: double);
    procedure DeplacerVueInt(const DepX, DepY: integer);

    procedure SetViewLimits(const X1, Y1, X2, Y2: double; const DebugTag: string); overload;
    procedure SetViewLimits(const QCoinBasGauche, QCointHautDroit: TPoint2Df; const DebugTag: string); overload;



    function  GetViewLimits(const QIdxOnglet: integer; out QCoinBasGauche, QCointHautDroit: TPoint2Df): boolean;

    procedure CentrerVueSurPointXY(const QX, QY: double; const DoMarker: boolean; const LbMarker: string);

    procedure ZoomFact(const Fact: double);
    procedure Metafiltrer(const lbFiltres: string);
    //function  GetCanDraw(): Boolean;
    procedure ResetVue();
    procedure Finaliser();
    function  Initialiser(const QDT: TToporobotStructure2012;
                          const QBDD: TBDDEntites;
                          const QPtrPiloteDistoX: TPilotageDistoX;
                          const QMaillage: TMaillage;
                          const QProcUseMetaFiltre: TProcedureOfObject;
                          const QProcRefresh: TProcOfObjectWithOneBoolParameter;
                          const QProcRecalculerReseau: TProcedureOfObject): boolean;
    // série courante
    procedure SetCurrentNumeroSerieDistoX(const N: TNumeroSerie);
    procedure SetCurrentNumeroSerieAndPoint(const S: TNumeroSerie; const P: integer);
    procedure SetCurrentNumeroCode(const S: TNumeroCode);
    procedure SetCurrentNumeroExpe(const S: TNumeroExpe);
    procedure SetCurrentNumeroReseau(const S: integer);
    procedure SetCurrentNumeroSecteur(const S: integer);



    // station courante
    function  GetCurrentEntite(): TEntiteEtendue;
    procedure SetCurrentEntite(const E: TEntiteEtendue);
    // flags
    procedure SetElementsDrawn(const ED: TElementsDrawn);
    function  GetElementsDrawn: TElementsDrawn;
    procedure SetModeTravail(const MT: TModesTravail);
    // dessin
    procedure RedessinEcran(const DoRecalcDegrades: boolean; const DoExportInFile: boolean = false; const QFileName: string = '');
    // Mode de dessin: réseaux, secteurs, expés
    procedure SetModeRepresentation(const MR: TModeRepresentationGaleries);
    function  GetModeRepresentation(): TModeRepresentationGaleries;
    // gestion des onglets
    procedure SetCurrentIdxOnglet(const Idx: byte);
    function  GetCurrentOngletIdx(): integer;
    function  GetNbOngletsVues(): integer;
    function  GetOngletByIndex(const Idx: integer): TVue2DParams;
    function  GetCurrentOnglet(): TVue2DParams;
    procedure PutOngletByIndex(const Idx: integer;
                               const O: TVue2DParams;
                               const DoRedrawVue: boolean);

    procedure SetHighLightCurrentItem(const B: boolean);

    // chargement du maillage
    function ChargerMaillage(const QFileName: string): boolean;
    function ChargerMaillageHGT(const QFileName: string): boolean;
    function IsMaillageReady(): boolean;
    function GetMaillage(): TMaillage;
    function GetDocuTopo(): TToporobotStructure2012;


    // ajout
    function AjouterUneViseeALaSerieCourante(const QTypeVisee: TTypeDeVisee;
                                               const QLongueur, QAzimut, QPente, QLG, QLD, QHZ, QHN: double;
                                               const QIDLitteral, QCommentaire: string;
                                               const DoRefreshPlan: boolean): boolean;
    // extraire le pointeur sur la BDD entités
    function GetPtrBDDEntites(): TBDDEntites;
        // lbFiltres
    procedure ApplyMetaFiltre(const B: boolean; const QFiltre: string);

    procedure SetModeSelectionListe(const M: TModeSelectionListe);
    // localiser une station
    procedure LocaliserUneStation();

    // événement 'filtre changé'
    // créer un timelapse
    procedure CreerTimeLapse(const QDirectory: TStringDirectoryFilename);


    // pointeur sur le DistoX
    procedure setPtrDistoX(const D: TPilotageDistoX);



  end;

implementation

const
  FONT_HEIGHT_COTATION   = 12;
  FONT_HEIGHT_IDSTATIONS = 12;
  FONT_HEIGHT_NOEUDS     = 12;
  FONT_HEIGHT_ENTRANCES  = 12;
  FONT_HEIGHT_QUADRILLES = 12;





{$R *.lfm}
// provisoire ; à jeter après tests
function QGenererCodeTerrain(const NN: integer): string;
var
 c: Integer;
 Q: Char;
begin
 Result := '';
 for c := 0 to NN - 1 do
 begin
   Q := chr(65 + Trunc(26 * Random));
   Result += Q;
 end;
end;
procedure TGHTopoContext2DA.DisplayQuickAddVisee(const OS: TObjSerie);
var
  PM: TPoint2Df;
  R1, R2, DummyRect: TRect;
  PP: TPoint;
begin
  PM := MakeTPoint2Df(FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y);
  PP := GetCoordsPlan(PM);
  pnlQuickAddVisee.Left := PP.X;
  pnlQuickAddVisee.Top  := PP.Y;
  if (PP.X + pnlQuickAddVisee.Width  > Vue.Width)  then pnlQuickAddVisee.Left := Vue.Width   - pnlQuickAddVisee.Width;
  if (PP.y + pnlQuickAddVisee.Height > Vue.Height) then pnlQuickAddVisee.Top  := Vue.Height  - pnlQuickAddVisee.Height;



  lbSerieCourante.Caption := format('Station %d.%d', [OS.GetNumeroDeSerie(), OS.GetNbVisees()]);

  pnlQuickAddVisee.Visible := True;
  editIDTerrain.SetFocus;
end;

function TGHTopoContext2DA.Initialiser(const QDT: TToporobotStructure2012;
                                       const QBDD: TBDDEntites;
                                       const QPtrPiloteDistoX: TPilotageDistoX;
                                       const QMaillage: TMaillage;
                                       const QProcUseMetaFiltre: TProcedureOfObject;
                                       const QProcRefresh: TProcOfObjectWithOneBoolParameter;
                                       const QProcRecalculerReseau: TProcedureOfObject): boolean;
var
  C1, C2: TPoint3Df;
  NoOnglet: Integer;
  MyOnglet: TVue2DParams;
  DbgTag: String;
begin
  Result := false;
  AfficherMemoryUsage();
  pnlQuickAddVisee.Visible := false;
  pnlMaillage.Visible      := false;
  // DistoX
  FPtrDistoX := QPtrPiloteDistoX;
  mnuDistoX.Visible := false;


  FProcUseMetaFiltre                   := QProcUseMetaFiltre;
  FProcActualiserAffichagesBDD         := QProcRefresh;
  FProcPerformActionRecalcul           := QProcRecalculerReseau;

  FCurrentMarker := MakeTMarker(false, 0.0, 0.0, clAqua, '');
  Panel1.DoubleBuffered := True;
  FDessinEnCours := false;
  AfficherMessage(Format('%s.Initialise()', [ClassName]));
  FBDDEntites          := QBDD;
  FDocuTopo            := QDT;
  FModeSelectionListe  := mslSERIE;

  SetCurrentNumeroSerieAndPoint(1, 0);

  SetCurrentNumeroCode(1);
  SetCurrentNumeroExpe(1);
  SetCurrentNumeroReseau(0);
  SetCurrentNumeroSecteur(0);
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  AfficherMessageErreur(DbgTag);
  FDoHighLightCurrentItem := false;
  FMyMaillage        := QMaillage;

  FModesTravail    := mtREADY;
  FModeRepresentationGaleries := rgSEANCES;
  Result           := False;
  FCurrIdxOnglet   := 0;
  FCanDraw         := False;
  SetModeTravail(mtREADY);
  //FMaillageLoaded  := false;
  FMaillageReady   := False;
  AfficherMessageErreur(Format('%d entities', [FBDDEntites.GetNbEntitesVisees()]));
  SetCurrentEntite(FBDDEntites.GetEntiteVisee(0));
  FCurrentInternalIdxEntite := 0;



  FBDDEntites.SetMinMax(false);
  C1 := FBDDEntites.GetCoinBasGauche();
  C2 := FBDDEntites.GetCoinHautDroit();
  // trier par profondeurs
  FBDDEntites.SortByDepth();
  //SetElementsDrawn([edPolygonals, edQuadrilles, edWalls, edFillGalerie]);
  //SetElementsDrawn([edQuadrilles, edPolygonals, edStations, edIDStations, edCrossSections, edANTENNES, edANNOTATIONS]);
  SetElementsDrawn([edQuadrilles, edPolygonals, edCrossSections, edANTENNES, edANNOTATIONS]);
  SetNomOnglet(rsCDR_VUE2D_TAB_VUE_INITIALE);

  SetQuadrillageColor(clSilver);
  SetQuadrillageSpc(ProposerEquidistanceDef(C1, C2, 100.00));
  SetBackGrdColorOnglet(FCurrIdxOnglet,clWhite);
  SetLargeurViseesOnglet(FCurrIdxOnglet, 2);
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  SetViewLimits(C1.X, C1.Y, C2.X, C2.Y, DbgTag); // renseigne aussi l'onglet courant


  ApplyMetaFiltre(True, '');                   // renseigne aussi l'onglet courant
  ResetVue();
  // initialiser le tableau d'onglets avec les paramètres de l'onglet 0
  MyOnglet := FArrOngletsParams[0];
  MyOnglet.ongQdrType := qtGRID;
  MyOnglet.ongDegradeStart := clGreen;
  MyOnglet.ongDegradeStop  := clMaroon;
  MyOnglet.ongModeRepresentation := rgSEANCES;
  MyOnglet.ongFillOpacite := 128;
  MyOnglet.ongName := Format(rsCDR_VUE2D_TAB_VUE, [0]);
  PutOngletByIndex(0, MyOnglet, True);

  for NoOnglet := 1 to High(FArrOngletsParams) do
  begin
    MyOnglet.ongName := Format(rsCDR_VUE2D_TAB_VUE, [NoOnglet]);
    PutOngletByIndex(NoOnglet, MyOnglet, False);
  end;
  // série courante pour les acquisitions DistoX
  SetCurrentNumeroSerieDistoX(1);
  // OK
  FCanDraw := True;
  Result   := True;
  // Initialisation de l'interface
  InitCaptions();
  // Panneau Maillage inhibé si le maillage n'est pas assigné
  pnlMaillage.Visible := IIF(FMyMaillage.IsValidMaillage(), true, false);
  // redessiner
  Vue.Invalidate;
  // afficher mémoire utilisée
  AfficherMemoryUsage();
 //except
 //end;
end;

procedure TGHTopoContext2DA.SetCurrentNumeroSerieDistoX(const N: TNumeroSerie);
var
  SR: TObjSerie;
  QIdxSerie: integer;
begin
  if (FDocuTopo.GetSerieByNumeroSerie(N, SR, QIdxSerie)) then
    FCurrentNoSerieForDistoX := SR.GetNumeroDeSerie()
  else
    FCurrentNoSerieForDistoX := 1;
  lbNumSerieCouranteDistoX.Caption := Format('Série: %d - %s', [FCurrentNoSerieForDistoX, SR.GetNomSerie()]);
end;

procedure TGHTopoContext2DA.InitCaptions();
  procedure S666(const ACDC: TAction; const QCaption: string);
  begin
    ACDC.Caption := GetResourceString(QCaption);
    ACDC.Hint    := GetResourceString(QCaption);
  end;
begin
  S666(acZoomTout         , rsCDR_VUE2D_AC_ZOOM_ALL);
  S666(acZoomFenetre      , rsCDR_VUE2D_AC_ZOOM_WINDOW);
  S666(acPanVue           , rsCDR_VUE2D_AC_PAN_VUE);
  S666(acDistanceP2P      , rsCDR_VUE2D_AC_PAN_VUE);
  S666(acAddSerieHere     , rsCDR_VUE2D_ADD_SERIE_HERE);
  S666(acDisplayVoisinageCurrStation     , rsCDR_VUE2D_DISP_STATIONS_VOISINES);
  S666(acAddserieEntreCesPoints          , rsCDR_VUE2D_ADDSERIE_BETWEEN_STATION);
  S666(acIsovaleurSurZStation            , rsCDR_VUE2D_ISOVALEUR_FROM_Z);
  //S666(acAddViseeRadianteHere            , rsCDR_VUE2D_ADD_VISEE_RADIANTE_HERE);
  S666(acLocaliserUneStation             , rsCDR_VUE2D_LOCALISER_STATION);
  S666(acAddAnnotationHere               , rsCDR_VUE2D_ADD_ANNOTATION_HERE);


  {$IFDEF GROS_MINET}  // fonctionnalités désactivées pour Gros_Minet
    acCreerEntreeFromThisPoint.Visible     := false;
    acAddSerieHere.Visible                 := false;
    acAddViseeRadianteHere.Visible         := false;
    acAddStationHere.Visible               := false;
    acAddserieEntreCesPoints.Visible       := false;
    acIsovaleurSurZStation.Visible         := False;
  {$ENDIF}
end;

procedure TGHTopoContext2DA.SetCurrentNumeroSerieAndPoint(const S: TNumeroSerie; const P: integer);
begin
  try
    FDocuTopo.SetCurrentNumeroSerie(S);
    FDocuTopo.SetCurrentIndexPointOfSerie(P);
    lbCurrentIdxSerie.Caption := Format('Ser %d Pt %d', [FDocuTopo.GetCurrentNumeroSerie(), FDocuTopo.GetCurrentIndexPointOfSerie()]);
    Vue.Invalidate;
    AfficherMessageErreur('OK');
  except
    AfficherMessageErreur(Format('SetCurrentNumeroSerieAndPoint: %d of %d - %d', [
                                  FDocuTopo.GetCurrentNumeroSerie(),
                                  FDocuTopo.GetNbSeries(),
                                  FDocuTopo.GetCurrentIndexPointOfSerie()]));
  end;
end;

procedure TGHTopoContext2DA.SetCurrentNumeroCode(const S: TNumeroCode);
begin
  FDocuTopo.SetCurrentNumeroCode(S);
  Vue.Invalidate;
end;
procedure TGHTopoContext2DA.SetCurrentNumeroExpe(const S: TNumeroExpe);
begin
  FDocuTopo.SetCurrentNumeroExpe(S);
  Vue.Invalidate;
end;

procedure TGHTopoContext2DA.SetCurrentNumeroReseau(const S: integer);
begin
  FDocuTopo.SetCurrentNumeroReseau(S);
  Vue.Invalidate;
end;

procedure TGHTopoContext2DA.SetCurrentNumeroSecteur(const S: integer);
begin
  FDocuTopo.SetCurrentNumeroReseau(S);
  Vue.Invalidate;
end;




function TGHTopoContext2DA.GetCurrentEntite(): TEntiteEtendue;
begin
  Result := FCurrentEntite;
end;
procedure TGHTopoContext2DA.SetCurrentEntite(const E: TEntiteEtendue);
begin
  FCurrentEntite := E;
  SetCurrentNumeroSerieAndPoint(E.Entite_Serie, E.Entite_Station);
  SetStationInfo(E);
end;
//******************************************************************************
// Un peu bancal mais au moins, on a tout le réseau dans la vue ;)
// Repris du ResetVue() de GHCaveDraw
procedure TGHTopoContext2DA.ResetVue();
var
  cnMini: TPoint3Df;
  cnMaxi: TPoint3Df;
  M, dx, dy          : Extended;
  PM0, PM1: TPoint2Df;
  //AspectRatio        : Extended;
  //ratioX, ratioY     , QE, EtendueC, r,
  L: Extended;
  PP: TPoint;
begin
  if (not FCanDraw) then Exit;
  cnMini := FBDDEntites.GetCoinBasGauche();
  cnMaxi := FBDDEntites.GetCoinHautDroit();

  PP := MakeTPoint(Vue.Width, Vue.Height);
  PM1 := GetCoordsMonde(PP);
  PP := MakeTPoint(0, 0);
  PM0 := GetCoordsMonde(PP);

  DX := cnMaxi.X-cnMini.X;
  DY := cnMaxi.Y-cnMini.Y;
  M  := 0.02 * Hypot(DX, DY);
  L  := IIF(DX > DY, DX, DY);
  DX := PM1.X - PM1.X;
  DY := PM1.Y - PM0.Y;
  SetViewLimits(cnMini.X-M, cnMini.Y-M, cnMini.X + L + M, cnMini.Y + L + M, '');
  Vue.Invalidate;
end;

procedure TGHTopoContext2DA.SetStationInfo(const S: TEntiteEtendue);
var
  beuh: TPoint3Df;
  EWE: String;
begin
  beuh := FBDDEntites.GetDeltaXYZFromPositionSt0(S);
  lbInfos.Color := IIF(S.IsPOI, clYellow, clDefault);
  EWE := MakeLibelleStationTopo(S);
  lbInfos.Caption := Trim(Format('%s (Alt = %.2f; Z = %.2f ) %s', [EWE, S.Une_Station_2_Z, beuh.Z, S.oCommentaires]));
end;

procedure TGHTopoContext2DA.SpeedButton1Click(Sender: TObject);
begin
  pnlQuickAddVisee.Visible := false;
end;

procedure TGHTopoContext2DA.SpeedButton2Click(Sender: TObject);
var
  QL: Extended;
begin
  pnlQuickAddVisee.Visible := false;
  QL := editQLongueur.Value;
  if (QL < 0.001) then Exit;
  AjouterUneViseeALaSerieCourante(tgDEFAULT,
                                      QL,
                                      editQAzimut.Value,
                                      editQPente.Value,
                                      editQLG.Value,
                                      editQLD.Value,
                                      editQHZ.Value,
                                      editqhn.Value,
                                      trim(editIDTerrain.Text),
                                      trim(editObservations.Text),
                                      True);

end;


procedure TGHTopoContext2DA.chkDrawMaillageChange(Sender: TObject);
begin
  FMaillageReady := (chkDrawMaillage.Checked and FMyMaillage.IsValidMaillage());
  Vue.Invalidate; //RedessinEcran;
end;

procedure TGHTopoContext2DA.chkMetaFiltreActifChange(Sender: TObject);
begin

end;




procedure TGHTopoContext2DA.PopUpCdrVue2DPopup(Sender: TObject);
  procedure S666(const ACDC: TAction; const QCaption: string);
  begin
    ACDC.Caption := GetResourceString(QCaption);
    ACDC.Hint    := GetResourceString(QCaption);
  end;

begin
  mnuMetaFiltreStation.Caption := Format(GetResourcestring(rsCDR_VUE2D_MNU_FILTRES_STATION),
                                          [FCurrentEntite.Entite_Serie,
                                           FCurrentEntite.Entite_Station,
                                           FCurrentEntite.oIDLitteral]);
  // titres du sous-menu
  S666(acZoomTout                 , rsCDR_VUE2D_AC_ZOOM_ALL);
  S666(acZoomFenetre              , rsCDR_VUE2D_AC_ZOOM_WINDOW);
  S666(acPanVue                   , rsCDR_VUE2D_AC_PAN_VUE);
  S666(acDistanceP2P              , rsCDR_VUE2D_AC_DISTANCE_BT_STATIONS);
  S666(acMetaFiltreSerie          , Format(rsCDR_VUE2D_AC_METAFILTRE_SERIE        , [FCurrentEntite.Entite_Serie]));
  S666(acMetaFiltreTopoDuJour     , Format(rsCDR_VUE2D_AC_METAFILTRE_TOPO_DU_JOUR , [DateToStr(FCurrentEntite.DateLeve)]));
  S666(acMetaFiltreReseau         , Format(rsCDR_VUE2D_AC_METAFILTRE_RESEAU       , [FCurrentEntite.eReseau]));
  S666(acMetaFiltreEntree         , Format(rsCDR_VUE2D_AC_METAFILTRE_ENTRANCE     , [FCurrentEntite.eReseau]));

  S666(acMetaFiltreSecteur        , Format(rsCDR_VUE2D_AC_METAFILTRE_SECTEUR      , [FCurrentEntite.eSecteur]));

  S666(acMetaFiltreCode           , Format(rsCDR_VUE2D_AC_METAFILTRE_CODE         , [FCurrentEntite.eCode]));
  S666(acMetaFiltreExpe           , Format(rsCDR_VUE2D_AC_METAFILTRE_EXPE         , [FCurrentEntite.eExpe]));
  S666(acMetaFiltreTopoAnnee      , Format(rsCDR_VUE2D_AC_METAFILTRE_YEAR         , [YearOf(FCurrentEntite.DateLeve)]));
  S666(acMetaFiltreVueCourante    , rsCDR_VUE2D_AC_METAFILTRE_CURRENT_VIEW);
  S666(acIsovaleurSurZStation     , Format(rsCDR_VUE2D_AC_ISOVALEUR_Z, [FCurrentEntite.Une_Station_2_Z]));
end;

procedure TGHTopoContext2DA.sclDeplXChange(Sender: TObject);
var
  WU: Double;
begin
  WU := 0.05 * (FRegionXMaxi - FRegionXMini);
  DeplacerVue(-Sign(sclDeplX.Position) * WU, 0.00);
  vue.Invalidate;
  sclDeplX.Position:=0;
end;

procedure TGHTopoContext2DA.sclDeplYChange(Sender: TObject);
var
  WU: Double;
begin
  WU := 0.05 * (FRegionYMaxi - FRegionYMini);
  DeplacerVue(0.00, WU * sign(sclDeplY.Position));
  vue.Invalidate;
  sclDeplY.Position:=0;
end;


procedure TGHTopoContext2DA.Button2Click(Sender: TObject);
begin
  Vue.Invalidate; // RedessinEcran;
end;

procedure TGHTopoContext2DA.acMetaFiltreSerieExecute(Sender: TObject);
var
  FF: String;
begin
  FF := Format('%s=%d',[rsMETAFILTRE_SERIE, FCurrentEntite.Entite_Serie]);
  ApplyMetaFiltre(True, FF);
  vue.Invalidate; //RedessinEcran;
end;

procedure TGHTopoContext2DA.acMetaFiltreTopoAnneeExecute(Sender: TObject);
var
  YY,MM,DD: word;
  S1, S2, FF: string;
begin
  SafeDecodeDate(FCurrentEntite.DateLeve, YY, MM, DD);
  S1:=DateToStr(GetSecuredDate(YY, 01, 01));
  S2:=DateToStr(GetSecuredDate(YY, 12, 31));
  FF := Format('%s=(%s, %s)',[rsMETAFILTRE_DATE, S1, S2]);
  ApplyMetaFiltre(True, FF);
end;

procedure TGHTopoContext2DA.acMetaFiltreTopoDuJourExecute(Sender: TObject);
var
  YYYY, MM, DD: Word;
  S1, FF: String;
begin
  SafeDecodeDate(FCurrentEntite.DateLeve, YYYY, MM, DD);
  S1 := DateToStr(GetSecuredDate(YYYY, MM, DD));
  FF := Format('%s=%s',[rsMETAFILTRE_DATE, S1]);
  ApplyMetaFiltre(True, FF);
end;

procedure TGHTopoContext2DA.acMetaFiltreVueCouranteExecute(Sender: TObject);
var
  PP: TPoint;
  FC1, FC2: TPoint2Df;
  FF: String;
begin
  PP.X := 0;
  PP.Y := Vue.Height;
  FC1 := GetCoordsMonde(PP);
  PP.X := Vue.Width;
  PP.Y := 0;
  FC2 := GetCoordsMonde(PP);
  FF := Format('COORD_X=(%.2f, %.2f) & COORD_Y=(%.2f, %.2f)', [FC1.X, FC2.X, FC1.Y, FC2.Y]);
  ApplyMetaFiltre(True, FF);
  Vue.Invalidate; //RedessinEcran;
end;

procedure TGHTopoContext2DA.acMetaFiltreZoneExecute(Sender: TObject);
begin
  SetModeTravail(mtMETAFILTRE_ZONE_PREMIER_COIN);
end;

procedure TGHTopoContext2DA.acPanVueExecute(Sender: TObject);
begin
  self.SetModeTravail(mtPAN_PREMIER_POINT);
end;

procedure TGHTopoContext2DA.acSetCurrentIdxSerieStationExecute(Sender: TObject);
begin
  SetCurrentNumeroSerieAndPoint(FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station);
  vue.Invalidate;
end;

procedure TGHTopoContext2DA.acSetCurrentSerieExecute(Sender: TObject);
begin
  SetCurrentNumeroSerieDistoX(FCurrentEntite.Entite_Serie);
end;

procedure TGHTopoContext2DA.acSupprimerSerieExecute(Sender: TObject);
var
  MyNumeroSerie: TNumeroSerie;
  MySerie: TObjSerie;
  QInternaIdxSerie: integer;
begin
  MyNumeroSerie := self.FCurrentEntite.Entite_Serie;
  if (FDocuTopo.GetSerieByNumeroSerie(MyNumeroSerie, MySerie, QInternaIdxSerie)) then
  begin
    if (GHTopoQuestionOuiNon(Format('Supprimer %d: %s', [MySerie.GetNumeroDeSerie(), MySerie.GetNomSerie()]))) then
    begin
      FDocuTopo.RemoveSerie(QInternaIdxSerie);
    end;
  end
  else
  begin
    ShowMessage(GetResourceString(rsMATCHNOTFOUND));
  end;
end;

procedure TGHTopoContext2DA.acZoomFenetreExecute(Sender: TObject);
begin
  Self.SetModeTravail(mtZOOM_PREMIER_COIN);
end;

procedure TGHTopoContext2DA.acZoomToutExecute(Sender: TObject);
begin
  self.ResetVue();
end;

procedure TGHTopoContext2DA.btnMoreInfosClick(Sender: TObject);
begin
  AfficherInfosQRCodeStation(FBDDEntites, FCurrentEntite);
end;

procedure TGHTopoContext2DA.Button1Click(Sender: TObject);
var
  EWE: TCaption;
begin
  EWE := editMetaFiltre.Text;
  if (DisplayAndEditFiltres(FDocuTopo, EWE)) then
  begin
    editMetaFiltre.Text := EWE;
    ApplyMetaFiltre(chkMetaFiltreActif.Checked, trim(editMetaFiltre.Text));
  end;
end;

procedure TGHTopoContext2DA.acIsovaleurSurZStationExecute(Sender: TObject);
begin
  try
    if (FMyMaillage.IsValidMaillage() and FMaillageReady) then
    begin
    //FDoMaillageTrace := True;
      editIsoValeur.Value := FCurrentEntite.Une_Station_2_Z;
      vue.Invalidate; // RedessinEcran;
    end;
  except
    //FDoMaillageTrace := False;
  end;
end;

procedure TGHTopoContext2DA.acLocaliserUneStationExecute(Sender: TObject);
begin
  LocaliserUneStation();
end;

procedure TGHTopoContext2DA.acAddAnnotationHereExecute(Sender: TObject);
var
  AN: TAnnotation;
  EWE: TEntiteEtendue;
  QIdx: integer;
begin
  if (FBDDEntites.GetEntiteViseeFromXY(false, FMyPos.X, FMyPos.Y, EWE, QIdx)) then
  begin
    AN.Basepoint.eSerie := EWE.Entite_Serie;
    AN.Basepoint.ePoint := EWE.Entite_Station;
    AN.Offset := MakeTPoint3Df(FMyPos.X - EWE.Une_Station_2_X,
                               FMyPos.Y - EWE.Une_Station_2_Y,
                               0.00);
    AN.PosPtDeBase := 0;
    AN.Texte := '';
  end;
   if (EditerAnnotation(FDocuTopo, -1, AN)) then
   begin
     FDocuTopo.AddAnnotation(AN);
     Vue.Invalidate;
   end;
end;

procedure TGHTopoContext2DA.acAddserieEntreCesPointsExecute(Sender: TObject);
begin
   self.SetModeTravail(mtADD_SERIE_PREMIER_POINT);
end;

procedure TGHTopoContext2DA.acAddSerieHereExecute(Sender: TObject);
var
  MySerie: TObjSerie;
  QCBG, QCHD: TPoint2Df;
  QParamOnglet: TVue2DParams;
  FCO: Byte;
  QValue, DbgTag, WU: String;
  n: TNumeroSerie;
  QMargeX, QMargeY: double;
  EWE: TEntiteEtendue;
begin
  n := 1 + FDocuTopo.getMaxIdxSerie();
  QValue := Format('%d', [n]);
  if (InputQuery(Format('Nouvelle série depuis la station %d.%d', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station]),
                 'Numéro de série', QValue)) then
  begin
    n := StrToIntDef(Trim(QValue), 1 + FDocuTopo.getMaxIdxSerie());
    if (FDocuTopo.ExistsSerie(n)) then
    begin
      WU := Format(rsNUM_SERIE_ALREADY_ATTRIBUTED, [n]);
      if (not GHTopoQuestionOuiNon(GetResourceString(WU + ' - ' + rsALLOW_ATTRIBUTE_NUM_SERIE))) then Exit;
      n := -1;
    end;
    // on récupère les paramètres de vue et d'onglet
    FCO := FCurrIdxOnglet;
    QParamOnglet := GetOngletByIndex(FCO);
    GetViewLimits(FCurrIdxOnglet, QCBG, QCHD);
    QMargeX := 0.50 * (QCHD.X - QCBG.X);
    QMargeY := 0.50 * (QCHD.Y - QCBG.Y);
    if (FDocuTopo.CreateNewSerie(FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station,
                                 n,
                                 Format('Série ajoutée par pointage en %d.%d', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station]))) then
    begin
      // ajout d'une visée
      MySerie := FDocuTopo.GetLastSerie();
      Myserie.AddVisee(1, FCurrentEntite.eSecteur,
                       FCurrentEntite.eCode, FCurrentEntite.eExpe,
                       tgDEFAULT,
                       0.05, 45, -45, 0.0, 0.0, 0.0, 0.0,
                       '',
                       Format('Added during acAddSerieHereExecute(%s)', [Sender.ClassName]));
      Myserie.SetNoPointArr(-1);
      if (assigned(FProcPerformActionRecalcul))   then FProcPerformActionRecalcul();
      //on récupère la station nouvellement créée
      MySerie := FDocuTopo.GetLastSerie();
      FBDDEntites.GetEntiteViseeFromSerSt(MySerie.GetNumeroDeSerie(), 1, FCurrentEntite);
      showmessagefmt('%d.%d - %.2f, %.2f', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station, FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y]);
      ResetVue();
      SetViewLimits(FCurrentEntite.Une_Station_2_X - QMargeX, FCurrentEntite.Une_Station_2_Y - QMargeY,
                    FCurrentEntite.Une_Station_2_X + QMargeX, FCurrentEntite.Une_Station_2_Y + QMargeY,
                    'Centrage');
      //SetViewLimits(QCBG, QCHD, 'Centrage');
      FCurrIdxOnglet := FCO;
      PutOngletByIndex(FCO, QParamOnglet, false);
      DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
      CentrerVueSurPointXY(FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y, true, 'Nouvelle série ici');
      FBDDEntites.GetEntiteViseeFromSerSt(MySerie.GetNumeroDeSerie(), 1, EWE);
      SetCurrentEntite(EWE);
      // pour ajout de station depuis DistoX ou in-situ via le mini-formulaire
      SetCurrentNumeroSerieDistoX(MySerie.GetNumeroDeSerie());
    end
    else
    begin
      ShowMessage(GetResourceString(rsFAILED_TO_ADD_SERIE));
    end;
  end;
end;

procedure TGHTopoContext2DA.acAddStationAtThisSerieExecute(Sender: TObject);
var
  SR: TObjSerie;
  QInternalIdx: integer;
begin
  if (FDocuTopo.GetSerieByNumeroSerie(FCurrentEntite.Entite_Serie, SR, QInternalIdx)) then
  begin
    SetCurrentNumeroSerieDistoX(SR.GetNumeroDeSerie());
    DisplayQuickAddVisee(SR);
  end;
end;

function TGHTopoContext2DA.ReplaceEntitesCoords(const QE: TEntiteEtendue; const QQX, QQY, QQZ: double): TEntiteEtendue;
begin
  Result := QE;
  Result.Une_Station_1_X := QE.Une_Station_2_X;
  Result.Une_Station_1_Y := QE.Une_Station_2_Y;
  Result.Une_Station_1_Z := QE.Une_Station_2_Z;

  Result.Une_Station_2_X := Result.Une_Station_1_X + QQX;
  Result.Une_Station_2_Y := Result.Une_Station_1_Y + QQY;
  Result.Une_Station_2_Z := Result.Une_Station_1_Z + QQZ;

  Result.X1PD := QE.Une_Station_1_X;
  Result.Y1PD := QE.Une_Station_1_Y;
  Result.Z1PB := QE.Une_Station_1_Z;
  Result.X1PG := QE.Une_Station_1_X;
  Result.Y1PG := QE.Une_Station_1_Y;
  Result.Z1PH := QE.Une_Station_1_Z;
  Result.X2PD := QE.Une_Station_2_X;
  Result.Y2PD := QE.Une_Station_2_Y;
  Result.Z2PB := QE.Une_Station_2_Z;
  Result.X2PG := QE.Une_Station_2_X;
  Result.Y2PG := QE.Une_Station_2_Y;
  Result.Z2PH := QE.Une_Station_2_Z;
end;

function TGHTopoContext2DA.AjouterUneViseeALaSerieCourante(const QTypeVisee: TTypeDeVisee;
                                                           const QLongueur, QAzimut, QPente, QLG, QLD, QHZ, QHN: double;
                                                           const QIDLitteral, QCommentaire: string;
                                                           const DoRefreshPlan: boolean): boolean;
var
  QX, QY, QZ, QP: double;
  CC: TCode;
  QInternalIdxserie: integer;
  MySerie: TObjSerie;
  QNouvelleEntite: TEntiteEtendue;
  EE: TExpe;
  VS: TUneVisee;
  VA: TViseeAntenne;
begin
  Result := false;
  AfficherMessageErreur(Format('Entité courante: %d.%d - Code: %d - Expe: %d',
                               [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station,
                                FCurrentEntite.eCode, FCurrentEntite.eExpe
                               ]));
  if (not FDocuTopo.GetSerieByNumeroSerie(FCurrentNoSerieForDistoX, MySerie, QInternalIdxserie)) then
  begin
    AfficherMessageErreur('-- EE: Serie not found');
    Exit(false);
  end;
  FBDDEntites.GetEntiteViseeFromSerSt(MySerie.GetNumeroDeSerie(), MySerie.GetNbVisees() - 1, FCurrentEntite);
  //AfficherMessageErreur(Format('-- Série: %d - %s',  [MySerie.GetNumeroDeSerie(), MySerie.GetNomSerie()]));
  CC := FDocuTopo.GetCodeByNumero(FCurrentEntite.eCode);
  EE := FDocuTopo.GetExpeByNumero(FCurrentEntite.eExpe);
  VS.IDSecteur := FCurrentEntite.eSecteur;
  VS.TypeVisee := QTypeVisee;
  VS.Code      := CC.IDCode;
  VS.Expe      := EE.IDExpe;
  VS.Longueur  := QLongueur;
  VS.Azimut    := QAzimut;
  VS.Pente     := QPente;
  VS.LG        := QLG;
  VS.LD        := QLD;
  VS.HZ        := QHZ;
  VS.HN        := QHN;
  VS.IDTerrainStation := QIDLitteral;
  VS.Commentaires := QCommentaire;
  QX := 0.00; QY := 0.00; QZ := 0.00;
  // On fait un calcul individuel de visée pour éviter de recalculer tout le réseau
  CalculerVisee(VS, CC, EE, QX, QY, 1.00, QZ, QP);
  QNouvelleEntite := ReplaceEntitesCoords(FCurrentEntite, QX, QY, QZ);
  // c'est une visée de cheminement ?
  if (QTypeVisee in [tgDEFAULT, tgFOSSILE, tgVADOSE, tgENNOYABLE, tgSIPHON,
                     tgSURFACE, tgTUNNEL, tgMINE]) then
  begin
    // on ajoute la visée
    MySerie.AddVisee(VS);
    // on met à jour le couple série/station de fin
    MySerie.SetNoSerieArr(MySerie.GetNumeroDeSerie());
    MySerie.SetNoPointArr(MySerie.GetNbVisees() - 1);
    QNouvelleEntite.Entite_Serie   := MySerie.GetNumeroDeSerie();
    QNouvelleEntite.Entite_Station := MySerie.GetNbVisees() - 1;
    QNouvelleEntite.oIDLitteral    := QIDLitteral;
    QNouvelleEntite.oCommentaires  := QCommentaire;
    QNouvelleEntite.IsPOI          := false;
    // on ajoute l'entité calculée
    FBDDEntites.AddEntiteVisee(QNouvelleEntite);
    SetCurrentEntite(QNouvelleEntite);
    // on actualise la base sans recompiler
    //if (assigned(FProcActualiserAffichagesBDD)) then FProcActualiserAffichagesBDD(false);
    // et on met à jour la station courante
    if (FBDDEntites.GetEntiteViseeFromSerSt(MySerie.GetNumeroDeSerie(), MySerie.GetNbVisees() - 1, QNouvelleEntite)) then
    begin
      CentrerVueSurPointXY(FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y, true, 'Station added');
    end;
    result := true;
  end;
  if (DoRefreshPlan) then RedessinEcran(true);
end;
function TGHTopoContext2DA.AjouterUneAntenneALaStationCourante(const QLongueur, QAzimut, QPente: double;
                                                               const DoRefreshPlan: boolean): boolean;
var
  QX, QY, QZ, QP: double;
  CC: TCode;
  EE: TExpe;
  QNouvelleEntite: TEntiteEtendue;
  VS: TUneVisee;
  VA: TViseeAntenne;
begin
  Result := false;
  AfficherMessageErreur(Format('Entité courante: %d.%d - Code: %d - Expe: %d',
                               [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station,
                                FCurrentEntite.eCode, FCurrentEntite.eExpe
                               ]));

  //AfficherMessageErreur(Format('-- Série: %d - %s',  [MySerie.GetNumeroDeSerie(), MySerie.GetNomSerie()]));
  CC := FDocuTopo.GetCodeByNumero(FCurrentEntite.eCode);
  EE := FDocuTopo.GetExpeByNumero(FCurrentEntite.eExpe);
  VS := MakeTUneVisee(0, tgVISEE_RADIANTE, FCurrentEntite.eSecteur, CC.IDCode, EE.IDExpe, QLongueur, QAzimut, QPente, 0.00, 0.00, 0.00, 0.00);
  QX := 0.00; QY := 0.00; QZ := 0.00;
  // On fait un calcul individuel de visée pour éviter de recalculer tout le réseau
  CalculerVisee(VS, CC, EE, QX, QY, 1.00, QZ, QP);
  AfficherMessageErreur(Format('L=%.2f, A=%.2f, P= %.2f, QX = %.2f, QY = %.2f, QZ = %.2f',  [VS.Longueur, VS.Azimut, VS.Pente,  QX, QY, QZ]));
  QNouvelleEntite := ReplaceEntitesCoords(FCurrentEntite, QX, QY, QZ);
  // c'est une antenne ?
  VA := MakeTViseeAntenne(FDocuTopo.GetNbAntennes(),
                          FCurrentEntite.eEntrance, FCurrentEntite.eReseau, FCurrentEntite.eSecteur,
                          FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station,
                          FCurrentEntite.eCode, FCurrentEntite.eExpe,
                          QLongueur, QAzimut, QPente,
                          '','');
  FDocuTopo.AddViseeAntenne(VA);
  QX := 0.00; QY := 0.00; QZ := 0.00;
  CalculerVisee(VS, CC, EE, QX, QY, 1.0, QZ, QP);

  QNouvelleEntite := ReplaceEntitesCoords(FCurrentEntite, QX, QY, QZ);
  QNouvelleEntite.Type_Entite    := tgVISEE_RADIANTE;
  QNouvelleEntite.Entite_Serie   := FCurrentEntite.Entite_Serie;
  QNouvelleEntite.Entite_Station := FCurrentEntite.Entite_Station;
  QNouvelleEntite.oIDLitteral    := '';
  QNouvelleEntite.oCommentaires  := '';
  // on ajoute l'entité
  FBDDEntites.AddEntiteVisee(QNouvelleEntite);
  // on actualise la base sans recompiler
  //if (ass FProcActualiserAffichagesBDD(false);
  if (DoRefreshPlan) then RedessinEcran(true);
end;




procedure TGHTopoContext2DA.acAddViseeRadianteHereExecute(Sender: TObject);
var
  i: Integer;
  WU: String;
begin
  Randomize;
  WU := Format('%d.%d', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station]);
  ShowMessage('Insertion radiante ici: '+ WU);
  //FCurrentEntite;
  //(*
  for i := 0 to 100 do
  begin
    AjouterUneAntenneALaStationCourante(10.00 * Random(),
                                        360.00 * Random(),
                                        (180.0 * Random) - 90,
                                        false);
  end;
  Vue.Invalidate;

end;

procedure TGHTopoContext2DA.acCreerEntreeFromThisPointExecute(Sender: TObject);
var
  EE: TEntrance;
  WU: String;
  n: Integer;
begin
  WU := Format('%d.%d', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station]);
  EE := MakeTEntrance('Entrance ' + WU,
                      WU,
                      FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y, FCurrentEntite.Une_Station_2_Z,
                      FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station,
                      '');
  n := FDocuTopo.GetNbEntrees();
  if (EditerEntrance(FDocuTopo, n, FBDDEntites, EE)) then
  begin
    FDocuTopo.AddEntree(EE);
    n := FDocuTopo.GetNbEntrees();
    self.SetModeTravail(mtREADY);
  end;
end;

procedure TGHTopoContext2DA.acDisplayVoisinageCurrStationExecute(Sender: TObject);
begin
  DisplayStationsNearToCurrStation(FBDDEntites, FCurrentEntite);
end;



procedure TGHTopoContext2DA.acDistanceP2PExecute(Sender: TObject);
begin
  self.SetModeTravail(mtDISTANCE_PREMIER_POINT);
end;

procedure TGHTopoContext2DA.acMetaFiltreCodeExecute(Sender: TObject);
begin
  AppliquerLeFiltre(Format('%s=%d',[rsMETAFILTRE_CODE, FCurrentEntite.eCode]));
end;

procedure TGHTopoContext2DA.acMetaFiltreEntreeExecute(Sender: TObject);
begin
  AppliquerLeFiltre(Format('%s=%d',[rsMETAFILTRE_ENTREE_RATT, FCurrentEntite.eEntrance]));
end;
procedure TGHTopoContext2DA.acMetaFiltreExpeExecute(Sender: TObject);
begin
  AppliquerLeFiltre(Format('%s=%d',[rsMETAFILTRE_EXPE, FCurrentEntite.eExpe]));
end;

procedure TGHTopoContext2DA.acMetaFiltreReseauExecute(Sender: TObject);
begin
  AppliquerLeFiltre(Format('%s=%d',[rsMETAFILTRE_RESEAU, FCurrentEntite.eReseau]));
end;



procedure TGHTopoContext2DA.acMetaFiltreSecteurExecute(Sender: TObject);
begin
  AppliquerLeFiltre(Format('%s=%d',[rsMETAFILTRE_SECTEUR, FCurrentEntite.eSecteur]));
end;

procedure TGHTopoContext2DA.AppliquerLeFiltre(const FF: string);
begin
  ApplyMetaFiltre(True, FF);
  vue.Invalidate; //RedessinEcran;
end;

procedure TGHTopoContext2DA.DisplayFullInfosStation(const S: TEntiteEtendue);
var
  RS: TReseau;
  SC: TSecteur;
  QAltitudeEpicentre: double;
  L, A, P, G,D,H,B, dx, dy, dz: Double;
  EWE: TExpe;
  beuh: TPoint3Df;
  CDS: TCode;
  QIdxSerie: Integer;
  MySerie: TObjSerie;
  WU: String;
begin
  dx := S.Une_Station_2_X - S.Une_Station_1_X;
  dy := S.Une_Station_2_Y - S.Une_Station_1_Y;
  dz := S.Une_Station_2_Z - S.Une_Station_1_Z;
  GetBearingInc(dx, dy, dz,
                L,A,P,
                360.0,360.0);
  lbInfos.Caption:=Format(rsVUE2D_FMT_INFOS_STATION_RAPIDE,
                            [S.oIDLitteral,
                             S.Entite_Serie,
                             S.Entite_Station,
                             S.Une_Station_2_X,
                             S.Une_Station_2_Y,
                             S.Une_Station_2_Z]);
  //lbStation.Caption:= Format(rsVUE2D_FMT_INFOS_ID_STATION, [S.Entite_Serie, S.Entite_Station, S.oIDLitteral]);
  lbStation.Caption := MakeLibelleStationTopo(S);
  if (S.Type_Entite in [tgENTRANCE, tgFIXPOINT]) then
  begin
    lbInfosExpe.Caption     := '';
    lbMesures.Caption       := '';
    lbCouleurVisee.Color    := S.ColourByDepth;
    lbColorReseau.Color     := clGray;
    lbColorReseau.Caption   := '';
    lbNomReseau.Caption     := '';
    lbColorSecteur.Color    := clGray;
    lbColorSecteur.Caption  := '';
    lbNomSecteur.Caption    := '';
    lbColorSeance.Color     := clGray;
    lbColorSeance.Caption   := '';
    lbCode.Caption          := '';
  end
  else
  begin
    G:=Hypot2D((S.X2PG-S.Une_Station_2_X),
               (S.Y2PG-S.Une_Station_2_Y));
    D:=Hypot2D((S.X2PD-S.Une_Station_2_X),
               (S.Y2PD-S.Une_Station_2_Y));
    H:=S.Z2PH-S.Une_Station_2_Z;
    B:=S.Une_Station_2_Z - S.Z2PB;

    RS  := FBDDEntites.GetReseau(S.eReseau);
    SC  := FBDDEntites.GetSecteur(S.eSecteur);
    EWE := FBDDEntites.GetExpeByIndex(S.eExpe);
    CDS := FBDDEntites.GetCodeByIndex(S.eCode);
    lbCouleurVisee.Color    := S.ColourByDepth;
    lbColorReseau.Color     := RS.ColorReseau;
    lbColorReseau.Caption   := Format('%d',[S.eReseau]);
    lbNomReseau.Caption     := _AnsiToLCLStr(RS.NomReseau);
    lbColorSecteur.Color    := SC.CouleurSecteur;
    lbColorSecteur.Caption  := Format('%d',[S.eSecteur]);
    lbNomSecteur.Caption    := _AnsiToLCLStr(SC.NomSecteur);
    lbColorSeance.Color     := FBDDEntites.GetCouleurEntiteRGBByExpe(S);
    lbColorSeance.Caption   := Format('%d',[S.eExpe]);
    // infos sur le code
    lbCode.Caption          := MakeLibelleCode(CDS);//Format('%d - %.0f; %.0f', [CDS.IDCode, CDS.GradAz, CDS.GradInc]);
    // infos sur l'expé
    WU := DateToStr(GetSecuredDate(EWE.AnneeExpe, EWE.MoisExpe, EWE.JourExpe));
    lbInfosExpe.Caption     := MakeLibelleExpe(EWE); //Format('Expe %d: %s (%s, %s)', [EWE.IDExpe, WU, EWE.Operateur, EWE.ClubSpeleo]);
    lbMesures.Caption       := Format(rsVUE2D_FMT_INFOS_MESURES, [L, A, P, G,D,H,B]);
    // infos sur la série
    if (S.Type_Entite = tgVISEE_RADIANTE) then
    begin
      lbSerie.Caption := Format('Antenne #%d', [S.Entite_Serie]);
    end
    else
    begin
      QIdxSerie := FDocuTopo.GetIdxSerieByNumero(S.Entite_Serie);
      if (QIdxSerie > 0)  then
      begin
        MySerie := FDocuTopo.GetSerie(QIdxSerie);
        lbSerie.Caption := Format('Serie %d - %s', [MySerie.GetNumeroDeSerie, _AnsiToLCLStr(MySerie.GetNomSerie)]);
      end;

    end;
  end;
  // coordonnées
  beuh := FBDDEntites.GetDeltaXYZFromPositionSt0(S);
  lbCoordonnees.Caption := Format(rsVUE2D_FMT_INFOS_COORDONNEES, [
                                 FormatterNombreAvecSepMilliers(S.Une_Station_2_X),
                                 FormatterNombreAvecSepMilliers(S.Une_Station_2_Y),
                                 FormatterNombreAvecSepMilliers(S.Une_Station_2_Z),
                                 FormatterNombreAvecSepMilliers(beuh.X),
                                 FormatterNombreAvecSepMilliers(beuh.Y),
                                 FormatterNombreAvecSepMilliers(beuh.Z)
                                 ]);
  // entrée de rattachement
  //QEntree := FBDDEntites.GetEntree(S.eEntrance);
  lbEntreeRattachement.Caption := format('%d', [S.eEntrance]);
  // distances au versant si maillage OK
  if (FMyMaillage.IsValidMaillage()) then
  begin
    if (FMyMaillage.CalcAltitudeMaillageAtPoint(S.Une_Station_2_X, S.Une_Station_2_Y, QAltitudeEpicentre)) then
      lbMiscMaillage.Caption := format('Altitude surface à l''aplomb: %.2f - Dist: %.2f m',
                                       [QAltitudeEpicentre, QAltitudeEpicentre - S.Une_Station_2_Z])
    else
      lbMiscMaillage.Caption := 'Station hors maillage';
  end
  else
  begin
    lbMiscMaillage.Caption := 'Aucun maillage valide';
  end;
end;

procedure TGHTopoContext2DA.Finaliser();
begin
  try
  finally
  end;
end;
//******************************************************************************




procedure TGHTopoContext2DA.VueMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
const
  FMT_STATION_1 = '%d.%d - X = %.3f, Y = %.3f, Z = %.3f';
var
  // pointer la station la plus proche
  //ST1, ST2: TEntiteEtendue;
  QDist: double;
  QQ1: Boolean;
  QQ2: Boolean;
  QOffset, qC1, qC2, qD: TPoint2Df;
  //QQ1, QQ2: Boolean;
  ST1, ST2, QE: TEntiteEtendue;
  MyAnnotation: TAnnotation;
  DbgTag, WU: String;
  MySerieARaccorder: TObjSerie;
  QIdx, EWE: integer;
  procedure ReinitFZC(const DoFC1, DoFC2: boolean);
  begin
    if (DoFC1) then
    begin
      FZC1 := FMyPos;
      FZP1 := MakeTPoint(X, Y);
    end;
    if (DoFC2) then
    begin
      FZC2 := FMyPos;
      FZP2 := MakeTPoint(X, Y);
    end;
  end;
begin
  if (not FCanDraw) then Exit;
  // infos station
  if (not FBDDEntites.GetEntiteViseeFromXY(false, FMyPos.X, FMyPos.Y, QE, FCurrentInternalIdxEntite)) then Exit;
  FCurrentEntite := QE;
  SetStationInfo(FCurrentEntite);
  //SetCurrentEntite(QE);
  if (pnlFullInfos.Visible) then DisplayFullInfosStation(FCurrentEntite);
  case FModesTravail of
    mtREADY:
      pass;
    mtPAN_PREMIER_POINT:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtPAN_SECOND_POINT);
      end;
    mtPAN_SECOND_POINT:
      begin
        ReinitFZC(False, True);
        QOffset := MakeTPoint2Df(FZC2.X - FZC1.X, FZC2.Y - FZC1.Y);
        DeplacerVue(QOffset.X, QOffset.Y);
        Vue.Invalidate;
        SetModeTravail(mtREADY);
      end;
    mtZOOM_PREMIER_COIN:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtZOOM_SECOND_COIN);
      end;

    mtZOOM_SECOND_COIN:
      begin
        ReinitFZC(False, True);
        DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
        SetViewLimits(FZC1.X, FZC1.Y, FZC2.X, FZC2.Y, DbgTag);
        Vue.Invalidate;
        SetModeTravail(mtREADY);
      end;
    mtMETAFILTRE_ZONE_PREMIER_COIN:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtMETAFILTRE_ZONE_SECOND_COIN);
      end;
    mtMETAFILTRE_ZONE_SECOND_COIN:
      begin
        ReinitFZC(False, True);
        qC1 := MakeTPoint2Df(FZC1.X, FZC1.Y);
        qC2 := MakeTPoint2Df(FZC2.X, FZC2.Y);
        qD  := MakeTPoint2Df(qC2.X - qC1.X, qc2.Y - qC1.Y);
        // si zone trop étroite, abandonner
        if (Abs(qD.X) < 1.00) or (Abs(qD.Y) < 1.00) then Exit;
        // échanger de manière à rendre indifférent le sens du rectangle de sélection
        if (qC2.X < qC1.X) then Swap(qC1.X, qC2.X);
        if (qC2.Y < qC1.Y) then Swap(qC1.Y, qC2.y);
        AppliquerLeFiltre(Format('COORD_X=(%.2f, %.2f) & COORD_Y=(%.2f, %.2f)', [qC1.X, qC2.X, qC1.Y, qC2.Y]));
        SetModeTravail(mtREADY);
      end;
    mtDISTANCE_PREMIER_POINT:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtDISTANCE_SECOND_POINT);
      end;
    mtDISTANCE_SECOND_POINT:
      begin
        ReinitFZC(False, True);
        SetModeTravail(mtREADY);
        QQ1 := FBDDEntites.GetEntiteViseeFromXY(false, FZC1.X, FZC1.Y, ST1, FCurrentInternalIdxEntite);
        QQ2 := FBDDEntites.GetEntiteViseeFromXY(false, FZC2.X, FZC2.Y, ST2, FCurrentInternalIdxEntite);
        if (QQ1 and QQ2) then
        begin
          DispDistanceEntreStations(FDocuTopo, FBDDEntites, ST1, ST2, 'Distance entre deux points topo', '');
          vue.Invalidate;
        end;
      end;

    mtADD_SERIE_PREMIER_POINT:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtADD_SERIE_SECOND_POINT);
      end;

    mtADD_SERIE_SECOND_POINT:
      begin
        ReinitFZC(False, True);
        SetModeTravail(mtREADY);
        QQ1 := FBDDEntites.GetEntiteViseeFromXY(false, FZC1.X, FZC1.Y, ST1, FCurrentInternalIdxEntite);
        QQ2 := FBDDEntites.GetEntiteViseeFromXY(false, FZC2.X, FZC2.Y, ST2, FCurrentInternalIdxEntite);
        if (QQ1 and QQ2) then
        begin
          //ShowMessage('Insérer une série');
          //AfficherDistanceEntreStations(FBDDEntites, ST1, ST2);
        end;
      end;
    // Raccordement graphique du terminus d'un cheminement sur une station d'un autre cheminement
    mtBOUCLER_PREMIER_POINT:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtBOUCLER_SECOND_POINT);
      end;
    mtBOUCLER_SECOND_POINT:
      begin
        ReinitFZC(False, True);
        SetModeTravail(mtREADY);
        QQ1 := FBDDEntites.GetEntiteViseeFromXY(false, FZC1.X, FZC1.Y, ST1, FCurrentInternalIdxEntite);
        QQ2 := FBDDEntites.GetEntiteViseeFromXY(false, FZC2.X, FZC2.Y, ST2, FCurrentInternalIdxEntite);
        if (QQ1 and QQ2) then
        begin
          EWE := DispDistanceEntreStations(self.FDocuTopo, self.FBDDEntites, ST1, ST2, 'Mise en place d''un bouclage', 'Recalculer le réseau');
          if ((EWE > 0) AND FDocuTopo.GetSerieByNumeroSerie(ST1.Entite_Serie, MySerieARaccorder, QIdx)) then
          begin
            WU    := 'Accrocher le point:' + #10 +
                     Format(FMT_STATION_1, [ST1.Entite_Serie, ST1.Entite_Station, ST1.Une_Station_2_X, ST1.Une_Station_2_Y, ST1.Une_Station_2_Z]) + #10 +
                     'Au point:' + #10 +
                     Format(FMT_STATION_1, [ST2.Entite_Serie, ST2.Entite_Station, ST2.Une_Station_2_X, ST2.Une_Station_2_Y, ST2.Une_Station_2_Z]) + #10 +
                     Format('Distance: %.3f m', [QDist]);
            if (GHTopoQuestionOuiNon(WU)) then
            begin
              MySerieARaccorder.SetSeriePtArr(ST2.Entite_Serie, ST2.Entite_Station);
              if ((EWE = 2) and (GHTopoQuestionOuiNon('Le réseau va être recalculé'))) then
              begin
                if (assigned(FProcPerformActionRecalcul)) then FProcPerformActionRecalcul();
              end;
            end;
          end;
        end;
      end;
    // Extraction d'un profil MNT
    mtPROFIL_MNT_PREMIER_POINT:
      begin
        ReinitFZC(True, True);
        SetModeTravail(mtPROFIL_MNT_SECOND_POINT);
      end;
    mtPROFIL_MNT_SECOND_POINT:
      begin
        ReinitFZC(False, True);
        SetModeTravail(mtREADY);
        QQ1 := FBDDEntites.GetEntiteViseeFromXY(false, FZC1.X, FZC1.Y, ST1, FCurrentInternalIdxEntite);
        QQ2 := FBDDEntites.GetEntiteViseeFromXY(false, FZC2.X, FZC2.Y, ST2, FCurrentInternalIdxEntite);
        if (QQ1 and QQ2) then
        begin
          FMyMaillage.ExtractProfilTopo(FZC1, FZC2);
          FMyMaillage.ListerProfilsTopo();
          vue.Invalidate;
        end;
      end;

    mtADD_ANNOTATION:
      begin
        QQ1 := FBDDEntites.GetEntiteViseeFromXY(false, FMyPos.X, FMyPos.Y, ST1, FCurrentInternalIdxEntite);
        MyAnnotation.Basepoint.eSerie := ST1.Entite_Serie;
        MyAnnotation.Basepoint.ePoint := ST1.Entite_Station;
        MyAnnotation.Texte := 'toto';
        MyAnnotation.Offset := MakeTPoint3Df(FMyPos.X - ST1.Une_Station_2_X,
                                             FMyPos.Y - ST1.Une_Station_2_Y,
                                             0.00);
        MyAnnotation.PosPtDeBase := 0;
        MyAnnotation.IdxStyle := 0;
        if (EditerAnnotation(FDocuTopo, 0, MyAnnotation)) then
        begin
          SetModeTravail(mtREADY);
          FDocuTopo.AddAnnotation(MyAnnotation);
          Vue.Invalidate; //RedessinEcran;
          //if (Assigned(FProcActualiserAffichagesBDD)) then FProcActualiserAffichagesBDD(false);
        end;
      end;
  end;
end;

procedure TGHTopoContext2DA.VueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  PP: TPoint;
  QDistance: float;
  QAzimut: Double;
  procedure DrawLigne(const P1, P2: TPoint); inline;
  begin
    Vue.Canvas.Line(P1.X,P1.Y, P2.X, P2.Y);
  end;
  procedure DrawRectangle(const P1, P2: TPoint);
  begin
    Vue.Canvas.Brush.Style := bsClear;
    Vue.Canvas.Rectangle(P1.X, P1.Y, P2.X, P2.Y);
  end;
  procedure DrawRectangleBarre(const P1, P2: TPoint);
  begin
    Vue.Canvas.Brush.Style := bsClear;
    Vue.Canvas.Rectangle(P1.X, P1.Y, P2.X, P2.Y);
    Vue.Canvas.Line(P1.X, P1.Y, P2.X, P2.Y);
    Vue.Canvas.Line(P1.X, P2.Y, P2.X, P1.Y);
  end;


  procedure DrawPolygonaleRapide();
  var
    QW, QH: Double;
    i, Nb: integer;
    QPP1, QPP2: TPoint;
    MyEntite: TEntiteEtendue;

    QLimitesVue: TRect2Df;
    FP1, FP2: TPoint2Df;
    qdx, qdy: LongInt;
  begin
    Nb := FBDDEntites.GetNbEntitesVisees();
    // lenteurs constatées sur certains postes => au delà d'un certain seuil, on ne trace rien
    if (Nb > 40000) then Exit;
    // pour éviter de voir la silhouette fantome tronquée, on spécifie une marge
    // autour du rectangle de vue initial égale au déplacement maximal possible de la vue
    // ie: marges horizontales = largeur de la fenêtre vue initiale
    //     marges verticales   = hauteur de la fenêtre vue initiale
    QW := FRegionXMaxi - FRegionXMini;
    QH := FRegionYMaxi - FRegionYMini;
    QLimitesVue := MakeTRect2Df(FRegionXMini - QW, FRegionYMini - QH,
                                FRegionXMaxi + QW, FRegionYMaxi + QH);
    for i := 0 to Nb - 1 do
    begin
      MyEntite := FBDDEntites.GetEntiteVisee(i);
      if (Not (MyEntite.Type_Entite in [tgVISEE_RADIANTE, tgENTRANCE, tgFIXPOINT])) then
      begin
        FP1 := MakeTPoint2Df(MyEntite.Une_Station_1_X, MyEntite.Une_Station_1_Y);
        FP2 := MakeTPoint2Df(MyEntite.Une_Station_2_X, MyEntite.Une_Station_2_Y);
        if (IsSegmentInRectangle(FP1, FP2, QLimitesVue)) then
        begin
          QPP1 := GetCoordsPlan(FP1);
          QPP2 := GetCoordsPlan(FP2);
          //DrawLigne(QPP1, QPP2);
          qdx := FZP2.X - FZP1.X;
          qdy := FZP2.Y - FZP1.Y;
          QPP1.X := QPP1.X + qdx;
          QPP1.Y := QPP1.Y + qdy;
          QPP2.X := QPP2.X + qdx;
          QPP2.Y := QPP2.Y + qdy;
          DrawLigne(QPP1, QPP2);
        end;
      end;
    end;
  end;


  procedure DrawShapeElastique(const QX, QY: integer; const SType: byte; const DoDrawSilhouetteFantome: boolean = false);
    procedure CuiCui();
    const
      QHautTextInPxl = 20;
    var
      QPT: TPoint;
      QPM, QPM1, QPM2: TPoint2Df;
      Qdx, Qdy, Qdist, QH: Double;
    begin
      case SType of
        1: begin
             DrawLigne(FZP1, FZP2);
             if (DoDrawSilhouetteFantome) then DrawPolygonaleRapide();
           end;
        2: DrawRectangle(FZP1, FZP2);
        3: DrawRectangleBarre(FZP1, FZP2);
      end;
    end;
  begin
    CuiCui();
    FZP2 := MakeTPoint(QX, QY);
    CuiCui();
  end;
begin
  if (not FCanDraw) then Exit;
  PP := MakeTPoint(X, Y);
  FMyPos   := GetCoordsMonde(PP);
  lbGCSMouse.Caption:=Format('%.2f, %.2f',[FMyPos.X, FMyPos.Y]);
  case FModesTravail of
    mtREADY:
      begin
        FZP1 := MakeTPoint(X, Y);
        FZP2 := MakeTPoint(X, Y);
      end;
    mtPAN_SECOND_POINT           : DrawShapeElastique(X, Y, 1, True);
    mtBOUCLER_SECOND_POINT       : DrawShapeElastique(X, Y, 1);
    mtDISTANCE_SECOND_POINT      :
      begin
        DrawShapeElastique(X, Y, 1);
        QDistance := hypot(FMyPos.X - FZC1.X, FMyPos.Y - FZC1.Y);
        QAzimut   := GetAzimut(FMyPos.X - FZC1.X, FMyPos.Y - FZC1.Y, 360.00);
        lbAdditionalInfos.Caption := Format('%.2f m < %.2f°', [QDistance, QAzimut]);
      end;
    mtADD_SERIE_SECOND_POINT      : DrawShapeElastique(X, Y, 1);
    mtZOOM_SECOND_COIN            : DrawShapeElastique(X, Y, 2);
    mtMETAFILTRE_ZONE_SECOND_COIN : DrawShapeElastique(X, Y, 3);
    mtPROFIL_MNT_SECOND_POINT     : DrawShapeElastique(X, Y, 1);
  else
    pass;
  end;
end;
// En Lazarus, cet événement n'est pas traité
procedure TGHTopoContext2DA.VueMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
const
  Q_WHEEL_VERTICAL   = 0;
  Q_WHEEL_HORIZONTAL = 1;
  Q_WHEEL_ZOOM       = 2;
var
  QModeDeplVue: byte;
  QZoom, Qdx, Qdy: Extended;
  QOnglet: TVue2DParams;
  QR: Double;
  DbgTag: String;
begin
  // Valeur de WheelDelta retournée: = 120 ; négatif en tirant, positif en poussant
  // Utilisation des touches: convention Inkscape
  // Shift: []         = zoom
  //        [ssShift]  = pan horizontal
  //        [ssCtrl]   = pan vertical
  //QModeStr := 'Pan vertical';
  AfficherMessage('Wheel: ' + inttostr(WheelDelta));
  if      (ssShift in Shift) then QModeDeplVue := Q_WHEEL_HORIZONTAL  // pan H
  else if (ssCtrl  in Shift) then QModeDeplVue := Q_WHEEL_VERTICAL    // pan V
  else                            QModeDeplVue := Q_WHEEL_ZOOM;       // zoom
  QOnglet := FArrOngletsParams[FCurrIdxOnglet];
  QZoom := 1.00;
  Qdx   := 0.00;
  Qdy   := 0.00;
  QR    := Hypot2D(QOnglet.ongX2 - QOnglet.ongX1, QOnglet.ongY2 - QOnglet.ongY1);
  case QModeDeplVue of
    Q_WHEEL_VERTICAL:
    begin
      Qdy := 0.05 * QR * IIF(WheelDelta < 0, -1.0, 1.0);
      DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
      SetViewLimits(QOnglet.ongX1, QOnglet.ongY1 + Qdy,
                    QOnglet.ongX2, QOnglet.ongY2 + Qdy,
                    DbgTag);
    end;
    Q_WHEEL_HORIZONTAL:
    begin
      Qdx := 0.05 * QR * IIF(WheelDelta < 0, -1.0, 1.0);
      DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
      SetViewLimits(QOnglet.ongX1 + Qdx, QOnglet.ongY1 + Qdy,
                    QOnglet.ongX2 + Qdx, QOnglet.ongY2 + Qdy,
                    DbgTag);
    end;
    Q_WHEEL_ZOOM:
    begin
      QZoom := 1 + 0.05 * IIF(WheelDelta < 0, -1.0, 1.0);
      ZoomFact(QZoom);
    end;
  end;
  Vue.Invalidate;
end;

procedure TGHTopoContext2DA.VuePaint(Sender: TObject);
begin
  RedessinEcran(False);
end;

// cette fonction retourne d'autres paramètres
function TGHTopoContext2DA.GetRYMaxi(): double;
begin
  // calcul du rapport Hauteur/largeur de vue
  FRappHLVue := Vue.Height / Vue.Width;
  // calcul du rapport Ecran/Réel
  FRappScrReal := Vue.Width / (FRegionXMaxi - FRegionXMini);
  FInvRappScrReal := 1 / FRappScrReal;
  // calcul de la hauteur de visualisation
  Result := FRegionYMini + (FRegionXMaxi - FRegionXMini) * FRappHLVue;
end;

//******************************************************************************
// lbFiltres
procedure TGHTopoContext2DA.ApplyMetaFiltre(const B: boolean; const QFiltre: string);
var
  FF: String;
  QDevelViseesVisibles: double;
begin
  FF := IIF(B, QFiltre, '');
  FArrOngletsParams[FCurrIdxOnglet].ongVueFiltres := FF;
  FBDDEntites.MetaFiltre(FF, QDevelViseesVisibles);     // traitements
  lbLongViseesVisibles.Caption := Format('%.2f', [QDevelViseesVisibles]);
  FBDDEntites.SetMinMax(False);   // reset Mini et Maxi sans utilisation du filtre
  Vue.Invalidate;
  editMetaFiltre.Text := QFiltre; // restauration du filtre
end;

procedure TGHTopoContext2DA.btnApplyMetaFiltreClick(Sender: TObject);
begin
  ApplyMetaFiltre(chkMetaFiltreActif.Checked, trim(editMetaFiltre.Text));
end;


procedure TGHTopoContext2DA.btnCopyCoordsClick(Sender: TObject);
var
  CP: TClipboard;
begin
  CP := TClipboard.Create();
  try
    CP.AsText := lbCoordonnees.Caption;
  finally
    CP.Free;
  end;
end;

procedure TGHTopoContext2DA.btnHelpMetaFiltreClick(Sender: TObject);
begin
  DisplayHelpSystem('METAFILTRE');
end;

procedure TGHTopoContext2DA.btnListeProfilsClick(Sender: TObject);
begin
  DisplayListeProfils(FDocuTopo, FMyMaillage);
end;

//******************************************************************************
// gestion des onglets
// onglets
procedure TGHTopoContext2DA.PutOngletByIndex(const Idx: integer;
                                             const O: TVue2DParams;
                                             const DoRedrawVue: boolean);
var
  DbgTag: String;
begin
  FArrOngletsParams[Idx] := O;
  editMetaFiltre.Text := O.ongVueFiltres;
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  if (DoRedrawVue) then  // acquittement par redessin
  begin
    SetViewLimits(O.ongX1, O.ongY1, O.ongX2, O.ongY2, DbgTag);
    Vue.Invalidate;
  end;
end;

procedure TGHTopoContext2DA.SetHighLightCurrentItem(const B: boolean);
begin
  if (FDoHighLightCurrentItem <> B) then
  begin
    FDoHighLightCurrentItem := B;
    Vue.Invalidate;
  end;
end;


function TGHTopoContext2DA.ChargerMaillage(const QFileName: string): boolean;
begin
  Result  := (FMyMaillage.LireFichierMaillage(QFileName) >= 0);
  pnlMaillage.Visible := Result;
end;



procedure TGHTopoContext2DA.SetCurrentIdxOnglet(const Idx: byte);
var
  FCurrentOnglet: TVue2DParams;
  DbgTag: String;
begin
  AfficherMessage(Format('%s.SetCurrentOngletByIdx(%d)',[ClassName, Idx]));
  FCurrIdxOnglet := Idx;
  FCurrentOnglet := FArrOngletsParams[Idx];
  // mise à jour du champ lbFiltres
  AppliquerLeFiltre(FArrOngletsParams[Idx].ongVueFiltres);
  // recadrage
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  SetViewLimits(FCurrentOnglet.ongX1, FCurrentOnglet.ongY1,
                FCurrentOnglet.ongX2, FCurrentOnglet.ongY2,
                DbgTag);
  // recalculer les dégradés
  FBDDEntites.CalcCouleursByDepth(FCurrentOnglet.ongDegradeStart, FCurrentOnglet.ongDegradeStop);
  // mode de représentation
  SetModeRepresentation(FCurrentOnglet.ongModeRepresentation);
  Vue.Invalidate; //RedessinEcran;
end;




function TGHTopoContext2DA.GetNbOngletsVues(): integer;
begin
  Result := 1 + High(FArrOngletsParams);
end;

function TGHTopoContext2DA.GetCurrentOngletIdx(): integer;
begin
  Result := FCurrIdxOnglet;
end;

function TGHTopoContext2DA.GetOngletByIndex(const Idx: integer): TVue2DParams;
begin
  Result := FArrOngletsParams[Idx];
end;

function TGHTopoContext2DA.GetCurrentOnglet(): TVue2DParams;
begin
  Result := FArrOngletsParams[FCurrIdxOnglet];
end;



function TGHTopoContext2DA.GetPtrBDDEntites(): TBDDEntites;
begin
  Result := FBDDEntites;
end;

procedure TGHTopoContext2DA.SetNomOnglet(const S: string);
begin
  FArrOngletsParams[FCurrIdxOnglet].ongName := S;
end;

procedure TGHTopoContext2DA.SetProcUseMetaFiltre(const P: TProcedureOfObject);
begin
  FProcUseMetaFiltre := P;
end;


function TGHTopoContext2DA.GetNomOnglet(): string;
begin
  Result := FArrOngletsParams[FCurrIdxOnglet].ongName;
end;
procedure TGHTopoContext2DA.SetBackGrdColorOnglet(const QIdxOnglet: integer; const Col: TColor);
begin
  FArrOngletsParams[QIdxOnglet].ongBackGround := Col;
end;

function TGHTopoContext2DA.GetLargeurViseesOnglet(const QIdxOnglet: integer): integer;
begin
  result :=  FArrOngletsParams[QIdxOnglet].ongViseesLargeur;
end;

function TGHTopoContext2DA.GetModeRepresentation(): TModeRepresentationGaleries;
begin
  Result := FModeRepresentationGaleries;
end;

procedure TGHTopoContext2DA.SetLargeurViseesOnglet(const QIdxOnglet: integer; const qL: integer);
begin
  FArrOngletsParams[QIdxOnglet].ongViseesLargeur := qL;
end;

function TGHTopoContext2DA.GetBackGrdColorOnglet(const QIdxOnglet: integer): TColor;
begin
  Result := FArrOngletsParams[QIdxOnglet].ongBackGround;
end;




//******************************************************************************
// quadrillage
procedure TGHTopoContext2DA.SetQuadrillageSpc(const Spacing: double);
begin
  //FQuadrillageSpc := Spacing;
  FArrOngletsParams[FCurrIdxOnglet].ongQdrSpc := Spacing;
end;

function TGHTopoContext2DA.GetQuadrillageSpc(): double;
begin
  Result := FArrOngletsParams[FCurrIdxOnglet].ongQdrSpc;
end;

procedure TGHTopoContext2DA.SetQuadrillageColor(const Col: TColor);
begin
  if (Col <> FArrOngletsParams[FCurrIdxOnglet].ongQdrColor) then  FArrOngletsParams[FCurrIdxOnglet].ongQdrColor := Col;
end;

function TGHTopoContext2DA.GetQuadrillageColor(): TColor;
begin
  //Result := FQuadrillageColor;
  Result := FArrOngletsParams[FCurrIdxOnglet].ongQdrColor;
end;

procedure TGHTopoContext2DA.SetElementsDrawn(const ED: TElementsDrawn);
begin
  if (ED <> FArrOngletsParams[FCurrIdxOnglet].ongElementsDrawn) then FArrOngletsParams[FCurrIdxOnglet].ongElementsDrawn := ED;
end;

function  TGHTopoContext2DA.GetElementsDrawn: TElementsDrawn;
begin
  //Result := FElementsDrawn;
  Result := FArrOngletsParams[FCurrIdxOnglet].ongElementsDrawn;
end;

procedure TGHTopoContext2DA.QSetPen(const QPM: TPenMode; const QPS: TPenStyle; const QPW: integer; const QPC: TColor);
 begin
   Vue.Canvas.Pen.Mode := QPM;
   Vue.Canvas.Pen.Style:= QPS;
   Vue.Canvas.Pen.Color:= QPC;
   Vue.Canvas.Pen.Width:= QPW;
 end;
 procedure TGHTopoContext2DA.Rearmer();
 begin
   FAttenduSecondPoint        := False;
   QSetPen(pmCopy, psSolid, 0, clGray);
 end;
procedure TGHTopoContext2DA.SetModeTravail(const MT: TModesTravail);
begin
  if (MT = FModesTravail) then Exit; // ne rien faire si le mode de travail est inchangé
  FModesTravail := MT;
  case FModesTravail of
      mtREADY:
      begin
       Rearmer();
      end;
     mtPAN_PREMIER_POINT,
     mtDISTANCE_PREMIER_POINT,
     mtADD_SERIE_PREMIER_POINT,
     mtBOUCLER_PREMIER_POINT,
     mtPROFIL_MNT_PREMIER_POINT:
      begin
        FZC1 := FMyPos;
        FZC2 := FMyPos;
        FAttenduSecondPoint := false;
        QSetPen(pmCopy, psSolid, 0, clGray);
      end;
      mtPAN_SECOND_POINT,
      mtDISTANCE_SECOND_POINT,
      mtADD_SERIE_SECOND_POINT,
      mtBOUCLER_SECOND_POINT,
      mtPROFIL_MNT_SECOND_POINT:
      begin
        FAttenduSecondPoint := true;
        QSetPen(pmNotXor, psSolid, 0, clGray);
      end;
      mtZOOM_PREMIER_COIN, mtMETAFILTRE_ZONE_PREMIER_COIN:
      begin
        FZC1 := FMyPos;
        FZC2 := FMyPos;
        FAttenduSecondPoint := false;
        QSetPen(pmCopy, psSolid, 0, clGray);
      end;
      mtZOOM_SECOND_COIN, mtMETAFILTRE_ZONE_SECOND_COIN:
      begin
        FAttenduSecondPoint := true;
        QSetPen(pmNotXor, psSolid, 0, clGray);
      end;
  end;
  lbModeTravail.Caption    := ChooseString(Ord(MT),
                                       ['READY',                         //TModesTravail  = (mtREADY,
                                        'PAN_PT1', 'PAN_PT2',                    // mtPAN_PREMIER_POINT, mtPAN_SECOND_POINT,
                                        'ZOOM_C1', 'ZOOM_C2',                  //mtZOOM_PREMIER_COIN, mtZOOM_SECOND_COIN,
                                        'MF_ZONE', 'MF_ZONE',            //mtMETAFILTRE_ZONE_PREMIER_COIN, mtMETAFILTRE_ZONE_SECOND_COIN,
                                        'DISTANCE_PT1', 'DISTANCE_PT2',          //mtDISTANCE_PREMIER_POINT, mtDISTANCE_SECOND_POINT,
                                        'ADD_NOTE', 'DEL_NOTE',          //mtADD_ANNOTATION, mtSELECT_ANNOTATION);
                                        'ADD_SERIE_PT1', 'ADD_SERIE_PT2', // mtADD_SERIE_PREMIER_POINT, mtADD_SERIE_SECOND_POINT,
                                        'RACCORDER_PT1', 'RACCORDER_PT2', //mtBOUCLER_PREMIER_POINT, mtBOUCLER_SECOND_POINT
                                        'PROFIL_MNT_PT1','PROFIL_MNT_PT1' //mtPROFIL_MNT_PREMIER_POINT, mtPROFIL_MNT_SECOND_POINT
                                       ]);
  lbAdditionalInfos.Caption := ChooseString(Ord(MT),
                                       ['--',
                                        'Premier point', 'Second point',
                                        'Premier coin', 'Second coin',
                                        'Premier coin', 'Second coin',
                                        'Premier point', 'Second point',
                                        'Pick point', 'Pick near',
                                        'Pick point', 'Pick near',
                                        'Série à raccorder', 'Station de raccordement',
                                        'Premier point', 'Second point'
                                       ]);
end;



//******************************************************************************
procedure TGHTopoContext2DA.SetViewLimits(const X1, Y1, X2, Y2: double; const DebugTag: string);
const
  Epsilon = 1e-2;
var
  qX1, qX2, qY1, qY2: double;
  d1, d2: double;
begin
  AfficherMessage(Format('%s.SetViewLimits(%.2f, %.2f, %.2f, %.2f) - Tag: %s',
    [ClassName, X1, Y1, X2, Y2, DebugTag]));
  qX1 := X1;
  qY1 := Y1;
  qX2 := X2;
  qY2 := Y2;

  d1 := qX2 - qX1;
  d2 := qY2 - qY1;
  // si zone trop étroite, abandonner
  if (Abs(d1) < Epsilon) or (Abs(d2) < Epsilon) then
    Exit;
  // échanger de manière à rendre indifférent le sens du rectangle de sélection
  if (qX2 < qX1) then Swap(qX1, qX2);
  if (qY2 < qY1) then Swap(qY1, qY2);

  FRegionXMini := qX1;
  FRegionXMaxi := qX2;
  FRegionYMini := qY1;
  FRegionYMaxi := qY2;
  //*)
  // Redéfinition de la hauteur maxi
  FRegionYMaxi := GetRYMaxi;
  // on met tout çà dans l'onglet
  FArrOngletsParams[FCurrIdxOnglet].ongX1 := FRegionXMini;
  FArrOngletsParams[FCurrIdxOnglet].ongY1 := FRegionYMini;
  FArrOngletsParams[FCurrIdxOnglet].ongX2 := FRegionXMaxi;
  FArrOngletsParams[FCurrIdxOnglet].ongY2 := FRegionYMaxi;
end;
procedure TGHTopoContext2DA.SetViewLimits(const QCoinBasGauche, QCointHautDroit: TPoint2Df; const DebugTag: string);
begin
  SetViewLimits(QCoinBasGauche.X, QCoinBasGauche.Y, QCointHautDroit.X, QCointHautDroit.Y, DebugTag);
end;


procedure TGHTopoContext2DA.DeplacerVue(const QOffsetX, QOffsetY: double);
var
  DbgTag: String;
begin
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  SetViewLimits(FRegionXMini - QOffsetX, FRegionYMini - QOffsetY,
                FRegionXMaxi - QOffsetX, FRegionYMaxi - QOffsetY,
                DbgTag);
end;


function TGHTopoContext2DA.GetViewLimits(const QIdxOnglet: integer; out QCoinBasGauche, QCointHautDroit: TPoint2Df): boolean;
var
  EWE: TVue2DParams;
begin
  result := false;
  try
    EWE := FArrOngletsParams[QIdxOnglet];
    QCoinBasGauche  := MakeTPoint2Df(EWE.ongX1, EWE.ongY1);
    QCointHautDroit := MakeTPoint2Df(EWE.ongX2, EWE.ongY2);
    result := true;
  except
    ;
  end;
end;

//******************************************************************************
//******************************************************************************
function TGHTopoContext2DA.GetCoordsPlan(const PM: TPoint2Df): TPoint;
begin
  Result.X := Round((PM.X - FRegionXMini) * FRappScrReal);
  Result.Y := Round((FRegionYMaxi - PM.Y) * FRappScrReal);
end;

function TGHTopoContext2DA.GetCoordsMonde(const PP: TPoint): TPoint2Df;
begin
  Result.X :=  FInvRappScrReal * PP.X + FRegionXMini;
  Result.Y := -FInvRappScrReal * PP.Y + FRegionYMaxi;
end;
//******************************************************************************
procedure TGHTopoContext2DA.DeplacerVueInt(const DepX, DepY: integer);
var
  P : TPoint;
  dq, dx, dy: Double;
  DbgTag: String;
begin
  P.X := 0;
  P.Y := 0;
  FZC1:= GetCoordsMonde(P);
  P.X := Vue.Width   ; P.Y:=Vue.Height;
  FZC2:=GetCoordsMonde(P);

  dq  := 0.02 * (FZC2.X - FZC1.X);
  dx  :=  dq * DepX;
  dy  := -dq * DepY;
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  SetViewLimits(FRegionXMini + dx, FRegionYMini + dy,
                FRegionXMaxi + dx, FRegionYMaxi + dy,
                DbgTag);
  Vue.Invalidate; //RedessinEcran;
end;

// REDESSIN DE L'ECRAN
procedure TGHTopoContext2DA.RedessinEcran(const DoRecalcDegrades: boolean; const DoExportInFile: boolean = false; const QFileName: string = '');
const
  R666 = 4;
var
  R: TRect;
  TmpBuffer: TBGRABitmap;
  MonOnglet: TVue2DParams;
  MyRectVue: TRect2Df;
  function QGetCouleur(const QQE: TEntiteEtendue): TColor;
  begin
    case FModeRepresentationGaleries of
      rgRESEAUX      : Result := FBDDEntites.GetCouleurEntiteRGBByReseau(QQE);
      rgSECTEURS     : Result := FBDDEntites.GetCouleurEntiteRGBBySecteur(QQE);
      rgSEANCES      : Result := FBDDEntites.GetCouleurEntiteRGBByExpe(QQE);
      rgDEPTH        : Result := QQE.ColourByDepth;
      rgTYPES_VISEES : Result := ChooseColorByTypeEntite(QQE.Type_Entite);
    else
      Result := clGray;
    end;
  end;

  // polygonales
  procedure DrawPolygonals;
  var
    i, NbE:      integer;
    E:      TEntiteEtendue;
    P1, P2: TPoint;
    PM1, PM2: TPoint2Df;
    CS: TNumeroSerie;
    CC: TNumeroCode;
    CE: TNumeroExpe;
    CR: TNumeroReseau;
    CZ: TNumeroSecteur;
    procedure DrwLn666(const QW: integer; const QC: TColor; const QPT1, QPT2: TPoint);
    begin
      TmpBuffer.CanvasBGRA.Pen.Width := QW;
      TmpBuffer.CanvasBGRA.Pen.Color := QC;
      TmpBuffer.CanvasBGRA.MoveTo(QPT1);
      TmpBuffer.CanvasBGRA.LineTo(QPT2);
    end;
    procedure DrwLn777(const QPT1, QPT2: TPoint);
    begin
      DrwLn666(8, clRed, QPT1, QPT2);
      DrwLn666(6, clBlue, QPT1, QPT2);
    end;
  begin
    //AfficherMessage(' --> DrawPolygonals');
    // TODO Représentation de couleur pour les visées spéciales
    CS := FDocuTopo.GetCurrentNumeroSerie();
    CC := FDocuTopo.GetCurrentNumeroCode();
    CE := FDocuTopo.GetCurrentNumeroExpe();
    CR := FDocuTopo.GetCurrentNumeroReseau();
    CZ := FDocuTopo.GetCurrentNumeroSecteur();
    NbE := FBDDEntites.GetNbEntitesVisees();
    for i := 0 to NbE - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      if (E.Type_Entite = tgENTRANCE) then
      begin // dessin des entrées
        ;
      end
      else
      begin
        PM1 := MakeTPoint2Df(E.Une_Station_1_X, E.Une_Station_1_Y);
        PM2 := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
        if (IsSegmentInRectangle(PM1, PM2, MyRectVue)) then
        begin
          P1   := GetCoordsPlan(PM1);
          P2   := GetCoordsPlan(PM2);
          if (E.Drawn) then DrwLn666(FArrOngletsParams[FCurrIdxOnglet].ongViseesLargeur, QGetCouleur(E), P1, P2)
                       else DrwLn666(0, clGray, P1, P2);
          if (FDoHighLightCurrentItem) then
          begin
            case FModeSelectionListe of
              mslENTRANCES         : ; // if (E.Entite_Serie = CS) then DrwLn777(P1, P2);
              mslRESEAUX           : if (E.eReseau      = CR) then DrwLn777(P1, P2);
              mslSECTEURS          : if (E.eSecteur     = CZ) then DrwLn777(P1, P2);
              mslCODE              : if (E.eCode        = CC) then DrwLn777(P1, P2);
              mslEXPE              : if (E.eExpe        = CE) then DrwLn777(P1, P2);
              mslSERIE             : if (E.Entite_Serie = CS) then DrwLn777(P1, P2);
              mslTYPE_GALERIE      : ; //if (E.Entite_Serie = CS) then DrwLn777(P1, P2);
              mslDATE              : ; //if (E.Entite_Serie = CS) then DrwLn777(P1, P2);
              mslSTATIONS_CALCULEES: ;
            end;
          end;
        end;
      end; // if E.Type_Entite=tgENTRANCE
    end;
  end;
  procedure DrawSections;
  var
    i:      integer;
    E:      TEntiteEtendue;
    P1, P2: TPoint;
    PM1, PM2: TPoint2Df;
  begin
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (not E.Drawn) then Continue;
      PM1 := MakeTPoint2Df(E.X2PD, E.Y2PD);
      PM2 := MakeTPoint2Df(E.X2PG, E.Y2PG);
      if (IsSegmentInRectangle(PM1, PM2, MyRectVue)) then
      begin
        P1   := GetCoordsPlan(PM1);
        P2   := GetCoordsPlan(PM2);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color := clSilver;
        TmpBuffer.CanvasBGRA.MoveTo(P1); TmpBuffer.CanvasBGRA.LineTo(P2);
      end;
    end;
  end;

  procedure DrawGaleries;
  var
    i:      integer;
    E:      TEntiteEtendue;
    P:      array[0..3] of TPoint;
    q1, q2: boolean;
    PM0, PM1, PM2, PM3: TPoint2Df;
  begin
    q1 := (edFillGalerie in FArrOngletsParams[FCurrIdxOnglet].ongElementsDrawn);
    q2 := (edWalls in FArrOngletsParams[FCurrIdxOnglet].ongElementsDrawn);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      // topo de surface et cheminement spéciaux: sans objet
      if (E.Type_Entite = tgSURFACE)  then Continue;
      if (E.Type_Entite = tgENTRANCE) then Continue;
      // toutes visées hors topo de surface
      if (not E.Drawn) then  Continue;
      PM0 := MakeTPoint2Df(E.X1PG, E.Y1PG);
      PM1 := MakeTPoint2Df(E.X1PD, E.Y1PD);
      PM2 := MakeTPoint2Df(E.X2PD, E.Y2PD);
      PM3 := MakeTPoint2Df(E.X2PG, E.Y2PG);

      P[0] := GetCoordsPlan(PM0);
      P[1] := GetCoordsPlan(PM1);
      P[2] := GetCoordsPlan(PM2);
      P[3] := GetCoordsPlan(PM3);
      if (IsQuadInRectangle(PM0, PM1, PM2, PM3, MyRectVue)) then
      begin
        if (E.Type_Entite <> tgVISEE_RADIANTE) then
        begin
          if q1 then
          begin // remplissages
            //Brush.Color := FBDDEntites.GetCouleurEntiteRGBByExpe(E); //E.ColorEntite;
            TmpBuffer.CanvasBGRA.Brush.Opacity := FArrOngletsParams[FCurrIdxOnglet].ongFillOpacite;
            TmpBuffer.CanvasBGRA.Brush.Color := QGetCouleur(E);
            TmpBuffer.CanvasBGRA.Pen.Width := 0;
            TmpBuffer.CanvasBGRA.Pen.Color   := Brush.Color;
            TmpBuffer.CanvasBGRA.Polygon(P);
            TmpBuffer.CanvasBGRA.Brush.Opacity := 255;
          end;
          if q2 then
          begin // parois
            TmpBuffer.CanvasBGRA.Pen.Width := 0;
            TmpBuffer.CanvasBGRA.Pen.Color := clBlack;
            TmpBuffer.CanvasBGRA.MoveTo(P[0]); TmpBuffer.CanvasBGRA.LineTo(P[3]);
            TmpBuffer.CanvasBGRA.MoveTo(P[1]); TmpBuffer.CanvasBGRA.LineTo(P[2]);
          end;
        end
        else
        begin
          ;
        end;
      end;
    end;
  end;

  procedure DrawStations;
  var
    i:  integer;
    E:  TEntiteEtendue;
    P1: TPoint;
    PM: TPoint2Df;
    R : integer;
  begin
    //AfficherMessage(' --> DrawStations');
    R := 2;
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (not E.Drawn) then  Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      PM.X := E.Une_Station_2_X;
      PM.Y := E.Une_Station_2_Y;
      if (IsPointInRectangle(PM, MyRectVue)) then
      begin
        P1   := GetCoordsPlan(PM);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color   := clRed;
        TmpBuffer.CanvasBGRA.Brush.Color := TmpBuffer.CanvasBGRA.Pen.Color;
        TmpBuffer.CanvasBGRA.EllipseC(P1.X, P1.Y, R, R);
      end;
    end;
  end;
  procedure DrawPOIs();
  var
    i:  integer;
    E:  TEntiteEtendue;
    P1: TPoint;
    PM: TPoint2Df;
    R : integer;
  begin
    //AfficherMessage(' --> DrawStations');
    R := 4;
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (not E.Drawn) then  Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      if (E.IsPOI) then
      begin
        PM.X := E.Une_Station_2_X;
        PM.Y := E.Une_Station_2_Y;
        P1   := GetCoordsPlan(PM);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color   := clBlue;
        TmpBuffer.CanvasBGRA.Brush.Color := TmpBuffer.CanvasBGRA.Pen.Color;
        TmpBuffer.CanvasBGRA.EllipseC(P1.X, P1.Y, 2*R, 2*R);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color   := clAqua;
        TmpBuffer.CanvasBGRA.Brush.Color := TmpBuffer.CanvasBGRA.Pen.Color;
        TmpBuffer.CanvasBGRA.EllipseC(P1.X, P1.Y, R, R);
      end;
    end;
  end;

  procedure DrawCotation(const DoDelta: boolean);
  var
    i:  integer;
    E:  TEntiteEtendue;
    P:  TPoint;
    PM: TPoint2Df;
    QQ: TPoint3Df;
    WU: Double;
  begin
    //AfficherMessage(' --> DrawCotation');
    with TmpBuffer.CanvasBGRA do
    begin
      Pen.Width := 0;
      Brush.Color := clWhite;
      Brush.Style := bsSolid;
      Font.Color  := clBlue;
      Font.Name   := DEFAULT_FONT_NAME;
      Font.Style  := [fsItalic];
      Font.Height := FONT_HEIGHT_COTATION;
    end;
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      if (not E.Drawn) then Continue;

      QQ := FBDDEntites.GetDeltaXYZFromPositionSt0(E);
      PM.X := E.Une_Station_2_X;
      PM.Y := E.Une_Station_2_Y;
      if (IsPointInRectangle(PM, MyRectVue)) then
      begin
        P    := GetCoordsPlan(PM);
        if (DoDelta) then WU := QQ.Z
                     else WU := E.Une_Station_2_Z;
        TmpBuffer.CanvasBGRA.TextOut(P.X + 4, P.Y + 2, Format('%.1f', [WU]));
      end;
    end;
  end;

  procedure DrawIDStations;
  var
    i:  integer;
    E:  TEntiteEtendue;
    PM: TPoint2Df;
    P:  TPoint;
  begin
    with TmpBuffer.CanvasBGRA do
    begin
      Pen.Width := 0;
      Brush.Color := clWhite;
      Brush.Style := bsSolid;
      Font.Color  := clMaroon;
      Font.Name   := DEFAULT_FONT_NAME;
      Font.Style  := [];
      Font.Height := FONT_HEIGHT_IDSTATIONS;
    end;
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      if (not E.Drawn) then Continue;
      PM.X := E.Une_Station_2_X;
      PM.Y := E.Une_Station_2_Y;
      if (IsPointInRectangle(PM, MyRectVue)) then
      begin
        P    := GetCoordsPlan(PM);
        //TmpBuffer.CanvasBGRA.TextOut(P.X + 4, P.Y - Font.Size - 4, Trim(E.oIDLitteral));
        TmpBuffer.CanvasBGRA.TextOut(P.X + 4, P.Y - Font.Size - 4, MakeLibelleStationTopo(E));

      end;
    end;
  end;

  procedure DrawEntrancesOrPointEntities;
  var
    i, n:      integer;
    E:      TEntrance;
    P2: TPoint;
    PM:     TPoint2Df;
    R777: Int64;
    EWE: TEntiteEtendue;
  begin
    n := FBDDEntites.GetNbEntrees();
    if (n = 0) then exit;
    //AfficherMessage(' --> DrawEntrances');
    TmpBuffer.CanvasBGRA.Font.Color := clBlue;
    TmpBuffer.CanvasBGRA.Font.Style  := [fsBold];
    TmpBuffer.CanvasBGRA.Font.Height   := FONT_HEIGHT_ENTRANCES;
    R777 := Trunc(1.15 * R666);
    for i := 0 to n - 1 do
    begin
      E := FBDDEntites.GetEntree(i);
      TmpBuffer.CanvasBGRA.Pen.Width := 0;
      PM := MakeTPoint2Df(E.eXEntree, E.eYEntree);
      P2   := GetCoordsPlan(PM);
      TmpBuffer.CanvasBGRA.Brush.Color := clFuchsia;
      TmpBuffer.CanvasBGRA.EllipseC(P2.X, P2.Y, R666, R666);
      TmpBuffer.CanvasBGRA.Brush.Color := clWhite;
      TmpBuffer.CanvasBGRA.TextOut(P2.X + R777, P2.y + R777, _AnsiToLCLStr(E.eNomEntree));
    end; // for
  end;
  procedure DrawQuadrilles;
  var
    P1, P2:   TPoint;
    C1, C2:   TPoint2Df;
    // carré de la vue courante
    C1_VueCourante, C2_VueCourante: TPoint2Df;
    q, tt:     integer;
    A: double;
    S: string;
    qQuadrillageSpc: double;
  begin
    qQuadrillageSpc:= EnsureSetQuadrillageSpacing(FArrOngletsParams[FCurrIdxOnglet].ongQdrSpc);
    with FBDDEntites do
    begin
      with TmpBuffer.CanvasBGRA do
      begin
        Pen.Width := 0;
        Pen.Color := FArrOngletsParams[FCurrIdxOnglet].ongQdrColor;
        Font.Name := DEFAULT_FONT_NAME;
        Font.Height := FONT_HEIGHT_QUADRILLES;
      end;
      TmpBuffer.CanvasBGRA.Brush.Color := FArrOngletsParams[FCurrIdxOnglet].ongBackGround;
      // vue courante
      P1.X := 0;
      P1.Y := Vue.Height;
      C1_VueCourante := GetCoordsMonde(P1);
      P1.X := Vue.Width;
      P1.Y := 0;
      C2_VueCourante := GetCoordsMonde(P1);
      tt := trunc(C1_VueCourante.X / qQuadrillageSpc);
      A := qQuadrillageSpc * tt;
      while (A < C2_VueCourante.X) do
      begin
        C1 := MakeTPoint2Df(A, C1_VueCourante.Y);
        C2 := MakeTPoint2Df(A, C2_VueCourante.Y);
        P1   := GetCoordsPlan(C1);
        P2   := GetCoordsPlan(C2);
        TmpBuffer.CanvasBGRA.MoveTo(P1.X, P1.Y);
        TmpBuffer.CanvasBGRA.LineTo(P2.X, P2.Y);
        //-------- Coordonnées en rive
        S:=Format('%.0f',[A]);
        q:=P1.X - TmpBuffer.CanvasBGRA.TextWidth(S) div 2;
        TmpBuffer.CanvasBGRA.TextOut(q, 1, S);
        //----------------------------
        A := A + qQuadrillageSpc;
      end;
      //tt:=trunc(cnMini.Y / QdrEspc);
      tt := trunc(C1_VueCourante.Y / qQuadrillageSpc);
      A := qQuadrillageSpc * tt;
      while (A < C2_VueCourante.Y) do
      begin
        C1 := MakeTPoint2Df(C1_VueCourante.X, A);
        C2 := MakeTPoint2Df(C2_VueCourante.X, A);

        P1 := GetCoordsPlan(C1);
        P2 := GetCoordsPlan(C2);
        TmpBuffer.CanvasBGRA.MoveTo(P1); TmpBuffer.CanvasBGRA.LineTo(P2);
        //-------- Coordonnées en rive
        S:=Format('%.0f',[A]);
        q:=P1.Y - TmpBuffer.CanvasBGRA.TextHeight(S) div 2;
        TmpBuffer.CanvasBGRA.TextOut(2, q, S);
        //----------------------------
        A := A + qQuadrillageSpc;
      end;

    end;// with Ftable...
  end;
  procedure DrawCadre;
  var
    C1, C2: TPoint3Df;
    F1, F2: TPoint2Df;
    P1, P2: TPoint;
  begin
    C1 := FBDDEntites.GetCoinBasGauche;
    C2 := FBDDEntites.GetCoinHautDroit;
    F1 := MakeTPoint2Df(C1.X, C1.Y);
    F2 := MakeTPoint2Df(C2.X, C2.Y);
    P1 := GetCoordsPlan(F1);
    P2 := GetCoordsPlan(F2);
    with TmpBuffer.CanvasBGRA do
    begin
      Pen.Width := 0;
      Pen.Color:=clRed;
      MoveTo(P1.X, P1.Y);
      LineTo(P2.X, P1.Y);
      LineTo(P2.X, P2.Y);
      LineTo(P1.X, P2.Y);
      LineTo(P1.X, P1.Y);
    end;
  end;

  procedure DrawMaillage;
  var
    QIsoValeurs: TArrayOfFloats;
  begin
    if (not assigned(FMyMaillage)) then Exit;
    SetLength(QIsoValeurs, 1);
    QIsoValeurs[0] := editIsoValeur.Value;
    //try
      if (FMyMaillage.IsValidMaillage() and FMaillageReady) then
      begin
        AfficherMessage(Format('viewBox    : %.0f, %.0f -> %.0f, %.0f', [FZC1.X, FZC1.Y, FZC2.X, FZC2.Y]));
        FMyMaillage.TracerMaillage(TmpBuffer,
                                    GetCoordsPlan,
                                    10.00,
                                    QIsoValeurs,
                                    btnCouleurCourbeNiveau.ButtonColor,
                                    clFuchsia,
                                    false,
                                    false,
                                    MyRectVue);
      end;
    //except
    //end;
  end;
  procedure DrawAntennes;
  var
    i: Integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
    PM1, PM2: TPoint2Df;
  begin
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if ((E.Type_Entite = tgVISEE_RADIANTE) and (E.Drawn)) then
      begin
        PM1 := MakeTPoint2Df(E.Une_Station_1_X, E.Une_Station_1_Y);
        PM2 := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
        if (IsSegmentInRectangle(PM1, PM2, MyRectVue)) then
        begin
          P1 := GetCoordsPlan(PM1);
          P2 := GetCoordsPlan(PM2);
          TmpBuffer.CanvasBGRA.Pen.Width := 0;
          TmpBuffer.CanvasBGRA.Pen.Color := clSilver;
          TmpBuffer.CanvasBGRA.Brush.Color := clSilver;
          TmpBuffer.CanvasBGRA.MoveTo(P1.X, P1.Y);
          TmpBuffer.CanvasBGRA.LineTo(P2.X, P2.Y);
          //TmpBuffer.CanvasBGRA.EllipseC(P2.X, P2.Y, 2, 2);
        end;
      end; // if E.Type_Entite=tgENTRANCE
    end;
  end;
  procedure DrawMarker(const MK: TMarker);
  var
    PM: TPoint2Df;
    P1, P2: TPoint;
    SZ: TSize;
    EWE: TRect;
    WU: String;

  begin
     PM := MakeTPoint2Df(MK.X, MK.Y);
     if (IsPointInRectangle(PM, MyRectVue)) then
     begin
       P1 := GetCoordsPlan(PM);
       TmpBuffer.CanvasBGRA.Pen.Width := 0;
       TmpBuffer.CanvasBGRA.pen.Color := clBlack;
       TmpBuffer.CanvasBGRA.Brush.Style := bsSolid;
       TmpBuffer.CanvasBGRA.Brush.Color := MK.Couleur;
       // ligne d'attache
       P2 := Point(P1.X + 45, P1.Y + 45);
       TmpBuffer.CanvasBGRA.MoveTo(P1);
       TmpBuffer.CanvasBGRA.LineTo(P2);
       WU := ' ' + MK.Caption + ' ';
       SZ  := TmpBuffer.CanvasBGRA.TextExtent(WU);
       EWE := Rect(P2.X, P2.y, P2.x + SZ.cx, P2.y + SZ.cy);
       TmpBuffer.CanvasBGRA.Rectangle(EWE);
       TmpBuffer.CanvasBGRA.TextOut(P2.X + 1, P2.y, WU);
     end;
  end;
  procedure DrawAnnotations();
  var
    i, n: Integer;
    MyAnnotation: TAnnotation;
    QEntity: TEntiteEtendue;
    PM: TPoint2Df;
    QTxtAttr: TTexteAttributs;
  begin
    n := FDocuTopo.GetNbAnnotations();
    AfficherMessage(Format('-- DrawAnnotations(%d)', [n]));
    if (n = 0) then Exit;

    for i := 0 to n - 1 do
    begin
      MyAnnotation := FDocuTopo.GetAnnotation(i);
      if (FBDDEntites.GetEntiteViseeFromSerSt(MyAnnotation.Basepoint.eSerie,
                                               MyAnnotation.Basepoint.ePoint,
                                               QEntity)) then
      begin
        QTxtAttr := FDocuTopo.GetStyleAnnotation(MyAnnotation.IdxStyle);
        PM := MakeTPoint2Df(QEntity.Une_Station_2_X + MyAnnotation.Offset.X,
                            QEntity.Une_Station_2_Y + MyAnnotation.Offset.Y);
        QDrawTexte(TmpBuffer, PM.X, PM.Y, QTxtAttr, MyAnnotation.Texte);
      end;
    end;

  end;
  procedure DrawJonctions();
  const
    R = 3;
  var
    n, i: Integer;
    J : TJonctionXYZ;
    PM: TPoint2Df;
    P1: TPoint;
    QTxt: String;
  begin
    n := FBDDEntites.GetNbJonctions();
    AfficherMessage(Format('-- DrawJonctions(%d)', [n]));
    if (n = 0) then exit;
    TmpBuffer.CanvasBGRA.Font.Style  := TmpBuffer.CanvasBGRA.Font.Style + [fsBold];
    TmpBuffer.CanvasBGRA.Brush.Opacity := 128;
    for i := 0 to n - 1 do
    begin
      J  := FBDDEntites.GetJonction(i);
      //AfficherMessageErreur(Format('%d - %f, %f', [i, J.X, J.Y]));
      PM := MakeTPoint2Df(J.X, J.Y);
      if (IsPointInRectangle(PM, MyRectVue)) then
      begin
        P1   := GetCoordsPlan(PM);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color   := clRed;
        TmpBuffer.CanvasBGRA.Brush.Color := clBlue;
        TmpBuffer.CanvasBGRA.EllipseC(P1.X, P1.Y, R, R);
        TmpBuffer.CanvasBGRA.Brush.Color := clWhite;

        //QTxt := Format('%d.%d', [J.oNoSer, J.oNoSt]);
        //QRect := TmpBuffer.Canvas.TextExtent(QTxt);
        TmpBuffer.CanvasBGRA.TextOut(P1.x + R + 1, P1.y - R - 1, MakeLabelNoeud(J));
      end;
    end;
    TmpBuffer.CanvasBGRA.Font.Style := TmpBuffer.CanvasBGRA.Font.Style - [fsBold];
    TmpBuffer.CanvasBGRA.Brush.Opacity := 255;
  end;
  procedure DrawCurrentStationTopo();
  const DEMI_COTE = 6;
  var
    PP: TPoint;
    PM: TPoint2Df;
    QTxt: String;
    QRect: TSize;
  begin
    try
      PM := MakeTPoint2Df(FCurrentEntite.Une_Station_2_X, FCurrentEntite.Une_Station_2_Y);
      if (IsPointInRectangle(PM, MyRectVue)) then
      begin
        PP := GetCoordsPlan(PM);
        TmpBuffer.CanvasBGRA.Pen.Width := 0;
        TmpBuffer.CanvasBGRA.Pen.Color   := clBlue;
        TmpBuffer.CanvasBGRA.Brush.Style := bsSolid;
        TmpBuffer.CanvasBGRA.brush.Color := clAqua;
        TmpBuffer.CanvasBGRA.EllipseC(PP.X, PP.Y, DEMI_COTE, DEMI_COTE);
        TmpBuffer.CanvasBGRA.brush.Color := clYellow;

        TmpBuffer.CanvasBGRA.Font.Style := TmpBuffer.CanvasBGRA.Font.Style + [fsBold];
        QTxt  := MakeLibelleStationTopo(FCurrentEntite);// Format('%d.%d - %s', [FCurrentEntite.Entite_Serie, FCurrentEntite.Entite_Station, FCurrentEntite.oIDLitteral]);
        QRect := TmpBuffer.CanvasBGRA.TextExtent(QTxt);
        TmpBuffer.CanvasBGRA.TextOut(PP.X + DEMI_COTE + 1,
                                 PP.Y - DEMI_COTE - 1,
                                 QTxt);

        TmpBuffer.CanvasBGRA.Font.Style := TmpBuffer.CanvasBGRA.Font.Style - [fsBold];
      end;
    except
      ;
    end;
  end;

begin
  if (not FCanDraw) then Exit;
  // FDessinEnCours est un flag pour protéger la reconstruction du dessin
  if (FDessinEnCours) then
  begin
    AfficherMessage('*** Reconstruction non terminee ***');
    Exit;
  end;
  // attrapper le rectangle de vue
  MyRectVue := MakeTRect2Df(FRegionXMini, FRegionYMini, FRegionXMaxi, FRegionYMaxi);
  AfficherMessage(Format('MyRectVue: %.2f, %.2f > %.2f, %.2f', [MyRectVue.X1, MyRectVue.Y1, MyRectVue.X2, MyRectVue.Y2]));
  FDessinEnCours := True;
  // paramètres d'onglet
  MonOnglet := FArrOngletsParams[FCurrIdxOnglet];
  // recalculer les dégradés si DoRecalcDegrades est armé
  if (DoRecalcDegrades) then FBDDEntites.CalcCouleursByDepth(MonOnglet.ongDegradeStart, MonOnglet.ongDegradeStop);
  TmpBuffer := TBGRABitmap.Create(vue.Width, vue.Height);
  try
    R.Left   := Vue.Left;
    R.Top    := Vue.Top;
    R.Bottom := Vue.Top  + Vue.Height;
    R.Right  := Vue.Left + Vue.Width;
    TmpBuffer.CanvasBGRA.Brush.Color := MonOnglet.ongBackGround;
    TmpBuffer.CanvasBGRA.Brush.Opacity := 255;
    TmpBuffer.CanvasBGRA.FillRect(R);
     // maillage
    DrawMaillage;
    //DrawPipistrelle;
    DrawGaleries;
    if (edQuadrilles    in MonOnglet.ongElementsDrawn) then DrawQuadrilles;
    if (edANTENNES      in MonOnglet.ongElementsDrawn) then DrawAntennes;
    if (edPolygonals    in MonOnglet.ongElementsDrawn) then DrawPolygonals;
    if (edCrossSections in MonOnglet.ongElementsDrawn) then DrawSections;
    if (edStations      in MonOnglet.ongElementsDrawn) then DrawStations;
    if (edAltitudes     in MonOnglet.ongElementsDrawn) then DrawCotation(False);
    if (edCotes         in MonOnglet.ongElementsDrawn) then DrawCotation(True);
    if (edIDStations    in MonOnglet.ongElementsDrawn) then DrawIDStations;
    if (edENTRANCES     in MonOnglet.ongElementsDrawn) then DrawEntrancesOrPointEntities;
    if (edANNOTATIONS   in MonOnglet.ongElementsDrawn) then DrawAnnotations();
    if (edJONCTIONS     in MonOnglet.ongElementsDrawn) then DrawJonctions();
    DrawPOIs();
    if (FCurrentMarker.Displayed) then DrawMarker(FCurrentMarker);
    DrawPipistrelle(TmpBuffer);            // logo pipistrelle Nord
    // mise en évidence de l'entité courante (uniquement à l'écran)
    if (not DoExportInFile) then DrawCurrentStationTopo();
    //DrawBandeauDeuil(TmpBuffer.CanvasBGRA);
    TmpBuffer.Draw(vue.Canvas, 0, 0, True);

    {$IFDEF USE_TBGRA_BITMAP}
       //TmpBuffer.Draw(vue.Canvas, R, True);
    {$ELSE}
       if (DoExportInFile) then TmpBuffer.SaveToFile(QFileName)
                           else vue.Canvas.CopyRect(R, TmpBuffer.Canvas, R);
    {$ENDIF}

  finally
    TmpBuffer.Free;
    FDessinEnCours := False;
  end;
  // affichage de débug pour les onglets
  // if (pnlDebugOnglet.Visible) then DisplayDebugOnglet(FArrOngletsParams[FCurrIdxOnglet]);
end;


procedure TGHTopoContext2DA.SetModeRepresentation(const MR: TModeRepresentationGaleries);
var
  EWE: TVue2DParams;
begin
  FModeRepresentationGaleries := MR;
  // feedback: redessin vue
  EWE := FArrOngletsParams[FCurrIdxOnglet];
  FBDDEntites.CalcCouleursByDepth(EWE.ongDegradeStart, EWE.ongDegradeStop);
  //Vue.Invalidate;
end;
procedure TGHTopoContext2DA.SetModeSelectionListe(const M: TModeSelectionListe);
begin
  FModeSelectionListe := M;
end;
//------------------------------------------------------------------------------
// centrer le dessin sur le point trouvé


procedure TGHTopoContext2DA.CentrerVueSurPointXY(const QX, QY: double; const DoMarker: boolean; const LbMarker: string);
var
  C0, Offset: TPoint2Df;
  DbgTag: String;
begin
  try
    C0.X := 0.5 * (FRegionXMini + FRegionXMaxi);
    C0.Y := 0.5 * (FRegionYMini + FRegionYMaxi);
    Offset.X := QX - C0.X;
    Offset.Y := QY - C0.Y;
    DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
    SetViewLimits(FRegionXMini + Offset.X, FRegionYMini + Offset.Y,
                  FRegionXMaxi + Offset.X, FRegionYMaxi + Offset.Y,
                  DbgTag);
    SetMarker(MakeTMarker(DoMarker, QX, QY, clAqua, LbMarker));
    RedessinEcran(False);
  except
  end;

end;
procedure TGHTopoContext2DA.SetMarker(const M: TMarker);
begin
  FCurrentMarker := M;
end;

//******************************************************************************
procedure TGHTopoContext2DA.QDrawTexte(const Cnv: TBGRABitmap; const X, Y: double;  const Attr: TTexteAttributs; const Texte: string);
var
  PM, PM1: TPoint2Df;
  PP, PP1: TPoint;
  BB: TSize;
  dx, dy: integer;
begin
  PM  := MakeTPoint2Df(X, Y);
  PM1 := MakeTPoint2Df(PM.X, PM.Y + Attr.HauteurTexte);
  PP := GetCoordsPlan(PM);
  PP1:= GetCoordsPlan(PM1);
  Cnv.CanvasBGRA.Font.Color := Attr.FontColor;
  Cnv.CanvasBGRA.Font.Name  := Attr.FontName;
  Cnv.CanvasBGRA.Font.Style := Attr.FontStyle;
  Cnv.CanvasBGRA.Font.Height  := PP1.Y - PP.Y;
  Cnv.CanvasBGRA.Font.Orientation := Attr.AngleRot * 100;
  BB := Cnv.CanvasBGRA.TextExtent(Texte);
  // alignement vertical
  case Attr.Position of
    1, 2, 3: dy := -BB.cy;
    4, 5, 6: dy := -BB.cy div 2;
    7, 8, 9: dy := 0;
  else
    dy := 0;
  end;
  // alignement horizontal
  case Attr.Position of
    1, 4, 7: dx := 0;
    2, 5, 8: dx := -BB.cx div 2;
    3, 6, 9: dx := -BB.cx;
  else
    dx := 0;
  end;

  if (Attr.AngleRot = 0) then // process simplifié = dessin direct
  Cnv.CanvasBGRA.TextOut(PP.X + dx, PP.Y + dy, Texte);
end;



procedure TGHTopoContext2DA.lbInfosClick(Sender: TObject);
begin
  pnlFullInfos.Visible := not pnlFullInfos.Visible;
  if (pnlFullInfos.Visible) then DisplayFullInfosStation(FCurrentEntite);
end;






procedure TGHTopoContext2DA.TraiterViseeRecueDuDistoX(const V: TMesureViseeDistoX);
var
  MyVisee: TUneVisee;
  QErrMsg: string;
  n, i: Integer;
  M: TMesureViseeDistoX;
  QErr: TErrorConstruireUneViseeValide;
begin
  try
    if (not Assigned(FPtrDistoX)) then
    begin
      showmessage('FPtrDistoX non assigné');
      exit;
    end;
    QErrMsg := '';
    n := FPtrDistoX.GetNbMesuresAcquises();
    AfficherMessageErreur(Format('%s.TraiterViseeRecueDuDistoX(): Série dest: %d: L = %.2f, Az = %.2f, P = %.2f (%d mesures)',
                          [ClassName,
                           FCurrentNoSerieForDistoX,
                           V.Longueur, V.Azimut, V.Pente,
                           n]));

    // afficher les 3 dernières mesures

    if (n > 3) then
    begin
      for i := 0 to 3 do
      begin
        M := FPtrDistoX.GetMesureVisee(n - 1 - i);
        AfficherMessageErreur(Format('%d: %.3f, %.3f, %.3f', [i, M.Longueur, M.Azimut, M.Pente]));
      end;
    end;

    if (FPtrDistoX.ConstruireUneViseeValide(tgDEFAULT, FCurrentEntite.eSecteur, FCurrentEntite.eCode, FCurrentEntite.eExpe, MyVisee, QErr)) then
    begin
      ShowMessage('OK');
      AjouterUneViseeALaSerieCourante(tgDEFAULT,
                                      MyVisee.Longueur,
                                      MyVisee.Azimut,
                                      MyVisee.Pente,
                                      1.00, 1.00, 1.00, 1.00,
                                      '', //trim(editIDTerrain.Text),
                                      '', //trim(editObservations.Text),
                                      True);
    end
    else
    begin  // sinon ajouter la visée à la table des antennes
      ;
    end;
  except
    on e: exception do AfficherMessageErreur(E.Message);
  end;
end;




procedure TGHTopoContext2DA.Metafiltrer(const lbFiltres: string);
var
  QDevelViseesVisibles: double;
begin
  FBDDEntites.MetaFiltre(lbFiltres, QDevelViseesVisibles);
  lbLongViseesVisibles.Caption := Format('%.2f', [QDevelViseesVisibles]);
  FBDDEntites.SetMinMax(False);
  RedessinEcran(True);
end;



function TGHTopoContext2DA.ChargerMaillageHGT(const QFileName: string): boolean;
begin
  Result := FMyMaillage.ChargerMaillageDepuisHGT(QFileName);
end;

function TGHTopoContext2DA.IsMaillageReady(): boolean;
begin
  Result := FMaillageReady and FMyMaillage.IsValidMaillage();
end;

function TGHTopoContext2DA.GetMaillage(): TMaillage;
begin
  result := FMyMaillage;
end;

function TGHTopoContext2DA.GetDocuTopo(): TToporobotStructure2012;
begin
  Result := FDocuTopo;
end;

//******************************************************************************
procedure TGHTopoContext2DA.ZoomFact(const Fact: double);
var
  ZoomCentre: TPoint2Df;
  RX, RY: Double;
  DbgTag: String;
begin
  // point central
  ZoomCentre := MakeTPoint2Df(0.50 * (FRegionXMaxi + FRegionXMini),
                              0.50 * (FRegionYMaxi + FRegionYMini));
  // demi-rayon
  RX := 0.50 * (FRegionXMaxi - FRegionXMini) / Fact;
  RY := 0.50 * (FRegionYMaxi - FRegionYMini) / Fact;
  DbgTag := Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]);
  self.SetViewLimits(ZoomCentre.X - RX, ZoomCentre.Y - RY,
                     ZoomCentre.X + RX, ZoomCentre.Y + RY,
                     DbgTag);
  Vue.Invalidate;
end;

procedure TGHTopoContext2DA.LocaliserUneStation();
var
  EWE: String;
  QMyEntite: TEntiteEtendue;
begin
  EWE := '';
  if (InputQuery('Localiser une station', 'ID station', EWE)) then
  begin
    if (FBDDEntites.FindStationByCle(false, EWE, QMyEntite)) then
    begin
      EWE := Format('%d.%d', [QMyEntite.Entite_Serie, QMyEntite.Entite_Station]);
      self.CentrerVueSurPointXY(QMyEntite.Une_Station_2_X, QMyEntite.Une_Station_2_Y, True, EWE);
      self.SetCurrentEntite(QMyEntite);
    end
    else
    begin
      ShowMessage(rsMATCHNOTFOUND);
    end;
  end;
end;

procedure TGHTopoContext2DA.CreerTimeLapse(const QDirectory: TStringDirectoryFilename);
var
  NbSeances, i: Integer;
  QMetaFiltre: String;
  QFilename  : string;
  procedure CreerImg(const QFiltre: string; const QFileOutput: string);
  begin
    AfficherMessage(Format('--> %s -> %s', [QFiltre, QFileOutput]));
    ApplyMetaFiltre(True, QMetaFiltre);
    RedessinEcran(false, True, QFileOutput);
    Application.ProcessMessages;
  end;
begin
  AfficherMessage(Format('%s.CreerTimeLapse("%s")', [ClassName, QDirectory]));
  ForceDirectories(QDirectory);         // forcer la création du dossier
  NbSeances := FBDDEntites.GetNbExpes();
  i := 0;
  while (i < NbSeances) do
  begin
    QMetaFiltre := Format('%s < %d', [rsMETAFILTRE_EXPE, i]);
    QFilename := QDirectory + DirectorySeparator + Format('Img_%d.png', [i]);
    if (not FDessinEnCours) then
    begin
      CreerImg(QMetaFiltre, QFilename);
      Inc(i);
    end
    else
      AfficherMessage('** Is busy **');
  end;
end;

procedure TGHTopoContext2DA.setPtrDistoX(const D: TPilotageDistoX);
begin
  try
    ShowMessage(Format('%s.setPtrDistoX()', [ClassName]));
    FPtrDistoX := D;
    FPtrDistoX.SetProcTransmitViseeToGHTopoCtxt2D(TraiterViseeRecueDuDistoX);
    mnuDistoX.Visible := True;

  except
  end;
end;

// dessin de textes vectoriels (type AutoCAD old-school)
// TODO: Gestion de la rotation à compléter
//       Alphabet à compléter (au fil des besoins)
procedure TGHTopoContext2DA.DrawTexteVectoriel(const Position     : TPoint2Df;
                                               const HauteurChar, LargeurChar: double;
                                               const DistToLigneAttache: double;
                                               const Alignment    : Byte;
                                               const OrientationInRadians  : double;
                                               const Texte: string);
const
  NB_NODES_CHARS = 26;
  // coordonnées des noeuds de caractère exprimés dans un repère local (grille 4x8)
  CharNodes: array [0 .. NB_NODES_CHARS] of TPoint2Df = (
                (X : 0.00; Y :  0.00),
                (X : 1.00; Y :  0.00),
                (X : 2.00; Y :  0.00),
                (X : 3.00; Y :  0.00),
                (X : 4.00; Y :  0.00),
                (X : 4.00; Y :  1.00),
                (X : 4.00; Y :  3.00),
                (X : 4.00; Y :  4.00),
                (X : 4.00; Y :  5.00),
                (X : 4.00; Y :  7.00),
                (X : 4.00; Y :  8.00),
                (X : 3.00; Y :  8.00),
                (X : 2.00; Y :  8.00),
                (X : 1.00; Y :  8.00),
                (X : 0.00; Y :  8.00),
                (X : 0.00; Y :  7.00),
                (X : 0.00; Y :  5.00),
                (X : 0.00; Y :  4.00),
                (X : 0.00; Y :  3.00),
                (X : 0.00; Y :  1.00),
                (X : 1.00; Y :  4.00),
                (X : 2.00; Y :  4.00),
                (X : 3.00; Y :  4.00),
                (X : 1.00; Y :  0.25),
                (X : 1.25; Y :  0.25),
                (X : 1.25; Y :  0.50),
                (X : 1.00; Y :  0.50)
             );
var
  nbCar, i: integer;
  ca, sa: extended;
  QLargCaractereWSpacing: double;
  QModuleH, QModuleV, QLargeurTexte: Double;
  CoordCharNodes : array[0..NB_NODES_CHARS] of TPoint2Df;
  PP: TPoint;
  QOldColorPen: TColor;
  BP: TPoint2Df;
  procedure QPreparerParametresRotation();
  var
    QOr: Extended;
  begin
    QOr := OrientationInRadians;//IIF(OrientationInRadians > PI / 2, OrientationInRadians, OrientationInRadians - PI);
    sincos(QOr, sa, ca);
  end;
  // transforme un point exprimé dans le repère local au texte
  // en un point exprimé dan le GCS
  function QTransformPoint(const PT: TPoint2Df): TPoint2Df;
  begin
    Result.X := Position.X + ( ca * PT.X - sa * PT.Y);
    Result.Y := Position.Y + DistToLigneAttache + ( sa * PT.X + ca * PT.Y);
  end;
  // Dessin d'un caractère
  procedure QDrawCaractere(const QPos: integer; const QChar: Char);
    // Dessin d'une suite de segments, de type PU; PA x0, y0; PD; PA x1, y1; ...
    // sans lever la plume
    procedure QQ1(const QCarNodeList: array of integer);
    var
      QNbNds, p: Integer;
      QQPM: TPoint2Df;
      QQPP: TPoint;
    begin
      QNbNds := length(QCarNodeList);
      if (QNbNds < 2) then exit;
      QQPM := CoordCharNodes[QCarNodeList[0]];
      QQPP := GetCoordsPlan(QQPM);
      Vue.Canvas.MoveTo(QQPP);  //PU; PA x0, y0
      for p := 1 to QNbNds - 1 do
      begin
        QQPM := CoordCharNodes[QCarNodeList[p]];
        QQPP := GetCoordsPlan(QQPM);
        Vue.Canvas.LineTo(QQPP); // PA xn, yn;
      end;
    end;
  var
    n: Integer;
    QPM, QOffset: TPoint2Df;
  begin
    QOffset := MakeTPoint2Df(0.00, 0.00);
    case Alignment of
      5: QOffset.X := -QLargeurTexte / 2.00;
    else
      ;
    end;
    for n := 0 to NB_NODES_CHARS do
    begin
      QPM := MakeTPoint2Df(QPos * QLargCaractereWSpacing + QOffset.X + QModuleH * CharNodes[n].X,
                                                           QOffset.Y + QModuleV * CharNodes[n].Y);
      CoordCharNodes[n] := QTransformPoint(QPM);
    end;
    case QChar of
     '0': QQ1([1, 3, 5, 9, 11, 13, 15, 19, 9]);
     '1': begin
            QQ1([16, 12, 2]);
            QQ1([0, 2, 3, 4]);
          end;
     '2': QQ1([16, 15, 13, 11, 9, 8, 0, 4]);
     '3': begin
            QQ1([15, 13, 11, 9, 8, 22, 6, 5, 3, 1, 19]);
            QQ1([20, 22]);
          end;
     '4': QQ1([3, 11, 12, 18, 6]);
     '5': QQ1([10, 14, 16, 20, 22, 6, 5, 3, 1, 19]);
     '6': QQ1([9, 11, 13, 15, 19, 1, 3, 5, 6, 22, 17]);
     '7': QQ1([0, 9, 10, 14, 15]);
     '8': QQ1([20, 18, 19, 1, 3, 5, 6, 22, 8, 9, 11, 13, 15, 16, 20, 22]);
     '9': QQ1([19, 1, 3, 5, 9, 11, 13, 15, 16, 20, 7]);
     '.', ',': QQ1([23, 24, 25, 26, 23]);
    else
      ;//QQ1([1, 3, 5, 9, 11, 13, 15, 19, 9]);
    end;

  end;


begin
  AfficherMessageErreur(format('%s.DrawTexteVectoriel(HT=%.2f - Or: %.2f)', [ClassName, HauteurChar, radtodeg(OrientationInRadians)]));
  NbCar := length(Texte);
  QLargCaractereWSpacing := LargeurChar * 1.25; // = 8 unités pour le caractère + 2 unités pour l'espace intercaractères
  QModuleH := 0.250 * LargeurChar;
  QModuleV := 0.125 * HauteurChar;
  QLargeurTexte := NbCar * QLargCaractereWSpacing;
  PP := GetCoordsPlan(Position);
  QOldColorPen := Vue.Canvas.Pen.Color;
  Vue.Canvas.Pen.Color := clBlue;
  //Vue.Canvas.EllipseC(PP.X, PP.Y, 8, 8);
  QPreparerParametresRotation();
  for i := 0 to nbCar - 1 do
  begin
    BP := QTransformPoint(MakeTPoint2Df(i * QLargCaractereWSpacing, 0));
    PP := GetCoordsPlan(BP);
    //Vue.Canvas.EllipseC(PP.X, PP.Y, 3, 3);
    QDrawCaractere(i, Texte[1+i]);
  end;

  Vue.Canvas.Pen.Color := QOldColorPen;






end;

end.

