unit CadreHistoDepthExt;
{$INCLUDE CompilationParameters.inc}
// Date: 07/11/2013
// Statut: Fonctionnel
interface
uses
  StructuresDonnees,
  Common,
  UnitEntitesExtended,
  Math,
  Graphics,
  BGRABitmap,
  SVGCanvasUnit,
  Classes, SysUtils, FileUtil, Forms, Controls, ExtCtrls, StdCtrls;
type

  { TCdrDepthDiagramExt }

  TCdrDepthDiagramExt = class(TFrame)
    lbZMax: TLabel;
    lbZMin: TLabel;
    imgHistogrammes: TPaintBox;
    Panel1: TPanel;
    lbMousePos: TStaticText;
    procedure imgHistogrammesMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure imgHistogrammesPaint(Sender: TObject);
  private
    { private declarations }
    FDoDraw: boolean;
    FBDDEntites   : TBDDEntites;
    FDoubleBuffer : TBGRABitmap;
    FClassesRepartDepthDiagram: TVecteurDouble;
    FFiltres      : string;
    FNbBarres    : integer;
    FLongMini     : double;
    FColorBarreLine : TColor;
    FColorBarreFill : TColor;

    //FLargZoneDiagramme: double;
    //FHauteurZoneDiagramme: double; // longueur max des barres
    FPourcentageSeuilHightLight: double;


    FCMin : TPoint2Df;
    FCMax : TPoint2Df;
    FRx, FRy: double;
    //FZ, FH: Double;


    procedure DessinerHistogramme();

    function  QCalcP2D(const GP: TPoint2Df): TPoint;
    function  QCalcP2G(const PP: TPoint; out P2G: TPoint2Df): boolean;
    procedure QDrawLigne(const P1, P2: TPoint2Df);
    procedure QDrawRectangle(const P1, P2: TPoint2Df);
    procedure SetZoneDiagrammeLimits(const X1, Y1, X2, Y2: double);
    procedure SetColorsBarres(const CLB, CFB: TColor);
  public
    { public declarations }
    function Initialise(const DT: TBDDEntites;
                        const Filtres: string;
                        const NbBarres: integer;
                        const LMini: double;
                        const BarresLineColor: TColor;
                        const BarresFillColor: TColor;
                        const PourcentageSeuilHightLight: double): boolean;
    procedure ExportDiagramToSVG(const QFileName: TStringDirectoryFilename; const QSize: double);
  end;

implementation

{$R *.lfm}

{ TCdrDepthDiagramExt }

procedure TCdrDepthDiagramExt.SetZoneDiagrammeLimits(const X1, Y1, X2, Y2: double);
begin
  FCMin := MakeTPoint2Df(X1, Y1);
  FCMax := MakeTPoint2Df(X2, Y2);
  FRx   := 1 / (FCMax.X - FCMin.X);
  FRy   := 1 / (FCMax.Y - FCMin.Y);
end;

procedure TCdrDepthDiagramExt.SetColorsBarres(const CLB, CFB: TColor);
begin
  FColorBarreFill := CFB;
  FColorBarreLine := CLB;
end;

procedure TCdrDepthDiagramExt.imgHistogrammesMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  PG: TPoint2Df;
begin
  if (QCalcP2G(MakeTPoint(X, Y), PG)) then
    lbMousePos.Caption := format('%.2f, %.2f', [PG.X, PG.Y])
  else
    lbMousePos.Caption := 'In margins';
end;

procedure TCdrDepthDiagramExt.imgHistogrammesPaint(Sender: TObject);
begin
  DessinerHistogramme();
end;


procedure TCdrDepthDiagramExt.DessinerHistogramme();
var
  i, NbBarres           : integer;
  LargZoneDiagramme     : double;
  HauteurZoneDiagramme  : double;
  QZ, QH, PourcentageMax: double;
  R                     : TRect;
  CBG, CHD              : TPoint3Df;
  EWE: TPoint;
begin
  AfficherMessage(Format('%s.DessinerHistogramme("%s" - %f)',[ClassName, FFiltres, FPourcentageSeuilHightLight]));
  if (not FDoDraw) then Exit;
  // longueurs des barres
  NbBarres := 1 + High(FClassesRepartDepthDiagram);
  PourcentageMax := -1.00;
  // /!\ Les valeurs de FClassRepartDepthDiagram[] sont comprises entre 0.00 et 1.00
  for i := 0 to NbBarres - 1 do PourcentageMax := Max(PourcentageMax, FClassesRepartDepthDiagram[i]);
  LargZoneDiagramme    := PourcentageMax * 200.00;

  CBG := FBDDEntites.GetCoinBasGauche();
  CHD := FBDDEntites.GetCoinHautDroit();
  lbZMax.Caption := Format('%.0f', [CHD.Z]);
  lbZMin.Caption := Format('%.0f', [CBG.Z]);

  HauteurZoneDiagramme := CHD.Z - CBG.Z; //NbBarres * 10.00;
  SetZoneDiagrammeLimits(-15.00, CBG.Z, LargZoneDiagramme + 10.00, CHD.Z);
  try
    AfficherMessage(Format('Min: %.2f, %.2f - Max: %.2f, %.2f', [FCMin.X, FCMin.Y, FCMax.X, FCMax.Y]));
    AfficherMessage(Format('%d barres, LMax = %.2f', [NbBarres, LargZoneDiagramme]));
    FDoubleBuffer := TBGRABitmap.Create(imgHistogrammes.Width, imgHistogrammes.Height);
    //FDoubleBuffer.Height := ;
    //FDoubleBuffer.Width  := ;


    R.Left   := imgHistogrammes.Left;
    R.Top    := imgHistogrammes.Top;
    R.Bottom := imgHistogrammes.Top  + imgHistogrammes.Height;
    R.Right  := imgHistogrammes.Left + imgHistogrammes.Width;

    FDoubleBuffer.CanvasBGRA.Brush.Color    := clWhite;
    FDoubleBuffer.CanvasBGRA.Brush.Opacity  := 255;

    FDoubleBuffer.CanvasBGRA.FillRect(R);
    QDrawRectangle(FCMin, FCMax);
    FDoubleBuffer.CanvasBGRA.Pen.Width := 0;
    FDoubleBuffer.CanvasBGRA.Pen.Color := FColorBarreLine;


    QH := HauteurZoneDiagramme / NbBarres;
    QZ := CBG.Z;// + QH / 2;

    for i := 0 to NbBarres - 1 do
    begin
      QZ := CBG.Z + i * QH;
      FDoubleBuffer.CanvasBGRA.Brush.Color   := FColorBarreFill;
      FDoubleBuffer.CanvasBGRA.Brush.Opacity := 128;
      AfficherMessage(Format('-- %d QZ = %.2f - %.2f', [i, QZ, FClassesRepartDepthDiagram[i] * LargZoneDiagramme]));
      if (FClassesRepartDepthDiagram[i] > (PourcentageMax * FPourcentageSeuilHightLight)) then
      begin
        EWE := QCalcP2D(MakeTPoint2Df(0.00, QZ + QH));
        // masque évitant les superpositions disgracieuses
        FDoubleBuffer.CanvasBGRA.Brush.Color := clWhite;
        FDoubleBuffer.CanvasBGRA.pen.Style   := psClear;
        FDoubleBuffer.CanvasBGRA.Rectangle(0, EWE.Y - 20, EWE.X - 2, EWE.Y + 20);
        FDoubleBuffer.CanvasBGRA.TextOut(4, EWE.y, Format('%.0f', [CBG.Z +  i * (CHD.Z - CBG.Z) / NbBarres]));
        FDoubleBuffer.CanvasBGRA.pen.Style   := psSolid;
        FDoubleBuffer.CanvasBGRA.Brush.Color := clRed;

      end;
      QDrawRectangle(MakeTPoint2Df(0.00, QZ),
                     MakeTPoint2Df(FClassesRepartDepthDiagram[i] * LargZoneDiagramme, QZ + QH));

    end;
    FDoubleBuffer.CanvasBGRA.Pen.Color := clBlue;
    QDrawLigne(MakeTPoint2Df(0.00, CBG.Z), MakeTPoint2Df(0.00, CHD.Z));

    //imgHistogrammes.CanvasBGRA.CopyRect(R, FDoubleBuffer.CanvasBGRA, R);
    FDoubleBuffer.Draw(imgHistogrammes.Canvas, 0, 0, True);
  finally
    FDoubleBuffer.Free;
  end;   //*)

end;

function TCdrDepthDiagramExt.Initialise(const DT: TBDDEntites;
                                        const Filtres: string;
                                        const NbBarres: integer;
                                        const LMini: double;
                                        const BarresLineColor: TColor;
                                        const BarresFillColor: TColor;
                                        const PourcentageSeuilHightLight: double): boolean;
var
  QDevelViseesVisibles: double;
begin
  Result  := False;
  FDoDraw := false;
  FBDDEntites := DT;
  AfficherMessage(Format('%s.Initialise()', [ClassName]));
  try
    FFiltres      := Filtres;
    AfficherMessage(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
    FNbBarres    := NbBarres;
    FLongMini     := LMini;
    FColorBarreLine  := BarresLineColor;
    FColorBarreFill  := BarresFillColor;
    FPourcentageSeuilHightLight := PourcentageSeuilHightLight;
    FBDDEntites.MetaFiltre(FFiltres, QDevelViseesVisibles);
    // SetMinMax indispensable ici
    FBDDEntites.SetMinMax(True);
    FBDDEntites.ParseDepthHistogramme(FNbBarres, FLongMini, True);
    // préparatin du diagramme
    self.SetColorsBarres(BarresLineColor, BarresFillColor);
    FClassesRepartDepthDiagram := FBDDEntites.GetClassesRepartDepthDiagram();
    //imgHistogrammes.Invalidate;
    FDoDraw := True;
    //DessinerHistogramme();
    imgHistogrammes.Invalidate;
    Result := True;
   except
   end;

end;


function TCdrDepthDiagramExt.QCalcP2D(const GP: TPoint2Df): TPoint;
begin
  Result.x := round((imgHistogrammes.Width) * (GP.X - FCMin.X) * FRx);
  Result.y := imgHistogrammes.Height -
              (trunc(imgHistogrammes.Height * (GP.Y - FCMin.Y) * FRy));
end;
function TCdrDepthDiagramExt.QCalcP2G(const PP: TPoint; out P2G: TPoint2Df): boolean;
var
  rx, ry: Double;
begin
  result := false;
  if (not FDoDraw) then exit;
  if (Not InRange(PP.X, 0, imgHistogrammes.Width)) then Exit;
  if (Not InRange(PP.Y, 0, imgHistogrammes.Height)) then Exit;
  rx := (FCMax.X - FCMin.X) / imgHistogrammes.Width;
  ry := (FCMax.Y - FCMin.Y) / imgHistogrammes.Height;

  P2G.X := FCMin.X + rx * PP.X;         // OK
  P2G.Y := FCMin.Y + ry * (imgHistogrammes.Height - PP.Y);

  result := True;
end;

procedure TCdrDepthDiagramExt.QDrawLigne(const P1, P2: TPoint2Df);
var
  PP1, PP2: TPoint;
begin
  PP1 := QCalcP2D(P1);
  PP2 := QCalcP2D(P2);
  FDoubleBuffer.Canvas.Line(PP1, PP2);
end;
procedure TCdrDepthDiagramExt.QDrawRectangle(const P1, P2: TPoint2Df);
var
  PP1, PP2: TPoint;
begin
  PP1 := QCalcP2D(P1);
  PP2 := QCalcP2D(P2);
  PP2.X := Max(PP1.X + 2, PP2.X);
  FDoubleBuffer.CanvasBGRA.Rectangle(PP1.X, PP1.Y, PP2.X, PP2.Y);
end;

procedure TCdrDepthDiagramExt.ExportDiagramToSVG(const QFileName: TStringDirectoryFilename; const QSize: double);
const
  MAIN_LAYER            = 'Principal';
  GROUPE_DEPTH_DIAGRAM  = 'DepthDiagram';
  GROUPE_DEPTH_RETICULE = 'DepthReticule';

  STYLE_PETALES         = 'Petales';
  STYLE_MIRES           = 'Reticules';
var
  SVGCanvas: TSVGCanvas;
  j, NbPetales, i: Integer;
  Rapport,  RayonReticule, RayonMaxPetales, RayonPetale: Double;
  QPosCentre: TPoint2Df;
begin
  SVGCanvas := TSVGCanvas.Create;
  try
    QPosCentre := MakeTPoint2Df(QSize / 2, QSize / 2);
    // en-tetes
    SVGCanvas.Commentaire := '';
    SVGCanvas.Scale       := 1.00;
    if (Not SVGCanvas.InitializeDocument(QFileName,
                                         False,
                                         'RoseDiagram',
                                         Format('%s: RoseDiagram', [ExtractFileName(QFileName)]),
                                         0, 0, QSize, QSize)) then Exit;
    //RayonReticule   := QSize / 2.0;
    RayonMaxPetales := QSize * 0.40;
    Rapport := 0.5 * QSize / RayonMaxPetales;
    NbPetales := 1 + High(FClassesRepartDepthDiagram);

    // patterns
    SVGCanvas.BeginPatternsSection;
      // les patterns ici
    SVGCanvas.EndPatternsSection;
    // section de styles
    SVGCanvas.BeginStylesSection;
      // styles de cube englobant
      SVGCanvas.WriteStyleSolidPolygone('Cadre', clGray, clSilver, 255, 0.01, 'cadre');
      // style par défaut: gris
      SVGCanvas.WriteStyleSolidLine(STYLE_MIRES , clGray, 0.05, '');
      SVGCanvas.WriteStyleSolidLine('Arcs' , clRed, 0.5, '');
      SVGCanvas.WriteStyleSolidPolygone('Pies' , clBlue, clFuchsia, 255, 0.2, '');
      // pétales
      SVGCanvas.WriteStyleSolidPolygone(STYLE_PETALES , clBlue, clAqua, 128, 0.05, '');
    SVGCanvas.EndStylesSection;
    // dessin
    SVGCanvas.BeginDrawingSection();
      SVGCanvas.BeginLayer(MAIN_LAYER);
        SVGCanvas.BeginGroupe(GROUPE_DEPTH_DIAGRAM, '', 0.00, 0.00);

        SVGCanvas.EndGroupe(GROUPE_DEPTH_DIAGRAM);
        SVGCanvas.BeginGroupe(GROUPE_DEPTH_RETICULE, '', 0.00, 0.00);

        SVGCanvas.EndGroupe(GROUPE_DEPTH_RETICULE);
      SVGCanvas.EndLayer(MAIN_LAYER);
    SVGCanvas.EndDrawingSection();
    // cloture
    SVGCanvas.FinalizeDocument;
  finally
    SVGCanvas.Free;
  end;
end;

end.

