unit CadreHistoDirectionsExt;
{$INCLUDE CompilationParameters.inc}
// Date: 07/11/2013
// Statut: Fonctionnel

interface

uses
  Common,
  StructuresDonnees,
  UnitEntitesExtended,
  Graphics,
  BGRABitmap,
  Math,
  SVGCanvasUnit,
  Classes, SysUtils, FileUtil, Forms, Controls, ExtCtrls, StdCtrls;

type

  { TCdrRoseDiagramExt }

  TCdrRoseDiagramExt = class(TFrame)
    imgHistogrammes: TPaintBox;
    Panel1: TPanel;
    procedure imgHistogrammesPaint(Sender: TObject);
  private
    { private declarations }
    FDoDraw       : boolean;
    FBDDEntites   : TBDDEntites;
    FDoubleBuffer : TBGRABitmap;
    FFiltres      : string;
    FNbPetales    : integer;
    FLongMini     : double;
    FColorPetaleLine : TColor;
    FColorPetaleFill : TColor;
    FClassesRepartRoseDiagram: TVecteurDouble;
    procedure SetColorsPetales(const CLB, CFB: TColor);
    procedure DessinerHistogramme();

  public
    { public declarations }
    function Initialise(const DT: TBDDEntites;
                        const Filtres: string;
                        const NbPetales: integer;
                        const LMini: double;
                        const PetalesLineColor: TColor;
                        const PetalesFillColor: TColor): boolean;
    procedure ExportDiagramToSVG(const QFileName: TStringDirectoryFilename; const QSize: double);
  end; 

implementation

{$R *.lfm}
function TCdrRoseDiagramExt.Initialise(const DT: TBDDEntites;
                                       const Filtres: string;
                                       const NbPetales: integer;
                                       const LMini: double;
                                       const PetalesLineColor: TColor;
                                       const PetalesFillColor: TColor): boolean;
var
  QDevelViseesVisibles: double;
begin
  Result      := False;
  FBDDEntites := DT;
  FDoDraw     := false;
  AfficherMessage(Format('%s.Initialise()', [ClassName]));
  try
    FFiltres      := Filtres;
    AfficherMessage(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
    FNbPetales    := NbPetales;
    FLongMini     := LMini;
    FColorPetaleLine  := PetalesLineColor;
    FColorPetaleFill  := PetalesFillColor;
    FBDDEntites.MetaFiltre(FFiltres, QDevelViseesVisibles);
    // SetMinMax inutile
    FDoDraw := FBDDEntites.ParseRoseDiagram(FNbPetales, FLongMini);
    SetColorsPetales(PetalesLineColor, PetalesFillColor);
    FClassesRepartRoseDiagram := FBDDEntites.GetClassesRepartRoseDiagram();
    imgHistogrammes.Invalidate;
    //DessinerHistogramme();
    Result := True;
  except
  end;
end;

procedure TCdrRoseDiagramExt.imgHistogrammesPaint(Sender: TObject);
begin
  DessinerHistogramme;
end;

procedure TCdrRoseDiagramExt.SetColorsPetales(const CLB, CFB: TColor);
begin
  FColorPetaleLine := CLB;
  FColorPetaleFill := CFB;
end;

procedure TCdrRoseDiagramExt.DessinerHistogramme();
const Marge = 10;
var
  NbPetales: integer;
  i, j: integer;
  P1, P2: TPoint;
  R  : TRect;
  Rayon, Rapport, RayonByRapport, QMax666: Extended;
  Rd: Int64;
  cy, cx: Integer;
  Interval, pipi, Ang1, Ang2: ValReal;
  PG: array[0..3] of TPoint;
begin
  if (not FDoDraw) then Exit;

  NbPetales := 1 + High(FClassesRepartRoseDiagram);
  try
    AfficherMessage(Format('%s.DessinerHistogramme("%s")',[ClassName, FFiltres]));

    FDoubleBuffer := TBGRABitmap.Create(imgHistogrammes.Width, imgHistogrammes.Height);
    //FDoubleBuffer.Height := imgHistogrammes.Height;
    //FDoubleBuffer.Width  := imgHistogrammes.Width;
    R.Left   := imgHistogrammes.Left;
    R.Top    := imgHistogrammes.Top;
    R.Bottom := imgHistogrammes.Top  + imgHistogrammes.Height;
    R.Right  := imgHistogrammes.Left + imgHistogrammes.Width;

    // dessin du diagramme
    Rayon           := 10.00;
    Rapport         := 0.5 * (FDoubleBuffer.Width - Marge) / Rayon;
    RayonByRapport  := Rayon * Rapport;
    Rd := round(RayonByRapport);
    cx := FDoubleBuffer.Width div 2;
    cy := FDoubleBuffer.Height div 2;

    AfficherMessage('  Paramètres du diagramme:');
    AfficherMessage(Format(' Rayon = %.2f', [Rayon]));

    Interval := PI / NbPetales;

    FDoubleBuffer.CanvasBGRA.Brush.Style   := bsSolid;
    FDoubleBuffer.CanvasBGRA.Brush.Color   := clWhite;
    FDoubleBuffer.CanvasBGRA.Brush.Opacity := 255;
    FDoubleBuffer.CanvasBGRA.FillRect(R);
    // réticule
    FDoubleBuffer.CanvasBGRA.Pen.Color   := clGray;
    FDoubleBuffer.CanvasBGRA.Brush.Style := bsClear;
    Rd := round(RayonByRapport);
    FDoubleBuffer.CanvasBGRA.EllipseC(cx, cy, Rd, Rd);
    FDoubleBuffer.CanvasBGRA.EllipseC(cx, cy, Rd div 2, Rd div 2);
    FDoubleBuffer.CanvasBGRA.Pen.Color := clBlue;
    P1.X := Trunc(cx - Rd);
    P1.Y := Trunc(FDoubleBuffer.Height - cy);
    P2.X := Trunc(cx + Rd);
    P2.Y := Trunc(FDoubleBuffer.Height - cy);
    FDoubleBuffer.CanvasBGRA.MoveTo(P1); FDoubleBuffer.CanvasBGRA.LineTo(P2);
    //TextOut(P1.X + 1, P1.Y + 1, format('%.0f%%', [QMax666 * 100]));
    P1.X := Trunc(cx);
    P1.Y := Trunc(FDoubleBuffer.Height - (cy - rd));
    P2.X := Trunc(cx);
    P2.Y := Trunc(FDoubleBuffer.Height - (cy + Rd));
    FDoubleBuffer.CanvasBGRA.MoveTo(P1); FDoubleBuffer.CanvasBGRA.LineTo(P2);
    //--------------------------------
    FDoubleBuffer.CanvasBGRA.Brush.Style   := bsSolid;
    FDoubleBuffer.CanvasBGRA.Brush.Color   := FColorPetaleFill;
    FDoubleBuffer.CanvasBGRA.Brush.Opacity := 128;
    FDoubleBuffer.CanvasBGRA.Pen.Color     := FColorPetaleLine;
    QMax666 := -1.00;
    for i := 0 to NbPetales - 1 do QMax666 := Max(QMax666, FClassesRepartRoseDiagram[i]);
    // rosace
    //texte en clair donnant une erreur. TODO: Vérifier cette zone
    for j := 0 to 1 do
    begin
      pipi := PI * (j + 1.5);
      for i := 0 to NbPetales - 1 do
      begin
        Rd := round(FClassesRepartRoseDiagram[i] * RayonByRapport);
        //Ang1 := PI * j + PI + ((0.5 * pi) - i * Interval);
        //Ang2 := PI * j + PI + ((0.5 * pi) - (i + 1) * Interval);
        Ang1 := pipi - Interval * i;
        Ang2 := pipi - Interval * (i + 1);
        //double A1 = pipi + Interval * i;
        //double A2 = pipi + Interval * (i + 1);
        pg[0].X := cx;
        pg[0].Y := cy;
        pg[1].X := Trunc(cx + Rd * Cos(Ang1));
        pg[1].Y := Trunc(FDoubleBuffer.Height - (cy + Rd * Sin(Ang1)));
        pg[2].X := Trunc(cx + Rd * Cos(Ang2));
        pg[2].Y := Trunc(FDoubleBuffer.Height - (cy + Rd * Sin(Ang2)));
        pg[3].X := cx;
        pg[3].Y := cy;
        FDoubleBuffer.CanvasBGRA.Polygon(pg);
      end;
    end;
    FDoubleBuffer.Draw(imgHistogrammes.Canvas, 0, 0, True);

  finally
    FDoubleBuffer.Free;
  end;
end;

procedure TCdrRoseDiagramExt.ExportDiagramToSVG(const QFileName: TStringDirectoryFilename; const QSize: double);
const
  MAIN_LAYER           = 'Principal';
  GROUPE_ROSE_DIAGRAM  = 'RoseDiagram';
  GROUPE_ROSE_RETICULE = 'RoseReticule';

  STYLE_PETALES        = 'Petales';
  STYLE_MIRES          = 'Reticules';
var
  SVGCanvas: TSVGCanvas;
  j, NbPetales, i: Integer;
  Rapport,  RayonReticule, RayonMaxPetales, RayonPetale: Double;
  pipi, IntervalAng: ValReal;
  Ang1, Ang2: double;
  QPosCentre: TPoint2Df;
begin
  SVGCanvas := TSVGCanvas.Create;
  try
    QPosCentre := MakeTPoint2Df(QSize / 2, QSize / 2);
    // en-tetes
    SVGCanvas.Commentaire := '';
    SVGCanvas.Scale       := 1.00;
    if (Not SVGCanvas.InitializeDocument(QFileName,
                                         False,
                                         'RoseDiagram',
                                         Format('%s: RoseDiagram', [ExtractFileName(QFileName)]),
                                         0, 0, QSize, QSize)) then Exit;
    RayonReticule   := QSize / 2.0;
    RayonMaxPetales := QSize * 0.40;
    Rapport := 0.5 * QSize / RayonMaxPetales;
    NbPetales := 1 + High(FClassesRepartRoseDiagram);
    IntervalAng := PI / NbPetales;
    // patterns
    SVGCanvas.BeginPatternsSection;
      // les patterns ici
    SVGCanvas.EndPatternsSection;
    // section de styles
    SVGCanvas.BeginStylesSection;
      // styles de cube englobant
      SVGCanvas.WriteStyleSolidPolygone('Cadre', clGray, clSilver, 255, 0.01, 'cadre');
      // style par défaut: gris
      SVGCanvas.WriteStyleSolidLine(STYLE_MIRES , clGray, 0.05, '');
      SVGCanvas.WriteStyleSolidLine('Arcs' , clRed, 0.5, '');
      SVGCanvas.WriteStyleSolidPolygone('Pies' , clBlue, clFuchsia, 255, 0.2, '');
      // pétales
      SVGCanvas.WriteStyleSolidPolygone(STYLE_PETALES , clBlue, clAqua, 128, 0.05, '');
    SVGCanvas.EndStylesSection;
    // dessin
    SVGCanvas.BeginDrawingSection();
      SVGCanvas.BeginLayer(MAIN_LAYER);
        SVGCanvas.BeginGroupe(GROUPE_ROSE_DIAGRAM, '', 0.00, 0.00);
          for j := 0 to 1 do
          begin
            pipi := PI * (j + 1.5);
            for i := 0 to NbPetales - 1 do
            begin
              RayonPetale := FClassesRepartRoseDiagram[i] * RayonMaxPetales * Rapport;
              //Ang1 := PI * j + PI + ((0.5 * pi) - i * Interval);
              //Ang2 := PI * j + PI + ((0.5 * pi) - (i + 1) * Interval);
              Ang1 := radtodeg(pipi - IntervalAng * (i + 1));
              Ang2 := radtodeg(pipi - IntervalAng * (i));
              SVGCanvas.DrawArc(STYLE_PETALES, QPosCentre.X, QPosCentre.Y, RayonPetale, Ang1, Ang2, True);
            end;
          end;
        SVGCanvas.EndGroupe(GROUPE_ROSE_DIAGRAM);
        SVGCanvas.BeginGroupe(GROUPE_ROSE_RETICULE, '', 0.00, 0.00);
          SVGCanvas.DrawCircle(STYLE_MIRES, QPosCentre.X, QPosCentre.Y, RayonReticule);
          SVGCanvas.DrawCircle(STYLE_MIRES, QPosCentre.X, QPosCentre.Y, RayonReticule * 0.5);
          SVGCanvas.DrawLine(STYLE_MIRES, QPosCentre.X, QPosCentre.Y - RayonReticule,
                                          QPosCentre.X, QPosCentre.Y + RayonReticule);
          SVGCanvas.DrawLine(STYLE_MIRES, QPosCentre.X - RayonReticule, QPosCentre.Y,
                                          QPosCentre.X + RayonReticule, QPosCentre.Y);
        SVGCanvas.EndGroupe(GROUPE_ROSE_RETICULE);
      SVGCanvas.EndLayer(MAIN_LAYER);
    SVGCanvas.EndDrawingSection();
    // cloture
    SVGCanvas.FinalizeDocument;
  finally
    SVGCanvas.Free;
  end;
end;

end.

