unit CadreListeNearStations;

{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  //ToporobotClasses2012,
  StructuresDonnees,
  UnitEntitesExtended,
  Common,

  Classes, Graphics, SysUtils, FileUtil, Forms, Controls, StdCtrls, ComCtrls, Types, LCLType;

type

  { TCdrListeNearestStations }

  TCdrListeNearestStations = class(TFrame)
    hcColsTitres: THeaderControl;
    lbNbStationsFound: TLabel;
    lsbStations: TListBox;
    lbStationDeBase: TStaticText;
    procedure hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl; Section: THeaderSection);
    procedure lsbStationsDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
  private
    FBDDEntites    : TBDDEntites;
    FTempBDDEntites: TBDDEntites;
    FCurrEntite    : TEntiteEtendue;


  public
    function Initialiser(const BDD: TBDDEntites;
                         const QE: TEntiteEtendue;
                         const QDoMatchExact: boolean;
                         const QRayonCapture: double): boolean;
    procedure Relister(const QDoMatchExact: boolean;
                       const QRayonCapture: double);

    function  GetNbStationsFound(): integer;
    function  GetNearEntiteSelected(): TEntiteEtendue;
    procedure Finaliser();

  end;

implementation

{$R *.lfm}

{ TCdrListeNearestStations }
function TCdrListeNearestStations.Initialiser(const BDD: TBDDEntites;
                                              const QE: TEntiteEtendue;
                                              const QDoMatchExact: boolean;
                                              const QRayonCapture: double): boolean;
begin
  AfficherMessage(Format('%s.Initialiser', [ClassName]));
  Result := False;
  FBDDEntites     := BDD;
  FCurrEntite     := QE;
  FTempBDDEntites := TBDDEntites.Create;
  try
    FTempBDDEntites.Initialiser(MakeTPoint3Df(0.0, 0.0, 0.0));
    FTempBDDEntites.ResetTables();
    Relister(QDoMatchExact, QRayonCapture);
    result := True;
  except
  end;

end;

procedure TCdrListeNearestStations.Finaliser();
begin
  try
    FTempBDDEntites.Finaliser();
  finally
    FTempBDDEntites.Free;
  end;
end;

procedure TCdrListeNearestStations.Relister(const QDoMatchExact: boolean;
                                            const QRayonCapture: double);
var
  i, Nb: Integer;
  QSer, QSt: integer;
  EWE: TEntiteEtendue;
  WU: String;
  dx, dy, dz: double;
  zdar: TGHStringArray;
begin
  lsbStations.Clear;
  lbStationDeBase.Caption := Format('Voisinage de la station %d.%d (%s)', [FCurrEntite.Entite_Serie, FCurrEntite.Entite_Station, FCurrEntite.oIDLitteral]);
  FTempBDDEntites.ViderLesViseesEtAntennes();
  Nb := FBDDEntites.GetNbEntitesVisees();
  for i := 0 to Nb - 1 do
  begin
    EWE := FBDDEntites.GetEntiteVisee(i);
    if (EWE.Type_Entite = tgVISEE_RADIANTE) then Continue;  // les visées en antennes ne sont pas prises en compte
    // la station de base elle-même est bien sûr zappée
    if ((EWE.Entite_Serie   = FCurrEntite.Entite_Serie) and
        (EWE.Entite_Station = FCurrEntite.Entite_Station)) then Continue;
    // on taggue l'entité avec la distance
    dx := EWE.Une_Station_2_X - FCurrEntite.Une_Station_2_X;
    dy := EWE.Une_Station_2_Y - FCurrEntite.Une_Station_2_Y;
    dz := EWE.Une_Station_2_Z - FCurrEntite.Une_Station_2_Z;
    EWE.TagDouble := Hypot3D(dx, dy, dz);
    if (EWE.TagDouble < QRayonCapture) then FTempBDDEntites.AddEntiteVisee(EWE);
  end;
  // trier du plus proche au plus éloigner
  //showmessage('SortByTagDouble');
  FTempBDDEntites.SortByTagDouble();
  // et on liste
  lsbStations.Clear;
  Nb := FTempBDDEntites.GetNbEntitesVisees();
  if (Nb > 0) then
  begin
    for i := 0 to Nb -1 do
    begin
      EWE := FTempBDDEntites.GetEntiteVisee(i);
      WU  := Format('%d.%d', [EWE.Entite_Serie, EWE.Entite_Station]);
      lsbStations.Items.Add(WU);
    end;
    lsbStations.ItemIndex:=0;
  end;
  lbNbStationsFound.Caption := format('%d stations', [lsbStations.Count]);
end;

function TCdrListeNearestStations.GetNbStationsFound(): integer;
begin
  Result := FTempBDDEntites.GetNbEntitesVisees();
end;

function TCdrListeNearestStations.GetNearEntiteSelected(): TEntiteEtendue;
var
  n: Integer;
begin
  Result := FBDDEntites.GetEntiteVisee(0);
  Result.Entite_Serie   := -1;
  Result.Entite_Station := -1;
  n := FTempBDDEntites.GetNbEntitesVisees();
  if (n = 0) then Exit;
  Result := FTempBDDEntites.GetEntiteVisee(lsbStations.ItemIndex);
end;

procedure TCdrListeNearestStations.hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl; Section: THeaderSection);
begin
  lsbStations.Invalidate;
end;

procedure TCdrListeNearestStations.lsbStationsDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  Q4  = 4; // compensation du décalage entre le header et la liste
  mg  = 1;
  // lgr = 25;
var
  EWE: TEntiteEtendue;
  C: TColor;
  m: integer;
  r: TRect;
  dp, dx, dy, dz: Extended;
  procedure DessineFiletColonne(const TB: integer); inline;
  begin
    lsbStations.Canvas.Line(TB, ARect.Top, TB, ARect.Bottom);
  end;
  // bg = couleur de fond; tc= couleur du texte
  procedure DessineItem(const bg,tc: TColor);
  VAR
    //TB: integer;
    HS: THeaderSection;
  begin
    with lsbStations do
    begin
      Canvas.FillRect(ARect);
      Canvas.Brush.Color:=bg;
      Canvas.Font.Color :=tc;
      Canvas.Pen.Color  :=clSilver; // pour les filets
      //TB := hcColsTitres.hea
      HS := hcColsTitres.Sections.Items[0];  // Station
      canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[EWE.Entite_Serie, EWE.Entite_Station]));
      HS := hcColsTitres.Sections.Items[1];  // IDTerrain
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(EWE.oIDLitteral));
      HS := hcColsTitres.Sections.Items[2];  // distance
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[EWE.TagDouble]));
      HS := hcColsTitres.Sections.Items[3];  // dp
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[dp]));
      HS := hcColsTitres.Sections.Items[4];  // dx
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[dx]));
      HS := hcColsTitres.Sections.Items[5];  // dy
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[dy]));
      HS := hcColsTitres.Sections.Items[6];  // dz
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[dz]));
      DessineFiletColonne(HS.Right);
    end;
  end;
begin
  EWE := FTempBDDEntites.GetEntiteVisee(Index);
  dx  := FCurrEntite.Une_Station_2_X - EWE.Une_Station_2_X;
  dy  := FCurrEntite.Une_Station_2_Y - EWE.Une_Station_2_Y;
  dz  := FCurrEntite.Une_Station_2_Z - EWE.Une_Station_2_Z;
  dp  := Hypot2D(dx, dy);
  with lsbStations do
  begin
    canvas.brush.color:=clwhite;
    //canvas.fillrect(rect);
    //on affiche le texte
    DessineItem(clwhite, clBlack);
    //affichage lorsque la ligne est sélectionnée
    if (odSelected in state) then begin
      canvas.brush.color:=clBlue;
      //canvas.fillrect(rect);
      //canvas.font.color:=clblue;
      //canvas.font.style:=Listbox1.canvas.font.style +[fsbold]+[fsitalic];
      DessineItem(clBlue, clWhite);
      //canvas.textout(rect.left+30,rect.top+2,listbox1.items[index]);
    end;
  end;

end;



end.

