// liste des séries admettant les filtres

unit CadreListeSeries;
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Classes, SysUtils, FileUtil, curredit, Forms, Controls, StdCtrls, ComCtrls, Dialogs, Graphics, LCLType, ActnList, Buttons, CheckLst
  , StructuresDonnees
  , ToporobotClasses2012
  , UnitObjetSerie
  , unitUtilsComposants
  {$IFDEF USE_GENERIC_TYPES}
  , UnitListesSimplesWithGeneriques
  {$ELSE}
  , UnitListesSimplesWithoutGeneriques
  {$ENDIF}
  , Types

  ;
type TModeFiltrage = (mfNONE, mfENTRANCES, mfRESEAUX);

type TButtonsShown = set of (tbsAPPLY, tbsADD, tbsREMOVE, tbsSORT, tbsCSV, tbsHELP);

type

  { TCdrListeSeries }

  TCdrListeSeries = class(TFrame)
    acCdrSerAddItem: TAction;
    acCdrSerApplyModifs: TAction;
    acCdrSerDeleteItem: TAction;
    acCdrSerExportListesCSV: TAction;
    acCdrSerFind: TAction;
    acnlstCadreSerie: TActionList;
    acCdrSerSort: TAction;
    acCdrSerHelp: TAction;
    cmbEntreesOuReseaux: TComboBox;
    cmbTypeFiltrage: TComboBox;
    editRechercheRapide: TEdit;
    hcColsTitres: THeaderControl;
    imglstCadreSerie: TImageList;
    Label1: TLabel;
    lbNbreElements: TLabel;
    lsbListe: TCheckListBox;
    SpeedButton1: TSpeedButton;
    SpeedButton12: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SpeedButton8: TSpeedButton;
    procedure acCdrSerAddItemExecute(Sender: TObject);
    procedure acCdrSerApplyModifsExecute(Sender: TObject);
    procedure acCdrSerExportListesCSVExecute(Sender: TObject);
    procedure acCdrSerFindExecute(Sender: TObject);
    procedure acCdrSerSortExecute(Sender: TObject);
    procedure acCdrSerHelpExecute(Sender: TObject);
    procedure cmbEntreesOuReseauxChange(Sender: TObject);
    procedure cmbTypeFiltrageChange(Sender: TObject);
    procedure editRechercheRapideKeyPress(Sender: TObject; var Key: char);
    procedure hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl; Section: THeaderSection);
    procedure lsbListeClick(Sender: TObject);
    procedure lsbListeDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
  private
    { private declarations }
    FBusy: boolean;
    FModeFiltrage: TModeFiltrage;
    FSelectedIndex: integer;
    FAllowMultiSelect: boolean;
    FListeSelectedSeries: TListeSelectedSeries;
    FDocuTopo: TToporobotStructure2012;
    FProcGetSerie    : TProcedureOfObject;
    FProcApplyModifs : TProcedureOfObject;
    procedure InitCmbFiltre(const F: TModeFiltrage);
    procedure InitHeaderListe();
    procedure ListerSeries(const FNumeroEntreeOuReseau: Integer; const QIdx: integer);
    procedure ListerEntrees();
    procedure ListerReseaux();
  public
    { public declarations }
    function Initialise(const FD: TToporobotStructure2012;
                        const QProcGetSerie, QProcApplyModifs: TProcedureOfObject;
                        const QButtons: TButtonsShown;
                        const AllowMultiSelectSeries: boolean): boolean;
    procedure Relister(const QIdx: integer);
    function  GetSelectedIndex(): integer;
    function  GetSelectedNumSerie(): TNumeroSerie;
    function  IsItemListeSelected(const Idx: integer): boolean;
    procedure Finalise();
  end;

implementation
uses
  Common,
  CallDialogsStdVersion;


const
  HEADER_LST_NUM_COL_COCHE       =  0;
  HEADER_LST_NUM_COL_NUM_SERIE   =  1;
  HEADER_LST_NUM_COL_ST_DEPART   =  2;
  HEADER_LST_NUM_COL_ST_ARRIVEE  =  3;
  HEADER_LST_NUM_COL_NOM_SERIE   =  4;


  {$IFDEF GROS_MINET}
  HEADER_LST_NUM_COL_ENTREE      =  5;
  HEADER_LST_NUM_COL_RESEAU      = -1;
  HEADER_LST_NUM_COL_NB_VISEES   =  6;
  {$ELSE}
  HEADER_LST_NUM_COL_ENTREE      =  5;
  HEADER_LST_NUM_COL_RESEAU      =  6;
  HEADER_LST_NUM_COL_NB_VISEES   =  7;
  {$ENDIF}

{$R *.lfm}

{ TCdrListeSeries }


// NOTA: TCdrListeSeries possède un pointeur sur l'objet TToporobotStructure2012
// Par conséquent, on peut faire toutes les opérations sur les séries

procedure TCdrListeSeries.acCdrSerApplyModifsExecute(Sender: TObject);
begin
  if (lsbListe.ItemIndex < 0) then Exit;
  if (assigned(FProcApplyModifs)) then FProcApplyModifs;
end;

procedure TCdrListeSeries.acCdrSerExportListesCSVExecute(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QIdxFilter: integer;
begin
  if (DoDialogSaveFile('Fichier CSV|*.csv', '.csv', QFileName, QIdxFilter)) then
  begin
    FDocuTopo.ExportListeEntetesSeries(QFileName);
  end;
end;

procedure TCdrListeSeries.acCdrSerFindExecute(Sender: TObject);
var
  QIdx: integer;
begin
  if (FindIdxOccurrenceInCheckListBox(lsbListe, trim(editRechercheRapide.Text), QIdx)) then
    lsbListe.ItemIndex := QIdx
  else
    ShowMessage(GetResourceString(rsMATCHNOTFOUND));
end;

procedure TCdrListeSeries.acCdrSerSortExecute(Sender: TObject);
begin
  FDocuTopo.SortSeries;
  Relister(0);
end;

procedure TCdrListeSeries.acCdrSerHelpExecute(Sender: TObject);
begin
  DisplayHelpSystem('LISTE_SERIES');
end;

procedure TCdrListeSeries.acCdrSerAddItemExecute(Sender: TObject);
var
  EWE: String;
begin
  EWE := GetResourceString(rsDLG_BDD_INSERT_SERIE);
  if (not GHTopoQuestionOuiNon(EWE)) then exit;
  if (FDocuTopo.CreateNewSerie(1,0, -1, rs2NDMEMBER)) then
  begin
    Relister(FDocuTopo.GetNbSeries() - 1);
  end
  else
  begin
    ShowMessage('Echec creation série');
  end;
end;

procedure TCdrListeSeries.cmbEntreesOuReseauxChange(Sender: TObject);
begin
  FSelectedIndex := 0;
  ListerSeries(cmbEntreesOuReseaux.ItemIndex, 0);
end;

procedure TCdrListeSeries.cmbTypeFiltrageChange(Sender: TObject);
begin
  FModeFiltrage := TModeFiltrage(cmbTypeFiltrage.ItemIndex);
  case FModeFiltrage of
    mfNONE      : ;
    mfENTRANCES : ListerEntrees();
    mfRESEAUX   : ListerReseaux();
  end;
  ListerSeries(0, 0);
  cmbEntreesOuReseaux.Enabled := cmbTypeFiltrage.ItemIndex <> ord(mfNONE);
end;


procedure TCdrListeSeries.editRechercheRapideKeyPress(Sender: TObject; var Key: char);
begin
  if (key = #13) then acCdrSerFindExecute(self);
end;

procedure TCdrListeSeries.Finalise();
begin
  try
    FListeSelectedSeries.ClearListe();
  finally
    FListeSelectedSeries.Free;
  end;
end;

function TCdrListeSeries.GetSelectedIndex(): integer;
var
  n: Integer;
begin
  n := lsbListe.Count;
  Result := IIF((n > 0), FSelectedIndex, -1);
  AfficherMessage(Format('%s.GetSelectedIndex: %d / %d', [ClassName, Result, n]));
end;

function TCdrListeSeries.GetSelectedNumSerie(): TNumeroSerie;
begin
  Result := FListeSelectedSeries.GetElement(lsbListe.ItemIndex);
end;

function TCdrListeSeries.IsItemListeSelected(const Idx: integer): boolean;
begin
  Result := lsbListe.Checked[Idx];
end;



procedure TCdrListeSeries.hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl; Section: THeaderSection);
begin
  lsbListe.Invalidate;
end;

procedure TCdrListeSeries.lsbListeClick(Sender: TObject);
begin
  ;
end;


procedure TCdrListeSeries.InitCmbFiltre(const F: TModeFiltrage);
begin
  cmbEntreesOuReseaux.Enabled := false;
  FModeFiltrage := F;
  cmbTypeFiltrage.Clear;
  cmbTypeFiltrage.Items.Add(GetResourceString(rsTBS_TOUT));
  cmbTypeFiltrage.Items.Add(GetResourceString(rsTBS_ENTRANCE));
  {$IFNDEF GROS_MINET}  // Sylvestre CLEMENT n'utilise pas les réseaux
    cmbTypeFiltrage.Items.Add(GetResourceString(rsTBS_RESEAUX));
  {$ENDIF}
  cmbTypeFiltrage.ItemIndex := Ord(FModeFiltrage);
  cmbEntreesOuReseaux.Clear;
end;

procedure TCdrListeSeries.InitHeaderListe();
const
  LC_COCHES           = 30;
  LC_ID_NUMERIQUES    = 60;
  LC_ID_SERIE_STATION = 70;
  LC_NOM_SERIE        = 300;
  LC_NOM_ENTREE       = 250;
  LC_NOM_RESEAU       = 250;
  LC_NB_POINTS        = 60;
var
 ht: THeaderSection;
 FNbItems: Integer;
 miou: String;
 procedure AjouterTitreColonne(const Titre: string; const LGMin, LG: integer);
 begin
   ht := hcColsTitres.Sections.Add;
   ht.Text := Titre;
   ht.MinWidth := LGMin;
   ht.Width    := LG;
 end;
 procedure AjouterTitreColonneRes(const RS: string; const LGMin, LG: integer);
 begin
   AjouterTitreColonne(GetResourceString(RS), LGMin, LG);
 end;
begin
  // purge des titres des headers
  hcColsTitres.Sections.Clear;

  miou := GetResourceString(rsTBS_SERIES);
  FNbItems := FDocuTopo.GetNbSeries;
  //pnlListes.Caption := Format('%s (%d items)', [miou, FNbItems]);
  with hcColsTitres do
  begin
    AjouterTitreColonneRes('X', LC_COCHES, LC_COCHES);
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_ID, LC_ID_NUMERIQUES, LC_ID_NUMERIQUES);
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_DEPART, LC_ID_SERIE_STATION, LC_ID_SERIE_STATION);
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_ARRIVEE, LC_ID_SERIE_STATION, LC_ID_SERIE_STATION);
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_NOM, 100, LC_NOM_SERIE);
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_ENTREE, 100, LC_NOM_ENTREE);
    {$IFDEF GROS_MINET}
    ;
    {$ELSE}
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_RESEAU, 100, LC_NOM_RESEAU);
    {$ENDIF}
    AjouterTitreColonneRes(rsSELECT_LISTE_SERIES_NBPOINTS, LC_NB_POINTS, LC_NB_POINTS);
  end;
end;


function TCdrListeSeries.Initialise(const FD: TToporobotStructure2012;
                                    const QProcGetSerie, QProcApplyModifs: TProcedureOfObject;
                                    const QButtons: TButtonsShown;
                                    const AllowMultiSelectSeries: boolean): boolean;
  procedure S777(const ACDC: Taction; const S: string; const QVisible: boolean);
  begin
    ACDC.Caption := GetResourceString(S);
    ACDC.Hint    := GetResourceString(S);
    ACDC.Visible := QVisible;
  end;

begin
  result := false;
  AfficherMessageErreur(format('%s.Initialise()', [ClassName]));
  FBusy := false;
  FSelectedIndex := 0;
  try
    //FNumeroEntree := 0;
    FDocuTopo := FD;
    FProcGetSerie     := QProcGetSerie;
    FProcApplyModifs  := QProcApplyModifs;
    FListeSelectedSeries := TListeSelectedSeries.Create;
    InitCmbFiltre(mfNONE);
    InitHeaderListe();
    lsbListe.ItemHeight  := 20;
    lsbListe.MultiSelect := false; //
    FAllowMultiSelect := AllowMultiSelectSeries;
    ListerSeries(-1, 0);
    S777(acCdrSerAddItem        , rsCDR_SERIES_ADD_SERIE   , tbsADD    in QButtons);
    S777(acCdrSerDeleteItem     , rsCDR_SERIES_DELETE_SERIE, tbsREMOVE in QButtons);
    S777(acCdrSerApplyModifs    , rsCDR_SERIES_VALID_SERIE , tbsAPPLY  in QButtons);
    S777(acCdrSerSort           , rsCDR_SERIES_SORT_SERIES , tbsSORT   in QButtons);
    S777(acCdrSerExportListesCSV, rsCDR_SERIES_EXPORT_CSV  , tbsCSV    in QButtons);
    S777(acCdrSerHelp           , rsCDR_SERIES_HELP        , tbsHELP   in QButtons);
    Result := True;

  except
    pass;
  end;
end;


procedure TCdrListeSeries.ListerEntrees();
var
  i, Nb: Integer;
  EWE: TEntrance;
  WU: String;
begin
  if (FBusy) then exit;
  FBusy := True;
  try
    cmbEntreesOuReseaux.Clear;
    Nb := FDocuTopo.GetNbEntrees();
    for i := 0 to Nb - 1 do
    begin
      EWE := FDocuTopo.GetEntree(i);
      WU  := Format('%d: %s (%d.%d [%s])', [i, EWE.eNomEntree, EWE.eRefSer, EWE.eRefSt, EWE.eIDTerrain]);
      cmbEntreesOuReseaux.Items.Add(WU);
    end;
    cmbEntreesOuReseaux.DropDownCount := IIF(Nb > 30, 30, Nb);
    cmbEntreesOuReseaux.ItemIndex := 0;
    cmbEntreesOuReseaux.Enabled   := True;

  finally
    FBusy := false;
  end;

end;

procedure TCdrListeSeries.ListerReseaux();
var
  i, Nb: Integer;
  EWE  : TReseau;
  WU   : String;
begin
  if (FBusy) then exit;
  FBusy := True;
  try
    cmbEntreesOuReseaux.Clear;
    Nb := FDocuTopo.GetNbReseaux();
    for i := 0 to Nb - 1 do
    begin
      EWE := FDocuTopo.GetReseau(i);
      WU  := Format('%d: %s', [i, EWE.NomReseau]);
      cmbEntreesOuReseaux.Items.Add(WU);
    end;
    cmbEntreesOuReseaux.DropDownCount := IIF(Nb > 30, 30, Nb);
    cmbEntreesOuReseaux.ItemIndex := 0;
    cmbEntreesOuReseaux.Enabled := True;
  finally
    FBusy := false;
  end;
end;

procedure TCdrListeSeries.ListerSeries(const FNumeroEntreeOuReseau: Integer; const QIdx: integer);
var
  i, Nb: Integer;
  SR: TObjSerie;
  procedure AddSerieAtListe(const S: TObjSerie; const QIdx: integer);
  var
    WU: String;
  begin
    FListeSelectedSeries.AddElement(QIdx);
    WU := format('%d: (%d) - %s', [S.GetNumeroDeSerie(), S.GetNumeroEntrance(), S.GetNomSerie()]);
    lsbListe.Items.add(WU);
  end;
  procedure LOL(const B: boolean);
  begin
    acCdrSerApplyModifs.Enabled := B;
    acCdrSerDeleteItem.Enabled  := B;
    acCdrSerFind.Enabled        := B;
    acCdrSerSort.Enabled        := B;
  end;
begin
  AfficherMessageErreur('Entre dans ListerSeries');
  FSelectedIndex     := 0;
  if (FBusy) then exit;
  FBusy := True;
  try
    Nb := FDocuTopo.GetNbSeries();
    FListeSelectedSeries.ClearListe();
    lsbListe.Enabled := false;
    lsbListe.Clear;
    // -1 -> on liste tout
    if (FModeFiltrage = mfNONE) then
    begin
      for i := 0 to Nb - 1 do AddSerieAtListe(FDocuTopo.GetSerie(i), i);
    end
    else
    begin
      for i := 0 to Nb - 1 do
      begin
        SR := FDocuTopo.GetSerie(i);
        case FModeFiltrage of
          mfNONE      : AddSerieAtListe(SR, i);
          mfENTRANCES : if (FNumeroEntreeOuReseau = SR.GetNumeroEntrance()) then AddSerieAtListe(SR, i);
          mfRESEAUX   : if (FNumeroEntreeOuReseau = SR.GetNumeroReseau())    then AddSerieAtListe(SR, i);
        end;
      end;
    end;
    if (lsbListe.Items.Count = 0) then
    begin
      LOL(false);
      Exit;
    end;
    LOL(True);

    lsbListe.ItemIndex := 0;
    // et on se positionne sur l'item demandé
    if (IsInRange(QIdx, 0, lsbListe.Count - 1)) then
    begin
      FSelectedIndex     := QIdx;
      lsbListe.ItemIndex := QIdx;
    end;
    lbNbreElements.Caption := Format(rsCDR_SERIE_LB_NB_SERIES, [lsbListe.Count]);
  finally
    FBusy := false;
    lsbListe.Enabled := true;
  end;
end;

procedure TCdrListeSeries.Relister(const QIdx: integer);
begin
  FBusy := false;
  ListerSeries(0, QIdx);
end;

procedure TCdrListeSeries.lsbListeDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
    Q4  = 4; // compensation du décalage entre le header et la liste
    mg  = 1;
var
  C: TColor;
  m: integer;
  r: TRect;
  sr: TObjSerie;
  miou: string;
  procedure DessineFiletColonne(const TB: integer); inline;
  begin
    lsbListe.Canvas.Line(TB, ARect.Top, TB, ARect.Bottom);
  end;
  procedure DessineRectangleColore(const QBG, PC, BC: TColor; const L, R, T, B: integer);
  var
    QR: TRect;
  begin
    lsbListe.canvas.Pen.Color   := PC;
    lsbListe.canvas.Brush.Color := BC;
    QR.Left     := L;
    QR.Right    := R;
    QR.Top      := T;
    QR.Bottom   := B;
    lsbListe.canvas.Rectangle(QR);
    // et on restaure l'ancienne couleur
    lsbListe.Canvas.Brush.Color := QBG;
    lsbListe.canvas.Pen.Color   := clSilver;
  end;
  // bg = couleur de fond; tc= couleur du texte
  procedure DessineItem(const bg,tc: TColor);
  VAR
    //TB: integer;
    HS: THeaderSection;
    rs: TReseau;
    en: TEntrance;
    QR: TRect;
    ColorItemSelected: TColor;
  begin
    with lsbListe do
    begin
      Canvas.FillRect(ARect);
      canvas.Pen.Color:=clSilver;
      Canvas.Font.Color :=tc;
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_COCHE];  // ID série
      ColorItemSelected := IIF(lsbListe.Checked[Index], clRed, clWhite);
      DessineRectangleColore(bg, clBlack, ColorItemSelected, HS.Left + 4, HS.Right-8, ARect.Top + mg, ARect.Bottom - mg);
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_NUM_SERIE];  // ID série
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNumeroDeSerie()]));
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_ST_DEPART];  // Départ
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieDep, sr.GetNoPointDep]));
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_ST_ARRIVEE];  // Arrivée
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieArr, sr.GetNoPointArr]));
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_NOM_SERIE];  // nom
      QR := Rect(HS.Left, ARect.Top + mg, HS.Right - 8, ARect.Bottom - mg);
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextRect(QR, HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(sr.GetNomSerie));
      // entrée
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_ENTREE];  // nom
      QR := Rect(HS.Left, ARect.Top + mg, HS.Right - 8, ARect.Bottom - mg);
      DessineFiletColonne(HS.Left - Q4);
      en := FDocuTopo.GetEntree(sr.GetNumeroEntrance());
      Canvas.TextRect(QR, HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(en.eNomEntree));
      {$IFDEF GROS_MINET}
      ;   // pas d'affichage des réseaux
      {$ELSE}
      // réseau
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_RESEAU];  // nom

      DessineFiletColonne(HS.Left - Q4);
      rs := FDocuTopo.GetReseau(sr.GetNumeroReseau());
      DessineRectangleColore(bg, clBlack, rs.ColorReseau, HS.Left, HS.Left + 32, ARect.Top + mg, ARect.Bottom - mg);
      Canvas.TextOut(HS.Left + 32 + 4, ARect.Top+1, _AnsiToLCLStr(rs.NomReseau));
      {$ENDIF}
      Canvas.Brush.Color:=bg;
      canvas.Pen.Color:=clSilver;
      // Nb points
      HS := hcColsTitres.Sections.Items[HEADER_LST_NUM_COL_NB_VISEES];
      DessineFiletColonne(HS.Left - Q4);
      canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNbVisees]));
    end;
  end;
begin
  //try
    sr := FDocuTopo.GetSerie(FListeSelectedSeries.GetElement(Index));   //FDocumentToporobot.GetSerie(Index);
    if (odSelected in state) then DessineItem(clBlue, clWhite)
                             else DessineItem(clwhite, clBlack);
  //except
  //end;
end;


end.

