unit CadreVue3DExt;

{$INCLUDE CompilationParameters.inc}
// Date: 07/11/2013
// Statut: Opérationnel en interne
// TODO: Implanter l'interaction avec l'extérieur
// 07/04/2014: Export SVG OK
// 08/04/2014: Nouvelle version de Export SVG utilisant SVGCanvasUnit
// 11/02/2015: Paramètres de vue 3D sont groupés dans un TVue3DParams
// 27/04/2016: support du drag/drop souris pour les paramètres de vue 3D (aka GHTopo Delphi)
//             TODO: Des détails à revoir
interface

uses
  StructuresDonnees,
  Common,
  Classes,
  CalculMatriciel3x3,
  UnitEntitesExtended,
  UnitListesSimplesWithGeneriques,
  unitProfilTopo,
  UnitClasseMaillage,
  //FPimage, fpvectorial, fpvutils,    // Penser à installer le paquet fpvectorial (dans $(LAZARUS)/components/fpvectorial/)
  {$IFDEF MSWINDOWS}
  windows,
  {$ENDIF}
  SysUtils, FileUtil, Forms, Controls,  ExtCtrls,
  Graphics,
  BGRABitmap,
  unitUtilsComposants,
  SVGCanvasUnit;
// types locaux
type TViseesTopo2D = record
  ColorEntite         : integer;                 // ID unique de l'entité
  Type_Entite       : TTypeDeVisee;             // Type d'entité
  DateLeve          : TDateTime;
  Sub_System        : integer;                 // Sous-réseau
  Une_Station_1_X   : double;                  // Extrémités des visées
  Une_Station_1_Y   : double;
  Une_Station_2_X   : double;                // Extrémités des visées
  Une_Station_2_Y   : double;
  CoordEcrSt1X      : integer;
  CoordEcrSt2X      : integer;
  CoordEcrSt2Y      : integer;
  CoordEcrSt1Y      : integer;
  Depth             : Double;                // Profondeur (distance à l'observateur);
  Drawn             : boolean;
end;
type TPoint2DDepth = record
  X    : double;
  Y    : double;
  Depth: double;
end;

//******************************************************************************
type

  { TCdrVue3DExt }

  TCdrVue3DExt = class(TFrame)
    pnlVue: TPanel;
    Vue: TPaintBox;
    procedure VueMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure VueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure VueMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure VuePaint(Sender: TObject);
  private
    { private declarations }
    // position de la souris (début et fin)
    FPos0: TPoint;
    FBDDEntites: TBDDEntites;
    // callback pour rafraichissement
    FProcRefreshControls: TProcRefreshControlsBy3DView;
    // drapeaux
    FDoDraw        : boolean;         // dessin OK
    // paramètres de la vue 3D
    FVue3DParams: TVue3DParams;
    // tables de précalcul
    FCubeEnglobant : array[1..8] of TPoint;
    FPolygonale  : array of TViseesTopo2D; // polygonale
    //FTableQuads          : array of TQuad;         // table des facettes
    FTableTubesCavite    : TListePortionsTube; // array of TQuad;         // table des facettes
    // variables géométriques calculées
    FLookAt              : TPoint3Df;      // point d'observation
    FOffset              : TPoint3Df;      // décalage après centrage
    FLongueurDiagonale   : double;         // longueur de la diagonale du cube enveloppe de la cavité
    FTransfoInt          : TPoint;
    FRapportME           : double;
    // éléments de la matrice de transformation
    FMatAux: array[1..8] of Double;

    // maillage
    FMyMaillage      : TMaillage;

    procedure CalcProjPolygonale(const DoDestTCanvas: boolean; const DoCalcViseesRadiantes: boolean);
    procedure DrawAnnotations(const QBmp: TBGRABitmap);
    procedure DrawVolumesCavite(const QBmp: TBGRABitmap);
    procedure DrawCube(const QBmp: TBGRABitmap);
    procedure DrawDraftPolygonale();
    procedure DrawEntrances(const QBmp: TBGRABitmap);
    procedure DrawPolygonales(const QBmp: TBGRABitmap; const Colored: boolean);
    procedure DrawReferentiel(const QBmp: TBGRABitmap);
    function Get2DDepthCoordinates(const X, Y, Z: double): TPoint2DDepth;
    function GetScreenCoordinates(const X, Y: double; const DoConvertTCanvasCoords: boolean): TPoint;
    function MakePortionTubeVisee(const E: TEntiteEtendue; out APortionTube: TPortionTubeVisee): boolean;
    //function MakeFPColor(const C: TColor): TFPColor;
    procedure MakeVolumesCavite();
    function  PrecalculVolumesCavite(const DoDestTCanvas: boolean): boolean;
    procedure QSortDatasByDepth();
    procedure RedessinVue();
    procedure ResetParamTransformation();

    procedure SetParamTransformation(const QTheta, QPhi, QZoom, QMagnification: double; const DoDestTCanvas: boolean; const DoCalcViseesRadiantes: boolean);
    procedure PrecalculerMaillage();
    procedure TracerMaillage();
    procedure TracerProfils(const TmpBuffer: TBGRABitmap);


  public
    { public declarations }
    function  InitialiseVue3D(const QBDD      : TBDDEntites;
                              const MyMaillage: TMaillage;
                              const MyFiltre  : string;
                              const PC: TProcRefreshControlsBy3DView): boolean;


    procedure FinaliseVue3D();
    procedure SetAndRedessVue3D(const P: TVue3DParams); overload;
    procedure SetAndRedessVue3D(const QM: TModeRepresentationGaleries;
                                const QColorZMini, QColorZMaxi: TColor;
                                const QTheta, QPhi, QZoom, QFactZ: double;
                                const QFiltres: string; const QDoFiltrer: boolean;
                                const QDoDrawVolumes: boolean); overload;
    function  GetTheta: double;
    function  GetPhi: double;
    function  GetZoom: double;
    function  GetFactZ: double;
    function  GetColorZMini: TColor;
    function  GetColorZMaxi: TColor;
    function  GetFiltres: string;
    function  GetDoFiltrer: boolean;

    procedure SetParamsVue3D(const P: TVue3DParams);
    function  GetParamsVue3D(): TVue3DParams;
    //procedure ExporterVue3DEnSVG(const FileName: string; const FM: TModeRepresentationGaleries);
    procedure ExporterVue3DEnSVG_TSVGCanvas(const FileName: string;
                                            const DoMakeXHTML: boolean;
                                            const FM: TModeRepresentationGaleries;
                                            const FillOpacity: byte);
    procedure ExportGCP(const QFileName: TStringDirectoryFilename);

  end;

implementation

{$R *.lfm}

const
  PI180          = PI/180;
  NBFACESBYVISEE = 6;
  LOW_INDEX      = 1;

  NB_MAX_VISEES_ANTENNES_AFFICHEES = 50000;

{ TCdrVue3DExt }

function TCdrVue3DExt.InitialiseVue3D(const QBDD: TBDDEntites;
                                      const MyMaillage: TMaillage;
                                      const MyFiltre: string;
                                      const PC: TProcRefreshControlsBy3DView): boolean;
var
  QDevelViseesVisibles: double;
begin

  AfficherMessage(Format('%s.InitialiseVue3D()', [ClassName]));
  self.DoubleBuffered := True;
  Result       := False;
  // procédure de rappel
  FProcRefreshControls := PC;
  // pointeur sur les données calculées
  FBDDEntites  := QBDD;
  // désarmement des drapeaux
  // paramètres de la vue 3D
  FVue3DParams.VueFiltres    := MyFiltre;
  FVue3DParams.DoFiltrer     := True;
  FVue3DParams.Theta         := 45;
  FVue3DParams.Phi           := 32;
  FVue3DParams.Zoom          := 1.00;
  FVue3DParams.CoefMagnification  := 1.00;
  FVue3DParams.ModeRepresentation := rgDEPTH;
  FVue3DParams.Elements3DDessines := [edPolygonals, edStations, edIDStations, edAltitudes, edCotes,
                                      edWalls, edCrossSections, edFillGalerie,
                                      edQuadrilles, edBounds, edReferentiel,
                                      edENTRANCES, edANNOTATIONS, edANTENNES];
  FVue3DParams.FillOpacity        := 128;

  FVue3DParams.BackGround    := clWhite;
  FVue3DParams.ViseesLargeur := 3;

  FMyMaillage := MyMaillage;
  // chargement entités
  try
    // préparation tables temporaires
    with FBDDEntites do
    begin
      // filtres
      MetaFiltre(MyFiltre, QDevelViseesVisibles);
      // mini et maxi avec filtres activés
      SetMinMax(True);
      // initialisation des tables d'affichage
      SetLength(FPolygonale,0);
      SetLength(FPolygonale, GetNbEntitesVisees() + 1);
      FTableTubesCavite := TListePortionsTube.Create;
      FTableTubesCavite.ClearListe();
    end;
    // mise en place des paramètres
    ResetParamTransformation;
    // construction des volumes et précalcul
    MakeVolumesCavite;
    PrecalculVolumesCavite(True);
    // trier la topo
    QSortDatasByDepth;
    // redessin
    RedessinVue();
    // OK ?
    AfficherMemoryUsage();
    Result := True;
  except
  end;
end;



procedure TCdrVue3DExt.FinaliseVue3D();
begin
  try
    FTableTubesCavite.ClearListe();
  finally
    FTableTubesCavite.Free;
  end;
end;



// réglage et affichage de la vue
// Statut: OK
procedure TCdrVue3DExt.SetAndRedessVue3D(const QM: TModeRepresentationGaleries;
                                         const QColorZMini, QColorZMaxi: TColor;
                                         const QTheta, QPhi, QZoom, QFactZ: double;
                                         const QFiltres: string;
                                         const QDoFiltrer: boolean;
                                         const QDoDrawVolumes: boolean);
var
  QDevelViseesVisibles: double;
begin
  // filtres
  FVue3DParams.DoFiltrer := QDoFiltrer;
  if (QDoDrawVolumes) then Include(FVue3DParams.Elements3DDessines, edVolumes)
                      else Exclude(FVue3DParams.Elements3DDessines, edVolumes);
  if (FVue3DParams.DoFiltrer) then
  begin
    FVue3DParams.VueFiltres := QFiltres;
    FBDDEntites.MetaFiltre(QFiltres, QDevelViseesVisibles);
  end;
  FBDDEntites.SetMinMax(True); // toujours utiliser SetMinMax(True)

  ResetParamTransformation;
  SetParamsVue3D(FVue3DParams);
  // modes de représentation
  FVue3DParams.ModeRepresentation := QM;
  // calcul des dégradés (ssi les couleurs changent)
  if ((QColorZMini <> FVue3DParams.ColorZMini) OR (QColorZMaxi <> FVue3DParams.ColorZMaxi)) then
  begin
    FVue3DParams.ColorZMini := QColorZMini;
    FVue3DParams.ColorZMaxi := QColorZMaxi;
    FBDDEntites.CalcCouleursByDepth(FVue3DParams.ColorZMini, FVue3DParams.ColorZMaxi);
  end;
  // paramétrage angles et zooms
  SetParamTransformation(QTheta, QPhi, QZoom, QFactZ, True, True);
  // construction des volumes et précalcul
  MakeVolumesCavite;
  PrecalculVolumesCavite(True);
  // trier la topo
  QSortDatasByDepth;
  // redessin
  RedessinVue();
  // OK ?
end;
procedure TCdrVue3DExt.SetAndRedessVue3D(const P: TVue3DParams); overload;
var
  QDevelViseesVisibles: double;
begin
  FVue3DParams := P;
  // filtres
  if (FVue3DParams.DoFiltrer) then
  begin
    FVue3DParams.VueFiltres := FVue3DParams.VueFiltres;
    FBDDEntites.MetaFiltre(FVue3DParams.VueFiltres, QDevelViseesVisibles);
    FBDDEntites.SetMinMax(True);
  end;
  // calcul des dégradés (ssi les couleurs changent)
  if ((P.ColorZMini <> FVue3DParams.ColorZMini) OR (P.ColorZMaxi <> FVue3DParams.ColorZMaxi)) then
  begin
    FBDDEntites.CalcCouleursByDepth(FVue3DParams.ColorZMini, FVue3DParams.ColorZMaxi);
  end;
  // paramétrage angles et zooms
  SetParamTransformation(FVue3DParams.Theta, FVue3DParams.Phi, FVue3DParams.Zoom, FVue3DParams.CoefMagnification, True, True);
  // construction des volumes et précalcul
  MakeVolumesCavite;
  PrecalculVolumesCavite(True);
  // trier la topo
  QSortDatasByDepth;
  // redessin
  RedessinVue;
  // OK ?
end;

function TCdrVue3DExt.GetTheta: double;
begin
  Result := FVue3DParams.Theta;
end;

function TCdrVue3DExt.GetPhi: double;
begin
  Result := FVue3DParams.Phi;
end;

function TCdrVue3DExt.GetZoom: double;
begin
  Result := FVue3DParams.Zoom;
end;

function TCdrVue3DExt.GetFactZ: double;
begin
  Result := FVue3DParams.CoefMagnification;
end;

function TCdrVue3DExt.GetColorZMini: TColor;
begin
  Result := FVue3DParams.ColorZMini;
end;

function TCdrVue3DExt.GetColorZMaxi: TColor;
begin
  Result := FVue3DParams.ColorZMaxi;
end;

function TCdrVue3DExt.GetFiltres: string;
begin
  Result := FVue3DParams.VueFiltres;
end;

function TCdrVue3DExt.GetDoFiltrer: boolean;
begin
  Result := FVue3DParams.DoFiltrer;
end;

procedure TCdrVue3DExt.SetParamsVue3D(const P: TVue3DParams);
begin
  FVue3DParams := P;

end;



function TCdrVue3DExt.GetParamsVue3D(): TVue3DParams;
begin
  Result := FVue3DParams;
end;



// remise à zéro des paramètres de transformation
procedure TCdrVue3DExt.ResetParamTransformation();
var
  cnMini, cnMaxi: TPoint3Df;
begin
  cnMini := FBDDEntites.GetCoinBasGauche;
  cnMaxi := FBDDEntites.GetCoinHautDroit;
  with FBDDEntites do begin
    FLongueurDiagonale := Hypot3D(cnMaxi.X-cnMini.X,
                                  cnMaxi.Y-cnMini.Y,
                                  cnMaxi.Z-cnMini.Z);
    //AfficherMessage(Format('   Mini: %.2f - %.2f - %.2f', [cnMini.X, cnMini.Y, cnMini.Z]));
    //AfficherMessage(Format('   Maxi: %.2f - %.2f - %.2f', [cnMaxi.X, cnMaxi.Y, cnMaxi.Z]));

    FOffset := MakeTPoint3Df(-(cnMini.X+ 0.5*(cnMaxi.X - cnMini.X)),
                             -(cnMini.Y+ 0.5*(cnMaxi.Y - cnMini.Y)),
                             -(cnMini.Z+ 0.5*(cnMaxi.Z - cnMini.Z)));

    //AfficherMessage(Format('   Offset: %.2f - %.2f - %.2f', [FOffset.X, FOffset.Y, FOffset.Z]));
  end;
  //SetParamTransformation(0.00, 0.00, 1.00, 1.00);
  SetParamTransformation(45.00, 32.00, 1.00, 1.00, True, True);
end;
// paramètres de transformation angles en degrés
procedure TCdrVue3DExt.SetParamTransformation(const QTheta, QPhi, QZoom, QMagnification: double; const DoDestTCanvas: boolean; const DoCalcViseesRadiantes: boolean);
const
  Marge = 5;
var
  c: TPoint2DDepth;
  cnMini, cnMaxi: TPoint3Df;
  MiouTheta, MiouPhi: ValReal;
begin
  FVue3DParams.Zoom  := QZoom;
  FVue3DParams.Theta := QTheta;
  FVue3DParams.Phi   := QPhi;
  FVue3DParams.CoefMagnification := QMagnification;
  MiouTheta := FVue3DParams.Theta * PI180;
  MiouPhi   := FVue3DParams.Phi   * PI180;

  //AfficherMessage(Format('%s.setParamTransformation(Theta = %.0f; Phi = %.0f; Zoom: %.3f; Magn. = %.3f', [ClassName, QTheta, QPhi, QZoom, QMagnification]));
  FMatAux[1] := Sin(MiouTheta);
  FMatAux[2] := Sin(MiouPhi);                     // | -sin(A)           +cos(A)         0           0 | | x |   | X |
  FMatAux[3] := Cos(MiouTheta);                   // |                                                 | |   |   |   |
  FMatAux[4] := Cos(MiouPhi);                     // | -cos(A).sin(P)    -sin(A).sin(P)  cos(A)      0 | | y |   | Y |
  FMatAux[5] := FMatAux[3] * FMatAux[2];          // |                                                 |*|   | = |   |
  FMatAux[6] := FMatAux[1] * FMatAux[2];          // | -cos(A).cos(P)    -sin(A).cos(P)  -sin(P)     R | | z |   | Z |
  FMatAux[7] := FMatAux[3] * FMatAux[4];          // |                                                 | |   |   |   |
  FMatAux[8] := FMatAux[1] * FMatAux[4];          // | 0                 0               0           1 | | 1 |   | 1 |

  FTransfoInt.X := 0;
  FTransfoInt.Y := 0;
  FTransfoInt.X :=  Vue.Width  div 2;
  FTransfoInt.Y :=  Vue.Height div 2;
  FRapportME    := (Vue.Width/FLongueurDiagonale) *  FVue3DParams.Zoom;
  //AfficherMessage(Format('FRapportME=%.5f FT=(%d, %d)',
  //                       [FRapportME, FT.X, FT.Y]));
  //SetTransformationMatrix(FVue3DParams.Theta,  FVue3DParams.Phi, FZoom, FOffset);
  CalcProjPolygonale(DoDestTCanvas, DoCalcViseesRadiantes);
  // direction d'observation
  FLookAt := MakeTPoint3Df(Cos(MiouPhi)*Cos(MiouTheta),
                           Cos(MiouPhi)*sin(MiouTheta),
                           sin(MiouPhi));
  // cube
  cnMini := FBDDEntites.GetCoinBasGauche;
  cnMaxi := FBDDEntites.GetCoinHautDroit;
  c := Get2DDepthCoordinates(cnMini.X, cnMini.Y, cnMini.Z);
  FCubeEnglobant[1] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c := Get2DDepthCoordinates(cnMaxi.X, cnMini.Y, cnMini.Z);
  FCubeEnglobant[2] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c :=Get2DDepthCoordinates(cnMaxi.X, cnMaxi.Y, cnMini.Z);
  FCubeEnglobant[3] :=GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c := Get2DDepthCoordinates(cnMini.X, cnMaxi.Y, cnMini.Z);
  FCubeEnglobant[4] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c :=Get2DDepthCoordinates(cnMini.X, cnMini.Y, cnMaxi.Z);
  FCubeEnglobant[5] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c := Get2DDepthCoordinates(cnMaxi.X, cnMini.Y, cnMaxi.Z);
  FCubeEnglobant[6] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c := Get2DDepthCoordinates(cnMaxi.X, cnMaxi.Y, cnMaxi.Z);
  FCubeEnglobant[7] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  c := Get2DDepthCoordinates(cnMini.X, cnMaxi.Y, cnMaxi.Z);
  FCubeEnglobant[8] := GetScreenCoordinates(c.X, C.Y, DoDestTCanvas);
  // précalculer les volumes
  FDoDraw := PrecalculVolumesCavite(DoDestTCanvas);

end;


procedure TCdrVue3DExt.VueMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) then FPos0 := MakeTPoint(x, y);
end;





procedure TCdrVue3DExt.VueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  dx, dy: Integer;
begin
  if (Shift = [ssLeft]) then
  begin
    //AfficherMessageErreur(format('%d, %d', [X, Y]));
    //dx := (X - FPos0.X) div 64;
    //dy := (Y - FPos0.Y) div 128;
    dx := (X - (self.Width div 2));
    dy := (Y - (self.Height div 2));



    FVue3DParams.Theta := -(X - (self.Width div 2)); //-(X - FPos0.X);
    if (FVue3DParams.Theta > 360.0) then FVue3DParams.Theta := 0.0;
    if (FVue3DParams.Theta <   0.0) then FVue3DParams.Theta :=   360.0;

    FVue3DParams.Phi   :=  Y - (self.Height div 2); //Y - FPos0.Y;
    if (FVue3DParams.Phi >  90.0) then FVue3DParams.Phi :=  90.0;
    if (FVue3DParams.Phi <   0.0) then FVue3DParams.Phi :=   0.0;


    //AfficherMessageErreur(Format('(%d, %d)  T=%.2f, P=%.0f', [X, Y, FVue3DParams.Theta, FVue3DParams.Phi]));
    SetParamTransformation(FVue3DParams.Theta, FVue3DParams.Phi, FVue3DParams.Zoom, FVue3DParams.CoefMagnification, True, False);
    DrawDraftPolygonale;
    if (Assigned(FProcRefreshControls)) then FProcRefreshControls();

  end;
end;

procedure TCdrVue3DExt.VueMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  try
    SetAndRedessVue3D(FVue3DParams);
  except
  end;
end;




// coordonnées de transferts
function TCdrVue3DExt.Get2DDepthCoordinates(const X, Y, Z: double): TPoint2DDepth;
begin
  Result.X := -(X+FOffset.X) * FMatAux[1] +
               (Y+FOffset.Y) * FMatAux[3];
  Result.Y := -(X+FOffset.X) * FMatAux[5]
              -(Y+FOffset.Y) * FMatAux[6]+
               (FVue3DParams.CoefMagnification*(Z+FOffset.Z)) * FMatAux[4];
  Result.Depth:= (-(X+FOffset.Z) * FMatAux[7]
                  -(Y+FOffset.Z) * FMatAux[8]
                  -(Z+FOffset.Z) * FMatAux[2]);
end;
function TCdrVue3DExt.GetScreenCoordinates (const X,Y: double; const DoConvertTCanvasCoords: boolean): TPoint;
begin
  if (DoConvertTCanvasCoords) then
  begin
    Result.X := round(X * FRapportME) + FTransfoInt.X;
    Result.Y := Vue.Height - (round(Y * FRapportME) + FTransfoInt.Y);
  end
  else
  begin
    Result.X := round(X * FRapportME) + FTransfoInt.X;
    Result.Y := round(Y * FRapportME) + FTransfoInt.Y;
  end;
end;

procedure TCdrVue3DExt.VuePaint(Sender: TObject);
begin
  RedessinVue;
end;

// calcul de la polygonale projetée
procedure TCdrVue3DExt.CalcProjPolygonale(const DoDestTCanvas: boolean; const DoCalcViseesRadiantes: boolean);
var
  i:integer;
  V: TViseesTopo2D;
  E : TEntiteEtendue;
  d1: double;
  PtOut: TPoint2DDepth;
  P: TPoint;
begin
  for i := LOW_INDEX to FBDDEntites.GetNbEntitesVisees() - 1 do
  begin
    E := FBDDEntites.GetEntiteVisee(i);
    if ((not DoCalcViseesRadiantes) and (E.Type_Entite = tgVISEE_RADIANTE)) then Continue;
    V.ColorEntite := FBDDEntites.GetColorViseeFromModeRepresentation(FVue3DParams.ModeRepresentation, E);
    // à factoriser
    V.Type_Entite  := E.Type_Entite;
    V.DateLeve     := E.DateLeve;
    PtOut :=  Get2DDepthCoordinates(E.Une_Station_1_X,
                                    E.Une_Station_1_Y,
                                    E.Une_Station_1_Z);
    V.Une_Station_1_X := PtOut.X;
    V.Une_Station_1_Y := PtOut.Y;
    P:=GetScreenCoordinates(PtOut.X, PtOut.Y, DoDestTCanvas);
    V.CoordEcrSt1X:=P.X;
    V.CoordEcrSt1Y:=P.Y;
    d1:=PtOut.Depth;
    PtOut :=  Get2DDepthCoordinates(E.Une_Station_2_X,
                                    E.Une_Station_2_Y,
                                    E.Une_Station_2_Z);
    V.Une_Station_2_X := PtOut.X;
    V.Une_Station_2_Y := PtOut.Y;
    V.Depth           := 0.50 * (d1+PtOut.Depth);
    P := GetScreenCoordinates(PtOut.X, PtOut.Y, DoDestTCanvas);
    V.CoordEcrSt2X:=P.X;
    V.CoordEcrSt2Y:=P.Y;
    V.Drawn := E.Drawn;
    FPolygonale[i]:=V;
  end;
end;
// export de la coupe projetée en GCP  GHCaveDraw Centerline
procedure TCdrVue3DExt.ExportGCP(const QFileName: TStringDirectoryFilename);
var
  fp: TextFile;
  i, Nb, NbViseesAntenne: Integer;
  EWE  : TEntiteEtendue;
  V    : TViseesTopo2D;
  PtOutExtr1, PtOutExtr2: TPoint2DDepth;
  PD, PG                : TPoint2DDepth;
  QID: Int64;
  procedure WrtLn(const S: string); inline;
  begin
    writeln(fp, S);
  end;
begin
  if (Not FDoDraw) then Exit;
  Nb := FBDDEntites.GetNbEntitesVisees();
  AfficherMessage(Format('%s.ExportGCP(): %s - %d shots', [ClassName, QFileName, Nb]));
  AssignFile(fp, QFileName);
  NbViseesAntenne := 0;
  try
    ReWrite(fp);

    WrtLn(Format('# Coupe projetee: %s - %s', [QFileName, DateTimeToStr(Now())]));
    WrtLn(Format('# Theta = %.2f; Phi = %.2f; Magn = %.2f', [FVue3DParams.Theta, FVue3DParams.Phi, FVue3DParams.CoefMagnification]));

    WrtLn('# Base points');
    WrtLn('');
    WriteLn(fp, 'basepoints');
    for i := 0 to Nb - 1 do
    begin
      EWE := FBDDEntites.GetEntiteVisee(i);
      // NOTA: Les visées en antennes ne sont pas prises en compte par
      //       les fonctions de recherche et d'indexation de GHCaveDraw
      QID := MakeGHCaveDrawIDPtCenterline(EWE, NbViseesAntenne);


      PtOutExtr1 :=  Get2DDepthCoordinates(EWE.Une_Station_1_X,
                                           EWE.Une_Station_1_Y,
                                           EWE.Une_Station_1_Z);
      PtOutExtr2 :=  Get2DDepthCoordinates(EWE.Une_Station_2_X,
                                           EWE.Une_Station_2_Y,
                                           EWE.Une_Station_2_Z);
      //PD := Get2DDepthCoordinates(EWE.X1PD, EWE.Y1PD, EWE.Z1PB);
      //PG := Get2DDepthCoordinates(EWE.X1PD, EWE.Y1PD, EWE.Z1PB);

      //EWE.X1PD := EWE.Une_Station_1_X;
      //EWE.X1PD := EWE.Une_Station_1_X;


      WrtLn(Format(FMT_BASEPOINTS,
                  [QID  ,
                   EWE.oIDLitteral,
                   EWE.Type_Entite,
                   FBDDEntites.GetCouleurEntiteRGBByExpe(EWE), //EWE.ColorEntite,
                   PtOutExtr1.X, PtOutExtr1.Y, EWE.Une_Station_1_Z,
                   PtOutExtr2.X, PtOutExtr2.Y, EWE.Une_Station_2_Z,
                   EWE.X2PG, EWE.Y2PG,  EWE.Z1PB,
                   EWE.X2PD, EWE.Y2PD,  EWE.Z1PH]));

    end;
    WriteLn(fp, 'endbasepoints');
  finally
    Closefile(fp);
  end;
end;



// fabrication des volumes de la scène

function TCdrVue3DExt.MakePortionTubeVisee(const E: TEntiteEtendue; out APortionTube: TPortionTubeVisee): boolean;
var
  P1, P2, P3, P4: TPoint3Df;
  QC: TColor;
  function MakeQuad(const QP1, QP2, QP3, QP4: TPoint3Df): TQuad;
  var
    Q: TQuad;
  begin
    Result.Depth := 0.00;
    Result.VertexA := QP1;
    Result.VertexB := QP2;
    Result.VertexC := QP3;
    Result.VertexD := QP4;
  end;
begin
  result := false;
  if (E.Type_Entite in [tgENTRANCE, tgSURFACE, tgVISEE_RADIANTE]) then Exit(false);
  QC := FBDDEntites.GetColorViseeFromModeRepresentation(FVue3DParams.ModeRepresentation, E);
  APortionTube.Couleur     := QC;
  APortionTube.IdxExpe     := E.eExpe;
  APortionTube.IdxReseau   := E.eReseau;
  APortionTube.IdxSecteur  := E.eSecteur;
  APortionTube.Type_Entite := E.Type_Entite;
  APortionTube.Drawn       := E.Drawn;
  if (not APortionTube.Drawn) then Exit(false);
  with E do
  begin
    // paroi gauche Quad 1
    P1 := MakeTPoint3Df(X2PG, Y2PG, Z2PB);
    P2 := MakeTPoint3Df(X1PG, Y1PG, Z1PB);
    P3 := MakeTPoint3Df(X1PG, Y1PG, Z1PH);
    P4 := MakeTPoint3Df(X2PG, Y2PG, Z2PH);
    APortionTube.Facettes[1] := MakeQuad(P1, P2, P3, P4);
    // paroi droite Quad 2
    P1 := MakeTPoint3Df(X1PD, Y1PD, Z1PB);
    P2 := MakeTPoint3Df(X2PD, Y2PD, Z2PB);
    P3 := MakeTPoint3Df(X2PD, Y2PD, Z2PH);
    P4 := MakeTPoint3Df(X1PD, Y1PD, Z1PH);
    APortionTube.Facettes[2] := MakeQuad(P1, P2, P3, P4);
    // paroi de dessus Quad 3
    P1 := MakeTPoint3Df(X1PD, Y1PD, Z1PH);
    P2 := MakeTPoint3Df(X2PD, Y2PD, Z2PH);
    P3 := MakeTPoint3Df(X2PG, Y2PG, Z2PH);
    P4 := MakeTPoint3Df(X1PG, Y1PG, Z1PH);
    APortionTube.Facettes[3] := MakeQuad(P1, P2, P3, P4);
    // paroi de dessous Quad 4
    P1 := MakeTPoint3Df(X1PG, Y1PG, Z1PB);
    P2 := MakeTPoint3Df(X2PG, Y2PG, Z2PB);
    P3 := MakeTPoint3Df(X2PD, Y2PD, Z2PB);
    P4 := MakeTPoint3Df(X1PD, Y1PD, Z1PB);
    APortionTube.Facettes[4] := MakeQuad(P1, P2, P3, P4);
    // paroi de face
    P1 := MakeTPoint3Df(X1PG, Y1PG, Z1PB);
    P2 := MakeTPoint3Df(X1PD, Y1PD, Z1PB);
    P3 := MakeTPoint3Df(X1PD, Y1PD, Z1PH);
    P4 := MakeTPoint3Df(X1PG, Y1PG, Z1PH);
    APortionTube.Facettes[5] := MakeQuad(P1, P2, P3, P4);
    // paroi de derrière
    P1 := MakeTPoint3Df(X2PD, Y2PD, Z2PB);
    P2 := MakeTPoint3Df(X2PG, Y2PG, Z2PB);
    P3 := MakeTPoint3Df(X2PG, Y2PG, Z2PH);
    P4 := MakeTPoint3Df(X2PD, Y2PD, Z2PH);
    APortionTube.Facettes[6] := MakeQuad(P1, P2, P3, P4);
    result := True;
  end; //with E do begin
end;

procedure TCdrVue3DExt.MakeVolumesCavite();
var
  i: integer;
  P1, P2, P3, P4: TPoint3Df;
  E: TEntiteEtendue;
  QC : TColor;
  PTC: TPortionTubeVisee;
begin
  //AfficherMessage(Format('%s.MakeVolumesCavite',[ClassName]));
  FTableTubesCavite.ClearListe();
  for i := LOW_INDEX to FBDDEntites.GetNbEntitesVisees() - 1 do
  begin
    E := FBDDEntites.GetEntiteVisee(i);
    QC := FBDDEntites.GetColorViseeFromModeRepresentation(FVue3DParams.ModeRepresentation, E);
    if (MakePortionTubeVisee(E, PTC)) then FTableTubesCavite.AddElement(PTC);
  end; //for i:=0 to FTableEntites do begin
end;
// précalcul des volumes
function TCdrVue3DExt.PrecalculVolumesCavite(const DoDestTCanvas: boolean): boolean;
var
  i, Nb: integer;
  MyPortionTube: TPortionTubeVisee;
  procedure CalcPortionTube(var PTC: TPortionTubeVisee);
  var
    k: Integer;
    procedure Calc3DQuad(var QD: TQuad);

    var
      PP: TPoint2DDepth;
      U, V, W1, W2, M: TPoint3Df;
      NW, PS: Double;
    begin
      with QD do
      begin
        PP:=Get2DDepthCoordinates(VertexA.X, VertexA.Y, VertexA.Z);
        Vertex2D_A := GetScreenCoordinates(PP.X, PP.Y, DoDestTCanvas);
        PP:=Get2DDepthCoordinates(VertexB.X, VertexB.Y, VertexB.Z);
        Vertex2D_B := GetScreenCoordinates(PP.X, PP.Y, DoDestTCanvas);
        PP:=Get2DDepthCoordinates(VertexC.X, VertexC.Y, VertexC.Z);
        Vertex2D_C := GetScreenCoordinates(PP.X, PP.Y, DoDestTCanvas);
        PP:=Get2DDepthCoordinates(VertexD.X, VertexD.Y, VertexD.Z);
        Vertex2D_D := GetScreenCoordinates(PP.X, PP.Y, DoDestTCanvas);
        //test de visibilité
        U := MakeTPoint3Df(VertexB.X-VertexA.X, VertexB.Y-VertexA.Y, VertexB.Z-VertexA.Z);
        V := MakeTPoint3Df(VertexC.X-VertexA.X, VertexC.Y-VertexA.Y, VertexC.Z-VertexA.Z);
        W1:= ProduitVectoriel(U,V,False);
        U := MakeTPoint3Df(VertexC.X-VertexA.X, VertexC.Y-VertexA.Y, VertexC.Z-VertexA.Z);
        V := MakeTPoint3Df(VertexD.X-VertexA.X, VertexD.Y-VertexA.Y, VertexD.Z-VertexA.Z);
        W2:= ProduitVectoriel(U,V,False);
        W1.X := W1.X + W2.X;
        W1.Y := W1.Y + W2.Y;
        W1.Z := W1.Z + W2.Z;
        NW := 1e-12+Hypot3D(W1.X, W1.Y, W1.Z);
        PS := W1.X*FLookAt.X+
              W1.Y*FLookAt.Y+
              W1.Z*FLookAt.Z;
        FacetteVisible := ((PS/NW) > 0.00);
        // calcul de profondeur
        M := MakeTPoint3Df(0.25*(VertexA.X+VertexB.X+VertexC.X+VertexD.X),
                           0.25*(VertexA.Y+VertexB.Y+VertexC.Y+VertexD.Y),
                           0.25*(VertexA.Z+VertexB.Z+VertexC.Z+VertexD.Z));
        Depth := -FMatAux[7] * M.X
                 -FMatAux[8] * M.Y
                 -FMatAux[2] * M.Z;
      end;
    end; // with QD
  begin
    PTC.DepthField := 0.00;
    for k := 1 to 6 do
    begin
      Calc3DQuad(PTC.Facettes[k]);
      PTC.DepthField += PTC.Facettes[k].Depth;
    end;
    PTC.DepthField /= 6;
  end;

begin
  result := false;
  Nb := FTableTubesCavite.GetNbElements();
  if (Nb = 0) then exit(True);
  for i := LOW_INDEX to Nb - 1 do
  begin
    MyPortionTube := FTableTubesCavite.GetElement(i);
    CalcPortionTube(MyPortionTube);
    FTableTubesCavite.PutElement(i, MyPortionTube);
  end;
  result := true;
end; // precalcul

//*****************************
// tri en profondeur
procedure TCdrVue3DExt.QSortDatasByDepth();
  procedure QSortByDepth(var FPolygonale: array of TViseesTopo2D; lidx, ridx: integer);
  var
    k, e, mid: integer;
    Buffer   : TViseesTopo2D;
    bidon1,
    bidon2   : double;
  begin
    with self do
    begin
      if (lidx >= ridx) then Exit;
      mid := (lidx + ridx) div 2;
      Buffer := FPolygonale[lidx];
      FPolygonale[lidx]:=FPolygonale[mid];
      FPolygonale[mid]:=Buffer;
      e:=lidx;
      for k:=lidx+1 to ridx do
      begin
        if (FPolygonale[k].Depth > FPolygonale[lidx].Depth)  then
        begin
          Inc(e);
          Buffer := FPolygonale[e];
          FPolygonale[e]:=FPolygonale[k];
          FPolygonale[k]:=Buffer;
        end;
      end;
      Buffer := FPolygonale[lidx];
      FPolygonale[lidx]:=FPolygonale[e];
      FPolygonale[e]:=Buffer;
      QSortByDepth(FPolygonale,lidx, e-1);
      QSortByDepth(FPolygonale,e+1, ridx);
    end;
  end;
begin
  QSortByDepth(FPolygonale, LOW_INDEX, FBDDEntites.GetNbEntitesVisees() - 1);    // tri de la polygonale;
  FTableTubesCavite.SortByDepthField();                                  // tri des volumes
end;
//******************************************************************************
procedure TCdrVue3DExt.DrawCube(const QBmp: TBGRABitmap);
var i: integer;

begin
  QBmp.CanvasBGRA.Pen.Width := 0;
  QBmp.CanvasBGRA.Pen.Color:= FVue3DParams.ColorCube;
  for i := 2 to 4 do
  begin
    QBmp.CanvasBGRA.MoveTo(FCubeEnglobant[i]);
    QBmp.CanvasBGRA.LineTo(FCubeEnglobant[i+4]);
  end;
  QBmp.CanvasBGRA.MoveTo(FCubeEnglobant[1]);
  for i := 2 to 4 do QBmp.CanvasBGRA.LineTo(FCubeEnglobant[i]);
  QBmp.CanvasBGRA.LineTo(FCubeEnglobant[1]);
  for i := 5 to 8 do QBmp.CanvasBGRA.LineTo(FCubeEnglobant[i]);
  QBmp.CanvasBGRA.LineTo(FCubeEnglobant[5]);
end;
procedure TCdrVue3DExt.DrawEntrances(const QBmp: TBGRABitmap);
const R666 = 3;
var
  i, Nb: integer;
  ETT: TEntiteEtendue;
  MyEntrance: TEntrance;
  PtOut: TPoint2DDepth;
  P: TPoint;
begin
  Nb := FBDDEntites.GetNbEntrees();
  if (Nb = 0) then exit;

  CalcProjPolygonale(True, True); // indispensable: refresh
  AfficherMessage('  --> Entrees');
  QBmp.CanvasBGRA.Pen.Width := 0;
  QBmp.CanvasBGRA.Pen.Color:=clRED;


  QBmp.CanvasBGRA.Font.Height  := 14;
  QBmp.CanvasBGRA.Font.Style   := [fsBold];

  QBmp.CanvasBGRA.Font.Color := clBlue;
  QBmp.CanvasBGRA.Brush.Opacity := 255;

  for i := 0 to Nb - 1 do
  begin
    MyEntrance := FBDDEntites.GetEntree(i);
    PtOut :=  Get2DDepthCoordinates(MyEntrance.eXEntree,
                                    MyEntrance.eYEntree,
                                    MyEntrance.eZEntree);
    P := GetScreenCoordinates(PtOut.X, PtOut.Y, True);
    QBmp.CanvasBGRA.Brush.Color:=clFuchsia;
    QBmp.CanvasBGRA.EllipseC(P.x, P.y, R666, R666);
    QBmp.CanvasBGRA.Brush.Color := clWhite;
    QBmp.CanvasBGRA.TextOut(P.X + R666 + 2, P.Y + R666 + 2, MyEntrance.eNomEntree);
  end;
end;

procedure TCdrVue3DExt.DrawPolygonales(const QBmp: TBGRABitmap; const Colored: boolean);
const
  sz=4;
var
  i: integer;
begin
  // centerlines
  QBmp.CanvasBGRA.Pen.Color := clBlack;
  QBmp.CanvasBGRA.Pen.Width := 0;
  for i := 1 to FBDDEntites.GetNbEntitesVisees() - 1 do    // TODO: A revoir
  begin
    with FPolygonale[i] do
    begin
      if (Not Drawn) then Continue;
      if (Type_Entite = tgEntrance)       then Continue;
      if (Type_Entite = tgVISEE_RADIANTE) then Continue;
      if (Colored) then
      begin
        QBmp.CanvasBGRA.Pen.Color := ColorEntite;
        QBmp.CanvasBGRA.Pen.Width := FVue3DParams.ViseesLargeur;
      end;
      QBmp.CanvasBGRA.MoveTo(CoordEcrSt1X, CoordEcrSt1Y);
      QBmp.CanvasBGRA.LineTo(CoordEcrSt2X, CoordEcrSt2Y);
    end;
  end;
  // visées radiantes
  if (edANTENNES in FVue3DParams.Elements3DDessines) then
  begin
    QBmp.CanvasBGRA.Pen.Color := clGray;
    QBmp.CanvasBGRA.Pen.Width := 0;
    for i := 1 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      with FPolygonale[i] do
      begin
        if (Type_Entite = tgEntrance) then Continue;
        if (Not Drawn)                then Continue;
        if (Colored and (Type_Entite = tgVISEE_RADIANTE)) then
        begin
          QBmp.CanvasBGRA.MoveTo(CoordEcrSt1X, CoordEcrSt1Y);
          QBmp.CanvasBGRA.LineTo(CoordEcrSt2X, CoordEcrSt2Y);
        end;
      end;
    end;
  end;
end;
procedure TCdrVue3DExt.DrawReferentiel(const QBmp: TBGRABitmap);
const
  rSz = 40;
  rXo = 5 + rSz;
var
  rYo       : integer;
  X1, X2, X3: TPoint3Df;
  R1, R2, R3: TPoint2Df;
  Rp        : TPoint;
begin
  // calculs préliminaires
  X1 := MakeTPoint3Df(1.0, 0.0, 0.0);
  X2 := MakeTPoint3Df(0.0, 1.0, 0.0);
  X3 := MakeTPoint3Df(0.0, 0.0, 1.0);

  // transformations en repère local (ne pas utiliser Get2DDepthCoordinates())
  R1.X := -X1.X * FMatAux[1] +
           X1.Y * FMatAux[3];
  R1.Y := -X1.X * FMatAux[5] +
          -X1.Y * FMatAux[6] +
           X1.Z * FVue3DParams.CoefMagnification * FMatAux[4];
  R2.X := -X2.X * FMatAux[1] +
           X2.Y * FMatAux[3];
  R2.Y := -X2.X * FMatAux[5] +
          -X2.Y * FMatAux[6] +
           X2.Z * FVue3DParams.CoefMagnification * FMatAux[4];
  R3.X := -X3.X * FMatAux[1] +
           X3.Y * FMatAux[3];
  R3.Y := -X3.X * FMatAux[5] +
          -X3.Y * FMatAux[6] +
           X3.Z * FVue3DParams.CoefMagnification * FMatAux[4];

  rYo := QBmp.Height - rSz shl 1;
  QBmp.CanvasBGRA.Brush.Color:= FVue3DParams.BackGround;
  QBmp.CanvasBGRA.Font.Color:=clBlue;
  QBmp.CanvasBGRA.Pen.Width := 0;
  QBmp.CanvasBGRA.Pen.Color :=clRed;
  QBmp.CanvasBGRA.MoveTo(rXo, rYo);
  Rp.x := rXo + round(rSz * R1.X);
  Rp.y := rYo - round(rSz * R1.Y);
  QBmp.CanvasBGRA.LineTo(Rp.X, Rp.Y);
  QBmp.CanvasBGRA.TextOut(Rp.x+2, Rp.y+2,'X');

  QBmp.CanvasBGRA.MoveTo(rXo, rYo);
  Rp.x := rXo + round(rSz * R2.X);
  Rp.y := rYo - round(rSz * R2.Y);
  QBmp.CanvasBGRA.LineTo(Rp.X, Rp.Y);
  QBmp.CanvasBGRA.TextOut(Rp.x+2, Rp.y+2,'Y');

  QBmp.CanvasBGRA.MoveTo(rXo, rYo);
  Rp.x := rXo + round(rSz * R3.X);
  Rp.y := rYo - round(rSz * R3.Y);
  QBmp.CanvasBGRA.LineTo(Rp.X, Rp.Y);
  QBmp.CanvasBGRA.TextOut(Rp.x+2, Rp.y+2,'Z');
end;




procedure TCdrVue3DExt.DrawVolumesCavite(const QBmp: TBGRABitmap);
var
  i, k: integer;
  MyPortionTube: TPortionTubeVisee;
  procedure DrawQuad(const QD: TQuad);

  var
    P: array[0..3] of TPoint;
    v: integer;
  begin
     if (not QD.FacetteVisible)   then exit;
     P[0]:= QD.Vertex2D_A;  P[1]:= QD.Vertex2D_B;
     P[2]:= QD.Vertex2D_C;  P[3]:= QD.Vertex2D_D;

     QBmp.CanvasBGRA.Pen.Color:=clBlack;
     QBmp.CanvasBGRA.Pen.Width := 0;

     QBmp.CanvasBGRA.Brush.Color:= MyPortionTube.Couleur;
     QBmp.CanvasBGRA.MoveTo(P[0]);
     for v:=1 to 3 do QBmp.CanvasBGRA.LineTo(P[v].X,P[v].Y);
     QBmp.CanvasBGRA.Polygon(P);
  end;

begin
  QBmp.CanvasBGRA.Brush.Opacity := FVue3DParams.FillOpacity;
  for i := 1 to FTableTubesCavite.GetNbElements() - 1 do
  begin
    MyPortionTube := FTableTubesCavite.GetElement(i);
    if (MyPortionTube.Drawn) then
    begin
      for k := 1 to 6 do DrawQuad(MyPortionTube.Facettes[k]);
    end;
  end;
end;



procedure TCdrVue3DExt.DrawAnnotations(const QBmp: TBGRABitmap);
var
  i : integer;
  AN: TAnnotation;
  P2D: TPoint2DDepth;
  PP: TPoint;
  E: TEntiteEtendue;
  PosTexte: TPoint3Df;
begin
  ;
end;
// tracé rapide de la polygonale
procedure TCdrVue3DExt.DrawDraftPolygonale();
var
  //TmpBuffer: TBitMap;
  TmpBuffer: TBGRABitmap;
  R        : TRect;
begin
  if Not(FDoDraw) then Exit;
  try
    //TmpBuffer:=TBitmap.Create;
    TmpBuffer := TBGRABitmap.Create(Vue.Width, Vue.Height);
    with TmpBuffer do
    begin

      //Height:= Vue.Height;
      //Width := Vue.Width;
      R.Left:= Vue.Left;
      R.Top := Vue.Top;
      R.Bottom :=Vue.Top+Vue.Height;
      R.Right  :=Vue.Left+Vue.Width;
      CanvasBGRA.Pen.Width := 0;

      //CanvasBGRA.Pen.Mode := pmCopy;
      CanvasBGRA.Brush.Color := FVue3DParams.BackGround;
      CanvasBGRA.FillRect(R);
      DrawPolygonales(TmpBuffer, false); // polygonales
      //DrawEntrances(TmpBuffer);   // entrées
      DrawCube(TmpBuffer);        // cube (en dernier)
      DrawReferentiel(TmpBuffer); // dessin du référentiel
    end;
    //Vue.Canvas.CopyRect(R, TmpBuffer.Canvas, R);
    TmpBuffer.Draw(Vue.Canvas, 0, 0, True);
    //if (Assigned(FProcRefreshControls)) then FProcRefreshControls();
  finally
    TmpBuffer.Free;
  end;
end;

// redessin de la vue
procedure TCdrVue3DExt.RedessinVue();
var
  //TmpBuffer: TBitMap;
  TmpBuffer: TBGRABitmap;
  R        : TRect;
begin
  if (Not FDoDraw) then Exit;
  try
    //TmpBuffer := TBitmap.Create;
    TmpBuffer := TBGRABitmap.Create(Vue.Width, Vue.Height);
      R.Left:= Vue.Left;
      R.Top := Vue.Top;
      R.Bottom :=Vue.Top+Vue.Height;
      R.Right  :=Vue.Left+Vue.Width;
      TmpBuffer.CanvasBGRA.Pen.Width := 0;

      //CanvasBGRA.Pen.Mode:=pmCopy;
      TmpBuffer.CanvasBGRA.Brush.Color   := FVue3DParams.BackGround;
      TmpBuffer.CanvasBGRA.Brush.Opacity := 255; // opaque par défaut
      TmpBuffer.CanvasBGRA.FillRect(R);
      DrawPolygonales(TmpBuffer, true); // polygonales
      if (edVolumes        in FVue3DParams.Elements3DDessines) then DrawVolumesCavite(TmpBuffer);   // dessin des volumes
      if (edENTRANCES      in FVue3DParams.Elements3DDessines) then DrawEntrances(TmpBuffer);       // entrées
      if (edBounds         in FVue3DParams.Elements3DDessines) then DrawCube(TmpBuffer);            // cube (en dernier)
      if (edReferentiel    in FVue3DParams.Elements3DDessines) then DrawReferentiel(TmpBuffer);     // dessin du référentiel

      TracerProfils(TmpBuffer);                                                                              // Profils Topo
      AfficherMessage('Vue 3D ok');
    // bandeau de deuil
    //DrawBandeauDeuil(TmpBuffer.CanvasBGRA);
    TmpBuffer.Draw(Vue.Canvas, 0, 0, True);
    if (Assigned(FProcRefreshControls)) then FProcRefreshControls();
  finally
    TmpBuffer.Free;
  end;
end;

//******************************************************************************
// Réseaux  : OK (feuilles de styles)
// Secteurs : OK (feuilles de styles)
// Expés    : OK (feuilles de styles)
// Profondeurs: OK (couleur à la volée)
// Gris     : OK (feuilles de styles)
procedure TCdrVue3DExt.ExporterVue3DEnSVG_TSVGCanvas(const FileName: string; const DoMakeXHTML: boolean; const FM: TModeRepresentationGaleries; const FillOpacity: byte);
const
  NOM_STYLE_SILHOUETTE_DEFAUT   = 'SilhouetteDefaut';
  NOM_STYLE_FOND_CUBE_ENGLOBANT = 'FondCubeEnglobant';
  NOM_STYLE_CUBE_ENGLOBANT      = 'CubeEnglobant';
  FMT_STYLE_RESEAU  = 'Reseau%d';
  FMT_STYLE_SECTEUR = 'Secteur%d';
  FMT_STYLE_SEANCE  = 'Seance%d';
var
  SVGCanvas: TSVGCanvas;
  QViewBoxXMini: integer;
  QViewBoxYMini: integer;
  QViewBoxXMaxi: integer;
  QViewBoxYMaxi: integer;

  procedure CalcViewBox();
  const
    QQMax = 2000000000;
  var
    EWE: TPoint;
    i: Integer;
  begin
    QViewBoxXMini :=  QQMax;
    QViewBoxYMini :=  QQMax;
    QViewBoxXMaxi := -QQMax;
    QViewBoxYMaxi := -QQMax;
    // Cube englobant = contient tout le dessin
    for i := 1 to 8 do
    begin
      EWE := FCubeEnglobant[i];
      if (EWE.x > QViewBoxXMaxi) then QViewBoxXMaxi := EWE.x;
      if (EWE.x < QViewBoxXMini) then QViewBoxXMini := EWE.x;
      if (EWE.y > QViewBoxYMaxi) then QViewBoxYMaxi := EWE.y;
      if (EWE.y < QViewBoxYMini) then QViewBoxYMini := EWE.y;
    end;
  end;

  procedure DrawCube();
  var
    P: TArrayPoints2Df;
    i: integer;
  begin
    //AfficherMessage('  --> Cube');
    setlength(P, 4);
    P[0] := MakeTPoint2Df(FCubeEnglobant[1].X, FCubeEnglobant[1].Y);
    P[1] := MakeTPoint2Df(FCubeEnglobant[2].X, FCubeEnglobant[2].Y);
    P[2] := MakeTPoint2Df(FCubeEnglobant[3].X, FCubeEnglobant[3].Y);
    P[3] := MakeTPoint2Df(FCubeEnglobant[4].X, FCubeEnglobant[4].Y);
    //P[4] := MakeTPoint2Df(FCubeEnglobant[1].X, FCubeEnglobant[1].Y);
    SVGCanvas.DrawPolygon(NOM_STYLE_FOND_CUBE_ENGLOBANT, P);
    P[0] := MakeTPoint2Df(FCubeEnglobant[5].X, FCubeEnglobant[5].Y);
    P[1] := MakeTPoint2Df(FCubeEnglobant[6].X, FCubeEnglobant[6].Y);
    P[2] := MakeTPoint2Df(FCubeEnglobant[7].X, FCubeEnglobant[7].Y);
    P[3] := MakeTPoint2Df(FCubeEnglobant[8].X, FCubeEnglobant[8].Y);
    //P[4] := MakeTPoint2Df(FCubeEnglobant[5].X, FCubeEnglobant[5].Y);
    SVGCanvas.DrawPolygon(NOM_STYLE_CUBE_ENGLOBANT, P);
    for i:=1 to 4 do
    begin
      SVGCanvas.DrawLine(NOM_STYLE_CUBE_ENGLOBANT,
                         FCubeEnglobant[i].x, FCubeEnglobant[i].y,
                         FCubeEnglobant[i+4].X, FCubeEnglobant[i+4].Y);
    end;
  end;
  //----------------------------------------------------------------------------

  procedure DrawCavite();
  var
    i, k: integer;
    MyPortionTube: TPortionTubeVisee;
    procedure DessinerQuad(const QD: TQuad);
    var
      P: TArrayPoints2Df;
      v: integer;
      r,g,b: byte;
      QX, QY: double;
      QStyle: string;
    begin
      SetLength(P, 4);
      if (Not QD.FacetteVisible) then Exit;
      case FM of
        rgRESEAUX  : QStyle := Format(FMT_STYLE_RESEAU, [MyPortionTube.IdxReseau]);
        rgSECTEURS : QStyle := Format(FMT_STYLE_SECTEUR, [MyPortionTube.IdxSecteur]);
        rgSEANCES  : QStyle := Format(FMT_STYLE_SEANCE, [MyPortionTube.IdxExpe]);
        rgGRAY     : QStyle := NOM_STYLE_SILHOUETTE_DEFAUT;
      end;
      P[0].X := QD.Vertex2D_A.x;   P[0].Y := QD.Vertex2D_A.y;
      P[1].X := QD.Vertex2D_B.x;   P[1].Y := QD.Vertex2D_B.y;
      P[2].X := QD.Vertex2D_C.x;   P[2].Y := QD.Vertex2D_C.y;
      P[3].X := QD.Vertex2D_D.x;   P[3].Y := QD.Vertex2D_D.y;
      if (FM = rgDEPTH) then SVGCanvas.DrawPolygonSansStyle(clBlack, MyPortionTube.Couleur, 0.1, P, True)
                        else SVGCanvas.DrawPolygon(QStyle, P);
      //*)
    end;

  begin       ;
    for i := 1 to FTableTubesCavite.GetNbElements() - 1 do
    begin
      MyPortionTube := FTableTubesCavite.GetElement(i);
      if (MyPortionTube.Drawn) then
        for k := 1 to 6 do DessinerQuad(MyPortionTube.Facettes[k]);
    end;
  end;
  //----------------------------------------------------------------------------
  procedure ListerStylesReseaux();
  var
    i, Nb: Integer;
    EWE  : TReseau;
    NS   : String;
    WU: TColor;
  begin
    Nb := FBDDEntites.GetNbReseaux();
    for i := 0 to Nb - 1 do
    begin
      EWE := FBDDEntites.GetReseau(i);
      //AfficherMessage(EWE.NomReseau);
      NS  := Format(FMT_STYLE_RESEAU, [i]);
      WU  := FBDDEntites.GetCouleurRGBByIdxReseau(i);
      SVGCanvas.WriteStyleSolidPolygone(NS, clBlack, WU, FillOpacity, 0.1, EWE.NomReseau);
    end;
  end;
  procedure ListerStylesSecteurs();
  var
    i, Nb: Integer;
    EWE  : TSecteur;
    NS   : String;
    WU   : TColor;
  begin
    Nb := FBDDEntites.GetNbSecteurs();
    for i := 0 to Nb - 1 do
    begin
      EWE := FBDDEntites.GetSecteur(i);
      NS  := Format(FMT_STYLE_SECTEUR, [i]);
      WU := FBDDEntites.GetCouleurRGBByIdxSecteur(i);
      SVGCanvas.WriteStyleSolidPolygone(NS, clBlack, WU, FillOpacity, 0.1, EWE.NomSecteur);
    end;
  end;
  procedure ListerStylesSeances();
  var
    i, Nb: Integer;
    EWE  : TExpe;
    NS   : String;
    WU: TColor;
  begin
    Nb := FBDDEntites.GetNbExpes();
    for i := 0 to Nb - 1 do
    begin
      EWE := FBDDEntites.GetExpe(i);
      NS  := Format(FMT_STYLE_SEANCE, [EWE.IDExpe]);
      WU :=  FBDDEntites.GetCouleurRGBExpeByIDX(EWE.Couleur);
      SVGCanvas.WriteStyleSolidPolygone(NS, clBlack, WU, FillOpacity, 0.1, EWE.Commentaire);
    end;
  end;
  procedure zdar(const Q: boolean);
  begin
    SetParamTransformation(FVue3DParams.Theta, FVue3DParams.Phi, FVue3DParams.Zoom, FVue3DParams.CoefMagnification, Q, True);
    CalcProjPolygonale(Q, True);
    PrecalculVolumesCavite(Q);
  end;
begin
  if (Not FDoDraw) then Exit;
  AfficherMessage(Format('%s.ExporterVue3DEnSVG: %s ; %.0f, %.0f', [ClassName,FileName, FVue3DParams.Theta, FVue3DParams.Phi]));
  SVGCanvas := TSVGCanvas.Create;
  try
    zdar(False); // calcul pour SVG
      // en-tetes
      SVGCanvas.Commentaire := '';
      SVGCanvas.Scale       := 1.00;
      CalcViewBox();
      AfficherMessage(Format('-- ViewBox:  %d, %d -> %d, %d', [QViewBoxXMini, QViewBoxYMini, QViewBoxXMaxi, QViewBoxYMaxi]));
      if (Not SVGCanvas.InitializeDocument(FileName,
                                           DoMakeXHTML,
                                           'Vue 3D by ' + ApplicationName,
                                           Format('%s: Vue 3D (%.0f deg / %.0f deg)', [ExtractFileName(FileName), FVue3DParams.Theta, FVue3DParams.Phi]),
                                           QViewBoxXMini, QViewBoxYMini, QViewBoxXMaxi, QViewBoxYMaxi)) then Exit;
      SVGCanvas.BeginPatternsSection;                                                              // patterns
      SVGCanvas.EndPatternsSection;                                                                // les patterns ici
      SVGCanvas.BeginStylesSection;                                                                 // section de styles
        SVGCanvas.WriteStyleSolidLine(NOM_STYLE_CUBE_ENGLOBANT, clred, 0.1, 'bounding box cube');   // styles de cube englobant
        SVGCanvas.WriteStyleSolidPolygone(NOM_STYLE_SILHOUETTE_DEFAUT, clBlack, clGray, FillOpacity, 0.1, 'Default silhouettes');  // style par défaut: gris
        SVGCanvas.WriteStyleSolidPolygone(NOM_STYLE_FOND_CUBE_ENGLOBANT, clBlack, clSilver, FillOpacity, 0.1, 'Background');       // style du 'sol'
        case FM of          // les styles de galeries ici
          rgRESEAUX  : ListerStylesReseaux();
          rgSECTEURS : ListerStylesSecteurs();
          rgSEANCES  : ListerStylesSeances();
        end;
      SVGCanvas.EndStylesSection();
      SVGCanvas.BeginDrawingSection();
        DrawCube();                 // cube
        DrawCavite();               // dessin proprement dit
      SVGCanvas.EndDrawingSection();
      SVGCanvas.FinalizeDocument();         // cloture
    zdar(True);               // restitution du contexte
  finally
    SVGCanvas.Free;
  end;
end;

procedure TCdrVue3DExt.PrecalculerMaillage();
begin
  AfficherMessage(Format('%s.PrecalculerMaillage(): Maillage %sVALIDE', [classname, IIF(FMyMaillage.IsValidMaillage(), '', 'IN')]));
  if (not FMyMaillage.IsValidMaillage()) then Exit;
end;


procedure TCdrVue3DExt.TracerMaillage();
begin
  AfficherMessage(Format('%s.TracerMaillage(): Maillage %sVALIDE', [classname, IIF(FMyMaillage.IsValidMaillage(), '', 'IN')]));
  if (not FMyMaillage.IsValidMaillage()) then Exit;
end;
procedure TCdrVue3DExt.TracerProfils(const TmpBuffer: TBGRABitmap);
var
  i, Nb: Integer;
  procedure QTracerProfil(const QProfil: TProfilTopo);
  var
    NbP, p: Integer;
    MyPointProfil: TPoint3DfOrderedByP;
    PP: TPoint2DDepth;
    PC: TPoint;
  begin
    NbP := QProfil.GetNbElements();
    AfficherMessageErreur(Format('  Tracé du profil (%d points)', [NbP]));
    TmpBuffer.CanvasBGRA.Pen.Color := QProfil.ProfilColor;
    TmpBuffer.CanvasBGRA.Pen.Width := 2;

    MyPointProfil := QProfil.GetElement(0);
    PP := Get2DDepthCoordinates(MyPointProfil.X, MyPointProfil.Y, MyPointProfil.Z);
    PC := GetScreenCoordinates(PP.X, PP.Y, True);
    TmpBuffer.CanvasBGRA.MoveTo(PC);
    for p := 1 to NbP - 1 do
    begin
      MyPointProfil := QProfil.GetElement(p);
      PP := Get2DDepthCoordinates(MyPointProfil.X, MyPointProfil.Y, MyPointProfil.Z);
      PC := GetScreenCoordinates(PP.X, PP.Y, True);
      TmpBuffer.CanvasBGRA.LineTo(PC);
    end;

  end;
begin
  AfficherMessageErreur(Format('%s.TracerMaillage(): Maillage %sVALIDE', [classname, IIF(FMyMaillage.IsValidMaillage(), '', 'IN')]));
  if (not FMyMaillage.IsValidMaillage()) then Exit;
  Nb := FMyMaillage.GetNbProfilsTopo();
  if (Nb = 0) then Exit;
  for i := 0 to Nb - 1 do QTracerProfil(FMyMaillage.GetProfilTopo(i));
end;


end.

