unit CadreVue3DOpenGLExt;

{$INCLUDE CompilationParameters.inc}
{$DEFINE GLU_PERSPECTIVE}

// Date: 08/11/2013
//------------------------------------------------------------------------------
// Nécessite l'installation du composant LazarusOpenGL
// ouvrir le paquet $(LAZARUS)/components/opengl/lazopenglcontext.lpk
// et l'installer.
//------------------------------------------------------------------------------
// Statut: Opérationnel - Des détails à revoir
// Couleurs par mode de représentation: OK
// Paramétrage angles et couleurs: OK
// 01/06/2015 : Support des maillages de surface (OK avec les TIN, OK avec les grid)
// 25/04/2016 : Les maillages MNT deviennent un objet interne (aucune liaison avec ceux de la fenêtre principale de GHTopo)

interface

uses
  StructuresDonnees,
  Common,
  CalculMatriciel3x3,
  UnitEntitesExtended,
  UnitClasseMaillage,
  GL, glu,
  Dialogs,
  Classes, SysUtils, FileUtil, OpenGLContext, Forms, Controls, Graphics;
// couleurs OpenGL
type TGLColor = record
  R: GLFloat;
  G: GLFloat;
  B: GLFloat;
  A: GLFloat;
end;
//*)
type

  { TCdrVue3DOpenGLExt }

  TCdrVue3DOpenGLExt = class(TFrame)
    OpenGLControl1: TOpenGLControl;
    procedure OpenGLControl1Paint(Sender: TObject);
    procedure OpenGLControl1Resize(Sender: TObject);
  private
    { private declarations }
    // callback pour rafraichissement
    FProcRefreshControls: TProcRefreshControlsBy3DView;
    // filtres
    FFiltres   : string;
    FDoFiltrer : boolean;
    // éléments dessinés
    FElementsDrawn: TElementsDrawn;

    FBDDEntites: TBDDEntites;
    FMyMaillage: TMaillage;
    // encombrement
    FCoin1, FCoin2: TPoint3Df;
    Fcs1, Fcs2    : TPoint3Df; // valeurs ci-dessus mises à l'échelle
    // décalage
    FExcentr      : TPoint3Df;
    // modes de dessin
    FModeRepresentationGaleries: TModeRepresentationGaleries;
    // drapeaux
    FDoDraw        : boolean;              // dessin OK
    // listes d'affichage
    FglListCUBE          : TGLuint;       // cube
    FglListREFERENTIEL   : TGLuint;       // référentiel
    FglListPOLYGONALS    : TGLuint;       // polygonales de la cavité
    FglListRADIANT_SHOTS : TGLuint;       // visées radiantes
    FglListVOLUMES     : TGLuint;       // volumes de la cavité
    FglListMAILLAGE    : TGLuint;       // maillage

    // caractéristiques de lumière
    Flight_ambient      : array[0..3] of GLfloat; // lumière ambiante
    // couleurs
    FColorBackGrnd    : TGLColor;
    FColorCube        : TGLColor;
    FColorReferentiel : TGLColor;

    FColorZMini       : TGLColor;
    FColorZMaxi       : TGLColor;
    // variables géométriques en entrée
    //FZoom                : double;         // zoom XYZ
    FTheta               : Double;         // angle de rotation   (degré)
    FPhi                 : double;         // angle d'inclinaison (degré)
    FFov                 : double;         // angle d'ouverture de la caméra = 60° * Zoom
    FCoefMagnification   : double;         // facteur d'échelle Z
    function InitContextOpenGL: boolean;
    function ConstruireScene: boolean;
    procedure MakeCUBE();
    procedure MakePOLYGONALES(const M: double);
    procedure MakeRADIANT_SHOTS(const M: double);
    procedure MakeVOLUMES(const M: double);
    procedure MakeMAILLAGE(const M: double);
    procedure ReconstruitVue();


  public
    { public declarations }
    { public declarations }
    function  InitialiserVue3D(const QBDD: TBDDEntites;
                               const MyFiltre: string; const FP: TProcRefreshControlsBy3DView): boolean;
    procedure FinaliserVue3D;
    function  GetMaillage(): TMaillage;

    procedure SetParamsVue3D(const QM: TModeRepresentationGaleries;
                             const ED: TElementsDrawn;
                             const QTheta, QPhi, QZoom, QFactZ: double);
    procedure SetColorsElementsScene(const QM: TModeRepresentationGaleries; const QColorBackGrnd, QColorCube, QColorReferentiel: TColor);
    procedure SetColorsByDepth(const QM: TModeRepresentationGaleries; const QColorZMini, QColorZMaxi: TColor);
    procedure SetParamsAndRegenVue3D(const QTheta, QPhi, QZoom, QFactZ: double; const QFiltres: string; const DoFiltrer: boolean);

    function  GetTheta: double;
    function  GetPhi: double;
    function  GetZoom: double;
    function  GetFactZ: double;
    function  GetColorZMini: TColor;
    function  GetColorZMaxi: TColor;
    function  GetFiltres(): string;
    function  GetDoFiltrer(): boolean;
    function  ChargerMaillageMAI(const FichierMAI: string): boolean;
    function  ChargerMaillageSUR(const FichierSUR: string): boolean;
    procedure SetParamsMaillage(const TM: TMNTModeDessinMaillage; const Transparence: double);

  end;

implementation

{$R *.lfm}
const
  LOW_INDEX = 1;

{$IFDEF GLU_PERSPECTIVE}
   const MULTIPLICATEUR = 1/10000;// 1/10000
{$ELSE}
   const MULTIPLICATEUR = 1.00;// 1/10000
{$ENDIF}

 // couleur OpenGL
 function PascalToGLColor(const Coul: TColor): TGLColor;
 const
   m = 1/256;
 var
   c: TGLColor;
 begin
   c.R := Red(Coul)* m;
   c.G := Green(Coul)* m;
   c.B := Blue(Coul)* m;
   c.A := 1.00;
   Result := c;
 end;
 // Retourne sous forme de couleur Pascal la couleur OpenGL passée en argument
 function GLToPascalColor(const Coul: TGLColor): TColor;
 const
   m = 256;
 begin
   Result := RGBToColor(Round(Coul.R * m), Round(Coul.G * m), Round(Coul.B * m));
 end;

(*
// Retourne sous forme de couleur OpenGL la couleur passée en argument
function GetPasColorToGLColor(const Coul: TColor): TGLColor;
const
  m = 1/256;
var
  c: TGLColor;
begin
  c.R:=Red(Coul)* m;
  c.G:=Green((Coul)* m;
  c.B:=Blue(Coul)* m;
  c.A:=1.00;
  Result:=c;
end;
function GetGLColorPasColor(const C: TGLColor): TColor;
const
  m = 256;
begin
  Result := RGB(round(C.R * 3), round(C.G * m), round(C.B * m));
end;
//*)

{ TCdrVue3DOpenGLExt }

function TCdrVue3DOpenGLExt.InitialiserVue3D(const QBDD: TBDDEntites;
                                            const MyFiltre: string;
                                            const FP: TProcRefreshControlsBy3DView): boolean;
var
  QDevelViseesVisibles: double;
begin
  AfficherMessage(Format('%s.InitialiseVue3D()', [ClassName]));
  FFiltres    := MyFiltre;
  FDoFiltrer  := false;
  Result      := False;
  // pointeur sur la BDD
  FBDDEntites := QBDD;
  // pointeur sur le maillage
  FMyMaillage := TMaillage.Create(Application);
  FMyMaillage.Initialiser(FBDDEntites);

  // désarmement des drapeaux
  FDoDraw      := False;              // dessin OK
  FTheta       := 45;
  FPhi         := 32;
  FFov         := 60.00;

  // fonction de rappel
  FProcRefreshControls := FP;
  // mode de dessin par défaut = séances
  FModeRepresentationGaleries := rgSEANCES;
  // couleurs par défaut:
  FColorZMini       := PascalToGLColor(clAqua);
  FColorZMaxi       := PascalToGLColor(clRed);
  // chargement entités
  try
    // préparation tables temporaires
    with FBDDEntites do
    begin

      // filtres
      MetaFiltre(MyFiltre, QDevelViseesVisibles);
      // mini et maxi avec filtres
      SetMinMax(True);
      // dégradé de couleurs
      FBDDEntites.CalcCouleursByDepth(GLToPascalColor(FColorZMini), GLToPascalColor(FColorZMaxi));
    end;
    // cube enveloppe
    FCoin1 := FBDDEntites.GetCoinBasGauche;
    FCoin2 := FBDDEntites.GetCoinHautDroit;
    // démarrage du contexte OpenGL
    Result := InitContextOpenGL();

    Result := True;
  except
  end;
end;

procedure TCdrVue3DOpenGLExt.FinaliserVue3D;
begin
  try
    FMyMaillage.Finaliser;
  finally
    FMyMaillage.Free;
  end;
end;



procedure TCdrVue3DOpenGLExt.SetParamsAndRegenVue3D(const QTheta, QPhi, QZoom, QFactZ: double; const QFiltres: string; const DoFiltrer: boolean);
var
  QDevelViseesVisibles: double;
begin
  // filtres
  if (DoFiltrer) then
  begin
    FBDDEntites.MetaFiltre(QFiltres, QDevelViseesVisibles);
    FBDDEntites.SetMinMax(True);
  end;
  FTheta :=  QTheta;
  FPhi   :=  -(90 - QPhi);
  //FZoom  := QZoom;
  FFov   := 60 / QZoom;
  FCoefMagnification := QFactZ;
  ConstruireScene;
  ReconstruitVue;
end;

function TCdrVue3DOpenGLExt.GetTheta: double;
begin
  Result := FTheta;
end;

function TCdrVue3DOpenGLExt.GetPhi: double;
begin
  Result := FPhi;
end;

function TCdrVue3DOpenGLExt.GetZoom: double;
begin
  Result := FFov;
end;

function TCdrVue3DOpenGLExt.GetFactZ: double;
begin
  Result := FCoefMagnification;
end;

function TCdrVue3DOpenGLExt.GetColorZMini: TColor;
begin
  Result := GLToPascalColor(FColorZMini);
end;

function TCdrVue3DOpenGLExt.GetColorZMaxi: TColor;
begin
  Result := GLToPascalColor(FColorZMaxi);
end;

function TCdrVue3DOpenGLExt.GetFiltres: string;
begin

end;

function TCdrVue3DOpenGLExt.GetMaillage: TMaillage;
begin
  Result := FMyMaillage;
end;

function TCdrVue3DOpenGLExt.GetDoFiltrer: boolean;
begin

end;

function TCdrVue3DOpenGLExt.ChargerMaillageMAI(const FichierMAI: string): boolean;
begin
  Result := (FMyMaillage.LireFichierMaillage(FichierMAI) > 0);
  //
  FMyMaillage.SetModeDessinMaillage(M3D_BLEND);//M3D_WIRE_FRAME;
  FMyMaillage.SetModeDegrade(true);
  FMyMaillage.SetAlphaBlending(0.80);
  FMyMaillage.SetMaxCouleur(clMaroon);
  FMyMaillage.SetMinCouleur(clGreen);
  ConstruireScene;
  ReconstruitVue;
end;

function TCdrVue3DOpenGLExt.ChargerMaillageSUR(const FichierSUR: string): boolean;
begin
  Result := FMyMaillage.ChargerMaillageDepuisSUR(FichierSUR);
  //
  FMyMaillage.SetModeDessinMaillage(M3D_BLEND);//M3D_WIRE_FRAME;
  FMyMaillage.SetModeDegrade(true);
  FMyMaillage.SetAlphaBlending(0.80);
  FMyMaillage.SetMaxCouleur(clMaroon);
  FMyMaillage.SetMinCouleur(clGreen);
  ConstruireScene;
  ReconstruitVue;

end;

procedure TCdrVue3DOpenGLExt.SetColorsElementsScene(const QM: TModeRepresentationGaleries; const QColorBackGrnd, QColorCube, QColorReferentiel: TColor);
begin
  FModeRepresentationGaleries := QM;
  FColorBackGrnd    := PascalToGLColor(QColorBackGrnd);
  FColorCube        := PascalToGLColor(QColorCube);
  FColorReferentiel := PascalToGLColor(QColorReferentiel);
end;

procedure TCdrVue3DOpenGLExt.SetColorsByDepth(const QM: TModeRepresentationGaleries; const QColorZMini, QColorZMaxi: TColor);
begin
  AfficherMessage(Format('%s.SetColorsByDepth($%X, $%X)', [ClassName, QColorZMini, QColorZMaxi]));
  FModeRepresentationGaleries := QM;
  FColorZMini       := PascalToGLColor(QColorZMini);
  FColorZMaxi       := PascalToGLColor(QColorZMaxi);
  FBDDEntites.CalcCouleursByDepth(GLToPascalColor(FColorZMini), GLToPascalColor(FColorZMaxi));
end;

procedure TCdrVue3DOpenGLExt.SetParamsVue3D(const QM: TModeRepresentationGaleries;
                                            const ED: TElementsDrawn;
                                            const QTheta, QPhi, QZoom, QFactZ: double);
begin
  FModeRepresentationGaleries := QM;
  FElementsDrawn  := ED;
  FTheta :=  QTheta;
  FPhi   :=  -(90 - QPhi);
  //FZoom  := QZoom;
  FFov   := 60 / QZoom;
  FCoefMagnification := QFactZ;
  ReconstruitVue;
end;


procedure TCdrVue3DOpenGLExt.OpenGLControl1Paint(Sender: TObject);
begin
  ReconstruitVue;
end;

procedure TCdrVue3DOpenGLExt.OpenGLControl1Resize(Sender: TObject);
begin
  OpenGLControl1.Invalidate;
end;

//******************************************************************************
function TCdrVue3DOpenGLExt.InitContextOpenGL: boolean;
const
  B = 0.2;
begin
  AfficherMessage(Format('%s.InitContextOpenGL()',[ClassName]));
  FDoDraw := False;
  Result  := False;
  try
    if (OpenGLControl1.MakeCurrent) then
    begin
      // couleur
      FColorBackGrnd    := PascalToGLColor(clBlack);
      FColorCube        := PascalToGLColor(clRED);
      FColorReferentiel := PascalToGLColor(clBlue);
      // lumière ambiante
      Flight_ambient[0]:=B ;
      Flight_ambient[1]:=B ;
      Flight_ambient[2]:=B ;
      Flight_ambient[3]:=1.0;

      FFov  := 60.00;
      // construction de la scène
      FDoDraw := ConstruireScene();
      Result  := FDoDraw;
    end;
  except
  end;
end;
// construction de la scène
function TCdrVue3DOpenGLExt.ConstruireScene: boolean;
begin
  AfficherMessage(Format('%s.ConstruireScene', [ClassName]));
  Result := False;

  try
    // Le filtrage n'est plus du ressort de cette fonction
    //FBDDEntites.MetaFiltre(Filtres);
    // cube enveloppe
    AfficherMessage('-- >Cube enveloppe');
    FglListCUBE := glGenLists(1);
    glNewList(FglListCUBE, GL_COMPILE);
      MakeCUBE();
    glEndList();
    // polygonales
    AfficherMessage('-- >Polygonales');
    FglListPOLYGONALS:=glGenLists(1);
    glNewList(FglListPOLYGONALS, GL_COMPILE);
      MakePOLYGONALES(MULTIPLICATEUR);
    glEndList;
    // visées radiantes
    AfficherMessage('-- >Visees Radiantes');
    FglListRADIANT_SHOTS := glGenLists(1);
    glNewList(FglListRADIANT_SHOTS, GL_COMPILE);
      MakeRADIANT_SHOTS(MULTIPLICATEUR);
    glEndList();
    // volumes des conduits
    AfficherMessage('-- >Conduits');
    FglListVOLUMES := glGenLists(1);
    glNewList(FglListVOLUMES, GL_COMPILE);
      MakeVOLUMES(MULTIPLICATEUR);
    glEndList;
    // maillage
    AfficherMessage('-- >Maillage');
    FglListMAILLAGE := glGenLists(1);
    glNewList(FglListMAILLAGE, GL_COMPILE);
      MakeMAILLAGE(MULTIPLICATEUR);
    glEndList;

    AfficherMessage('Construction des listes OK');
    Result := True;
  except
  end;
end;


//------------------------------------------------------------------------------
// éléments de la scène
// construction des éléments de la vue
procedure TCdrVue3DOpenGLExt.MakeCUBE;
var
  i: integer;
  t: array[0..1] of GLFloat;
  dx, dy, dz: double;
begin
  {$IFDEF GLU_PERSPECTIVE}
    Fcs1.X:=FCoin1.X * MULTIPLICATEUR;
    Fcs1.Y:=FCoin1.Y * MULTIPLICATEUR;
    Fcs1.Z:=FCoin1.Z * MULTIPLICATEUR;
    Fcs2.X:=FCoin2.X * MULTIPLICATEUR;
    Fcs2.Y:=FCoin2.Y * MULTIPLICATEUR;
    Fcs2.Z:=FCoin2.Z * MULTIPLICATEUR;
  {$ELSE}
    dx := (FCoin2.X - FCoin1.X) * MULTIPLICATEUR / 2;
    dy := (FCoin2.Y - FCoin1.Y) * MULTIPLICATEUR / 2;
    dz := (FCoin2.Z - FCoin1.Z) * MULTIPLICATEUR / 2;

    Fcs1.X := - dx;      Fcs2.X:= dx;
    Fcs1.Y := - dy;      Fcs2.Y:= dy;
    Fcs1.Z := - dz;      Fcs2.Z:= dz;

    Fcs1.X:=FCoin1.X * MULTIPLICATEUR;
    Fcs1.Y:=FCoin1.Y * MULTIPLICATEUR;
    Fcs1.Z:=FCoin1.Z * MULTIPLICATEUR;
    Fcs2.X:=FCoin2.X * MULTIPLICATEUR;
    Fcs2.Y:=FCoin2.Y * MULTIPLICATEUR;
    Fcs2.Z:=FCoin2.Z * MULTIPLICATEUR;
  {$ENDIF}
  t[0]:=Fcs1.Z;  t[1]:=Fcs2.Z;
  for i:=0 to 1 do begin
    glBegin(GL_LINE_LOOP);
     glVertex3d(Fcs1.X, Fcs1.Y, t[i]);
     glVertex3d(Fcs2.X, Fcs1.Y, t[i]);
     glVertex3d(Fcs2.X, Fcs2.Y, t[i]);
     glVertex3d(Fcs1.X, Fcs2.Y, t[i]);
    glEnd();
  end;
  glBegin(GL_LINES);
    glVertex3d(Fcs1.X, Fcs1.Y, t[0]);
    glVertex3d(Fcs1.X, Fcs1.Y, t[1]);
    glVertex3d(Fcs2.X, Fcs1.Y, t[0]);
    glVertex3d(Fcs2.X, Fcs1.Y, t[1]);
    glVertex3d(Fcs2.X, Fcs2.Y, t[0]);
    glVertex3d(Fcs2.X, Fcs2.Y, t[1]);
    glVertex3d(Fcs1.X, Fcs2.Y, t[0]);
    glVertex3d(Fcs1.X, Fcs2.Y, t[1]);
  glEnd();
end;

procedure TCdrVue3DOpenGLExt.MakePOLYGONALES(const M: double);
var
  i, Nb : Integer;
  EWE   : TEntiteEtendue;
  c     : TGLColor;
begin
  Nb := FBDDEntites.GetNbEntitesVisees();
  AfficherMessage(Format('%s.MakePOLYGONALES(%d entities)', [ClassName, Nb]));
  glBegin(GL_LINES);
  for i:= LOW_INDEX to Nb - 1 do
   begin
     EWE := FBDDEntites.GetEntiteVisee(i);
     //AfficherMessage(Format('%f %f %f',[TableEntites[i].Une_Station_2_X, TableEntites[i].Une_Station_2_Y, TableEntites[i].Une_Station_2_Z]));
     if (IsViseetInCaveOrTunnel(EWE) and EWE.Drawn) then
     begin
       c := PascalToGLColor(FBDDEntites.GetColorViseeFromModeRepresentation(FModeRepresentationGaleries, EWE));
       glColor3d(c.R, c.G, c.B);
       glVertex3d(EWE.Une_Station_1_X * M,
                  EWE.Une_Station_1_Y * M,
                  EWE.Une_Station_1_Z * M);
       glVertex3d(EWE.Une_Station_2_X * M,
                  EWE.Une_Station_2_Y * M,
                  EWE.Une_Station_2_Z * M);

     end;
   end;
  glEnd;
end;
procedure TCdrVue3DOpenGLExt.MakeRADIANT_SHOTS(const M: double);
var
  i, Nb: Integer;
  EWE: TEntiteEtendue;
  QGrayScale: byte;
begin
  Nb := FBDDEntites.GetNbEntitesVisees();
  AfficherMessage(Format('%s.MakeRADIANT_SHOTS(%d entities)', [ClassName, Nb]));
  glBegin(GL_LINES);
  QGrayScale := 64;
  glColor3d(QGrayScale, QGrayScale, QGrayScale);
  for i:= LOW_INDEX to Nb - 1 do
  begin
    EWE := FBDDEntites.GetEntiteVisee(i);
    //AfficherMessage(Format('%f %f %f',[TableEntites[i].Une_Station_2_X, TableEntites[i].Une_Station_2_Y, TableEntites[i].Une_Station_2_Z]));
    if (EWE.Type_Entite = tgVISEE_RADIANTE) then
    begin
      //c := PascalToGLColor(FBDDEntites.GetColorViseeFromModeRepresentation(FModeRepresentationGaleries, EWE));
      glVertex3d(EWE.Une_Station_1_X * M,
                 EWE.Une_Station_1_Y * M,
                 EWE.Une_Station_1_Z * M);
      glVertex3d(EWE.Une_Station_2_X * M,
                 EWE.Une_Station_2_Y * M,
                 EWE.Une_Station_2_Z * M);
    end;
  end;
  glEnd();
end;

procedure TCdrVue3DOpenGLExt.MakeVOLUMES(const M: double);
const
  NB_FACETTES = 14;
var
  i: integer;
  Vertex  : array[1..NB_FACETTES] of TPoint3Df;
  Normales: array[1..NB_FACETTES] of TPoint3Df;
  function GetNormale(const dx, dy, dz: double):TPoint3Df;
  const EPSILON=1e-5;
  var
    l: double;
  begin
    l:=Hypot3D(dx+EPSILON,dy+EPSILON,dz+EPSILON);
    Result.X:=dx/l;
    Result.Y:=dy/l;
    Result.Z:=dz/l;
    //Result.T:=1.00;
  end;
  procedure PutVertex(const q: integer);
  begin
    glNormal3d(Normales[q].X, Normales[q].Y, Normales[q].Z);
    glVertex3d(Vertex[q].X, Vertex[q].Y, Vertex[q].Z);;
  end;
  procedure DrawTube(const EE: TEntiteEtendue);
  var
    c: TGLColor;
    p,q,r: integer;
    z1, z2: Double;
    v1, v2: TPoint3Df;
  begin
    // vertex
    (*
                V4 ---------------------------- V10
               /  \
              /     \
             /        \
            V5          \ V3                V11           V9
             !            !
             !            !
             !            !
             !            !
            V6           V2                 V12           V8
             \          /
              \       /
               \    /
                \ /
                V1                               V7

    Triangles 'pairs'

    V1 V7 V2
    V2 V8 V3
    V3 V9 V4
    V4 V10 V5
    V5 V11 V6
    V6 V12 V1

    Triangles impairs

    V2 V7 V8
    V3 V8 V9
    V4 V9 V10
    V5 V10 V11
    V6 V11 V12
    V1 V12 V7



    //*)
    // calcul des sommets
    with EE do
    begin
      if (Not Drawn) then Exit;
      if (Not IsViseetInCaveOrTunnel(EE)) then Exit;

      z1:=0.50 * (Z1PB+Une_Station_1_Z) * M;
      z2:=0.50 * (Z1PH+Une_Station_1_Z) * M;

      Vertex[1] := MakeTPoint3Df(Une_Station_1_X * M, Une_Station_1_Y * M, Z1PB * M);
      Vertex[2] := MakeTPoint3Df(X1PD * M, Y1PD * M, z1);
      Vertex[3] := MakeTPoint3Df(Vertex[2].X, Vertex[2].Y, z2);
      Vertex[4] := MakeTPoint3Df(Vertex[1].X, Vertex[1].Y, Z1PH * M);
      Vertex[5] := MakeTPoint3Df(X1PG * M, Y1PG * M, z2);
      Vertex[6] := MakeTPoint3Df(Vertex[5].X, Vertex[5].Y, z1);

      z1:=0.50 * (Z2PB + Une_Station_2_Z) * M;
      z2:=0.50 * (Z2PH + Une_Station_2_Z) * M;

      Vertex[7] := MakeTPoint3Df(Une_Station_2_X * M, Une_Station_2_Y * M, Z2PB * M);
      Vertex[8] := MakeTPoint3Df(X2PD * M, Y2PD * M, z1);
      Vertex[9] := MakeTPoint3Df(Vertex[8].X, Vertex[8].Y, z2);
      Vertex[10]:= MakeTPoint3Df(Vertex[7].X, Vertex[7].Y, Z2PH * M);
      Vertex[11]:= MakeTPoint3Df(X2PG * M, Y2PG * M, z2);
      Vertex[12]:= MakeTPoint3Df(Vertex[11].X, Vertex[11].Y, z1);
      // calcul des normales
      Normales[1]:= MakeTPoint3Df(0.0, 0.0, -1.0);

      Normales[2]:=GetNormale(X1PD - Une_Station_1_X,
                              Y1PD - Une_Station_1_Y,
                              Z1PB - Une_Station_1_Z);
      Normales[3]:=GetNormale(X1PD - Une_Station_1_X,
                              Y1PD - Une_Station_1_Y,
                              Z1PH - Une_Station_1_Z);
      Normales[4]:= MakeTPoint3Df(0.0, 0.0, 1.0);

      Normales[5]:=GetNormale(X1PG - Une_Station_1_X,
                              Y1PG - Une_Station_1_Y,
                              Z1PH - Une_Station_1_Z);
      Normales[6]:=GetNormale(X1PG - Une_Station_1_X,
                              Y1PG - Une_Station_1_Y,
                              Z1PB - Une_Station_1_Z);
      Normales[7] := MakeTPoint3Df(0.0, 0.0, -1.0);

      Normales[8]:=GetNormale(X2PD - Une_Station_2_X,
                              Y2PD - Une_Station_2_Y,
                              Z2PB - Une_Station_2_Z);
      Normales[9]:=GetNormale(X2PD - Une_Station_2_X,
                              Y2PD - Une_Station_2_Y,
                              Z2PH - Une_Station_2_Z);
      Normales[10] := MakeTPoint3Df(0.0, 0.0, 1.0);
      Normales[11]:=GetNormale(X2PG - Une_Station_2_X,
                               Y2PG - Une_Station_2_Y,
                               Z2PH - Une_Station_2_Z);
      Normales[12]:=GetNormale(X2PG - Une_Station_2_X,
                               Y2PG - Une_Station_2_Y,
                               Z2PB - Une_Station_2_Z);
      V1 := MakeTPoint3Df(Vertex[2].X - Vertex[6].X,
                          Vertex[2].Y - Vertex[6].Y,
                          Vertex[2].Z - Vertex[6].Z);
      V2 := MakeTPoint3Df(Vertex[5].X - Vertex[6].X,
                          Vertex[5].Y - Vertex[6].Y,
                          Vertex[5].Z - Vertex[6].Z);
      Normales[13] := ProduitVectoriel(V1,V2,True);
      V1 := MakeTPoint3Df(Vertex[8].X - Vertex[12].X,
                          Vertex[8].Y - Vertex[12].Y,
                          Vertex[8].Z - Vertex[12].Z);
      V2 := MakeTPoint3Df(Vertex[9].X - Vertex[12].X,
                          Vertex[9].Y - Vertex[12].Y,
                          Vertex[9].Z - Vertex[12].Z);
      Normales[14]:=ProduitVectoriel(V1,V2,True);
    end;
    //********

    //c := PascalToGLColor(EE.ColourByDepth);
    c := PascalToGLColor(FBDDEntites.GetColorViseeFromModeRepresentation(FModeRepresentationGaleries, EE));

    glMaterialfv(GL_FRONT, GL_AMBIENT, addr(c));

    glBegin(GL_TRIANGLES);
    // Triangles pairs
    for p := 1 to 6 do
    begin
      q := p + 6;
      //if (q > 12) then q := 7;
      r := p + 1;
      if (r > 6) then r := 1;
      PutVertex(p);
      PutVertex(q);
      PutVertex(r);

    end;
    // Triangles impairs
    for q := 7 to 12 do begin
      p := q - 5;
      if ( p > 6) then p := 1;
      r := p+6;
       PutVertex(p);
      PutVertex(q);
      PutVertex(r);
    end;
    glEnd;
    // abouts (OK)
    glBegin(GL_TRIANGLE_FAN);
     glNormal3d(Normales[13].X, Normales[13].Y, Normales[13].Z);
     //glVertex3d(Vertex[1].X, Vertex[1].Y, Vertex[1].Z);
     for p:=1 to 6 do glVertex3d(Vertex[p].X, Vertex[p].Y, Vertex[p].Z);
    glEnd;    //***** ok
     glBegin(GL_TRIANGLE_FAN);
     glNormal3d(Normales[14].X, Normales[14].Y, Normales[14].Z);
     //glVertex3d(Vertex[7].X, Vertex[7].Y, Vertex[7].Z);
     for p:=12 downto 7 do glVertex3d(Vertex[p].X, Vertex[p].Y, Vertex[p].Z);
    glEnd;
  end;
begin
 AfficherMessage(Format('%s.DrawGLConduits(%d entities)', [ClassName, FBDDEntites.GetNbEntitesVisees()]));
 for i:=LOW_INDEX to FBDDEntites.GetNbEntitesVisees() - 1 do
 begin
   DrawTube(FBDDEntites.GetEntiteVisee(i));
 end;
end;

procedure TCdrVue3DOpenGLExt.MakeMAILLAGE(const M: double);
begin
 AfficherMessage(Format('%s.MakeMaillage(%d triangles, %d vertex)', [ClassName, FMyMaillage.GetNbTriangles, FMyMaillage.GetNbVertex]));
 //ShowMessage(Format('%s.MakeMaillage(%d triangles, %d vertex)', [ClassName, FMyMaillage.GetNbTriangles, FMyMaillage.GetNbVertex]));
 FMyMaillage.ConstruireMaillages(M, 1.00);
end;

// affichage de la vue
procedure TCdrVue3DOpenGLExt.ReconstruitVue;
var
  ErrCode: TGLuint;
  QEchelle: Extended;
begin
  AfficherMessage(Format('%s.RebuildVue()',[ClassName]));
  if (Not FDoDraw) then Exit;
  glEnable(GL_DEPTH_TEST);
  glClearColor(FColorBackGrnd.R, FColorBackGrnd.G, FColorBackGrnd.B, FColorBackGrnd.A);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glClearDepth(1.0);
  glDepthFunc(GL_LESS);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_CULL_FACE);

  //-------------------

  {$IFDEF GLU_PERSPECTIVE}
    // calcul d'excentrement
    FExcentr.X := 0.50 * (Fcs2.X - Fcs1.X);
    FExcentr.X := Fcs1.X + FExcentr.X;
    FExcentr.Y := 0.50 * (Fcs2.Y - Fcs1.Y);
    FExcentr.Y := Fcs1.Y + FExcentr.Y;
    FExcentr.Z := 0.50 * (FCoefMagnification * (Fcs2.Z - Fcs1.Z));
    FExcentr.Z := FCoefMagnification * Fcs1.Z + FExcentr.Z;
    AfficherMessage(Format('-- >Enveloppe: %.0f, %.0f, %.0f -> %.0f, %.0f, %.0f', [
                    FCoin1.X, FCoin1.Y, FCoin1.Z,  FCoin2.X, FCoin2.Y, FCoin2.Z
                   ]));
    AfficherMessage(Format('-- >(scaled: ): %.0f, %.0f, %.0f -> %.0f, %.0f, %.0f', [
                    Fcs1.X, Fcs1.Y, Fcs1.Z,  Fcs2.X, Fcs2.Y, Fcs2.Z
                   ]));
    // mise en place projection
    glViewport(0,0,OpenGLControl1.Width, OpenGLControl1.Height);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity();
    //gluOrtho2D:=;
    gluPerspective(FFov, OpenGLControl1.Width / OpenGLControl1.Height, 0.01, 1000.0);    // Fonctionne bien
  {$ELSE}
    // calcul d'excentrement
    FExcentr.X := 0.50 * (c2.X - c1.X);
    FExcentr.X := c1.X + FExcentr.X;
    FExcentr.Y := 0.50 * (c2.Y - c1.Y);
    FExcentr.Y := c1.Y + FExcentr.Y;
    FExcentr.Z := 0.50 * (FMagnification*(c2.Z - c1.Z));
    FExcentr.Z := FMagnification*c1.Z + FExcentr.Z;
    glViewport(0,0,OpenGLControl1.Width, OpenGLControl1.Height);
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity();
    AfficherMessage(Format('-- > glFrustum: %.2f, %.2f, %.2f, %.2f, %.2f, %.2f',
                           [c1.X, c2.X, c1.Y, c2.Y, -c1.Z, -c2.Z]));
    //glFrustum (c1.X*2, c2.X * 2, c1.y * 2, c2.y * 2, -c2.z * 2, -c1.z * 2);
    glOrtho(c1.X*2, c2.X * 2, c1.y * 2, c2.y * 2, c1.z * 2, c2.z * 2);
  {$ENDIF}



  //FFov, RenderWindow.Width/RenderWindow.Height, 0.01,10000
  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity();
  // transformations
  {$IFDEF GLU_PERSPECTIVE}

    //------------- transformations d'ensemble
    glTranslated(0,0, -(Fcs2.Z - Fcs1.Z) * FCoefMagnification * 10); // FMagnification * 10
    glRotated(FPhi  , 1,0,0);
    glRotated(FTheta, 0,0,1);


    glTranslated(-FExcentr.X,
                 -FExcentr.Y,
                 -FExcentr.Z);
    QEchelle := 1.00;
    glScalef(QEchelle, QEchelle, QEchelle * FCoefMagnification);
    AfficherMessage(Format('Transformations pour FOV = %.2f, Theta = %.0f, Phi: %.0f, Scale = %.0f',[FFov, FTheta, FPhi, QEchelle]));

  {$ELSE}

    Diago := Hypot3D(c2.X - c1.X, c2.y - c1.y, c2.z - c1.z);
    Radius:= 1.5* Diago;
    AngTheta := FTheta * PI/180;
    AngPhi   := FPhi * PI/180;
    RP       := Radius * cos(AngPhi);
    // centre
    Centre.X := (c2.X + c1.X) / 2;
    Centre.Y := (c2.Y + c1.Y) / 2;
    Centre.Z := (c2.Z + c1.Z) / 2;
    // oeil

    PointVue.X    := Centre.X + RP * cos(AngTheta);
    PointVue.Y    := Centre.Y + RP * sin(AngTheta);
    PointVue.Z    := Centre.Z + Radius * sin(AngPhi);

    // Up
    AppareilPhotoUp.X := sclUpX.Position/100;
    AppareilPhotoUp.Y := sclUpY.Position/100;
    AppareilPhotoUp.Z := sclUpZ.Position/100;


    AfficherMessage(Format('%s: X = %.2f, Y: %.2f, Z = %.2f',['Point de vue', PointVue.X, PointVue.Y, PointVue.Z]));
    AfficherMessage(Format('%s: X = %.2f, Y: %.2f, Z = %.2f',['Centre', Centre.X, Centre.Y, Centre.Z]));
    AfficherMessage(Format('%s: X = %.2f, Y: %.2f, Z = %.2f',['Up', AppareilPhotoUp.X, AppareilPhotoUp.Y, AppareilPhotoUp.Z]));

    gluLookAt(PointVue.X, PointVue.Y, PointVue.Z,
              Centre.X, Centre.Y, Centre.Z,
              AppareilPhotoUp.X, AppareilPhotoUp.Y, AppareilPhotoUp.Z);
  {$ENDIF}

  // le dessin ici
  //************************************
  glDisable(GL_BLEND);
  glDisable(GL_COLOR_MATERIAL);

  // dessin d'objets volumiques
  glLightfv (GL_LIGHT0, GL_AMBIENT, @Flight_ambient); // lumière ambiante
  glEnable(GL_LIGHTING);   	                      // Active l'éclairage
  glEnable(GL_LIGHT0);
  // dessin des volumes cavité
  AfficherMessage('-- >Volumes');
  //glEnable(GL_COLOR_MATERIAL);
  if (edVolumes in FElementsDrawn) then glCallList(FglListVOLUMES);


  //--------------------------------
  // dessin du maillage

  AfficherMessage('-- >Maillage: ' + booltostr(FMyMaillage.IsValidMaillage, 'OK', 'KO'));
  if (FMyMaillage.IsValidMaillage and (edMaillages in FElementsDrawn)) then
  begin
    try
      AfficherMessage('---- Maillage en cours de dessin');
      case FMyMaillage.GetModeDessinMaillage of
        M3D_BLEND:       // transparent
        begin
           //glDepthMask(GL_FALSE);
           glEnable(GL_COLOR_MATERIAL);
           glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
           glEnable(GL_BLEND);
           glCallList(FglListMAILLAGE);

           //glDepthMask(GL_TRUE);
           glDisable(GL_BLEND);
           glDisable(GL_COLOR_MATERIAL);
        end;
        M3D_MESH:        // solide
        begin
          glEnable(GL_COLOR_MATERIAL);
          glCallList(FglListMAILLAGE);
          glDisable(GL_COLOR_MATERIAL);
        end;
        M3D_WIRE_FRAME:  // fil de fer
        begin
          glDisable(GL_LIGHTING);
          glCallList(FglListMAILLAGE);
        end;
      else
        ;
      end; //case
    except
    end;
  end; // if (FMyMaillage.IsValidMaillage) then
  AfficherMessage('-- >Maillage OK');
  //*)


  // dessin d'objets filaires
  glDisable(GL_LIGHTING);
  glDisable(GL_LIGHT0);
  // le cube enveloppe
  glColor3d(FColorCube.R,
            FColorCube.G,
            FColorCube.B);
  AfficherMessage('-- >Cube enveloppe');
  if (edBounds      in FElementsDrawn) then glCallList(FglListCUBE);

  // dessin des polygonales
  AfficherMessage('-- >Polygonales');
  if (edPolygonals  in FElementsDrawn) then glCallList(FglListPOLYGONALS);
  AfficherMessage('-- >Visees radiantes');
  if (edANTENNES    in FElementsDrawn) then glCallList(FglListRADIANT_SHOTS);


  // on envoie
  glFlush;
  ErrCode := glGetError;
  if (ErrCode <> 0) then AfficherMessage(Format('OpenGL error: %d - %s',[ErrCode, glGetString(ErrCode)])); //rsOPENGLERROR);

  // échange buffer
  OpenGLControl1.SwapBuffers;
  // actualisation des coontrôles
  if (Assigned(FProcRefreshControls)) then FProcRefreshControls();
end;
procedure TCdrVue3DOpenGLExt.SetParamsMaillage(const TM: TMNTModeDessinMaillage; const Transparence: double);
begin
  FMyMaillage.SetModeDessinMaillage(TM);
  FMyMaillage.SetAlphaBlending(Transparence);
  ConstruireScene;
  ReconstruitVue;
end;

end.

