unit CdrHierarchieObjets;
// Affichage du document dessin sous forme d'arbre
// 16/08/2015: Sélection, Edition, suppression d'objets

{$INCLUDE CompilationParameters.inc}

interface

uses
  GHCD_Types,
  {$IFDEF LANGUAGE_FRENCH}
    UnitMessages_fr,
  {$ENDIF}
  GeneralFunctions,
  CallDialogs,
  UnitDocDessin,
  Classes, SysUtils, FileUtil, Forms, Controls, ComCtrls, StdCtrls, ActnList;

type

  { TCadreHierarchieObjets }

  TCadreHierarchieObjets = class(TFrame)
    acDeleteobject: TAction;
    acEditObject: TAction;
    ActionList1: TActionList;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    ImageList1: TImageList;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    StaticText3: TStaticText;
    TreeView1: TTreeView;
    procedure acDeleteobjectExecute(Sender: TObject);
    procedure acEditObjectExecute(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FrameClick(Sender: TObject);
    procedure TreeView1Click(Sender: TObject);
    procedure TreeView1SelectionChanged(Sender: TObject);
  private
    { private declarations }
    // document de dessin
    FDocumentDessin   : TDocumentDessin;
    FProcHandleTTreeviewItem: TProcHandleTTreeviewItem;
    FCurrentTypeObject: TTypeObjet;
    FCurrentIdxObject : integer;
    procedure DeleteNode(Node: TTreeNode);
    function GetAbsolutePosition(const FD: TDocumentDessin;
                                 const QIdxBasepoint: TIDBaseStation;
                                 const QIDGroupe: TIDGroupeEntites;
                                 const QOffset: TPoint3Df): TPoint3Df;
    function GetLevelOfNode(const Node: TTreeNode): integer;
  public
    { public declarations }
    procedure SetDocDessin(const FD: TDocumentDessin);
    function  GetDocDessin(): TDocumentDessin;

    procedure ListerLesObjets;
    procedure ViderListe;

    procedure SetProcHandleItemSelected(const P: TProcHandleTTreeviewItem);
    function GetCurrentTypeObject: TTypeObjet;
    function GetCurrentIdxObject : integer;


  end;

implementation

{$R *.lfm}
// pas de fonction de recherche de profondeur dans TTreenode
function TCadreHierarchieObjets.GetLevelOfNode(const Node: TTreeNode): integer;
  procedure CalcDepth(const QN: TTreeNode; var depth: integer);
  begin
    if (QN.Parent <> nil) then
    begin
      CalcDepth(QN.Parent, depth);
      depth += 1;
    end;
  end;
begin
  Result := 0;
  CalcDepth(Node, Result);
end;

procedure TCadreHierarchieObjets.TreeView1Click(Sender: TObject);
begin

end;
function TCadreHierarchieObjets.GetAbsolutePosition(const FD: TDocumentDessin;
                                                    const QIdxBasepoint: TIDBaseStation;
                                                    const QIDGroupe: TIDGroupeEntites;
                                                    const QOffset: TPoint3Df): TPoint3Df;
var
  BP : TBaseStation;
  GRP: TGroupeEntites;
  DG : TPoint3Df;
begin
  FD.GetBasePointByIndex(QIdxBasepoint, BP);
  GRP := FD.GetGroupeByIDGroupe(QIDGroupe);
  DG  := MakeTPoint3Df(0.00, 0.00, 0.00);
  if (GRP.DecalageActif) then DG := MakeTPoint3Df(GRP.Decalage.X, GRP.Decalage.Y, GRP.Decalage.Z);

  Result := MakeTPoint3Df(BP.PosStation.X + DG.X + QOffset.X,
                          BP.PosStation.Y + DG.Y + QOffset.Y,
                          BP.PosStation.Z + DG.Z + QOffset.Z);
end;

procedure TCadreHierarchieObjets.acDeleteobjectExecute(Sender: TObject);
var
  EWE: String;
begin
  case FCurrentTypeObject of
    tobjSCRAP     : EWE := Format(rsMSG_DELETE_SCRAP     , [FCurrentIdxObject]);
    tobjCOURBE    : EWE := Format(rsMSG_DELETE_COURBE    , [FCurrentIdxObject]);
    tobjPOLYLIGNE : EWE := Format(rsMSG_DELETE_POLYLIGNE , [FCurrentIdxObject]);
    tobjPOLYGONE  : EWE := Format(rsMSG_DELETE_POLYGONE  , [FCurrentIdxObject]);
    tobjLIGNE     : EWE := Format(rsMSG_DELETE_LINE      , [FCurrentIdxObject]);
    tobjSYMBOLE   : EWE := Format(rsMSG_DELETE_SYMBOLE   , [FCurrentIdxObject]);
    tobjTEXTE     : EWE := Format(rsMSG_DELETE_TEXTE     , [FCurrentIdxObject]);
  else
    Exit;
  end;
  if (not QuestionOuiNon(EWE)) then Exit;
  case FCurrentTypeObject of
    tobjSCRAP     : FDocumentDessin.DeleteScrap(FCurrentIdxObject);
    tobjCOURBE    : FDocumentDessin.DeleteCourbe(FCurrentIdxObject);
    tobjPOLYLIGNE : FDocumentDessin.DeletePolyligne(FCurrentIdxObject);
    tobjPOLYGONE  : FDocumentDessin.DeletePolygone(FCurrentIdxObject);
    tobjLIGNE     : FDocumentDessin.DeleteSimpleLigne(FCurrentIdxObject);
    tobjSYMBOLE   : FDocumentDessin.DeleteSymbole(FCurrentIdxObject);
    tobjTEXTE     : FDocumentDessin.DeleteTexte(FCurrentIdxObject);
  else
    Exit;
  end;
  ListerLesObjets;
end;

procedure TCadreHierarchieObjets.acEditObjectExecute(Sender: TObject);
var
  MyScrap: TScrap;
  MySymbole: TSymbole;
  MyTexte: TTextObject;
  QAbsolutePosition: TPoint3Df;
begin
  case FCurrentTypeObject of
    tobjSCRAP:
      begin
        MyScrap := FDocumentDessin.GetScrap(FCurrentIdxObject);
        if (EditerScrap(FDocumentDessin, MyScrap)) then
        begin
          FDocumentDessin.PutScrap(FCurrentIdxObject, MyScrap);
          ListerLesObjets;
        end;
      end;
    tobjCOURBE:
      begin
        ;
      end;
    tobjPOLYLIGNE:
      begin
        ;
      end;
    tobjPOLYGONE:
      begin
        ;
      end;
    tobjLIGNE:
      begin

      end;
    tobjSYMBOLE:
      begin
        MySymbole := FDocumentDessin.GetSymbole(FCurrentIdxObject);
        QAbsolutePosition := GetAbsolutePosition(FDocumentDessin, MySymbole.IDBaseStation, MySymbole.IDGroupe, MySymbole.Offset);
        (*
        if (EditerSymboleObject(FDocumentDessin,
                                FCurrentIdxObject,
                                medMODIF,
                                MySymbole.IDGroupe,
                                MySymbole.TypeObject,
                                MySymbole,
                                QAbsolutePosition,
                                nil)) then
        begin
          FDocumentDessin.PutSymbole(FCurrentIdxObject, MySymbole);
          ListerLesObjets;
        end;
        //*)
      end;
    tobjTEXTE:
      begin
        MyTexte := FDocumentDessin.GetTexte(FCurrentIdxObject);
        QAbsolutePosition := GetAbsolutePosition(FDocumentDessin, MySymbole.IDBaseStation, MySymbole.IDGroupe, MySymbole.Offset);
        (*if (EditerTextObject(FDocumentDessin, FCurrentIdxObject, medMODIF,
                                MySymbole.IDGroupe, MySymbole.TypeObject,
                                MySymbole)) then
        begin
          FDocumentDessin.PutTexte(FCurrentIdxObject, MyTexte);
          ListerLesObjets;
        end;
        //*)
      end
  else
    Exit;
  end;
end;

procedure TCadreHierarchieObjets.Button3Click(Sender: TObject);
begin

end;

procedure TCadreHierarchieObjets.TreeView1SelectionChanged(Sender: TObject);
var
  EWE: TTreeNode;
  Level: Integer;
  procedure GetObject(const QNode: TTreeNode);
  var
    NP: TTreeNode;
    pTypeObject: ^TTypeObjet;
    pIdxObject: PInt64;
  begin
    NP := QNode.Parent;
    pTypeObject := NP.Data;
    FCurrentTypeObject := pTypeObject^;
    pIdxObject := QNode.Data;
    FCurrentIdxObject  := pIdxObject^;
  end;

begin
  FCurrentIdxObject  := -1;
  FCurrentTypeObject := tobjNONE;
  StaticText1.Caption := '';
  StaticText2.Caption := '';
  StaticText3.Caption := '';

  try
    EWE    := TreeView1.Selected;
    Level  := GetLevelOfNode(EWE);
    //NodeParent := EWE.Parent;
    StaticText1.Caption := Format('%d; L=%d; I=%d', [EWE.AbsoluteIndex, Level, EWE.Index]);
    case (Level) of
      0: ;
      1: ;
      2: ;
      3: GetObject(EWE);
    end;
    StaticText2.Caption := ChooseString(Ord(FCurrentTypeObject), ['Scrap', 'Courbes', 'Polylignes', 'Polygones', 'Lignes', 'Symboles', 'Textes', 'Aucun', 'Inconnu']);
    if (Assigned(FProcHandleTTreeviewItem)) then FProcHandleTTreeviewItem();

  except

  end;
end;

{ TCadreHierarchieObjets }
procedure TCadreHierarchieObjets.DeleteNode(Node: TTreeNode);
begin
  while Node.HasChildren do DeleteNode(Node.GetLastChild);
  if (Node.Data <> nil) then Dispose(Node.Data);
  TreeView1.Items.Delete(Node) ;
end;

procedure TCadreHierarchieObjets.FrameClick(Sender: TObject);
begin

end;


procedure TCadreHierarchieObjets.ViderListe;
var
  i, nb: Integer;
  MyNode: TTreeNode;
begin
  nb := TreeView1.Items.Count;
  //AfficherMessageErreur(Format('ViderListe: %d', [Nb]));

  if (nb = 0) then Exit;
  TreeView1.Items.Clear;

  (*
  for i := 0 to nb - 1 do
  begin
    MyNode := TreeView1.Items[i];
    DeleteNode(MyNode);
  end;
      end;
  //*)
  FCurrentTypeObject := tobjNONE;
  FCurrentIdxObject  := -1;
  //AfficherMessageErreur('ViderListe OK');
end;

procedure TCadreHierarchieObjets.SetDocDessin(const FD: TDocumentDessin);
begin
  FDocumentDessin := FD;
end;

function TCadreHierarchieObjets.GetDocDessin(): TDocumentDessin;
begin
  Result := FDocumentDessin;
end ;

procedure TCadreHierarchieObjets.ListerLesObjets;
var
  i, NbGrp: Integer;
  RootNode: TTreeNode;
  MyGroupe: TGroupeEntites;
  grpNode: TTreeNode;
  procedure ListerGroupe(const ND: TTreeNode; const GP: TGroupeEntites);
  var
    NbTotalObj: integer;
    s, NbObj: Integer;
    MyScrap: TScrap;
    NdObjs: TTreeNode;
    MyCourbe: TCourbe;
    MyPolyligne: TPolyLigne;
    MyPolygone: TPolygone;
    MySimpleLigne: TSimpleLigne;
    MySymbole: TSymbole;
    MyTexte: TTextObject;
    procedure AddNodeSectionTypeObject(const QNd: TTreeNode; const QT: TTypeObjet; const DescTypeObjet: string; const QNbObjets: integer);
    var
      pTypeObjet: ^TTypeObjet;
      WU: String;
      QQNd: TTreeNode;
    begin
      New(pTypeObjet);
      pTypeObjet^ := QT;
      WU := Format('%s: %d objets', [DescTypeObjet, QNbObjets]);
      TreeView1.Items.AddChildObject(QNd, WU, pTypeObjet);
    end;
    procedure AddNodeObject(const QNd: TTreeNode; const QIdx: integer; const QS: string);
    var
      pIdxObject: PInt64;
    begin
      New(pIdxObject);
      pIdxObject^ := QIdx;
      TreeView1.Items.AddChildObject(QNd, Format('[%d] %s', [pIdxObject^, QS]), pIdxObject);
    end;
  begin
    NbObj := FDocumentDessin.GetNbScrapsPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjSCRAP, 'Scraps', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbScraps();
      NdObjs := ND.Items[0];
      for s := 0 to NbTotalObj - 1 do
      begin
        MyScrap := FDocumentDessin.GetScrap(s);
        if (MyScrap.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Scrap%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbCourbesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjCOURBE, 'Courbes', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbCourbes();
      NdObjs := ND.Items[1];
      for s := 0 to NbTotalObj - 1 do
      begin
        MyCourbe := FDocumentDessin.GetCourbe(s);
        if (MyCourbe.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Courbe%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbPolylignesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjPOLYLIGNE, 'Polylignes', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbPolylignes();
      NdObjs := ND.Items[2];
      for s := 0 to NbTotalObj - 1 do
      begin
        MyPolyligne := FDocumentDessin.GetPolyligne(s);
        if (MyPolyligne.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Polyligne%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbPolygonesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjPOLYGONE, 'Polygones', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbPolygones();
      NdObjs := ND.Items[3];
      for s := 0 to NbTotalObj - 1 do
      begin
        MyPolygone := FDocumentDessin.GetPolygone(s);
        if (MyPolygone.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Polygone%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbSimplesLignesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjLIGNE, 'SimplesLignes', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbSimpleLignes();
      NdObjs := ND.Items[4];
      for s := 0 to NbTotalObj - 1 do
      begin
        MySimpleLigne := FDocumentDessin.GetSimpleLigne(s);
        if (MySimpleLigne.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('SimpleLigne%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbSymbolesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjSYMBOLE, 'Symboles', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbSymboles();
      NdObjs := ND.Items[5];
      for s := 0 to NbTotalObj - 1 do
      begin
        MySymbole := FDocumentDessin.GetSymbole(s);
        if (MySymbole.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Symbole%d', [s]));
      end;
    end;
    NbObj := FDocumentDessin.GetNbTextesPourUnGroupe(GP.IDGroupeEntites);
    AddNodeSectionTypeObject(ND, tobjTEXTE, 'Textes', NbObj);
    if (NbObj > 0) then
    begin
      NbTotalObj := FDocumentDessin.GetNbTextes();
      NdObjs := ND.Items[6];
      for s := 0 to NbTotalObj - 1 do
      begin
        MyTexte := FDocumentDessin.GetTexte(s);
        if (MyTexte.IDGroupe = GP.IDGroupeEntites) then AddNodeObject(NdObjs, s, Format('Texte%d: %s', [s, MyTexte.Text]));
      end;
    end;
  end;
begin
  AfficherMessage('ListerLesObjets');
  ViderListe;
  AfficherMessage('-- 001');
  TreeView1.Items.Add(nil, FDocumentDessin.GetDocumentName);
  AfficherMessage('-- 002');
  RootNode := TreeView1.Items[0];
  AfficherMessage('-- 003');
  NbGrp := FDocumentDessin.GetNbGroupes();
  if (NbGrp = 0) then Exit;
  for i := 0 to NbGrp - 1 do
  begin
    MyGroupe := FDocumentDessin.GetGroupeByIdx(i);
    TreeView1.Items.AddChild(RootNode, MyGroupe.NomGroupe);
    grpNode := RootNode.Items[i];
    ListerGroupe(grpNode, MyGroupe);
  end;
end;

procedure TCadreHierarchieObjets.SetProcHandleItemSelected(const P: TProcHandleTTreeviewItem);
begin
  FProcHandleTTreeviewItem := P;
end;

function TCadreHierarchieObjets.GetCurrentTypeObject: TTypeObjet;
begin
  Result := FCurrentTypeObject;
end ;

function TCadreHierarchieObjets.GetCurrentIdxObject: integer;
begin
  Result := FCurrentIdxObject;
end ;

end.

