unit CentreImpressionExt;
// Centre d'impression nouvelle version
// Date: 08/11/2013
// Statut: Opérationnel. Des détails à compléter et à revoir
// Interfaçage par pointeur sur TBDDEntites: OK
// Prévisualisation: OK
// Impression: OK - des détails à revoir.
// 04/08/2014: Nouveau dessin des entrées
// 09/08/2017: Réactivation du module à la demande de S. Clément
// 09/08/2017: Suppression du type TPageProperties
//             et du membre FPrinter: Printer est déjà une variable déclarée dans Printers.pas


{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  CallDialogsStdVersion, unitUtilsComposants,
  //CadreFiltres,
  Common,
  math,
  types,
  UnitEntitesExtended,
  Printers, OSPrinters, // OSPrinters est INDISPENSABLE ici !!
  Classes, SysUtils, FileUtil, curredit, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, ComCtrls, Buttons;
// types de données locaux
type

  { TfrmPrintingCenterExt }

  TfrmPrintingCenterExt = class(TForm)
    BitBtn1: TBitBtn;
    btnApply: TButton;
    btnColorQuadrilles: TColorButton;
    btnColorTextCotation: TColorButton;
    btnDegradeStart: TColorButton;
    btnColorTextIDStations: TColorButton;
    btnDegradeStop: TColorButton;
    btnHelpFiltres: TButton;
    btnPreview: TButton;
    btnSelectPrinter: TButton;
    btnStartImpression: TButton;
    btnApplyParams2D: TButton;
    chkAltitudes: TCheckBox;
    chkAnnotations: TCheckBox;
    chkAntennes: TCheckBox;
    chkCotes: TCheckBox;
    chkEntrances: TCheckBox;
    chkIDStations: TCheckBox;
    chkNoeuds: TCheckBox;
    chkParois: TCheckBox;
    chkPolygonales: TCheckBox;
    chkRegle: TCheckBox;
    chkRemplissage: TCheckBox;
    chkSections: TCheckBox;
    chkStations: TCheckBox;
    cmbEchelle: TComboBox;
    cmbRepresentation: TComboBox;
    cmbTypeQuadrillage: TComboBox;
    editCotFontHeightMM: TCurrencyEdit;
    editLargeurTraitPolygonale: TCurrencyEdit;
    editQdrSpacing: TCurrencyEdit;
    editFiltres: TEdit;
    editTailleRegle: TCurrencyEdit;
    editIDStsFontHeightMM: TCurrencyEdit;
    grbPrinterNames: TGroupBox;
    GroupBox1: TGroupBox;
    chkGrpElementsDessin: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    lbCoteMaxi: TLabel;
    lbCoteMini: TLabel;
    lbEchelle: TLabel;
    lbMouseCoordinates: TStaticText;
    lbNbPages: TLabel;
    lbOrientation: TLabel;
    lbPageFormat: TLabel;
    lbPrintCurrent: TLabel;
    lbPrinterName: TLabel;
    lbQdrSpacing: TLabel;
    lbRepresentationMode: TLabel;
    PaintBoxVue: TPaintBox;
    Panel1: TPanel;
    pnlPrintSettings: TPanel;
    pnlCadre: TPanel;
    pnlProgressPrinting: TPanel;
    progbarPrinting: TProgressBar;
    procedure btnPreviewClick(Sender: TObject);
    procedure btnStartImpressionClick(Sender: TObject);
    procedure btnSelectPrinterClick(Sender: TObject);
    procedure btnApplyParams2DClick(Sender: TObject);
    procedure chkGrpElementsDessinClick(Sender: TObject);
    procedure cmbEchelleChange(Sender: TObject);
    procedure cmbTypeQuadrillageChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure lbPageFormatClick(Sender: TObject);
    procedure PaintBoxVueDblClick(Sender: TObject);
    procedure PaintBoxVueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxVuePaint(Sender: TObject);
  private
    { private declarations }
    FVue2DParams: TVue2DParams;
    // BDD entités
    FBDDEntites: TBDDEntites;
    FBddDemarreeDepuisFichier: boolean;
    // nombre de pages sur X et Y
    // tableau des pages dessinées
    FNbPagesX         : integer;
    FNbPagesY         : integer;
    FTableauPagesDrawn: array of array of boolean;
    FDoDrawPages      : boolean;
    // modes de représentation
    //FModeRepresentationGaleries: TModeRepresentationGaleries;
    // variables internes
    FRappScrReal : double;
    FInvRappScrReal: double;
    // coordonnées internes de la souris dans la vue
    FPP: TPoint;
    // limites du dessin
    FRXMini      : double;
    FRXMaxi      : double;
    FRYMini      : double;
    FRYMaxi      : double;
    // variables diverses

    // échelle du dessin
    FEchelle: double;
    // quadrillage
    FQdrCrossSize  : double;


    // Règle
    FRegleDisplayed: boolean;
    FRegleSize     : double;

    // dessiner les annotations si elles sont chargées
    // Fontes pour les stations
    //FAttributsTexteStations: TTexteAttributs;
    //FAttributsTexteCotes   : TTexteAttributs;
    // paramétrage imprimante
    procedure CalcIndexPageIncludingPt(const Pt: TPoint2Df; var IdxX, IdxY: integer);
    procedure ImprimerLaTopo();
    procedure PrintAPage(const Echelle: double; const L, C: integer);
    procedure SetMyCurrentPrinter();
    // éléments à représenter
    function  GetVue2DParamsFromDialog(): TVue2DParams;
    procedure SetVue2DParamsToDialogs(const FE: TVue2DParams);
    // conversion de coordonnées
    function GetCoordsMonde(const PP: TPoint): TPoint2Df;
    function GetCoordsPlan(const PM: TPoint2Df): TPoint;

    // recadrage du dessin
    procedure Recadrer();
    procedure CalcNbPagesAndEmpty();
    procedure DrawApercu();
  public
    { public declarations }
    function InitialiseByPointer(const QBDD: TBDDEntites;
                                 const QVue2DParams: TVue2DParams;
                                 const QFiltres: string): boolean;
    // initialisation depuis un fichier (FBDDEntites est local)
    function InitialiseByFichier(const QFileName: string;
                                 const QVue2DParams: TVue2DParams;
                                 const QFiltres: string; const PositionPointZero: TPoint3Df): boolean;
    // libération du contexte
    procedure Finaliser();
  end;

var
  frmPrintingCenterExt: TfrmPrintingCenterExt;

implementation

{$R *.lfm}
const ONE_INCH_IN_MM: double = 25.40;
// fonctions locales hors objet
function Millimetres2PixelsXf(const Millims: double): integer;
begin
  Result:=trunc(100 * Printer.XDPI * Millims / ONE_INCH_IN_MM) div 100;
end;
function Millimetres2PixelsYf(const Millims: double): integer;
begin
  Result:=trunc(100 * Printer.YDPI * Millims / ONE_INCH_IN_MM) div 100;
end;
function Pixels2MillimetresX(const Pixls: integer): Double;
begin
  Result := ONE_INCH_IN_MM * Pixls / Printer.XDPI;
end;
function Pixels2MillimetresY(const Pixls: integer): Double;
begin
  Result := ONE_INCH_IN_MM * Pixls / Printer.YDPI;
end;


{ TfrmPrintingCenterExt }
// définir imprimante courante: paramètres, etc ...
procedure TfrmPrintingCenterExt.SetMyCurrentPrinter;
begin
  lbPrinterName.Caption := Printer.PrinterName;
  lbOrientation.Caption := ChooseString(Ord(Printer.Orientation),
                                          ['PORTRAIT', 'LANDSCAPE', 'REVERSE LANDSCAPE', 'REVERSEPORTRAIT']);
  lbPageFormat.Caption  := Format('%f mm x %f mm', [Pixels2MillimetresX(Printer.PageWidth),
                                                    Pixels2MillimetresY(Printer.PageHeight)]);

  // reconstruire le plan
  Recadrer();
  // calculer le nombre de pages
  CalcNbPagesAndEmpty();
  // redessiner
  DrawApercu();
end;




// conversion de coordonnées
function TfrmPrintingCenterExt.GetCoordsPlan (const PM: TPoint2Df): TPoint;
begin
  // rotation et mise à l'échelle
  (* Formule: (F = FRappScrReal)
  | Xp |   | +F    0    -F * FRXMini |   | PM.X |
  |    |   |                         |   |      |
  | Yp | = |  0   -F    +F * FRYMaxi | * | PM.Y |
  |    |   |                         |   |      |
  |  1 |   |  0    0          1      |   |   1  |
  //*)
  Result.X:=Round((PM.X - FRXMini) * FRappScrReal);
  Result.Y:=Round((FRYMaxi-PM.Y) * FRappScrReal);
end;
function TfrmPrintingCenterExt.GetCoordsMonde(const PP: TPoint): TPoint2Df;
begin
  (* Formule: (F = FRappScrReal)
  | Xm |   | +1/F    0     FRXMini   |   | PP.X |
  |    |   |                         |   |      |
  | Ym | = |  0   -1/F     FRYMaxi   | * | PP.Y |
  |    |   |                         |   |      |
  |  1 |   |  0    0          1      |   |   1  |
  //*)
  Result.X:=  FInvRappScrReal * PP.X + FRXMini;
  Result.Y:= -FInvRappScrReal * PP.Y + FRYMaxi;
end;

// cadrage du dessin
// pas besoin de fonction de zoom
// (sinon, on aurait utilisé un TCdrVisualisateur2D)
procedure TfrmPrintingCenterExt.Recadrer;
var
  Marge: double;
  qdx, qdy: Double;
  C1, C2: TPoint3Df;

begin
  AfficherMessage(Format('%s.Recadrer',[ClassName]));

  with FBDDEntites do begin
    C1 := GetCoinBasGauche;
    C2 := GetCoinHautDroit;

    Marge:=0.02 * Hypot2D(C2.X - C1.X,
                          C2.Y - C1.Y);

    FRXMini:=C1.X - Marge;
    FRXMaxi:=C2.X + Marge;
    FRYMini:=C1.Y - Marge;
    FRYMaxi:=C2.Y + Marge;
  end;
  qdx := FRXMaxi - FRXMini;
  qdy := FRYMaxi - FRYMini;
  //pnlCadre.top:=0;
  //pnlCadre.left:=0;
  FRappScrReal := IIF((qdx > qdy), pnlCadre.Width / qdx, pnlCadre.Height / qdy);
  FInvRappScrReal:=1/FRappScrReal;
  AfficherMessage(Format('%s.Recadrer OK',[ClassName]));
end;

// récupération des paramètres depuis les contrôles
function TfrmPrintingCenterExt.GetVue2DParamsFromDialog(): TVue2DParams;
begin
  Result.ongElementsDrawn := [];
  if (chkEntrances.Checked)   then Result.ongElementsDrawn := Result.ongElementsDrawn + [edENTRANCES]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edENTRANCES];
  if (chkStations.Checked)    then Result.ongElementsDrawn := Result.ongElementsDrawn + [edStations]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edStations];
  if (chkAltitudes.Checked)   then Result.ongElementsDrawn := Result.ongElementsDrawn + [edAltitudes]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edAltitudes];
  if (chkCotes.Checked)       then Result.ongElementsDrawn := Result.ongElementsDrawn + [edCotes]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edCotes];
  if (chkParois.Checked)      then Result.ongElementsDrawn := Result.ongElementsDrawn + [edWalls]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edWalls];
  if (chkIDStations.Checked)  then Result.ongElementsDrawn := Result.ongElementsDrawn + [edIDStations]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edIDStations];
  if (chkSections.Checked)    then Result.ongElementsDrawn := Result.ongElementsDrawn + [edCrossSections]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edCrossSections];
  if (chkRemplissage.Checked) then Result.ongElementsDrawn := Result.ongElementsDrawn + [edFillGalerie]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edFillGalerie];
  if (chkPolygonales.Checked) then Result.ongElementsDrawn := Result.ongElementsDrawn + [edPolygonals]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edPolygonals];
  if (chkAntennes.Checked)    then Result.ongElementsDrawn := Result.ongElementsDrawn + [edANTENNES]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edANTENNES];
  if (chkAnnotations.Checked) then Result.ongElementsDrawn := Result.ongElementsDrawn + [edANNOTATIONS]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edANNOTATIONS];
  if (chkNoeuds.Checked)      then Result.ongElementsDrawn := Result.ongElementsDrawn + [edJONCTIONS]
                              else Result.ongElementsDrawn := Result.ongElementsDrawn - [edJONCTIONS];
  // récupérer les valeurs de quadrillage depuis les box
  Result.ongQdrSpc        := editQdrSpacing.Value;
  Result.ongQdrColor      := btnColorQuadrilles.ButtonColor;
  Result.ongQdrType       := TQdrType(cmbTypeQuadrillage.ItemIndex);


  FQdrCrossSize  := Result.ongQdrSpc / 5.00 ; // provisoire
  FEchelle       := 1 / StrToIntDef(cmbEchelle.Text, 1000);
  FRegleSize     := editTailleRegle.Value;
  Result.ongDegradeStart := btnDegradeStart.ButtonColor;
  Result.ongDegradeStop  := btnDegradeStop.ButtonColor;
  // largeur de trait
  Result.ongViseesLargeur := Millimetres2PixelsYf(editLargeurTraitPolygonale.Value);
  // mode de representation
  Result.ongModeRepresentation := TModeRepresentationGaleries(cmbRepresentation.ItemIndex);
end;


// options de dessin
procedure TfrmPrintingCenterExt.SetVue2DParamsToDialogs(const FE: TVue2DParams);
begin
  FVue2DParams := FE;
  chkPolygonales.Checked := (edPolygonals     in FE.ongElementsDrawn);
  chkStations.Checked    := (edStations       in FE.ongElementsDrawn);
  chkAltitudes.Checked   := (edAltitudes      in FE.ongElementsDrawn);
  chkCotes.Checked       := (edCotes          in FE.ongElementsDrawn);
  chkIDStations.Checked  := (edIDStations     in FE.ongElementsDrawn);
  chkParois.Checked      := (edWalls          in FE.ongElementsDrawn);
  chkRemplissage.Checked := (edFillGalerie    in FE.ongElementsDrawn);
  chkSections.Checked    := (edCrossSections  in FE.ongElementsDrawn);
  chkEntrances.Checked   := (edENTRANCES      in FE.ongElementsDrawn);
  chkAntennes.Checked    := (edANTENNES       in FE.ongElementsDrawn);
  chkAnnotations.Checked := (edANNOTATIONS    in FE.ongElementsDrawn);
  chkNoeuds.Checked      := (edJONCTIONS      in FE.ongElementsDrawn);

  btnColorQuadrilles.ButtonColor := FE.ongQdrColor;
  cmbTypeQuadrillage.ItemIndex   := Ord(FE.ongQdrType);
  editQdrSpacing.Value           := FE.ongQdrSpc;

  cmbRepresentation.ItemIndex := ord(FE.ongModeRepresentation);

end;






function TfrmPrintingCenterExt.InitialiseByPointer(const QBDD: TBDDEntites;
                                                   const QVue2DParams: TVue2DParams;
                                                   const QFiltres: string): boolean;
var
  i  : Integer;
  WU : TElementsDrawn;
  QDevelViseesVisibles: double;
begin
  AfficherMessage(Format('%s.InitialiseByPointer', [ClassName]));
  Result := False;
  // init de quelques variables
  SetVue2DParamsToDialogs(QVue2DParams);
  FBddDemarreeDepuisFichier   := false;


  // affecter pointeur sur la BDD
  FBDDEntites := QBDD;
  try
    FBDDEntites.MetaFiltre(QFiltres, QDevelViseesVisibles);
    // Initialisation de l'imprimante: Nécessite le paquet Printers4Lazarus,
    // qui est à ajouter à la liste des paquets requis par GHTopo
    // Ouvrir l'inspecteur de projet
    // Cliquer sur [ + ] en tete de la fenetre
    // Une fenetre s'ouvre
    // Onglet __/ Nouvelle Condition \__
    // Dérouler la première combobox
    // Sélectionner 'Printer4Lazarus'
    // Les unités Printers et OSPrinters sont INDISPENSABLES !!
    // Printer est une variable globale initialisée par le paquet.
    // Ne pas utiliser  TPrinter.Create;
    AfficherMessage('Fixation variables locales');
    // fixation de variables locales
    //FModeRepresentationGaleries := rgSEANCES;
    //WU := [edQuadrilles, edPolygonals, edWalls, edFillGalerie];

    // quadrillage
    //FQdrEspc       := 100.00;
    //FQdrEspc       := ProposerEquidistanceDef(FBDDEntites.GetCoinBasGauche, FBDDEntites.GetCoinHautDroit, 100.00);
    FRegleSize     := 2 * FVue2DParams.ongQdrSpc;
    FQdrCrossSize  := FVue2DParams.ongQdrSpc / 5;
    FEchelle       := 1 / 1000.00;

    editTailleRegle.Value   := FRegleSize;

    AfficherMessage('Setting printer');
    // définition de l'imprimante
    SetMyCurrentPrinter;
    // redessin
    AfficherMessage('Redessin preview');
    Recadrer;
    DrawApercu;
    //------------------------------
    AfficherMessage('init OK');
    Result := True;
  except
  end;
end;

procedure TfrmPrintingCenterExt.lbPageFormatClick(Sender: TObject);
begin

end;

function TfrmPrintingCenterExt.InitialiseByFichier(const QFileName: string;
                                                   const QVue2DParams: TVue2DParams;
                                                   const QFiltres: string; const PositionPointZero: TPoint3Df): boolean;
begin
  Result := False;
  FBddDemarreeDepuisFichier := True;
  FBDDEntites := TBDDEntites.Create;
  try
    FBDDEntites.Initialiser(PositionPointZero);
    Result := FBDDEntites.LoadFromFile(QFileName);
  except
  end;
end;

procedure TfrmPrintingCenterExt.Finaliser;
begin
  if (FBddDemarreeDepuisFichier) then
  begin
    try
      FBDDEntites.Finaliser();
    finally
      FBDDEntites.Free;
    end;
  end;
end;
// calcul des pages vides
// calculer le nombre de pages et celles qui sont vides
procedure TfrmPrintingCenterExt.CalcNbPagesAndEmpty;
var
  i,j, v: integer;
  E1    : TEntiteEtendue;
  L1, H1: double;
  QPageWidthInMM, QPageHeightInMM, QInvEchelle: Double;

  R666  : TRect2Df;
  RVS   : TRect2Df;
  Grayed: boolean;
begin
  QPageWidthInMM  := Pixels2MillimetresX(Printer.PageWidth);
  QPageHeightInMM := Pixels2MillimetresY(Printer.PageHeight);

  AfficherMessage('Calcul du nombre de pages');
  // calculer le nombre de pages
  FNbPagesX := 1 + Trunc(1000 * (FRXMaxi - FRXMini) * FEchelle / QPageWidthInMM);
  FNbPagesY := 1 + Trunc(1000 * (FRYMaxi - FRYMini) * FEchelle / QPageHeightInMM);
  lbNbPages.Caption :=Format('%d x %d pages',[FNbPagesX, FNbPagesY]);
  SetLength(FTableauPagesDrawn, 0, 0);
  SetLength(FTableauPagesDrawn, FNbPagesX, FNbPagesY);
  QInvEchelle := 1 / (1000 * FEchelle);
  L1 := QPageWidthInMM  * QInvEchelle ;
  H1 := QPageHeightInMM * QInvEchelle;
  for i:=0 to FNbPagesX-1 do
    for j:=0 to FNbPagesY-1 do
    begin
      R666.X1 := FRXMini + i * L1;
      R666.Y1 := FRYMini + j * H1;

      R666.X2 := FRXMini + (1+i) * L1;
      R666.Y2 := FRYMini + (1+j) * H1;
      // supprimer les pages vides
      Grayed:=false;
      for v:=1 to FBDDEntites.GetNbEntitesVisees() - 1 do
      begin
        E1 := FBDDEntites.GetEntiteVisee(v);
        if (E1.Type_Entite = tgEntrance) then Continue;
        RVS.X1 := Min(E1.Une_Station_1_X, E1.Une_Station_2_X);
        RVS.Y1 := Min(E1.Une_Station_1_Y, E1.Une_Station_2_Y);

        RVS.X2 := Max(E1.Une_Station_1_X, E1.Une_Station_2_X);
        RVS.Y2 := Max(E1.Une_Station_1_Y, E1.Une_Station_2_Y);
        Grayed := Grayed or IntersectRectangles(R666, RVS);
      end;
      FTableauPagesDrawn[i,j]:=Grayed;
      AfficherMessage(Format('--> Page %d.%d: %s',[i, j, IIF((FTableauPagesDrawn[i,j]), 'Affichée', 'Masquée')]));
    end;
    FDoDrawPages:=True;
end;

procedure TfrmPrintingCenterExt.chkGrpElementsDessinClick(Sender: TObject);
begin

end;

//******************************************************************************
// dessin de l'aperçu
procedure TfrmPrintingCenterExt.DrawApercu;
var
  TmpBuffer      : TBitmap;
  R              : TRect;
  procedure QResetPenBrush(const C: TColor);
  begin
    with TmpBuffer.Canvas do begin
      Pen.Color   := C;
      Brush.Color := C;
    end;
  end;
  procedure DrawStations();
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawStations');
    QResetPenBrush(clRed);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE) then Continue;
      PM := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
      P1   := GetCoordsPlan(PM);
      TmpBuffer.Canvas.EllipseC(P1.X, P1.Y, 2, 2);
    end;
  end;
  procedure DrawBoundingBox();
  var
    P1, P2: TPoint;
    PM    : TPoint2Df;
    C1, C2: TPoint3Df;
  begin
    with FBDDEntites do
    begin
      // TODO: Pourquoi métafiltrer ?
      // TODO: Factoriser
      //MetaFiltre(CdrMetaFiltre1.GetFiltre);
      C1 := GetCoinBasGauche;
      C2 := GetCoinHautDroit;
      PM := MakeTPoint2Df(C1.X, C1.Y);
      P1 := GetCoordsPlan(PM);
      PM := MakeTPoint2Df(C2.X, C2.Y);
      P2:=GetCoordsPlan(PM);
      with TmpBuffer.Canvas do
      begin
        Pen.Width:=0;
        Pen.Color:=clSilver;
        Brush.Style:=bsClear;
        Rectangle(P1.X, P1.Y, P2.X, P2.Y);
      end;
    end;
  end;
  // affichage des pages (avec mention de pages vides)
  procedure DrawPages();
  var
    i,j   : integer;
    P1, P2: TPoint;
    PM    : TPoint2Df;
    L1, H1, QInvEchelle, QPageWidthInMM, QPageHeightInMM: Double;
    R666  : TRect2Df;
    //E1    : TEntite;
    //Grayed: boolean;
  begin
    if not FDoDrawPages then Exit;
    with TmpBuffer.Canvas do
    begin
      // todo
      Pen.Width:=0;
      Pen.Color:=clBlue;
      Brush.Style:=bsClear;

      Font.Height := 4;
      Font.Color:=clRed;
      QInvEchelle := 1 / (1000*FEchelle);
      QPageWidthInMM  := Pixels2MillimetresX(Printer.PageWidth);
      QPageHeightInMM := Pixels2MillimetresY(Printer.PageHeight);

      L1 := QPageWidthInMM   * QInvEchelle;
      H1 := QPageHeightInMM  * QInvEchelle;
      for i:=0 to FNbPagesX-1 do
        for j:=0 to FNbPagesY-1 do
        begin
          PM := MakeTPoint2Df(FRXMini + i * L1, FRYMini + j * H1);
          R666.X1 := PM.X;
          R666.Y1 := PM.Y;

          P1   := GetCoordsPlan(PM);
          PM := MakeTPoint2Df(FRXMini + (1+i) * L1, FRYMini + (1+j) * H1);
          R666.X2 := PM.X;
          R666.Y2 := PM.Y;

          P2   := GetCoordsPlan(PM);
          Brush.Style := bsSolid;      // toujours spécifier styles de brosses
          if (FTableauPagesDrawn[i,j]) then
            Brush.Color:=clWhite
          else
            Brush.Color:=clGray;

          Rectangle(P1.X, P1.Y, P2.X, P2.Y);
          //TextOut(p1.X + 2, p1.Y-(Font.Height shl 2), format('%d.%d',[i,j]));
          TextOut(p1.X + 2, p1.Y-(Font.Height shl 2),
                  format('%d',[1 + FNbPagesY*i+j]));
        end;
    end;
  end;
  procedure DrawEntrances();
  const R666 = 3;
  var
    i     : integer;
    E     : TEntrance;
    P2    : TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawEntrances');
    QResetPenBrush(clFuchsia);
    for i:=0 to FBDDEntites.GetNbEntrees - 1 do
    begin
      E := FBDDEntites.GetEntree(i);
      with TmpBuffer.Canvas do
      begin
        PM := MakeTPoint2Df(E.eXEntree, E.eYEntree);
        P2:=GetCoordsPlan(PM);
        EllipseC(P2.X, P2.Y, R666, R666);
      end;
    end; // for
  end;
  procedure DrawQuadrilles();
  var
    P1, P2: TPoint;
    C1, C2: TPoint2Df;
    Coin1, Coin2: TPoint3Df;
    t: integer;
    A,B, B0, qQuadrillageSpc: double;
  begin
    TmpBuffer.Canvas.Pen.Color := FVue2DParams.ongQdrColor;
    TmpBuffer.Canvas.Pen.Width := 0;
    Coin1 := FBDDEntites.GetCoinBasGauche;
    Coin2 := FBDDEntites.GetCoinHautDroit;
    qQuadrillageSpc := EnsureSetQuadrillageSpacing(FVue2DParams.ongQdrSpc);
    AfficherMessage(Format('Quadrillage: Spc: %.0f - X = (%.0f, %.0f) - Y = (%.0f, %.0f',
                   [qQuadrillageSpc, Coin1.X, Coin2.X, Coin1.Y, Coin2.Y]));
    case (FVue2DParams.ongQdrType) of
      qtNONE: ;
      qtGRID:
      begin
        t := trunc(Coin1.X / qQuadrillageSpc);
        A := qQuadrillageSpc * t;
        while (A < Coin2.X) do
        begin
          // AfficherMessage(Format('%.2f  %.2f',[A, QdrEspc]));
          C1 := MakeTPoint2Df(A, Coin1.Y);
          C2 := MakeTPoint2Df(A, Coin2.Y);
          P1 := GetCoordsPlan(C1);
          P2 := GetCoordsPlan(C2);

          TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
          A := A + qQuadrillageSpc;
        end;
        t := trunc(Coin1.Y / qQuadrillageSpc);
        A := qQuadrillageSpc * t;
        while (A < Coin2.Y) do
        begin
          C1 := MakeTPoint2Df(Coin1.X, A);
          C2 := MakeTPoint2Df(Coin2.X, A);
          P1 := GetCoordsPlan(C1);
          P2 := GetCoordsPlan(C2);
          TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
          A:= A + qQuadrillageSpc;
        end;
      end;
      qtCROSS:
      begin
        t := trunc(Coin1.X / qQuadrillageSpc);
        A := qQuadrillageSpc * t;
        t := trunc(Coin1.Y / qQuadrillageSpc);
        B := qQuadrillageSpc * t;
        B0:= B;
        while (A<Coin2.X) do
        begin
          B:=B0;
          while (B<Coin2.Y) do
          begin
            C1 := MakeTPoint2Df(A - FQdrCrossSize, B);
            C2 := MakeTPoint2Df(A + FQdrCrossSize, B);
            P1 := GetCoordsPlan(C1);
            P2 := GetCoordsPlan(C2);
            TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
            C1 := MakeTPoint2Df(A, B - FQdrCrossSize);
            C2 := MakeTPoint2Df(A, B + FQdrCrossSize);
            P1 := GetCoordsPlan(C1);
            P2 := GetCoordsPlan(C2);
            TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
            B := B + qQuadrillageSpc;
          end;
          A:=A + qQuadrillageSpc;
        end;
      end;
      qtPOINTS: ;
    end; // case QdrType
  end;


  //procedure DrawGaleries(const Filled: boolean);
  procedure DrawGaleries();
  var
    i: integer;
    E: TEntiteEtendue;
    P: array[0..3] of TPoint;
    PM    : TPoint2Df;
    q1, q2: boolean;
  begin
    q1 := (edFillGalerie in FVue2DParams.ongElementsDrawn);
    q2 := (edWalls       in FVue2DParams.ongElementsDrawn);

    AfficherMessage(Format(' --> DrawGaleries avec: %s, %s', [IIF(q1,'FILLS, ',''), IIF(q2,'WALLS','')]));
    // propriétés de pinceau et de brosses par défaut
    TmpBuffer.Canvas.Brush.Style := bsSolid;
    TmpBuffer.Canvas.Pen.Style   := psSolid;
    TmpBuffer.Canvas.Brush.Color := clSilver;
    TmpBuffer.Canvas.Pen.Color   := clBlack;
    TmpBuffer.Canvas.Pen.Width   := 0;

    for i:=0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE) then Continue;
      if (Not E.Drawn) then Continue;  // prise en compte du MétaFiltre
      PM := MakeTPoint2Df(E.X1PG, E.Y1PG);
      P[0]:=GetCoordsPlan(PM);
      PM := MakeTPoint2Df(E.X1PD, E.Y1PD);
      P[1]:=GetCoordsPlan(PM);
      PM := MakeTPoint2Df(E.X2PD, E.Y2PD);
      P[2]:=GetCoordsPlan(PM);
      PM := MakeTPoint2Df(E.X2PG, E.Y2PG);
      P[3]:=GetCoordsPlan(PM);
      TmpBuffer.Canvas.Pen.Width := 0; // Millimetres2PixelsXf(0.25);

      if (q1) then
      begin // remplissages
        TmpBuffer.Canvas.Brush.Color := FBDDEntites.GetColorViseeFromModeRepresentation(FVue2DParams.ongModeRepresentation, E);
        TmpBuffer.Canvas.Pen.Color   := Brush.Color;
        TmpBuffer.Canvas.Polygon(P);                   // DONE: Polygon ne remplit pas avec la couleur - Corrigé par Brush.Style := bsSolid;
      end;
      //Continue;
      if (q2) then
      begin // parois
        TmpBuffer.Canvas.Pen.Color := clBlack;
        TmpBuffer.Canvas.MoveTo(P[0]);  TmpBuffer.Canvas.LineTo(P[3]);
        TmpBuffer.Canvas.MoveTo(P[1]);  TmpBuffer.Canvas.LineTo(P[2]);
      end;
    end;
  end;
  procedure DrawPolygonals();
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawPolygonals');
    TmpBuffer.Canvas.Pen.Width := 2; // suffisant ici. FVue2DParams.ongViseesLargeur;//Millimetres2PixelsXf(0.25);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE) then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      PM := MakeTPoint2Df(E.Une_Station_1_X, E.Une_Station_1_Y);
      P1:=GetCoordsPlan(PM);
      PM := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
      P2:=GetCoordsPlan(PM);
      TmpBuffer.Canvas.Pen.Color := FBDDEntites.GetColorViseeFromModeRepresentation(FVue2DParams.ongModeRepresentation, E);
      TmpBuffer.Canvas.Line(P1, P2);
    end;
  end;
   procedure DrawViseesRadiantes();
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawViseesRadiantes');
    TmpBuffer.Canvas.Pen.Width := 0; // suffisant ici Millimetres2PixelsXf(0.01);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgVISEE_RADIANTE) then
      begin;
        PM := MakeTPoint2Df(E.Une_Station_1_X, E.Une_Station_1_Y);
        P1:=GetCoordsPlan(PM);
        PM := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
        P2:=GetCoordsPlan(PM);
        TmpBuffer.Canvas.Pen.Color := clGray;
        TmpBuffer.Canvas.Line(P1, P2);
      end;
    end;
  end;
  procedure DrawSections();
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawSections');
    TmpBuffer.Canvas.Pen.Width := 0; // Millimetres2PixelsXf(0.05);
    TmpBuffer.Canvas.Pen.Color := clGray;
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
     E := FBDDEntites.GetEntiteVisee(i);
     if (E.Type_Entite = tgENTRANCE) then Continue;
     PM := MakeTPoint2Df(E.X2PD, E.Y2PD);
     P1:=GetCoordsPlan(PM);
     PM := MakeTPoint2Df(E.X2PG, E.Y2PG);
     P2:=GetCoordsPlan(PM);
     TmpBuffer.Canvas.Line(P1, P2);
    end;
  end;
  procedure DrawIDStations();
  var
    i: integer;
    E: TEntiteEtendue;
    P1    : TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage(' --> DrawIDStations');
    with TmpBuffer.Canvas do begin
      Brush.Color := clWhite;
      Font.Color  := clBlue;
      Font.Name   := DEFAULT_FONT_NAME;
      Font.Height := 5;
    end;
    for i:=0 to FBDDEntites.GetNbEntitesVisees() -1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE) then Continue;
      PM := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
      P1 := GetCoordsPlan(PM);
      TmpBuffer.Canvas.TextOut(P1.X+2, P1.Y+2, Trim(E.oIDLitteral));
    end;
  end;
  procedure DrawRegle(const TailleRegle: double);
  const
    NB_CARREAUX_X = 8;
  var
    i : Integer;
    Mg: double;
    QXo, QYo, QY1: double;
    LargeurCarreau: Extended;
    HauteurCarreau: Extended;
    procedure QDrawRect(const X, Y, L, H: double; const LC, FC: TColor);
    var
      QP1, QP2: TPoint;
      QPM: TPoint2Df;
    begin
      QPM := MakeTPoint2Df(X, Y);
      QP1 := GetCoordsPlan(QPM);
      QPM := MakeTPoint2Df(X + L, Y + H);
      QP2 := GetCoordsPlan(QPM);
      with TmpBuffer.Canvas do
      begin
        Pen.Width    := 0;
        Pen.Color    := LC;
        Brush.Color  := FC;
        Rectangle(QP1.x, QP1.y, QP2.x, QP2.y);
      end;
    end;
    procedure QDrawTextLegende(const X, Y, L, H: double);
    var
      HauteurTexte: integer;
      QP1, QP2: TPoint;
      QPM     : TPoint2Df;
      WU : string;
      EWE: TSize;
    begin
      with TmpBuffer.Canvas do
      begin
        Pen.Width  := 0;
        Brush.Color:= clWhite;
        Font.Style := [fsBold];
        Font.Color := clBlue;
        WU := Format('%.0f m',[L]);
        // zéro de gauche
        QPM := MakeTPoint2Df(X, Y);
        QP1 := GetCoordsPlan(QPM);
        QPM := MakeTPoint2Df(X, Y + H);
        QP2   := GetCoordsPlan(QPM);
        HauteurTexte := QP2.Y - QP1.Y;
        Font.Height  := HauteurTexte;
        EWE := TextExtent('0');
        //AfficherMessage(Format('----> %d, %d',[EWE.cx, EWE.cy]));
        TextOut(QP1.X, QP1.Y - EWE.cy, '0');
        QPM := MakeTPoint2Df(X + L, Y);
        QP1   := GetCoordsPlan(QPM);
        EWE := TextExtent(WU);
        QP1.X := QP1.X - EWE.cx;
        TextOut(QP1.X, QP1.Y - EWE.Cy, WU);
      end;
    end;
  begin
    Mg := (FRXMaxi - FRXMini) / 50; // marge
    QXo := FRXMini + Mg;
    QYo := FRYMini + Mg;
    LargeurCarreau := TailleRegle / NB_CARREAUX_X;
    HauteurCarreau := TailleRegle / 25;
    AfficherMessage(Format(' --> DrawRegle: L = %.0f m at (%.0f, %.0f)', [TailleRegle, FRXMini + Mg, FRYMini + Mg]));
    // cadre périmétrique
    QDrawRect(QXo, QYo, TailleRegle, 2 * HauteurCarreau, clBlack, clWhite);
    // carreaux alternés
    for i := 0 to NB_CARREAUX_X - 1 do
    begin
      if (Odd(i)) then QY1 := QYo + HauteurCarreau else QY1 := QYo;
      QDrawRect(QXo + (i * LargeurCarreau), QY1, LargeurCarreau, HauteurCarreau, clBlack, clGray);
    end;
    // légende
    QDrawTextLegende(QXo, QYo + 2 * HauteurCarreau + Mg/4, TailleRegle, HauteurCarreau);
  end;
  (*
  procedure DrawAnnotations;
  var
    A: TAnnotation;
    i: integer;
    PM: TPoint2Df;
    PP: TPoint;
    // positionnement du texte
    PosTXT: TPoint;
    // encombrement du texte
    ExTxt : TSize;

    s: string;
    // pour positionnement relatif à un point topo
    E: TEntite;

  begin
    AfficherMessage(' --> DrawAnnotations');
    if not (FTableEntites.LesAnnotationsSontOK) then Exit;
    try
      with FTableEntites.MyTableAnnotations do begin
        if NbAnnotations=0 then Exit;
        with TmpBuffer.Canvas do begin
          for i:=0 to FTableEntites.MyTableAnnotations.NbAnnotations-1 do begin
            A:=FTableEntites.MyTableAnnotations.GetAnnotation(i);
            if Not(A.Displayed) then Continue;
            case A.ModePosition of
              0: begin // positionnement absolu
                   PM.X:=A.X;
                   PM.Y:=A.Y;
                 end;
              1: begin // positionnement relatif à une station
                   E := FTableEntites.GetEntiteFromSerSt(A.BaseRefSer, A.BaseRefSt);
                   PM.X := E.Une_Station_2_X + A.DX;
                   PM.Y := E.Une_Station_2_Y + A.DY;
                   A.Z  := E.Une_Station_2_Z + A.DZ;

                 end;
            end;
            PP:=GetCoordsPlan(PM);

            Font.Name:=IIF(A.FontName=NAME_FOR_DEFAULT_FONT, 'Arial', A.FontName);
            Font.Size:=A.FontSize;
            Font.Color:=A.FontColor;
            Brush.Color:=clWhite;
            //Brush.Color:=FBackGround;
            if A.FontBold then Font.Style:=Font.Style+[fsBold]
                          else Font.Style:=Font.Style-[fsBold];
            if A.FontItalic then Font.Style:=Font.Style+[fsItalic]
                            else Font.Style:=Font.Style-[fsItalic];
            if A.FontUnderline then Font.Style:=Font.Style+[fsUnderline]
                               else Font.Style:=Font.Style-[fsUnderline];

            S:=LLANFAIR_PG(A.Caption, A.MaxLength, A.Z,
                           A.BaseRefSer, A.BaseRefSt);
            // positionnement du texte

            //PP.X := 0;
            //PP.Y := 0;

            ExTxt :=  TextExtent(S);
            case A.PosPtDeBase of
              0,1: begin  // défaut = en bas à gauche
                     PosTXT.X := PP.X;
                     PosTXT.Y := PP.Y - ExTxt.cy;
                   end;
              2  : begin
                     PosTXT.X := PP.X - ExTxt.cx div 2;
                     PosTXT.Y := PP.Y - ExTxt.cy;
                   end;
              3  : begin
                     PosTXT.X := PP.X - ExTxt.cx;
                     PosTXT.Y := PP.Y - ExTxt.cy;
                   end;
              4  : begin
                     PosTXT.X := PP.X;
                     PosTXT.Y := PP.Y - ExTxt.cy div 2;
                   end;
              5  : begin
                     PosTXT.X := PP.X - ExTxt.cx div 2;
                     PosTXT.Y := PP.Y - ExTxt.cy div 2;
                   end;
              6  : begin
                     PosTXT.X := PP.X - ExTxt.cx;
                     PosTXT.Y := PP.Y - ExTxt.cy div 2;
                   end;
              7  : begin
                     PosTXT.X := PP.X;
                     PosTXT.Y := PP.Y;
                   end;
              8  : begin
                     PosTXT.X := PP.X - ExTxt.cx div 2;
                     PosTXT.Y := PP.Y;
                   end;
              9  : begin
                     PosTXT.X := PP.X - ExTxt.cx;
                     PosTXT.Y := PP.Y;
                   end;
            else
              begin
                PosTXT.X := PP.X;
                PosTXT.Y := PP.Y - ExTxt.cy;
              end;
            end;
            // calcul de BoundingBox
            A.BoundingBox.Left    := PosTXT.X;
            A.BoundingBox.Top     := PosTXT.Y;
            A.BoundingBox.Right   := PosTXT.x + Extxt.cx;
            A.BoundingBox.Bottom  := PosTXT.Y + Extxt.cy;
            // et sauvegarde Bounding Box
            FTableEntites.MyTableAnnotations.PutAnnotation(A, i);

            TextOut(PosTXT.X, PosTXT.Y, S);
            // débogage: Tracé du point de base
            Rectangle(PP.X, PP.Y,
                      PP.X + 2, PP.Y + 2);
          end; // for i
        end; // with TmpBuffer.Canvas do begin
      end; // with FTableEntites.MyTableAnnotations do begin
    except
    end;
  end;
  //*)
begin
  AfficherMessage(Format('%s.DrawApercu',[ClassName]));
  FVue2DParams := GetVue2DParamsFromDialog();
  try
    TmpBuffer := TBitmap.Create;
    with TmpBuffer do begin
      Height:= PaintBoxVue.Height;
      Width := PaintBoxVue.Width;
      //affichermessage(format('%d %d',[PaintBoxVue.Height,PaintBoxVue.Width]));
      R.Left:=PaintBoxVue.Left;
      R.Top :=PaintBoxVue.Top;
      R.Bottom:=PaintBoxVue.Top  + PaintBoxVue.Height;
      R.Right :=PaintBoxVue.Left + PaintBoxVue.Width;
      Canvas.Brush.Color:=clWhite;
      Canvas.FillRect(R);
      // dessin des limites
      DrawBoundingBox;
      // dessin des pages
      DrawPages();
      // dessin des éléments
      DrawQuadrilles();
      if (edPolygonals    in FVue2DParams.ongElementsDrawn) then DrawPolygonals();
      //if edFillGalerie  in FVue2DParams.ongElementsDrawn then
      DrawGaleries();
      if (edEntrances     in FVue2DParams.ongElementsDrawn) then DrawEntrances();
      if (edAnnotations   in FVue2DParams.ongElementsDrawn) then ; //DrawAnnotations;
      if (edCrossSections in FVue2DParams.ongElementsDrawn) then DrawSections();
      if (edStations      in FVue2DParams.ongElementsDrawn) then DrawStations();
      if (edIDStations    in FVue2DParams.ongElementsDrawn) then DrawIDStations();
      if (edANTENNES      in FVue2DParams.ongElementsDrawn) then DrawViseesRadiantes();
      //*)
      // dessin de la règle
      if (FRegleDisplayed) then DrawRegle(FRegleSize);
    end;
    PaintBoxVue.Canvas.CopyRect(R, TmpBuffer.Canvas, R);
  finally
    TmpBuffer.Free;
  end;
end;
// retourne les index de pages en fonction des coordonnées réelles
procedure TfrmPrintingCenterExt.CalcIndexPageIncludingPt(const Pt: TPoint2Df; var IdxX, IdxY: integer);
var
  i,j   : integer;
  L1, H1, QInvEchelle, QPageWidthInMM, QPageHeightInMM: Double;
  R666  : TRect2Df;
begin
  IdxX:=-1;
  IdxY:=-1;
  AfficherMessage(Format('%s.CalcIndexPageIncludingPt(%.2f, %.2f)',
                         [ClassName, Pt.X, Pt.Y]));
  QInvEchelle := 1 / (1000*FEchelle);
  QPageWidthInMM  := Pixels2MillimetresX(Printer.PageWidth);
  QPageHeightInMM := Pixels2MillimetresY(Printer.PageHeight);

  L1 := QPageWidthInMM  * QInvEchelle;
  H1 := QPageHeightInMM * QInvEchelle;
  for i:=0 to FNbPagesX-1 do
  begin
    for j:=0 to FNbPagesY-1 do
    begin
      R666.X1 := FRXMini + i * L1;
      R666.Y1 := FRYMini + j * H1;
      R666.X2 := FRXMini + (1+i) * L1;
      R666.Y2 := FRYMini + (1+j) * H1;
      if IsInRange(Pt.X, R666.X1, R666.X2) and
         IsInRange(Pt.Y, R666.Y1, R666.Y2) then
      begin
        IdxX:=i; IdxY:=j;
        AfficherMessage(Format('Page %d, %d sélectionnée',[i,j]));
        Exit;
      end;
    end;  // for i,j
  end;
end;

//******************************************************************************
// imprimer la topo
// Statut: OK
procedure TfrmPrintingCenterExt.ImprimerLaTopo();
var
  L,C,N,Q: integer;
begin
  pnlPrintSettings.Visible     := False;
  pnlProgressPrinting.Visible  := True;


  lbPrintCurrent.Caption := GetResourceString(rsPRN_START_PRINTING);
  FVue2DParams := GetVue2DParamsFromDialog();

  Printer.BeginDoc;
  AfficherMessage('>> 002');
  N:=0;
  progbarPrinting.Min := 0;
  Q := FNbPagesX * FNbPagesY;
  progbarPrinting.Max := Q;
  try
    for C:=0 to FNbPagesX-1 do
    begin
      for L:=0 to FNbPagesY-1 do
      begin
        Inc(N);
        progbarPrinting.Position := N;
        if (Not FTableauPagesDrawn[C,L]) then Continue;
        try
          AfficherMessage(Format('>> %d-%d - 003',[c,l]));
          PrintAPage(FEchelle, L, C);
          Printer.NewPage;
          Application.ProcessMessages;
          lbPrintCurrent.Caption:= Format('Printing %d/%d', [N,Q]);
        except
          AfficherMessage(Format('>> Printing error in page %d %d',[C,L]));
        end;
      end;
    end;
    Printer.EndDoc;
    AfficherMessage(GetResourceString(rsPRN_PRINTING_DONE));
  except
  end;
  pnlProgressPrinting.Visible  := False;
  pnlPrintSettings.Visible     := True;
end;
//******************************************************************************
// imprimer une page
// Statut: Opérationnel mais qques détails à revoir
// TODO: Pb de lignes superflues dans les entrées
// TODO: Optimiser ce code
procedure TfrmPrintingCenterExt.PrintAPage(const Echelle: double; const L,C: integer);

const
  RD=8;
var
  WU: String;
  function Coord2Draft(const Xf, Yf: double): TPoint;
  var
    QXf, QYf: Double;
  begin
    // transformation en millimètres et mise à l'échelle
    QXf:=((-FRXMini+Xf) * 1000.00) * Echelle;
    QYf:=((-FRYMini+Yf) * 1000.00) * Echelle;

    Result.x := -Printer.PageWidth * C + Millimetres2PixelsXf(QXf);
    Result.y := Printer.PageHeight * L +
                Printer.PageHeight - Millimetres2PixelsYf(QYf);
  end;

  //procedure DrawGaleries(const Filled: boolean);
  procedure DrawGaleries();
  var
    i: integer;
    E: TEntiteEtendue;
    P: array[0..3] of TPoint;
    q1, q2: boolean;
  begin
    AfficherMessage('--> DrawGaleries()');
    Printer.Canvas.Pen.Width  := 0;
    q1 := (edFillGalerie in FVue2DParams.ongElementsDrawn);
    q2 := (edWalls       in FVue2DParams.ongElementsDrawn);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE)          then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE)    then Continue;

      if (Not E.Drawn) then Continue;    // prise en compte du MétaFiltre
      P[0] := Coord2Draft(E.X1PG, E.Y1PG);
      P[1] := Coord2Draft(E.X1PD,E.Y1PD);
      P[2] := Coord2Draft(E.X2PD,E.Y2PD);
      P[3] := Coord2Draft(E.X2PG,E.Y2PG);
      if (q1) then
      begin
        Printer.Canvas.Brush.Color := FBDDEntites.GetColorViseeFromModeRepresentation(FVue2DParams.ongModeRepresentation, E);
        Printer.Canvas.Pen.Color   := Printer.Canvas.Brush.Color;
        Printer.Canvas.Polygon(P);
      end;
      if (q2) then
      begin // parois
        Printer.Canvas.Pen.Color:=clBlack;
        Printer.Canvas.Line(P[0], P[3]);
        Printer.Canvas.Line(P[1], P[2]);
      end;
    end;
  end;
  procedure DrawPolygonals(const BlackLines: boolean);
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
  begin
    AfficherMessage('--> DrawPolygonals()');
    for I := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(I);
      if (E.Type_Entite = tgENTRANCE)       then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      Printer.Canvas.Pen.Width := FVue2DParams.ongViseesLargeur;
      Printer.Canvas.Pen.Color := FBDDEntites.GetColorViseeFromModeRepresentation(FVue2DParams.ongModeRepresentation, E);
      if (BlackLines) then Printer.Canvas.Pen.Color:=clBlack;
      //Pen.Color := E.ColorEntite;
      P1 := Coord2Draft(E.Une_Station_1_X, E.Une_Station_1_Y);
      P2 := Coord2Draft(E.Une_Station_2_X, E.Une_Station_2_Y);
      Printer.Canvas.Line(P1, P2);
    end;
  end;
  procedure DrawSections();
  var
    i: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
  begin
    AfficherMessage('--> DrawSections()');
    Printer.Canvas.Pen.Width := Millimetres2PixelsXf(0.10);
    for I:=0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(I);
      if (E.Type_Entite = tgENTRANCE)       then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      Printer.Canvas.Pen.Color := clGray;
      P1:=Coord2Draft(E.X2PD, E.Y2PD);
      P2:=Coord2Draft(E.X2PG, E.Y2PG);
      Printer.Canvas.Line(P1, P2);
    end;
  end;
  procedure DrawQuadrilles();
  var
    P1, P2: TPoint;
    C1, C2: TPoint2Df;
    Coin1, Coin2: TPoint3Df;
    t: integer;
    A,B, B0, qQuadrillageSpc: double;
    S: string;
  begin
    AfficherMessage('--> DrawQuadrilles()');
    Coin1 := FBDDEntites.GetCoinBasGauche;
    Coin2 := FBDDEntites.GetCoinHautDroit;
    qQuadrillageSpc := EnsureSetQuadrillageSpacing(FVue2DParams.ongQdrSpc);
    Printer.Canvas.Pen.Color := FVue2DParams.ongQdrColor;
    Printer.Canvas.Pen.Width := Millimetres2PixelsXf(0.10);
    case FVue2DParams.ongQdrType of
      qtNONE: ;
      qtGRID: begin
        t := trunc(Coin1.X / qQuadrillageSpc);
        A := FVue2DParams.ongQdrSpc * t;
        Printer.Canvas.Brush.Color:= clWhite;
        Printer.Canvas.Font.Color := clBlack;
        Printer.Canvas.Font.Name  := DEFAULT_FONT_NAME;
        Printer.Canvas.Font.Style := [fsItalic];
        Printer.Canvas.Font.Height := Millimetres2PixelsYf(3.00);
        while (A < Coin2.X) do
        begin
          C1 := MakeTPoint2Df(A, Coin1.Y);
          C2 := MakeTPoint2Df(A, Coin2.Y);
          P1 := Coord2Draft(C1.X, C1.Y);
          P2 := Coord2Draft(C2.X, C2.Y);
          Printer.Canvas.MoveTo(P1); Printer.Canvas.LineTo(P2);
          // cotation
          S:=Format('%.0f',[A]);
          Printer.Canvas.TextOut(P1.X - Printer.Canvas.TextWidth(S) div 2, P1.Y - 2, S);
          A := A + qQuadrillageSpc;
         end;
         t := trunc(Coin1.Y / qQuadrillageSpc);
         A := FVue2DParams.ongQdrSpc * t;
         while (A<Coin2.Y) do
         begin
           C1 := MakeTPoint2Df(Coin1.X, A);
           C2 := MakeTPoint2Df(Coin2.X, A);
           P1:=Coord2Draft(C1.X, C1.Y);
           P2:=Coord2Draft(C2.X, C2.Y);
           Printer.Canvas.MoveTo(P1); Printer.Canvas.LineTo(P2);
           S:=Format('%.0f ',[A]);
           Printer.Canvas.TextOut(P1.X + 2, P1.Y - Printer.Canvas.TextHeight(S) div 2, S);
           A:= A + qQuadrillageSpc;
         end;
        end;
      qtCROSS:
      begin
        t := trunc(Coin1.X / qQuadrillageSpc);
        A := qQuadrillageSpc * t;
        t := trunc(Coin1.Y / qQuadrillageSpc);
        B := qQuadrillageSpc * t;
        B0:= B;
        while (A < Coin2.X) do
        begin
         B:=B0;
         while (B < Coin2.Y) do
         begin
           C1 := MakeTPoint2Df(A - FQdrCrossSize, B);
           C2 := MakeTPoint2Df(A + FQdrCrossSize, B);
           P1 := Coord2Draft(C1.X, C1.Y);
           P2 := Coord2Draft(C2.X, C2.Y);
           Printer.Canvas.MoveTo(P1); Printer.Canvas.LineTo(P2);
           C1 := MakeTPoint2Df(A, B - FQdrCrossSize);
           C2 := MakeTPoint2Df(A, B + FQdrCrossSize);
           P1 := Coord2Draft(C1.X, C1.Y);
           P2 := Coord2Draft(C2.X, C2.Y);
           Printer.Canvas.MoveTo(P1); Printer.Canvas.LineTo(P2);
           B := B + qQuadrillageSpc;
         end;
         A := A + qQuadrillageSpc;
        end;
      end;
      qtPOINTS: ;
    end; // case QdrType
  end;
  // dessin du cadre de découpage
  procedure DrawCorners();
  var
    CornerSize: integer;
  begin
    AfficherMessage('--> DrawCorners()');
    CornerSize:= Printer.PageWidth div 20;
    Printer.Canvas.Pen.Color:=clBlue;
    Printer.Canvas.Pen.Width:=0;

    Printer.Canvas.MoveTo(0,CornerSize);
    Printer.Canvas.LineTo(0,0);
    Printer.Canvas.LineTo(CornerSize,0);

    Printer.Canvas.MoveTo(0, Printer.PageHeight-1-CornerSize);
    Printer.Canvas.LineTo(0, Printer.PageHeight-1);
    Printer.Canvas.Lineto(CornerSize,Printer.PageHeight-1);

    Printer.Canvas.MoveTo(Printer.PageWidth-1-CornerSize,0);
    Printer.Canvas.LineTo(Printer.PageWidth-1, 0);
    Printer.Canvas.LineTo(Printer.PageWidth-1,CornerSize);

    Printer.Canvas.MoveTo(Printer.PageWidth-1-CornerSize, Printer.PageHeight-1);
    Printer.Canvas.LineTo(Printer.PageWidth-1, Printer.PageHeight-1);
    Printer.Canvas.LineTo(Printer.PageWidth-1, Printer.PageHeight-1-CornerSize);

        // numéros de page
    Printer.Canvas.Font.Size   := 8;
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.TextOut(10,10, Format('Page %d %d - by GHTopo',[C+1,L+1]));
  end;


  procedure DrawEntrances();
  var
    R666 : integer;
    i: integer;
    E: TEntiteEtendue;
    P2: TPoint;
    PM    : TPoint2Df;
  begin
    AfficherMessage('--> DrawEntrances()');
    R666 := Millimetres2PixelsXf(0.80);
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgENTRANCE) then
      begin // dessin des entrées
        PM := MakeTPoint2Df(E.Une_Station_2_X, E.Une_Station_2_Y);
        P2 := Coord2Draft(PM.X, PM.Y);
        Printer.Canvas.Pen.Width  := 0;
        Printer.Canvas.Brush.Color := clFuchsia;
        Printer.Canvas.EllipseC(P2.X, P2.Y, R666, R666);
      end;  //
    end; // for
  end;
  (*
  procedure DrawAnnotations;
  var
    A: TAnnotation;
    i: integer;
    PP: TPoint;
    PM: TPoint2Df;

    //TxtDecalage: TPoint;
    // positionnement du texte
    PosTXT: TPoint;
    // encombrement du texte
    ExTxt : TSize;

    s: string;
    // pour positionnement relatif à un point topo
    E: TEntite;
  begin
    if not (FTableEntites.LesAnnotationsSontOK) then Exit;
    try
      with FPrinter.Canvas do begin
        if FTableEntites.MyTableAnnotations.NbAnnotations=0 then Exit;
        for i:=0 to FTableEntites.MyTableAnnotations.NbAnnotations-1 do begin
          A:=FTableEntites.MyTableAnnotations.GetAnnotation(i);
          if Not(A.Displayed) then Continue;
          case A.ModePosition of
            0: begin // positionnement absolu
                 PM.X:=A.X;
                 PM.Y:=A.Y;
               end;
            1: begin // positionnement relatif à une station
                 E := FTableEntites.GetEntiteFromSerSt(A.BaseRefSer, A.BaseRefSt);
                 PM.X := E.Une_Station_2_X + A.DX;
                 PM.Y := E.Une_Station_2_Y + A.DY;
                 A.Z  := E.Une_Station_2_Z + A.DZ;

               end;
          end;
          PP:=Coord2Draft(PM.X, PM.Y);

          Font.Name:=IIF(A.FontName=NAME_FOR_DEFAULT_FONT, 'Arial', A.FontName);
          Font.Size:=A.FontSize;
          Font.Color:=A.FontColor;
          Brush.Color:=clWhite;
          //Brush.Color:=FBackGround;
          if A.FontBold then Font.Style:=Font.Style+[fsBold]
                        else Font.Style:=Font.Style-[fsBold];
          if A.FontItalic then Font.Style:=Font.Style+[fsItalic]
                          else Font.Style:=Font.Style-[fsItalic];
          if A.FontUnderline then Font.Style:=Font.Style+[fsUnderline]
                             else Font.Style:=Font.Style-[fsUnderline];

          S:=LLANFAIR_PG(A.Caption, A.MaxLength, A.Z,
                         A.BaseRefSer, A.BaseRefSt);
          // positionnement du texte

          //PP.X := 0;
          //PP.Y := 0;

          ExTxt :=  TextExtent(S);
          case A.PosPtDeBase of
            0,1: begin  // défaut = en bas à gauche
                   PosTXT.X := PP.X;
                   PosTXT.Y := PP.Y - ExTxt.cy;
                 end;
            2  : begin
                   PosTXT.X := PP.X - ExTxt.cx div 2;
                   PosTXT.Y := PP.Y - ExTxt.cy;
                 end;
            3  : begin
                   PosTXT.X := PP.X - ExTxt.cx;
                   PosTXT.Y := PP.Y - ExTxt.cy;
                 end;
            4  : begin
                   PosTXT.X := PP.X;
                   PosTXT.Y := PP.Y - ExTxt.cy div 2;
                 end;
            5  : begin
                   PosTXT.X := PP.X - ExTxt.cx div 2;
                   PosTXT.Y := PP.Y - ExTxt.cy div 2;
                 end;
            6  : begin
                   PosTXT.X := PP.X - ExTxt.cx;
                   PosTXT.Y := PP.Y - ExTxt.cy div 2;
                 end;
            7  : begin
                   PosTXT.X := PP.X;
                   PosTXT.Y := PP.Y;
                 end;
            8  : begin
                   PosTXT.X := PP.X - ExTxt.cx div 2;
                   PosTXT.Y := PP.Y;
                 end;
            9  : begin
                   PosTXT.X := PP.X - ExTxt.cx;
                   PosTXT.Y := PP.Y;
                 end;
          else
            begin
              PosTXT.X := PP.X;
              PosTXT.Y := PP.Y - ExTxt.cy;
            end;
          end;
          // calcul de BoundingBox
          A.BoundingBox.Left    := PosTXT.X;
          A.BoundingBox.Top     := PosTXT.Y;
          A.BoundingBox.Right   := PosTXT.x + Extxt.cx;
          A.BoundingBox.Bottom  := PosTXT.Y + Extxt.cy;
          // et sauvegarde Bounding Box
          FTableEntites.MyTableAnnotations.PutAnnotation(A, i);

          TextOut(PosTXT.X, PosTXT.Y, S);
          // débogage: Tracé du point de base
          Rectangle(PP.X, PP.Y,
                    PP.X + 2, PP.Y + 2);


        end; // for
      end; //with FPrinter.Canvas do begin
    except
    end;
  end;
  //*)
  procedure DrawIDStations();
  var
    i: integer;
    E: TEntiteEtendue;
    P1: TPoint;
  begin
    AfficherMessage('--> DrawIDStations()');
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.Font.Color  :=  btnColorTextIDStations.ButtonColor;
    Printer.Canvas.Font.Name   := DEFAULT_FONT_NAME;
    Printer.Canvas.Font.Height := Millimetres2PixelsYf(editIDStsFontHeightMM.Value);
    Printer.Canvas.Font.Style  := [];
    for I:=0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(I);
      if (E.Type_Entite = tgENTRANCE)       then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;
      Printer.Canvas.Pen.Color := clGray;
      P1 := Coord2Draft(E.Une_Station_2_X, E.Une_Station_2_Y);
      Printer.Canvas.TextOut(P1.X+2+RD, P1.Y+2+RD, Trim(E.oIDLitteral));
    end;
  end;
  procedure DrawStations();
  var
    i: integer;
    E: TEntiteEtendue;
    P1: TPoint;
  begin
    AfficherMessage('--> DrawStations()');
    Printer.Canvas.Pen.Color   := clRed;
    Printer.Canvas.Brush.Color := Printer.Canvas.Pen.Color;
    for i:=0 to FBDDEntites.GetNbEntitesVisees() - 1 do begin
     E := FBDDEntites.GetEntiteVisee(i);
     // TODO: Revoir cette zone
     if (E.Type_Entite = tgENTRANCE)       then Continue;
     if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;

     P1 := Coord2Draft(E.Une_Station_2_X,E.Une_Station_2_Y);
     Printer.Canvas.EllipseC(P1.X, P1.Y, RD, RD);
    end;
  end;

  // dessin du cartouche et du cadre périmétrique
  procedure DrawCartouche();
  const
    MG = 2.0; // marge 20 m
  var
    P1, P2: TPoint; //, P3, P4: TPoint;
  begin
    AfficherMessage('--> DrawCartouche()');

    P1 := Coord2Draft(FRXMini + MG, FRYMini + MG);
    P2 := Coord2Draft(FRXMaxi - MG, FRYMaxi - MG);
    //P3 := Coord2Draft(FTableEntites.cnMaxi.X, FTableEntites.cnMaxi.Y);
    //P4 := Coord2Draft(FTableEntites.cnMini.X, FTableEntites.cnMaxi.Y);
    Printer.Canvas.Pen.Color  :=clBlack;
    Printer.Canvas.Pen.Width  := Millimetres2PixelsXf(0.45);
      //Brush.Color:=Pen.Color;
    Printer.Canvas.MoveTo(P1.X, P1.Y); // ne pas utiliser MakeTPoint
    Printer.Canvas.LineTo(P2.X, P1.Y);
    Printer.Canvas.LineTo(P2.X, P2.Y);
    Printer.Canvas.LineTo(P1.X, P2.Y);
    Printer.Canvas.LineTo(P1.X, P1.Y);
    Printer.Canvas.Pen.Width  := 0;
  end;
  // cotation
  procedure DrawCotationStations(const DoDeltaCoords: boolean);
  const
    PROPORTIONAL_FONT_HEIGHT = 0.005;
  var
    i: integer;
    E: TEntiteEtendue;
    P1: TPoint;
    QQ: TPoint3Df;
    WU: Double;
  begin
    AfficherMessage('--> DrawCotationStations()');
    Printer.Canvas.Brush.Color := clWhite;
    Printer.Canvas.Font.Color  := clBlue;
    Printer.Canvas.Font.Name   := DEFAULT_FONT_NAME;
    Printer.Canvas.Font.Color  := btnColorTextCotation.ButtonColor;
    Printer.Canvas.Font.Height := Millimetres2PixelsYf(editCotFontHeightMM.Value);
    // hauteur de texte proportionnelle à la taille de la page
    // Printer.Canvas.Font.Height := round(min(Printer.PageHeight, Printer.PageWidth) * PROPORTIONAL_FONT_HEIGHT);
    Printer.Canvas.Font.Style  := [];
    for i := 0 to FBDDEntites.GetNbEntitesVisees() - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(I);
      QQ := FBDDEntites.GetDeltaXYZFromPositionSt0(E);
      if (E.Type_Entite = tgENTRANCE)       then Continue;
      if (E.Type_Entite = tgVISEE_RADIANTE) then Continue;

      Printer.Canvas.Pen.Color := clGray;
      P1 := Coord2Draft(E.Une_Station_2_X, E.Une_Station_2_Y);
      if (DoDeltaCoords) then WU := QQ.Z
                         else WU := E.Une_Station_2_Z;

      Printer.Canvas.TextOut(P1.X+2+RD, P1.Y+2-RD, Format('%f',[WU]));
    end;
  end;
  // visées radiantes
  procedure DrawViseesRadiantes();
  var
    i, n: integer;
    E: TEntiteEtendue;
    P1, P2: TPoint;
  begin
    n := FBDDEntites.GetNbEntitesVisees();
    AfficherMessage(Format('--> DrawViseesRadiantes(): %d', [n]));
    Printer.Canvas.Pen.Color := clGray;
    Printer.Canvas.pen.Width := 1;
    for i := 0 to n - 1 do
    begin
      E := FBDDEntites.GetEntiteVisee(i);
      if (E.Type_Entite = tgVISEE_RADIANTE) then
      begin
        P1 := Coord2Draft(E.Une_Station_1_X, E.Une_Station_1_Y);
        P2 := Coord2Draft(E.Une_Station_2_X, E.Une_Station_2_Y);
        Printer.Canvas.Line(P1, P2);
      end;
    end;
  end;
  // dessin de la règle
  procedure DrawRegle(const TailleRegle: double);
  const
    NB_CARREAUX_X = 8;
  var
    i : Integer;
    Mg: double;
    QXo, QYo, QY1: double;
    LargeurCarreau: Extended;
    HauteurCarreau: Extended;
    procedure QDrawRect(const X, Y, L, H: double; const LC, FC: TColor);
    var
      QP1, QP2: TPoint;
    begin
      QP1 := Coord2Draft(X, Y);
      QP2 := Coord2Draft(X+L, Y+H);
      Printer.Canvas.Pen.Width    := 0;
      Printer.Canvas.Pen.Color    := LC;
      Printer.Canvas.Brush.Color  := FC;
      Printer.Canvas.Rectangle(QP1.x, QP1.y, QP2.x, QP2.y);
    end;
    procedure QDrawTextLegende(const X, Y, L, H: double);
    var
      HauteurTexte: integer;
      QP1, QP2: TPoint;
      WU : string;
      EWE: TSize;
    begin
      Printer.Canvas.Brush.Color:= clWhite;
      Printer.Canvas.Font.Style := [fsBold];
      Printer.Canvas.Font.Color := clBlue;
      WU := Format('%.0f m',[L]);
      QP1   := Coord2Draft(X, Y);
      QP2   := Coord2Draft(X, Y + H);
      HauteurTexte := QP2.Y - QP1.Y;
      Printer.Canvas.Font.Height  := HauteurTexte;
      EWE := Printer.Canvas.TextExtent('0');
      Printer.Canvas.TextOut(QP1.X, QP1.Y - EWE.cy, '0');
      QP1   := Coord2Draft(X + L, Y);
      EWE   := Printer.Canvas.TextExtent(WU);
      QP1.X := QP1.X - EWE.cx;
      Printer.Canvas.TextOut(QP1.X, QP1.Y - EWE.Cy, WU);
    end;
  begin
    Mg := (FRXMaxi - FRXMini) / 50; // marge
    QXo := FRXMini + Mg;
    QYo := FRYMini + Mg;
    LargeurCarreau := TailleRegle / NB_CARREAUX_X;
    HauteurCarreau := TailleRegle / 25;
    AfficherMessage(Format(' --> DrawRegle: L = %.0f m at (%.0f, %.0f)', [TailleRegle, FRXMini + Mg, FRYMini + Mg]));
    // cadre périmétrique
    QDrawRect(QXo, QYo, TailleRegle, 2 * HauteurCarreau, clBlack, clWhite);
    // carreaux alternés
    for i := 0 to NB_CARREAUX_X - 1 do
    begin
      if (Odd(i)) then QY1 := QYo + HauteurCarreau else QY1 := QYo;
      QDrawRect(QXo + (i * LargeurCarreau), QY1, LargeurCarreau, HauteurCarreau, clBlack, clGray);
    end;
    // légende
    QDrawTextLegende(QXo, QYo + 2 * HauteurCarreau + Mg/4, TailleRegle, HauteurCarreau);
  end;

  //*)
begin
  try
    AfficherMessage(Format('Impression carré %d.%d (page %d)',[L,C, Printer.PageNumber]));
    WU := ChooseString(Ord(Printer.Orientation), ['PORTRAIT', 'LANDSCAPE', 'REVERSE LANDSCAPE', 'REVERSEPORTRAIT']);
    AfficherMessage('-- Orientation: ' + WU);
    DrawQuadrilles();
    DrawGaleries();

    if (edPolygonals    in FVue2DParams.ongElementsDrawn) then
    begin
      if (edFillGalerie in FVue2DParams.ongElementsDrawn) then DrawPolygonals(True)
                                                          else DrawPolygonals(False);
    end;
    if (edEntrances     in FVue2DParams.ongElementsDrawn) then DrawEntrances;
    if (edAnnotations   in FVue2DParams.ongElementsDrawn) then ; // DrawAnnotations;
    if (edcrossSections in FVue2DParams.ongElementsDrawn) then DrawSections;
    if (edStations      in FVue2DParams.ongElementsDrawn) then DrawStations;
    if (edIDStations    in FVue2DParams.ongElementsDrawn) then DrawIDStations;
    if (edAltitudes     in FVue2DParams.ongElementsDrawn) then DrawCotationStations(false);
    if (edCotes         in FVue2DParams.ongElementsDrawn) then DrawCotationStations(True);
    if (edANTENNES      in FVue2DParams.ongElementsDrawn) then DrawViseesRadiantes();


    DrawCorners();         // coins de découpage
    DrawCartouche();       // Cartouche
    DrawRegle(FRegleSize); // Règle
  except
  end;
end;


//******************************************************************************
procedure TfrmPrintingCenterExt.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Finaliser();
end;

procedure TfrmPrintingCenterExt.btnPreviewClick(Sender: TObject);
begin
  FVue2DParams := GetVue2DParamsFromDialog();
  //CalcNbPagesAndEmpty;
  Recadrer();
  DrawApercu();
end;

procedure TfrmPrintingCenterExt.btnStartImpressionClick(Sender: TObject);
begin
  if (GHTopoQuestionOuiNon('Lancer l''impression ?')) then ImprimerLaTopo();
end;

procedure TfrmPrintingCenterExt.btnApplyParams2DClick(Sender: TObject);
begin
  FVue2DParams := GetVue2DParamsFromDialog();
  if (FVue2DParams.ongQdrSpc < 5.00) then FVue2DParams.ongQdrSpc := 5.00;
  //CalcNbPagesAndEmpty;
  Recadrer();
  DrawApercu();
end;

procedure TfrmPrintingCenterExt.btnSelectPrinterClick(Sender: TObject);
begin
  SelectionnerImprimante(Printer);
  SetMyCurrentPrinter();

end;

procedure TfrmPrintingCenterExt.cmbEchelleChange(Sender: TObject);
var
  WU: Integer;
begin
  WU := StrToIntDef(cmbEchelle.Text, 1000);
  if (WU = 0) then WU := 1000;
  FEchelle := 1 / WU;
  CalcNbPagesAndEmpty();
  Recadrer();
  DrawApercu();
end;

procedure TfrmPrintingCenterExt.cmbTypeQuadrillageChange(Sender: TObject);
begin
  FVue2DParams.ongQdrType  := TQdrType(cmbTypeQuadrillage.ItemIndex);
  Recadrer();
  DrawApercu();
end;



procedure TfrmPrintingCenterExt.FormCreate(Sender: TObject);
begin
  InitialiserComboBoxQuadrillage(cmbTypeQuadrillage, qtGRID);
  InitialiserComboBoxRepresentationVisees(cmbRepresentation, rgSEANCES);
  // éléments du dessin

  chkGrpElementsDessin.Caption := GetResourceString(rsLAYERS);
  chkRegle.Caption             := GetResourceString(rsREGLE);

  grbPrinterNames.Caption      := GetResourceString(rsPRN_TBPRINTER);
  lbQdrSpacing.Caption         := GetResourceString(rsQDRSPACING);
  lbEchelle.Caption            := GetResourceString(rsECHELLE);
  btnStartImpression.Caption   := GetResourceString(rsSTARTPRINTING);
  btnPreview.Caption           := GetResourceString(rsPREVIEW);
  // règle
  FRegleDisplayed              := true;
  chkRegle.Checked             := FRegleDisplayed;
  FRegleSize                   := 100.00;
  editTailleRegle.Value        := FRegleSize;
  //-----------------------
  // combo des quadrillages

  (*
  // espacement des croix/grille
  with cmbUsualQdrSpacing do
  begin
     Clear;
     Items.Add(Format('%.0f',[FQdrEspc])); // item0 = Espacement par défaut
     Items.Add('10');
     Items.Add('20');
     Items.Add('25');
     Items.Add('50');
     Items.Add('100');
     Items.Add('200');
     Items.Add('250');
     Items.Add('500');
     Items.Add('1000');
     ItemIndex := 0;
  end;
  //*)
end;

procedure TfrmPrintingCenterExt.PaintBoxVueDblClick(Sender: TObject);
var
  PM: TPoint2Df;
  IdxX, IdxY: integer;
begin
  PM:=GetCoordsMonde(FPP);
  IdxX := 0; IdxY := 0;
  CalcIndexPageIncludingPt(PM, IdxX, IdxY);
  if (IdxX > -1) and (IdxY>-1) then begin
    affichermessage(format('%d %d',[IdxX, IdxY]));
    FTableauPagesDrawn[IdxX, IdxY]:=Not(FTableauPagesDrawn[IdxX, IdxY]);
    DrawApercu;
  end;
end;

procedure TfrmPrintingCenterExt.PaintBoxVueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  PM: TPoint2Df;
begin
  try
    FPP.X:=X;
    FPP.Y:=Y;
    PM := GetCoordsMonde(FPP);
    lbMouseCoordinates.Caption:=Format('%.2f, %.2f',[PM.X, PM.Y]);
  except
  end;
end;

procedure TfrmPrintingCenterExt.PaintBoxVuePaint(Sender: TObject);
begin
  DrawApercu;
end;

end.

