unit dlgParametrerOngletVue2D;
// Date: 28/08/2012
// Dialogue pour paramétrage des onglets de la vue 2D
// 20/11/2014: Support des largeurs de trait pour les polygonales (lié aux visées en antenne)
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  CallDialogsStdVersion, unitUtilsComposants,
  Common,
  StructuresDonnees, Classes, SysUtils, FileUtil, curredit, Forms, Controls,
  Graphics, Dialogs, StdCtrls, Buttons, Spin, ExtCtrls, ColorBox, ComCtrls;

type

  { TfrmParametrerOngletVue2D }

  TfrmParametrerOngletVue2D = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    btnQdrColor: TColorButton;
    btnDegradeStart: TColorButton;
    btnCubeColor: TColorButton;
    btnBGColor: TColorButton;
    btnDegradeStop: TColorButton;
    btnCoupeDevQdrColor: TColorButton;
    chkCubeBounds: TCheckBox;
    chkNoeuds: TCheckBox;
    chkAntennes: TCheckBox;
    chkAnnotations: TCheckBox;
    chkEntrances: TCheckBox;
    chkCotes: TCheckBox;
    chkAltitudes: TCheckBox;
    chkIDStations: TCheckBox;
    chkParois: TCheckBox;
    chkPolygonales: TCheckBox;
    chkQuadrillage: TCheckBox;
    chkCoupeDevQuadrillage: TCheckBox;
    chkVolumesVisibles: TCheckBox;
    chkRemplissage: TCheckBox;
    chkSections: TCheckBox;
    chkStations: TCheckBox;
    cmbRepresentation: TComboBox;
    cmbTypeQuadrillage: TComboBox;
    editFactZ: TCurrencyEdit;
    editQdrSpc: TComboBox;
    editCoupeDevQdrSpc: TComboBox;
    GroupBox1: TGroupBox;
    lbCoteMini: TLabel;
    lbCoteMaxi: TLabel;
    lbRepresentationMode: TLabel;
    Panel1: TPanel;
    pnlDegradeAltitudes: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    lbX1Y1: TStaticText;
    lbX2Y2: TStaticText;
    editLargeurTraitPolygonale: TSpinEdit;
    pnlParametresSpecifiques2D: TPanel;
    Panel2: TPanel;
    pnlParametresSpecifiquesCoupeDev: TPanel;
    pnlParametresSpecifiques3D: TPanel;
    sclFillOpacity: TTrackBar;


    procedure BitBtn1Click(Sender: TObject);
    procedure btnCubeColorClick(Sender: TObject);
    procedure btnDegradeStartClick(Sender: TObject);
    procedure chkCoupeDevQuadrillageChange(Sender: TObject);
    procedure editCoupeDevQdrSpcChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure GroupBox1Click(Sender: TObject);
    procedure Panel2Click(Sender: TObject);
  private
    FC1, FC2: TPoint2Df; // sauvegarde des coordonnées de zone

    { private declarations }
    FModeParametrage: TModeDialogParamVues;
    FCurrParams3D: TVue3DParams;
    FCurrParams2D: TVue2DParams;
    FCurrCoupeDeveloppeeParams: TCoupeDeveloppeeParams;
    procedure SetChksElementsDrawn(const E: TElementsDrawn);
    function  GetChksElementsDrawn: TElementsDrawn;
    procedure SetPanelParamVisible;
  public
    { public declarations }
    procedure SetModeParametrageDialog(const Q: TModeDialogParamVues);
    procedure SetValuesOnglet2D(const O: TVue2DParams);
    procedure SetValuesOnglet3D(const O: TVue3DParams);
    procedure SetValuesCoupeDeveloppee(const O: TCoupeDeveloppeeParams);


    function  GetValuesOnglet2D(): TVue2DParams;
    function  GetValuesOnglet3D(): TVue3DParams;
    function  GetValuesCoupeDeveloppee(): TCoupeDeveloppeeParams;
  end;

var
  frmParametrerOngletVue2D: TfrmParametrerOngletVue2D;

implementation

{$R *.lfm}



procedure TfrmParametrerOngletVue2D.FormShow(Sender: TObject);
begin
  self.Position := poScreenCenter;
  InitialiserComboBoxQuadrillage(cmbTypeQuadrillage, qtGRID);
  InitialiserComboBoxRepresentationVisees(cmbRepresentation, rgSEANCES);
  SetPanelParamVisible();
end;

procedure TfrmParametrerOngletVue2D.GroupBox1Click(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.Panel2Click(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.btnDegradeStartClick(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.chkCoupeDevQuadrillageChange(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.editCoupeDevQdrSpcChange(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.BitBtn1Click(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.btnCubeColorClick(Sender: TObject);
begin

end;

procedure TfrmParametrerOngletVue2D.SetChksElementsDrawn(const E: TElementsDrawn);
begin
  chkPolygonales.Checked      := (edPolygonals     in E);
  chkStations.Checked         := (edStations       in E);
  chkAltitudes.Checked        := (edAltitudes      in E);
  chkCotes.Checked            := (edCotes          in E);
  chkIDStations.Checked       := (edIDStations     in E);
  chkParois.Checked           := (edWalls          in E);
  chkRemplissage.Checked      := (edFillGalerie    in E);
  chkSections.Checked         := (edCrossSections  in E);
  chkQuadrillage.Checked      := (edQuadrilles     in E);
  chkEntrances.Checked        := (edENTRANCES      in E);
  chkAntennes.Checked         := (edANTENNES       in E);
  chkAnnotations.Checked      := (edANNOTATIONS    in E);
  chkNoeuds.Checked           := (edJONCTIONS      in E);
  chkVolumesVisibles.Checked  := (edVolumes        in E);
  chkCubeBounds.Checked       := (edBounds         in E);

end;


function TfrmParametrerOngletVue2D.GetChksElementsDrawn(): TElementsDrawn;
begin
  Result := [];
  if (chkEntrances.Checked)   then Result := Result + [edENTRANCES]
                              else Result := Result - [edENTRANCES];
  if (chkStations.Checked)    then Result := Result + [edStations]
                              else Result := Result - [edStations];
  if (chkAltitudes.Checked)   then Result := Result + [edAltitudes]
                              else Result := Result - [edAltitudes];
  if (chkCotes.Checked)       then Result := Result + [edCotes]
                              else Result := Result - [edCotes];
  if (chkParois.Checked)      then Result := Result + [edWalls]
                              else Result := Result - [edWalls];
  if (chkIDStations.Checked)  then Result := Result + [edIDStations]
                              else Result := Result - [edIDStations];
  if (chkSections.Checked)    then Result := Result + [edCrossSections]
                              else Result := Result - [edCrossSections];
  if (chkRemplissage.Checked) then Result := Result + [edFillGalerie]
                              else Result := Result - [edFillGalerie];
  if (chkQuadrillage.Checked) then Result := Result + [edQuadrilles]
                              else Result := Result - [edQuadrilles];
  if (chkPolygonales.Checked) then Result := Result + [edPolygonals]
                              else Result := Result - [edPolygonals];
  if (chkAntennes.Checked)    then Result := Result + [edANTENNES]
                              else Result := Result - [edANTENNES];
  if (chkAnnotations.Checked) then Result := Result + [edANNOTATIONS]
                              else Result := Result - [edANNOTATIONS];
  if (chkNoeuds.Checked)      then Result := Result + [edJONCTIONS]
                              else Result := Result - [edJONCTIONS];
  if (chkVolumesVisibles.Checked) then Result := Result + [edVolumes]
                                  else Result := Result - [edVolumes];
  if (chkCubeBounds.Checked)  then Result := Result + [edBounds]
                              else Result := Result - [edBounds];


end;

procedure TfrmParametrerOngletVue2D.SetPanelParamVisible;
var
  EWE: String;
begin
  EWE := 'Parametres pour la ';
  case FModeParametrage of
    mpdVUE2D            : self.Caption := EWE + 'Vue en Plan';
    mpdVUE3D            : self.Caption := EWE + 'Vue 3D';
    mpdCOUPE_DEVELOPPEE : self.Caption := EWE + 'Coupe developpee';
  end;
  pnlParametresSpecifiques2D.Visible       := false;
  pnlParametresSpecifiques3D.Visible       := false;
  pnlParametresSpecifiquesCoupeDev.Visible := false;
  pnlDegradeAltitudes.Visible              := false;
  case FModeParametrage of
    mpdVUE2D            : pnlParametresSpecifiques2D.Visible := true;
    mpdVUE3D            : pnlParametresSpecifiques3D.Visible := true;
    mpdCOUPE_DEVELOPPEE : pnlParametresSpecifiquesCoupeDev.Visible := true;
  end;
  case FModeParametrage of
    mpdVUE2D,
    mpdVUE3D            : pnlDegradeAltitudes.Visible := true;
    mpdCOUPE_DEVELOPPEE : pnlDegradeAltitudes.Visible := false;
  end;

end;

procedure TfrmParametrerOngletVue2D.SetModeParametrageDialog(const Q: TModeDialogParamVues);
begin
  FModeParametrage := Q;
end;



procedure TfrmParametrerOngletVue2D.SetValuesOnglet3D(const O: TVue3DParams);
begin
  self.Caption := 'Settings vue 3D';
  // sauvegarde paramètres
  FCurrParams3D := O;
  // panneaux paramètres spécifiques 3D
  pnlParametresSpecifiques2D.Visible := false;
  pnlParametresSpecifiques3D.Visible := true;
  // éléments à dessiner
  SetChksElementsDrawn(O.Elements3DDessines);
  // facteur Z
  editFactZ.Value := O.CoefMagnification;
  // doit-on dessiner les volumes
  chkVolumesVisibles.Checked  := (edVolumes in  O.Elements3DDessines);
  // couleurs
  btnBGColor.ButtonColor      := O.BackGround;
  btnCubeColor.ButtonColor    := O.ColorCube;
  btnDegradeStart.ButtonColor := O.ColorZMini;
  btnDegradeStop.ButtonColor  := O.ColorZMaxi;
  // opacité des remplissages
  sclFillOpacity.Position               := O.FillOpacity;
  // épaisseurs trait
  editLargeurTraitPolygonale.Value := O.ViseesLargeur;
end;


procedure TfrmParametrerOngletVue2D.SetValuesOnglet2D(const O: TVue2DParams);
begin
  self.Caption := 'Settings vue 2D';


  cmbTypeQuadrillage.ItemIndex      := Ord(O.ongQdrType);
  // sauvegarde paramètres
  FCurrParams2D := O;

  // panneau paramètres spécifiques 2D
  pnlParametresSpecifiques2D.Visible := True;
  pnlParametresSpecifiques3D.Visible := False;

  // éléments à dessiner
  SetChksElementsDrawn(O.ongElementsDrawn);

  // autres valeurs
  editQdrSpc.Text                   := Format('%.0f',[O.ongQdrSpc]);
  btnBGColor.ButtonColor            := O.ongBackGround;
  btnQdrColor.ButtonColor           := O.ongQdrColor;

  editLargeurTraitPolygonale.Value  := O.ongViseesLargeur;
  // sauvegarde temp des coordonnées de la zone d'affichage (non modifiables)
  FC1.X := O.ongX1;  FC1.Y := O.ongY1;
  FC2.X := O.ongX2;  FC2.Y := O.ongY2;
  lbX1Y1.Caption := Format('Du coin: %.0f - %.0f', [FC1.X, FC1.Y]);
  lbX2Y2.Caption := Format('Au coin: %.0f - %.0f', [FC2.X, FC2.Y]);
  // dégradé
  btnDegradeStart.ButtonColor       := O.ongDegradeStart;
  btnDegradeStop.ButtonColor        := O.ongDegradeStop;

  // opacité des remplissages
  sclFillOpacity.Position           := O.ongFillOpacite;
  // mode de représentation
  cmbRepresentation.ItemIndex := ord(O.ongModeRepresentation);
end;

function TfrmParametrerOngletVue2D.GetValuesOnglet2D: TVue2DParams;
begin
  // récup des params non modifiés ou masqués
  Result := FCurrParams2D;
  with Result do
  begin
    // couleur de fond
    ongBackGround      := btnBGColor.ButtonColor;
    // éléments à dessiner
    ongElementsDrawn := GetChksElementsDrawn();
    // quadrillage
    ongQdrColor        := btnQdrColor.ButtonColor;
    ongQdrType         := TQdrType(cmbTypeQuadrillage.ItemIndex);

    ongQdrSpc          := ConvertirEnNombreReel(Trim(editQdrSpc.Text), 10.00);
    if (ongQdrSpc < 10.00) then ongQdrSpc := 10.00; // valeur minimale de l'espacement
    // récupération des coordonnées de zone non modifiables
    ongX1 := FC1.X;   ongY1 := FC1.Y;
    ongX2 := FC2.X;   ongY2 := FC2.Y;
    // dégradé
    ongDegradeStart := btnDegradeStart.ButtonColor;
    ongDegradeStop  := btnDegradeStop.ButtonColor;
    // largeur de trait
    ongViseesLargeur := editLargeurTraitPolygonale.Value;
    // opacité des remplissages
    ongFillOpacite   := sclFillOpacity.Position;
    // mode de representation
    ongModeRepresentation := TModeRepresentationGaleries(cmbRepresentation.ItemIndex);
  end;
end;

function TfrmParametrerOngletVue2D.GetValuesOnglet3D: TVue3DParams;
begin
  // récup des params non modifiés ou masqués
  Result := FCurrParams3D;
  with Result do
  begin
    // couleur de fond
    BackGround      := btnBGColor.ButtonColor;
    // éléments à dessiner
    Elements3DDessines := GetChksElementsDrawn();
    // facteur Z
    CoefMagnification := editFactZ.Value;

    // cube
    ColorCube     := btnCubeColor.ButtonColor;
    // dégradés
    ColorZMini    := btnDegradeStart.ButtonColor;
    ColorZMaxi    := btnDegradeStop.ButtonColor;
    // opacité des remplissages
    FillOpacity   := sclFillOpacity.Position;
    // largeur de trait
    ViseesLargeur := editLargeurTraitPolygonale.Value;
  end;
end;
//------------------------------------------------------------------------------
function TfrmParametrerOngletVue2D.GetValuesCoupeDeveloppee: TCoupeDeveloppeeParams;
begin
  // récupération des valeurs non modifiées ou masquées
  Result := FCurrCoupeDeveloppeeParams;
  Result.ElementsDrawn := GetChksElementsDrawn;
  Result.BackGround    := btnBGColor.ButtonColor;
  Result.QdrColor      := btnCoupeDevQdrColor.ButtonColor;
  Result.QdrSpc        := ConvertirEnNombreReel(editCoupeDevQdrSpc.Text, 10.00);
  Result.ColorZMini    := btnDegradeStart.ButtonColor;
  Result.ColorZMaxi    := btnDegradeStop.ButtonColor;
  result.ViseesLargeur := editLargeurTraitPolygonale.Value;

end;
procedure TfrmParametrerOngletVue2D.SetValuesCoupeDeveloppee(const O: TCoupeDeveloppeeParams);
begin
  FCurrCoupeDeveloppeeParams := O;
  SetChksElementsDrawn(FCurrCoupeDeveloppeeParams.ElementsDrawn);
  btnBGColor.ButtonColor := FCurrCoupeDeveloppeeParams.BackGround;
  btnCoupeDevQdrColor.ButtonColor:= FCurrCoupeDeveloppeeParams.QdrColor;
  editCoupeDevQdrSpc.Text := Format('%.0f', [FCurrCoupeDeveloppeeParams.QdrSpc]);
  btnDegradeStart.ButtonColor := FCurrCoupeDeveloppeeParams.ColorZMini;
  btnDegradeStop.ButtonColor := FCurrCoupeDeveloppeeParams.ColorZMaxi;
  editLargeurTraitPolygonale.Value := FCurrCoupeDeveloppeeParams.ViseesLargeur;
end;


end.

