unit dlgSelectDansListes;
// différent de la version Delphi. Retourne un nombre
// Date: 19/04/2012
// Statut: Fonctionnel
// 20/04/2012: DONE: Redessin de la liste après modification de largeur de colonne
// 04/05/2012: DONE: Retourne les index TOPOROBOT et non les index internes
// 01/12/2013: DONE: GetResourceString() systématiquement utilisé pour les captions
// 16/04/2014: Suppression de la fonction vide GetIndexByInternalIndex
// 13/10/2016: Ajout d'une box de recherche rapide

{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  UnitClassPalette, Common,
  StructuresDonnees,
  ToporobotClasses2012,
  unitobjetserie,
  unitUtilsComposants,
  Classes,
  SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ComCtrls,
  types, LCLType, Buttons;

type

  { TdlgSelectElement }

  TdlgSelectElement = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    btnFind: TButton;
    editRechercheRapide: TEdit;
    hcColsTitres: THeaderControl;
    lbNbElements: TLabel;
    lbTitreListe: TLabel;
    ListeElements: TListBox;
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure hcColsTitresResize(Sender: TObject);
    procedure hcColsTitresSectionResize(HeaderControl: TCustomHeaderControl;
      Section: THeaderSection);
    procedure ListeElementsClick(Sender: TObject);
    procedure ListeElementsDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
  private
    { private declarations }
    FReady: boolean;
    FDocToporobot: TToporobotStructure2012;
    FNumeroElement: integer;
    FModeSelection: TModeSelectionListe;
    FNbItems: LongInt;
    FMyPalette: TPalette256;
    // tableau d'index
    FTableauIndex: array of Integer;
    function FindInternalIdx(const Idx: integer): integer;
  public
    { public declarations }
    function GetNumeroElement: integer;
    function InitialiseListeSelection(const DT: TToporobotStructure2012;
                                      const MS: TModeSelectionListe;
                                      const QIndex: integer;
                                      const UseInternalIndex: boolean): boolean;
    function GetIndexElement(const UseInternalIndex: boolean): integer;
  end; 

var
  dlgSelectElement: TdlgSelectElement;

implementation

{$R *.lfm}
function TdlgSelectElement.FindInternalIdx(const Idx: integer): integer;
var
 i: integer;
begin
  Result := -1;
  if SizeOf(FTableauIndex) = 0 then Exit;
  try
    for i:= 0 to High(FTableauIndex) do
    begin
      if (FTableauIndex[i] = Idx) then
      begin
        Result := i;
        Exit;
      end;
    end;

  except
  end;
end;

function TdlgSelectElement.InitialiseListeSelection(const DT: TToporobotStructure2012;
                                                    const MS: TModeSelectionListe;
                                                    const QIndex: integer;
                                                    const UseInternalIndex: boolean): boolean;
var
  i: integer;
  miou: string;
  ht:  THeaderSection;
  //MyIndex: LongInt;
  CC: TCode;
  EN: TEntrance;
  EE: TExpe;
  SSR: TObjSerie;
  RR: TReseau;
  SS: TSecteur;

  //R: TReseau;
  procedure AjouterTitreColonne(const Titre: string; const LG: integer);
  begin
    ht := hcColsTitres.Sections.Add;
    ht.Text := GetResourceString(Titre);
    ht.MinWidth := LG;

  end;
  procedure AfficherNbElements();
    procedure MiouMiou(const Miou: string); inline;
    begin
      lbNbElements.Caption := format(GetResourceString(Miou), [ListeElements.Count]);
    end;
  begin
    case FModeSelection of
      mslENTRANCES   : MiouMiou(rsNB_ELEMENTS_ENTRANCES);
      mslRESEAUX     : MiouMiou(rsNB_ELEMENTS_RESEAUX);
      mslSECTEURS    : MiouMiou(rsNB_ELEMENTS_SECTEURS);
      mslCODE        : MiouMiou(rsNB_ELEMENTS_CODES);
      mslEXPE        : MiouMiou(rsNB_ELEMENTS_EXPES);
      mslSERIE       : ;
      mslTYPE_GALERIE: ;
      mslDATE        : ;
      mslSTATIONS_CALCULEES: ;
    end;
  end;
begin
  FReady := False;
  Result := False;
  //ShowMessage('InitialiseListeSelection: '+ inttostr(QIndex) + ' - ' + BoolToStr(UseInternalIndex, 'Index internes', 'Refs Toporobot'));
  FDocToporobot := DT;

  FModeSelection := MS;
  // purge des titres des headers
  hcColsTitres.Sections.Clear;
  // titres
  case FModeSelection of
    mslENTRANCES:
      begin
        miou := GetResourceString(rsSELECT_LISTE_ENTREES);
        FNbItems          := FDocToporobot.GetNbEntrees;
        self.Caption      := GetResourceString(rsSELECT_LISTE_ENTRANCE);
        with hcColsTitres do begin
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_ID, 60);
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_NOM, 340);
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_REF, 100);
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_X, 140);
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_Y, 140);
          AjouterTitreColonne(rsSELECT_LISTE_ENTREES_Z, 140);
        end;
      end;
    mslRESEAUX:
      begin
        miou := GetResourceString(rsSELECT_LISTE_RESEAUX);
        FNbItems := FDocToporobot.GetNbReseaux;
        self.Caption      := GetResourceString(rsSELECT_LISTE_RESEAU);
        with hcColsTitres do begin
          AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_ID , 60);
          AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_COLOR , 100);
          AjouterTitreColonne(rsSELECT_LISTE_RESEAUX_NOM , 340);
        end;
      end;
    mslSECTEURS:
      begin
        miou := GetResourceString(rsSELECT_LISTE_SECTEURS);
        FNbItems := FDocToporobot.GetNbSecteurs;
        self.Caption      := GetResourceString(rsSELECT_LISTE_SECTEUR);
        with hcColsTitres do
        begin
          AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_ID    , 60);
          AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_COLOR , 100);
          AjouterTitreColonne(rsSELECT_LISTE_SECTEURS_NOM   , 340);
        end;
        //*)
      end;
    mslCODE:
      begin
        miou := GetResourceString(rsSELECT_LISTE_CODES);
        FNbItems := FDocToporobot.GetNbCodes;
        self.Caption      := GetResourceString(rsSELECT_LISTE_CODE);
        with hcColsTitres do begin
          AjouterTitreColonne(rsSELECT_LISTE_CODES_ID , 60);
          AjouterTitreColonne(rsSELECT_LISTE_CODES_AZIMUTS , 100);
          AjouterTitreColonne(rsSELECT_LISTE_CODES_PENTES , 100);
          AjouterTitreColonne(rsSELECT_LISTE_CODES_OBS , 500);
        end;
      end;
    mslEXPE:
      begin
        // préparation de la palette
        FMyPalette := TPalette256.Create;
        FMyPalette.GenerateTOPOROBOTPalette;
        // -------------------------
        miou := GetResourceString(rsSELECT_LISTE_EXPES);
        FNbItems := FDocToporobot.GetNbExpes;
        self.Caption      := GetResourceString(rsSELECT_LISTE_EXPE);
        with hcColsTitres do
        begin
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_ID,  60);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_COULEUR , 100);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_DATE , 120);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_OPERATEUR_TOPO , 200);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_CLUB_SPELEO , 200);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_DECLINAISON , 100);
          AjouterTitreColonne(rsSELECT_LISTE_EXPES_OBS , 500);
        end;
      end;
    mslSERIE:
      begin
        miou := GetResourceString(rsSELECT_LISTE_SERIES);
        FNbItems := FDocToporobot.GetNbSeries;
        self.Caption      := GetResourceString(rsSELECT_LISTE_SERIE);
        with hcColsTitres do
        begin
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_ID, 60);
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_DEPART, 100);
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_ARRIVEE, 100);
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_NOM, 400);
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_RESEAU, 300);
          AjouterTitreColonne(rsSELECT_LISTE_SERIES_NBPOINTS, 80);
        end;
      end;
  end;
  lbTitreListe.Caption := Format(rsSELECT_LISTE_NB_ELEMENTS, [FNbItems, miou]);
  Result:=False;
  // liste
  ListeElements.ItemHeight := 22;
  try

    SetLength(FTableauIndex, 0);
    if FNbItems = 0 then Exit;
    ListeElements.Clear;
    SetLength(FTableauIndex, FNbItems);
    for i:=0 to FNbItems - 1 do
    begin
      //R:=GetReseau(i);
      //lsbReseaux.Items.Add(Format('%000d - %s',  [R.IdxReseau, R.NomReseau]));
      case FModeSelection of
         mslENTRANCES:
          begin
            EN := FDocToporobot.GetEntree(i);
            FTableauIndex[i] := i;
            ListeElements.Items.Add(Format('%d: %s', [i, EN.eNomEntree]));
          end;
        mslRESEAUX:
          begin
            RR := FDocToporobot.GetReseau(i);
            FTableauIndex[i] := i;
            ListeElements.Items.Add(Format('%d: %s', [i, RR.NomReseau]));
          end;
        mslSECTEURS:
          begin
            SS := FDocToporobot.GetSecteur(i);
            FTableauIndex[i] := i;
            ListeElements.Items.Add(Format('%d: %s', [i, SS.NomSecteur]));
          end;
        mslCODE :
          begin
            CC := FDocToporobot.GetCode(i);
            FTableauIndex[i] := CC.IDCode;
            ListeElements.Items.Add(Format('%d: %s', [CC.IDCode, CC.Commentaire]));
          end;
        mslEXPE :
          begin
            EE := FDocToporobot.GetExpe(i);
            FTableauIndex[i] := EE.IDExpe;
            ListeElements.Items.Add(Format('%d: %s - %s', [EE.IDExpe, Format('%.2d/%.2d/%.4d', [EE.JourExpe, EE.MoisExpe, EE.AnneeExpe]), EE.Commentaire]));
          end;
        mslSERIE :
          begin
            SSR := FDocToporobot.GetSerie(i);
            FTableauIndex[i] := SSR.GetNumeroDeSerie();
            ListeElements.Items.Add(Format('%d: %s', [SSR.GetNumeroDeSerie(), SSR.GetNomSerie()]));
          end;
        else
          FTableauIndex[i] := i;
      end;

    end;
    if (UseInternalIndex) then FNumeroElement := QIndex
                          else FNumeroElement := FindInternalIdx(QIndex);
    ListeElements.ItemIndex := FNumeroElement;
    AfficherNbElements();
    Result := True;
    FReady := True;
  except
  end;
end;

function  TdlgSelectElement.GetNumeroElement: integer;
begin
  Result := FNumeroElement;
end;
function TdlgSelectElement.GetIndexElement(const UseInternalIndex: boolean): integer;
begin
  if (not FReady) then Exit;
  Result := -1;
  try
    if (UseInternalIndex) then Result := FNumeroElement
                          else Result := FTableauIndex[FNumeroElement];
  except
  end;
end;



procedure TdlgSelectElement.ListeElementsDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  Q4  = 4; // compensation du décalage entre le header et la liste
  mg  = 1;
  // lgr = 25;
var
  C: TColor;
  m: integer;
  r: TRect;
  rs: TReseau;
  es: TEntrance;
  cs: TCode;
  ss: TExpe;
  sr: TObjSerie;
  miou: string;
  sc: TSecteur;
  procedure DessineFiletColonne(const TB: integer); inline;
  begin
    ListeElements.Canvas.Line(TB, ARect.Top, TB, ARect.Bottom);
  end;
  // bg = couleur de fond; tc= couleur du texte
  procedure DessineItem(const bg,tc: TColor);
  VAR
    //TB: integer;
    HS: THeaderSection;
  begin
    with ListeElements do begin
      case FModeSelection of
        mslENTRANCES:
          begin
            Canvas.Brush.Color:= bg;
            Canvas.FillRect(ARect);

            Canvas.Font.Color := tc;
            Canvas.Pen.Color  :=clSilver; // pour les filets
            //TB := hcColsTitres.hea
            HS := hcColsTitres.Sections.Items[0];  // ID entrée
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[1];  // Nom entrée
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(es.eNomEntree));
            HS := hcColsTitres.Sections.Items[2];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%d.%d',[es.eRefSer, es.eRefSt]));
            HS := hcColsTitres.Sections.Items[3];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eXEntree));
            HS := hcColsTitres.Sections.Items[4];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eYEntree));
            HS := hcColsTitres.Sections.Items[5];
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, FormatterNombreAvecSepMilliers(es.eZEntree));
            DessineFiletColonne(HS.Right);
          end;
        mslRESEAUX:
          begin
            // offset:=Rect.Left + 30 + lgr;
            //Canvas.Brush.Color:=clWhite;
            Canvas.FillRect(ARect);
            canvas.Pen.Color:=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[0];  // ID réseau
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[1];  // Couleur réseau
            DessineFiletColonne(HS.Left - Q4);
            canvas.Pen.Color:=clBlack;
            canvas.Brush.Color:=rs.ColorReseau; // couleur du réseau
            r.Left :=HS.Left;
            r.Right:=HS.Right - 8;
            r.Top  :=ARect.Top + mg;
            r.Bottom:=ARect.Bottom - mg;
            canvas.Rectangle(R);
            Canvas.Brush.Color:=bg;
            canvas.Pen.Color:=clSilver;
            HS := hcColsTitres.Sections.Items[2];  // Nom réseau
            DessineFiletColonne(HS.Left - Q4);


            //Canvas.Font.Color :=clBlack;
            Canvas.Font.Color :=tc;
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(rs.NomReseau));
          end;
        mslSECTEURS:
          begin
            // offset:=Rect.Left + 30 + lgr;
            //Canvas.Brush.Color:=clWhite;
            Canvas.FillRect (ARect);
            canvas.Pen.Color  := clSilver;
            Canvas.Font.Color := tc;

            HS := hcColsTitres.Sections.Items[0];  // ID secteur
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[Index]));
            HS := hcColsTitres.Sections.Items[1];  // Couleur secteur
            DessineFiletColonne(HS.Left - Q4);
            canvas.Pen.Color:=clBlack;
            canvas.Brush.Color:=sc.CouleurSecteur; // couleur du secteur
            r.Left :=HS.Left;
            r.Right:=HS.Right - 8;
            r.Top  :=ARect.Top + mg;
            r.Bottom:=ARect.Bottom - mg;
            canvas.Rectangle(R);
            Canvas.Brush.Color:=bg;
            canvas.Pen.Color:=clSilver;
            HS := hcColsTitres.Sections.Items[2];  // Nom secteur
            DessineFiletColonne(HS.Left - Q4);


            //Canvas.Font.Color :=clBlack;
            Canvas.Font.Color :=tc;
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(sc.NomSecteur));
          end;
        mslCODE:
          begin
            Canvas.FillRect(ARect);
            HS := hcColsTitres.Sections.Items[0];  // Nom réseau

            Canvas.Brush.Color:=bg;
            Canvas.Font.Color :=tc;
            canvas.Pen.Color:=clSilver;

            canvas.TextOut(HS.Left + 4  , ARect.Top+1, Format('%d', [cs.IDCode]));

            HS := hcColsTitres.Sections.Items[1];  // az
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4  , ARect.Top+1, Format('%.0f', [cs.GradAz]));
            HS := hcColsTitres.Sections.Items[2];  // p
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.0f', [cs.Gradinc]));
            HS := hcColsTitres.Sections.Items[3];  // Commentaires
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(cs.Commentaire));
          end;
        mslEXPE:
          begin
            try
              miou := DateToStr(GetSecuredDate(ss.AnneeExpe,ss.MoisExpe,ss.JourExpe));
            except
              miou := '01/01/2000';
            end;

            Canvas.FillRect(ARect);
            canvas.Pen.Color  :=clSilver;
            Canvas.Font.Color :=tc;

            HS := hcColsTitres.Sections.Items[0];  // ID expé
            canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[ss.IDExpe]));
            HS := hcColsTitres.Sections.Items[1];  // Couleur expé
            DessineFiletColonne(HS.Left - Q4);
            // couleur de l'expé
            canvas.Pen.Color:=clBlack;
            canvas.Brush.Color:=FMyPalette.GetColorByIndex(ss.Couleur);

            r.Left :=HS.Left;
            r.Right:=HS.Left + 32;
            r.Top  :=ARect.Top + mg;
            r.Bottom:=ARect.Bottom - mg;
            canvas.Rectangle(R);

            Canvas.Brush.Color:=bg;
            Canvas.TextOut(HS.Left + 32 + 4, ARect.Top+1, Format('%d',[ss.Couleur]));

            //Canvas.Brush.Color:=bg;

            canvas.Pen.Color:=clSilver;

            HS := hcColsTitres.Sections.Items[2];  // date
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, miou);

            HS := hcColsTitres.Sections.Items[3];  // spéléomètre
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.Operateur));

            HS := hcColsTitres.Sections.Items[4];  // spéléographe
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.ClubSpeleo));

            HS := hcColsTitres.Sections.Items[5];  // déclinaison
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, Format('%.3f',[ss.Declinaison]));


            HS := hcColsTitres.Sections.Items[6];  // commentaires
            DessineFiletColonne(HS.Left - Q4);
            canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(ss.Commentaire));
          end;
        mslSERIE:
          begin
            with ListeElements do
            begin
              Canvas.FillRect(ARect);
              canvas.Pen.Color:=clSilver;
              Canvas.Font.Color :=tc;

              HS := hcColsTitres.Sections.Items[0];  // ID série
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNumeroDeSerie()]));
              HS := hcColsTitres.Sections.Items[1];  // Départ
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieDep, sr.GetNoPointDep]));
              HS := hcColsTitres.Sections.Items[2];  // Arrivée
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d.%d',[sr.GetNoSerieArr, sr.GetNoPointArr]));
              HS := hcColsTitres.Sections.Items[3];  // nom
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1, _AnsiToLCLStr(sr.GetNomSerie));
              // réseau
              HS := hcColsTitres.Sections.Items[4];  // nom
              DessineFiletColonne(HS.Left - Q4);
              rs := FDocToporobot.GetReseau(sr.GetNumeroReseau);

              canvas.Pen.Color:=clBlack;
              canvas.Brush.Color:=rs.ColorReseau;
              r.Left :=HS.Left;
              r.Right:=HS.Left + 32;
              r.Top  :=ARect.Top + mg;
              r.Bottom:=ARect.Bottom - mg;
              canvas.Rectangle(R);
              Canvas.Brush.Color:= bg;
              Canvas.TextOut(HS.Left + 32 + 4, ARect.Top+1, _AnsiToLCLStr(rs.NomReseau));

              Canvas.Brush.Color:=bg;

              canvas.Pen.Color:=clSilver;

              HS := hcColsTitres.Sections.Items[5];  // Nb points
              DessineFiletColonne(HS.Left - Q4);
              canvas.TextOut(HS.Left + 4, ARect.Top+1,  Format('%d',[sr.GetNbVisees]));
            end;
          end;
      end; //case FModeSelection of
    end;
  end;
begin
  if (Not FReady) then Exit;

  case FModeSelection of
    mslENTRANCES : es := FDocToporobot.GetEntree(Index);
    mslRESEAUX   : rs := FDocToporobot.GetReseau(Index);
    mslSECTEURS  : sc := FDocToporobot.GetSecteur(Index);
    mslCODE      : cs := FDocToporobot.GetCode(Index);
    mslEXPE      : ss := FDocToporobot.getExpe(Index);
    mslSERIE     : sr := FDocToporobot.GetSerie(Index);
  end; //case FModeSelection of
  if (odSelected in state) then DessineItem(clBlue, clWhite)
                           else DessineItem(clwhite, clBlack);
end;

procedure TdlgSelectElement.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  ListeElements.Clear;
  // libération de la palette utilisées par le mode Expés
  if (FModeSelection = mslEXPE) then begin
    try
      ;
    finally
      FMyPalette.Free;
    end;
  end;
end;

procedure TdlgSelectElement.FormCloseQuery(Sender: TObject;
  var CanClose: boolean);
begin
end;

procedure TdlgSelectElement.FormCreate(Sender: TObject);
begin

end;

procedure TdlgSelectElement.hcColsTitresResize(Sender: TObject);
begin
end;

procedure TdlgSelectElement.hcColsTitresSectionResize(
  HeaderControl: TCustomHeaderControl; Section: THeaderSection);
begin
  ListeElements.Invalidate;
end;

procedure TdlgSelectElement.ListeElementsClick(Sender: TObject);
begin
  FNumeroElement := ListeElements.ItemIndex;
end;

procedure TdlgSelectElement.BitBtn1Click(Sender: TObject);
begin
end;

procedure TdlgSelectElement.BitBtn2Click(Sender: TObject);
begin
  FNumeroElement := -1;
end;

procedure TdlgSelectElement.btnFindClick(Sender: TObject);
var
  QIdx: integer;
begin
  if (FindIdxOccurrenceInListBox(ListeElements, trim(editRechercheRapide.Text), QIdx)) then
    ListeElements.ItemIndex := QIdx
  else
    ShowMessage(GetResourceString(rsMATCHNOTFOUND));
end;




end.

