unit dlgEditeurSectionTransversale;

{$INCLUDE CompilationParameters.inc}

interface

uses
  GHCD_Types,
  {$IFDEF LANGUAGE_FRENCH}
    UnitMessages_fr,
  {$ENDIF}
  GeneralFunctions, math, unitSectionsTransversales, UnitDocDessin,
  cdrSectionTransversale, CallDialogs, Classes, SysUtils, FileUtil, curredit,
  SynEdit, SynHighlighterAny, SynCompletion, Forms, Controls, Graphics, Dialogs,
  Buttons, StdCtrls, ExtCtrls, ActnList, ComCtrls, Menus, LCLType;

type

  { TfrmEditeurSections }

  TfrmEditeurSections = class(TForm)
    acCourbe: TAction;
    acPanDown: TAction;
    acPanLeft: TAction;
    acPanRight: TAction;
    acPanUp: TAction;
    acPolygone: TAction;
    acReset: TAction;
    acSelectCourbe: TAction;
    acSelectPolygone: TAction;
    acSelectSimpleLigne: TAction;
    acSelectTexte: TAction;
    acSimpleLigne: TAction;
    acTexte: TAction;
    acOpenSection: TAction;
    acSaveSection: TAction;
    acNouvelleSection: TAction;
    acSerialiser: TAction;
    acParser: TAction;
    acHelp: TAction;
    acQuit: TAction;
    ActionList1: TActionList;
    acValidateCourbe: TAction;
    acValidateCourbePolygone: TAction;
    acZoomMoins: TAction;
    acZoomPlus: TAction;
    acZoomTout: TAction;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    CadreSectionTransversale1: TCadreSectionTransversale;
    cmbGroupes: TComboBox;
    editBasePointID: TEdit;
    editOffsetX: TCurrencyEdit;
    editIdxGroupe: TCurrencyEdit;
    editDemiOuverture: TCurrencyEdit;
    editAngleOrientation: TCurrencyEdit;
    editOffsetY: TCurrencyEdit;
    ImageList1: TImageList;
    imglstMenus: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    MainMenu1: TMainMenu;
    memoHerisson: TMemo;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    mnuAide: TMenuItem;
    mnuFichier: TMenuItem;
    PageControl1: TPageControl;
    Panel1: TPanel;
    popupCourbePoly: TPopupMenu;
    popupGeneral: TPopupMenu;
    lbBasePointIDTerrain: TStaticText;
    StaticText1: TStaticText;
    SynKwdsGHCaveDraw: TSynAnySyn;
    SynEdit1: TSynEdit;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton10: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure acHelpExecute(Sender: TObject);
    procedure acNouvelleSectionExecute(Sender: TObject);
    procedure acOpenSectionExecute(Sender: TObject);
    procedure acParserExecute(Sender: TObject);
    procedure acQuitExecute(Sender: TObject);
    procedure acSaveSectionExecute(Sender: TObject);
    procedure acSerialiserExecute(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
  private
    { private declarations }
    FMyDocTopo              : TDocumentDessin;
    FHerissonViseesRadiantes: THerissonViseesRadiantes;
    FCurrentCrossSection    : TCrossSection;
    FAutoCompletion         : TSynCompletion;
    procedure AnalyserEtTracerSection();
    procedure InitCaptionsMenus();
    procedure InitColorationSyntaxique();
    function  initSectionVierge(): boolean;
    procedure ListerGroupes();
    procedure Serialiser();
  public
    { public declarations }
    function Initialiser(const FD: TDocumentDessin; const BP: TBaseStation; const QOffsetX, QOffsetY: double): boolean;
    procedure Finaliser();
  end;

var
  frmEditeurSections: TfrmEditeurSections;

implementation

{$R *.lfm}

{ TfrmEditeurSections }

procedure TfrmEditeurSections.InitColorationSyntaxique();
var
  n, i: Integer;
  procedure F666(const WU: string);
  begin
    SynKwdsGHCaveDraw.KeyWords.Add(uppercase(WU)); // keywords must be uppercased
  end;
begin
  AfficherMessage(format('%s.InitColorationSyntaxique()', [ClassName]));

  SynKwdsGHCaveDraw.KeyWords.Clear;
  F666(CROSS_PROFILES_SECTION);
    F666(CROSS_SECTION);
      F666(CROSS_SECTION_COURBES);
        F666(CROSS_SECTION_A_COURBE);
          F666(CROSS_SECTION_ARC_COURBE);
        F666(END_CROSS_SECTION_A_COURBE);
      F666(END_CROSS_SECTION_COURBES);
      F666(CROSS_SECTION_POLYGONES);
        F666(CROSS_SECTION_A_POLYGONE);
          F666(CROSS_SECTION_POLY_VERTEX);
        F666(END_CROSS_SECTION_A_POLYGONE);
      F666(END_CROSS_SECTION_POLYGONES);
      F666(CROSS_SECTION_LINES);
        F666(CROSS_SECTION_SIMPLELINE);
      F666(END_CROSS_SECTION_LINES);
      F666(CROSS_SECTION_TEXTES);
        F666(CROSS_SECTION_SIMPLETEXT);
      F666(END_CROSS_SECTION_TEXTES);
    F666(END_CROSS_SECTION);
  F666(END_CROSS_PROFILES_SECTION);
  SynKwdsGHCaveDraw.Markup  := false;
  SynKwdsGHCaveDraw.Enabled := True;
  // contrôle
  n := SynKwdsGHCaveDraw.KeyWords.count;
  SynEdit1.Clear;
  SynEdit1.BeginUpdate();
    for i := 0 to n - 1 do SynEdit1.Lines.Add(SynKwdsGHCaveDraw.KeyWords.Strings[i]);
  SynEdit1.EndUpdate;
  // autocompletion
  FAutoCompletion.ItemList.Clear;
  for i := 0 to n - 1 do FAutoCompletion.ItemList.Add(LowerCase(SynKwdsGHCaveDraw.KeyWords.Strings[i]));

end;



procedure TfrmEditeurSections.acHelpExecute(Sender: TObject);
begin
  ;
end;

procedure TfrmEditeurSections.acNouvelleSectionExecute(Sender: TObject);
begin
  if (not (QuestionOuiNon(GetResourceString(rsEDITOR_SECTIONS_MSG_BEFORE_RESET_SECTION)))) then Exit;
  initSectionVierge();
  self.Serialiser();
  PageControl1.ActivePageIndex := 0;
end;

procedure TfrmEditeurSections.acOpenSectionExecute(Sender: TObject);
var
  QFileName: TStringDirectoryFileName;
begin
  QFileName := '';
  if (DoDialogOpenFilename('Fichiers texte|*.txt', QFileName)) then
  begin
    initSectionVierge();
    SynEdit1.Lines.LoadFromFile(QFileName);
    AnalyserEtTracerSection();
  end;
end;

procedure TfrmEditeurSections.acParserExecute(Sender: TObject);
begin
  AnalyserEtTracerSection();
  PageControl1.ActivePageIndex := 0;
end;

procedure TfrmEditeurSections.acQuitExecute(Sender: TObject);
begin
  self.Close;
end;

procedure TfrmEditeurSections.acSaveSectionExecute(Sender: TObject);
var
  QFilename: TStringDirectoryFileName;
begin
  QFilename := Format('Section_%d.txt', [FHerissonViseesRadiantes.getBaseStation().IDStation]);
  if (DoDialogSaveFilename('Fichiers texte|*.txt|Tous|*.*', '.txt', QFilename)) then
  begin
    Serialiser();
    SynEdit1.Lines.SaveToFile(QFilename);
  end;
end;

procedure TfrmEditeurSections.acSerialiserExecute(Sender: TObject);
begin
  self.Serialiser();
  PageControl1.ActivePageIndex := 1;
end;

procedure TfrmEditeurSections.Button3Click(Sender: TObject);
var
  LeHerisson: THerissonViseesRadiantes;
  n, i: Integer;
  BP: TBaseStation;
  VV: TViseeRayonnante;

  procedure fatche(const s: string);
  begin
    memoHerisson.Lines.Add(s);
  end;
begin
  LeHerisson := CadreSectionTransversale1.GetHerisson();
  memoHerisson.Lines.Clear;
  n := LeHerisson.getNbViseesRayonnantes();
  BP := LeHerisson.getBaseStation();
  fatche(Format('Basepoint: %d - %s',  [BP.IDStation, BP.IDTerrain]));
  fatche(Format('X = %.3f, Y = %.3f, Z = %.3f', [BP.PosStation.X, BP.PosStation.Y, BP.PosStation.Z]));
  fatche(format('Angle rotation de la section = %.2f', [LeHerisson.getAngleOrientationSection()]));
  fatche(format('====== %d ===========', [n]));
  if (n > 0) then
  begin
    for i := 0 to n - 1 do
    begin
      VV := LeHerisson.getViseeRayonnante(i);
      fatche(format('-- %d: %.3f, %.3f, %.3f', [i, VV.Longueur, VV.Azimut, VV.Pente]));
    end;
  end;
end;

procedure TfrmEditeurSections.AnalyserEtTracerSection();
var
  LS: TStringList;
  n, i, QNumeroLigneEnErreur: Integer;
  QMessageErreur: string;
begin
  LS := TStringList.Create;
  try
    LS.Clear;
    n := SynEdit1.Lines.Count;
    if (n = 0) then Exit;
    for i := 0 to n - 1 do LS.Add(SynEdit1.Lines[i]);
    FCurrentCrossSection.Analyser(LS, QNumeroLigneEnErreur, QMessageErreur);
    CadreSectionTransversale1.Redessin();
  finally
    LS.Free;
  end;
  PageControl1.ActivePageIndex := 0;
end;






function TfrmEditeurSections.initSectionVierge(): boolean;
var
  B: TBaseStation;
  LS: TStringList;
  n, i: Integer;
  w: Boolean;
begin
  result := false;
  B.IDStation := 10010020;
  B.IDTerrain := 'LLANFAIRPWLL';
  B.PosStation.X := 403460.00;
  B.PosStation.Y := 3089600.00;
  B.PosStation.Z := 540.00;


  FCurrentCrossSection.SetIDGroupe(editIdxGroupe.AsInteger);
  FCurrentCrossSection.SetDecalageXY(-23.25, 13.78);
  // on ajoute quelques objets ici
  //FCurrentCrossSection.addSimpleLigneByValues(nolFRACTURE, -12.255, -9.881, 11.022, 3.274);
  //FCurrentCrossSection.addTexteByValues(notCOTATION, 0, -8.47,  -5.81, GetLlanfairPG());
  w := CadreSectionTransversale1.CdrDrwInitialiser(FHerissonViseesRadiantes);
  CadreSectionTransversale1.SetSectionTransversale(FCurrentCrossSection);
  CadreSectionTransversale1.SetViewLimits(-10, -10, 10, 10);
  result := true;
end;




procedure TfrmEditeurSections.Serialiser();
var
  LS: TStringList;
  n, i: Integer;
begin
  SynEdit1.ClearAll;
  LS := TStringList.Create;
  LS.Clear;
  try
    FCurrentCrossSection.Serialiser(LS);
    n := LS.Count;
    for i := 0 to n - 1 do SynEdit1.Lines.add(LS.Strings[i]);
    LS.Clear;
  finally
    LS.Free;
  end;
end;










procedure TfrmEditeurSections.InitCaptionsMenus();
  procedure S777(const WU: TAction; const CaptionHint: string);
  begin
    WU.Caption  := GetResourceString(CaptionHint);
    WU.Hint     := GetResourceString(CaptionHint);
  end;
begin
  mnuFichier.Caption    := GetResourceString(rsMNU_FILE);
    S777(acNouvelleSection  , rsEDITOR_SECTIONS_MNU_NOUVEAU);
    S777(acOpenSection      , rsEDITOR_SECTIONS_MNU_OPEN);
    S777(acSaveSection      , rsEDITOR_SECTIONS_MNU_SAVE);
    S777(acSerialiser       , rsEDITOR_SECTIONS_MNU_SERIALISE);
    S777(acParser           , rsEDITOR_SECTIONS_MNU_PARSE);
    S777(acQuit             , rsEDITOR_SECTIONS_MNU_QUIT);
  mnuAide.Caption       := GetResourceString(rsMNU_AIDE);
    S777(acHelp             , rsEDITOR_SECTIONS_MNU_HELP);
end;

function TfrmEditeurSections.Initialiser(const FD: TDocumentDessin; const BP: TBaseStation; const QOffsetX, QOffsetY: double): boolean;
var
  EWE: TBaseStation;
  WU : TToporobotIDStation;
  Ang: float;

begin
  Result := False;


  AfficherMessage(Format('%s.Initialiser: %d', [ClassName, BP.IDStation]));
  FMyDocTopo := FD;
  FAutoCompletion         := TSynCompletion.Create(Application);
  FAutoCompletion.Editor  := SynEdit1;
  FAutoCompletion.ShortCut:= VK_F2;

  InitColorationSyntaxique();
  InitCaptionsMenus;
  FHerissonViseesRadiantes := THerissonViseesRadiantes.Create;
  FCurrentCrossSection     := TCrossSection.Create;
  try
    FHerissonViseesRadiantes.Initialiser(FD, BP, 0.00);
    FCurrentCrossSection.Initialiser(BP);
    FCurrentCrossSection.SetDecalageXY(QOffsetX, QOffsetY);
    editOffsetX.Value := QOffsetX;
    editOffsetY.Value := QOffsetY;

    EWE := FHerissonViseesRadiantes.getBaseStation();
    WU  := GetToporobotIDStation(EWE);
    Ang := FHerissonViseesRadiantes.getAngleOrientationSection();


    editAngleOrientation.Value := Ang;
    self.Caption := Format('Section transversale: %d.%d (%s) - Orientation: %.0f', [WU.aSerie, WU.aStation, WU.aIDTerrain, Ang]);
    ListerGroupes();
    self.Caption := GetResourceString(rsEDITOR_SECTIONS_MAIN_TITLE);
    editBasePointID.Text         := GetToporobotIDStationAsString(BP, false);
    lbBasePointIDTerrain.Caption := BP.IDTerrain;
    // autocompletion
    Result := true;
  except
  end;
end;
procedure TfrmEditeurSections.Finaliser;
begin
  AfficherMessage(Format('%s.Finaliser', [ClassName]));
  try
    FHerissonViseesRadiantes.Finaliser();
    FCurrentCrossSection.Finaliser();
  finally
    //FHerissonViseesRadiantes.Free;
    //FCurrentCrossSection.Free;
    //FAutoCompletion.Free;
  end;
end;

procedure TfrmEditeurSections.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := false;
  if (QuestionOuiNon(GetResourceString(rsEDITOR_SECTIONS_MSG_BEFORE_RESET_SECTION))) then CanClose := true;
end;


procedure TfrmEditeurSections.ListerGroupes;
var
  i, Nb: Integer;
  GRP: TGroupeEntites;
begin
  Nb := FMyDocTopo.GetNbGroupes();
  cmbGroupes.Clear;
  for i := 0 to Nb - 1 do
  begin
    GRP := FMyDocTopo.GetGroupeByIdx(i);
    cmbGroupes.Items.Add(Format('%d - %s', [GRP.IDGroupeEntites, GRP.NomGroupe]));
  end;
  cmbGroupes.ItemIndex := 0;
end;
end.


