unit dlgExportVersSIGExt;
// Export vers SIG et le logiciel GHCaveDraw
// 22/10/2013: Une polygonale GHCaveDraw est un fichier *.gcp au lieu de *.gcd
// 14/11/2013: Opérationnel
{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  UnitEntitesExtended,
  CallDialogsStdVersion,
  ConvertisseurJPC,
  Common,
  Classes, SysUtils, FileUtil, curredit, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Buttons, EditBtn, ComCtrls;
type TKeyValue = record
  K: string;
  V: string;
end;



type

  { TfrmExportVersSIGExt }

  TfrmExportVersSIGExt = class(TForm)
    BitBtn1: TBitBtn;
    btnProcess: TButton;
    chkSilhouette: TCheckBox;
    chkUseColorGroupes: TCheckBox;
    btnCouleurSilhouette: TColorButton;
    editFileName: TFileNameEdit;
    grbxSystemeCoordonnees: TGroupBox;
    GroupBox2: TGroupBox;
    lbFichier: TLabel;
    LbStepProcess: TLabel;
    lsbSystemesCoordonnees: TListBox;
    Panel1: TPanel;
    pnlProgression: TPanel;
    ProgressBar1: TProgressBar;
    rdgrpTarget: TRadioGroup;
    procedure btnCouleurSilhouetteClick(Sender: TObject);
    procedure btnProcessClick(Sender: TObject);
    procedure chkUseColorGroupesChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure rdgrpTargetClick(Sender: TObject);
  private
    FBDDEntites: TBDDEntites;
    FBddDemarreeDepuisFichier: boolean;
     // convertisseur EPSG
    FConvertisseurEPSG: TConversionSysteme;
    FKVA : array[0..2] of TKeyValue;
    procedure AfficherProgression(const Etape: string; const Done, Starting, Ending: integer);
    procedure SetExtensionFichier(const Idx: integer);
    function SetFileFilter(const Idx: integer): string;

    { private declarations }
  public
    { public declarations }
    function InitialiseByPointer(const QBDD: TBDDEntites; const QFiltres: string): boolean;
  end;

var
  frmExportVersSIGExt: TfrmExportVersSIGExt;

implementation

{$R *.lfm}
function TfrmExportVersSIGExt.InitialiseByPointer(const QBDD: TBDDEntites; const QFiltres: string): boolean;
var
  i: Integer;
  EWE: TLabelSystemesCoordsEPSG;
begin
  Result := False;
  FBddDemarreeDepuisFichier := false;
  FBDDEntites := QBDD;
  //-----------------------------
  AfficherMessage('Démarrage de l''exportateur GIS');
  // type de sortie
  FKVA[0].K := 'Google Earth KML'             ; FKVA[0].V := 'kml';
  FKVA[1].K := 'Standard GPX'                 ; FKVA[1].V := 'gpx';
  FKVA[2].K := 'OpenStreetMap Leaflet'        ; FKVA[2].V := 'htm';
  //FKVA[3].K := 'AutoCAD DXF'                  ; FKVA[3].V := 'dxf';
  //FKVA[4].K := 'Scalable Vector Graphics SVG' ; FKVA[4].V := 'svg';

  // système de coordonnées
  FConvertisseurEPSG := TConversionSysteme.Create;
  if (not FConvertisseurEPSG.Initialiser) then
  begin
    ShowMessage(GetResourceString(rsMSG_PROJ4S_FAIL));
    Exit(false);
  end;
  lsbSystemesCoordonnees.Clear;
  for i := 0 to FConvertisseurEPSG.GetNbSystemes - 1 do
  begin
    EWE := FConvertisseurEPSG.GetCodeEPSGNomSysteme(i);
    lsbSystemesCoordonnees.Items.Add(Format('EPSG:%d - %s', [EWE.CodeEPSG, EWE.NomEPSG]));
  end;
  lsbSystemesCoordonnees.ItemIndex := FConvertisseurEPSG.GetIndexSystemeByCodeEPSG(QBDD.GetCodeEPSG);
  try
    FBDDEntites.SetProcDisplayProgression(self.AfficherProgression);
    // remplissage des radiobox
    rdgrpTarget.Items.Clear;
    for i := 0 to High(FKVA) do rdgrpTarget.Items.Add(FKVA[i].K);
    rdgrpTarget.ItemIndex := 2; // pour GPX
    // nom du fichier de sortie
    editFileName.InitialDir := GetGHTopoDirectory();
    editFileName.FileName   := 'ExportGIS001.gpx';
    editFileName.Filter     := SetFileFilter(rdgrpTarget.ItemIndex);
    SetExtensionFichier(rdgrpTarget.ItemIndex);
    // OK ?
    Result := True;
  except
  end;
end;

procedure TfrmExportVersSIGExt.chkUseColorGroupesChange(Sender: TObject);
var
  EWE: Boolean;
begin
  EWE := not(chkUseColorGroupes.Checked);
  chkSilhouette.Enabled        := EWE;
  btnCouleurSilhouette.Enabled := EWE;
end;

procedure TfrmExportVersSIGExt.FormShow(Sender: TObject);
begin
  self.Caption                   := GetResourceString(rsEXPORT_SIG_TITRE);
  grbxSystemeCoordonnees.Caption := GetResourceString(rsEXPORT_SIG_SYST_COORDONNEES);
  rdgrpTarget.Caption            := GetResourceString(rsEXPORT_SIG_SIG_CIBLE);
  lbFichier.Caption              := GetResourceString(rsEXPORT_SIG_LB_FICHIER);
  chkUseColorGroupes.Caption     := GetResourceString(rsEXPORT_SIG_USE_COLORS);
  chkSilhouette.Caption          := GetResourceString(rsEXPORT_SIG_SILHOUETTE);
  btnProcess.Caption             := GetResourceString(rsEXPORT_SIG_BTN_GO);
end;

procedure TfrmExportVersSIGExt.rdgrpTargetClick(Sender: TObject);
begin
  SetExtensionFichier(rdgrpTarget.ItemIndex);
  SetFileFilter(rdgrpTarget.ItemIndex);
end;

procedure TfrmExportVersSIGExt.btnProcessClick(Sender: TObject);
var
  WU : TOutputFormatGIS;
  EWE: TLabelSystemesCoordsEPSG;
  qFilename: String;
begin
  Panel1.Enabled := False;
  WU := TOutputFormatGIS(rdgrpTarget.ItemIndex);
  try
    qFilename := trim(editFileName.Text);
    if (FileExists(qFilename)) then
    begin
      if (Not GHTopoQuestionOuiNon(Format('Le fichier %s existe - Ecraser ?', [qFilename]))) then Exit;
    end;
    EWE := FConvertisseurEPSG.GetCodeEPSGNomSysteme(lsbSystemesCoordonnees.ItemIndex);
    AfficherMessage(Format('Export SIG: %d - %s', [EWE.CodeEPSG, EWE.NomEPSG]));
    if (EWE.CodeEPSG = -1) then
    begin
      ShowMessage('Aucun système de coordonnées trouvé. Arrêt');
      exit;
    end;
    FBDDEntites.ExportForCarto(qFilename,
                               FConvertisseurEPSG,
                               EWE.CodeEPSG,
                               WU,
                               chkSilhouette.Checked,
                               '', //trim(editFiltres.Text),
                               '', //trim(editPrefixStations.Text),
                               btnCouleurSilhouette.ButtonColor,
                               False); //chkUseColorGroupes.Checked);
    LbStepProcess.Caption := Format('Done.  %d entities exported', [FBDDEntites.GetNbEntitesVisees()]);
    ProgressBar1.Position := 0;
  finally
    Panel1.Enabled := true;
  end;
end;

procedure TfrmExportVersSIGExt.btnCouleurSilhouetteClick(Sender: TObject);
begin

end;



function TfrmExportVersSIGExt.SetFileFilter(const Idx: integer): string;
const QFMT = '%s (*.%s)|*.%s';
begin
  Result := Format(QFMT, [FKVA[Idx].K, FKVA[Idx].V, FKVA[Idx].V]);
end;

procedure TfrmExportVersSIGExt.SetExtensionFichier(const Idx: integer);
begin
  AfficherMessage(Format('SetFilterFichier: %d', [Idx]));
  editFileName.Text := ChangeFileExt(editFileName.FileName, '.' + FKVA[Idx].V);
  with editFileName do
  begin
    Initialdir := ExtractFileDir(editFileName.Text);
    Filter     := SetFileFilter(rdgrpTarget.ItemIndex);
    DefaultExt := ChooseString(rdgrpTarget.ItemIndex,
                                [FKVA[0].V,
                                 FKVA[1].V,
                                 FKVA[2].V
                                ]);
  end;
end;
procedure TfrmExportVersSIGExt.AfficherProgression(const Etape: string; const Done, Starting, Ending: integer);
begin
  if (Done < 0) then Exit;
  try

    if ((Done mod 100 = 0)) then  // les appels
    begin
      if (Starting <> ProgressBar1.Min) then ProgressBar1.Min := Starting;
      if (Ending   <> ProgressBar1.Max) then ProgressBar1.Max := Ending;

      ProgressBar1.Position := Done;
      LbStepProcess.Caption := Etape;
      Application.ProcessMessages;
    end;
  except
    ;
  end;
end;

end.

