unit dlgStatistiquesExt;
// 09/06/2015: i18n + réactivation diagramme en barres
// 29/02/2016: Formattage OOo, inactivation des valeurs aberrantes, i18n, corrections
// 02/01/2017: Refonte du look, adaptation au 800x480

// TODO: Dans le dlg Statistiques, les visées de type 7 sont appelées 'visées de raccord'. Renommer la colonne correspondante
//       Corriger bug dans la ventilation des la spéléométrie

{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  ToporobotClasses2012,
  UnitEntitesExtended,
  CallDialogsStdVersion,
  Common,
  CadreHistoDirectionsExt,
  CadreHistoDepthExt,
  Classes, SysUtils, FileUtil, curredit, folderlister, SynEdit,
  SynPluginSyncroEdit, SpinEx, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls, Grids, ExtCtrls, Buttons, Types;

type

  { TfrmStatistiquesExt }

  TfrmStatistiquesExt = class(TForm)
    BitBtn1: TBitBtn;
    btnColorBarresFill: TColorButton;
    btnColorBarresLine: TColorButton;
    btnCopyTableauRecap: TButton;
    btnCopyTableauCoords: TButton;
    btnCopyTableauEntrees: TButton;
    btnCopyTableauStatsReseaux: TButton;
    btnRedrawDiagrams: TButton;
    Button1: TButton;
    btnExportRoseDiagramSVG: TButton;
    btnExportDepthDiagramSVG: TButton;
    CdrDepthDiagramExt1: TCdrDepthDiagramExt;
    CdrRoseDiagramExt1: TCdrRoseDiagramExt;
    cmbAffichageTableaux: TComboBox;
    cmbReseauOuSecteur: TComboBox;
    editFiltres: TEdit;
    editLongViseeMini: TCurrencyEdit;
    editNbPetales: TCurrencyEdit;
    editNbBarresDepth: TCurrencyEdit;
    grdSyntheseSpeleometrique: TStringGrid;
    lbNbStations: TLabel;
    Label2: TLabel;
    lbLongMiniVisee1: TLabel;
    lbNbBarres: TLabel;
    lbNbPetales: TLabel;
    lbNbReseauxSecteurs: TLabel;
    lbNbItems: TLabel;
    pagectlHistogrammes: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel5: TPanel;
    grdCoordsStations: TStringGrid;
    grdEntrances: TStringGrid;
    memoInfosBDD: TSynEdit;
    grdStatsReseauxSecteurs: TStringGrid;
    sclPourcentageSeuilHightLight: TScrollBar;
    TabCtrlStatistiques: TPageControl;
    tabshtRoseDiagram: TTabSheet;
    tabShtDepthHisto: TTabSheet;
    tabShtReseauSecteur: TTabSheet;
    tabShtEntrances: TTabSheet;
    tabShtSynthese: TTabSheet;
    tabShtCoordonnees: TTabSheet;
    tabShtDiagrams: TTabSheet;
    tabShtTableaux: TTabSheet;
    procedure btnColorBarresFillColorChanged(Sender: TObject);
    procedure btnColorBarresLineColorChanged(Sender: TObject);
    procedure btnCopyTableauEntreesClick(Sender: TObject);
    procedure btnCopyTableauRecapClick(Sender: TObject);
    procedure btnCopyTableauCoordsClick(Sender: TObject);
    procedure btnCopyTableauStatsReseauxClick(Sender: TObject);
    procedure btnExportDepthDiagramSVGClick(Sender: TObject);
    procedure btnRedrawDiagramsClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure btnExportRoseDiagramSVGClick(Sender: TObject);
    procedure cmbAffichageTableauxChange(Sender: TObject);
    procedure cmbReseauOuSecteurChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure grdSyntheseSpeleometriqueDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect; aState: TGridDrawState);
  private
    { private declarations }
    FTabSpeleoParReseaux: TArraySpeleometrieReseauOuSecteur;
    FTabSpeleoParSecteurs: TArraySpeleometrieReseauOuSecteur;
    FDocTopo   : TToporobotStructure2012;
    FBDDEntites: TBDDEntites;
    FBddDemarreeDepuisFichier : boolean;
    FOldFiltre : string;
    FModeAffichageTableau: TModeSelectionListe;
    function ListerCoordonneesStations(): boolean;
    procedure InitialiseStats();
    procedure AfficherInfosBDD();
    procedure RedessinerLesDiagrammes();
    procedure RefreshTableau(const QMode: TModeSelectionListe);
    procedure RefresTableauReseauxSecteurs(const QMode: integer);
  public
    function InitialiseByPointer(const QDTT: TToporobotStructure2012;
                                 const QBDD: TBDDEntites;
                                 const QFiltres: string): boolean;
    // initialisation depuis un fichier (FBDDEntites est local)
    function InitialiseByFichier(const QFileName: string; const QFiltres: string; const PositionPointZero: TPoint3Df): boolean;
    // libération du contexte
    procedure Finaliser();

  end;

var
  frmStatistiquesExt: TfrmStatistiquesExt;

implementation

{$R *.lfm}

{ TfrmStatistiquesExt }
procedure TfrmStatistiquesExt.InitialiseStats();
begin
  try
    AfficherMessage('Etablissement des statistiques');
    // setminmax
    FBDDEntites.SetMinMax(False);
    // recenser dates
    FBDDEntites.RecenserDates;
    // statistiques
    cmbReseauOuSecteur.ItemIndex := 0;
    FBDDEntites.CalculSpeleometrie;
    RefreshTableau(mslRESEAUX);

    FBDDEntites.CalcSpeleometrieParReseauxOuSecteurs(0, FTabSpeleoParReseaux);
    FBDDEntites.CalcSpeleometrieParReseauxOuSecteurs(1, FTabSpeleoParSecteurs);
    RefresTableauReseauxSecteurs(0);
    //RefresTableauReseauxSecteurs(1);
    // infos diverses
    AfficherInfosBDD();
    TabCtrlStatistiques.ActivePageIndex := 0;
    cmbAffichageTableaux.ItemIndex := 0;
  finally
  end;
end;

procedure TfrmStatistiquesExt.AfficherInfosBDD();
var
  LS: TStringList;
  i, Nb: Integer;
begin
  LS := TStringList.Create;
  try
    LS.Clear;
    FDocTopo.GetInfosCavite(LS);
    memoInfosBDD.Enabled := false;
    memoInfosBDD.Lines.Clear;
    Nb := LS.Count;
    for i := 0 to Nb - 1 do memoInfosBDD.Lines.Add(_AnsiToLCLStr(LS[i]));

    LS.Clear;
    memoInfosBDD.Enabled := True;
  finally
    LS.Free;
  end;
end;


procedure TfrmStatistiquesExt.cmbAffichageTableauxChange(Sender: TObject);
begin
  case cmbAffichageTableaux.ItemIndex of
    0: RefreshTableau(mslRESEAUX);
    1: RefreshTableau(mslSECTEURS);
    2: RefreshTableau(mslCODE);
    3: RefreshTableau(mslEXPE);
    4: RefreshTableau(mslDATE)
  else
    RefreshTableau(mslRESEAUX);
  end;
end;

procedure TfrmStatistiquesExt.cmbReseauOuSecteurChange(Sender: TObject);
begin
  RefresTableauReseauxSecteurs(cmbReseauOuSecteur.ItemIndex);
end;

procedure TfrmStatistiquesExt.FormCreate(Sender: TObject);
begin
  ;
end;

procedure TfrmStatistiquesExt.btnCopyTableauRecapClick(Sender: TObject);
begin
   GRDCCopierUnTableau(grdSyntheseSpeleometrique);
end;

procedure TfrmStatistiquesExt.btnCopyTableauStatsReseauxClick(Sender: TObject);
begin
  GRDCCopierUnTableau(grdStatsReseauxSecteurs);
end;

procedure TfrmStatistiquesExt.btnExportDepthDiagramSVGClick(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QIdxFilter: integer;
begin
  QFileName := GetGHTopoDirectory() + 'DepthDiagram1.svg';
  if (DoDialogSaveFile('Fichiers SVG (*.svg)|.svg', '.svg', QFileName, QIdxFilter)) then
    CdrDepthDiagramExt1.ExportDiagramToSVG(QFileName, 100.0);
end;

procedure TfrmStatistiquesExt.btnCopyTableauCoordsClick(Sender: TObject);
begin
  GRDCCopierUnTableau(grdCoordsStations);
end;


procedure TfrmStatistiquesExt.btnCopyTableauEntreesClick(Sender: TObject);
begin
  GRDCCopierUnTableau(grdEntrances);
end;


procedure TfrmStatistiquesExt.btnColorBarresFillColorChanged(Sender: TObject);
begin
  self.btnRedrawDiagramsClick(self);
end;

procedure TfrmStatistiquesExt.btnColorBarresLineColorChanged(Sender: TObject);
begin
  self.btnRedrawDiagramsClick(self);
end;


procedure TfrmStatistiquesExt.btnRedrawDiagramsClick(Sender: TObject);
begin
   CdrDepthDiagramExt1.Initialise(FBDDEntites,
                                 editFiltres.Text,
                                 editNbBarresDepth.AsInteger,
                                 editLongViseeMini.Value,
                                 btnColorBarresLine.ButtonColor,
                                 btnColorBarresFill.ButtonColor,
                                 sclPourcentageSeuilHightLight.Position / 100
                                );
   CdrRoseDiagramExt1.Initialise(FBDDEntites,
                                 editFiltres.Text,
                                 editNbPetales.AsInteger,
                                 editLongViseeMini.Value,
                                 btnColorBarresLine.ButtonColor,
                                 btnColorBarresFill.ButtonColor
                                );
  //*)
end;

procedure TfrmStatistiquesExt.Button1Click(Sender: TObject);
var
  i, Nb: Integer;
  NbEntreesTraitees: Integer;
  EWE: TEntrance;
  qr: Integer;
  WU: TEntiteEtendue;
begin
  Nb := FBDDEntites.GetNbEntrees();
  grdEntrances.RowCount := Nb + 1;
  grdEntrances.ColCount := 7;
  grdEntrances.Cells[0, 0] := 'No'      ; grdEntrances.ColWidths[0] :=  40;
  grdEntrances.Cells[1, 0] := 'Nom'     ; grdEntrances.ColWidths[1] := 160;
  grdEntrances.Cells[2, 0] := 'Station' ; grdEntrances.ColWidths[2] := 140;
  grdEntrances.Cells[3, 0] := 'X'       ; grdEntrances.ColWidths[3] := 160;
  grdEntrances.Cells[4, 0] := 'Y'       ; grdEntrances.ColWidths[4] := 160;
  grdEntrances.Cells[5, 0] := 'Z'       ; grdEntrances.ColWidths[5] := 100;
  grdEntrances.Cells[6, 0] := 'Observations'  ; grdEntrances.ColWidths[6] :=  666;

  // NOTA: Les entrées ont déjà été recensées lors du calcul

  for i := 0 to Nb - 1 do
  begin
    qr := i + 1;
    EWE := FBDDEntites.GetEntree(i);
    grdEntrances.Cells[0, qr] := format('%d', [qr]);
    grdEntrances.Cells[1, qr] := EWE.eNomEntree;
    grdEntrances.Cells[2, qr] := format('%d.%d', [EWE.eRefSer, EWE.eRefSt]);
    grdEntrances.Cells[3, qr] := FormatterNombreOOo(EWE.eXEntree);
    grdEntrances.Cells[4, qr] := FormatterNombreOOo(EWE.eYEntree);
    grdEntrances.Cells[5, qr] := FormatterNombreOOo(EWE.eZEntree);
    grdEntrances.Cells[6, qr] := EWE.eObserv;
  end;
end;

procedure TfrmStatistiquesExt.btnExportRoseDiagramSVGClick(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QIdxFilter: integer;
begin
  QFileName := GetGHTopoDirectory() + 'RoseDiagram1.svg';
  if (DoDialogSaveFile('Fichiers SVG (*.svg)|.svg', '.svg', QFileName, QIdxFilter)) then
    CdrRoseDiagramExt1.ExportDiagramToSVG(QFileName, 100.0);
end;


function TfrmStatistiquesExt.InitialiseByPointer(const QDTT: TToporobotStructure2012;
                                                 const QBDD: TBDDEntites;
                                                 const QFiltres: string): boolean;
var
  QDevelViseesVisibles: double;
begin
  AfficherMessage('-- InitialiseByPointer');
  Result := False;
  // d'abord, sauvegarder le filtre courant
  FOldFiltre := QFiltres;
  FBddDemarreeDepuisFichier := false;
  FDocTopo    := QDTT;
  FBDDEntites := QBDD;
  try
    FBDDEntites.MetaFiltre(QFiltres, QDevelViseesVisibles);
    InitialiseStats();
    Result := True;
  finally
  end;

end;

function TfrmStatistiquesExt.InitialiseByFichier(const QFileName: string; const QFiltres: string; const PositionPointZero: TPoint3Df): boolean;
begin
  Result := False;
  // d'abord, sauvegarder le filtre courant
  FOldFiltre := QFiltres;
  FBddDemarreeDepuisFichier := True;
  FBDDEntites := TBDDEntites.Create;
  try
   FBDDEntites.Initialiser(PositionPointZero);
   Result := FBDDEntites.LoadFromFile(QFileName);
  except
  end;
end;

procedure TfrmStatistiquesExt.Finaliser();
var
  QDevelViseesVisibles: double;
begin
  if (FBddDemarreeDepuisFichier) then
  begin
    try
      FBDDEntites.Finaliser();
    finally
      FBDDEntites.Free;
    end;
  end
  else
  begin
    FBDDEntites.MetaFiltre(FOldFiltre, QDevelViseesVisibles);   // restauration du filtre
  end;
end;
procedure TfrmStatistiquesExt.RefreshTableau(const QMode: TModeSelectionListe);
const
  NB_COLONNES = 11;
var
  i, Nb: integer;
  Totaux: TVentilationSpeleometrie;
  procedure AddValeursSpeleometrie(const QQ: TVentilationSpeleometrie);
  begin
    Totaux.Ennoyables += QQ.Ennoyables;
    Totaux.Filons     += QQ.Filons;
    Totaux.Fossiles   += QQ.Fossiles;
    Totaux.Siphons    += QQ.Siphons;
    Totaux.Speciaux   += QQ.Speciaux;
    Totaux.Tunnels    += QQ.Tunnels;
    Totaux.Vadoses    += QQ.Vadoses;
  end;

  procedure WriteHeader();
  const
    WU = 110;
  begin
    with grdSyntheseSpeleometrique do
    begin
      ColWidths[0] := 300;
      case QMode of
        mslRESEAUX   : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE0);
        mslSECTEURS  : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE1);
        mslCODE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE2);
        mslEXPE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE3);
        mslDATE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE4);
      end;
      Cells[1, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_FOSSILES)    ; ColWidths[1] := WU;
      Cells[2, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_VADOSES)     ; ColWidths[2] := WU;
      Cells[3, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_ENNOYABLES)  ; ColWidths[3] := WU;
      Cells[4, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_SIPHONS)     ; ColWidths[4] := WU;
      Cells[5, 0] := '';                 ColWidths[5] := 10;
      Cells[6, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_NATURELS)    ; ColWidths[6] := WU + 10;
      Cells[7, 0] := '';                 ColWidths[7] := 10;
      Cells[8, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_SPECIAUX)    ; ColWidths[8] := WU;
      Cells[9, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_TUNNELS)     ; ColWidths[9] := WU;
      Cells[10,0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_MINES)       ; ColWidths[10] := WU;
    end;
  end;
  procedure WriteLigne(const Idx: integer);
  var
    Q: integer;
    SR: TVentilationSpeleometrie;
    qR: TReseau;
    qS: TSecteur;
    qC: TCode;
    qE: TExpe;
    qD: TDateTime;
  begin
    Q := 1 + Idx;

    case FModeAffichageTableau of
      mslRESEAUX:
        begin
           qR := FBDDEntites.GetReseau(Idx);
           grdSyntheseSpeleometrique.Cells[0, Q] := GetResourceString(qR.NomReseau);
           SR := FBDDEntites.GetSpeleometrieParReseauxByIdx(Idx);
         end;
      mslSECTEURS:
        begin
          qS := FBDDEntites.GetSecteur(Idx);
          grdSyntheseSpeleometrique.Cells[0, Q] := GetResourceString(qS.NomSecteur);
          SR := FBDDEntites.GetSpeleometrieParSecteurByIdx(Idx);
        end;
      mslCODE:
        begin
          qC := FBDDEntites.GetCode(Idx);
          grdSyntheseSpeleometrique.Cells[0, Q] := GetResourceString(qC.Commentaire);
          SR := FBDDEntites.GetSpeleometrieParCodeByIdx(Idx);
        end;
      mslEXPE:
        begin
          qE := FBDDEntites.GetExpe(Idx);
          grdSyntheseSpeleometrique.Cells[0, Q] := Format('%d - %s', [qE.IDExpe, GetResourceString(qE.Commentaire)]);
          SR := FBDDEntites.GetSpeleometrieParExpeByIdx(Idx);
        end;
      mslDATE:
        begin
          qD := FBDDEntites.GetUneDate(Idx);
          grdSyntheseSpeleometrique.Cells[0, Q] := Format('%s', [DateToStr(qD)]);
          SR := FBDDEntites.GetSpeleometrieParDateByIdx(Idx);
        end;
    end;
    AddValeursSpeleometrie(SR);     // calcul du total
    grdSyntheseSpeleometrique.Cells[1, Q] := FormatterNombreOOo(SR.Fossiles);
    grdSyntheseSpeleometrique.Cells[2, Q] := FormatterNombreOOo(SR.Vadoses);
    grdSyntheseSpeleometrique.Cells[3, Q] := FormatterNombreOOo(SR.Ennoyables);
    grdSyntheseSpeleometrique.Cells[4, Q] := FormatterNombreOOo(SR.Siphons);
    grdSyntheseSpeleometrique.Cells[5, Q] := '';
    grdSyntheseSpeleometrique.Cells[6, Q] := FormatterNombreOOo(SR.Fossiles + SR.Vadoses + SR.Ennoyables + SR.Siphons);
    grdSyntheseSpeleometrique.Cells[7, Q] := '';
    grdSyntheseSpeleometrique.Cells[8, Q] := FormatterNombreOOo(SR.Speciaux);
    grdSyntheseSpeleometrique.Cells[9, Q] := FormatterNombreOOo(SR.Tunnels);
    grdSyntheseSpeleometrique.Cells[10,Q] := FormatterNombreOOo(SR.Filons);
  end;
  procedure WriteFooter();
  var
    Q: Integer;
  begin
    Q := grdSyntheseSpeleometrique.RowCount - 2;
    grdSyntheseSpeleometrique.Cells[0, Q] := '';
    grdSyntheseSpeleometrique.Cells[1, Q] := FormatterNombreOOo(Totaux.Fossiles); //format('%.2f', [Totaux.Fossiles]);//
    grdSyntheseSpeleometrique.Cells[2, Q] := FormatterNombreOOo(Totaux.Vadoses);
    grdSyntheseSpeleometrique.Cells[3, Q] := FormatterNombreOOo(Totaux.Ennoyables);
    grdSyntheseSpeleometrique.Cells[4, Q] := FormatterNombreOOo(Totaux.Siphons);
    grdSyntheseSpeleometrique.Cells[5, Q] := '';
    grdSyntheseSpeleometrique.Cells[6, Q] := FormatterNombreOOo(Totaux.Fossiles + Totaux.Vadoses + Totaux.Ennoyables + Totaux.Siphons);
    grdSyntheseSpeleometrique.Cells[7, Q] := '';
    grdSyntheseSpeleometrique.Cells[8, Q] := FormatterNombreOOo(Totaux.Speciaux);
    grdSyntheseSpeleometrique.Cells[9, Q] := FormatterNombreOOo(Totaux.Tunnels);
    grdSyntheseSpeleometrique.Cells[10,Q] := FormatterNombreOOo(Totaux.Filons);

    Q := grdSyntheseSpeleometrique.RowCount - 1;
    grdSyntheseSpeleometrique.Cells[0, Q] := 'Total general';
    grdSyntheseSpeleometrique.Cells[1, Q] := 'Naturels';
    grdSyntheseSpeleometrique.Cells[2, Q] := FormatterNombreOOo(Totaux.Fossiles + Totaux.Vadoses + Totaux.Ennoyables + Totaux.Siphons); //format('%.2f', [Totaux.Fossiles]);//
    grdSyntheseSpeleometrique.Cells[3, Q] := 'Total';
    grdSyntheseSpeleometrique.Cells[4, Q] := FormatterNombreOOo(Totaux.Fossiles + Totaux.Vadoses + Totaux.Ennoyables + Totaux.Siphons + Totaux.Speciaux); //format('%.2f', [Totaux.Fossiles]);//


  end;
begin
  FModeAffichageTableau := QMode;
  Totaux.Ennoyables := 0.00;
  Totaux.Filons     := 0.00;
  Totaux.Fossiles   := 0.00;
  Totaux.Siphons    := 0.00;
  Totaux.Speciaux   := 0.00;
  Totaux.Tunnels    := 0.00;
  Totaux.Vadoses    := 0.00;
  case FModeAffichageTableau of
    mslRESEAUX  : Nb := FBDDEntites.GetNbReseaux();
    mslSECTEURS : Nb := FBDDEntites.GetNbSecteurs();
    mslCODE     : Nb := FBDDEntites.GetNbCodes();
    mslEXPE     : Nb := FBDDEntites.GetNbExpes();
    mslDATE     : Nb := FBDDEntites.GetNbDates();
  end;
  case FModeAffichageTableau of
    mslRESEAUX  : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_RESEAUX)  ,[Nb]);
    mslSECTEURS : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_SECTEURS) ,[Nb]);
    mslCODE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_CODES)    ,[Nb]);
    mslEXPE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_SEANCES)  ,[Nb]);
    mslDATE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_DATES)    ,[Nb]);
  end;


  grdSyntheseSpeleometrique.ColCount := NB_COLONNES;
  grdSyntheseSpeleometrique.RowCount := 1 + Nb + 1 + 1 + 1;
  grdSyntheseSpeleometrique.Clean;
  WriteHeader();
  for i := 0 to Nb -1 do
  begin
    WriteLigne(i);
  end;
  WriteFooter();
end;



//******************************************************************************
procedure TfrmStatistiquesExt.FormShow(Sender: TObject);
begin

  self.Caption              := GetResourceString(rsSTATISTIQUES_TITRE);
  tabShtTableaux.Caption    := GetResourceString(rsSTATISTIQUES_TAB_TABLEAUX);
  tabShtDiagrams.Caption    := GetResourceString(rsSTATISTIQUES_TAB_DIAGRAMMES);
  tabShtSynthese.Caption    := GetResourceString(rsSTATISTIQUES_TAB_SYNTHESE);
  tabShtReseauSecteur.Caption := GetResourceString(rsSTATISTIQUES_TAB_RESEAU_SECTEURS);
  tabShtCoordonnees.Caption := GetResourceString(rsSTATISTIQUES_TAB_CORDONNEES);

  editLongViseeMini.Value     := 0.40;
  editNbPetales.AsInteger     := 40;
  editNbBarresDepth.AsInteger := 40;
  sclPourcentageSeuilHightLight.Position := 60;

  cmbAffichageTableaux.Clear;
  cmbAffichageTableaux.Style := csDropDownList;
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE0));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE1));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE2));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE3));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE4));
  cmbAffichageTableaux.ItemIndex := 0;
  tabshtRoseDiagram.Caption     := GetResourceString(rsSTATISTIQUES_LBL_TITRE_ROSE_DIAGRAM);
  tabShtDepthHisto.Caption      := GetResourceString(rsSTATISTIQUES_LBL_TITRE_DEPTH_DIAGRAM);
  lbLongMiniVisee1.Caption      := GetResourceString(rsSTATISTIQUES_LBL_LONG_MINI_VISEE);
  lbNbPetales.Caption           := GetResourceString(rsSTATISTIQUES_LBL_NB_PETALES);
  lbNbBarres.Caption            := GetResourceString(rsSTATISTIQUES_LBL_NB_BARRES);
  btnRedrawDiagrams.Caption  := GetResourceString(rsSTATISTIQUES_BTN_REDESSINER);


  btnCopyTableauRecap.Caption        := GetResourceString(rsBTN_COPIER_TABLEAU);
  btnCopyTableauCoords.Caption       := GetResourceString(rsBTN_COPIER_TABLEAU);
  btnCopyTableauStatsReseaux.Caption := GetResourceString(rsBTN_COPIER_TABLEAU);
  // charger fichier de coordonnees;
  tabShtCoordonnees.Visible := ListerCoordonneesStations();
  // onglets visibles dans le mode Gros Minet
  tabShtReseauSecteur.Visible := true;

  {$ifdef GROS_MINET}
  tabShtReseauSecteur.Visible := false;

  {$endif}


  // affichage des diagrammes
  RedessinerLesDiagrammes;
end;

procedure TfrmStatistiquesExt.grdSyntheseSpeleometriqueDrawCell(Sender: TObject; aCol, aRow: Integer; aRect: TRect; aState: TGridDrawState);
var
  n, HauteurLigne: LongInt;
  EWE: TColor;
  Nb: Integer;
  // Alignment: idem clavier numérique
  procedure DessinerCell(const BG: TColor; const S: string; const Alignment: byte);
  var
    RT: TSize;
    QX, QY: LongInt;
  begin
    grdSyntheseSpeleometrique.Canvas.Brush.Color := BG;
    grdSyntheseSpeleometrique.Canvas.FillRect(aRect);
    RT := grdSyntheseSpeleometrique.Canvas.TextExtent(S);
    case Alignment of
      1, 4, 7: QX := aRect.Left + 6;
      2, 5, 8: QX := (aRect.Left + aRect.Right) div 2 - RT.Width div 2;
      3, 6, 9: QX := aRect.Right - RT.Width - 6;
    end;
    case Alignment of
      1, 2, 3: QY := (aRect.Bottom - RT.Height) - 2;
      4, 5, 6: QY := (aRect.Top + aRect.Bottom) div 2 - RT.Height div 2;
      7, 8, 9: QY :=  aRect.Top + 2;
    end;
    grdSyntheseSpeleometrique.Canvas.TextOut(QX, QY, S);
  end;

begin
  HauteurLigne := grdSyntheseSpeleometrique.RowHeights[0];
  grdSyntheseSpeleometrique.Canvas.Brush.Color := clCream;
  if ((aRow = 0) or (aCol = 0)) then
  begin
    DessinerCell(clSilver, grdSyntheseSpeleometrique.Cells[aCol, aRow], 4);
    exit;
  end;
  if (aCol = 7) then
  begin
    DessinerCell(clAqua, grdSyntheseSpeleometrique.Cells[aCol, aRow], 5);
    exit;
  end;
  // séparateur
  Nb := 0;
  case FModeAffichageTableau of
    mslENTRANCES           : Nb := FBDDEntites.GetNbEntrees();
    mslRESEAUX             : Nb := FBDDEntites.GetNbReseaux();
    mslSECTEURS            : Nb := FBDDEntites.GetNbSecteurs();
    mslCODE                : Nb := FBDDEntites.GetNbCodes();
    mslEXPE                : Nb := FBDDEntites.GetNbExpes();
    mslDATE                : Nb := FBDDEntites.GetNbDates();
  end;
  //*)
  if (aRow = Nb + 1) then
  begin
    grdSyntheseSpeleometrique.RowHeights[aRow] := 10;
    DessinerCell(clRed, grdSyntheseSpeleometrique.Cells[aCol, aRow], 5);
    exit;
  end;

  EWE := clCream;
  grdSyntheseSpeleometrique.RowHeights[aRow] := HauteurLigne;
  if (gdSelected in aState) then EWE := clBlue;
  DessinerCell(EWE, grdSyntheseSpeleometrique.Cells[aCol, aRow], 6);
end;


procedure TfrmStatistiquesExt.RedessinerLesDiagrammes;
begin
  CdrRoseDiagramExt1.Initialise(FBDDEntites,
                                editFiltres.Text,
                                editNbPetales.AsInteger,
                                editLongViseeMini.Value,
                                btnColorBarresLine.ButtonColor,
                                btnColorBarresFill.ButtonColor
                                );
  CdrDepthDiagramExt1.Initialise(FBDDEntites,
                                 editFiltres.Text,
                                 editNbBarresDepth.AsInteger,
                                 editLongViseeMini.Value,
                                 btnColorBarresLine.ButtonColor,
                                 btnColorBarresFill.ButtonColor,
                                 sclPourcentageSeuilHightLight.Position / 100.0
                                );
end;
// lister coordonnées stations
function TfrmStatistiquesExt.ListerCoordonneesStations(): boolean;
const
  COL_MESURES     = 4;
  COL_ID_LITTERAL = 11;
  COL_COORDS_X    = 12;
  COL_COMMENTAIRE = 15;
var
  i, Nb: Integer;
  EWE  : TEntiteEtendue;
  NbSts: Integer;
  c    : Integer;
begin
  Result := False;
  AfficherMessage(Format('%s.ListeCoordonneesStations(): %d entites', [ClassName, Nb]));
  //try
    Nb := FBDDEntites.GetNbEntitesVisees();
    // trier le tableau par séries/stations
    FBDDEntites.SortBySerSts;
    NbSts       := 0;
    grdCoordsStations.Visible := False;
    with grdCoordsStations do
    begin
      RowCount := 1;
      ColCount := 13;
      Cells[0, 0]   := 'Station';  ColWidths[0] := 120;
      Cells[1, 0]   := 'ID';       ColWidths[1] := 110;
      Cells[2, 0]   := 'Long';     ColWidths[2] := 110;
      Cells[3, 0]   := 'Azimut';   ColWidths[3] := 110;
      Cells[4, 0]   := 'Pente';    ColWidths[4] := 110;
      Cells[5, 0]   := 'LG';       ColWidths[5] := 110;
      Cells[6, 0]   := 'LD';       ColWidths[6] := 110;
      Cells[7, 0]   := 'HZ';       ColWidths[7] := 110;
      Cells[8, 0]   := 'HN';       ColWidths[8] := 110;
      for c := 1 to 3 do
      begin
        Cells[8 + c, 0] := chr(87 + c);
        ColWidths[8 + c] := 110;
      end;
      Cells[12, 0]   := 'Observ';       ColWidths[12] := 300;
    end;
    for i := 0 to Nb - 1 do
    begin
      EWE := FBDDEntites.GetEntiteVisee(i);
      if (IsViseeInSerie(EWE)) then
      begin
        Inc(NbSts);
        grdCoordsStations.RowCount := 1 + NbSts;
        grdCoordsStations.Cells[0, NbSts] := Format('%d.%d', [EWE.Entite_Serie, EWE.Entite_Station]);
        grdCoordsStations.Cells[1, NbSts] := EWE.oIDLitteral;
        //AfficherMessageErreur(Format('%d.%d - %.3f %.3f %.3f', [EWE.Entite_Serie, EWE.Entite_Station, EWE.oLongueur, EWE.oAzimut, EWE.oPente]));

        grdCoordsStations.Cells[2, NbSts] := FormatterNombreOOo(EWE.oLongueur, 3, False);
        grdCoordsStations.Cells[3, NbSts] := FormatterNombreOOo(EWE.oAzimut, 3, False);
        grdCoordsStations.Cells[4, NbSts] := FormatterNombreOOo(EWE.oPente, 3, False);

        grdCoordsStations.Cells[5, NbSts] := FormatterNombreOOo(EWE.oLG, 3, False);
        grdCoordsStations.Cells[6, NbSts] := FormatterNombreOOo(EWE.oLD, 3, False);
        grdCoordsStations.Cells[7, NbSts] := FormatterNombreOOo(EWE.oHZ, 3, False);
        grdCoordsStations.Cells[8, NbSts] := FormatterNombreOOo(EWE.oHN, 3, False);

        grdCoordsStations.Cells[ 9, NbSts] := FormatterNombreOOo(EWE.Une_Station_2_X, 3, False);
        grdCoordsStations.Cells[10, NbSts] := FormatterNombreOOo(EWE.Une_Station_2_Y, 3, False);
        grdCoordsStations.Cells[11, NbSts] := FormatterNombreOOo(EWE.Une_Station_2_Z, 3, False);
        grdCoordsStations.Cells[12, NbSts] := _AnsiToLCLStr(EWE.oCommentaires);
      end;
    end;
    lbNbStations.caption := Format('Coordonnées des %d stations', [NbSts]);
    grdCoordsStations.Visible := True;
    Result := (NbSts > 0);
  //finally

  //end;
end;
procedure TfrmStatistiquesExt.RefresTableauReseauxSecteurs(const QMode: integer);
var
  i, Nb: Integer;
  EWE: TSpeleometrieReseauOuSecteur;
  WU: Boolean;
  procedure S666Str(const QCol, QRow: integer; const QValid: boolean; const QValeur: string);
  begin
    grdStatsReseauxSecteurs.Cells[QCol, QRow] := IIF(QValid, QValeur, '');
  end;
  procedure S666Int(const QCol, QRow: integer; const QValid: boolean; const QValeur: integer);
  begin
    grdStatsReseauxSecteurs.Cells[QCol, QRow] := IIF(QValid, Format('%d', [QValeur]), '');
  end;
  procedure S666Float(const QCol, QRow: integer; const QValid: boolean; const QValeur: double);
  begin
    //grdStatsReseauxSecteurs.Cells[QCol, QRow] := IIF(QValid, Format('%.2f', [QValeur]), '');
    grdStatsReseauxSecteurs.Cells[QCol, QRow] := IIF(QValid, FormatterNombreOOo(QValeur, 3, false), '');

  end;

begin
  case QMode of
    0: begin
         Nb := 1 + High(FTabSpeleoParReseaux);
         lbNbReseauxSecteurs.caption := Format('%d réseaux', [Nb]);
       end;
    1: begin
         Nb := 1 + High(FTabSpeleoParSecteurs);
         lbNbReseauxSecteurs.caption := Format('%d secteurs', [Nb]);
       end;
  end;
  grdStatsReseauxSecteurs.RowCount := Nb + 1;
  grdStatsReseauxSecteurs.ColCount := 13;
  grdStatsReseauxSecteurs.ColWidths[0] := 40;
  for i := 1 to grdStatsReseauxSecteurs.ColCount - 1 do grdStatsReseauxSecteurs.ColWidths[i] := 110;

  S666Str(0, 0, True, 'ID' );
  S666Str(1, 0, True, 'Nom');
  grdStatsReseauxSecteurs.ColWidths[1] := 320;
  S666Str(2, 0, True, 'Dev');
  S666Str(3, 0, True, 'X Min');
  S666Str(4, 0, True, 'X Max');
  S666Str(5, 0, True, 'Y Min');
  S666Str(6, 0, True, 'Y Max');
  S666Str(7, 0, True, 'Z Min');
  S666Str(8, 0, True, 'Z Max');
  S666Str(9, 0, True, '|<- X ->|');
  S666Str(10, 0, True, '|<- Y ->|');
  S666Str(11, 0, True, '|<- Z ->|');
  S666Str(12, 0, True, 'Nb Sts');
  for i := 0 to Nb - 1 do
  begin
    case QMode of
      0: EWE := FTabSpeleoParReseaux[i];
      1: EWE := FTabSpeleoParSecteurs[i];
    end;
    WU := EWE.DonneesValides;
    S666Int(0,  i+1, True, EWE.IDReseauSecteur);                 // toujours affiché
    S666Str(1,  i+1, True, _AnsiToLCLStr(EWE.NomReseauSecteur)); // toujours affiché
    S666Float(2,  i+1, WU, EWE.Developpement);
    S666Float(3,  i+1, WU, EWE.CoordMini.X);
    S666Float(4,  i+1, WU, EWE.CoordMaxi.X);
    S666Float(5,  i+1, WU, EWE.CoordMini.Y);
    S666Float(6,  i+1, WU, EWE.CoordMaxi.Y);
    S666Float(7,  i+1, WU, EWE.CoordMini.Z);
    S666Float(8,  i+1, WU, EWE.CoordMaxi.Z);
    S666Float(9,  i+1, WU, EWE.Etendue.X);
    S666Float(10, i+1, WU, EWE.Etendue.Y);
    S666Float(11, i+1, WU, EWE.Etendue.Z);
    S666Int(12, i+1, WU, EWE.NbVisees);
  end;
end;
end.

