unit dlgstatistiquesext2;

{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees, UnitEntitesExtended, Common, CadreHistoDirectionsExt,
  CadreHistoDepthExt, CadreHistoDepth, Classes, SysUtils, FileUtil, curredit,
  SynEdit, SynMemo, SynPluginSyncroEdit, TAGraph, TASeries, Forms, Controls,
  Graphics, Dialogs, ComCtrls, StdCtrls, Grids, ExtCtrls, Buttons;

type

  { TfrmStatistiquesExt2 }

  TfrmStatistiquesExt2 = class(TForm)
    BitBtn1: TBitBtn;
    btnColorBarresFillDepth: TColorButton;
    btnColorBarresLineDepth: TColorButton;
    btnColorPetalesFill: TColorButton;
    btnColorPetalesLine: TColorButton;
    btnCopyTableauRecap: TButton;
    btnCopyTableauCoords: TButton;
    btnRedrawRoseDiagram: TButton;
    Button4: TButton;
    CdrHistoAltitudesExt1: TCdrHistoAltitudesExt;
    CdrRoseDiagramExt1: TCdrRoseDiagramExt;
    Chart1: TChart;
    Chart1BarSeries1: TBarSeries;
    cmbAffichageTableaux: TComboBox;
    editFiltresDepth: TEdit;
    editLongViseeMini: TCurrencyEdit;
    editLongViseeMiniDepth: TCurrencyEdit;
    editNbPetales: TCurrencyEdit;
    editNbPetalesDepth: TCurrencyEdit;
    editRoseFiltres: TEdit;
    grdSyntheseSpeleometrique: TStringGrid;
    Label10: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lbNbItems: TLabel;
    lbTitreDepthDiagramme: TStaticText;
    lbTitreRoseDiagramme: TStaticText;
    memoSynthese: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    grdCoordsStations: TStringGrid;
    TabCtrlStatistiques: TPageControl;
    tabShtCoordonnees: TTabSheet;
    tabShtDiagrams: TTabSheet;
    tabShtTableaux: TTabSheet;
    tabShtSynthese: TTabSheet;
    procedure btnColorPetalesLineClick(Sender: TObject);
    procedure btnCopyTableauRecapClick(Sender: TObject);
    procedure btnCopyTableauCoordsClick(Sender: TObject);
    procedure btnRedrawRoseDiagramClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure cmbAffichageTableauxChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { private declarations }
    FBDDEntites: TBDDEntites;
    FBddDemarreeDepuisFichier : boolean;
    function ChargerFichierCoordonnees(const FichierCoords: string): boolean;
    procedure InitialiseStats;
    procedure RedessinerLesDiagrammes;
    procedure RefreshTableau(const Mode: TModeSelectionListe);
  public
    function InitialiseByPointer(const QBDD: TBDDEntites; const QFiltres: string): boolean;
    // initialisation depuis un fichier (FBDDEntites est local)
    function InitialiseByFichier(const QFileName: string; const QFiltres: string): boolean;
    // libération du contexte
    procedure Finaliser;

  end;

var
  frmStatistiquesExt: TfrmStatistiquesExt2;

implementation

{$R *.lfm}

{ TfrmStatistiquesExt2 }
procedure TfrmStatistiquesExt2.InitialiseStats;
begin
  try
    // setminmax
    FBDDEntites.SetMinMax(False);
    // recenser dates
    FBDDEntites.RecenserDates;
    // statistiques
    FBDDEntites.CalculSpeleometrie;
    RefreshTableau(mslRESEAUX);
  finally
  end;
end;


procedure TfrmStatistiquesExt2.cmbAffichageTableauxChange(Sender: TObject);
begin
  case cmbAffichageTableaux.ItemIndex of
    0: RefreshTableau(mslRESEAUX);
    1: RefreshTableau(mslSECTEURS);
    2: RefreshTableau(mslCODE);
    3: RefreshTableau(mslEXPE);
    4: RefreshTableau(mslDATE)
  else
    RefreshTableau(mslRESEAUX);
  end;
end;

procedure TfrmStatistiquesExt2.FormCreate(Sender: TObject);
begin
  ;
end;

procedure TfrmStatistiquesExt2.btnCopyTableauRecapClick(Sender: TObject);
begin
   GRDCCopierUnTableau(grdSyntheseSpeleometrique);
end;

procedure TfrmStatistiquesExt2.btnCopyTableauCoordsClick(Sender: TObject);
begin
  GRDCCopierUnTableau(grdCoordsStations);
end;

procedure TfrmStatistiquesExt2.btnColorPetalesLineClick(Sender: TObject);
begin

end;


procedure TfrmStatistiquesExt2.btnRedrawRoseDiagramClick(Sender: TObject);
begin
   CdrRoseDiagramExt1.Initialise(FBDDEntites,
                                editRoseFiltres.Text,
                                editNbPetales.AsInteger,
                                editLongViseeMini.Value,
                                btnColorPetalesLine.ButtonColor,
                                btnColorPetalesFill.ButtonColor
                               );
end;

procedure TfrmStatistiquesExt2.Button4Click(Sender: TObject);
begin
   CdrHistoAltitudesExt1.Initialise(FBDDEntites,
                                 editFiltresDepth.Text,
                                 editNbPetalesDepth.AsInteger,
                                 editLongViseeMiniDepth.Value,
                                 btnColorBarresLineDepth.ButtonColor,
                                 btnColorBarresFillDepth.ButtonColor
                                );
end;


function TfrmStatistiquesExt2.InitialiseByPointer(const QBDD: TBDDEntites; const QFiltres: string): boolean;
begin
  AfficherMessage('-- InitialiseByPointer');
  Result := False;
  FBddDemarreeDepuisFichier := false;
  FBDDEntites := QBDD;
  try
    FBDDEntites.MetaFiltre(QFiltres);
    InitialiseStats;
    Result := True;
  finally
  end;

end;

function TfrmStatistiquesExt2.InitialiseByFichier(const QFileName: string; const QFiltres: string): boolean;
begin
  Result := False;
  FBddDemarreeDepuisFichier := True;
  FBDDEntites := TBDDEntites.Create;
  try
   FBDDEntites.Initialise;
   Result := FBDDEntites.LoadFromFile(QFileName);
  except
  end;

end;

procedure TfrmStatistiquesExt2.Finaliser;
begin
  if (FBddDemarreeDepuisFichier) then
  begin
    try
      FBDDEntites.Finalise;
    finally
      FBDDEntites.Free;
    end;
  end;
end;
procedure TfrmStatistiquesExt2.RefreshTableau(const Mode: TModeSelectionListe);
const
  NB_COLONNES = 11;
var
  i, Nb: integer;
  procedure WriteHeader(const QMode: TModeSelectionListe);
  const
    WU = 100;
  begin
    with grdSyntheseSpeleometrique do
    begin
      ColWidths[0] := 400;
      case QMode of
        mslRESEAUX   : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE0);
        mslSECTEURS  : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE1);
        mslCODE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE2);
        mslEXPE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE3);
        mslDATE      : Cells[0, 0] := GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE4);
      end;
      Cells[1, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_FOSSILES)    ; ColWidths[1] := WU;
      Cells[2, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_VADOSES)     ; ColWidths[2] := WU;
      Cells[3, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_ENNOYABLES)  ; ColWidths[3] := WU;
      Cells[4, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_SIPHONS)     ; ColWidths[4] := WU;
      Cells[5, 0] := '';                 ColWidths[5] := 10;
      Cells[6, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_NATURELS)    ; ColWidths[6] := WU + 10;
      Cells[7, 0] := '';                 ColWidths[7] := 10;
      Cells[8, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_SPECIAUX)    ; ColWidths[8] := WU;
      Cells[9, 0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_TUNNELS)     ; ColWidths[9] := WU;
      Cells[10,0] := GetResourceString(rsSTATISTIQUES_TYPE_SHOT_MINES)       ; ColWidths[10] := WU;
    end;
  end;
  procedure WriteLigne(const QMode: TModeSelectionListe; const Idx: integer);
  var
    Q: integer;
    SR: TVentilationSpeleometrie;
    qR: TReseau;
    qS: TSecteur;
    qC: TCode;
    qE: TExpe;
    qD: TDateTime;
  begin
    with grdSyntheseSpeleometrique do
    begin
      Q := 1 + Idx;
      case Mode of
        mslRESEAUX:
          begin
             qR := FBDDEntites.GetReseau(Idx);
             Cells[0, Q] := _AnsiToLCLStr(qR.NomReseau);
             SR := FBDDEntites.GetSpeleometrieParReseauxByIdx(Idx);
           end;
        mslSECTEURS:
          begin
            qS := FBDDEntites.GetSecteur(Idx);
            Cells[0, Q] := _AnsiToLCLStr(qS.NomSecteur);
            SR := FBDDEntites.GetSpeleometrieParSecteurByIdx(Idx);
          end;
        mslCODE:
          begin
            qC := FBDDEntites.GetCode(Idx);
            Cells[0, Q] := _AnsiToLCLStr(qC.Commentaire);
            SR := FBDDEntites.GetSpeleometrieParCodeByIdx(Idx);
          end;
        mslEXPE:
          begin
            qE := FBDDEntites.GetExpe(Idx);
            Cells[0, Q] := Format('%d - %s', [qE.IDExpe, _AnsiToLCLStr(qE.Commentaire)]);
            SR := FBDDEntites.GetSpeleometrieParExpeByIdx(Idx);
          end;
        mslDATE:
          begin
            qD := FBDDEntites.GetUneDate(Idx);
            Cells[0, Q] := Format('%s', [DateToStr(qD)]);
            SR := FBDDEntites.GetSpeleometrieParDateByIdx(Idx);
          end;

      end;
      Cells[1, Q] := Format('%.2f', [SR.Fossiles]);
      Cells[2, Q] := Format('%.2f', [SR.Vadoses]);
      Cells[3, Q] := Format('%.2f', [SR.Ennoyables]);
      Cells[4, Q] := Format('%.2f', [SR.Siphons]);
      Cells[5, Q] := '';
      Cells[6, Q] := Format('%.2f', [SR.Fossiles + SR.Vadoses + SR.Ennoyables + SR.Siphons]);
      Cells[7, Q] := '';
      Cells[8, Q] := Format('%.2f', [SR.Speciaux]);
      Cells[9, Q] := Format('%.2f', [SR.Tunnels]);
      Cells[10,Q] := Format('%.2f', [SR.Filons]);
    end;
    //*)
  end;
begin
  AfficherMessage(Format('%s.RefreshTableau', [ClassName]));
  AfficherMessage(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
  case Mode of
    mslRESEAUX  : Nb := FBDDEntites.GetNbReseaux;
    mslSECTEURS : Nb := FBDDEntites.GetNbSecteurs;
    mslCODE     : Nb := FBDDEntites.GetNbCodes;
    mslEXPE     : Nb := FBDDEntites.GetNbExpes;
    mslDATE     : Nb := FBDDEntites.GetNbDates;
  end;
  case Mode of
    mslRESEAUX  : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_RESEAUX)  ,[Nb]);
    mslSECTEURS : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_SECTEURS) ,[Nb]);
    mslCODE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_CODES)    ,[Nb]);
    mslEXPE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_SEANCES)  ,[Nb]);
    mslDATE     : lbNbItems.Caption := Format(GetResourceString(rsSTATISTIQUES_NB_DATES)    ,[Nb]);
  otherwise
    lbNbItems.Caption := '';
  end;
  with grdSyntheseSpeleometrique do
  begin
    ColCount := NB_COLONNES;
    RowCount := 1+Nb;
    WriteHeader(Mode);
    for i := 0 to Nb -1 do WriteLigne(Mode, i);
  end;
end;
//******************************************************************************
procedure TfrmStatistiquesExt2.FormShow(Sender: TObject);
begin

  self.Caption              := GetResourceString(rsSTATISTIQUES_TITRE);
  tabShtTableaux.Caption    := GetResourceString(rsSTATISTIQUES_TAB_TABLEAUX);
  tabShtDiagrams.Caption    := GetResourceString(rsSTATISTIQUES_TAB_DIAGRAMMES);
  tabShtSynthese.Caption    := GetResourceString(rsSTATISTIQUES_TAB_SYNTHESE);
  tabShtCoordonnees.Caption := GetResourceString(rsSTATISTIQUES_TAB_CORDONNEES);


  cmbAffichageTableaux.Clear;
  cmbAffichageTableaux.Style := csDropDownList;
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE0));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE1));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE2));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE3));
  cmbAffichageTableaux.Items.Add(GetResourceString(rsSTATISTIQUES_CMB_MODE_TABLE4));
  cmbAffichageTableaux.ItemIndex := 0;
  btnCopyTableauRecap.Caption := rsSTATISTIQUES_COPY_TABLEAU;
  btnCopyTableauCoords.Caption := rsSTATISTIQUES_COPY_TABLEAU;

  // charger fichier de coordonnees;
  tabShtCoordonnees.Visible := ChargerFichierCoordonnees(GetGHTopoDirectory() + TEMP_COORDS_FILE);
  // affichage des diagrammes
  RedessinerLesDiagrammes;
end;



procedure TfrmStatistiquesExt2.RedessinerLesDiagrammes;
begin
  CdrRoseDiagramExt1.Initialise(FBDDEntites,
                                editRoseFiltres.Text,
                                editNbPetales.AsInteger,
                                editLongViseeMini.Value,
                                btnColorPetalesLine.ButtonColor,
                                btnColorPetalesFill.ButtonColor
                               );
  CdrHistoAltitudesExt1.Initialise(FBDDEntites,
                                 editFiltresDepth.Text,
                                 editNbPetalesDepth.AsInteger,
                                 editLongViseeMiniDepth.Value,
                                 btnColorBarresLineDepth.ButtonColor,
                                 btnColorBarresFillDepth.ButtonColor
                                );
end;
// charger le fichier de coordonnées XYZ
// traitements à la volée, suffisants ici
function TfrmStatistiquesExt2.ChargerFichierCoordonnees(const FichierCoords: string): boolean;
const
  COL_MESURES     = 4;
  COL_ID_LITTERAL = 11;
  COL_COORDS_X    = 12;
  COL_COMMENTAIRE = 15;

  MS_NONE        = 00;
  MS_ENTRANCES   = 01;
  MS_SERIES      = 02;
var
  fp  : TextFile;
  EWE : string;
  ModeSection: Integer;
  NbSts, n, c: integer;
  WU: TStringArray;
  Ser, St: Integer;
  qIDSt     : string;
  qLong, qAz, qPente : double;
  qLG, qLD, qHZ, qHN : double;
  qX, qY, qZ: double;
  qObs      : string;
begin
  AfficherMessage(Format('%s.ChargerFichierCoordonnees(%s)', [ClassName, FichierCoords]));
  Result := False;
  ModeSection := MS_NONE;   // Sections parcourues
  NbSts       := 0;

  with grdCoordsStations do
  begin
    RowCount := 1;
    ColCount := 13;
    Cells[0, 0]   := 'Station';  ColWidths[0] := 60;
    Cells[1, 0]   := 'ID';       ColWidths[1] := 120;
    Cells[2, 0]   := 'Long';     ColWidths[2] := 90;
    Cells[3, 0]   := 'Azimut';   ColWidths[3] := 80;
    Cells[4, 0]   := 'Pente';    ColWidths[4] := 80;
    Cells[5, 0]   := 'LG';       ColWidths[5] := 90;
    Cells[6, 0]   := 'LD';       ColWidths[6] := 90;
    Cells[7, 0]   := 'HZ';       ColWidths[7] := 90;
    Cells[8, 0]   := 'HN';       ColWidths[8] := 90;
    for c := 1 to 3 do
    begin
      Cells[8 + c, 0] := chr(87 + c);
      ColWidths[8 + c] := 100;
    end;
    Cells[12, 0]   := 'Observ';       ColWidths[12] := 300;

  end;
  AssignFile(fp, FichierCoords);
  try
    Reset(fp);
    grdCoordsStations.Visible := False;
    while (not Eof(fp)) do
    begin
      ReadLn(fp, EWE);
      if (Trim(EWE) = '') then Continue; // lignes vides
      if (Trim(EWE) = '[Entrances]') then ModeSection := MS_ENTRANCES;
      if (Trim(EWE) = '[Series]')    then ModeSection := MS_SERIES;
      case ModeSection of
        MS_NONE: ;
        MS_ENTRANCES:
          begin
            ;
          end;
        MS_SERIES:
          begin
            try
              WU := Split(EWE, #9);
              Ser  := StrToIntDef(WU[0], -1);
              St   := StrToIntDef(WU[1], -1);
              if (Ser < 0) then Continue;
              qIDSt := WU[COL_ID_LITTERAL];
              qLong := ConvertirEnNombreReel(WU[COL_MESURES + 0], -1.00);
              qAz   := ConvertirEnNombreReel(WU[COL_MESURES + 1], -1.00);
              qPente:= ConvertirEnNombreReel(WU[COL_MESURES + 2], -1.00);
              qLG := ConvertirEnNombreReel(WU[COL_MESURES + 3], -1.00);
              qLD := ConvertirEnNombreReel(WU[COL_MESURES + 4], -1.00);
              qHZ := ConvertirEnNombreReel(WU[COL_MESURES + 5], -1.00);
              qHN := ConvertirEnNombreReel(WU[COL_MESURES + 6], -1.00);

              qX := ConvertirEnNombreReel(WU[COL_COORDS_X + 0], -1.00);
              qY := ConvertirEnNombreReel(WU[COL_COORDS_X + 1], -1.00);
              qZ := ConvertirEnNombreReel(WU[COL_COORDS_X + 2], -1.00);
              qObs := WU[COL_COMMENTAIRE];

              Inc(NbSts);
              with grdCoordsStations do
              begin
                RowCount := 1 + NbSts;
                Cells[0, NbSts] := Format('%d.%d', [Ser, St]);
                Cells[1, NbSts] := qIDSt;

                Cells[2, NbSts] := Format('%.2f', [qLong]);
                Cells[3, NbSts] := Format('%.2f', [qAz]);
                Cells[4, NbSts] := Format('%.2f', [qPente]);

                Cells[5, NbSts] := Format('%.2f', [qLG]);
                Cells[6, NbSts] := Format('%.2f', [qLD]);
                Cells[7, NbSts] := Format('%.2f', [qHZ]);
                Cells[8, NbSts] := Format('%.2f', [qHN]);

                Cells[9, NbSts] := Format('%.2f', [qX]);
                Cells[10, NbSts] := Format('%.2f', [qY]);
                Cells[11, NbSts] := Format('%.2f', [qZ]);
                Cells[12, NbSts] := AnsiToUtf8(qObs);
              end;

            except
            end;
          end;
      end;
      grdCoordsStations.Visible := True;
      Result := True;
    end;
  finally
    CloseFile(fp);
  end;

end;

end.

