unit frmAjouterSeriesDepuisAutreDoc;


{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  ToporobotClasses2012,
  Classes, SysUtils, FileUtil, DateTimePicker, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Buttons, CadreListeSeries, UnitObjetSerie;

type TTodoActionWithExistentSerie = (aesAPPEND_VISEES, aesREPLACE_VISEES, aesIGNORE_SERIE);
type

  { TdlgAddSeriesFromOtherDoc }

  TdlgAddSeriesFromOtherDoc = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    CdrListeSeries1: TCdrListeSeries;
    chkCreateNewCode: TCheckBox;
    chkCreateNewExpe: TCheckBox;
    editDateExpe: TDateTimePicker;
    Panel1: TPanel;
    Panel2: TPanel;
    rgbxTodoWithSerie: TRadioGroup;
    procedure BitBtn1Click(Sender: TObject);
  private
    FDocTopoCible : TToporobotStructure2012;
    FDocTopoSource: TToporobotStructure2012;
    function AjouterLesSeries(): boolean;
    function ChargerLaTopoExt(const FC: TStringDirectoryFileName): boolean;
  public
    function  Initialiser(const FD: TToporobotStructure2012; const FC: TStringDirectoryFilename): boolean;
    procedure Finaliser();

  end;

var
  dlgAddSeriesFromOtherDoc: TdlgAddSeriesFromOtherDoc;

implementation

{$R *.lfm}

{ TdlgAddSeriesFromOtherDoc }


function TdlgAddSeriesFromOtherDoc.Initialiser(const FD: TToporobotStructure2012; const FC: TStringDirectoryFilename): boolean;
var
  QButtons: TButtonsShown;
begin
  result := false;

  chkCreateNewCode.Checked := True;
  chkCreateNewExpe.Checked := True;
  editDateExpe.Date := Now() - 1.00; // les reports se faisant généralement le lendemain, ceci rétablit la date de séance
  rgbxTodoWithSerie.ItemIndex := Ord(aesAPPEND_VISEES);
  QButtons := [tbsHELP];
  FDocTopoCible  := FD;
  FDocTopoSource := TToporobotStructure2012.Create;
  try
    AfficherMessage(Format('%s.Initialiser(): %s', [ClassName, FC]));;
    if (ChargerLaTopoExt(FC)) then
    begin
      Result := CdrListeSeries1.Initialise(FDocTopoSource, nil, nil, QButtons, True);
      //result := True;
    end;
  except
    pass;
  end;
end;
procedure TdlgAddSeriesFromOtherDoc.Finaliser();
begin
  try
    FDocTopoSource.Finaliser();
  finally
    FDocTopoSource.Free;
  end;
end;
function TdlgAddSeriesFromOtherDoc.AjouterLesSeries(): boolean;
var
  Nb, i, NbSts, St, QStartIdxVisee, QInternalIdx, QNbSts: Integer;
  MySerieSource, MySerieCible, SerieExistante: TObjSerie;
  QTransfertSerieOK: Boolean;
  MyCode: TCode;
  MyExpe: TExpe;
  YYYY, MM, DD: word;
  QTodoWithSeriesExistantes: TTodoActionWithExistentSerie;
  MyReseau: TReseau;
  MySecteur: TSecteur;
  procedure MiouMiou(const QSerieDest: TObjSerie; const QV : TUneVisee; const QSt: integer);
  var
    QVisee: TUneVisee;
    QArrayAntennes: TArrayOfTViseeAntenne;
    a: Integer;
  begin
    QVisee := QV;
    QVisee.Code      := MyCode.IDCode;
    QVisee.Expe      := MyExpe.IDExpe;
    QVisee.IDSecteur := 0;
    QSerieDest.AddVisee(QVisee);
    // et les antennes

    if (FDocTopoSource.ExtractViseesAntennesOfStation(QSerieDest.GetNumeroDeSerie(), QSt, QArrayAntennes) > 0) then
    begin
      for a := 0 to High(QArrayAntennes) do
      begin
        FDocTopoCible.AddViseeAntenne(QArrayAntennes[a]);
      end;
    end;

  end;
begin
  Nb := FDocTopoSource.GetNbSeries();
  AfficherMessageErreur(Format('%s.AjouterLesSeries(): %d series', [ClassName, Nb]));
  AfficherMessageErreur('En provenance du document: ' + FDocTopoSource.GetDatabaseName);
  if (Nb <= 1) then Exit;
  QTodoWithSeriesExistantes := TTodoActionWithExistentSerie(rgbxTodoWithSerie.ItemIndex);
  // préparer les codes et expés
  if (chkCreateNewCode.Checked) then
  begin
    MyCode := FDocTopoCible.GetLastCode();
    MyCode.Commentaire := 'Topo du ' + DateTimeToStr(editDateExpe.Date);
    MyCode.IDCode += 1;
    FDocTopoCible.AddCode(MyCode);
  end;
  if (chkCreateNewCode.Checked) then
  begin
    MyExpe := FDocTopoCible.GetLastExpe();
    DecodeDate(editDateExpe.Date, YYYY, MM, DD);
    MyExpe.AnneeExpe := YYYY; MyExpe.MoisExpe := MM; MyExpe.JourExpe := DD;
    MyExpe.IDExpe  += 1;
    MyExpe.Couleur := (MyExpe.Couleur + 1) MOD 256; // propale: ajouter 1 à l'index couleur de la dernière expé
    FDocTopoCible.AddExpe(MyExpe);
  end;
  // attraper les derniers codes, reseau, secteur, expé
  MyReseau  := FDocTopoCible.GetLastReseau();
  MySecteur := FDocTopoCible.GetLastSecteur();
  MyCode    := FDocTopoCible.GetLastCode();
  MyExpe    := FDocTopoCible.GetLastExpe();

  for i := 1 to Nb - 1 do
  begin
    if (CdrListeSeries1.IsItemListeSelected(i)) then
    begin
      MySerieSource := FDocTopoSource.GetSerie(i);
      AfficherMessageErreur(Format('%d: %d - %s', [i, MySerieSource.GetNumeroDeSerie(), MySerieSource.GetNomSerie()]));
      if (FDocTopoCible.GetSerieByNumeroSerie(MySerieSource.GetNumeroDeSerie(), SerieExistante, QInternalIdx)) then
      begin
        AfficherMessageErreur(Format('-- Serie %d existante - Actions: %s', [SerieExistante.GetNumeroDeSerie(), ChooseString(Ord(QTodoWithSeriesExistantes), ['Ajouter visées en queue', 'Remplacer visées', 'Ne rien faire'])]));
        NbSts := MySerieSource.GetNbVisees();
        case QTodoWithSeriesExistantes of
          aesAPPEND_VISEES:
          begin
            QStartIdxVisee := SerieExistante.GetNbVisees();
            if (QStartIdxVisee < (NbSts - 1)) then
            begin
              for St := QStartIdxVisee to NbSts - 1 do MiouMiou(SerieExistante, MySerieSource.GetVisee(St), St);
              SerieExistante.SetNoPointArr(SerieExistante.GetNbVisees() - 1);
            end
            else AfficherMessageErreur('-- Aucune visée à ajouter');
          end;
          aesREPLACE_VISEES:
          begin
            SerieExistante.ClearStations();
            QStartIdxVisee := 0;
            for St := QStartIdxVisee to NbSts - 1 do MiouMiou(SerieExistante, MySerieSource.GetVisee(St), St);
            SerieExistante.SetNoPointArr(SerieExistante.GetNbVisees() - 1);
          end;
          aesIGNORE_SERIE: pass; // ne rien faire ici
        end;
      end
      else
      begin // série non trouvée = on en crée une
        QTransfertSerieOK := false;
        // /!\ Ne pas faire un simple FDocTopoCible.AddSerie(MySerieSource):
        //     MySerieSource est un pointeur d'objet appartenant à FDocTopoSource,
        //     lequel sera détruit à la fermeture du dialogue et MySerieSource avec ... ;
        MySerieCible := TObjSerie.Create;
        try
          MySerieCible.ClearStations();
          MySerieCible.SetChanceObstacle(MySerieSource.GetChance(), MySerieSource.GetObstacle());
          MySerieCible.SetSeriePtExtremites(MySerieSource.GetNoSerieDep(), MySerieSource.GetNoPointDep(),
                                            MySerieSource.GetNoSerieArr(), MySerieSource.GetNoPointArr());
          MySerieCible.SetNumeroSerie(MySerieSource.GetNumeroDeSerie());
          MySerieCible.SetNomObsSerie(MySerieSource.GetNomSerie(), '*** Imported *** ' + MySerieSource.GetObsSerie());
          MySerieCible.SetRaideur(MySerieSource.GetRaideur());
          MySerieCible.SetCouleur(MySerieSource.GetCouleur());
          MySerieCible.SetNumeroEntrance(0);
          MySerieCible.SetNumeroReseau(0);
          // les stations -- TODO: A valider
          QStartIdxVisee := 0;
          NbSts := MySerieSource.GetNbVisees();
          for St := QStartIdxVisee to NbSts - 1 do MiouMiou(MySerieCible, MySerieSource.GetVisee(St), St);
          QTransfertSerieOK := True; // création OK ? On autorise l'ajout
        except
          MySerieCible.Free;  // en cas d'échec, on libère immédiatement
        end;
        if (QTransfertSerieOK) then
        begin
          FDocTopoCible.AddSerie(MySerieCible);
          MySerieCible := FDocTopoCible.GetLastSerie();
          (*AfficherMessageErreur(Format('-- %d: %d [%d.%d -> %d.%d]: %s',
                              [i,
                               MySerieCible.GetNumeroDeSerie(),
                               MySerieCible.GetNoSerieDep(), MySerieCible.GetNoPointDep(),
                               MySerieCible.GetNoSerieArr(), MySerieCible.GetNoPointArr(),
                               MySerieCible.GetNomSerie()
                              ]));
          //*)

        end; // if (QTransfertSerieOK) then
      end;   // if (FDocTopoCible.GetSerieByNumeroSerie(MySerieSource.GetNumeroDeSerie(), SerieExistante, QSInternalIdx)) then
    end;     // if (CdrListeSeries1.IsItemListeSelected(i)) then
  end;       // for i := 1 to Nb - 1 do
end;

procedure TdlgAddSeriesFromOtherDoc.BitBtn1Click(Sender: TObject);
begin
  Showmessage('Prêt à ajouter séries');
  AjouterLesSeries();
  //FDocuTopo.CreateNewSerie(1,0, -1, GetResourceString(rsNOUVELLE_SERIE));



end;

//******************************************************************************
// charger la topo
// Chargement de la topo
function TdlgAddSeriesFromOtherDoc.ChargerLaTopoExt(const FC: TStringDirectoryFileName): boolean;
begin
  Result := false;
  AfficherMessage(Format('%s.ChargerLaTopoExt: %s', [ClassName, FC]));
  FDocTopoSource.ReInitialiser(True);

  FDocTopoSource.SetDatabaseName('Doc_to_append');
  // vérifier si c'est un fichier Text
  if (Pos('.text', LowerCase(FC))>0) then
  begin
    if (FDocTopoSource.LoadFichierText(FC) < 0) then
    begin
      //showmessage('gmj');
      AfficherMessageErreur('Le fichier comporte des erreurs - Voir les consoles');
      FDocTopoSource.Finaliser;   // Echec = on détruit l'objet
      Exit;
    end;
  end
  else if (Pos('.gtx', FC) > 0) then // fichier GHTopo XML ?
  begin
    if (FDocTopoSource.LoadFromXML(FC) < 0) then
    begin
      AfficherMessageErreur('Le fichier comporte des erreurs. Il doit être audité dans un éditeur XML');
      FDocTopoSource.Finaliser();
      Exit;
    end;
  end
  else
  begin // sinon, c'est un fichier supposé Tab ou XTB
    if (FDocTopoSource.LoadFichierTab(FC)<0) then
    begin
      AfficherMessageErreur('Le fichier comporte des erreurs - Voir le rapport');
      // Echec = on détruit l'objet
      FDocTopoSource.Finaliser();
      Exit;
    end;
  end;
  // tout est OK ?
  self.Caption:=Format(rsMAINMENUCAPTION, [ExtractFileName(FC)]);
  Application.ProcessMessages;
  Result := True;
end;
end.

