unit frmListeGroupes;

{$mode delphi}

interface

uses
  GHCD_Types,
  GeneralFunctions,
  LCLType,
  UnitDocDessin, CdrListeGroupes,
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls, Buttons, Types;

type

  { TdlgListeGroupes }

  TdlgListeGroupes = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    editFindingGroupe: TEdit;
    hcColonnes: THeaderControl;
    Label1: TLabel;
    lbNbGroupes: TLabel;
    lsbGroupes: TListBox;
    procedure lsbGroupesDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
  private
    { private declarations }
    FDocDessin : TDocumentDessin;
    procedure ListerLesGroupes(const IdxGr: integer);
  public
    { public declarations }
    function Initialiser(const FD: TDocumentDessin; const GroupeIdx: integer): boolean;
  end;

var
  dlgListeGroupes: TdlgListeGroupes;

implementation

{$R *.lfm}
{ TdlgListeGroupes }

function TdlgListeGroupes.Initialiser(const FD: TDocumentDessin;
                                      const GroupeIdx: integer): boolean;
begin
  result := false;
  try
    FDocDessin := FD;
    ListerLesGroupes(GroupeIdx);
    result := true;
  except
  end;
end;

procedure TdlgListeGroupes.ListerLesGroupes(const IdxGr: integer);
var
  n, WU, i: Integer;
  EWE: TGroupeEntites;
begin
  lsbGroupes.Visible := false;
  lsbGroupes.Clear;
  n := FDocDessin.GetNbGroupes();
  WU := IIF(IdxGr < 0, n-1, IdxGr);
  for i := 0 to n-1 do
  begin
    EWE := FDocDessin.GetGroupeByIdx(i);
    lsbGroupes.Items.Add(Format('%.3d (%.0f) - %s', [EWE.IDGroupeEntites, EWE.ZOrder, EWE.NomGroupe]));
  end;
  lsbGroupes.ItemIndex := WU;
  lsbGroupes.Visible := true;
  lbNbGroupes.Caption := Format('%d groupes', [WU]);
end;

// La factorisation de ce code, utilisé en deux endroits du projet, est très difficile.
procedure TdlgListeGroupes.lsbGroupesDrawItem(Control: TWinControl; Index: Integer; ARect: TRect; State: TOwnerDrawState);
const
  Q4  = 4; // compensation du décalage entre le header et la liste
  mg  = 1;
var
  MyListeDeroulante: TListBox;
  MyGRP: TGroupeEntites;
  WU: TColor;
  procedure DessineCoche(const TB: integer; const IsChecked: boolean; const bg: TColor);
  var
    QR: TRect;
    H: Integer;
  begin
    MyListeDeroulante.Canvas.Pen.Style   := psSolid;
    MyListeDeroulante.Canvas.Brush.Style := bsSolid;
    MyListeDeroulante.Canvas.Pen.Color   := clBlack;

    QR.Left   := ARect.Left   + TB + 2;
    QR.Top    := ARect.Top    + 1;
    QR.Bottom := ARect.Bottom - 1;
    H := QR.Bottom - QR.Top;
    QR.Right  := QR.Left + H;
    MyListeDeroulante.Canvas.Brush.Color := clWhite;
    if (IsChecked) then MyListeDeroulante.Canvas.Brush.Color := clRed;
    MyListeDeroulante.Canvas.Rectangle(QR);
    MyListeDeroulante.Canvas.Brush.Color := bg;
  end;
  procedure DessineRectCouleur(const TB: integer; const W: integer;const Coul: TColor; const bg: TColor);
  var
    QR: TRect;
  begin
    MyListeDeroulante.Canvas.Pen.Style   := psSolid;
    MyListeDeroulante.Canvas.Brush.Style := bsSolid;
    MyListeDeroulante.Canvas.Brush.Color := Coul;
    MyListeDeroulante.Canvas.Pen.Color   := clBlack;
    QR.Left   := ARect.Left   + TB + 3;
    QR.Top    := ARect.Top    + 1;
    QR.Bottom := ARect.Bottom - 1;
    QR.Right  := QR.Left + W - 4;
    MyListeDeroulante.Canvas.Rectangle(QR);
    MyListeDeroulante.Canvas.Brush.Color := bg;
  end;
  procedure DessineFiletColonne(const TB: integer);
  begin
    MyListeDeroulante.Canvas.Pen.Color   := clSilver; // pour les filets
    MyListeDeroulante.Canvas.MoveTo(TB, ARect.Top);
    MyListeDeroulante.Canvas.LineTo(TB, ARect.Bottom);
  end;
  procedure DessineItem(const bg,tc: TColor);
  VAR
    HS: THeaderSection;
    QR: TRect;
  begin
    MyListeDeroulante.Canvas.Brush.Color := bg;
    MyListeDeroulante.Canvas.Font.Color  := tc;
    MyListeDeroulante.Canvas.FillRect(ARect);
    HS := hcColonnes.Sections.Items[0];  // affiché
    // mettre en évidence les groupes vides
    if (MyGRP.NombreObjets = 0) then DessineRectCouleur(HS.Left - Q4, HS.Width, clFuchsia, bg);
    //MyListeDeroulante.Canvas.TextOut(HS.Left + 4, ARect.Top,  Format('%d',[MyGRP.NombreObjets]));
    HS := hcColonnes.Sections.Items[1];  // affiché
    DessineCoche(HS.Left, MyGRP.Displayed, bg);
    HS := hcColonnes.Sections.Items[2];  // verrouillé
    DessineCoche(HS.Left, MyGRP.Locked, bg);
    DessineFiletColonne(HS.Left - Q4);
    HS := hcColonnes.Sections.Items[3];  // couleur
    MyListeDeroulante.Canvas.Brush.Color := MyGRP.CouleurGroupe;
    DessineRectCouleur(HS.Left - Q4, HS.Width, MyGRP.CouleurGroupe, bg);
    DessineFiletColonne(HS.Left - Q4);
    HS := hcColonnes.Sections.Items[4];  // num  groupe
    MyListeDeroulante.Canvas.TextOut(HS.Left + 4, ARect.Top,  Format('%d',[MyGRP.IDGroupeEntites]));
    DessineFiletColonne(HS.Left - Q4);
    HS := hcColonnes.Sections.Items[5];  // ZOrder
    MyListeDeroulante.Canvas.TextOut(HS.Left + 4, ARect.Top,  Format('%.0f',[MyGRP.ZOrder]));
    DessineFiletColonne(HS.Left - Q4);
    HS := hcColonnes.Sections.Items[6];  // Nom entrée
    DessineFiletColonne(HS.Left - Q4);
    MyListeDeroulante.Canvas.TextOut(HS.Left + 4, ARect.Top, MyGRP.NomGroupe);
  end;
begin
  MyListeDeroulante := lsbGroupes;
  try
    MyGRP := FDocDessin.GetGroupeByIdx(Index);
    WU := clWhite;
    DessineItem(WU, clBlack);
    if (odSelected in state) then
    begin
      MyListeDeroulante.Canvas.brush.color := clBlue;
      DessineItem(clBlue, clWhite);
    end;
  except
    AfficherMessageErreur('Warn: Liste groupes vide');
  end;
end;

end.

