unit frmMaintenanceDocument;

{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Classes, SysUtils,
  StructuresDonnees,
  Common,
  CallDialogsStdVersion,
  ToporobotClasses2012,
  UnitEntitesExtended,
  CadreViseesAntenne,
  CadreListeSeries,
  CadreListesSimples,
  UnitClassPalette, UnitObjetSerie,
  ConvertisseurJPC,
  FileUtil, curredit, Forms, Controls, Graphics, Dialogs, Buttons, ComCtrls, ExtCtrls, StdCtrls;

type

  { TdlgMaintenanceDocTopo }

  TdlgMaintenanceDocTopo = class(TForm)
    BitBtn1: TBitBtn;
    btnReattribuerAuxSeriesSelectionnees: TButton;
    btnReattribuerCode: TButton;
    btnReattribuerIDExpe: TButton;
    btnReattribuerIDSerie: TButton;
    btnModifier: TButton;
    btnSelectCode: TButton;
    btnSelectExpe: TButton;
    btnSelectReseau: TButton;
    btnSelectEntrance: TButton;
    btnSupprimerSeriesSelectionnees: TButton;
    Button1: TButton;
    Button2: TButton;
    CdrAntennes1: TCdrAntennes;
    CdrListeSeries1: TCdrListeSeries;
    CdrListesSimples1: TCdrListesSimples;
    cmbCodesEPSG: TComboBox;
    editNameSpace: TEdit;
    editCurrNumeroCode: TCurrencyEdit;
    editCurrNumeroExpe: TCurrencyEdit;
    editCurrNumeroReseau: TCurrencyEdit;
    editCurrNumeroEntree: TCurrencyEdit;
    editNomEtude: TEdit;
    editAncienIDCode: TCurrencyEdit;
    editAncienIDExpe: TCurrencyEdit;
    editAncienIDSerie: TCurrencyEdit;
    editNouvelIDCode: TCurrencyEdit;
    editNouvelIDExpe: TCurrencyEdit;
    editNouvelIDSerie: TCurrencyEdit;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    PageControl1: TPageControl;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel5: TPanel;
    Panel7: TPanel;
    Panel8: TPanel;
    Panel9: TPanel;
    lbMessages: TStaticText;
    tabShtListesSimples: TTabSheet;
    tabshtSeries: TTabSheet;
    tabShtAntennes: TTabSheet;
    tabshtGeneral: TTabSheet;
    procedure btnModifierClick(Sender: TObject);
    procedure btnReattribuerAuxSeriesSelectionneesClick(Sender: TObject);
    procedure btnReattribuerCodeClick(Sender: TObject);
    procedure btnReattribuerIDExpeClick(Sender: TObject);
    procedure btnReattribuerIDSerieClick(Sender: TObject);
    procedure btnSelectCodeClick(Sender: TObject);
    procedure btnSelectEntranceClick(Sender: TObject);
    procedure btnSelectExpeClick(Sender: TObject);
    procedure btnSelectReseauClick(Sender: TObject);
    procedure btnSupprimerSeriesSelectionneesClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Label2Click(Sender: TObject);
    procedure Label5Click(Sender: TObject);
    procedure Label9Click(Sender: TObject);
  private
    FDocTopo: TToporobotStructure2012;
    FBDDEntites: TBDDEntites;
    FPalette256: TPalette256;
    FConvertisseurCoords: TConversionSysteme;
    FDoPostAction: boolean;
    procedure ExtractSerie();
    procedure GenererSubsetXTB(const QFilename: TStringDirectoryFilename);
    procedure ReattribuerEntreeReseauCodeExpe(const QIdxEntrance: TNumeroEntrance;
                                         const QIdxReseau  : TNumeroReseau;
                                         const QIdxCode    : TNumeroCode;
                                         const QIdxExpe    : TNumeroExpe);
    // reprojection des entrées
    procedure ReprojeterLesEntrees(const OldEPSG, NewEPSG: TLabelSystemesCoordsEPSG);
  public
    function  Initialiser(const FD: TToporobotStructure2012;
                         const FE: TBDDEntites;
                         const FP: TPalette256;
                         const FC: TConversionSysteme): boolean;
    procedure Finaliser();
    function  DoPostAction(): boolean;


  end;

var
  dlgMaintenanceDocTopo: TdlgMaintenanceDocTopo;

implementation

{$R *.lfm}

{ TdlgMaintenanceDocTopo }


procedure TdlgMaintenanceDocTopo.ReattribuerEntreeReseauCodeExpe(const QIdxEntrance: TNumeroEntrance;
                                                            const QIdxReseau  : TNumeroReseau;
                                                            const QIdxCode    : TNumeroCode;
                                                            const QIdxExpe    : TNumeroExpe);
var
  i, NbSeries, NbVisees, j: Integer;
  MySerie    : TObjSerie;
  MyVisee: TUneVisee;
begin
  if (not GHTopoQuestionOuiNon('Cette fonction peut désorganiser la base - Continuer ?')) then Exit;
  AfficherMessage(format('%s.ReattribuerEntreeReseauCodeExpe(%d, %d, %d, %d', [ClassName, QIdxEntrance, QIdxReseau, QIdxCode, QIdxExpe]));

  // on checke les numéros de expé, code, réseau
  if (not FDocTopo.ExistsIdxEntrance(QIdxEntrance)) then Exit;
  if (not FDocTopo.ExistsIdxReseau(QIdxReseau)) then Exit;
  if (not FDocTopo.ExistsIdxCode(QIdxCode)) then Exit;
  if (not FDocTopo.ExistsIdxExpe(QIdxExpe)) then Exit;
  NbSeries := FDocTopo.GetNbSeries();
  for i := 0 to NbSeries - 1 do
  begin
    if (CdrListeSeries1.IsItemListeSelected(i)) then
    begin
      MySerie := FDocTopo.GetSerie(i);
      NbVisees := MySerie.GetNbVisees();
      lbMessages.Caption := Format('--> %d: %d (%d stations): %s', [i, MySerie.GetNumeroDeSerie(), NbVisees, MySerie.GetNomSerie()]);
      MySerie.SetNumeroEntrance(QIdxEntrance);
      MySerie.SetNumeroReseau(QIdxReseau);

      for j := 0 to NbVisees - 1 do
      begin
        MyVisee        := MySerie.GetVisee(j);
        MyVisee.Code   := QIdxCode;
        MyVisee.Expe   := QIdxExpe;
        MySerie.PutVisee(j, MyVisee);
      end;
    end;
  end;
end;

procedure TdlgMaintenanceDocTopo.Label2Click(Sender: TObject);
begin

end;

procedure TdlgMaintenanceDocTopo.Label5Click(Sender: TObject);
begin

end;

procedure TdlgMaintenanceDocTopo.Label9Click(Sender: TObject);
begin

end;

procedure TdlgMaintenanceDocTopo.ExtractSerie();
begin
  pass;
end;

procedure TdlgMaintenanceDocTopo.GenererSubsetXTB(const QFilename: TStringDirectoryFilename);
var
  MySubsetXTB: TToporobotStructure2012;
  WU: String;
  Nb, i, NbSeriesSelectionnees: Integer;
  MySerie: TObjSerie;
begin
  AfficherMessage(format('%s.GenererSubsetXTB(%s)', [ClassName, QFilename]));
  MySubsetXTB := TToporobotStructure2012.Create;
  try
    MySubsetXTB.SetDatabaseName(QFilename);
    MySubsetXTB.SetCodeEPSGSystemeCoordonnees(FDocTopo.GetCodeEPSGSystemeCoordonnees());
    MySubsetXTB.SetNomEtude('Sous-ensemble de ' + FDocTopo.GetNomEtude());
    MySubsetXTB.SetDefaultCoords(MakeTPoint3Df(0.00, 0.00, 0.00));
    WU := ''; // mettre ici les numéros de séries extraites;
    MySubsetXTB.SetCommentairesEtude(WU);
    MySubsetXTB.ClearListeSeries();
    MySubsetXTB.ViderListeViseesAntennes();
    MySubsetXTB.ViderTablesSimples();
    // création de l'entrée 1.0
    MySubsetXTB.AddEntree(MakeTEntrance('Point_0', '1.0', 0.00, 0.00, 0.00, 1, 0, 'Point de rattachement'));
    // les réseaux
    Nb := FDocTopo.GetNbReseaux();
    for i := 0 to Nb - 1 do
    begin
      MySubsetXTB.AddReseau(FDocTopo.GetReseau(i));
    end;
    // les secteurs
    Nb := FDocTopo.GetNbSecteurs();
    for i := 0 to Nb - 1 do
    begin
      MySubsetXTB.AddSecteur(FDocTopo.GetSecteur(i));
    end;
    // les codes
    Nb := FDocTopo.GetNbCodes();
    for i := 0 to Nb - 1 do
    begin
      MySubsetXTB.AddCode(FDocTopo.GetCode(i));
    end;
    // lex expés
    Nb := FDocTopo.GetNbExpes();
    for i := 0 to Nb - 1 do
    begin
      MySubsetXTB.AddExpe(FDocTopo.GetExpe(i));
    end;
    // les séries (uniquement les séries sélectionnées)
    Nb := CdrListeSeries1.lsbListe.Count;
    FDocTopo.CreateNewSerie(1,0, -1, 'serie 0');
    // première série
    MySerie := FDocTopo.GetSerie(1);
    MySubsetXTB.AddSerie(MySerie);
    MySubsetXTB.AddSerie(MySerie);

    NbSeriesSelectionnees := 0;
    for i := 0 to Nb - 1 do
    begin
      if (CdrListeSeries1.IsItemListeSelected(i)) then
      begin
        Inc(NbSeriesSelectionnees);
        //if (Nb = 1) then MySerie.SetSeriePtDep(1, 0);
        MySerie := FDocTopo.GetSerie(i);
        if (NbSeriesSelectionnees = 1) then
        begin
          MySerie.SetSeriePtDep(1, 1);
        end;
        MySubsetXTB.AddSerie(MySerie);
      end;
    end;
    MySubsetXTB.SaveToFile(QFilename, mtabEXTENDEDTAB, tfWINDOWS);

    // finalisation
    MySubsetXTB.Finaliser();
  finally
    MySubsetXTB.Free;
  end;
end;

procedure TdlgMaintenanceDocTopo.btnReattribuerIDExpeClick(Sender: TObject);
var
  E1, E2: TNumeroExpe;
begin
  E1 := editAncienIDExpe.AsInteger;
  E2 := editNouvelIDExpe.AsInteger;
  FDocTopo.ReplaceIdxExpes(E1, E2);
  FDocTopo.ReplaceIndexInStations(E1, E2, mrsEXPES);
  //FDocTopo.ReplaceIndexInAntennes(E1, E2, [mraEXPES]);
  FDocTopo.ReattribuerCodesExpesViseesAntennes();
end;

procedure TdlgMaintenanceDocTopo.btnReattribuerIDSerieClick(Sender: TObject);
var
  E2, E1: TNumeroSerie;
  MySerie: TObjSerie;
  QIdx: integer;
begin
  E1 := editAncienIDSerie.AsInteger;
  E2 := editNouvelIDSerie.AsInteger;
  if (FDocTopo.ExistsSerie(E2)) then
  begin
    FDocTopo.GetSerieByNumeroSerie(E2, MySerie, QIdx);
    ShowMessageFmt('Série %d déjà attribuée: "%s"', [MySerie.GetNumeroDeSerie(), MySerie.GetNomSerie()]);
    exit;
  end;
  FDocTopo.ReplaceIndexSeriesInEntrances(E1, E2);
  FDocTopo.ReplaceIdxSeries(E1, E2);
  //FDocTopo.ReplaceIndexInAntennes(E1, E2, [mraEXPES]);
  FDocTopo.ReattribuerCodesExpesViseesAntennes();
  CdrListeSeries1.Relister(0);
end;

procedure TdlgMaintenanceDocTopo.btnSelectCodeClick(Sender: TObject);
var
  n: LongInt;
begin
  n := editCurrNumeroCode.AsInteger;
  if (SelectionDansListe(FDocTopo, mslCODE, false, n)) then  editCurrNumeroCode.AsInteger := n;
end;

procedure TdlgMaintenanceDocTopo.btnSelectEntranceClick(Sender: TObject);
var
  n: LongInt;
begin
  n := editCurrNumeroReseau.AsInteger;
  if (SelectionDansListe(FDocTopo, mslENTRANCES, false, n)) then  editCurrNumeroEntree.AsInteger := n;
end;

procedure TdlgMaintenanceDocTopo.btnSelectExpeClick(Sender: TObject);
var
  n: LongInt;
begin
  n := editCurrNumeroExpe.AsInteger;
  if (SelectionDansListe(FDocTopo, mslEXPE, false, n)) then  editCurrNumeroExpe.AsInteger := n;
end;

procedure TdlgMaintenanceDocTopo.btnSelectReseauClick(Sender: TObject);
var
  n: LongInt;
begin
  n := editCurrNumeroReseau.AsInteger;
  if (SelectionDansListe(FDocTopo, mslRESEAUX, false, n)) then  editCurrNumeroReseau.AsInteger := n;
end;

procedure TdlgMaintenanceDocTopo.btnSupprimerSeriesSelectionneesClick(Sender: TObject);
var
  n, i: Integer;
begin
  n := FDocTopo.GetNbSeries();
  if (not GHTopoQuestionOuiNon('Ceci supprimera DEFINITIVEMENT les séries sélectionnées - Continuer ?')) then exit;
  for i := n - 1 downto 1 do
  begin
    if (CdrListeSeries1.IsItemListeSelected(i)) then FDocTopo.RemoveSerie(i);
  end;
  CdrListeSeries1.Relister(0);
end;

procedure TdlgMaintenanceDocTopo.Button1Click(Sender: TObject);
begin

end;

procedure TdlgMaintenanceDocTopo.Button2Click(Sender: TObject);
var
  QFileName: TStringDirectoryFilename;
  QIdxFilter: integer;
  MyTextEditor: String;
begin
  if (not GHTopoQuestionOuiNon('Fonctionnalité expérimentale' + #13#10 +
                               'Le fichier XTB généré peut devoir être modifié' + #13#10 +
                               'Ceci ouvrira le fichier résultant dans un éditeur de texte '  + #13#10 +
                               'Continuer')) then Exit;
  QFileName := GetGHTopoDirectory() + 'Subset_001.xtb';
  if (DoDialogSaveFile('Fichier XTB', '.xtb', QFileName, QIdxFilter)) then
  begin
    GenererSubsetXTB(QFileName);
    MyTextEditor := 'C:\Program Files\Notepad++\notepad++.exe';
    RunExternalProgram(MyTextEditor, '', QFilename);
  end;

end;

procedure TdlgMaintenanceDocTopo.btnReattribuerCodeClick(Sender: TObject);
var
  E1, E2: TNumeroCode;
begin
  E1 := editAncienIDCode.AsInteger;
  E2 := editNouvelIDCode.AsInteger;
  FDocTopo.ReplaceIdxCodes(E1, E2);
  FDocTopo.ReplaceIndexInStations(E1, E2, mrsCODES);
  FDocTopo.ReattribuerCodesExpesViseesAntennes();
end;

procedure TdlgMaintenanceDocTopo.btnModifierClick(Sender: TObject);
var
  OldEPSG, NewEPSG: TLabelSystemesCoordsEPSG;
begin
  FDocTopo.SetNameSpace(trim(editNameSpace.Text));
  FDocTopo.SetNomEtude(trim(editNomEtude.Text));
  self.Caption := 'Maintenance du document: ' + FDocTopo.GetNomEtude();
  OldEPSG := FDocTopo.GetCodeEPSGSystemeCoordonnees();
  NewEPSG := FConvertisseurCoords.GetCodeEPSGNomSysteme(cmbCodesEPSG.ItemIndex);
  if (NewEPSG.CodeEPSG <> OldEPSG.CodeEPSG) then
  begin
    if (GHTopoQuestionOuiNon('Ceci reprojettera les coordonnées des entrées - Continuer')) then
    begin
      FDocTopo.SetCodeEPSGSystemeCoordonnees(NewEPSG);
      ReprojeterLesEntrees(OldEPSG, NewEPSG);
      FDoPostAction := true;
    end
    else
    begin
      cmbCodesEPSG.ItemIndex := FConvertisseurCoords.GetIndexSystemeByCodeEPSG(OldEPSG.CodeEPSG);
    end;
  end;
end;

procedure TdlgMaintenanceDocTopo.btnReattribuerAuxSeriesSelectionneesClick(Sender: TObject);
begin
  ReattribuerEntreeReseauCodeExpe(editCurrNumeroEntree.AsInteger, editCurrNumeroReseau.AsInteger,
                                  editCurrNumeroCode.AsInteger, editCurrNumeroExpe.AsInteger);
  CdrListeSeries1.Relister(0);
end;


function TdlgMaintenanceDocTopo.Initialiser(const FD: TToporobotStructure2012;
                                            const FE: TBDDEntites;
                                            const FP: TPalette256;
                                            const FC: TConversionSysteme): boolean;
var
  n, i: Integer;
  EWE: TLabelSystemesCoordsEPSG;
  MyCode: TCode;
  MyExpe: TExpe;
begin
  result := false;
  try
    FDoPostAction := false;
    FDocTopo := FD;
    FBDDEntites := FE;
    FPalette256 := FP;
    FConvertisseurCoords := FC;
    self.Caption := 'Maintenance du document: ' + FDocTopo.GetNomEtude();
    editNomEtude.Text   := FDocTopo.GetNomEtude();
    editNameSpace.Text := FDocTopo.GetNameSpace();


    CdrListesSimples1.Initialiser(FDocTopo, FBDDEntites, FPalette256, mbddCODES);
    CdrAntennes1.Initialise(FD);
    CdrListeSeries1.Initialise(FD, ExtractSerie, nil, [tbsSORT, tbsCSV, tbsHELP], True);

    // lister les systèmes de coordonnées
    cmbCodesEPSG.Clear;
    n := FConvertisseurCoords.GetNbSystemes();
    for i := 0 to n - 1 do
    begin
      EWE := FConvertisseurCoords.GetCodeEPSGNomSysteme(i);
      cmbCodesEPSG.Items.add(Format('EPSG:%d - %s', [EWE.CodeEPSG, EWE.NomEPSG]));
    end;
    EWE := FDocTopo.GetCodeEPSGSystemeCoordonnees();
    cmbCodesEPSG.ItemIndex := FConvertisseurCoords.GetIndexSystemeByCodeEPSG(EWE.CodeEPSG);
    // derniers items: code, expé, réseau
    MyCode   := FDocTopo.GetLastCode();
    MyExpe   := FDocTopo.GetLastExpe();
    editCurrNumeroExpe.AsInteger := MyExpe.IDExpe;
    editCurrNumeroCode.AsInteger := MyCode.IDCode;
    editCurrNumeroReseau.AsInteger := FDocTopo.GetNbReseaux() - 1;
    result := True;
  finally
  end;
end;

procedure TdlgMaintenanceDocTopo.ReprojeterLesEntrees(const OldEPSG, NewEPSG: TLabelSystemesCoordsEPSG);
var
  EWE: String;
  Nb, i: Integer;
  MyEntrance: TEntrance;
  SrcPoint, DestPoint: TProjUV;
begin
  EWE := Format('%s.ReprojeterLesEntrees: EPSG:%d -> EPSG:%d', [ClassName, OldEPSG.CodeEPSG, NewEPSG.CodeEPSG]);
  AfficherMessage(EWE);
  AfficherMessageErreur(EWE);
  Nb := FDocTopo.GetNbEntrees();
  for i := 0 to Nb - 1 do
  begin
    MyEntrance := FDocTopo.GetEntree(i);
    SrcPoint.U := MyEntrance.eXEntree;
    SrcPoint.V := MyEntrance.eYEntree;

    DestPoint := FConvertisseurCoords.ConversionSyst1ToSyst2EPSG(OldEPSG.CodeEPSG, NewEPSG.CodeEPSG, SrcPoint);
    MyEntrance.eXEntree := DestPoint.U;
    MyEntrance.eYEntree := DestPoint.V;
    AfficherMessageErreur(Format('--> Entree %d: %s: %.2f, %.2f -> %.2f, %.2f', [i, MyEntrance.eNomEntree, SrcPoint.U, SrcPoint.V, DestPoint.U, DestPoint.V]));
    FDocTopo.PutEntree(i, MyEntrance);
  end;
  // et on redéfinit le point zéro
  MyEntrance := FDocTopo.GetEntree(0);
  FDocTopo.SetRefSeriePoint(MyEntrance.eRefSer, MyEntrance.eRefSt);
  FDocTopo.SetDefaultCoords(MyEntrance.eXEntree, MyEntrance.eYEntree, MyEntrance.eZEntree);

end;

procedure TdlgMaintenanceDocTopo.Finaliser();
begin
  pass;
end;

function TdlgMaintenanceDocTopo.DoPostAction(): boolean;
begin
  result := FDoPostAction;
end;

end.

