unit frmPrintingCenter;
// Réutilise des éléments du fichier dlgPrintCenter.pas de GHTopo
// Statut: Fonctionnel mais vérifier les pbs avec PDFCreator
// Initialisation de l'imprimante: Nécessite le paquet Printers4Lazarus,
// qui est à ajouter à la liste des paquets requis par GHTopo
// Ouvrir l'inspecteur de projet
// Cliquer sur [ + ] en tete de la fenetre
// Une fenetre s'ouvre
// Onglet __/ Nouvelle Condition \__
// Dérouler la première combobox
// Sélectionner 'Printer4Lazarus'
// Les unités Printers et OSPrinters sont INDISPENSABLES !!
// Printer est initialisé par le paquet. Ne pas utiliser  TPrinter.Create;
// Eventuellement: Mettre TOUT LE CONTENU du dossier ../printers/win32/ dans ../printers/
// 04/02/2014: Pb des accents fixé
// 04/02/2014: Facteurs d'échelle
// 04/02/2014: Les courbes sans barbules utilisent le dessin des courbes de Bezier natives
// 19/02/2015: Support des polylignes
//******************************************************************************
// Nota: Les fortes différences du Canvas d'imprimante et ceux des Canvas d'écran
//       ou d'image, ainsi que les exigences spécifiques à l'impression (affichage ou non
//       en fonction de l'échelle notamment) ne permettent pas une factorisation
//       efficace du code.
{$INCLUDE CompilationParameters.inc}
interface
uses
  SysUtils
  {$IFDEF LANGUAGE_FRENCH}
  , UnitMessages_fr
  {$ENDIF}
  , GHCD_Types
  , GeneralFunctions
  , CallDialogs
  , Math
  , UnitDocDessin
  , types
  {$IFDEF MSWINDOWS}
  , Windows
  {$ENDIF}
  , Classes
  , Graphics
  , Printers
  //, OSPrinters, // OSPrinters est INDISPENSABLE ici !!
  , PrintersDlgs
  , Printer4Lazarus
  , Controls, Forms, Dialogs
  , StdCtrls, ExtCtrls, ComCtrls, Buttons, ToolEdit, CurrEdit;

type TModeTravailPlan = (mtpNONE, mtpPREMIER_COIN, mtpSECOND_COIN, mtpCOIN_DE_BASE);
type TArray2DOfBool = array of array of boolean;

type TPageProperties = record
  Portrait       : boolean;

  PageWidthInMM  : double;
  PageHeightInMM : double;
end;
type

  { TdlgPrintingCenter }

  TdlgPrintingCenter = class(TForm)
    btnStylesObjets: TBitBtn;
    btnImprimer: TBitBtn;
    btnClose: TBitBtn;
    btnApplyEchelle: TButton;
    btnDefinirZone: TButton;
    chkDoPrintPhotos: TCheckBox;
    editHautBarreEchelle: TCurrencyEdit;
    editLongBarreEchelle: TCurrencyEdit;
    editCrossSize: TCurrencyEdit;
    editEtendueX1: TCurrencyEdit;
    editEtendueY1: TCurrencyEdit;
    editZoneDessMinX: TCurrencyEdit;
    editZoneDessMaxX: TCurrencyEdit;
    editEtendueX2: TCurrencyEdit;
    editZoneDessMinY: TCurrencyEdit;
    editZoneDessMaxY: TCurrencyEdit;
    editEtendueY2: TCurrencyEdit;
    grbxPrinters: TGroupBox;
    grbxQuadrillage: TGroupBox;

    cmbPrinters: TComboBox;   // OK
    grbxZoneDessin: TGroupBox;
    grbxEchelle: TGroupBox;
    grbxElementsDessines: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    lbRegle: TLabel;
    lbTailleCroix: TLabel;
    lbEchelle: TLabel;
    lbDessinConduits: TLabel;
    lbCurrentPrinter: TLabel;
    lbMaxi1: TLabel;
    lbMini1: TLabel;
    lbModeTravail: TLabel;
    lbMessages: TLabel;
    lbGrdSpacing: TLabel;
    lbTypeGrd: TLabel;
    lbPrintCurrent: TLabel;
    lbNbPages: TStaticText;
    lbQdrColor: TStaticText;

    lbOrientation: TLabel;    // OK
    btnConfigPrinter: TButton; //OK
    lbPageFormat: TLabel;

    pnlCadre: TPanel;
    PaintBoxVue: TPaintBox;
    lbMouseCoordinates: TStaticText;

    pnlProgressPrinting: TPanel;
    progbarPrinting: TProgressBar;

    lbMini: TLabel;
    lbMaxi: TLabel;
    lbYAxis: TLabel;
    lbXAxis: TLabel;

    editEchelle: TComboBox;
    cmbModeDessinConduits: TComboBox;
    cmbUsualQdrSpacing: TComboBox;
    cmbTypeQdr: TComboBox;

    procedure btnDefinirZoneClick(Sender: TObject);
    procedure btnDoPrintClick(Sender: TObject);
    procedure btnImprimerClick(Sender: TObject);
    procedure btnStylesObjetsClick(Sender: TObject);
    procedure chkDoPrintPhotosChange(Sender: TObject);
    procedure cmbPrintersChange(Sender: TObject);
    procedure editEchelleChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    function  InitializePrintCenter(const ParamsGrilleDessin: TParamsGrilleDessin): integer;
    procedure FinalizePrintCenter;
    // définition du crayon
    procedure DefineCrayon(const qStyle: TPenStyle;
                       const qWidth: double;
                       const qColor: TColor);
    procedure PaintBoxVueMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

    // paramétrage de la  grille
    procedure SetGridParameters(const PGD: TParamsGrilleDessin);
    // conversion de coordonnées
    function GetCoordsPlan (const PM: TPoint2Df): TPoint;
    function GetCoordsMonde(const PP: TPoint): TPoint2Df;

    // conversion de coordonnées imprimante
    function PrinterCoord2Draft(const L,C: integer; const Xf, Yf, Echelle: double): TPoint;

    // cadrage du dessin
    procedure Recadrer;
    // dessin aperçcu
    procedure DrawApercu;
    // choix de l'imprimante
    procedure ChoosePrinter(const Idx: integer);
    procedure SetMyCurrentPrinter;
    // calcul du nombre de pages et exclusion des pages vides
    procedure CalcNbPagesAndEmpty;
    // retourne les index de pages en fonction des coordonnées réelles
    procedure CalcIndexPageIncludingPt(const Pt: TPoint2Df; var IdxX, IdxY: integer);
    // imprimer la topo
    procedure PrintTopo;
    procedure PrintAPage(const PageProp: TPageProperties;
                         const Echelle: double;
                         const L,C: integer;
                         const DoPrintPhoto: boolean);
    // définition des styles
    function DefinitStyleEtVisibiliteCourbe(const ILS: TNatureObjetCourbe): boolean;
    function DefinitStyleEtVisibiliteLigne(const ILS: TNatureObjetLigne): boolean;
    function DefinitStyleEtVisibilitePolygone(const IPS: TNatureObjetPolygone): boolean;
    function DefinitStyleEtVisibiliteTexte(const IPS: TNatureObjetTexte; out TS: TStyleTexte): boolean;


    // dessin des entités
    // dessin d'un triangle, très utilisé => procédure distincte


    // définir mode de travail
    procedure SetModeTravail(const M: TModeTravailPlan);
    //--------------------------------------------------------------------------
    procedure btnConfigPrinterClick(Sender: TObject);
    procedure cbPrintersChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure PaintBoxVuePaint(Sender: TObject);
    procedure PaintBoxVueDblClick(Sender: TObject);
    procedure PaintBoxVueMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure cmbTypeQdrDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure Button2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure cmbUsualQdrSpacingChange(Sender: TObject);
    procedure cmbTypeQdrChange(Sender: TObject);
    procedure btnApplyEchelleClick(Sender: TObject);
    procedure lbQdrColorClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure editCrossSizeChange(Sender: TObject);
    procedure chkDoPrintPhotosClick(Sender: TObject);
    procedure btnPlanFullPageClick(Sender: TObject);
  private
    // coodonnées souris
    FPP : TPoint;
    { Déclarations privées }
    FDocumentDessin: TDocumentDessin;
    FPrinter        : TPrinter;
    FPageProperties : TPageProperties;
    // quadrillage
    FParamsGrilleDessin: TParamsGrilleDessin;
    // limites de la topographie
    FRMini      : TPoint3Df;
    FRMaxi      : TPoint3Df;

    // limites de la zone de dessin
    FZDessMini  : TPoint3Df;
    FZDessMaxi  : TPoint3Df;
    FZP1: TPoint;
    FZP2: TPoint;
    FAttenduSecondPoint : boolean;

    // nombres de pages
    FNbPagesX,
    FNbPagesY   : integer;
    FTableauPagesDrawn: TArray2DOfBool; //array of array of boolean;
    FDoDrawPages      : boolean;
    // échelle du dessin exprimée sous forme de coefficient; eg: Ech 1/1000 -> FEchelle = 0.001
    FEchelle: TFacteurEchelle;
    // barre d'échelle
    FLongBarreEchelle: double;
    FHautBarreEchelle: double;

    // variables diverses

    FRappScrReal : double;
    FInvRappScrReal: double;
    // Bitmaps de remplissage
    FbmpClay   : TBitmap;
    FbmpSand   : TBitmap;
    FbmpEboulis: TBitmap;
    FbmpGalets : TBitMap;
    FbmpSnow   : TBitmap;
    FbmpClayGalets: TBitmap;
    // mode de travail
    // 0 = rien
    // 1 = zone d'impression
    FModeTravailPlan: TModeTravailPlan;
    // imprimer les photos
    FDoPrintPhotos  : boolean;
    function  CalcFontHeightFromHauteurEnMetres(const Hauteur: double): integer;
    procedure TracerRotatedTexte(const ACanvas: TCanvas; const X, Y,
      Angle: integer; const Texte: string);
  public
    { Déclarations publiques }
    property MyDocumentDessin: TDocumentDessin read FDocumentDessin write FDocumentDessin;
  end;

var
  dlgPrintingCenter: TdlgPrintingCenter;

implementation
//uses
  //frmStylesObjets;

{$R *.lfm}


// -----------------------------------------------------------------------------
function Millimetres2PixelsXf(const PRN: TPrinter; const Millims: double): integer;
begin
  //Result:=trunc(100*GetDeviceCaps(PRN.Handle, LOGPIXELSX)* Millims / 25.40) div 100;
  Result := trunc(100 * PRN.XDPI * Millims / 25.40) div 100;
end;
function Millimetres2PixelsYf(const PRN: TPrinter; const Millims: double): integer;
begin
  //Result:=trunc(100 * GetDeviceCaps(PRN.Handle, LOGPIXELSY)* Millims / 25.40) div 100;
  Result := trunc(100 * PRN.YDPI * Millims / 25.40) div 100;
end;
function Pixels2MillimetresX(const PRN: TPrinter; const Pixls: integer): Double;
begin
  //Result:=25.40 * Pixls / GetDeviceCaps(PRN.Handle, LOGPIXELSX);
  Result:=25.40 * Pixls / PRN.XDPI;
end;
function Pixels2MillimetresY(const PRN: TPrinter; const Pixls: integer): Double;
begin
  //Result:=25.40 * Pixls / GetDeviceCaps(PRN.Handle, LOGPIXELSY);
  Result:=25.40 * Pixls / PRN.YDPI;
end;

function GetPageProperties(const PRN: TPrinter): TPageProperties;
begin
  with Result do begin
    Portrait := (PRN.Orientation = poPortrait);
    PageWidthInMM  := Pixels2MillimetresX(PRN, PRN.PageWidth);
    PageHeightInMM := Pixels2MillimetresY(PRN, PRN.PageHeight);
  end;
end;
//------------------------------------------------------------------------------
// Le TCanvas de Lazarus possède un paramètre 'Orientation' (absent du TCanvas de Delphi 5)
// qui n'est rien d'autre que l'angle de rotation en 1/10 de degré
// 3 instructions toutes simples LOL !
procedure TdlgPrintingCenter.TracerRotatedTexte(const ACanvas: TCanvas; const X, Y, Angle: integer; const Texte: string);
begin
  ACanvas.Font.Orientation := Angle * 10; // angle en 1/10e de degré
  ACanvas.TextOut(X, Y, Texte);
  ACanvas.Font.Orientation := 0; // penser à remettre à l'horizontale XD
end;
// définition du crayon
procedure TdlgPrintingCenter.DefineCrayon(const qStyle: TPenStyle;
                                          const qWidth: double;
                                          const qColor: TColor);
begin
  FPrinter.Canvas.Pen.Style := qStyle;;
  FPrinter.Canvas.Pen.Width := Millimetres2PixelsXf(FPrinter, qWidth);
  FPrinter.Canvas.Pen.Color := qColor; // lar
end;

procedure TdlgPrintingCenter.PaintBoxVueMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
const
  MARGE_10MM = 10.0;
var
  PM: TPoint2Df;
  dx, dy: double;
begin
  FPP := MakeTPoint(X, Y);
  PM := GetCoordsMonde(FPP);
  case FModeTravailPlan of
    mtpNONE: ;
    mtpPREMIER_COIN:
       begin
         FZDessMini := MakeTPoint3Df(PM.X, PM.Y, 0.00);
         FZP1 := FPP;
         FZP2 := FPP;
         SetModeTravail(mtpSECOND_COIN);
       end;
    mtpSECOND_COIN:
       begin
         FZDessMaxi := MakeTPoint3Df(PM.X, PM.Y, 0.00);
         FZP2 := FPP;
         if (FZDessMini.X > FZDessMaxi.X) then Swap(FZDessMini.X, FZDessMaxi.X);
         if (FZDessMini.Y > FZDessMaxi.Y) then Swap(FZDessMini.Y, FZDessMaxi.Y);

         editZoneDessMinX.Value := FZDessMini.X;
         editZoneDessMinY.Value := FZDessMini.Y;

         editZoneDessMaxX.Value := FZDessMaxi.X;
         editZoneDessMaxY.Value := FZDessMaxi.Y;

         CalcNbPagesAndEmpty;
         DrawApercu;
         SetModeTravail(mtpNONE);
       end;
    mtpCOIN_DE_BASE: // dessin d'une zone à partir du point courant, à l'échelle fixée et couvrant toute la page
       begin
         FZDessMini := MakeTPoint3Df(PM.X, PM.Y, 0.00);
         dx := (Pixels2MillimetresX(FPrinter, FPrinter.PageWidth) - MARGE_10MM) / (FEchelle * 1000);
         dy := (Pixels2MillimetresY(FPrinter, FPrinter.PageHeight)- MARGE_10MM) / (FEchelle * 1000);
         FZDessMaxi := MakeTPoint3Df(PM.X + dx, PM.Y + dy, 0.00);
         (*
         ShowMessagefmt('De %.0f, %0f à %0f, %0f - L=%0f H=%.0f - Echelle: %.0f',
                        [FZDessMini.X, FZDessMini.Y,
                         FZDessMaxi.X, FZDessMaxi.Y,
                         FZDessMaxi.X - FZDessMini.X,
                         FZDessMaxi.Y - FZDessMini.Y,
                         1/FEchelle
                        ]);
         //*)
         editZoneDessMinX.Value := FZDessMini.X;
         editZoneDessMinY.Value := FZDessMini.Y;

         editZoneDessMaxX.Value := FZDessMaxi.X;
         editZoneDessMaxY.Value := FZDessMaxi.Y;

         CalcNbPagesAndEmpty;
         DrawApercu;
         SetModeTravail(mtpNONE);
       end;
  end;

end;

// définir mode de travail
procedure TdlgPrintingCenter.SetModeTravail(const M: TModeTravailPlan);
  procedure SetLbls(const MT, Msg: string);
  begin
    lbMessages.Caption    := MT;
    lbModeTravail.Caption := Msg;
  end;
begin
  FAttenduSecondPoint := false;
  FModeTravailPlan := M;
  case FModeTravailPlan of
    mtpNONE:
      begin
        PaintBoxVue.Canvas.Pen.Mode := pmCopy;
        SetLbls('PRET', '');
      end;
    mtpPREMIER_COIN:
      begin
        SetLbls('Zone de dessin', 'Premier  coin');
      end;
    mtpSECOND_COIN:
      begin
        FAttenduSecondPoint := True;
        PaintBoxVue.Canvas.Pen.Mode := pmNotXor;
        SetLbls('Zone de dessin', 'Deuxième coin');
      end;
    mtpCOIN_DE_BASE:
      begin
        SetLbls('Zone de dessin', 'Point de base');
      end
  else SetLbls('PRET', '');
  end;
end;

// paramétrage de la grille
procedure TdlgPrintingCenter.SetGridParameters(const PGD: TParamsGrilleDessin);
begin
  FParamsGrilleDessin := PGD;

  //AfficherMessage(Format('%f',[PGD.GrdSpcMainGrid]));
end;
// imprimer la topo
procedure TdlgPrintingCenter.PrintTopo;
var
  L,C,N,Q: integer;
begin
  AfficherMessage(ClassName, 'PrintTopo');
  pnlProgressPrinting.Visible:=True;
  lbPrintCurrent.Caption:='Démarrage de l''impression';
  // Récupération de toutes les valeurs

  with FPrinter do
  begin
    try
      Refresh;
      //AfficherMessage('>> 001');
      BeginDoc;
      //AfficherMessage('>> 002');
      N:=0;
      progbarPrinting.Min := 0;
      Q := FNbPagesX*FNbPagesY;
      progbarPrinting.Max := Q;
      for C:=0 to FNbPagesX-1 do
      begin
        for L:=0 to FNbPagesY-1 do
        begin
          Inc(N);
          progbarPrinting.Position := N;
          if (Not FTableauPagesDrawn[C,L]) then Continue;
          try
            AfficherMessage(Format('>> %d-%d - 003',[c,l]));
            PrintAPage(FPageProperties,
                       FEchelle,
                       L,C,
                       True); //FDoPrintPhotos);
            Application.ProcessMessages;
            lbPrintCurrent.Caption:=Format('Impression en cours de la page %d/%d',
                                                [N,Q]);
            // nouvelle page
            // DONE: Pb résolu
            // TODO: Implémenter les changements dans GHTopo
            if (N <> Q) then FPrinter.NewPage;
          except
            AfficherMessageErreur(Format('>> *** Erreur impression page %d %d ***',[C, L]));
          end;
        end;
      end;
      EndDoc;
      AfficherMessage('>> Impresion terminée');
    except

    end;
  end;
  pnlProgressPrinting.Visible:=False;
end;
procedure TdlgPrintingCenter.PrintAPage(const PageProp: TPageProperties;
                                        const Echelle: double;
                                        const L,C: integer;
                                        const DoPrintPhoto: boolean);
begin
  ;
end;

//**********************************
// conversion de coordonnées
function TdlgPrintingCenter.GetCoordsPlan (const PM: TPoint2Df): TPoint;
begin
  // rotation et mise à l'échelle
  (* Formule: (F = FRappScrReal)

  | Xp |   | +F    0    -F * FRXMini |   | PM.X |
  |    |   |                         |   |      |
  | Yp | = |  0   -F    +F * FRYMaxi | * | PM.Y |
  |    |   |                         |   |      |
  |  1 |   |  0    0          1      |   |   1  |

  //*)
  //T.X := FZoom * (PM.X * FCosX[FTheta] - PM.Y * FSinX[FTheta]); //+TX
  //T.Y := FZoom * (PM.X * FSinX[FTheta] + PM.Y * FCosX[FTheta]); //
  //T.Y := (PM.Y - FRYMini) * FRappScrReal;
  Result.X:=Round((PM.X - FRMini.X) * FRappScrReal);
  Result.Y:=Round((FRMaxi.Y-PM.Y) * FRappScrReal);
end;
function TdlgPrintingCenter.GetCoordsMonde(const PP: TPoint): TPoint2Df;
begin
  (* Formule: (F = FRappScrReal)

  | Xm |   | +1/F    0     FRXMini   |   | PP.X |
  |    |   |                         |   |      |
  | Ym | = |  0   -1/F     FRYMaxi   | * | PP.Y |
  |    |   |                         |   |      |
  |  1 |   |  0    0          1      |   |   1  |
  //*)
  Result.X:=  FInvRappScrReal * PP.X + FRMini.X;
  Result.Y:= -FInvRappScrReal * PP.Y + FRMaxi.Y;
end;

function TdlgPrintingCenter.PrinterCoord2Draft(const L, C: integer; const Xf,
  Yf, Echelle: double): TPoint;
var
  QXf, QYf: Double;
begin
  // transformation en millimètres et mise à l'échelle
  // utilisation de la zone de dessin
  QXf := ((-FZDessMini.X + Xf) * 1000.00) * Echelle;
  QYf := ((-FZDessMini.Y + Yf) * 1000.00) * Echelle;
  // mode normal
  (*
  QXf := ((-FRMini.X + Xf) * 1000.00) * Echelle;
  QYf := ((-FRMini.Y + Yf) * 1000.00) * Echelle;
  //*)
  Result.x := -FPrinter.PageWidth * C +
               Millimetres2PixelsXf(FPrinter, QXf);
  Result.y := FPrinter.PageHeight * L +
              FPrinter.PageHeight - Millimetres2PixelsYf(FPrinter, QYf);
end;
// def des styles
function TdlgPrintingCenter.DefinitStyleEtVisibiliteCourbe(
  const ILS: TNatureObjetCourbe): boolean;
var
  LS: TStyleCourbe;
begin
  Result := True;
  LS := FDocumentDessin.GetStyleCourbe(Ord(ILS));
  DefineCrayon(psSolid, LS.PrintLineWidth, LS.LineColor); //clBlack);
end;
// définit le style de ligne
// retourne Vrai si la ligne doit être tracée
function TdlgPrintingCenter.DefinitStyleEtVisibiliteLigne(const ILS: TNatureObjetLigne): boolean;
var
  LS: TStyleLigne;
  Q : TColor;
begin
  try
    LS := FDocumentDessin.GetStyleLigne(Ord(ILS));
    Q := IIF((ILS = nolPENTE), LS.LineColor, clBlack);
    DefineCrayon(psSolid, LS.PrintLineWidth, Q); //clBlack);
    Result := (LS.SeuilVisibilite > FEchelle);
  except
    Result := False;
  end;
end;
// style et visibilité d'un polygone
function TdlgPrintingCenter.DefinitStyleEtVisibilitePolygone(
  const IPS: TNatureObjetPolygone): boolean;
var
  PS: TStylePolygone;
  procedure SetSolidRemplissage(const QBackColor: TColor);
  begin
    Printer.Canvas.Brush.Bitmap := nil;  // Vide l'éventuel bitmap (indispensable en Lazarus)
    Printer.Canvas.Brush.Style  := bsSolid;
    Printer.Canvas.Brush.Color  := QBackColor;
  end;
  procedure SetRemplissage(const QBMP: TBitmap; const QBackColor: TColor);
  begin
    //cnv.Brush.Style  := bsClear; // en Delphi
    Printer.Canvas.Brush.Style  := bsSolid; // en Lazarus
    Printer.Canvas.Brush.Color  := QBackColor;
    Printer.Canvas.Brush.Bitmap := QBMP;
  end;
begin
  result := False;
  try
    PS := FDocumentDessin.GetStylePolygone(ord(IPS));

    Printer.Canvas.Pen.Color    := PS.LineColor;
    Printer.Canvas.Brush.Color  := PS.FillColor;
    //Printer.Canvas.Pen.Style    := psClear;
    Printer.Canvas.Pen.Style    := psSolid;
    case IPS of
      nopARGILE   : SetRemplissage(FbmpClay       , clWhite);
      nopSABLE    : SetRemplissage(FbmpSand       , clWhite);
      nopBLOCS    : SetRemplissage(FbmpEboulis    , clSilver);
      nopGALETS   : SetRemplissage(FbmpGalets     , clSilver);
      nopNEIGE    : SetRemplissage(FbmpSnow       , clAqua);
      nopVARVES   : SetRemplissage(FbmpClayGalets , clWhite);
    else
       SetSolidRemplissage(PS.FillColor);
    end;
    Result := (PS.SeuilVisibilite > FEchelle);
  except
  end;
end;
function TdlgPrintingCenter.DefinitStyleEtVisibiliteTexte(const IPS: TNatureObjetTexte; out TS: TStyleTexte): boolean;
begin
  TS := FDocumentDessin.GetStyleTexte(ord(IPS));
  Result := (TS.SeuilVisibilite > FEchelle);
end;



// dessin d'une ligne
procedure TdlgPrintingCenter.PrinterDrawSimpleLigne(const SL: TSimpleLigne;
                                 const Index: integer;
                                 const L, C: integer; const MyEchelle: double);
const
  RR          = 3;
  LONG_FLECHE = 0.25;
  PHI_2       = 20 * PI/180;
var
  //P1, P2: TBaseStation;
  E1, E2, E3, E4: TPoint2Df;
  PP: TPoint;
  Ang   , AAA: double;
  errCode: integer;
begin
  //if (Not DefinitStyleEtVisibiliteLigne(SL.IDStyleLigne)) then Exit;


  FDocumentDessin.GetCoordsGCS(SL.IDBaseStExt1, SL.IDGroupe, SL.OffsetExtr1, E1, errCode);
  if (ErrCode = -1) then Exit;
  FDocumentDessin.GetCoordsGCS(SL.IDBaseStExt2, SL.IDGroupe, SL.OffsetExtr2, E2, errCode);
  if (ErrCode = -1) then Exit;

  PrinterTraceVers(E1.X, E1.Y, False, L, C, MyEchelle);
  PrinterTraceVers(E2.X, E2.Y, True,  L, C, MyEchelle);
  // si le style est une ligne de pente, dessiner une flèche simple
  // à l'extrémité 2
  if (SL.IDStyleLigne = nolPENTE) then
  begin
   // AfficherMessage('kjkj');
    Ang := ArcTan2(E2.Y - E1.Y, E2.X - E1.X) + PI;
    AAA := Ang - PHI_2;
    E3.X  := E2.X + LONG_FLECHE * cos(AAA);
    E3.Y  := E2.Y + LONG_FLECHE * sin(AAA);
    AAA := Ang + PHI_2;
    E4.X  := E2.X + LONG_FLECHE * cos(AAA);
    E4.Y  := E2.Y + LONG_FLECHE * sin(AAA);
    PrinterDrawTriangle(E2, E3, E4, clBlack, clGray, L, C, MyEchelle)

  end;
end;
// dessin d'un triangle
// très utilisé => procédure distincte
procedure TdlgPrintingCenter.PrinterDrawTriangle(const P1, P2, P3: TPoint2Df;
                                                 const LineColor, FillColor : TColor;
                                                 const L, C: integer; const MyEchelle: double);
var
  Sommets: array[0..2] of TPoint;
begin
  Sommets[0] := PrinterCoord2Draft(L, C, P1.X, P1.Y, MyEchelle);
  Sommets[1] := PrinterCoord2Draft(L, C, P2.X, P2.Y, MyEchelle);
  Sommets[2] := PrinterCoord2Draft(L, C, P3.X, P3.Y, MyEchelle);
  Printer.Canvas.Pen.Color   := Linecolor;
  Printer.Canvas.Brush.Color := FillColor;
  Printer.Canvas.Polygon(Sommets)
end;

// tracé de texte avec la fonte et attributs courants
// Alignements:   7    8    9
//                4    5    6
//                1    2    3

procedure TdlgPrintingCenter.PrinterDrawTexte(const Texte: string;
                           const QX, QY: double;
                           const Alignement: byte; // alignement
                           const L,C: integer;
                           const AngleRot: double;
                           const TxtHeight, TxtWidth: double;
                           const MyEchelle: double);
var
  ExTxt: TSize;
  PP   : TPoint;
  PosTXT: TPoint;
begin
  with FPrinter.Canvas do
  begin
    PP := PrinterCoord2Draft(L, C, QX, QY, MyEchelle);
    Font.Height := Millimetres2PixelsXf(FPrinter, TxtHeight);
    ExTxt       := TextExtent(Texte);
    //TextOut(PP.X, PP.Y, Texte);
    // 7 8 9
    // 4 5 6
    // 1 2 3
    //   0
    case Alignement of
      0,1: PosTXT := MakeTPoint(PP.X, PP.Y - ExTxt.cy);
      2  : PosTXT := MakeTPoint(PP.X - ExTxt.cx div 2, PP.Y - ExTxt.cy);
      3  : PosTXT := MakeTPoint(PP.X - ExTxt.cx, PP.Y - ExTxt.cy);
      4  : PosTXT := MakeTPoint(PP.X, PP.Y - ExTxt.cy div 2);
      5  : PosTXT := MakeTPoint(PP.X - ExTxt.cx div 2, PP.Y - ExTxt.cy div 2);
      6  : PosTXT := MakeTPoint(PP.X - ExTxt.cx, PP.Y - ExTxt.cy div 2);
      7  : PosTXT := MakeTPoint(PP.X, PP.Y);
      8  : PosTXT := MakeTPoint(PP.X - ExTxt.cx div 2, PP.Y);
      9  : PosTXT := MakeTPoint(PP.X - ExTxt.cx, PP.Y);
    else
      PosTXT := MakeTPoint(PP.X, PP.Y - ExTxt.cy);
    end;
    TextOut(PosTXT.X, PosTXT.Y, TxtGHCAnsiToUTF8(Texte));
  end;
end;
// tracés de lignes
procedure TdlgPrintingCenter.PrinterTraceVers(const XX, YY: Double; const Drawn: boolean;
                                              const L, C: integer; const MyEchelle: double);
var
  PP: TPoint;
begin
  PP := PrinterCoord2Draft(L, C, XX, YY, MyEchelle);
  if (Drawn) then FPrinter.Canvas.LineTo(PP) else FPrinter.Canvas.MoveTo(PP);
end;
// dessin d'une courbe de Bézier
// TODO: Procédure d'ajout d'un nouveau type d'objet: Ecrire ici le code de dessin
procedure TdlgPrintingCenter.PrinterDrawBezierArc(const B: TBezierArc;
                                                  const Barbule: TBarbule;
                                                  const L,C: integer;
                                                  const MyEchelle: double);
var
  Nb, i : integer;
  PC1, PC2: TPoint2Df; // points de contrôle
  LC: double; // longueur des segments P1>PC1>PC2>P2
  Bezier: TArrayPoints2Df;
  Ang, DeltaX, DeltaY, dx, dy: double;
  L_Barbule: double;
  QArcBezier: array[0..3] of TPoint;
begin
  PC1 := MakeTPoint2Df(B.PT1.X + B.Tgt1.X, B.PT1.Y + B.Tgt1.Y);
  PC2 := MakeTPoint2Df(B.PT2.X + B.Tgt2.X, B.PT2.Y + B.Tgt2.Y);
  // si pas de barbules, on trace une vraie courbe de Bezier
  // les calculs suivants étant inutiles
  if (Barbule = tbNONE) then
  begin
    QArcBezier[0] := PrinterCoord2Draft(L, C, B.PT1.X, B.PT1.Y, MyEchelle);
    QArcBezier[1] := PrinterCoord2Draft(L, C, PC1.X, PC1.Y, MyEchelle);
    QArcBezier[2] := PrinterCoord2Draft(L, C, PC2.X, PC2.Y, MyEchelle);
    QArcBezier[3] := PrinterCoord2Draft(L, C, B.PT2.X, B.PT2.Y, MyEchelle);
    FPrinter.Canvas.PolyBezier(QArcBezier);
  end
  else
  begin // if (Barbule != tbNONE) then
    // calcul du nb de subdivisions
    LC := Hypot(PC1.X - B.PT1.X,
                PC1.Y - B.PT1.Y) +
          Hypot(PC2.X - PC1.X,
                PC2.Y - PC1.Y) +
          Hypot(B.PT2.X - PC2.X,
                B.PT2.Y - PC2.Y);

    Nb := round(LC / B.Pas);
    if Nb=0 then Nb:=1;

    Bezier := CalcBezierCurve(B.PT1, PC1, PC2, B.PT2, Nb);

    // tracé de la courbe sauf pour chenal de voute
    if (Barbule <> tbCHENAL_VOUTE) then
    begin
      PrinterTraceVers(Bezier[0].X, Bezier[0].Y, False, L, C, MyEchelle);
      for i := 0 to High(Bezier) do PrinterTraceVers(Bezier[i].X, Bezier[i].Y, True,
                                                     L, C, MyEchelle);
    end;
    // tracé des barbules
    L_Barbule := B.Pas;
    FPrinter.Canvas.Brush.Color := clSilver;
    //if Not(Barbule = tbNONE) then
    //begin
    FPrinter.Canvas.Pen.Width := 0;
    for i := 1 to High(Bezier) do
    begin
      DeltaY := Bezier[i].Y - Bezier[i-1].Y;
      DeltaX := Bezier[i].X - Bezier[i-1].X;
      Ang    := ArcTan2(DeltaY, DeltaX) + PI_2;
      dx     := L_Barbule * cos(Ang);
      dy     := L_Barbule * sin(Ang);
      DeltaY := 0.5 * DeltaY + Bezier[i-1].Y;
      DeltaX := 0.5 * DeltaX + Bezier[i-1].X;
      case Barbule of
        tbRESSAUT, tbMINI_RESSAUT: // ressauts et petites marches
          begin  // _|_|_|_|
            PrinterTraceVers(DeltaX, DeltaY, False, L, C, MyEchelle);
            PrinterTraceVers(DeltaX+dx, DeltaY+dy, True, L, C, MyEchelle);
          end;
        tbSURPLOMB: // surplombs
          begin
            PrinterTraceVers(Bezier[i].X, Bezier[i].Y, False, L, C, MyEchelle);
            PrinterTraceVers(DeltaX+dx, DeltaY+dy, True, L, C, MyEchelle);
            PrinterTraceVers(Bezier[i-1].X, Bezier[i-1].Y, True, L, C, MyEchelle);
            //VueDessin.TraceVers(Bezier[i].X, Bezier[i].Y, True);
          end;
        tbCHENAL_VOUTE: // _|_  _|_ chenaux de voute
          begin
            if ((i MOD 2) = 0) then
            begin
               PrinterTraceVers(Bezier[i-1].X, Bezier[i-1].Y, False, L, C, MyEchelle);
               PrinterTraceVers(Bezier[i].X, Bezier[i].Y, True, L, C, MyEchelle);
               PrinterTraceVers(DeltaX, DeltaY, False, L, C, MyEchelle);
               PrinterTraceVers(DeltaX+dx, DeltaY+dy, True, L, C, MyEchelle);
            end;
          end;
      end;
    end;
    //end;
  end;
end;
// dessin d'une courbe
procedure TdlgPrintingCenter.PrinterDrawCourbe(const MyCourbe: TCourbe;
                                               const IdxCourbe: Int64;
                                               const L,C: integer;
                                               const MyEchelle: double);
var
  i, NB: integer; // NB Points
  AC    : TArcCourbe;
  AB    : TBezierArc;
  SL    : TStyleCourbe;
  function GetBezierArc(const A: TArcCourbe): TBezierArc;
 // var
    //P1, P2: TBaseStation;
   // PT1, PT2: TPoint2Df;
  var
    errCode: integer;
  begin
    FDocumentDessin.GetCoordsGCS(A.IDStationP1, MyCourbe.IDGroupe, A.OffsetP1, Result.PT1, errCode);
    if (ErrCode = -1) then Exit;
    Result.Tgt1 := MakeTPoint2Df(A.TangP1.X, A.TangP1.Y);
    FDocumentDessin.GetCoordsGCS(A.IDStationP2, MyCourbe.IDGroupe, A.OffsetP2, Result.PT2, errCode);
    if (ErrCode = -1) then Exit;
    Result.Tgt2 := MakeTPoint2Df(A.TangP2.X, A.TangP2.Y);
    Result.Pas    :=  SL.LongBarbules;
  end;
begin
  NB := High(MyCourbe.Arcs) + 1;
  if (Nb < 1) then Exit;
  SL := FDocumentDessin.GetStyleCourbe(Ord(MyCourbe.IDStyleCourbe)); // pour les barbules
  DefinitStyleEtVisibiliteCourbe(MyCourbe.IDStyleCourbe);
  for i:=0 to NB - 1 do begin
    AC := MyCourbe.Arcs[i];
    AB := GetBezierArc(AC);
    PrinterDrawBezierArc(AB, SL.Barbules, L, C, MyEchelle);
  end;
end;
procedure TdlgPrintingCenter.PrinterDrawPolygone(const MyPolygon: TPolygone;
                       const IdxPolygone: Int64;
                       const L,C: integer;
                       const MyEchelle: double);
var
  i, NB: integer; // NB Points
  V: TVertexPolygon;
  //PB: TBaseStation;
  PM: TPoint2Df;
  //PP: TPoint;
  PTSV: array of TPoint;
  CP  : TPoint;  // point central du polygone
  GP  : TGroupeEntites;
  errCode: integer;
begin
  GP := FDocumentDessin.GetGroupeByIDGroupe(MyPolygon.IDGroupe);
  if not(GP.Displayed) then Exit;
  if (Not DefinitStyleEtVisibilitePolygone(MyPolygon.IDStylePolygone)) then Exit;
  NB := High(MyPolygon.Sommets);
  if (Nb < 1) then Exit;
  SetLength(PTSV, NB+1);
  for i:=0 to NB do
  begin
    V := MyPolygon.Sommets[i];
    FDocumentDessin.GetCoordsGCS(V.IDStation, MyPolygon.IDGroupe, V.Offset, PM, errCode);
    if (ErrCode = -1) then Exit;
    PTSV[i] := PrinterCoord2Draft(L, C, PM.X, PM.Y, MyEchelle);
  end;
  FPrinter.Canvas.Polygon(PTSV);
  // dessin des gros blocs
  if (MyPolygon.IDStylePolygone = nopGROS_BLOC) then
  begin
    // calcul du point central
    CP := MakeTPoint(0,0);
    Nb := High(PTSV);
    for i := 0 to NB do CP := MakeTPoint(PTSV[i].X + CP.X, PTSV[i].Y + CP.Y);
    CP.X := CP.X div (Nb+1);
    CP.Y := CP.Y div (Nb+1);
    FPrinter.Canvas.Pen.Style := psSolid;
    FPrinter.Canvas.Pen.Color := clBlack;
    for i := 0 to NB do begin
      FPrinter.Canvas.MoveTo(CP.X, CP.Y);
      FPrinter.Canvas.LineTo(PTSV[i].X , PTSV[i].Y);
    end;
    FPrinter.Canvas.Polyline(PTSV);
    FPrinter.Canvas.MoveTo(PTSV[NB]); FPrinter.Canvas.LineTo(PTSV[0]);
  end;
  // dessin des polygones avec contours (parcours, etc ...)
  if (MyPolygon.IDStylePolygone = nopCHEMINS) then
  begin
    FPrinter.Canvas.Pen.Style := psSolid;
    //FPrinter.Canvas.Pen.Color := psSolid;
    FPrinter.Canvas.Polygon(PTSV);
  end;
  //---------------------------------------------------------------------------
  SetLength(PTSV, 0);
end;
// PrinterDrawPolyline et PrinterDrawPolygone non factorisés
// en raison du rôle de la polyligne dans GHCaveDraw
procedure TdlgPrintingCenter.PrinterDrawPolyline(const MyPolyLine: TPolyLigne;
                                                 const IdxPolyline: Int64;
                                                 const L, C: integer; const MyEchelle: double);
var
  GP: TGroupeEntites;
  i, NB: Integer;
  V: TVertexPolygon;
  PM: TPoint2Df;
  PTSV: array of TPoint;
  errCode: integer;
begin
  GP := FDocumentDessin.GetGroupeByIDGroupe(MyPolyLine.IDGroupe);
  if not(GP.Displayed) then Exit;
  // Rappel: Une polyligne = une courbe du point de vue des styles
  if (Not DefinitStyleEtVisibiliteCourbe(MyPolyLine.IDStylePolyLine)) then Exit;
  NB := High(MyPolyLine.Sommets);
  if (Nb < 1) then Exit;
  SetLength(PTSV, NB+1);
  for i:=0 to NB do
  begin
    V := MyPolyLine.Sommets[i];
    FDocumentDessin.GetCoordsGCS(V.IDStation, MyPolyLine.IDGroupe, V.Offset, PM, errCode);
    if (ErrCode = -1) then Exit;
    PTSV[i] := PrinterCoord2Draft(L, C, PM.X, PM.Y, MyEchelle);
  end;
  FPrinter.Canvas.Pen.Style := psSolid;
  FPrinter.Canvas.Polyline(PTSV);
  //---------------------------------------------------------------------------
  SetLength(PTSV, 0);
end;

// dessin de texte
procedure TdlgPrintingCenter.PrinterDrawTexteObject(const TX: TTextObject;
                               const IdxTxtObj: Int64;
                               const L,C: integer;
                               const MyEchelle: double);
var
  PM: TPoint2Df;
  BP: TBaseStation;
  GP: TGroupeEntites;
  ST: TStyleTexte;
  S: string;
  errCode: integer;
begin
 if (TX.IDStyleTexte = notDEBUG) then Exit; // IDStyleTexte = 0 > débogage, non imprimé.
 if (not DefinitStyleEtVisibiliteTexte(TX.IDStyleTexte, ST)) then Exit;

 FDocumentDessin.GetCoordsGCS(TX.IDBaseSt, TX.IDGroupe, TX.Offset, PM, errCode);
 if (ErrCode = -1) then Exit;
 if (not FDocumentDessin.GetBasePointByIndex(TX.IDBaseSt, BP)) then Exit;


 //S  := LLANFAIR_PG(TX.Text, TX.MaxLength, BP.PosStation.Z, BP.Caption);
 S  := InterpreterTexteAnnotation(TX.Text, TX.MaxLength, BP);
 // fonte
 with FPrinter.Canvas do begin
   Brush.Color := clWhite;
   Font.Name   := ST.FontName;
   Font.Style  := ST.FontStyle;
   Font.Color  := ST.FontColor;
 end;
 PrinterDrawTexte(S,
                  PM.X, PM.Y,
                  TX.Alignment,
                  L, C,
                  0.00,
                  ST.FontPrnHeight, 10.00,
                  MyEchelle);
end;
// initialisation du centre d'impression
function TdlgPrintingCenter.InitializePrintCenter(const ParamsGrilleDessin: TParamsGrilleDessin) : integer;
begin
  AfficherMessage(self.ClassName, 'InitializePrintCenter');
  FParamsGrilleDessin := ParamsGrilleDessin;
  Result:=-1;
  FRappScrReal := 1.00;
  FInvRappScrReal := 1.00;
  FEchelle   := 1/1000.0;
  FDoDrawPages  := False;
  FDoPrintPhotos:= False;
  FAttenduSecondPoint := False;
  // combo des espacements
  with cmbUsualQdrSpacing do begin
     Clear;
     Items.Add(Format('%.0f',[ParamsGrilleDessin.GrdSpcMainGrid])); // item0 = Espacement par défaut
     Items.Add('10');
     Items.Add('20');
     Items.Add('25');
     Items.Add('50');
     Items.Add('100');
     Items.Add('200');
     Items.Add('250');
     Items.Add('500');
     Items.Add('1000');
     ItemIndex := 0;
  end;
  FLongBarreEchelle := ParamsGrilleDessin.GrdSpcMainGrid;
  FHautBarreEchelle := ParamsGrilleDessin.GrdSpcMainGrid / 20;
  // combo des quadrillages
  with cmbTypeQdr do begin
    Clear;
    Items.Add(ListeTypesQuadrillages[0]);
    Items.Add(ListeTypesQuadrillages[1]);
    Items.Add(ListeTypesQuadrillages[2]);
  end;
  cmbTypeQdr.ItemIndex     := Ord(ParamsGrilleDessin.GrdTypeQuadrillage);
  lbQdrColor.Color         := ParamsGrilleDessin.GrdMainGridColor;
  // croix
  editCrossSize.Value      := ParamsGrilleDessin.GrdCrossSize;
  //FListeAnnotations:=TAnnotationList.Create;
  //***********************************************************************
  //   Bug avec le canvas de l'imprimante LaserJet                        *
  //   Utiliser "FPrinter := Printer;" si vous avez cette imprimante      *
  //   sinon, utiliser " FPrinter:=TPrinter.Create;"                      *
  //***********************************************************************
  AfficherMessage('-- Starting printer object');
  FPrinter := Printer;
  //try
    // si aucune imprimante installée: quitter
    if FPrinter.Printers.Count = 0 then begin
      Result:=-2;
      Exit;
    end;
    // initialisation dessin
    Recadrer;
    AfficherMessage('-- 1');

    // limites de la topo
    editEtendueX1.Value  := FRMini.X;
    editEtendueY1.Value  := FRMini.Y;
    editEtendueX2.Value  := FRMaxi.X;
    editEtendueY2.Value  := FRMaxi.Y;

    AfficherMessage('-- 2');
    // zone de dessin par défaut
    FZDessMini := FRMini;
    FZDessMaxi := FRMaxi;
    AfficherMessage('-- 3');
    editZoneDessMinX.Value := FZDessMini.X;
    editZoneDessMaxX.Value := FZDessMaxi.X;

    editZoneDessMinY.Value := FZDessMini.Y;
    editZoneDessMaxY.Value := FZDessMaxi.Y;
    AfficherMessage('-- 4');
    // redessiner
    DrawApercu;
    AfficherMessage('-- 5');
    // mode de travail
    SetModeTravail(mtpNONE);

    Result:=1;
    Exit;
  //except
    Result:=-1;
  //end;

end;
procedure TdlgPrintingCenter.FinalizePrintCenter;
begin
  try
    //***************************************************************
    //   Bug avec le canvas de l'imprimante LaserJet                *
    //   Désactiver cette ligne si vous avez cette imprimante       *
    //***************************************************************
    //   FPrinter.Free;
  finally

  end;
end;
// définir imprimante courante: paramètres, etc ...
procedure TdlgPrintingCenter.SetMyCurrentPrinter;
begin
  AfficherMessage(ClassName, 'SetMyCurrentPrinter');
  FPageProperties := GetPageProperties(FPrinter);
  with FPageProperties do
  begin
    lbOrientation.Caption:=IIF(Portrait, 'PORTRAIT', 'LANDSCAPE');
    lbPageFormat.Caption:=Format('%f mm x %f mm', [PageWidthInMM,
                                                   PageHeightInMM]);
  end;
  // reconstruire le plan
  Recadrer;
  // calculer le nombre de pages
  CalcNbPagesAndEmpty;
  // redessiner
  DrawApercu;;
  //*)
end;
// choix de l'imprimante
procedure TdlgPrintingCenter.ChoosePrinter(const Idx: integer);
begin
  with FPrinter do
  begin
    PrinterIndex:=Idx;
    lbCurrentPrinter.Caption:=Printers[PrinterIndex];
  end;
end;
// cadrage du dessin
// pas besoin de fonction de zoom
// (sinon, on aurait utilisé un TCdrVisualisateur2D)
procedure TdlgPrintingCenter.Recadrer;
var
  Marge: double;
  qdx, qdy: Double;
begin
  AfficherMessage(Format('%s.Recadrer',[ClassName]));
  with FDocumentDessin do
  begin
    FRMini := GetCoordsMini;
    FRMaxi := GetCoordsMaxi;
    Marge:=0.02 * Hypot(FRMaxi.X-FRMini.X,
                        FRMaxi.Y-FRMini.Y);
    FRMini.X:=FRMini.X-Marge;
    FRMaxi.X:=FRMaxi.X+Marge;
    FRMini.Y:=FRMini.Y-Marge;
    FRMaxi.Y:=FRMaxi.Y+Marge;
  end;
  qdx := FRMaxi.X - FRMini.X;
  qdy := FRMaxi.Y - FRMini.Y;
  //pnlCadre.top:=0;
  //pnlCadre.left:=0;

  if (qdx>qdy) then
    FRappScrReal:= pnlCadre.Width / qdx
  else
    FRappScrReal:= pnlCadre.Height / qdy;
  FInvRappScrReal:=1/FRappScrReal;
  AfficherMessage(Format(' --> Mini: %.2f - %.2f', [FRMini.X, FRMaxi.X]));
  AfficherMessage(Format(' --> Mini: %.2f - %.2f', [FRMini.Y, FRMaxi.Y]));
end;

// dessin aperçcu
procedure TdlgPrintingCenter.DrawApercu;
var
  TmpBuffer: TBitmap;
  R        : TRect;
  // dessin pages
  procedure DrawPages;
  var
    i,j,v : integer;
    P1, P2: TPoint;
    PM    : TPoint2Df;
    L1, H1: Double;
    RVS   : TRect2Df;
    BS    : TBaseStation;
    //Grayed: boolean;
  begin
    if not FDoDrawPages then Exit;
    with TmpBuffer.Canvas do begin
      // todo
      Pen.Width   := 0;
      Pen.Color   := clBlue;
      // Brush.Style := bsClear; // en Delphi
      Brush.Style := bsSolid;    // en Lazarus
      Font.Height := 4;
      Font.Color  :=clRed;
      L1 := FPageProperties.PageWidthInMM  / (1000*FEchelle);
      H1 := FPageProperties.PageHeightInMM / (1000*FEchelle) ;
      for i:=0 to FNbPagesX-1 do
        for j:=0 to FNbPagesY-1 do
        begin
          PM := MakeTPoint2Df(FZDessMini.X + i * L1, FZDessMini.Y + j * H1);
          P1 := GetCoordsPlan(PM);
          PM := MakeTPoint2Df(FZDessMini.X + (1+i) * L1, FZDessMini.Y + (1+j) * H1);
          P2   := GetCoordsPlan(PM);
          if (FTableauPagesDrawn[i,j]) then
          begin
            Brush.Color:=clWhite;
            Line(P1.X, P1.Y, P2.X, P2.Y);
          end
          else
          begin
            Brush.Color:=clGray;
          end;
          Rectangle(P1.X, P1.Y, P2.X, P2.Y);
          //TextOut(p1.X + 2, p1.Y-(Font.Height shl 2), format('%d.%d',[i,j]));
          TextOut(p1.X + 2, p1.Y-(Font.Height shl 2),
                  format('%d',[1 + FNbPagesY*i+j]));
        end;
    end;
  end;
  // quadrillages
  procedure DrawQuadrilles;
  var
    QSpc      : double;
    QCrossSize: double;
    P1, P2: TPoint;
    C1, C2: TPoint2Df;
    t: integer;
    A,B, B0: double;
  begin
    QSpc       := FParamsGrilleDessin.GrdSpcMainGrid;
    QCrossSize := FParamsGrilleDessin.GrdCrossSize;
    (*
    AfficherMessage(Format('DrawQuadrilles(Mini: %f %f - Maxi: %f %f (%f %f)- %f',
                       [FZDessMini.X, FZDessMini.Y, FZDessMaxi.X, FZDessMaxi.Y,
                        FZDessMaxi.X - FZDessMini.X,
                        FZDessMaxi.Y - FZDessMini.Y,
                        QSpc]));
    //*)
    with TmpBuffer.Canvas do begin
      Pen.Style := psSolid;
      Pen.Color := FParamsGrilleDessin.GrdMainGridColor;
      Pen.Width :=0;
    end;
    case FParamsGrilleDessin.GrdTypeQuadrillage of
     tqGRID:
      begin
        t := trunc(FZDessMini.X / QSpc);
        A := QSpc * t;
        while (A<FZDessMaxi.X) do
        begin
          C1 := MakeTPoint2Df(A, FZDessMini.Y);
          C2 := MakeTPoint2Df(A, FZDessMaxi.Y);
          P1 := GetCoordsPlan(C1);
          P2 := GetCoordsPlan(C2);
          TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
          A := A + QSpc;
        end;
        //t:=trunc(FRMini.Y / QSpc);
        t := trunc(FZDessMini.Y / QSpc);
        A:=QSpc * t;
        while (A<FZDessMaxi.Y) do
        begin
          C1 := MakeTPoint2Df(FZDessMini.X, A);
          C2 := MakeTPoint2Df(FZDessMaxi.X, A);
          P1 := GetCoordsPlan(C1);
          P2 := GetCoordsPlan(C2);
          TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
          A := A + QSpc;
        end;
      end; // tqGRID
     tqCROSS:
      begin
          t:=trunc(FZDessMini.X / QSpc); A:=QSpc * t;
          t:=trunc(FZDessMini.Y / QSpc); B:=QSpc * t;
          B0:=B;
          while (A<FZDessMaxi.X) do begin
           B:=B0;
           while (B<FZDessMaxi.Y) do begin
             C1.X := A - QCrossSize; C1.Y:=B;
             C2.X := A + QCrossSize; C2.Y:=B ;
             P1 := GetCoordsPlan(C1);
             P2 := GetCoordsPlan(C2);
             TmpBuffer.Canvas.MoveTo(P1);  TmpBuffer.Canvas.LineTo(P2);
             C1.X :=A; C1.Y:=B - QCrossSize;
             C2.X :=A; C2.Y:=B + QCrossSize;
             P1 := GetCoordsPlan(C1);
             P2 := GetCoordsPlan(C2);
             TmpBuffer.Canvas.MoveTo(P1); TmpBuffer.Canvas.LineTo(P2);
             B := B + QSpc;
           end;
           A := A + QSpc;
          end;
          //*)
      end; // tqCROSS

    end; // case QdrType
  end;
  // dessin du cartouche
  procedure DrawCartouche;
  begin

  end;
  // dessin polygonales
  procedure DrawPolygonale;
  var
    i: integer;
    P: TBaseStation;
    PM: TPoint2Df;
    PP: TPoint;
  begin
    with TmpBuffer.Canvas do
    begin
      Pen.Width := 0;
      for i:= 0 to FDocumentDessin.GetNbBaseStations - 1 do
      begin
        P := FDocumentDessin.GetBaseStation(i);
        if (P.TypeStation = 1) then Continue; // entrées
        Pen.Color := P.Couleur;
        PM := MakeTPoint2Df(P.PosExtr0.X, P.PosExtr0.Y);
        PP := GetCoordsPlan(PM);
        MoveTo(PP);
        PM := MakeTPoint2Df(P.PosStation.X, P.PosStation.Y);
        PP := GetCoordsPlan(PM);
        LineTo(PP);
      end;
    end;
  end;
  // dessin zone d'impression
  procedure DrawPrintingZone;
  var
    PM: TPoint2Df;
    P1, P2: TPoint;
  begin
    with TmpBuffer.Canvas do
    begin
      Pen.Width := 0;
      Pen.Color := clRed;
      PM := MakeTPoint2Df(FZDessMini.X, FZDessMini.Y);
      P1 := GetCoordsPlan(PM);
      PM := MakeTPoint2Df(FZDessMaxi.X, FZDessMaxi.Y);
      P2   := GetCoordsPlan(PM);
      MoveTo(P1.X, P1.Y); // ne pas utiliser MakeTPoint
      LineTo(P2.X, P1.Y);
      LineTo(P2.X, P2.Y);
      LineTo(P1.X, P2.Y);
      LineTo(P1.X, P1.Y);
    end;
  end;
  procedure DrawScraps;
  var
    i, s, Nb: Integer;
    MyScrap: TScrap;
    PTS: array of TPoint;
    VX: TVertexPolygon;
    BS: TBaseStation;
    PM: TPoint2Df;
    PP: TPoint;
    NbVtx: Integer;
  begin
    AfficherMessage('---- Scraps');
    Nb := FDocumentDessin.GetNbScraps;
    if (Nb = 0) then Exit;
    for i := 0 to Nb-1 do
    begin
      try
        MyScrap := FDocumentDessin.GetScrap(i);
        NbVtx   := Length(MyScrap.Sommets);
        //AfficherMessage(Format('%d: %s - %d', [i, MyScrap.Nom, NbVtx]));
        SetLength(PTS, NbVtx);
        for s := 0 to NbVtx - 1 do
        begin
          VX := MyScrap.Sommets[s];
          FDocumentDessin.GetBasePointByIndex(VX.IDStation, BS);
          PM := MakeTPoint2Df(BS.PosStation.X + VX.Offset.X,
                              BS.PosStation.Y + VX.Offset.Y);
          PP := GetCoordsPlan(PM);
          PTS[s] := PP;
        end;
        TmpBuffer.Canvas.Pen.Style   := psClear;
        TmpBuffer.Canvas.Brush.Color := MyScrap.Couleur;
        TmpBuffer.Canvas.Brush.Style := bsSolid;
        TmpBuffer.Canvas.Polygon(PTS);

      except
        AfficherMessageErreur('Scrap KO');
      end;
    end;
  end;
begin
  AfficherMessage(Format('%s.DrawApercu',[ClassName]));
  try
    TmpBuffer:=TBitmap.Create;
    with TmpBuffer do begin
      Height:= PaintBoxVue.Height;
      Width := PaintBoxVue.Width;
      R.Left:=PaintBoxVue.Left;
      R.Top :=PaintBoxVue.Top;
      R.Bottom:=PaintBoxVue.Left+PaintBoxVue.Height;
      R.Right :=PaintBoxVue.Top+PaintBoxVue.Width;
      Canvas.Brush.Color:=clAqua;
      Canvas.FillRect(R);
      DrawPages;          // dessin pages
      DrawQuadrilles;     // quadrillage
      DrawScraps;         // dessin des scraps
      DrawPolygonale;     // dessin polygonale pour simple aperçu
      DrawPrintingZone;   // zone d'impression
    end;
    PaintBoxVue.Canvas.CopyRect(R, TmpBuffer.Canvas, R);
  finally
    TmpBuffer.Free;
  end;
end;
// calcul du nombre de pages et exclusion des pages vides
procedure TdlgPrintingCenter.CalcNbPagesAndEmpty;
var
  i,j, v: integer;
  BS    : TBaseStation;
  L1, H1: Double;
  R666  : TRect2Df;
  RVS   : TRect2Df;
  Grayed: boolean;
begin
  AfficherMessage(ClassName, 'CalcNbPagesAndEmpty');

  // calculer le nombre de pages, mode zone d'impression
  //AfficherMessage(Format('--> %f %f - %f %f',[FZDessMini.X, FZDessMini.Y, FZDessMaxi.X, FZDessMaxi.Y]));
  FNbPagesX:=1+Trunc(1000*(FZDessMaxi.X - FZDessMini.X) * FEchelle / FPageProperties.PageWidthInMM);
  FNbPagesY:=1+Trunc(1000*(FZDessMaxi.Y - FZDessMini.Y) * FEchelle / FPageProperties.PageHeightInMM);
  //*)
  lbNbPages.Caption :=Format('%d x %d pages',[FNbPagesX, FNbPagesY]);
  SetLength(FTableauPagesDrawn, FNbPagesX, FNbPagesY);

  L1 := FPageProperties.PageWidthInMM  / (1000*FEchelle);
  H1 := FPageProperties.PageHeightInMM / (1000*FEchelle) ;
  for i:=0 to FNbPagesX-1 do
    for j:=0 to FNbPagesY-1 do begin
      // Mode Zone de dessin
      R666.X1 := FZDessMini.X + i * L1;
      R666.Y1 := FZDessMini.Y + j * H1;

      R666.X2 := FZDessMini.X + (1+i) * L1;
      R666.Y2 := FZDessMini.Y + (1+j) * H1;
      //*)
      (* mode normal
      R666.X1 := FRMini.X + i * L1;
      R666.Y1 := FRMini.Y + j * H1;

      R666.X2 := FRMini.X + (1+i) * L1;
      R666.Y2 := FRMini.Y + (1+j) * H1;
      //*)
      // supprimer les pages vides
      Grayed:=false;
      for v:=1 to FDocumentDessin.GetNbBaseStations - 1 do
      begin
        BS:=FDocumentDessin.GetBaseStation(v);
        if BS.TypeStation = 1 then Continue;

        RVS.X1 := RealMin(BS.PosExtr0.X, BS.PosStation.X);
        RVS.Y1 := RealMin(BS.PosExtr0.Y, BS.PosStation.Y); //Min(E1.Une_Station_1_Y, E1.Une_Station_2_Y);

        RVS.X2 := RealMax(BS.PosExtr0.X, BS.PosStation.X); //Max(E1.Une_Station_1_X, E1.Une_Station_2_X);
        RVS.Y2 := RealMax(BS.PosExtr0.Y, BS.PosStation.Y); //Max(E1.Une_Station_1_Y, E1.Une_Station_2_Y);
        Grayed := Grayed or IntersectRectangles(R666, RVS);
      end;
      FTableauPagesDrawn[i,j] := Grayed;
    end;
    FDoDrawPages:=True;
end;

// zone de la carte

// retourne les index de pages en fonction des coordonnées réelles
procedure TdlgPrintingCenter.CalcIndexPageIncludingPt(const Pt: TPoint2Df;
                                                      var IdxX, IdxY: integer);
var
  i,j   : integer;
  L1, H1: Double;
  R666  : TRect2Df;
begin
  IdxX:=-1;
  IdxY:=-1;
  L1 := FPageProperties.PageWidthInMM  / (1000*FEchelle);
  H1 := FPageProperties.PageHeightInMM / (1000*FEchelle) ;
  for i:=0 to FNbPagesX-1 do
    for j:=0 to FNbPagesY-1 do begin
      R666.X1 := FZDessMini.X + i * L1;
      R666.Y1 := FZDessMini.Y + j * H1;

      R666.X2 := FZDessMini.X + (1+i) * L1;
      R666.Y2 := FZDessMini.Y + (1+j) * H1;
      (* mode normal
      R666.X1 := FRMini.X + i * L1;
      R666.Y1 := FRMini.Y + j * H1;

      R666.X2 := FRMini.X + (1+i) * L1;
      R666.Y2 := FRMini.Y + (1+j) * H1;
      //*)
      if IsInRange(Pt.X, R666.X1, R666.X2) and
         IsInRange(Pt.Y, R666.Y1, R666.Y2) then
      begin
        IdxX:=i; IdxY:=j;
        //AfficherMessage(Format('Page %d, %d sélectionnée',[i,j]));
        Exit;
      end;
    end;  // for i,j
end;


//-------- procs d'événement
procedure TdlgPrintingCenter.btnConfigPrinterClick(Sender: TObject);
begin
  with TPrinterSetupDialog.Create(Application) do
  begin
    try
      try
        if (Execute) then
        begin
          SetMyCurrentPrinter;
        end;
      except

      end;
    finally
      Free;
    end;
  end;
end;

procedure TdlgPrintingCenter.cbPrintersChange(Sender: TObject);
begin

end;

procedure TdlgPrintingCenter.cmbPrintersChange(Sender: TObject);
begin
  ChoosePrinter(cmbPrinters.ItemIndex);
  SetMyCurrentPrinter;
end;

procedure TdlgPrintingCenter.editEchelleChange(Sender: TObject);
begin

end;


procedure TdlgPrintingCenter.btnDoPrintClick(Sender: TObject);
begin

end;

procedure TdlgPrintingCenter.btnDefinirZoneClick(Sender: TObject);
begin
  SetModeTravail(mtpPREMIER_COIN);
end;

procedure TdlgPrintingCenter.btnImprimerClick(Sender: TObject);
begin
   PrintTopo;
end;

procedure TdlgPrintingCenter.btnStylesObjetsClick(Sender: TObject);
begin
  EditerStylesObjets(FDocumentDessin, nil);
end;

procedure TdlgPrintingCenter.chkDoPrintPhotosChange(Sender: TObject);
begin
  FDoPrintPhotos := chkDoPrintPhotos.Checked;
end;

procedure TdlgPrintingCenter.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin
  try
    AfficherMessage('--> Liberation des bitmaps de motifs');
  finally
    FbmpClay.free;
    FbmpSand.free;
    FbmpEboulis.free;
    FbmpGalets.Free;
    FbmpSnow.free;
    FbmpClayGalets.free;
  end;
end;

procedure TdlgPrintingCenter.FormShow(Sender: TObject);
var
  InitialDir: string;
  i: integer;
begin
  // titre
  self.Caption := rsDLG_PRINT_TITRE;
  // captions
  grbxPrinters.Caption          := rsDLG_PRINT_GRBX_PRINTERS;
    btnConfigPrinter.Caption    := rsDLG_PRINT_LB_PARAMETRES_PRN;
  grbxEchelle.Caption           := rsDLG_PRINT_GRBX_ECHELLE;
    lbEchelle.Caption           := rsDLG_PRINT_LB_SCALING;
    lbRegle.Caption             := rsDLG_PRINT_LB_REGLE;
  grbxQuadrillage.Caption       := rsDLG_PRINT_GRBX_QUADRILLAGE;
    lbTypeGrd.Caption           := rsDLG_PRINT_LB_TYPE_QDR;
    lbGrdSpacing.Caption        := rsDLG_PRINT_LB_SPACING;
    lbTailleCroix.Caption       := rsDLG_PRINT_LB_TAILLE_CROIX;
  grbxZoneDessin.Caption        := rsDLG_PRINT_GRBX_ZONEDESSIN;
    btnDefinirZone.Caption      := rsDLG_PRINT_DEFINIR_ZONE;
  grbxElementsDessines.Caption  := rsDLG_PRINT_GRBX_ELEMENTS_DRAWN;
    btnStylesObjets.Caption     := rsDLG_PRINT_BTN_STYLES_OBJETS;

  // bitmaps pour les remplissages
  InitialDir := ExtractFilePath(ParamStr(0)) + 'Icones' + PathDelim;
  FbmpClay    := TBitmap.Create;
  FbmpSand    := TBitmap.Create;
  FbmpEboulis := TBitmap.Create;
  FbmpGalets  := TBitmap.Create;
  FbmpSnow    := TBitmap.Create;
  FbmpClayGalets := TBitmap.Create;
  FbmpClay.LoadFromFile(InitialDir + 'clay.bmp');
  FbmpSand.LoadFromFile(InitialDir + 'sand.bmp');
  FbmpEboulis.LoadFromFile(InitialDir + 'eboulis.bmp');
  FbmpGalets.LoadFromFile(InitialDir + 'galets.bmp');
  FbmpSnow.LoadFromFile(InitialDir + 'snow.bmp');
  FbmpClayGalets.LoadFromFile(InitialDir + 'clay_galets.bmp');

  // liste des imprimantes
  with cmbPrinters do begin
    Clear;
    for i:=0 to FPrinter.Printers.Count-1 do begin
      Items.Add(FPrinter.Printers[i]);
    end;
    //FPrinter.PrinterIndex
    // choix de l'imprimante par défaut
    ChoosePrinter(-1);
    cmbPrinters.ItemIndex:=FPrinter.PrinterIndex;
  end;
  // imprimante courante
  SetMyCurrentPrinter;
end;

procedure TdlgPrintingCenter.PaintBoxVuePaint(Sender: TObject);
begin
  try
    DrawApercu;
  except
  end;
end;

procedure TdlgPrintingCenter.PaintBoxVueDblClick(Sender: TObject);
var
  PM: TPoint2Df;
  IdxX, IdxY: integer;
begin
  PM := GetCoordsMonde(FPP);
  CalcIndexPageIncludingPt(PM, IdxX, IdxY);
  if (IdxX>-1) and (IdxY>-1) then begin
    //AfficherMessage(format('%d %d',[IdxX, IdxY]));
    FTableauPagesDrawn[IdxX, IdxY]:=Not(FTableauPagesDrawn[IdxX, IdxY]);
    DrawApercu;
  end;
end;

procedure TdlgPrintingCenter.PaintBoxVueMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  PM: TPoint2Df;
  procedure QDrawRectangle(const P1, P2: TPoint; const Diagonales: boolean);
  begin
    with PaintBoxVue.Canvas do begin
      MoveTo(P1.X,P1.Y);
      LineTo(P2.X,P1.Y);
      LineTo(P2.X,P2.Y);
      LineTo(P1.X,P2.Y);
      LineTo(P1.X,P1.Y);
      if (Diagonales) then
      begin
        Line(P1.X, P1.Y, P2.X, P2.Y);
        Line(P1.X, P2.Y, P2.X, P1.Y);
      end;
    end;
  end;
  procedure QDrawShapeElastique(const QX, QY: integer; const SType: byte);
  begin
    with PaintBoxVue.Canvas do
    begin
       FZP2.X:=QX;
       FZP2.Y:=QY;
       QDrawRectangle(FZP1, FZP2, True);
    end;
  end;
begin
  try
    if (FAttenduSecondPoint) then
    begin
      QDrawShapeElastique(FPP.X, FPP.Y, 0);
      FPP.X:=X;
      FPP.Y:=Y;
      PM:=GetCoordsMonde(FPP);
      lbMouseCoordinates.Caption:=Format('%.2f, %.2f',[PM.X, PM.Y]);
      QDrawShapeElastique(FPP.X, FPP.Y, 0);
    end;

  except
  end;
end;

procedure TdlgPrintingCenter.FormCreate(Sender: TObject);
begin
  (*
  // dimensionnement en fonction de la taille d'écran
  case Screen.Width of
    800:  begin
            pnlCadre.Width := 510;

          end;
    1024: begin
            pnlCadre.Width := 650;

          end;
    1280: begin
            pnlCadre.Width := 850;
          end;
  end;
  pnlCadre.Height := pnlCadre.Width;
  self.Width      := pnlCadre.Width + pnlCadre.Left + 10;
  self.Height     := pnlCadre.Width + 80;
  //*)
end;

procedure TdlgPrintingCenter.cmbTypeQdrDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Q : integer;
  procedure DessinLigneHZ(const Y, L: integer);
  begin
    cmbTypeQdr.Canvas.MoveTo(Rect.Left  + 2, Y);
    cmbTypeQdr.Canvas.LineTo(Rect.Left + L - 4, Y);
  end;
  procedure DessinLigneVT(const X: integer);
  begin
    cmbTypeQdr.Canvas.MoveTo(X, Rect.Top + 2);
    cmbTypeQdr.Canvas.LineTo(X, Rect.Bottom - 2);
  end;

begin
  // Items.Add('Grille');
  //    Items.Add('Croix');
  //    Items.Add('Points');
  cmbTypeQdr.Canvas.Pen.Color := clBlack;
  (*
  if (odSelected in State) then
    cmbTypeQdr.Canvas.Brush.Color := clAqua
  else
    cmbTypeQdr.Canvas.Brush.Color := clWhite;
  cmbTypeQdr.Canvas.FillRect(Rect);
  //*)
  with cmbTypeQdr do begin
   case Index of
    0: begin
       end;
    1: begin// grille
         Q := cmbTypeQdr.ItemHeight;
         DessinLigneVT(  Q div 4);
         DessinLigneVT(  Q div 2);
         DessinLigneVT(3*Q div 4);
         //Q := Rect.Bottom - Rect.Top;
         DessinLigneHZ(Rect.Top +  Q div 4, Q);
         DessinLigneHZ(Rect.Top +  Q div 2, Q);
         DessinLigneHZ(Rect.Top +3*Q div 4, Q);
       end;
    2: begin // croix
         Q := cmbTypeQdr.ItemHeight;
         //Q := Rect.Right - Rect.Left;
         DessinLigneVT(  Q div 2);
         //Q := Rect.Bottom - Rect.Top;
         DessinLigneHZ(Rect.Top +  Q div 2, Q);

       end;

   else
     ;
   end;
  end;


end;

procedure TdlgPrintingCenter.Button2Click(Sender: TObject);
begin
  PrintTopo;
end;

procedure TdlgPrintingCenter.Button1Click(Sender: TObject);
var
  C, L: integer;
begin
  for C := 0 to FNbPagesX-1 do
    for L:= 0 to FNbPagesY-1 do
      FTableauPagesDrawn[C, L]:= False;
  DrawApercu;
end;

procedure TdlgPrintingCenter.cmbUsualQdrSpacingChange(Sender: TObject);
begin
  FParamsGrilleDessin.GrdSpcMainGrid := StrToFloat(cmbUsualQdrSpacing.Text);
  if FParamsGrilleDessin.GrdSpcMainGrid < QDR_MIN_SPACING then Exit;


  if (FParamsGrilleDessin.GrdSpcMainGrid < QDR_MIN_SPACING) then
     FParamsGrilleDessin.GrdSpcMainGrid := QDR_MIN_SPACING;
  //CalcNbPagesAndEmpty;

  FLongBarreEchelle := FParamsGrilleDessin.GrdSpcMainGrid;
  FHautBarreEchelle := FLongBarreEchelle / 20;
  editLongBarreEchelle.Value := FLongBarreEchelle;
  Recadrer;
  DrawApercu;
end;

procedure TdlgPrintingCenter.cmbTypeQdrChange(Sender: TObject);
begin
  case cmbTypeQdr.ItemIndex of
    0: FParamsGrilleDessin.GrdTypeQuadrillage := tqNONE;
    1: FParamsGrilleDessin.GrdTypeQuadrillage := tqGRID;
    2: FParamsGrilleDessin.GrdTypeQuadrillage := tqCROSS;
  end;
  DrawApercu;
end;

procedure TdlgPrintingCenter.btnApplyEchelleClick(Sender: TObject);
var
  Q: integer;
begin
  Q := StrToIntDef(editEchelle.Text, 0);
  if (Q <> 0) then begin
    FEchelle := 1 / Q;
    CalcNbPagesAndEmpty;
    DrawApercu;
  end;
end;

procedure TdlgPrintingCenter.lbQdrColorClick(Sender: TObject);
begin
  lbQdrColor.Color := ChooseColor(lbQdrColor.Color);
  FParamsGrilleDessin.GrdMainGridColor := lbQdrColor.Color;
  DrawApercu;
end;

procedure TdlgPrintingCenter.Button4Click(Sender: TObject);
begin
  SetModeTravail(mtpPREMIER_COIN);
end;

(*
procedure TdlgPrintingCenter.Button5Click(Sender: TObject);
var
  DLGSTY: TdlgStylesObjets;
begin
  DLGSTY := TdlgStylesObjets.Create(self);
  try
    DLGSTY.MyDocumentDessin := self.FDocumentDessin;
    DLGSTY.ShowModal;
  finally
    DLGSTY.Release;
  end;
end;
//*)
procedure TdlgPrintingCenter.editCrossSizeChange(Sender: TObject);
begin
  FParamsGrilleDessin.GrdCrossSize := editCrossSize.Value;
end;

procedure TdlgPrintingCenter.chkDoPrintPhotosClick(Sender: TObject);
begin
  FDoPrintPhotos := chkDoPrintPhotos.Checked;
end;

procedure TdlgPrintingCenter.btnPlanFullPageClick(Sender: TObject);
begin
  SetModeTravail(mtpCOIN_DE_BASE);
end;

function TdlgPrintingCenter.CalcFontHeightFromHauteurEnMetres(const Hauteur: double): integer;
var
  r: Extended;
  c1, c2: TPoint3Df;
  PP1: TPOINT;
  PP2: TPOINT;
begin
  try
    c1 := FDocumentDessin.GetCoordsMini;
    PP1 := GetCoordsPlan(MakeTPoint2Df(c1.X, c1.Y));
    PP2 := GetCoordsPlan(MakeTPoint2Df(c1.X, c1.Y - Hauteur));
    Result := PP2.Y - PP1.Y;
  except
    Result := 6;
  end;
end;
end.

