unit frmExportToSIG;
{$INCLUDE CompilationParameters.inc}

interface

uses
  GHCD_Types,
  {$IFDEF LANGUAGE_FRENCH}
    UnitMessages_fr,
  {$ENDIF}
  GeneralFunctions, UnitDocDessin, Classes, SysUtils, FileUtil, curredit, Forms,
  Controls, Graphics, Dialogs, Buttons, EditBtn, StdCtrls, ExtCtrls, ComCtrls,
  ColorBox;

type

  { TdlgExportSIG }

  TdlgExportSIG = class(TForm)
    BitBtn1: TBitBtn;
    btnDoExport: TButton;
    chkDoUseDefaultStyle: TCheckBox;
    btnColorSilhouette: TColorButton;
    editMapWidth: TCurrencyEdit;
    editMapHeight: TCurrencyEdit;
    editFileName: TFileNameEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    lbEtape: TLabel;
    pnlProgression: TPanel;
    ProgressBar1: TProgressBar;
    rgbxFormatsOutput: TRadioGroup;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    trkBarTransparence: TTrackBar;
    procedure btnDoExportClick(Sender: TObject);
    procedure rgbxFormatsOutputClick(Sender: TObject);
  private
    { private declarations }
    FMyDocDessin: TDocumentDessin;
    procedure AfficherProgression(const Etape: string; const Min, Max, Position: integer);
    procedure SetFileFilter(const F, D: string);
  public
    { public declarations }
    function Initialiser(const FD: TDocumentDessin): boolean;
  end;

var
  dlgExportSIG: TdlgExportSIG;

implementation

{$R *.lfm}

procedure TdlgExportSIG.AfficherProgression(const Etape: string; const Min, Max, Position: integer);
begin
  lbEtape.Caption := Etape;
  lbEtape.Refresh;
  ProgressBar1.Min := Min;
  ProgressBar1.Max := Max;
  ProgressBar1.Position := Position;
end;

procedure TdlgExportSIG.btnDoExportClick(Sender: TObject);
begin
  pnlProgression.Visible := true;
  FMyDocDessin.SetProcAfficherProgression(self.AfficherProgression);
  case rgbxFormatsOutput.ItemIndex of
    0: FMyDocDessin.ExporterScrapsToKMLorOSM(editFileName.FileName,
                                             gisGOOGLE_EARTH,
                                             chkDoUseDefaultStyle.Checked,
                                             btnColorSilhouette.ButtonColor,
                                             255 - trkBarTransparence.Position);

    1: FMyDocDessin.ExporterScrapsToKMLorOSM(editFileName.FileName,
                                             gisOPEN_STREET_MAP,
                                             chkDoUseDefaultStyle.Checked,
                                             btnColorSilhouette.ButtonColor,
                                             255 - trkBarTransparence.Position);
    2: FMyDocDessin.GenererCarteLeaflet(editFileName.FileName,
                                             editMapWidth.AsInteger, editMapHeight.AsInteger,
                                             chkDoUseDefaultStyle.Checked,
                                             btnColorSilhouette.ButtonColor,
                                             255 - trkBarTransparence.Position);

  else
    ;
  end;
  FMyDocDessin.SetProcAfficherProgression(nil);
  pnlProgression.Visible := False;
end;

function TdlgExportSIG.Initialiser(const FD: TDocumentDessin): boolean;
begin
  Result := false;
  pnlProgression.Visible := false;
  FMyDocDessin := FD;
  try
    btnDoExport.Caption             := GetResourceString(rsMISC_BTN_DO_EXPORT_SIG);
    chkDoUseDefaultStyle.Caption    := GetResourceString(rsMISC_CHK_USE_DEFAULT_STYLE);
    editMapWidth.AsInteger  := Screen.Width - 60;
    editMapHeight.AsInteger := Screen.Height - 200;


    editFileName.FileName      := 'scraps.kml';
    editFileName.DialogOptions := editFileName.DialogOptions + [ofOverwritePrompt];
    editFileName.InitialDir := GetGHCaveDrawDirectory();
    SetFileFilter('Fichier Google Earth', 'kml');

    rgbxFormatsOutput.ItemIndex    := 0;
    chkDoUseDefaultStyle.Checked   := True;
    btnColorSilhouette.ButtonColor := clBlue;
    trkBarTransparence.Position    := 64;
    result := True;
  except
  end;
end;

procedure TdlgExportSIG.rgbxFormatsOutputClick(Sender: TObject);
begin
  case rgbxFormatsOutput.ItemIndex of
    0: SetFileFilter('Fichier Google Earth', 'kml');
    1: SetFileFilter('Fichier OpenStreetMap', 'osm');
    2: SetFileFilter('Document-programme Leaflet', 'htm');
  end;
end;

procedure TdlgExportSIG.SetFileFilter(const F, D: string);
begin
  editFileName.Filter := Format('%s (*.%s)|*.%s', [F, D, D]);
  editFileName.DefaultExt := '.' + D;
  editFileName.FileName := ChangeFileExt(editFileName.FileName, '.' + D);
end;


end.

