unit frmFrontalSeries;

{$INCLUDE CompilationParameters.inc}

interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  Classes, SysUtils,
  ToporobotClasses2012,
  unitobjetserie,
  UnitEntitesExtended,
  cadreNavigateurSeries, CadreSerieIndependant,
  FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls;

type

  {TfrmGestionSeries }

  TfrmGestionSeries = class(TForm)
    CdrNavigateurSeries1: TCdrNavigateurSeries;
    CdrSerieIndependant1: TCdrSerieIndependant;
    Panel1: TPanel;
    Panel2: TPanel;
    procedure CdrNavigateurSeries1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure lbNomReseauClick(Sender: TObject);
    procedure Panel1Click(Sender: TObject);
    procedure Panel2Click(Sender: TObject);
    procedure pnlEnteteSerieClick(Sender: TObject);
  private
    { private declarations }
    FDocuTopo: TToporobotStructure2012;
    FBDDEntites: TBDDEntites;
    procedure CreateNewSerie();
    procedure PreparerCadreSerie(const S: TObjSerie; const QIdx: integer);
    procedure GotoSerie();
    function  GotoSerieDisAllowed(): Boolean;
    procedure ImplementerModifsSerie();
  public
    { public declarations }
    function Initialiser(const B: TToporobotStructure2012;
                         const E: TBDDEntites): boolean;
    procedure Finaliser();
  end;

var
  frmGestionSeries: TfrmGestionSeries;

implementation

{$R *.lfm}

{ TfrmGestionSeries }

procedure TfrmGestionSeries.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := false;
end;

procedure TfrmGestionSeries.FormCreate(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.GotoSerie();
var
  n: Integer;
  MySerie: TObjSerie;
begin
  n := CdrNavigateurSeries1.GetIdxSerie();
  MySerie := FDocuTopo.GetSerie(n);
  PreparerCadreSerie(MySerie, n);
  affichermessage(format('%d', [n]));
end;

function TfrmGestionSeries.GotoSerieDisAllowed(): Boolean;
begin
  Result := CdrSerieIndependant1.IsDataModified();
end;

procedure TfrmGestionSeries.ImplementerModifsSerie();
var
  n, QOldIdxSerie: Integer;
  EWE: TObjSerie;
begin
  n := CdrNavigateurSeries1.GetIdxSerie();
  if (n = 0) then // DONE: la série 0 n'est pas modifiable
  begin
    ShowMessage(rsMSG_SERIE_INITIALISATION);
    Exit;
  end;
  EWE := FDocuTopo.GetSerie(n);

  QOldIdxSerie := EWE.GetNumeroDeSerie();  // sauvegarder l'ancien numéro de série
  //ShowMessageFmt('Série %d sélectionnée: %d: %s', [n, EWE.GetIndexSerie(), EWE.GetNomSerie()]);
  if (CdrSerieIndependant1.ImplementerModifs()) then
  begin
    CdrSerieIndependant1.RefreshTableaux();
    EWE := FDocuTopo.GetSerie(n);
    // ici, on ne reliste pas la table des séries.
    // comme une série est un pointeur,
    // les modifications apparaîtront dans la liste au prochain clic
    //CdrListeSeries1.Relister(0);
    // si le numéro de série a changé, on liste les antennes
    //if (QOldIdxSerie <> EWE.GetNumeroDeSerie()) then
    //begin
      //CdrAntennes1.RemplirTableau();
    //end;
  end;


end;

procedure TfrmGestionSeries.CdrNavigateurSeries1Click(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.CreateNewSerie;
begin
  FDocuTopo.CreateNewSerie(1,0, -1, GetResourceString(rsNOUVELLE_SERIE));
end;

procedure TfrmGestionSeries.Finaliser();
begin
  ShowMessageFmt('%s.Finaliser()', [classname]);
end;

function TfrmGestionSeries.Initialiser(const B: TToporobotStructure2012;
                                       const E: TBDDEntites): boolean;
const QH = 110;
begin
  result := false;
  FDocuTopo := B;
  FBDDEntites:= E;
  self.Caption := FDocuTopo.GetNomEtude() + ': Séries';
  self.Left := 4;
  self.Top  := QH;
  self.Height := Screen.Height - QH - 120 ;
  CdrNavigateurSeries1.Initialiser(FDocuTopo,
                                   self.GotoSerieDisAllowed,
                                   self.GotoSerie,
                                   self.CreateNewSerie,
                                   self.ImplementerModifsSerie);
  result := True;
end;

procedure TfrmGestionSeries.lbNomReseauClick(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.Panel1Click(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.Panel2Click(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.pnlEnteteSerieClick(Sender: TObject);
begin

end;

procedure TfrmGestionSeries.PreparerCadreSerie(const S: TObjSerie; const QIdx: integer);
begin
  CdrSerieIndependant1.Initialise(FDocuTopo, FBDDEntites, S, QIdx);
end;

end.

