unit frmtabletteeditserie;

// Editeur d'en-tête de série pour la version Tablette de GHTopo

{$INCLUDE CompilationParameters.inc}

interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  ToporobotClasses2012,
  UnitEntitesExtended,
  ObjetSerie,
  CallDialogsStdVersion,
  Classes, SysUtils, FileUtil, curredit, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Buttons;

type

  { TdlgEnteteSerie }

  TdlgEnteteSerie = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    btnGetSerStArriveeByID: TButton;
    btnGetSerStDepartByID: TButton;
    btnSelectReseau: TButton;
    Button1: TButton;
    chkLocked: TCheckBox;
    cmbChance: TComboBox;
    cmbObstacle: TComboBox;
    editCommentaire: TEdit;
    editNomSerie: TEdit;
    editNumeroReseau: TCurrencyEdit;
    editNumeroSerie: TCurrencyEdit;
    editPointArrivee: TCurrencyEdit;
    editPointDepart: TCurrencyEdit;
    editRaideur: TCurrencyEdit;
    editSerieArrivee: TCurrencyEdit;
    editSerieDepart: TCurrencyEdit;
    Label1: TLabel;
    Label2: TLabel;
    lbIDLStArr: TStaticText;
    lbIDLStDep: TStaticText;
    lblArrivee: TLabel;
    lblChances: TLabel;
    lblDepart: TLabel;
    lblObstacles: TLabel;
    lblReseau: TLabel;
    lblSerie: TLabel;
    lblSerieName: TLabel;
    procedure btnGetSerStArriveeByIDClick(Sender: TObject);
    procedure btnGetSerStDepartByIDClick(Sender: TObject);
    procedure btnSelectReseauClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FDocuToporobot: TToporobotStructure2012;
    FBDDEntites: TBDDEntites;
    FCurrentSerie : TObjSerie;

    procedure InitCaptions;

    procedure SetValeursSerie;
    function  GetValeursSerie: boolean;
    { private declarations }

  public
    { public declarations }
    procedure Initialise(const DT: TToporobotStructure2012; const ET: TBDDEntites; const QS: TObjSerie);
    function  GetSerie: TObjSerie;
  end;

var
  dlgEnteteSerie: TdlgEnteteSerie;

implementation

{$R *.lfm}
// Un TToporobotStructure2012 est indispensable pour l'appel
// On se réserve aussi la possibilité d'utiliser TBDDEntites pour attrapper les stations les plus proches de l'arrivée
// des sélecteurs de Code et de Expe
procedure TdlgEnteteSerie.Initialise(const DT: TToporobotStructure2012;
                                     const ET: TBDDEntites;
                                     const QS: TObjSerie);
begin
  FDocuToporobot := DT;
  FBDDEntites    := ET;
  FCurrentSerie  := QS;
  InitCaptions;
  SetValeursSerie;
end;

function TdlgEnteteSerie.GetSerie: TObjSerie;
begin
  GetValeursSerie;
  Result := FCurrentSerie;
end;

procedure TdlgEnteteSerie.btnSelectReseauClick(Sender: TObject);
begin
   editNumeroReseau.AsInteger := SelectionDansListe(FDocuToporobot,
                                                    mslRESEAUX,
                                                    editNumeroReseau.AsInteger,
                                                    False);
end;

procedure TdlgEnteteSerie.Button1Click(Sender: TObject);
var
  EWE: TEntiteEtendue;
  QSr, QPt: integer;
begin
   (*
  // si chkLocked est armé, on passe les séries/point des editbox
  if (chkLocked.Checked) then
  begin
    QSr := editSerieArrivee.AsInteger;
    QPt := editPointArrivee.AsInteger;
  end
  else
  begin
    // sinon, recherche de la station la plus proche du point d'extrémité du cheminement courant
    QSr := FCurrentSerie.GetIndexSerie;
    QPt := FCurrentSerie.GetNbVisees - 1;
  end;
  if (ChooseNearStationAtPoint(FBDDEntites, QSr, QPt, EWE)) then
  begin
    ShowMessageFmt('Entite: %d.%d', [EWE.Entite_Serie, EWE.Entite_Station]);
  end;
  //*)
end;

procedure TdlgEnteteSerie.btnGetSerStDepartByIDClick(Sender: TObject);
var
  QSr: LongInt;
  QPt: LongInt;
  EWE: TEntiteEtendue;
  QPosXY: TPoint3Df;
begin
  QSr := editSerieDepart.AsInteger;
  QPt := editPointDepart.AsInteger;
  if (ChooseNearStationAtPoint(FBDDEntites, QSr, QPt, QPosXY, EWE)) then
  begin
    editSerieDepart.AsInteger := EWE.Entite_Serie;
    editPointDepart.AsInteger := EWE.Entite_Station;
  end;
end;

procedure TdlgEnteteSerie.btnGetSerStArriveeByIDClick(Sender: TObject);
var
  QSr: LongInt;
  QPt: LongInt;
  EWE: TEntiteEtendue;
  QPosXYZ: TPoint3Df;
begin
  if (chkLocked.Checked) then
  begin
    QSr := editSerieArrivee.AsInteger;
    QPt := editPointArrivee.AsInteger;
  end
  else
  begin
    // sinon, recherche de la station la plus proche du point d'extrémité du cheminement courant
    QSr := FCurrentSerie.GetIndexSerie;
    QPt := FCurrentSerie.GetNbVisees - 1;
  end;
  if (ChooseNearStationAtPoint(FBDDEntites, QSr, QPt, QPosXYZ, EWE)) then
  begin
    ShowMessageFmt('Entite: %d.%d', []);
    editSerieArrivee.AsInteger := EWE.Entite_Serie;
    editPointArrivee.AsInteger := EWE.Entite_Station;
  end;

  (*
  IDL := '';
  if (GHTopoInputQuery(GetResourceString(rsDLG_FIND_PT_BY_ID_TITLE),
                      GetResourceString(rsDLG_FIND_PT_BY_ID_PROMPT),
                      IDL)) then
  begin
    if (FDocuToporobot.FindPtTopoByCle(IDL, S, P)) then
    begin
      editSerieArrivee.AsInteger := S;
      editPointArrivee.AsInteger := P;
      IDL := '   ' + IDL;
      S := Pos('|', IDL);
      Q := Trim(Copy(IDL, 1, S - 1));
      lbIDLStArr.Caption := Q;
    end
    else
      ShowMessage(GetResourceString(rsMATCHNOTFOUND));
  end;
  //*)
end;

procedure TdlgEnteteSerie.FormShow(Sender: TObject);
begin
  self.Position := poScreenCenter;
end;

procedure TdlgEnteteSerie.InitCaptions;
begin
  //btnValider.Caption  :=rsCDR_SERIE_VALIDATE;
  lblSerie.Caption        := GetResourceString(rsCDR_SERIE_NUMERO);
  lblSerieName.Caption    := GetResourceString(rsCDR_SERIE_NAME);
  lblDepart.Caption       := GetResourceString(rsCDR_SERIE_DEPART);
  lblArrivee.Caption      := GetResourceString(rsCDR_SERIE_ARRIVEE);
  lblChances.Caption      := GetResourceString(rsCDR_SERIE_CHANCE);
  lblObstacles.Caption    := GetResourceString(rsCDR_SERIE_OBSTACLE);
  lblReseau.Caption       := GetResourceString(rsCDR_SERIE_LB_RESEAU);
  chkLocked.Caption       := GetResourceString(rsCDR_SERIE_LOCKED);
  with cmbChance do
  begin
    Clear;
    Items.Add(GetResourceString(rsCDR_SERIE_CHANCE0));
    Items.Add(GetResourceString(rsCDR_SERIE_CHANCE1));
    Items.Add(GetResourceString(rsCDR_SERIE_CHANCE2));
    Items.Add(GetResourceString(rsCDR_SERIE_CHANCE3));
  end;
  with cmbObstacle do
  begin
    Clear;
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE0));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE1));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE2));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE3));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE4));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE5));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE6));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE7));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE8));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE9));
    // spécifique GHTopo ( équivalent à AUTRE pour Toporobot)
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE10));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE11));
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE12));
    // baisodromes ^^
    Items.Add(GetResourceString(rsCDR_SERIE_OBSTACLE13));
    DropDownCount := cmbObstacle.Items.Count;
  end;
end;

procedure TdlgEnteteSerie.SetValeursSerie;
begin
  editNumeroSerie.AsInteger  := FCurrentSerie.GetIndexSerie;
  editNumeroReseau.AsInteger := FCurrentSerie.GetNoReseau;
  editNomSerie.Text          := _AnsiToLCLStr(FCurrentSerie.GetNomSerie);
  editSerieDepart.AsInteger  := FCurrentSerie.GetNoSerieDep;
  editPointDepart.AsInteger  := FCurrentSerie.GetNoPointDep;
  editSerieArrivee.AsInteger := FCurrentSerie.GetNoSerieArr;
  editPointArrivee.AsInteger := FCurrentSerie.GetNoPointArr;
  editRaideur.Value          := FCurrentSerie.GetRaideur;
  cmbChance.ItemIndex        := FCurrentSerie.GetChance;
  cmbObstacle.ItemIndex      := FCurrentSerie.GetObstacle;
  editCommentaire.Text       := _AnsiToLCLStr(FCurrentSerie.GetObsSerie);
end;

// Sauvegarde modifs formulaire et tableau
function TdlgEnteteSerie.GetValeursSerie: boolean;
begin
  Result := False;
  try
    with FCurrentSerie do
    begin
      SetIndexSerie(editNumeroSerie.AsInteger);
      SetNomSerie(Trim(_LCLStrToAnsi(editNomSerie.Text)));
      SetNoReseau(editNumeroReseau.AsInteger);
      SetSeriePtExtremites(editSerieDepart.AsInteger, editPointDepart.AsInteger, editSerieArrivee.AsInteger, editPointArrivee.AsInteger);
      SetChanceObstacle(cmbChance.ItemIndex, cmbObstacle.ItemIndex);
      SetRaideur(editRaideur.Value);
      SetObsSerie(Trim(_LCLStrToAnsi(editCommentaire.Text)));
    end;
    Result := True;
  except
  end;
end;


end.

