{$DEFINE USE_GHTOPO_FPC}
{.$UNDEF USE_GHTOPO_FPC}
{$DEFINE USE_GHCAVEDRAW}
{$UNDEF USE_GHCAVEDRAW}
{$INCLUDE CompilationParameters.inc}

// 07/04/2014: Révision du code
// 08/04/2014:
// -- Ajout de DrawPolygonPlein(): Dessin d'un polygone sans utilisation de CSS
// -- Inversion des paramètres de DrawBezierCurve()
// -- Fonction Initialise(): Nom de l'application automatique
// Dessin de polygones: Utilise 'path'
// 09/04/2014:
// -- ViewBox OK
// -- Export SVG et XHTML
// -- Adaptation dans GHCaveDraw
// 06/05/2015: Modification de l'initialisateur
// Positionnement des symboles et textes:
// - Point de base: toujours x="0,00" y="0.00"
// - utilisation de transform pour mise en position définitive
// transform="translate=(<X>, <Y>) [scale=(<fx, fy>)] [rotate=<angle_in_degrees>]"


unit SVGCanvasUnit;
interface
uses
  SysUtils,
  Classes,
  {$IFDEF USE_GHTOPO_FPC}
    StructuresDonnees,
    Common,
  {$ENDIF}
  {$IFDEF USE_GHCAVEDRAW}
    GHCD_Types,
    GeneralFunctions,
  {$ENDIF}
  Math,
  Graphics;
type

{ TSVGCanvas }

 TSVGCanvas = class
    constructor Create;
    destructor  Destroy; override;
  private
    //FSVGFile: string;
    FDoGenerateXHTML: boolean;
    FCommentaire: string;
    FNbWrittenLines: integer;
    FScale: double;
    FFont : TFontPSProperties;
    FPen  : TPenPSProperties;
    FBrush: TBrushPSProperties;
    FXMin,
    FXMax,
    FYMin,
    FYMax : Double;
    // pour SVG uniquement: définition de la chaine de caractères de description
    // du crayon et de la brosse
    FPenStr  : string;
    FBrushStr: string;
         // limites du dessin
    procedure SetDrawingBounds(const X1, Y1, X2, Y2: Double);

  public
    function  InitializeDocument(const QFileName: string; const DoXHTML: boolean; const QTitre, QDescro: string;
                                 const X1, Y1, X2, Y2: Double): boolean;
    procedure FinalizeDocument();
    // groupes
    procedure BeginGroupe(const GroupName, Description: string; const dx, dy: double);
    procedure BeginSubGroupe(const MainGroupName, SubGroupName, Description, SVGStyle: string);
    procedure EndSubGroupe(const MainGroupName, SubGroupName: string);
    procedure EndGroupe(const GroupName: string);
    // définition des patterns
    procedure BeginPatternsSection();
    procedure EndPatternsSection();
    // définition des feuilles de styles
    procedure BeginStylesSection();
    procedure EndStylesSection();
    procedure WriteStyleSolidLine(const Nomstyle: string; const CouleurLigne: TColor; const LargeurLigne: double; const DescroStyle: string);
    procedure WriteStyleSolidPolygone(const Nomstyle: string; const CouleurLigne, CouleurPolygone: TColor; const FillOpacity: byte; const LargeurLigne: double; const DescroStyle: string);
    procedure WriteStyleText(const Nomstyle: string; const QFontName: string; const QFontAttr: TFontStyles; const QFontColor: TColor; const QFontSize: double; const DescroStyle: string);
    // section dessin (contenu)
    procedure BeginDrawingSection();
    procedure EndDrawingSection();

    // définition des couleurs et fontes
    procedure SetPen(const Value: TPenPSProperties);
    function  GetPen: TPenPSProperties;
    procedure SetBrush(const Value: TBrushPSProperties);
    function  GetBrush(): TBrushPSProperties;
    procedure SetFont(const Value: TFontPSProperties);
    procedure SetDefaultPen(const Value: TPenPSProperties);
    procedure SetDefaultBrush(const Value: TBrushPSProperties);

    //****************************
    // définition de commentaires
    procedure WriteCommentaire(const s: string);
    procedure WriteCommand(const s: string);
    procedure WriteVerbatimLine(const s: string);

    // définition de couches
    procedure BeginLayer(const LayerName: string);
    procedure EndLayer(const LayerName: string);
    // routines de dessin
    procedure DrawPoint(const CSSClass: string; const X,Y: Double);
    procedure DrawLine(const CSSClass: string; const X1, Y1, X2, Y2: Double);
    procedure DrawCircle(const CSSClass: string; const XC, YC, R: double);
    procedure DrawArc(const CSSClass: string; const XC, YC, R, Ang0, Ang1: double; const DoPie: boolean);
    procedure DrawPolylign(const CSSClass: string; const Points: TArrayPoints2Df);
    procedure DrawPolygon(const CSSClass: string; const Points: TArrayPoints2Df);
    procedure DrawPolygoneInscrit(const CSSClass: string; const N: integer; const X0, Y0, R: double);

    procedure DrawPolygonSansStyle(const CouleurLigne, CouleurRemplissage: TColor; const LigneWidth: double; const Points: TArrayPoints2Df; const Closed: boolean);
    procedure DrawPolygonOrPolyline(const CSSClass: string; const Points: TArrayPoints2Df; const Closed: boolean);


    procedure DrawText(const CSSClass: string;  const Alignment: byte; const X,Y, Rotation: Double; const Text: string);
    procedure DrawImage(const FichierImage: string; const X, Y: double; const W, H: double);

    function  SVGColor(const C:TColor):string;
    // conversion de coordonnées repère GHCD vers repère SVG
    function  CoordsGCSToSVG(const MyPt: TPoint2Df): TPoint2Df;
    // dessin d'un  rectangle
    function  DrawRectangle(const QClassStyle: string; const X1, Y1, X2, Y2: double): boolean;
    // dessin d'un symbole
    function  DrawSymbole(const IDSymbole: string; const X, Y, L, H, Rotate: double): boolean;
    // tracé de courbes de Bézier à arcs multiples
    function  DrawBezierCurve(const QClassStyle: string; const BC : TArrayPoints2Df; const Closed: boolean): boolean;
    //*******************
    property  Commentaire: string read FCommentaire write FCommentaire;
    property  Scale: double read FScale write FScale;
end;

implementation
var
  pSVGFILE: TextFile;

procedure DisplayMsg(const Str: string);
begin
  ;//WriteLn(Str);
end;
procedure WriteLine(const S: string);
begin
  WriteLn(pSVGFILE, s);
end;

constructor TSVGCanvas.Create;
begin
 inherited Create;
 DisplayMsg(Format('%s.Create',[ClassName]));
 try

 except
 end;
end;
destructor TSVGCanvas.Destroy;
begin
 DisplayMsg(Format('%s.Free',[ClassName]));
 inherited Destroy;
end;


procedure TSVGCanvas.DrawArc(const CSSClass: string; const XC, YC, R, Ang0, Ang1: double; const DoPie: boolean);
var
  PosCentre, PointDepart, PointArrivee: TPoint2Df;
  AD, AA   : double;
  WU, QCmd: String;
  largeArcFlag: byte;
begin
  largeArcFlag := IIF((AD - AA) <= 180.0, 0, 1);
  AD := degtorad(Ang0);
  AA := degtorad(Ang1);
  PointDepart  := CoordsGCSToSVG(MakeTPoint2Df(XC + R * cos(AD),
                                               YC + R * sin(AD)));
  PointArrivee := CoordsGCSToSVG(MakeTPoint2Df(XC + R * cos(AA),
                                               YC + R * sin(AA)));


  //String d = " M" + departX + "," + departY + " A" + radius + "," + radius + " 0 " + sweepFlag + " 1 " + " " + arriveX + "," + arriveY;
  WU := 'M';
  if (DoPie) then WU += Format(' %.3f, %.3f ', [XC, YC]);
  WU += Format('%.3f, %.3f A %.3f, %.3f %d %d %d %.3f, %.3f',
               [PointDepart.X, PointDepart.Y,
                R, R,
                0, largeArcFlag, 0,
                PointArrivee.X, PointArrivee.Y]);
  if (DoPie) then WU += 'Z';
  QCmd := Format('  <path class="%s" d="%s" />', [CSSClass, WU ]);

  WriteLine(QCmd);
end;



// Conversion des coordonnées
function TSVGCanvas.CoordsGCSToSVG(const MyPt: TPoint2Df): TPoint2Df;
begin
  Result.X := MyPt.X - FXMin;
  //Result.Y := (FYMax - FYMin) - (MyPt.Y - FYMin);
  Result.Y := (FYMax - MyPt.Y);
end;
//**** Définition des pinceaux et couleurs
function  TSVGCanvas.SVGColor(const C:TColor):string;
var
  r,g,b: byte;
begin
  R := Red(C); G := Green(C); B := Blue(C);
  Result:=Format('#%.2X%.2X%.2X',[R,G,B]);
end;

function TSVGCanvas.GetPen: TPenPSProperties;
begin
  Result:=FPen;
end;
procedure TSVGCanvas.SetPen(const Value: TPenPSProperties);
begin
  FPen:=Value;
  FPenStr:=Format('stroke: %s; stroke-width: %dmm;',[SVGColor(FPen.Color), round(FPen.fWidth)]);
end;
procedure TSVGCanvas.SetBrush(const Value: TBrushPSProperties);
begin
  FBrush:=Value;
  FBrushStr := Format('fill="%s" fill-opacity="%.2f"',[SVGColor(FBrush.Color), FBrush.Opacity / 255.0]);
  // if >sans remplissage> then FBrushStr:='fill="none"';
end;
function  TSVGCanvas.GetBrush: TBrushPSProperties;
begin
  Result:=FBrush;
end;

procedure TSVGCanvas.SetFont(const Value: TFontPSProperties);
begin
  ;
end;
procedure TSVGCanvas.SetDefaultPen(const Value: TPenPSProperties);
begin
;
end;
procedure TSVGCanvas.SetDefaultBrush(const Value: TBrushPSProperties);
begin
;
end;
//*******************
procedure TSVGCanvas.SetDrawingBounds(const X1, Y1, X2, Y2: Double);
begin
  AfficherMessage(Format('%s.SetDrawingBounds: (%.f, %.f) -> (%.f, %.f)', [ClassName, X1, Y1, X2, Y2]));
  FXMin:=X1;
  FYMin:=Y1;
  FXMax:=X2;
  FYMax:=Y2;
end;

procedure TSVGCanvas.BeginPatternsSection();
begin
  WriteCommentaire('*** Patterns ***');
  WriteCommand('<defs>');
end;

procedure TSVGCanvas.EndPatternsSection();
begin
  WriteCommand('</defs>');
  WriteCommentaire('*** End Patterns ***');
end;

procedure TSVGCanvas.BeginStylesSection();
begin
  WriteCommentaire('*** Style sheets ***');
  WriteCommand('<defs>');
  WriteCommand('  <style type="text/css">');
  WriteVerbatimLine('    <![CDATA[');
end;

procedure TSVGCanvas.EndStylesSection();
begin
  WriteVerbatimLine('    ]]>');
  WriteCommand('  </style>');
  WriteCommand('</defs>');
  WriteCommentaire('*** End Style sheets ***');
end;

procedure TSVGCanvas.WriteStyleSolidLine(const Nomstyle: string; const CouleurLigne: TColor; const LargeurLigne: double; const DescroStyle: string);
const
  FORMAT_STYLE = '      .%s {stroke:%s; stroke-width:%.2fmm; fill:none} /* %s */';
begin
  WriteVerbatimLine(Format(FORMAT_STYLE, [Nomstyle, ColorToHTMLColor(CouleurLigne), LargeurLigne, DescroStyle]));
end;

procedure TSVGCanvas.WriteStyleSolidPolygone(const Nomstyle: string; const CouleurLigne, CouleurPolygone: TColor; const FillOpacity: byte; const LargeurLigne: double; const DescroStyle: string);
const
  FORMAT_STYLE = '      .%s {stroke:%s; stroke-width:%.2fmm; fill:%s; fill-opacity:%.2f} /* %s */';
begin
  WriteVerbatimLine(Format(FORMAT_STYLE, [Nomstyle,
                           ColorToHTMLColor(CouleurLigne),
                           LargeurLigne,
                           ColorToHTMLColor(CouleurPolygone),
                           FillOpacity / 255.0,
                           DescroStyle]));
end;

procedure TSVGCanvas.WriteStyleText(const Nomstyle: string;
                                    const QFontName: string;
                                    const QFontAttr: TFontStyles;
                                    const QFontColor: TColor;
                                    const QFontSize: double;
                                    const DescroStyle: string);
const
  FORMAT_STYLE = '      .%s {%s} /* %s */';
var
  WU, FS, FB, FF, FH, FC: string;
begin
  WU := 'normal';
  if (fsItalic in QFontAttr) then WU := 'italic';
  FC := '';
  FS := Format('font-style:%s; ', [WU]);
  FB := IIF ((fsBold in QFontAttr), 'font-weight:bold; ', '');
  FF := Format('font-family:%s; ', [QFontName]);
  FH := Format('font-size:%.2fmm; ', [QFontSize]);
  WriteVerbatimLine(Format(FORMAT_STYLE,
                   [Nomstyle,
                    FC + FS + FB + FF + FH,
                    DescroStyle
                   ]));

  (*
  font-size : la taille du texte, avec les unités que vous voulez ;
font-style : peut prendre les valeurs normal, italic pour que le texte soit en italique, et oblique pour que le texte soit oblique (plus penché qu’en italique) ;
font-weight qui peut la valeur bold pour que le texte voulu apparaîsse en gras ;
font-variant qui, lorsque qu’elle prend la valeur small-caps, transforme le texte en petite majuscule ;
font-family : la nom de la police (par exemple : Times New Roman, Bodoni, Garamond, Minion

  //*)
end;

procedure TSVGCanvas.BeginDrawingSection();
begin
  WriteCommentaire('BeginDrawingSection');
end;

procedure TSVGCanvas.EndDrawingSection();
begin
  WriteCommentaire('EndDrawingSection()');
end;


procedure TSVGCanvas.WriteCommentaire(const s: string);
begin
  WriteLine(Format('<!-- %s -->',[s]));
end;
procedure TSVGCanvas.WriteCommand(const s: string);
begin
  WriteLine(Trim(S));
end;
// pour écriture de texte indenté notamment
procedure TSVGCanvas.WriteVerbatimLine(const s: string);
begin
  WriteLine(s);
end;


function  TSVGCanvas.InitializeDocument(const QFileName: string; const DoXHTML: boolean; const QTitre, QDescro: string;
                                        const X1, Y1, X2, Y2: Double): boolean;
const
  CHAR_ENCODING = 'ISO-8859-1';    // UTF-8
  MARGE_PERIMETRIQUE = 20.00;
var
  QC1, QC2: TPoint2Df;
begin
  Result:=False;
  self.SetDrawingBounds(X1, Y1, X2, Y2);
  FDoGenerateXHTML := DoXHTML;
  FNbWrittenLines:=0;
  assignFile(pSVGFILE, QFileName);
  try
    ReWrite(pSVGFILE);
    // conversion
    QC1 := CoordsGCSToSVG(MakeTPoint2Df(FXMin - MARGE_PERIMETRIQUE, FYMin - MARGE_PERIMETRIQUE));
    QC2 := CoordsGCSToSVG(MakeTPoint2Df(FXMax + MARGE_PERIMETRIQUE, FYMax + MARGE_PERIMETRIQUE));
    AfficherMessage(Format('-- X1 = %f, Y1 = %f -> X2 = %f, Y2 = %f', [FXMin, FYMin, FXMax, FYMax]));
    // "0.00 746.00 1302.00 0.00"
    AfficherMessage(Format('-- X1 = %f, Y1 = %f -> X2 = %f, Y2 = %f', [QC1.X, QC2.Y, QC2.X, QC1.Y]));

    //-------------------------------
    // écriture de l'en tête ici
    // ------------------------------
    WriteLine(Format('<?xml version ="1.0" encoding="%s"?>',[CHAR_ENCODING])); //"UTF-8"?>')

    if (FDoGenerateXHTML) then
    begin
      WriteLine('<!DOCTYPE html  PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">');
      WriteLine('<html xmlns="http://www.w3.org/1999/xhtml">');
      WriteLine('<head>');
      WriteLine(Format('<title>%s</title>', [QTitre]));
      WriteLine('</head>');
      WriteLine('<body>');
      WriteLine(Format('<h1>%s</h1>', [QDescro]));
    end
    else
    begin
      WriteLine('<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">');
    end;
    WriteLine('<!-- SVG File generated by ' + ApplicationName);
    WriteLine(Format(' File         : %s',[QFileName]));
    WriteLine(Format(' Date         : %s',[DateTimeToStr(Now)]));
    WriteLine(Format(' Drawing area : (%.2f, %.2f) to (%.2f, %.2f)',[FXMin, FYMin, FXMax, FYMax]));
    WriteLine(Format(' Comments     : %s',[FCommentaire]));
    WriteLine('-->');
    WriteLine(Format('<svg xmlns="http://www.w3.org/2000/svg" ' +
                     'xml:space="preserve" '+
                     'width="%.2f%%" height="%.2f%%" ' +
                     'style="shape-rendering:geometricPrecision; '+
                     'text-rendering:geometricPrecision; image-rendering:optimizeQuality; fill-rule:evenodd; clip-rule:evenodd" ' +
                     'viewBox="%.2f %.2f %.2f %.2f" ' +
                     'preserveAspectRatio="%s" ' +
                     'xmlns:xlink="http://www.w3.org/1999/xlink">' ,
                     [100.0, 100.0,               // 100% de l'espace
                      QC1.X, QC2.Y, QC2.X, QC1.Y, // ordre différent des coordonnées usuelles (dû à la conversion de coordonnées)
                      'xMidYMid meet'             // centré sur la zone
                     ]));
    Result:=True;
  except
    DisplayMsg('Error initializing text file');
    CloseFile(pSVGFILE);
  end;
end;
procedure TSVGCanvas.FinalizeDocument();
begin
  try
    WriteLine('</svg>');
    // XHTML
    if (FDoGenerateXHTML) then
    begin
      WriteLine('</body>');
      WriteLine('</html>');
    end;
  finally
    CloseFile(pSVGFILE);
  end;
end;
//********* Définition de couches
procedure TSVGCanvas.BeginLayer(const LayerName: string);
begin
  WriteLine(Format('<!-- Begin Layer: %s -->',[LayerName]));
  WriteLine(Format('<g id="%s">',[LayerName]));
end;
procedure TSVGCanvas.EndLayer(const LayerName: string);
begin
  WriteLine('</g>');
  WriteLine(Format('<!-- End Layer: %s -->',[LayerName]));
end;
//********* Définition de groupes
procedure TSVGCanvas.BeginGroupe(const GroupName, Description: string; const dx, dy: double);
begin
  WriteLine(Format('  <!-- Begin Groupe: [%s] = %s -->',[GroupName, Description]));
  WriteLine(Format('  <g id="%s" transform="translate(%.2f, %.2f)">',[GroupName, dx, -dy]));
  WriteLine(Format('  <desc>%s</desc>',[Description]));

end;
procedure TSVGCanvas.BeginSubGroupe(const MainGroupName, SubGroupName, Description, SVGStyle: string);
begin
  WriteLine(Format('    <!---- Begin SubGroupe: [%s_%s] = %s -->',[MainGroupName, SubGroupName, Description]));
  WriteLine(Format('    <g id="%s_%s" style="%s"><desc>%s</desc>',[MainGroupName, SVGStyle, Description]));
end;
procedure TSVGCanvas.EndSubGroupe(const MainGroupName, SubGroupName: string);
begin
  WriteLine('    </g>');
  WriteLine(Format('    <!-- End SubGroupe: %s_%s -->',[MainGroupName, SubGroupName]));
end;
procedure TSVGCanvas.EndGroupe(const GroupName: string);
begin
  WriteLine('  </g>');
  WriteLine(Format('  <!-- End Groupe: %s -->',[GroupName]));
end;

//********* routines de dessin
// rectangle
function TSVGCanvas.DrawRectangle(const QClassStyle: string; const X1, Y1, X2, Y2: double): boolean;
var
  P, P1, P2: TPoint2Df;
  dx, dy: double;
begin
  P  := MakeTPoint2Df(X1, Y1);
  P1 := CoordsGCSToSVG(P);
  P  := MakeTPoint2Df(X2, Y2);
  P2 := CoordsGCSToSVG(P);
  dx := Abs(P2.X - P1.X);
  dy := P2.Y - P1.Y;
  if (dy<0) then begin dy := -dy; P1.Y := P2.Y; end;

  Result := True;
  WriteCommand(Format('      <rect class="%s" x="%.2f" y="%.2f" width="%.2f" height="%.2f" />',
                      [QClassStyle,
                       P1.X, P1.Y,
                       dx, dy
                      ]));
end;
// tracé de courbes de Bézier à arcs multiples
function TSVGCanvas.DrawBezierCurve(const QClassStyle: string; const BC : TArrayPoints2Df; const Closed: boolean): boolean;
var
  QCmd: string;
  i, Nb: integer;
  Pt   : TPoint2Df;
begin
  Result := True;
  Nb := High(BC);
  if (Nb < 1) then Exit;
  // tracé du premier point
  Pt := CoordsGCSToSVG(BC[0]);
  QCmd := Format('  <path class="%s" d="M %.2f %.2f C ', [ QClassStyle, Pt.X, Pt.Y ]);
  for i := 1 to Nb - 1 do    // tracé des points intermédiaires
  begin
    Pt   := CoordsGCSToSVG(BC[i]);
    QCmd += Format('%.2f %.2f, ', [ Pt.X, Pt.Y ]);
  end;
  // dernier point et clôture de la commande SVG
  Pt := CoordsGCSToSVG(BC[Nb]);
  QCmd += Format('%.2f %.2f', [Pt.X, Pt.Y]);
  if (Closed) then QCmd += 'Z';
  QCmd += '" />';
  // flush de la commande path
  WriteLine(QCmd);
end;
// dessin d'un symbole
function TSVGCanvas.DrawSymbole(const IDSymbole: string; const X, Y, L, H, Rotate: double): boolean;
var
  PM: TPoint2Df;
begin
  PM := MakeTPoint2Df(X, Y);
  PM := CoordsGCSToSVG(PM);
  WriteLine(Format('<use  x="%f" y="%f" width="%f" height="%f" transform="translate(%.2f, %.2f) rotate(%.2f)" xlink:href="#%s"/>', [0.00, 0.00, L, H, PM.X, PM.Y, Rotate, IDSymbole]));
end;
procedure TSVGCanvas.DrawPoint(const CSSClass: string; const X,Y: Double);
var
  PM: TPoint2Df;
begin
 ;
end;
procedure TSVGCanvas.DrawCircle(const CSSClass: string; const XC, YC, R: double);
var
  PT, PT1: TPoint2Df;
begin
  PT := MakeTPoint2Df(XC, YC);
  PT1 := CoordsGCSToSVG(PT);
  WriteLine(Format('<circle class="%s" cx="%f" cy="%f" r="%f" transform="translate(%.2f, %.2f)" />', [CSSClass, 0.00, 0.00, R, PT1.X, PT1.Y]));
  //<circle cx="20" cy="20" r="20" style="stroke: none; fill: #ff0000;" />
end;

procedure TSVGCanvas.DrawLine(const CSSClass: string; const X1, Y1, X2, Y2: Double);
var
  PT, PT1, PT2: TPoint2Df;
begin
  PT := MakeTPoint2Df(X1, Y1);
  PT1 := CoordsGCSToSVG(PT);
  PT := MakeTPoint2Df(X2, Y2);
  PT2 := CoordsGCSToSVG(PT);
  WriteLine(Format('   <line class="%s" x1="%f" y1="%f" x2="%f" y2="%f"/>',
                   [CSSClass,
                    PT1.X, PT1.Y,
                    PT2.X, PT2.Y
                   ]));
end;

// dessin d'une polyligne
procedure TSVGCanvas.DrawPolylign(const CSSClass: string; const Points: TArrayPoints2Df);
begin
  DrawPolygonOrPolyline(CSSClass, Points, false); // polylign
end;
procedure TSVGCanvas.DrawPolygon(const CSSClass: string; const Points: TArrayPoints2Df);
begin
  DrawPolygonOrPolyline(CSSClass, Points, true);  // polygon
end;
// polygone régulier inscrit dans le cercle de rayon R
procedure TSVGCanvas.DrawPolygoneInscrit(const CSSClass: string; const N: integer; const X0, Y0, R: double);
const PI_SUR_2 = PI/2.0;
var
  Points: TArrayPoints2Df;
  Ang: Extended;
  i: Integer;
  WU: Extended;
begin
  SetLength(Points, n);
  Ang := 2*PI/n;
  for i := 0 to n-1 do
  begin
    WU := i * Ang + PI_SUR_2;
    Points[i].X := X0 + R * cos(WU);
    Points[i].Y := Y0 + R * sin(WU);
  end;
  DrawPolygonOrPolyline(CSSClass, Points, true)
end;

// dessin sans utilisation de styles (indispensable pour vue 3D en dégradé)
procedure TSVGCanvas.DrawPolygonSansStyle(const CouleurLigne, CouleurRemplissage: TColor; const LigneWidth: double; const Points: TArrayPoints2Df; const Closed: boolean);
var
  i: integer;
  Cd: string;
  PX: TPoint2Df;
begin
  if (High(Points)<1) then Exit;
  PX := CoordsGCSToSVG(Points[0]);
  Cd := Format('   <path fill="%s" stroke="%s" stroke-width="%.2f" d="', [ColorToHTMLColor(CouleurRemplissage), ColorToHTMLColor(CouleurLigne), LigneWidth]);
  Cd += Format('M %f, %f ', [PX.X, PX.Y]);
  for i:=1 to High(Points) do begin
    PX := CoordsGCSToSVG(Points[i]);
    Cd := Cd + Format(' L %f, %f',[PX.X, PX.Y])
  end;
  if (Closed) then Cd += 'Z';
  Cd += '"/>';
  WriteLine(Cd);
end;

// dessin d'un polygone ou d'une polyligne
procedure TSVGCanvas.DrawPolygonOrPolyline(const CSSClass: string;
                                           const Points: TArrayPoints2Df;
                                           const Closed: boolean);
var
  i: integer;
  Cd: string;
  PX: TPoint2Df;
begin
  if (High(Points)<1) then Exit;
  PX := CoordsGCSToSVG(Points[0]);
  Cd := Format('<path class="%s" ', [CSSClass]);
  Cd += Format('d="M%f, %f ', [PX.X, PX.Y]);
  for i:=1 to High(Points) do begin
    PX := CoordsGCSToSVG(Points[i]);
    Cd += Format('L %f, %f ',[PX.X, PX.Y]);
  end;
  if (Closed) then Cd += 'Z';
  Cd += '"/>';
  WriteLine(Cd);
end;
// Alignment: idem GHCaveDraw
// Rotation: En degrés et dans le sens trigo, zéro à droite
procedure TSVGCanvas.DrawText(const CSSClass: string; const Alignment: byte; const X,Y, Rotation : Double; const Text: string);
var
  P: TPoint2Df;
  QA: string;
begin
  P := MakeTPoint2Df(X, Y);
  P := CoordsGCSToSVG(P);
  case Alignment of
    0, 1, 4, 7: QA := 'start';
       2, 5, 8: QA := 'middle';
       3, 6, 9: QA := 'end';
  end;
  //Ansi
 // <text class="StationMarkerText" x="0.00" y="0.00" text-anchor="start" transform="translate(237.13, 77.53) rotate(90)"><tspan>-79.52</tspan></text>
  WriteLine(Format('  <text class="%s" x="%f" y="%f" text-anchor="%s" transform="translate(%.2f, %.2f) rotate(%f)"><tspan>%s</tspan></text>',
            [CSSClass, 0.00, 0.00, QA, P.X, P.Y, -Rotation, WideString(Text)]));
end;
// implantation d'une image
procedure TSVGCanvas.DrawImage(const FichierImage: string; const X, Y: double; const W, H: double);
var
  P: TPoint2Df;
begin
  P := MakeTPoint2Df(X, Y);
  P := CoordsGCSToSVG(P);
  WriteVerbatimLine(Format('<image xlink:href="%s" x="%.2f" y="%.2f" width="%.2f" height="%.2f" />',
                           [FichierImage, P.X, P.Y, W, H]));
end;


end.


