// Unité en vigueur 21/01/2014
// 23/01/2014 : Calculs d'altitudes Z = f(x, y)
// 01/06/2015 : Intégration dans GHTopoFPC OK
// 01/06/2015 : Courbes de niveau avec équidistance
// 05/06/2015 : Les maillages 'grille' sont systématiquement convertis en TIN au chargement
// 05/06/2015 : Remplacement des tableaux dynamiques Vertexes et Triangles par des listes génériques
// 05/06/2015 : Transformation d'un maillage 'grille' en maillage 'TIN' OK
// 09/06/2015 : Import des maillages VisualTopo (SUR)
// 25/04/2016 : Révision de cette unité: ajout des () aux fonctions sans paramètres
// 03/07/2018 : Export X3D  (ébauche)
// 07/09/2018 : Profil topographique (classe TProfilTopo): OK pour extraction
// 17/09/2018 : Accélération du tracé des maillages: élimination des éléments hors fenêtre de vue

unit UnitClasseMaillage;
interface
{$INCLUDE CompilationParameters.inc}
uses
  StructuresDonnees,
  {$IFDEF USE_GENERIC_TYPES}
  UnitListesSimplesWithGeneriques,
  {$ELSE}
  UnitListesSimplesWithoutGeneriques,
  {$ENDIF}
  UnitEntitesExtended,
  unitProfilTopo,
  Common,
  math,
  BGRABitmap,
  GL, GLU, //OpenGLContext,
  FastGEO,
  SysUtils,Classes, Graphics, Dialogs;
// intersection droite/triangle
type TIntersectPlanTriangle = record
  X1, Y1, Z1: double;
  X2, Y2, Z2: double;
end;


type TOpenGLColor = array[0..3] of GLFloat;
 // couleurs OpenGL
type TGLColor = record
  R: GLFloat;
  G: GLFloat;
  B: GLFloat;
  A: GLFloat;
end;


type

{ TMaillage }

 TMaillage = class
   constructor Create(AOwner: TObject);

   private
     FBDDEntites    : TBDDEntites; // // pour construction des transects
     ZMatrix        : TMatrix;
     FTableVertex   : TMNTTableVertex;
     FTableTriangles: TMNTTableTriangles;
     //Vertexes   : array of TVertex;
     //Triangles  : array of TTriangleABC;

     FTypeMaillage: TMNTTypeMaillage;
     FMaillageValide        : boolean;
     FModeDessinMaillage    : TMNTModeDessinMaillage;

     FCoordsMini: TPoint3Df;
     FCoordsMaxi: TPoint3Df;

     FNbVertexX : integer;
     FNbVertexY : integer;
     FNomTypeMaillage: string;


     FPasX      : double;
     FPasY      : double;


     FCaption     : string;

     // couleurs et transparence
     FColor,
     FColorMin,
     FColorMax    : TGLColor;
     FAlphaBlendingMaillage: GLdouble;
     FDegrade     : boolean;


     // pointeur sur fonctions de conversion pour le canevas
     FProcGCSToSRC : TProcGCSToSRC;
     // triangles sélectionnés uniquement ?
     FDoDrawSelectedTrianglesOnly: boolean;
     // liste des extractions de profils de terrain
     FListeProfilsTopo: TListeProfilsTopo;


     // les deux types de maillages sont convertis 'in fine' en maillages 'Triangles'
     function ConvertitMaillageGridEnTIN(): boolean;

     function GetCoordosPlan(const PM: TPoint3Df): TPoint;
     function GetCoordosVertexToPlan(const PM: TVertex): TPoint;
     function GetNearTriangleIdx(const QX, QY, QZ: double): integer;
     function GetTriangleIdxContenantPoint(const QX, QY: double): integer;
     function IntersectPlanVerticalAndTriangle(const QT: TMNTTriangleABC; const ExtrProfil1, ExtrProfil2: TPoint3Df; out QIntersect: TIntersectPlanTriangle): boolean;

     function  TriangleContainsPoint(const T: TMNTTriangleABC; const QX, QY: double): boolean;
     procedure SetMaillageValide(const B: boolean; const PrgmLineTag: integer);
     procedure SetTypeMaillage(const B: TMNTTypeMaillage; const PrgmLineTag: integer);

   public

     function  Initialiser(const BE: TBDDEntites): boolean;
     procedure Finaliser();
     function  ReDimMaillage(): int64;
     function  IsValidMaillage(): boolean;
     procedure ConstruireMaillageTriangleWire(const QScale, QMagn: double);
     procedure ConstruireMaillageTriangleSolide(const QScale, QMagn: double);


     procedure ConstruireMaillages(const QScale, QMagn: double);
     procedure CalculerTINNormales(const QScale, QMagn: double);
     procedure AffecterCouleur(const z, zmini, zmaxi: double; const DoDegrade: boolean);
     function  LireFichierMaillage(const FichierMaillage: string): Int64;
     function  ChargerMaillageDepuisHGT(const FichierHGT: string): boolean;
     function  ChargerMaillageDepuisSUR(const FichierSUR: string): boolean; // maillage Visual Topo
     // MNT grille: Transformation en image
     function  MakeImageFromGRD(const DoYReverse: boolean): TBitmap;
     procedure ExportToSUR(const FichierSUR: string);


     // getters et setters vertex
     procedure AddVertex(const V : TVertex); inline;
     function  GetVertex(const Idx: integer): TVertex; inline;
     procedure PutVertex(const Idx: integer; const V: TVertex); inline;
     function  GetNbVertex(): integer; inline;

     // getters et setters triangles
     procedure AddTriangle(const V : TMNTTriangleABC); inline;
     function  GetTriangle(const Idx: integer): TMNTTriangleABC; inline;
     procedure PutTriangle(const Idx: integer; const V: TMNTTriangleABC); inline;
     function  GetNbTriangles(): integer; inline;

     // pour les MNT grilles
     // n'a de sens que si FTypeMaillage = tmREGULAR_GRID
     function  GetNbCellsX(): integer;
     function  GetNbCellsY(): integer;

     // setters et getters
     // mode de dessin
     function  GetModeDessinMaillage(): TMNTModeDessinMaillage; inline;
     procedure SetModeDessinMaillage(const M: TMNTModeDessinMaillage); inline;

     // couleurs
     function  GetDefaultCouleur(): TColor; inline;
     procedure SetDefaultCouleur(const C: TColor); inline;
     function  GetMinCouleur(): TColor; inline;
     procedure SetMinCouleur(const C: TColor); inline;
     function  GetMaxCouleur(): TColor; inline;
     procedure SetMaxCouleur(const C: TColor); inline;
     // alpha
     procedure SetAlphaBlending(const A: double); inline;
     function  GetAlphaBlending(): double; inline;
     // membres à lecture seule
     // vertex et triangles

     // transformation d'un triangle TTriangulation en TGeoTriangle;
     function GetGeoTriangle2D(const QT: TMNTTriangleABC): TGeoTriangle2D;
     function GetGeoTriangle3D(const QT: TMNTTriangleABC): TGeoTriangle3D;
     // calcul du BB d'un triangle
     // -----------------------------------------------------------------------------
     function CalcBoundingBoxTriangle(const T: TMNTTriangleABC): TMNTTriangleABC;



     // -- type de maillage
     function  GetTypeMaillage(): TMNTTypeMaillage; inline;

     // -- points extrêmes du maillage
     function  GetCoordsMini(): TPoint3Df; inline;
     function  GetCoordsMaxi(): TPoint3Df; inline;
     // -- Maillages à pas régulier


     // calcul de la distance au versant
     function CalcDistAzPAuVersant(const QX, QY, QZ: double; out QL, QAz, QP: double): boolean;
     function CalcDistanceAuVersant(const QX, QY, QZ: double): double; overload;
     function CalcDistanceAuVersant(const PT: TPoint3Df): double; overload;
     // tracé du maillage sur un canevas.
     // Paramètres:
     // Cnv: Un TCanvas
     // QP: Une fonction de conversion TPoint2Df vers TPoint
     // Isovaleur: Une altitude; -1 = ne pas calculer
     procedure TracerMaillage(const Bmp: TBGRABitmap;
                              const QP: TProcGCSToSRC;
                              const EquidistanceCourbes: double;
                              const Isovaleurs: TArrayOfFloats;
                              const CouleurCourbeNiveau: TColor;
                              const CouleurIsovaleur: TColor;
                              const DoDrawTriangles: boolean;
                              const DoDrawVertex: boolean;
                              const viewBox: TRect2Df);

     // altitude d'un point du maillage
     function CalcAltitudeMaillageAtPoint(const QX, QY: double; out QZ: double): boolean;
     // dégradé
     procedure SetModeDegrade(const B: boolean); inline;
     function  GetModeDegrade: boolean; inline;

     procedure ExportX3D(const QFichierX3D: TStringDirectoryFilename; const DoXML: boolean);
     // extraction d'un profil topographique entre deux points
     function  ExtractProfilTopo(const Pt1, Pt2: TPoint2Df): boolean;
     procedure AddProfilTopo(const P: TProfilTopo); inline;
     function  GetProfilTopo(const Idx: integer): TProfilTopo; inline;
     procedure PutProfilTopo(const Idx: integer; const P: TProfilTopo); inline;
     procedure RemoveProfilTopo(const Idx: integer);

     function  GetNbProfilsTopo(): integer; inline;
     procedure ListerProfilsTopo();
     // générer un maillage à pas régulier
     // la méthode utilisée est abrutie, conne, bête et méchante:
     // a) Extraction de la Bounding-Box du maillage
     // b) Calcul de l'interdistance en X et Y
     // c) Deux boucles imbriquées, dans lesquelles on calcule l'altitude du point

     // A voir: possibilité d'extraire un 'mouchoir à carreaux' d'orientation quelconque
     // Ici, on extrait des profils équidistants
     function  ConstruireMaillageAPasRegulier(const NbMaillesX, NbMaillesY: integer): boolean;

     procedure SaveProfilsToFile(const QFilename: string);
     procedure LoadProfilsFromFile(const QFilename: string);
end;


implementation

function IsValidTriangle(const TR: TMNTTriangleABC): boolean;
var
  AB, BC, CA: Boolean;
begin
  if ((TR.PointA < 0) or (TR.PointB < 0) or (TR.PointC < 0)) then Exit(false);
  AB := (TR.PointA - TR.PointB) = 0;
  BC := (TR.PointB - TR.PointC) = 0;
  CA := (TR.PointC - TR.PointA) = 0;
  if (AB OR BC OR CA) then Exit(false);
  Result := True;
end;

function MakeTVertex(const QID: Int64; const QX, QY, QZ: double): TVertex;
begin
  Result.ID := QID;
  Result.X := QX;
  Result.Y := QY;
  Result.Z := QZ;
  Result.Norme := 0.00;
  Result.NormX := 0.00;
  Result.NormY := 0.00;
  Result.NormZ := 0.00;
end;

// couleur OpenGL
// Retourne sous forme de couleur OpenGL la couleur passée en argument
function GetGLColor(const Coul: TColor; const Alpha: Double): TGLColor;
const
  m = 1/256;
begin
  Result.R := Red(Coul)* m;
  Result.G := Green(Coul)* m;
  Result.B := Blue(Coul)* m;
  Result.A := Alpha;
end;

function PascalToGLColor(const Coul: TColor): TGLColor;
const
  m = 1/256;
var
  c: TGLColor;
begin
  c.R := Red(Coul)* m;
  c.G := Green(Coul)* m;
  c.B := Blue(Coul)* m;
  c.A := 1.00;
  Result := c;
end;
// Retourne sous forme de couleur Pascal la couleur OpenGL passée en argument
function GLToPascalColor(const Coul: TGLColor): TColor;
const
  m = 256;
begin
  Result := RGBToColor(Round(Coul.R * m), Round(Coul.G * m), Round(Coul.B * m));
end;



// TMaillage
// intersection d'un triangle


procedure TMaillage.CalculerTINNormales(const QScale, QMagn: double);
var
  i, Nb: integer;
  V1, V2, W: TPoint3Df;
  Norm, NormW: double;
  MM: Extended;
  VV: TVertex;
  VA, VB, VC: TVertex;
  TR: TMNTTriangleABC;
begin
  MM := QScale * QMagn;
  AfficherMessage(Format('%s.CalculerTINNormales: %d vertex, %d triangles',[ClassName, GetNbVertex, GetNbTriangles]));
  //FTableVertex.ClearListe;
  Nb := GetNbVertex;
  for i:=0 to Nb -1 do
  begin
    VV := GetVertex(i);

    VV.NormX:=0.0;
    VV.NormY:=0.0;
    VV.NormZ:=0.0;
    VV.Norme:=0.0;
    PutVertex(i, VV);
  end;
  Nb := GetNbTriangles;
  for i:=0 to Nb - 1 do
  begin
    TR := GetTriangle(i);
    with TR do
      begin
        // vecteurs v1 et v2
        VA := GetVertex(PointA);
        VB := GetVertex(PointB);
        VC := GetVertex(PointC);
        V1.X := MM * (VB.X - VA.X);
        V1.Y := MM * (VB.Y - VA.Y);
        V1.Z := MM * (VB.Z - VA.Z);
        V2.X := MM * (VC.X - VA.X);
        V2.Y := MM * (VC.Y - VA.Y);
        V2.Z := MM * (VC.Z - VA.Z);
        // produit vectoriel: normale du triangle
        W.X := V1.Y * V2.Z - V1.Z * V2.Y;
        W.Y := V1.Z * V2.X - V1.X * V2.Z;
        W.Z := V1.X * V2.Y - V1.Y * V2.X;

        Norm:=sqrt(sqr(w.x)+sqr(w.y)+sqr(w.z));
        VA.Norme += Norm;
        VA.NormX += W.X; VA.NormY += W.Y; VA.NormZ += W.Z;
        VB.Norme += Norm;
        VB.NormX += W.X; VB.NormY += W.Y; VB.NormZ += W.Z;
        VC.Norme += Norm;
        VC.NormX += W.X; VC.NormY += W.Y;
        VC.NormZ += W.Z;
        PutVertex(PointA, VA);
        PutVertex(PointB, VB);
        PutVertex(PointC, VC);
      end;
  end;
  // normalisation
  Nb := GetNbVertex;
  for i:= 0 to Nb - 1 do
  begin
    VV := GetVertex(i);

    VV.NormX := VV.NormX/(VV.Norme+1e-12);
    VV.NormY := VV.NormY/(VV.Norme+1e-12);
    VV.NormZ := VV.NormZ/(VV.Norme+1e-12);
    PutVertex(i, VV);
  end;
end;
procedure TMaillage.ConstruireMaillages(const QScale, QMagn: double);
begin
 AfficherMessage(Format('%s.ConstruireMaillages: %d',[ClassName, Ord(FTypeMaillage)]));
 AfficherMessage(Format('Bounds: (%.2f, %.2f, %.2f) -> (%.2f, %.2f, %.2f)',
                        [FCoordsMini.X, FCoordsMini.Y, FCoordsMini.Z,
                         FCoordsMaxi.X, FCoordsMaxi.Y, FCoordsMaxi.Z]));
  //CalculerTINNormales(QScale, QMagn);
  case FModeDessinMaillage of
    M3D_WIRE_FRAME    : ConstruireMaillageTriangleWire(QScale, QMagn);
    M3D_MESH,M3D_BLEND: ConstruireMaillageTriangleSolide(QScale, QMagn);
  end;
end;

function TMaillage.Initialiser(const BE: TBDDEntites): boolean;
begin
  Result := false;
  AfficherMessage(Format('%s.Initialiser',[ClassName]));
  FBDDEntites := BE;
  SetLength(ZMatrix,1, 1);
  SetMaillageValide(false, 359);
  FDegrade          := False;
  FColorMin         := GetGLColor(clRed, 1.00);
  FColorMax         := GetGLColor(clBlue, 1.00);
  FTableVertex.ClearListe;
  FTableTriangles.ClearListe;
  // profil topo
  FListeProfilsTopo.ClearListeProfils();
  Result := true;
end;

procedure TMaillage.Finaliser();
begin
  AfficherMessage(Format('%s.Finaliser',[ClassName]));
  //SetLength(ZMatrix,1, 1);
  // AfficherMessage(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
  try
    FTableVertex.ClearListe;
    FTableTriangles.ClearListe;
    FListeProfilsTopo.ClearListeProfils();
  finally
    FTableVertex.Free;
    FTableTriangles.Free;
    FListeProfilsTopo.Free;
  end;
end;

function TMaillage.ReDimMaillage(): int64;
var
  cx, cy: integer;
begin
  cx := 1 + FNbVertexX ;
  cy := 1 + FNbVertexY;
  AfficherMessage(Format('%s.ReDimMaillage: %d, %d',[ClassName, cx, cy]));
  try
    Result:=-2;
    SetLength(ZMatrix,1, 1);
    SetLength(ZMatrix,   cy, cx);
    //ShowMessageFmt('%d %d', [FNbLignesX,FNbLignesY]);
    Result:= FNbVertexX * FNbVertexY;

  finally
  end;
end;









procedure TMaillage.AffecterCouleur(const z, zmini, zmaxi: double; const DoDegrade: boolean);
var
  c: TGLColor;
begin
  if (DoDegrade) then
  begin
    c:=GetGLColor(GetColorDegrade(z, zmini, zmaxi,
                                  GLToPascalColor(FColorMin),
                                  GLToPascalColor(FColorMax)), FAlphaBlendingMaillage);
  end else c:= FColorMax;
  glColor4fv(@c);
end;

procedure TMaillage.SetModeDessinMaillage(const M: TMNTModeDessinMaillage);
begin
  FModeDessinMaillage := M;
end;

procedure TMaillage.SetTypeMaillage(const B: TMNTTypeMaillage; const PrgmLineTag: integer);
var
  EWE: String;
begin
  EWE := ChooseString(Ord(B), ['tmUNKNOWN', 'tmREGULAR_GRID', 'tmTRIANGLES']);
  AfficherMessageErreur(Format('--> SetMaillageValide: %s - Ligne: %d', [EWE, PrgmLineTag]));

  FTypeMaillage := B;
end;

function TMaillage.GetModeDessinMaillage(): TMNTModeDessinMaillage;
begin
  Result := FModeDessinMaillage;
end;

function TMaillage.GetNbCellsX(): integer;
begin
  Result := -1;
  if (FTypeMaillage = tmREGULAR_GRID) then Result := FNbVertexX - 1;
end;

function TMaillage.GetNbCellsY(): integer;
begin
  Result := -1;
  if (FTypeMaillage = tmREGULAR_GRID) then Result := FNbVertexY - 1;
end;




function TMaillage.IsValidMaillage(): boolean;
begin
  Result := FMaillageValide;
end;





function TMaillage.GetTypeMaillage(): TMNTTypeMaillage;
begin
  Result := FTypeMaillage;
end;



//*)
// distance au versant; retourne long, azimut et pente
function TMaillage.CalcDistAzPAuVersant(const QX, QY, QZ: double; out QL, QAz, QP: double): boolean;
var
  WU: TGeoFloat;
  i, j: Integer;
  S: TPoint3Df;
  _dx, _dy, _dz: double;
  Qdx, Qdy, Qdz: Double;
  EWE: TMNTTriangleABC;
begin
  WU := 1e18;
  Result := False;
  if (not FMaillageValide) then Exit;
  case FTypeMaillage of
    tmUNKNOWN: exit;

    tmTRIANGLES:
      begin


      end;
  otherwise
    pass;
  end;
end;

function TMaillage.CalcDistanceAuVersant(const QX, QY, QZ: double): double;
begin

end;
// teste si le triangle T contient le point
function TMaillage.TriangleContainsPoint(const T: TMNTTriangleABC; const QX, QY: double): boolean;
var
  S1, S2, S3: TVertex;
begin
  Result := False;
  S1 := GetVertex(T.PointA);
  S2 := GetVertex(T.PointB);
  S3 := GetVertex(T.PointC);
  Result := PointInTriangle(QX, QY, S1.X, S1.Y, S2.X, S2.Y, S3.X, S3.Y);
end;


// retourne le index du triangle contenant le point (X, Y) ou -1 si échec
function TMaillage.GetTriangleIdxContenantPoint(const QX, QY: double): integer;
var
  i, n: Integer;
  EWE : TMNTTriangleABC;
begin
  Result := -1;
  n := self.GetNbTriangles();
  if (n = 0) then exit(-1);
  // Faire toujours partir les index de triangles à partir de 1 et non de zéro
  for i := 1 to n - 1 do
  begin
    EWE := GetTriangle(i);
    if (TriangleContainsPoint(EWE, QX, QY)) then Exit(i);
  end;
end;
// calcul de l'altitude du point (x, y)
function TMaillage.CalcAltitudeMaillageAtPoint(const QX, QY: double; out QZ: double): boolean;
var
  n : integer;
  S1, S2, S3: TVertex;
  EWE: TMNTTriangleABC;
begin
  AfficherMessage(Format('%s.CalcAltitudeMaillageAtPoint(%.2f, %.2f)', [ClassName, QX, QY]));
  Result := False;
  n := GetTriangleIdxContenantPoint(QX, QY);
  AfficherMessage(Format('Triangle %d trouve', [n]));
  if (n >= 0) then
  begin
    EWE := GetTriangle(n);
    S1 := GetVertex(EWE.PointA);
    S2 := GetVertex(EWE.PointB);
    S3 := GetVertex(EWE.PointC);
    QZ := GeoCalcAltitudeXYInTriangle(S1.X, S1.Y, S1.Z,
                                      S2.X, S2.Y, S2.Z,
                                      S3.X, S3.Y, S3.Z,
                                      QX, QY);
    Result := True;
  end;
end;

procedure TMaillage.SetModeDegrade(const B: boolean);
begin
  FDegrade := B;
end;

function TMaillage.GetModeDegrade: boolean;
begin
  Result := FDegrade;
end;

procedure TMaillage.ExportX3D(const QFichierX3D: TStringDirectoryFilename; const DoXML: boolean);
const
  BALISE_SCENE                          = 'Scene';
  BALISE_SHAPE                          = 'Shape';
  BALISE_APPEARANCE                     = 'Appearance';
  BALISE_INDEXED_FACE_SET               = 'IndexedFaceSet';
  SHAPE_NAME_MNT                        = 'MNT';
var
  fp: TextFile;
  Nb, i: Integer;
  procedure WriteLine(const S: string); inline;
  begin
    WriteLn(fp, S);
  end;
  procedure WriteHeader();
  begin
    if (DoXML) then
    begin
      WriteLine(Format('<?xml version ="1.0" encoding="%s"?>',['UTF-8'])); //""?>')
      WriteLine('<!DOCTYPE X3D PUBLIC "ISO//Web3D//DTD X3D 3.2//EN"');
      WriteLine('"http://www.web3d.org/specifications/x3d-3.2.dtd">');
      WriteLine('<X3D profile="Interchange" version="3.2"');
      WriteLine('   xmlns:xsd="http://www.w3.org/2001/XMLSchema-instance"');
      WriteLine('   xsd:noNamespaceSchemaLocation="http://www.web3d.org/specifications/x3d-3.2.xsd">');

    end else
    begin
      WriteLine('#X3D V3.2 utf8');
      WriteLine('PROFILE Interchange');
      WriteLine('META "generator" "view3dscene, http://castle-engine.sourceforge.net/view3dscene.php"');
      WriteLine(Format('META "source" "%s"', [ExtractFileName(QFichierX3D)]));
    end;

  end;
  procedure WriteFooter();
  begin
    if (DoXML) then
    begin
      WriteLine('</X3D>');
    end else
    begin
      pass;
    end;
  end;

  procedure BeginScene(const NameScene: string);
  begin
    if (DoXML) then
    begin
      WriteLine(Format('  <%s>', [BALISE_SCENE]));
      WriteLine(Format('    <NavigationInfo type=''"%s" "%s" "%s" "%s"''/>', ['EXAMINE', 'WALK', 'FLY', 'ANY']));
    end else
    begin

      WriteLine('NavigationInfo {');
      WriteLine('  avatarSize [');
      WriteLine(format('    %.5f, %.5f, %.5f,', [0.25, 1.60, 0.75]));
      WriteLine('  ]');
      WriteLine('  type [');
      WriteLine('    "EXAMINE",');
      WriteLine('    "WALK",');
      WriteLine('    "FLY",');
      WriteLine('    "ANY",');
      WriteLine('  ]');
      WriteLine('}');

      pass; // aucun marqueur de scène

    end;
  end;
  procedure EndScene();
  begin
    if (DoXML) then
    begin
      WriteLine(Format('  </%s>', [BALISE_SCENE]));
    end else
    begin
      pass; // aucun marqueur de scène
    end;

  end;
  procedure BeginShape(const NameShape: string);
  begin
    if (DoXML) then
    begin
      WriteLine(Format('    <%s>', [BALISE_SHAPE]));
    end else
    begin
      WriteLine('Shape {');
    end;
  end;
  procedure DescribeShapeMaterial(const NomShape: string);
  begin
    if (DoXML) then
    begin
      WriteLine(Format('     <%s>', [BALISE_APPEARANCE]));
      WriteLine('        <Material/>');
      WriteLine(Format('     </%s>', [BALISE_APPEARANCE]));
    end else
    begin
      WriteLine(#9 + 'appearance Appearance {');
		  WriteLine(#9 + #9 + 'material Material {');
		  WriteLine(#9 + #9 + '}');
	    WriteLine(#9 + '}');
    end;
  end;

  procedure EndShape(const NameShape: string);
  begin
    if (DoXML) then
    begin
      WriteLine(Format('    </%s>', [BALISE_SHAPE]));
    end else
    begin
      WriteLine('}');
    end;
  end;
  procedure BeginIndexedFaceSetSection(const DoColorPerVertex, DoSolid: boolean);
  var
    B1, B2: String;
  begin
    if (DoXML) then
    begin
      B1 := BoolToStr(DoColorPerVertex, 'true', 'false');
      B2 := BoolToStr(DoSolid         , 'true', 'false');
      Write(fp, Format('      <%s colorPerVertex=''%s'' solid=''%s'' ', [BALISE_INDEXED_FACE_SET, B1, B2]));
    end else
    begin
      //WriteLine(#9 + #9 + 'colorPerVertex ' + BoolToStr(DoColorPerVertex, 'TRUE', 'FALSE')
    end;
  end;

  procedure EndIndexedFaceSetSection();
  begin
    if (DoXML) then
    begin
      WriteLine(Format('      </%s>', [BALISE_INDEXED_FACE_SET]));
    end else
    begin
      WriteLine(#9 + '}');
    end;

  end;

  procedure BeginIndexFaceSetList();
  begin
    if (DoXML) then
    begin
      Write(fp, ' coordIndex=''');
      Write(fp, #13#10);

    end else
    begin
    	WriteLine(#9 + 'geometry IndexedFaceSet {');
    	WriteLine(#9 + #9 + 'solid FALSE');
      WriteLine(#9 + #9 + 'coordIndex [');
    end;
  end;
  procedure EndIndexFaceSetList();
  begin
    if (DoXML) then
    begin
      WriteLine('''>');
    end else
    begin
      WriteLine(#9 + #9 + ']');
    end;
  end;
  procedure BeginCoordinatesList();
  begin
    if (DoXML) then
    begin
      WriteLine('    <Coordinate point=''');
    end else
    begin
      WriteLine(#9 + #9 + 'colorPerVertex FALSE'); //+ BoolToStr(DoColorPerVertex, 'TRUE', 'FALSE')
      WriteLine(#9 + #9 + 'coord Coordinate {');
			WriteLine(#9 + #9 + #9 + 'point [');
    end;
  end;
  procedure EndCoordinatesList();
  begin
    if (DoXML) then
    begin
      WriteLine('''/>');
    end else
    begin
      WriteLine(#9 + #9 + #9 +']');
      WriteLine(#9 + #9 + '}');
    end;
  end;

  procedure Write3DFace(const T: TMNTTriangleABC);
  begin

    if (DoXML) then
    begin
      WriteLine(Format('%d %d %d %d ', [T.PointA, T.PointB, T.PointC, -1]));
    end else
    begin // 818, 958, 817, -1,
      WriteLine(Format(#9 + #9 + #9 + '%d, %d, %d, %d, ', [T.PointA, T.PointB, T.PointC, -1]));
    end;

  end;
  procedure WriteVertex(const V: TVertex);
  begin
    if (DoXML) then
    begin
      WriteLine(Format('%.3f %.3f %.3f ', [V.X, V.Y, V.Z]));
    end else
    begin     //439697.8125 3071659.75 898,
      WriteLine(Format(#9 + #9 + #9 + '%.3f, %.3f, %.3f, ', [V.X, V.Y, V.Z]));
    end;
  end;
begin
  AfficherMessage(format('%s.ExportX3D(): %d vertex, %d triangles, Output file: %s',
                         [ClassName,
                          GetNbVertex(),
                          GetNbTriangles(),
                          QFichierX3D
                         ]));
  AssignFile(fp, QFichierX3D);
  try
    ReWrite(fp);
    WriteHeader();
      BeginScene('MaScene');
        BeginShape(SHAPE_NAME_MNT);
          DescribeShapeMaterial(SHAPE_NAME_MNT);
          BeginIndexedFaceSetSection(false, false);
            BeginIndexFaceSetList();
              Nb := GetNbTriangles();
              for i := 0 to Nb-1 do Write3DFace(GetTriangle(i));
            EndIndexFaceSetList();

            BeginCoordinatesList();
              Nb := GetNbVertex();
              for i := 0 to Nb - 1 do WriteVertex(GetVertex(i));
            EndCoordinatesList();
          EndIndexedFaceSetSection();
        EndShape(SHAPE_NAME_MNT);
      EndScene();

    WriteFooter();
  finally
    closefile(fp);
  end;
end;


function TMaillage.GetNearTriangleIdx(const QX, QY, QZ: double): integer;
var
  EWE: TMNTTriangleABC;
  VA, VB, VC: TVertex;
  i, Nb: Integer;
begin
  Result := -1;
  Nb := GetNbTriangles;
  for i := 0 to Nb - 1 do
  begin
    EWE := GetTriangle(i);
    VA  := GetVertex(EWE.PointA);
    VB  := GetVertex(EWE.PointB);
    VC  := GetVertex(EWE.PointC);
    //dx  := (VA.X
    //+ VB.X + VC.X) / 3;
    //d   := MinimumDistanceFromPointToTriangle(Px,Py,x1,y1,x2,y2,x3,y3:TGeoFloat):TGeoFloat;

  end;
end;

function TMaillage.CalcDistanceAuVersant(const PT: TPoint3Df): double;
begin
  Result := CalcDistanceAuVersant(PT.X, PT.Y, PT.Z);
end;
//******************************************************************************

function TMaillage.GetDefaultCouleur(): TColor;
begin
  Result := GLToPascalColor(FColor);
end;

procedure TMaillage.SetDefaultCouleur(const C: TColor);
begin
  FColor := PascalToGLColor(C);
end;

procedure TMaillage.SetMaillageValide(const B: boolean; const PrgmLineTag: integer);
begin
  AfficherMessageErreur(Format('--> SetMaillageValide: %s - Ligne: %d', [ BoolToStr(B, True), PrgmLineTag]));
  FMaillageValide := B;
end;

function TMaillage.GetMinCouleur(): TColor;
begin
  Result := GLToPascalColor(FColorMin);
end;

procedure TMaillage.SetMinCouleur(const C: TColor);
begin
  FColorMin := PascalToGLColor(C);
end;

function TMaillage.GetMaxCouleur(): TColor;
begin
  Result := GLToPascalColor(FColorMax);
end;

procedure TMaillage.SetMaxCouleur(const C: TColor);
begin
  FColorMax := PascalToGLColor(C);
end;

procedure TMaillage.SetAlphaBlending(const A: double);
begin
  FAlphaBlendingMaillage := GLdouble(A);
end;

function TMaillage.GetAlphaBlending(): double;
begin
  Result := FAlphaBlendingMaillage;
end;

function TMaillage.GetCoordsMini(): TPoint3Df;
begin
  Result := FCoordsMini;
end;

function TMaillage.GetCoordsMaxi(): TPoint3Df;
begin
  Result := FCoordsMaxi;
end;


// MNT à TIN (triangles)

procedure TMaillage.ConstruireMaillageTriangleWire(const QScale, QMagn: double);
var
  i: integer;
  MM: Extended;
  QZMin: Extended;
  QZMax: Extended;
  TR   : TMNTTriangleABC;
  VA, VB, VC: TVertex;

begin
  AfficherMessage(Format('%s.ConstruireMaillageTriangleWire: %d triangles, %d vertex',[ClassName, GetNbTriangles, GetNbVertex]));
  AfficherMessage(Format('-- > Alpha = %f',[FAlphaBlendingMaillage]));
  AfficherMessage(Format('-- > Scale = %f',[QScale]));
  AfficherMessage(Format('-- > Magn  = %f',[QMagn]));

  MM := QScale * QMagn;
  QZMin := FCoordsMini.Z * MM;
  QZMax := FCoordsMaxi.Z * MM;
  for i:=0 to GetNbTriangles - 1 do
    begin
      //if FDoDrawSelectedTrianglesOnly then begin
      //  if Not Triangles[i].Marked then Continue;
      //end;
      glBegin(GL_LINE_LOOP);
      TR := GetTriangle(i);
      VA := GetVertex(TR.PointA);
      VB := GetVertex(TR.PointB);
      VC := GetVertex(TR.PointC);

      AffecterCouleur(VA.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VA.X * MM, VA.Y * MM, VA.Z * MM);
      AffecterCouleur(VB.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VB.X * MM, VB.Y * MM, VB.Z * MM);
      AffecterCouleur(VC.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VC.X * MM, VC.Y * MM, VC.Z * MM);
      AffecterCouleur(VA.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VA.X * MM, VA.Y * MM, VA.Z * MM);
      glEnd;
  end;
 
end;


// MNT à TIN (triangles)
procedure TMaillage.ConstruireMaillageTriangleSolide(const QScale, QMagn: double);
var
  i: integer;
  MM: Extended;
  QZMin: Extended;
  QZMax: Extended;
  TR: TMNTTriangleABC;
  VA, VB, VC: TVertex;
  
begin
  CalculerTINNormales(QScale, QMagn);
  AfficherMessage('ConstruireMaillageTriangleSolide:');
  AfficherMessage(Format('--> Couleur max: %f %f %f %f',[FColorMin.R, FColorMin.G, FColorMin.B, FAlphaBlendingMaillage]));
  AfficherMessage(Format('--> Couleur min: %f %f %f %f',[FColorMax.R, FColorMax.G, FColorMax.B, FAlphaBlendingMaillage]));
  glBegin(GL_TRIANGLES);
  MM := QScale * QMagn;
  QZMin := FCoordsMini.Z * MM;
  QZMax := FCoordsMaxi.Z * MM;
  // recto
  for i:=0 to GetNbTriangles - 1 do
    begin
      TR := GetTriangle(i);
      VA := GetVertex(TR.PointA);
      VB := GetVertex(TR.PointB);
      VC := GetVertex(TR.PointC);
      if (FDoDrawSelectedTrianglesOnly) then
      begin
        if Not TR.Marked then Continue;
      end;
      glNormal3F(VA.NormX, VA.NormY, VA.NormZ);
      AffecterCouleur(VA.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VA.X * MM, VA.Y * MM, VA.Z * MM);
      glNormal3F(VB.NormX, VB.NormY, VB.NormZ);
      AffecterCouleur(VB.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VB.X * MM, VB.Y * MM, VB.Z * MM);
      glNormal3F(VC.NormX, VC.NormY, VC.NormZ);
      AffecterCouleur(VC.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VC.X * MM, VC.Y * MM, VC.Z * MM);
    end;
  // verso
  for i:=0 to GetNbTriangles - 1 do
    begin
      TR := GetTriangle(i);
      VA := GetVertex(TR.PointA);
      VB := GetVertex(TR.PointB);
      VC := GetVertex(TR.PointC);
      if (FDoDrawSelectedTrianglesOnly) then
      begin
        if Not TR.Marked then Continue;
      end;
      glNormal3F(-VC.NormX, -VC.NormY, -VC.NormZ);
      AffecterCouleur(VC.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VC.X * MM, VC.Y * MM, VC.Z * MM);
      glNormal3F(-VB.NormX, -VB.NormY, -VB.NormZ);
      AffecterCouleur(VB.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VB.X * MM, VB.Y * MM, VB.Z * MM);
      glNormal3F(-VA.NormX, -VA.NormY, -VA.NormZ);
      AffecterCouleur(VA.Z * MM, QZMin, QZMax, FDegrade);
      glVertex3f(VA.X * MM, VA.Y * MM, VA.Z * MM);
    end;
  glEnd; // GL_TRIANGLES;
end;

function TMaillage.GetCoordosVertexToPlan(const PM: TVertex): TPoint;
var
  EWE : TPoint2Df;
begin
  EWE.X  := PM.X;
  EWE.Y  := PM.Y;
  Result := FProcGCSToSRC(EWE);
end;
function TMaillage.GetCoordosPlan(const PM: TPoint3Df): TPoint;
var
  EWE : TPoint2Df;
begin
  EWE.X  := PM.X;
  EWE.Y  := PM.Y;
  Result := FProcGCSToSRC(EWE);
end;



procedure TMaillage.TracerMaillage(const Bmp: TBGRABitmap;
                                   const QP: TProcGCSToSRC;
                                   const EquidistanceCourbes: double;
                                   const Isovaleurs: TArrayOfFloats;
                                   const CouleurCourbeNiveau: TColor;
                                   const CouleurIsovaleur: TColor;
                                   const DoDrawTriangles: boolean;
                                   const DoDrawVertex: boolean;
                                   const viewBox: TRect2Df);
var
  TR: TMNTTriangleABC;
  i: Integer;
  VX1, VX2, VX3: TVertex;
  PP1, PP2, PP3: TPoint;
  PM: TPoint3Df;
  QT: TGeoTriangle3D;
  I1, I2: TGeoPoint3D;
  PP: TPoint;
  QZ: Double;
  NbC, Nb: Integer;
  MyIsovaleur: Double;
  MyProfilTopo: TProfilTopo;
  PPP: TPoint3DfOrderedByP;
begin
  AfficherMessage(Format('%s.TracerMaillage(%d vertex, %d triangles)', [ClassName, GetNbVertex(), GetNbTriangles()]));

  FProcGCSToSRC := QP;
  //try
    AfficherMessage(Format('Coords mini: %.0f, %.0f, %.0f', [FCoordsMini.X, FCoordsMini.Y, FCoordsMini.Z]));
    AfficherMessage(Format('Coords mini: %.0f, %.0f, %.0f', [FCoordsMaxi.X, FCoordsMaxi.Y, FCoordsMaxi.Z]));
    AfficherMessage(Format('viewBox    : %.0f, %.0f -> %.0f, %.0f', [viewBox.X1, viewBox.Y1, viewBox.X2, viewBox.Y2]));

    if (DoDrawVertex) then
    begin
      AfficherMessage('--> Draw vertex');
      bmp.CanvasBGRA.Pen.Color := clBlue;
      bmp.CanvasBGRA.Pen.Style := psSolid;
      bmp.CanvasBGRA.Brush.Color := clAqua;
      bmp.CanvasBGRA.Pen.Width := 0;
      for i := 0 to GetNbVertex - 1 do
      begin
        VX1 := GetVertex(i);

        if (IsPointInRectangle(MakeTPoint2Df(VX1.X, VX1.Y), viewBox)) then
        begin
          PP1 := GetCoordosVertexToPlan(VX1);
          bmp.CanvasBGRA.EllipseC(PP1.X, PP1.Y, 2, 2);
          bmp.CanvasBGRA.Font.Height := 10;
          bmp.CanvasBGRA.TextOut(PP1.X + 3, PP1.Y + 3, Format('%d', [VX1.ID]));
        end;
      end;
    end;
    // représentation des triangles
    if (DoDrawTriangles) then
    begin
      AfficherMessage('--> Draw triangles');
      bmp.CanvasBGRA.Pen.Color := clYellow;
      bmp.CanvasBGRA.Pen.Style := psSolid;
      bmp.CanvasBGRA.Pen.Width := 0;
      for i:= 0 to GetNbTriangles - 1 do
      begin
        TR := GetTriangle(i);
        VX1 := GetVertex(TR.PointA);
        VX2 := GetVertex(TR.PointB);
        VX3 := GetVertex(TR.PointC);
        if (IsPointInRectangle(MakeTPoint2Df(VX1.X, VX1.Y), viewBox) OR
            IsPointInRectangle(MakeTPoint2Df(VX2.X, VX2.Y), viewBox) OR
            IsPointInRectangle(MakeTPoint2Df(VX3.X, VX3.Y), viewBox)) then
        begin
          PP1 := GetCoordosVertexToPlan(VX1);
          PP2 := GetCoordosVertexToPlan(VX2);
          PP3 := GetCoordosVertexToPlan(VX3);
          bmp.CanvasBGRA.MoveTo(PP1);
          bmp.CanvasBGRA.LineTo(PP2);
          bmp.CanvasBGRA.LineTo(PP3);
          bmp.CanvasBGRA.LineTo(PP1);
        end;
      end;
    end;
    // tracé des isovaleurs
    MyIsovaleur := Isovaleurs[0];
    // courbes de niveaux générales
    for i:= 0 to GetNbTriangles - 1 do
    begin
      TR := GetTriangle(i);

      VX1 := GetVertex(TR.PointA);
      VX2 := GetVertex(TR.PointB);
      VX3 := GetVertex(TR.PointC);

      if (IsPointInRectangle(MakeTPoint2Df(VX1.X, VX1.Y), viewBox) OR
          IsPointInRectangle(MakeTPoint2Df(VX2.X, VX2.Y), viewBox) OR
          IsPointInRectangle(MakeTPoint2Df(VX3.X, VX3.Y), viewBox)) then
      begin
        QT := GetGeoTriangle3D(TR);
        QZ := 10*Trunc(TR.BoundingBox.C1.Z / 10) + 0.01 - 10.00;

        while(QZ < TR.BoundingBox.C2.Z) do
        begin
          bmp.CanvasBGRA.Pen.Color := CouleurCourbeNiveau;
          bmp.CanvasBGRA.Pen.Style := psSolid;
          NbC := Trunc(QZ);
          bmp.CanvasBGRA.Pen.Width := IIF((NbC mod 5 = 0), 2, 0);
          if (IntersectPlanHorizontalTriangle(QZ, QT, I1, I2)) then
          begin
            PM.X := I1.X; PM.Y := I1.Y; PM.Z := MyIsovaleur;
            PP1  := GetCoordosPlan(PM);
            PM.X := I2.X; PM.Y := I2.Y;
            PP2  := GetCoordosPlan(PM);
            bmp.CanvasBGRA.MoveTo(PP1.X, PP1.Y);
            bmp.CanvasBGRA.LineTo(PP2.X, PP2.Y);
          end;
          QZ  += 2.00;
          //NbC += 1;
        end;
      end;
    end; // for i:= 0 to GetNbTriangles - 1 do
    // isovaleur spécifiée
    if (MyIsovaleur > 0.00) then
    begin
      bmp.CanvasBGRA.Pen.Color := CouleurIsovaleur;
      bmp.CanvasBGRA.Pen.Style := psSolid;
      bmp.CanvasBGRA.Pen.Width := 3;
      for i:= 0 to GetNbTriangles() - 1 do
      begin
        TR := GetTriangle(i);
        QT := GetGeoTriangle3D(TR);
        if (IntersectPlanHorizontalTriangle(MyIsovaleur + 1e-03, QT, I1, I2)) then
        begin
          //AfficherMessage('Isovaleur');
          PM := MakeTPoint3Df(I1.X, I1.Y, MyIsovaleur);
          PP1  := GetCoordosPlan(PM);
          PM := MakeTPoint3Df(I2.X, I2.Y, MyIsovaleur);
          PP2  := GetCoordosPlan(PM);
          bmp.CanvasBGRA.MoveTo(PP1.X, PP1.Y);
          bmp.CanvasBGRA.LineTo(PP2.X, PP2.Y);
        end;
      end;
    end;
    // profils topo

    Nb := self.GetNbProfilsTopo();
    AfficherMessage(Format('%d profils', [Nb]));
    if (Nb > 0) then
    begin
      for i := 0 to Nb - 1 do
      begin
        MyProfilTopo := GetProfilTopo(i);
        bmp.CanvasBGRA.Pen.Color := MyProfilTopo.ProfilColor;
        bmp.CanvasBGRA.Pen.Style := psSolid;
        bmp.CanvasBGRA.Pen.Width := 2;
        AfficherMessage(Format('Profil %d - %d points', [i, MyProfilTopo.GetNbElements()]));
        if (MyProfilTopo.GetNbElements() > 0) then
        begin
          PPP := MyProfilTopo.GetElement(0);
          PM := MakeTPoint3Df(PPP.X, PPP.Y, PPP.Z);
          PP1  := GetCoordosPlan(PM);
          PPP  := MyProfilTopo.GetElement(MyProfilTopo.GetNbElements() - 1);
          PM   := MakeTPoint3Df(PPP.X, PPP.Y, PPP.Z);
          PP2  := GetCoordosPlan(PM);
          bmp.CanvasBGRA.MoveTo(PP1.X, PP1.Y);
          bmp.CanvasBGRA.LineTo(PP2.X, PP2.Y);
        end;
      end;
    end;
  //except
  //end;
end;

// transformation d'un triangle TTriangulation en TGeoTriangle;
function TMaillage.GetGeoTriangle2D(const QT: TMNTTriangleABC): TGeoTriangle2D;
var
 V1, V2, V3: TVertex;
begin
 V1 := GetVertex(QT.PointA);
 V2 := GetVertex(QT.PointB);
 V3 := GetVertex(QT.PointC);

 Result[1].x := V1.X;
 Result[1].y := V1.Y;

 Result[2].x := V2.X;
 Result[2].y := V2.Y;

 Result[3].x := V3.X;
 Result[3].y := V3.Y;
end;
// transformation d'un triangle TTriangulation en TGeoTriangle;
function TMaillage.GetGeoTriangle3D(const QT: TMNTTriangleABC): TGeoTriangle3D;
var
 V1, V2, V3: TVertex;
begin
 V1 := GetVertex(QT.PointA);
 V2 := GetVertex(QT.PointB);
 V3 := GetVertex(QT.PointC);

 Result[1].x := V1.X;
 Result[1].y := V1.Y;
 Result[1].z := V1.Z;


 Result[2].x := V2.X;
 Result[2].y := V2.Y;
 Result[2].z := V2.Z;


 Result[3].x := V3.X;
 Result[3].y := V3.Y;
 Result[3].z := V3.Z;
end;

function TMaillage.CalcBoundingBoxTriangle(const T: TMNTTriangleABC): TMNTTriangleABC;
const
  INF = 1E24;
var
  QC1, QC2: TPoint3Df;
  procedure WU(const P: integer);
  var
    VX: TVertex;
  begin
    VX := GetVertex(P);
    if (VX.X < QC1.X) then QC1.X := VX.X;
    if (VX.Y < QC1.Y) then QC1.Y := VX.Y;
    if (VX.Z < QC1.Z) then QC1.Z := VX.Z;
    if (VX.X > QC2.X) then QC2.X := VX.X;
    if (VX.Y > QC2.Y) then QC2.Y := VX.Y;
    if (VX.Z > QC2.Z) then QC2.Z := VX.Z;
  end;
begin
  Result := T;
  QC1  := MakeTPoint3Df( INF,  INF,  INF);
  QC2  := MakeTPoint3Df(-INF, -INF, -INF);
  WU(T.PointA);
  WU(T.PointB);
  WU(T.PointC);
  Result.BoundingBox.C1 := QC1;
  Result.BoundingBox.C2 := QC2;
end;
// conversion d'un maillage SRTM HGT en maillage normal
// Modèle: SRTM3
// Résolution: 3 secondes d'arc soit (93 m) * sind(Latitude), avec sind: sinus d'unn angle en degrés
// La latitude et longitude du carré se déduisent du nom de fichier
// qui est de la forme ABBCDDD (eg: N42E131.hgt)
// avec:
// A   = N ou S
// BB  = latitude (00..89)
// C   = E ou W
// DDD = Longitude (000..179)
function TMaillage.ChargerMaillageDepuisHGT(const FichierHGT: string): boolean;
var
  fps: file of byte;
  fp: File;
  FF, A, BB, C, DDD: String;
  TailleFichier: Int64;
  EWE: Extended;
  NbCarreaux: Int64;
  Lat0: Extended;
  Long0: Extended;
  Buffer: array[0 .. 2401] of byte;
  BuffCount: integer;
  i, j: Integer;
  WU  : SmallInt;
  Cur: Integer;
  TailleBuffer: Integer;
  QAlt: Double;
  MyBitMap: TBitmap;
  h: Extended;
  Cz: Int64;

begin
  SetTypeMaillage(tmUNKNOWN, 1087);
  Result := False;
  AfficherMessage(Format('%s.ChargerMaillageDepuisHGT: %s', [ClassName, FichierHGT]));
  FF := ExtractFileName(FichierHGT);
  A  := Copy(FF, 1, 1);
  BB := Copy(FF, 2, 2);
  C  := Copy(FF, 4, 1);
  DDD:= Copy(FF, 5, 3);
  Lat0  := ConvertirEnNombreReel(BB, -1.00);
  Long0 := ConvertirEnNombreReel(DDD, -181.00);
  // taille du fichier et nombre de lignes/colonnes + contrôles initiaux
  AssignFile(fps, FichierHGT);
  try
    Reset(fps);
    TailleFichier := FileSize(fps);
    // la racine carrée de la demi-taille du fichier doit être strictement entière
    EWE := Sqrt(TailleFichier / 2.0);
    if (Abs(frac(EWE)) > 0) then Exit;
    NbCarreaux := Trunc(EWE);
    AfficherMessage(Format('A=%s, BB=%s, C=%s, DDD=%s, FS = %d, NC = %d', [A, BB, C, DDD, TailleFichier, NbCarreaux]));
    SetTypeMaillage(tmREGULAR_GRID, 1107);
    NbCarreaux := Trunc(EWE);
    FNbVertexX := 1201;
    FNbVertexY := 1201;
  finally
    CloseFile(fps);
  end;

  AssignFile(fp, FichierHGT);
  try
    Reset(fp, 1); // fichiers non typés -> deuxième paramètre indispensable
    // 92.76... = 3'' * circonférence équatoriale de la Terre en m / (3600'' * 360°)
    FPasX := 0.50 * (92.76851852 * sin(Lat0 * PIDiv180) +
                     92.76851852 * sin((1+Lat0) * PIDiv180));
    FPasY := 92.7685;
    SetLength(ZMatrix, FNbVertexX, FNbVertexY);
    TailleBuffer := Sizeof(Buffer);
    AfficherMessage(Format('-- Buffer de %d octets', [TailleBuffer]));
    FCoordsMaxi.Z := -1E24;
    FCoordsMini.Z := +1E24;
    for i := 0 to FNbVertexY - 1 do
    begin
      {$I-}
      BlockRead(fp, Buffer, TailleBuffer, BuffCount);
      {$I+}
      for j := 0 to FNbVertexX - 1 do
      begin
        Cur := j * 2;
        WU := Buffer[Cur] * 256 + Buffer[Cur+1];
        QAlt := WU;
        if (QAlt< -450.00) then QAlt := -450.00;
        if (QAlt > FCoordsMaxi.Z) then FCoordsMaxi.Z := QAlt;
        if (QAlt < FCoordsMini.Z) then FCoordsMini.Z := QAlt;
        // inverser les indices pour retrouver les bonnes coordonnées
        ZMatrix[i, j] := QAlt;
      end;
    end;
    AfficherMessage(Format('%f, %f', [FCoordsMini.Z, FCoordsMaxi.Z]));
  finally
    CloseFile(fp);
  end;
end;
function TMaillage.LireFichierMaillage(const FichierMaillage: string): Int64;
var
  NbLignesLues : Int64;
  VV           : TVertex;
  TR           : TMNTTriangleABC;
  i, j         : integer;
  pFileMAI     : TextFile;
  postab       : integer;
  s            : string;

  V1, V2, W    : TPoint3Df;

  PrmsLn       : TGHStringArray;
  MM: Extended;
  EWE: String;
  Nb: Integer;
  function LireLigne: string;
  var
    TmpBuff: string;
  begin
    Readln(PFileMAI,TmpBuff);
    Result := Trim(TmpBuff);
    NbLignesLues += 1;
  end;

begin
  AfficherMessage(Format('%s.LireFichierMaillage: %s',[ClassName, FichierMaillage]));
  NbLignesLues := 0;
  FNbVertexX   := 0;
  FNbVertexY   := 0;
  FTableVertex.ClearListe;
  FTableTriangles.ClearListe;
  FListeProfilsTopo.ClearListeProfils();
  // Maillage KO par défaut
  SetMaillageValide(false, 1169);
  // opaque par défaut
  FAlphaBlendingMaillage := 1.00;
  Result:=-1;
  if Not (FileExists(FichierMaillage)) then Exit;
  AssignFile(PFileMAI,FichierMaillage);
  try
    // Lecture du fichier
    Reset(PFileMAI);
    // Titre du maillage
    FCaption := LireLigne();
    AfficherMessage('--> Nom du maillage: ' + FCaption);
    // Type de maillage
    EWE := UpperCase(LireLigne());
    if      (EWE = 'REGULAR_GRID') then SetTypeMaillage(tmREGULAR_GRID, 1197)
    else if (EWE = 'TRIANGLES')    then SetTypeMaillage(tmTRIANGLES, 1198)
                                   else SetTypeMaillage(tmUNKNOWN, 1199);
    case FTypeMaillage of
      tmREGULAR_GRID:
        begin
          AfficherMessage('--> Type de maillage: MNT à pas régulier');
          //Couleur
          FColor     := GetGLColor(StrToInt(LireLigne()), FAlphaBlendingMaillage);
          FColorMax  := FColor;
          FColorMin  := GetGLColor(StrToInt(LireLigne()), FAlphaBlendingMaillage);
          // Nombre de lignes
          FNbVertexX := StrToIntDef(LireLigne(), 0);
          FNbVertexY := StrToIntDef(LireLigne(), 0);
          AfficherMessage(Format('--> Taille: %d  x %d', [FNbVertexX, FNbVertexY]));
          // FNbVertexY OU FNbVertexX nuls => maillage invalide --> [ ]
          if (FNbVertexY * FNbVertexX = 0) then Exit;
          // Coordonnées mini
          FCoordsMini.X := StrToFloat(LireLigne());
          FCoordsMini.Y := StrToFloat(LireLigne());

          // Coordonnées maxi
          FCoordsMaxi.X := StrToFloat(LireLigne());
          FCoordsMaxi.Y := StrToFloat(LireLigne());
          AfficherMessage(Format('--> Etendue X: %.2f - %.2f', [FCoordsMini.X, FCoordsMaxi.X]));
          AfficherMessage(Format('--> Etendue Y: %.2f - %.2f', [FCoordsMini.Y, FCoordsMaxi.Y]));
          // redimensionner les tableaux
          SetLength(ZMatrix,  FNbVertexY, FNbVertexX);
          // lecture d'une donnée obsolète
          EWE := LireLigne();
          // chargement de la table
          FCoordsMini.Z := +INFINI;
          FCoordsMaxi.Z := -INFINI;
          for i := 0 to FNbVertexY - 1 do
          begin
            AfficherMessage(Format('--> Lecture ligne: %d', [i]));
            for j := 0 to FNbVertexX - 1do
             begin
               ZMatrix[i,j] := strtofloat(LireLigne());
               if (ZMatrix[i,j] > FCoordsMaxi.Z) then FCoordsMaxi.Z:=ZMatrix[i,j];
               if (ZMatrix[i,j] < FCoordsMini.Z) then FCoordsMini.Z:=ZMatrix[i,j];
             end;
          end;
          EWE := ChooseString(Ord(FTypeMaillage), ['Inconnu', 'Grille', 'Triangles']);
          AfficherMessage(Format('Maillage "%s" de %d lignes x %d colonnes ; %d lignes lues', [EWE, FNbVertexY, FNbVertexX, NbLignesLues]));
          Result  := FNbVertexY * FNbVertexX;

          FNomTypeMaillage:=Format('Grille: (%d * %d)', [FNbVertexY, FNbVertexX]);
          // construire le TIN
          ConvertitMaillageGridEnTIN;
          SetMaillageValide(True, 1233);
        end;
    tmTRIANGLES:
      begin
        AfficherMessageErreur('--> Type de maillage: Réseau irrégulier de triangles');
        //Couleur
        FColor   := GetGLColor(StrToIntDef(LireLigne(), 0), FAlphaBlendingMaillage);
        FColorMax:= FColor;
        FColorMin:= GetGLColor(StrToIntDef(LireLigne(), 0), FAlphaBlendingMaillage);
        // section  Vertex
        EWE := LireLigne();
        // nombre de sommets
        Nb := StrToInt(LireLigne());
        AfficherMessage('--> Lecture section VERTEX');
        for i:=0 to Nb - 1 do
        begin
          PrmsLn := Split(LireLigne(), #9);
          VV.ID  :=StrToIntDef(PrmsLn[0],0);
          VV.X   :=StrToFloat(PrmsLn[1]);
          VV.Y   :=StrToFloat(PrmsLn[2]);
          VV.Z   :=StrToFloat(PrmsLn[3]);
          VV.NormX := 0.0;
          VV.NormY := 0.0;
          VV.NormZ := 0.0;
          VV.Norme := 0.0;
          AddVertex(VV);
        end;
        AfficherMessage(Format('--> %d vertex',[GetNbVertex]));
        EWE := LireLigne();   // lire le '-1'
        EWE := LireLigne();   // lire le TRIANGLES
        AfficherMessage('--> Lecture section TRIANGLES');
        Nb := StrToInt(LireLigne());
        for i := 0 to Nb - 1 do
        begin
          PrmsLn := Split(LireLigne(), #9);  // nombre de triangles
          TR.Numero:=StrToIntDef(PrmsLn[0],0);
          TR.PointA:=StrToIntDef(PrmsLn[1],0);
          TR.PointB:=StrToIntDef(PrmsLn[2],0);
          TR.PointC:=StrToIntDef(PrmsLn[3],0);
          TR.Marked:=IIF(StrToIntDef(PrmsLn[4],0) = 0, False, True);
          TR := CalcBoundingBoxTriangle(TR);

          if (IsValidTriangle(TR)) then AddTriangle(TR)
                                   else AfficherMessage(Format('--->Triangle dégénéré: %d - %d %d %d',[i,TR.PointA, TR.PointB,TR.PointC]))
        end;
        Nb := GetNbTriangles;
        AfficherMessage(Format('--> %d triangles',[Nb]));
        FNomTypeMaillage:=Format('MNT Triangles: (%d)',[Nb]);
        // initialiser tableaux inutilisés
        SetLength(ZMatrix, 1, 1);
        Result := Nb;
        // mini et maxi
        FCoordsMini.X:=  INFINI;
        FCoordsMaxi.X:= -INFINI;
        FCoordsMini.Y:=  INFINI;
        FCoordsMaxi.Y:= -INFINI;
        FCoordsMini.Z:=  INFINI;
        FCoordsMaxi.Z:= -INFINI;
        for i := 0 to GetNbVertex - 1 do
        begin
          VV := GetVertex(i);
          if ((i mod 1000) = 0) then  AfficherMessage(Format('%d - %f %f %f',[i, VV.X, VV.Y, VV.Z]));
          if (VV.X < FCoordsMini.X) then FCoordsMini.X := VV.X;
          if (VV.Y < FCoordsMini.Y) then FCoordsMini.Y := VV.Y;
          if (VV.Z < FCoordsMini.Z) then FCoordsMini.Z := VV.Z;
          if (VV.X > FCoordsMaxi.X) then FCoordsMaxi.X := VV.X;
          if (VV.Y > FCoordsMaxi.Y) then FCoordsMaxi.Y := VV.Y;
          if (VV.Z > FCoordsMaxi.Z) then FCoordsMaxi.Z := VV.Z;
        end;
        AfficherMessageErreur('Maillage TIN OK');
        SetMaillageValide(True, 1303);
      end;
    tmUNKNOWN:
       begin
         // initialiser tableaux inutilisés
         SetLength(ZMatrix,   1,1);
         SetMaillageValide(False, 1309);
         Result:=-64;
       end;
    end;
    // contrôle
    AfficherMessage('Etendue du maillage:');
    AfficherMessage(Format('--> X: De %.0f a %.0f', [FCoordsMini.X, FCoordsMaxi.X]));
    AfficherMessage(Format('--> Y: De %.0f a %.0f', [FCoordsMini.Y, FCoordsMaxi.Y]));
    AfficherMessage(Format('--> Z: De %.0f a %.0f', [FCoordsMini.Z, FCoordsMaxi.Z]));
  finally
    CloseFile(pFileMAI);
  end;
end;


// lire un maillage VisualTopo
function TMaillage.ChargerMaillageDepuisSUR(const FichierSUR: string): boolean;
var
  fp: TextFile;
  i, j: integer;
  EWE: TGHStringArray;
  dummy: String;
  MaLigne: String;
  QAT: String;
  QIN: Extended;
  function LireLigne: string;
  var
    WU: string;
  begin
    ReadLn(fp, WU);
    Result := Trim(WU);
  end;
begin
  AfficherMessage(Format('%s.ChargerMaillageDepuisSUR: %s', [ClassName, FichierSUR]));
  FTableVertex.ClearListe;
  FTableTriangles.ClearListe;
  // Maillage KO par défaut
  SetMaillageValide(false, 1346);
  // opaque par défaut
  FAlphaBlendingMaillage := 1.00;
  Result := False;
  SetTypeMaillage(tmREGULAR_GRID, 1364);
  AssignFile(fp, FichierSUR);
  try
    Reset(fp);
    // Ligne 1: Surface 390.000 410.020 0.070 287 3085.000 3095.080 0.090 113
    EWE := Split(LireLigne, ' ');
    FCoordsMini.X := 1000 * ConvertirEnNombreReel(EWE[1], -1.00);
    FCoordsMaxi.X := 1000 * ConvertirEnNombreReel(EWE[2], -1.00);
    FPasX         := 1000 * ConvertirEnNombreReel(EWE[3], -1.00);
    FNbVertexX    := StrToIntDef(EWE[4], -1);
    FCoordsMini.Y := 1000 * ConvertirEnNombreReel(EWE[5], -1.00);
    FCoordsMaxi.Y := 1000 * ConvertirEnNombreReel(EWE[6], -1.00);
    FPasY         := 1000 * ConvertirEnNombreReel(EWE[7], -1.00);
    FNbVertexY    := StrToIntDef(EWE[8], -1);
    AfficherMessage(Format('Sur X: De %.2f a %.2f; Spc = %.2f; NbVertex: %d', [FCoordsMini.X, FCoordsMaxi.X, FPasX, FNbVertexX]));
    AfficherMessage(Format('Sur Y: De %.2f a %.2f; Spc = %.2f; NbVertex: %d', [FCoordsMini.Y, FCoordsMaxi.Y, FPasY, FNbVertexY]));
    // Quelques tests
    if (0 = FNbVertexX * FNbVertexY) then Exit; // nombre de lignes ou colonnes nul -> []
    // matrice
    SetLength(ZMatrix, 1, 1);
    //SetLength(ZMatrix, FNbVertexX, FNbVertexY);
    // redimensionner les tableaux
    SetLength(ZMatrix,  FNbVertexY, FNbVertexX);
    // Ligne 2: vide
    MaLigne := LireLigne;
    // Ligne 3 et suivantes
    FCoordsMini.Z :=  INFINI;
    FCoordsMaxi.Z := -INFINI;
    for i := 0 to FNbVertexY - 1 do
    begin
      MaLigne := LireLigne;
      for j := 0 to FNbVertexX - 1 do
      begin
        QAT := ExtractFirstElementOfStr(MaLigne, ' ');
        QIN := ConvertirEnNombreReel(QAT, -500.00);
        if (QIN < FCoordsMini.Z) then FCoordsMini.Z := QIN;
        if (QIN > FCoordsMaxi.Z) then FCoordsMaxi.Z := QIN;
        // /!\ En format SUR, le YMin est dans ZMatrix[m-1, n]
        ZMatrix[FNbVertexY - i - 1, j] := QIN;
      end;
    end;
    AfficherMessage(Format('Alt: %.2f a %.2f', [FCoordsMini.Z, FCoordsMaxi.Z]));
    AfficherMessage('OK');
    Result := True;
  finally
    CloseFile(fp);
  end;
  try
    ConvertitMaillageGridEnTIN;
    SetMaillageValide(True, 1399);
  except
  end;
end;

// convertit un MNT grille en MNT triangles
function TMaillage.ConvertitMaillageGridEnTIN(): boolean;
var
  TR: TMNTTriangleABC;
  VX: TVertex;
  QIdxVertex : Int64;
  i, j: Integer;
  QX, QY: double;
  QEspcX, QEspcY: double;
  EWE : double;
  QIdxTriangle: Integer;
  QIdxVertexA: Integer;
  QIdxVertexB: Integer;
  QIdxVertexC: Integer;
  QIdxVertexD: Integer;
  dx, dy     : Extended;
  QMini, QMaxi: TPoint3Df;
  VV: TVertex;
  Q0: Integer;
begin
  Result := False;
  AfficherMessage(Format('%s.ConvertitMaillageGridEnTIN', [ClassName]));
  SetTypeMaillage(tmTRIANGLES, 1440);
  FTableVertex.ClearListe;
  FTableTriangles.ClearListe;

  // 13:06:19 | Maillage "Grille" de 124 lignes x 301 colonnes ; 37335 lignes lues
  //13:06:19 | --> Grille de vertex de 301 x 124 = 37324 vertex
  // construction des vertex
  QX := FCoordsMini.X;
  QY := FCoordsMini.Y;
  dx := FCoordsMaxi.X - FCoordsMini.X;
  dy := FCoordsMaxi.Y - FCoordsMini.Y;
  QEspcX := dx / (FNbVertexX - 1);
  QEspcY := dy / (FNbVertexY - 1);
  AfficherMessage(Format('--> Grille de vertex de FNbVertexY = %d lignes x FNbVertexX = %d colonnes = %d vertex attendus', [FNbVertexY, FNbVertexX, FNbVertexY * FNbVertexX]));
  AfficherMessage(Format('--> De (%.2f, %.2f) a (%.2f, %.2f)', [ FCoordsMini.X,  FCoordsMini.Y,  FCoordsMaxi.X,  FCoordsMaxi.Y]));
  AfficherMessage(Format('--> L = %.2f, H = %.2f, ex = %.2f, ey = %.2f', [dx, dy, QEspcX, QEspcY]));
  QIdxVertex := 0;
  for i := 0 to FNbVertexY - 1 do
  begin
    QY := FCoordsMini.Y + QEspcY * i;
    for j := 0 to FNbVertexX - 1 do
    begin
      QX := FCoordsMini.X + QEspcX * j;
      AddVertex(MakeTVertex(QIdxVertex, QX, QY, ZMatrix[i][j]));
      QIdxVertex += 1;
    end;
  end;
  AfficherMessage(Format('--> Construction des %d vertex OK', [GetNbVertex]));
  // contrôle: mini et maxi des vertex. A factoriser
  QMini := MakeTPoint3Df( INFINI,  INFINI,  INFINI);
  QMaxi := MakeTPoint3Df(-INFINI, -INFINI, -INFINI);
  for i := 0 to GetNbVertex - 1 do
  begin
    VV := GetVertex(i);
    if (VV.X < QMini.X) then QMini.X := VV.X;
    if (VV.Y < QMini.Y) then QMini.Y := VV.Y;
    if (VV.Z < QMini.Z) then QMini.Z := VV.Z;
    if (VV.X > QMaxi.X) then QMaxi.X := VV.X;
    if (VV.Y > QMaxi.Y) then QMaxi.Y := VV.Y;
    if (VV.Z > QMaxi.Z) then QMaxi.Z := VV.Z;
  end;
  AfficherMessage(Format('--> De (%.2f, %.2f, %.2f) a (%.2f, %.2f, %.2f)', [ QMini.X,  QMini.Y, QMini.Z, QMaxi.X,  QMaxi.Y, QMaxi.Z]));
  // 73800 triangles
  QIdxTriangle := 0;
  AfficherMessage(Format('i va de 0 a %d-1; j va de 0 a %d-1; - Reserved: %d, %d', [FNbVertexY, FNbVertexX, high(ZMatrix[0]), high(ZMatrix[1])]));
  for i := 1 to FNbVertexY - 1 do
  begin
    for j := 1 to FNbVertexX - 1 do
    begin
      try
        QIdxVertexA := FNbVertexX * (i - 1) + (j - 1);
        QIdxVertexB := FNbVertexX * (i - 1) + (j);
        QIdxVertexC := FNbVertexX * (i)     + (j - 1);
        QIdxVertexD := FNbVertexX * (i)     + (j);
        TR.Numero := QIdxTriangle;
        TR.PointA := QIdxVertexA;  TR.PointB := QIdxVertexB;  TR.PointC := QIdxVertexC;
        TR := CalcBoundingBoxTriangle(TR);
        AddTriangle(TR);
        QIdxTriangle += 1;
        TR.Numero := QIdxTriangle;
        TR.PointA := QIdxVertexB;  TR.PointB := QIdxVertexD;  TR.PointC := QIdxVertexC;
        TR := CalcBoundingBoxTriangle(TR);
        AddTriangle(TR);
        QIdxTriangle += 1;
      except
      end;
    end;
  end;
  AfficherMessage(Format('--> Construction des %d triangles OK', [GetNbTriangles]));
  AfficherMessage(Format('---> %d vertex (tableau de 0 .. %d vertex)', [QIdxVertex, GetNbVertex]));
  Result := True;
end;
//******************************************************************************
procedure TMaillage.AddVertex(const V: TVertex);
begin
  FTableVertex.AddElement(V);
end;
function TMaillage.GetVertex(const Idx: integer): TVertex;
begin
  Result := FTableVertex.GetElement(Idx);
end;
procedure TMaillage.PutVertex(const Idx: integer; const V: TVertex);
begin
  FTableVertex.PutElement(Idx, V);
end;
function TMaillage.GetNbVertex(): integer;
begin
  Result := FTableVertex.GetNbElements;
end;

constructor TMaillage.Create(AOwner: TObject);
begin
  inherited Create;
  FTableVertex      := TMNTTableVertex.Create;
  FTableTriangles   := TMNTTableTriangles.Create;
  FListeProfilsTopo := TListeProfilsTopo.Create;
end;

//------------------------------------------------------------------------------
procedure TMaillage.AddTriangle(const V: TMNTTriangleABC);
begin
  FTableTriangles.AddElement(V);
end;
function TMaillage.GetNbTriangles(): integer;
begin
  Result := FTableTriangles.GetNbElements;
end;
procedure TMaillage.PutTriangle(const Idx: integer; const V: TMNTTriangleABC);
begin
  FTableTriangles.PutElement(Idx, V);
end;
function TMaillage.GetTriangle(const Idx: integer): TMNTTriangleABC;
begin
  Result := FTableTriangles.GetElement(Idx);
end;
//------------------------------------------------------------------------------
// MNT à pas réguliers: Construction d'images
function TMaillage.MakeImageFromGRD(const DoYReverse: boolean): TBitmap;
var
  h: Double;
  i, j: Integer;
  Cz: Int64;
  WU: Integer;
begin
  // feedback: crée une image de taille n*n, en niveaux de gris
  AfficherMessage('Construction image');
  Result := TBitmap.Create;
  try
    Result.Width  := FNbVertexX;
    Result.Height := FNbVertexY;
    h := FCoordsMaxi.Z; // - FCoordsMini.Z;
    for i := 0 to FNbVertexY - 1 do
    begin
      WU := IIF(DoYReverse, FNbVertexY - i - 1, i);
      for j := 0 to FNbVertexX - 1 do
      begin
        Cz := round(256 * (ZMatrix[i, j] - FCoordsMini.Z) / h);

        Result.Canvas.Pixels[j, WU] := RGBToColor(Cz, Cz, Cz);
      end;
    end;
    //AfficherMessage('Sauvegarde image');
    //Result.SaveToFile(ChangeFileExt(FichierHGT, '.png'));
    AfficherMessage('Finished');
  finally
  //  MyBitMap.Free;
  end;
end;
// MNT à pas régulier: export format SUR
// Test OK
procedure TMaillage.ExportToSUR(const FichierSUR: string);
var
  fp: TextFile;
  QSpcX: Extended;
  QSpcY: Extended;
  i, j: Integer;
  QIN : double;
  EWE: String;
  procedure WriteLine(const S: string);
  begin
    WriteLn(fp, S);
  end;
begin
  AfficherMessage(Format('%s.ExportToSUR: %s - %d x %d', [ClassName, FichierSUR, FNbVertexX, FNbVertexY]));
  AssignFile(fp, FichierSUR);
  try
    ReWrite(fp);
    //Surface 390.000 410.020 0.070 287 3085.000 3095.080 0.090 113
    QSpcX := (FCoordsMaxi.X - FCoordsMini.X) / (FNbVertexX - 1);
    QSpcY := (FCoordsMaxi.Y - FCoordsMini.Y) / (FNbVertexY - 1);

    WriteLine(Format('Surface %.3f %.3f %.3f %d %.3f %.3f %.3f %d',
                     [FCoordsMini.X / 1000, FCoordsMaxi.X / 1000, QSpcX / 1000, FNbVertexX,
                      FCoordsMini.Y / 1000, FCoordsMaxi.Y / 1000, QSpcY / 1000, FNbVertexY
                     ]));
    WriteLine('');
    for i := 0 to FNbVertexY - 1 do
    begin
      EWE := '';
      for j := 0 to FNbVertexX - 1 do
      begin
        QIN := ZMatrix[FNbVertexY - i - 1, j];
        // /!\ En format SUR, le YMin est dans ZMatrix[m-1, n]
        EWE := EWE + Format('%.0f ', [QIN]);
      end;
      WriteLine(EWE);
    end;
  finally
    closefile(fp);
  end;
  Reset(fp);
end;

// extraction des coordonnées d'intersection d'un plan vertical et d'un triangle
// QT: Triangle traversé
// QX1, QX2, QY1, QY2: Coordonnées des extrémités de la trace horizontale du plan
function TMaillage.IntersectPlanVerticalAndTriangle(const QT: TMNTTriangleABC; const ExtrProfil1, ExtrProfil2: TPoint3Df; out QIntersect: TIntersectPlanTriangle): boolean;
var
  P1, P2, P3: TVertex;
  function CalcZ(const AP1, AP2: TVertex; const QIx, QIy: double): double;
  var
    QR1, QR2, m: double;
    dx, dy, dz : double;
  begin

    dx := QIx - AP1.X;
    dy := QIy - AP1.y;

    QR1 := Hypot(AP2.X - AP1.X, AP2.Y - AP1.Y);
    QR2 := Hypot(dx, dy);
    dz  := AP2.z - AP1.z;

    m   := QR2 / QR1;
    Result := AP1.Z + m * dz;
  end;
  function CalcIntersect(const AP1, AP2, AP3: TVertex) : boolean;
  var
    IX1, IY1: TGeoFloat;
  begin
    Result := False;
    // premier côté ?
    if Intersect(TGeoFloat(AP1.X), TGeoFloat(AP1.Y), TGeoFloat(AP2.X), TGeoFloat(AP2.Y),
                TGeoFloat(ExtrProfil1.X), TGeoFloat(ExtrProfil1.Y),
                TGeoFloat(ExtrProfil2.X), TGeoFloat(ExtrProfil2.Y),
                IX1, IY1) then
    begin
      QIntersect.X1 := IX1;
      QIntersect.Y1 := IY1;
      QIntersect.Z1 := CalcZ(AP1, AP2, IX1, IY1);


      // coté suivant ?
      if Intersect(TGeoFloat(AP2.X), TGeoFloat(AP2.Y), TGeoFloat(AP3.X), TGeoFloat(AP3.Y),
              TGeoFloat(ExtrProfil1.X), TGeoFloat(ExtrProfil1.Y),
              TGeoFloat(ExtrProfil2.X), TGeoFloat(ExtrProfil2.Y),
              IX1, IY1)
      then
      begin
        QIntersect.X2 := IX1;
        QIntersect.Y2 := IY1;
        QIntersect.Z2 := CalcZ(AP2, AP3, IX1, IY1);
      end else
      begin
        Intersect(TGeoFloat(AP3.X), TGeoFloat(AP3.Y), TGeoFloat(AP1.X), TGeoFloat(AP1.Y),
              TGeoFloat(ExtrProfil1.X), TGeoFloat(ExtrProfil1.Y),
              TGeoFloat(ExtrProfil2.X), TGeoFloat(ExtrProfil2.Y),
              IX1, IY1);
        QIntersect.X2 := IX1;
        QIntersect.Y2 := IY1;
        QIntersect.Z2 := CalcZ(AP3, AP1, IX1, IY1);
      end;
      Result := True;
    end;

  end;

begin
  Result := False;

  P1 := GetVertex(QT.PointA);
  P2 := GetVertex(QT.PointB);
  P3 := GetVertex(QT.PointC);
  (*
  AfficherMessage(Format('P1 =(%.2f, %.2f, %.2f), P2 = (%.2f, %.2f, %.2f), P3 = (%.2f, %.2f, %.2f)',
                              [P1.X, P1.Y, P1.Z,
                               P2.X, P2.Y, P2.Z,
                               P3.X, P3.Y, P3.Z
                              ]));
  //*)
  if      CalcIntersect(P1, P2, P3) then Exit(True)
  else if CalcIntersect(P2, P3, P1) then Exit(True)
  else if CalcIntersect(P3, P1, P2) then Exit(True);
  //*}
end;
(*
function TMaillage.IntersectionTriangleSegment(const P1, P2: TPoint3Df; const Triangle: TMNTTriangleABC; var Intersects: TArrayPoints3Df): boolean;
var
  // plan vertical d'intersection
  DemiPlanTriangle1: TGeoTriangle3D;
  DemiPlanTriangle2: TGeoTriangle3D;

  AreteAB      : TGeoLine3D;
  AreteBC      : TGeoLine3D;
  AreteCA      : TGeoLine3D;

  MyVertexA  , MyVertexB, MyVertexC: TVertex;

  IPtArete     : TGeoPoint3D;
  function MakeArete(const V1, V2: TVertex): TGeoLine3D;
  begin
    Result[1].X := V1.X;  Result[1].Y := V1.Y; Result[1].z := V1.Z;
    Result[2].X := V2.X;  Result[2].Y := V2.Y; Result[2].z := V2.Z;
  end;
  procedure AppendIntersect(const IP: TGeoPoint3D);
  var
    QIntersectPt: TPoint3Df;
    n: Integer;
  begin
    AfficherMessageErreur('');
    AfficherMessageErreur('---- AppendIntersect()');
    n := Length(Intersects);
    SetLength(Intersects, n+1);
    QIntersectPt := MakeTPoint3Df(IP.x, IP.y, IP.z);
    Intersects[n-1] := QIntersectPt;
  end;
begin

  result := false;

    // Plan de profil, composé de deux triangles
    DemiPlanTriangle1[1].x := P1.X; DemiPlanTriangle1[1].Y := P1.y; DemiPlanTriangle1[1].z := 10000.00;//P2.Z;
    DemiPlanTriangle1[2].x := P1.X; DemiPlanTriangle1[2].Y := P1.y; DemiPlanTriangle1[2].z := -1000.00; //P1.Z;
    DemiPlanTriangle1[3].x := P2.X; DemiPlanTriangle1[3].Y := P2.y; DemiPlanTriangle1[3].z := -1000.00;

    DemiPlanTriangle2[1].x := P1.X; DemiPlanTriangle2[1].Y := P1.y; DemiPlanTriangle2[1].z := 10000.00;
    DemiPlanTriangle2[2].x := P2.X; DemiPlanTriangle2[2].Y := P2.y; DemiPlanTriangle2[2].z := -1000.00;
    DemiPlanTriangle2[3].x := P2.X; DemiPlanTriangle2[3].Y := P2.y; DemiPlanTriangle2[3].z := 10000.00;

    // sommets du triangle
    MyVertexA := GetVertex(Triangle.PointA);
    MyVertexB := GetVertex(Triangle.PointB);
    MyVertexC := GetVertex(Triangle.PointC);
    AfficherMessageErreur(Format('%d: %d: %f, %f, %f - %d: %f, %f, %f - %d: %f, %f, %f',
                                 [Triangle.Numero,
                                  Triangle.PointA, MyVertexA.X, MyVertexA.Y, MyVertexA.Z,
                                  Triangle.PointB, MyVertexB.X, MyVertexB.Y, MyVertexB.Z,
                                  Triangle.PointC, MyVertexC.X, MyVertexC.Y, MyVertexC.Z]));

    // arêtes
    AreteAB := MakeArete(MyVertexA, MyVertexB);
    AreteBC := MakeArete(MyVertexB, MyVertexC);
    AreteCA := MakeArete(MyVertexC, MyVertexA);
    AfficherMessageErreur(Format('AB: %f, %f, %f -> %f, %f, %f -- BC: %f, %f, %f -> %f, %f, %f -- CA: %f, %f, %f -> %f, %f, %f',
                                 [AreteAB[1].X, AreteAB[1].Y, AreteAB[1].Z, AreteAB[2].X, AreteAB[2].Y, AreteAB[2].Z,
                                  AreteBC[1].X, AreteBC[1].Y, AreteBC[1].Z, AreteBC[2].X, AreteBC[2].Y, AreteBC[2].Z,
                                  AreteCA[1].X, AreteCA[1].Y, AreteCA[1].Z, AreteCA[2].X, AreteCA[2].Y, AreteCA[2].Z
                                 ]));

    if (Intersect(AreteAB, DemiPlanTriangle1, IPtArete)) then AppendIntersect(IPtArete);
    if (Intersect(AreteBC, DemiPlanTriangle1, IPtArete)) then AppendIntersect(IPtArete);
    if (Intersect(AreteCA, DemiPlanTriangle1, IPtArete)) then AppendIntersect(IPtArete);

    if (Intersect(AreteAB, DemiPlanTriangle2, IPtArete)) then AppendIntersect(IPtArete);
    if (Intersect(AreteBC, DemiPlanTriangle2, IPtArete)) then AppendIntersect(IPtArete);
    if (Intersect(AreteCA, DemiPlanTriangle2, IPtArete)) then AppendIntersect(IPtArete);

  Result := (length(Intersects) > 0);
end;
//*)

function TMaillage.ExtractProfilTopo(const Pt1, Pt2: TPoint2Df): boolean;
var
  Nb, i, k, NbVisees, v: Integer;
  TR: TMNTTriangleABC;
  QPt1, QPt2: TPoint3Df;
  EWE: String;
  wu: Boolean;
  IntersectPlanTriangle: TIntersectPlanTriangle;
  PP: TPoint3DfOrderedByP;
  MyProfilTopo: TProfilTopo;
  MyVisee: TEntiteEtendue;
begin
  result := false;
  QPt1 := MakeTPoint3Df(Pt1.X, Pt1.Y, FCoordsMini.Z);
  QPt2 := MakeTPoint3Df(Pt2.X, Pt2.Y, FCoordsMaxi.Z);
  EWE := Format('%s.ExtractProfilTopo(%.2f, %.2f, %.2f -> %.2f, %.2f, %.2f)', [ClassName, QPt1.X, QPt1.Y, QPt1.Z, QPt2.X, QPt2.Y, QPT2.Z]);
  AfficherMessage(EWE);
  AfficherMessageErreur(EWE);
  result := false;
  MyProfilTopo := TProfilTopo.Create;
  try
    MyProfilTopo.Initialiser(Pt1, Pt2);
    Nb := self.GetNbTriangles();
    for i := 1 to Nb - 1 do
    begin
      try
        TR := GetTriangle(i);
        wu := IntersectPlanVerticalAndTriangle(TR, QPt1, QPt2, IntersectPlanTriangle);
        if (wu) then
        begin
          PP.P := Hypot(IntersectPlanTriangle.X1 - QPt1.X, IntersectPlanTriangle.Y1 - QPt1.Y);
          PP.X := IntersectPlanTriangle.X1;
          PP.Y := IntersectPlanTriangle.Y1;
          PP.Z := IntersectPlanTriangle.Z1;
          MyProfilTopo.AddElement(PP);
          PP.P := Hypot(IntersectPlanTriangle.X2 - QPt1.X, IntersectPlanTriangle.Y2 - QPt1.Y);
          PP.X := IntersectPlanTriangle.X2;
          PP.Y := IntersectPlanTriangle.Y2;
          PP.Z := IntersectPlanTriangle.Z2;
          MyProfilTopo.AddElement(PP);
        end;
      except
        pass;
      end;
    end;
    MyProfilTopo.SortByAbscisses();
    MyProfilTopo.Epurage();
    Nb := MyProfilTopo.GetNbElements();
    if (Nb > 0) then
    begin
      MyProfilTopo.ProfilName  := 'Profil_' + IntToStr(FListeProfilsTopo.GetNbElements() + 1);
      MyProfilTopo.ProfilColor := clBlue;
      // extraction des transects: galeries recoupées
      NbVisees := FBDDEntites.GetNbEntitesVisees();
      for v := 0 to NbVisees - 1 do
      begin
        MyVisee := FBDDEntites.GetEntiteVisee(v);
        if (MyVisee.Entite_Serie > 0) AND (Intersect(MyVisee.Une_Station_1_X, MyVisee.Une_Station_1_Y, MyVisee.Une_Station_2_X, MyVisee.Une_Station_2_Y, Pt1.X, Pt1.Y, Pt2.X, Pt2.Y)) then
        begin
          MyProfilTopo.AddGalerieRecoupee(MyVisee);
          AfficherMessage(format('--> %d.%d', [MyVisee.Entite_Serie, MyVisee.Entite_Station]));
        end;
      end;
      AddProfilTopo(MyProfilTopo);
      Result := True;
    end;
  except
    MyProfilTopo.Free;
  end;
end;

procedure TMaillage.AddProfilTopo(const P: TProfilTopo);
begin
  FListeProfilsTopo.AddElement(P);
end;

function TMaillage.GetProfilTopo(const Idx: integer): TProfilTopo;
begin
  Result := FListeProfilsTopo.GetElement(Idx);
end;
procedure TMaillage.PutProfilTopo(const Idx: integer; const P: TProfilTopo);
begin
  FListeProfilsTopo.PutElement(Idx, P);
end;
procedure TMaillage.RemoveProfilTopo(const Idx: integer);
begin
  AfficherMessageErreur(Format('%s.RemoveProfilTopo()', [ClassName]));
  FListeProfilsTopo.RemoveProfil(Idx);
end;

function TMaillage.GetNbProfilsTopo(): integer;
begin
  result := FListeProfilsTopo.GetNbElements();
end;

procedure TMaillage.ListerProfilsTopo();
var
  Nb, i, NbPts, j: Integer;
  MyProfilTopo: TProfilTopo;
  Pt: TPoint3DfOrderedByP;
begin
  Nb := FListeProfilsTopo.GetNbElements();
  AfficherMessageErreur(Format('%s.ListerProfilsTopo(): %d profils', [ClassName, Nb]));
  if (Nb = 0) then Exit;
  for i := 0 to Nb -1 do
  begin
    MyProfilTopo := GetProfilTopo(i);
    NbPts := MyProfilTopo.GetNbElements();

    AfficherMessageErreur(Format('Profil: %d - %s - %d points', [i, MyProfilTopo.ProfilName, NbPts]));
    if (NbPts > 0) then
    begin
      for j := 0 to NbPts - 1 do
      begin
        Pt := MyProfilTopo.GetElement(j);
        AfficherMessageErreur(Format('  %d; %.2f, %.2f', [j, Pt.P, Pt.Z]));
      end;
    end;
  end;
end;

function TMaillage.ConstruireMaillageAPasRegulier(const NbMaillesX, NbMaillesY: integer): boolean;
var
  C1, C2: TPoint3Df;
begin
  AfficherMessageErreur(Format('%s.ConstruireMaillageAPasRegulier(): %d x %d', [ClassName, NbMaillesX, NbMaillesY]));
  C1 := self.GetCoordsMini();
  C2 := self.GetCoordsMaxi();

end;

procedure TMaillage.SaveProfilsToFile(const QFilename: string);
var
  fp: TextFile;
  i, Nb: Integer;
  EWE: String;
  PP: TProfilTopo;
  PT1, PT2: TPoint2Df;
begin
  AfficherMessageErreur(Format('%s.SaveProfilsToFile(): %s', [ClassName, QFilename]));
  Nb := GetNbProfilsTopo();
  if (Nb = 0) then Exit;
  EWE := '%d' + TAB +
         '%d' + TAB + '%d' + TAB + '%d' + TAB +    // couleur
         '%.2f' + TAB + '%.2f' + TAB +           // extr 1
         '%.2f' + TAB + '%.2f' + TAB +           // extr 2
         '%s';
  AssignFile(fp, QFilename);
  try
    Rewrite(fp);
    for i := 0 to Nb - 1 do
    begin
      PP := GetProfilTopo(i);
      PT1 := PP.GetExtremite1();
      PT2 := PP.GetExtremite2();
      writeln(fp, Format(EWE, [i, Red(PP.ProfilColor), Green(PP.ProfilColor), Blue(PP.ProfilColor),
                               PT1.X, PT1.Y,
                               PT2.X, PT2.Y,
                               PP.ProfilName
                              ]));
    end;
  finally
    CloseFile(fp);
  end;

end;
// ajoute les profils
procedure TMaillage.LoadProfilsFromFile(const QFilename: string);
var
  fp: TextFile;
  WU, QNom: string;
  AR: TGHStringArray;
  PT1, PT2: TPoint2Df;
  QColor: TColor;
  PP: TProfilTopo;
  n: Integer;
begin
  AfficherMessageErreur(Format('%s.LoadProfilsFromFile(): %s', [ClassName, QFilename]));
  AssignFile(fp, QFilename);
  try
    Reset(fp);
    while (not eof(fp)) do
    begin
      ReadLn(fp, WU);
      AR := Split(WU, #9);
      //0 1 2 3      4          5          6         7            8
      //0	0	0	255	403080.31	3091003.68	403405.57	3090831.37	Profil_1
      QColor := RGBToColor(StrToIntDef(AR[1], 0)   and 255,
                           StrToIntDef(AR[2], 0)   and 255,
                           StrToIntDef(AR[3], 255) and 255);
      PT1  := MakeTPoint2Df(ConvertirEnNombreReel(AR[4], 0.00),
                            ConvertirEnNombreReel(AR[5], 0.00));
      PT2  := MakeTPoint2Df(ConvertirEnNombreReel(AR[6], 0.00),
                            ConvertirEnNombreReel(AR[7], 0.00));
      QNom := Trim(AR[8]);
      if (ExtractProfilTopo(PT1, PT2)) then
      begin
        n := GetNbProfilsTopo() - 1;
        PP := GetProfilTopo(n);
        PP.ProfilColor := QColor;
        PP.ProfilName  := QNom;
      end;
    end;
  finally
    CloseFile(fp);
  end;

end;

end.
