unit unitConteneurCodeCalcul;
// Conteneur commun au code de calcul et à l'analyse de graphe
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  {$IFDEF USE_GENERIC_TYPES}
  UnitListesSimplesWithGeneriques,
  {$ELSE}
  UnitListesSimplesWithoutGeneriques,
  {$ENDIF}
  ToporobotClasses2012,
  UnitClassPalette,
  Common, UnitObjetSerie,
  Classes, SysUtils, math,
  Forms; // pour Application  ;

type

{ TGrapheDuReseau }

 { TConteneurCodeCalcul }

 TConteneurCodeCalcul = class
  private  // visible uniquement dans la classe et non ds les classes héritées
    // callback pour suivi de progression
    FProcDispProgression: TProcDisplayProgression;
    FTableJonctions   : TTableJonctionsXYZ;
    FTableBranches    : TTableBranchesXYZ;
    //FTableBranchesElaguees: TTableBranchesXYZ;
  protected // = visible dans les classes héritées
    FDocTopo          : TToporobotStructure2012;
    FPalette256       : TPalette256;
  public
    property  DocuTopo : TToporobotStructure2012 read FDocTopo;
    property  ProcDispProgression: TProcDisplayProgression read FProcDispProgression;

    function  Initialiser(const FD: TToporobotStructure2012; const FP: TPalette256): boolean;
    procedure Finaliser();
    procedure ViderTableJonctionsBranches();
    procedure SetProcDisplayProgression(const QProcProgression: TProcDisplayProgression);

    // version Noeuds et Branches
    procedure AddJunction(const Jnct: TJonctionXYZ);
    function  GetJonction(const NoJonction: integer): TJonctionXYZ;
    function  GetIdxJonctionBySerSt(const Serie, Station: integer): integer;
    function  GetNbJonctions(): integer;
    function  RecenserJonctions(): integer;
    procedure RecenserBranches();
    // gestions des branches (version moins 'propre' que les versions Python et Java, mais largement suffisante. A remplacer mais non prioritaire)
    procedure AddBranche(var LaBranche: TBrancheXYZ);
    procedure RemoveBranche(const NumBrch: integer);
    procedure PutBranche(const NoBr: integer; const LaBranche: TBrancheXYZ);
    function  GetBranche(const Idx: integer): TBrancheXYZ;
    function  GetNbBranches(): integer;

    procedure addViseeAtBranche(var BR: TBrancheXYZ; const V: TUneVisee);
    procedure CalculerAccroissements();
end;
implementation

const FMT_NDSER_PT = '%d-%d';

//******************************************************************************
procedure TConteneurCodeCalcul.Finaliser;
begin
  AfficherMessage(Format('%s.Finaliser()', [ClassName]));
  try
    ViderTableJonctionsBranches();
  finally
    FTableJonctions.Free;
    FTableBranches.Free;
  end;
end;

function TConteneurCodeCalcul.Initialiser(const FD: TToporobotStructure2012; const FP: TPalette256): boolean;
begin
  Result := false;
  AfficherMessage(Format('%s.Initialiser()', [ClassName]));
  FDocTopo := FD;
  FPalette256 := FP;
  // création des tables
  FTableJonctions := TTableJonctionsXYZ.Create;
  FTableBranches  := TTableBranchesXYZ.Create;
  ViderTableJonctionsBranches();
  FProcDispProgression := nil;
  result := True;
end;









//******************************************************************************
// méthodes relatives aux jonctions
function TConteneurCodeCalcul.GetJonction(const NoJonction: integer): TJonctionXYZ;
begin
  Result := FTableJonctions.GetElement(NoJonction);
end;

function TConteneurCodeCalcul.GetNbJonctions: integer;
begin
  Result := FTableJonctions.GetNbElements();
end;

procedure TConteneurCodeCalcul.AddJunction(const Jnct: TJonctionXYZ);
begin
  FTableJonctions.AddElement(Jnct);
end;
//*************************************************************
// recherche de toutes les entrées, jonctions et culs de sacs
function TConteneurCodeCalcul.RecenserJonctions(): integer;
var
  Entree : TEntrance;
  Serie  : TObjSerie;
  ViseeAntenne: TViseeAntenne;
  Ser, St: integer;
  ListeJnct: TStringList;
  J    : TJonctionXYZ;
  WU: TGHStringArray;
begin

  AfficherMessage(Format('%s.RecenserJonctions',[self.ClassName]));
  if (Assigned(self.ProcDispProgression)) then self.ProcDispProgression('Recensement des jonctions', -1, -1, -1);
  result := -1;
  AfficherMessage(GetResourceString(rsRECENSEM_JONC));
  ListeJnct := TStringList.Create;
  try
    ListeJnct.Clear;
    ListeJnct.Sorted     := True;
    ListeJnct.Duplicates := dupIgnore;
    // Les coordonnées d'accrochage des entrées de cavités sont forcément des jonctions
    AfficherMessage(Format('-- %d entrances', [FDocTopo.GetNbEntrees]));
    for Ser:=0 to FDocTopo.GetNbEntrees - 1 do
    begin
      Entree:=FDocTopo.GetEntree(Ser);
      ListeJnct.Add(Format(FMT_NDSER_PT,[Entree.eRefSer,Entree.eRefSt]));
    end; //*)
    AfficherMessage(Format('-- %d series', [FDocTopo.GetNbSeries]));
    // Les série/point de départ/arrivée sont forcément des jonctions (noeuds)
    for Ser:=0 to FDocTopo.GetNbSeries - 1 do
    begin
      Serie := FDocTopo.GetSerie(Ser);
      ListeJnct.Add(Format(FMT_NDSER_PT,[Serie.GetNoSerieDep,  Serie.GetNoPointDep]));
      ListeJnct.Add(Format(FMT_NDSER_PT,[Serie.GetNoSerieArr,  Serie.GetNoPointArr]));
    end;
    // Les départs de visées en antenne sont aussi des jonctions
    // en interne, l'ID de noeud est un littéral
    for St := 0 to ListeJnct.Count - 1 do
    begin
      J.IDJonction := ListeJnct.Strings[St];
      J.NoNoeud    := St;
      WU := Split(J.IDJonction, '-');
      J.NoSer      := StrToIntDef(WU[0], -1);
      J.NoSt       := StrToIntDef(WU[1], -1);
      J.NbRefs     := 0;
      AddJunction(J);
    end;
    Application.ProcessMessages;
    // écriture de la liste des jonctions dans le log
    // On passe ici ? OK, c'est bon
    Result := GetNbJonctions();
    AfficherMessage(format('-- > %d junctions', [Result]));
  finally
    // on libère la table provisoire
    ListeJnct.Free;
  end;
end;

procedure TConteneurCodeCalcul.AddBranche(var LaBranche: TBrancheXYZ);
var
  UneVisee   : TUneVisee;
begin
  with FTableBranches do
  begin
    // créer la table des stations
    setlength(LaBranche.PointsTopo, 0);        //LaBranche.PointsTopo := TListePointsTopo.Create;
                                               //LaBranche.PointsTopo.ClearListe();
    UneVisee := EmptyVisee('');
    addViseeAtBranche(LaBranche, UneVisee); //LaBranche.PointsTopo.AddElement(UneVisee);
                                            //LaBranche.NbPoints := LaBranche.PointsTopo.GetNbElements();
    FTableBranches.AddElement(LaBranche);
  end;
end;
function TConteneurCodeCalcul.GetBranche(const Idx: integer): TBrancheXYZ;
begin
  Result := FTableBranches.GetElement(Idx);
end;

procedure TConteneurCodeCalcul.RemoveBranche(const NumBrch: integer);   // OK
begin
  FTableBranches.RemoveElement(NumBrch);
end;

procedure TConteneurCodeCalcul.SetProcDisplayProgression(const QProcProgression: TProcDisplayProgression);
begin
  FProcDispProgression := QProcProgression;
end;

procedure TConteneurCodeCalcul.ViderTableJonctionsBranches;
begin
  FTableJonctions.ClearListe();
  FTableBranches.ClearListe();
//  FTableBranchesElaguees.ClearListe();
end;

procedure TConteneurCodeCalcul.PutBranche(const NoBr: integer; const LaBranche: TBrancheXYZ);
begin
  FTableBranches.PutElement(NoBr, LaBranche);
end;

procedure TConteneurCodeCalcul.addViseeAtBranche(var BR: TBrancheXYZ; const V: TUneVisee);
var
  n: Integer;
begin
  // Il n'y a pas d'opérations de suppression de visées dans un TBrancheXYZ
  // Un tableau Pascal suffit pour contenir les visées: il peut être
  // agrandi par un SetLength, qui agit comme le 'REDIM PRESERVE' du Basic.
  n := Length(BR.PointsTopo);
  SetLength(BR.PointsTopo, n + 1);
  n := High(BR.PointsTopo);
  BR.PointsTopo[n] := V;
  BR.NbPoints := 1 + n;
end;

function TConteneurCodeCalcul.GetIdxJonctionBySerSt(const Serie, Station: integer): integer;
var
  i   : integer;
  Jonc: TJonctionXYZ;
begin
  Result:=-1;
  for i:=0 to GetNbJonctions - 1 do
  begin
    Jonc := GetJonction(i);
    if ((Serie = Jonc.NoSer) and (Station = Jonc.NoSt)) then Exit(i);       // nouvelle méthode
  end;
end;


function TConteneurCodeCalcul.GetNbBranches(): integer;
begin
  Result := FTableBranches.GetNbElements();
end;

// découpage des séries en branches
procedure TConteneurCodeCalcul.RecenserBranches;
var
  Entr : TEntrance;
  Serie: TObjSerie;
  Visee: TUneVisee;
  Ser, Vis: integer;
  l1, a1, p1: double;
  Br   : integer;
  Nd   , QNbSeries, QNbVisees: integer;
  Branche0,
  Branche1, QBranche: TBrancheXYZ;
  QDeltaX: Extended;
  QDeltaY: Extended;
  QDeltaZ: Extended;
  QPosPtZero: TPoint3Df;
begin
  AfficherMessage(GetResourceString(rsFINDING_BRANCHES));
  //FTableBranches.ClearListe();
  // première branche
  with Branche0 do
  begin
    NoSerie     :=1;
    NoEntranceRatt := 0;
    NoReseau    :=0;
    NoBranche   :=1;
    NomBranche  := Format('Branche %d',[1]);
    NoeudDepart := 0;
    NoeudArrivee:= 1;
    Rigidite    := 1.00;
    SetLength(PointsTopo, 0);
    DeltaX:=0.01;
    DeltaY:=0.01;
    DeltaZ:=0.01;
  end;
  AddBranche(Branche0);
  Br := 2;
  //***********************
  // balayage des séries
  QNbSeries := FDocTopo.GetNbSeries();
  for Ser := 1 to QNbSeries - 1 do
  begin
    if (Assigned(self.ProcDispProgression) and (Ser MOD 100 = 0)) then self.ProcDispProgression(Format('Découpage en arcs de la série %d / %d', [Ser, QNbSeries - 1]), Ser, 0 , QNbSeries - 1);
    Serie := FDocTopo.GetSerie(Ser);
    Nd := GetIdxJonctionBySerSt(Serie.GetNoSerieDep, Serie.GetNoPointDep);

    // début de série = nouvelle branche
    //++++++++++++++++++++
    // Numéro de série = ID de la série
    Branche0.NoSerie        := Serie.GetNumeroDeSerie();
    Branche0.NoReseau       := Serie.GetNumeroReseau();    // Réseau
    Branche0.NoEntranceRatt := Serie.GetNumeroEntrance();
    Branche0.NoBranche      := Br;
    Branche0.NomBranche     := Format('Branche %d',[Br]);
    Branche0.NoeudDepart    := Nd;
    Branche0.Rigidite       := Serie.GetRaideur();
    AddBranche(Branche0);
    QNbVisees := Serie.GetNbVisees();
    for Vis := 1 to QNbVisees - 1 do
    begin
      Visee := Serie.GetVisee(Vis);
      Nd    := GetIdxJonctionBySerSt(Serie.GetNumeroDeSerie(), Vis);
      //Visee.NoViseeSer:=Vis;
      Visee.NoVisee := Vis;

      //Visee.TypeGalerie := 0; // type de galerie (provisoire)
      Branche1 := GetBranche(Br);                    //AddBrStation(Br, Visee);
      addViseeAtBranche(Branche1, Visee);             //Branche1 := QGetBranche(Br);
      PutBranche(Br, Branche1);

      Branche0.PointsTopo := Branche1.PointsTopo;
      Branche0.NbPoints   := 1 + High(Branche0.PointsTopo); // Branche1.NbPoints;
      //if ((Nd > -1) AND (Vis < Serie.GetNbVisees() - 1)) then
      if ((Nd > -1) AND (Vis < QNbVisees - 1)) then
      begin
        Branche0.NoeudArrivee := Nd;
        PutBranche(Br, Branche0);
        Inc(Br);

        Branche0.NoBranche   := Br;
        Branche0.NomBranche  := Format('Branche %d',[Br]);
        Branche0.NoeudDepart := Nd;
        AddBranche(Branche0);
      end;
      // fin de série = fin de branche: cloturer la branche
      if (Vis = QNbVisees - 1) then
      begin
        Nd := GetIdxJonctionBySerSt(Serie.GetNoSerieArr(), Serie.GetNoPointArr());
        Branche0.NoeudArrivee := Nd;
        PutBranche(Br, Branche0);
        Inc(Br);
      end;
    end;
  end;

  //**********************************************************
  // ajout des barres fictives (entrées de cavités)
  AfficherMessage(GetResourceString(rsADDENTRANCES));
  // La première entrée est déjà prise en compte
  for Ser := 0 to FDocTopo.GetNbEntrees() - 1 do
  begin   // initialement: for i:=1 to ...
    Entr:= FDocTopo.GetEntree(Ser);
    Branche0.NoSerie        := Entr.eRefSer;
    Branche0.NoEntranceRatt := 0;
    Branche0.NoReseau       := 0;
    Branche0.NoeudDepart    := GetIdxJonctionBySerSt(1,0);
    Branche0.NoeudArrivee   := GetIdxJonctionBySerSt(Entr.eRefSer, Entr.eRefSt);
    Branche0.Rigidite       := 1000.0;
    QPosPtZero              := FDocTopo.GetPositionDuPointZero();
    QDeltaX                 := Entr.eXEntree - QPosPtZero.X;
    QDeltaY                 := Entr.eYEntree - QPosPtZero.Y;
    QDeltaZ                 := Entr.eZEntree - QPosPtZero.Z;
    Branche0.DeltaX         := QDeltaX; //Entr.eDeltaX;
    Branche0.DeltaY         := QDeltaY; //Entr.eDeltaY;
    Branche0.DeltaZ         := QDeltaZ; //Entr.eDeltaZ;
    //AfficherMessage(Format('--> %d: %f %f %f',[Ser, Entr.eDeltaX, Entr.eDeltaY, Entr.eDeltaZ]));
    Visee.Code := 0; //1
    Visee.Expe := 0; //1

    l1 := 0.0;    a1 := 0.00;    p1 := 0.00;
    GetBearingInc(QDeltaX, QDeltaY, QDeltaZ, l1, a1, p1, 400.0, 400.0);
    Visee.NoVisee  := 1;
    Visee.Longueur := l1;
    Visee.Azimut   := a1;
    Visee.Pente    := p1;
    Visee.LD       := 0.00;
    Visee.LG       := Visee.LD;
    Visee.HZ       := Visee.LD;
    Visee.HN       := Visee.LD;
    Visee.Commentaires     := '';
    Visee.IDTerrainStation := Format(FMTSERST,[Entr.eRefSer, Entr.eRefSt]);
    Visee.TypeVisee        := tgENTRANCE;
    Visee.Commentaires     := SafeTruncateString(Entr.eNomEntree, 15);

    AddBranche(Branche0);
    QBranche := GetBranche(Br);    //;;;
    addViseeAtBranche(QBranche, Visee);                                 //AddBrStation(Br, Visee);
    PutBranche(Br, QBranche);
    //Visee := QBranche.PointsTopo[1]; //Visee := GetBrStation(Br, 1);
    Inc(Br);
  end;
  //**********************************************************
  AfficherMessage(GetResourceString(rsFINDING_BRANCHES) + ' OK');
end;

// calcul des accroissements sur les trois axes des branches
// TODO: Vérifier ce code au niveau des corrections sur pentes.
procedure TConteneurCodeCalcul.CalculerAccroissements();
var
  Br, Vs : integer;
  Branche: TBrancheXYZ;
  Visee  : TUneVisee;
  DX, DY, DZ, DP: double;
  WU: Boolean;
begin
  if (Assigned(self.ProcDispProgression)) then self.ProcDispProgression('Calcul accroissements des branches', -1, -1, -1);
  for Br:=1 to GetNbBranches() - 1 do
  begin

    Branche:= GetBranche(Br);
      DX := 0.0; DY := 0.0; DZ := 0.0;
      for Vs := 0 to High(Branche.PointsTopo) do //Branche.PointsTopo.GetNbElements() - 1 do
      begin
        Visee := Branche.PointsTopo[Vs];
        CalculerVisee(Visee,
                      FDocTopo.GetCodeByNumero(Visee.Code),
                      FDocTopo.GetExpeByNumero(Visee.Expe),
                      DX, DY,
                      1.00,
                      DZ, DP);
        Branche.PointsTopo[Vs] := Visee; ////PutBrStation(Br, Vs, Visee);
      end; //for Vs:=0 to Branche.PointsTopo.Count-1 do begin
      Branche.DeltaX:=DX;
      Branche.DeltaY:=DY;
      Branche.DeltaZ:=DZ;
    PutBranche(Br, Branche);
  end; // for Br:=1 to TableBranches.Count-1 do begin

  // Nettoyage des branches à accroissement nul
  // cause d'un bug très sournois
  for Br := GetNbBranches() - 1 downto 0 do
  begin
    Branche := GetBranche(Br);
    WU := (Branche.NoeudArrivee > 1) and // ne pas supprimer la branche 0-1
          (IsZero(Branche.DeltaX) and IsZero(Branche.DeltaY) and IsZero(Branche.DeltaZ));
    if (WU) then RemoveBranche(Br);
  end;
end;


end.
