unit unitdistox;
// By JP CASSOU under Licence GPL
// Uses TLazSerial library (author: Jurassik Pork)
// 05/02/2015: Calcul de calibration OK (méthode standard)
// 20/05/2015: Bug à la libération de TPilotageDistoX fixé
// 08/06/2016: Les listes passent aux génériques
// 31/01/2017: Nouvelle version. Dépilage des mesures topo OK,
{$INCLUDE CompilationParameters.inc}

interface

uses
  Classes, SysUtils, Math,
  StructuresDonnees,
  Common,
  {$IFDEF USE_GENERIC_TYPES}
  UnitListesSimplesWithGeneriques,
  {$ELSE}
  UnitListesSimplesWithoutGeneriques,
  {$ENDIF}
  {$IFDEF MSWINDOWS}
  LazSerial, //synaser,  // Pour la version Linux, retirer la dépendance
  LazSynaSer,
  {$ENDIF}
  {$IFDEF LINUX}
  LazSerial, synaser,
  {$ENDIF}
  {$IFDEF MAC_OS_X_VERSION_MIN_REQUIRED}
  {$ENDIF}
  StdCtrls,
  ExtCtrls, // pour Timer
  CalculMatriciel3x3,  // Outils de calcul matriciel 3x3 évitant de sortir l'artillerie lourde
  Grids,
  Forms, Graphics;
const
  // axes
  mrX_AXIS = 1;
  mrY_AXIS = 2;
  mrZ_AXIS = 3;
  // mappage de la mémoire du DistoX
  //ADDR_START_SECTION_CONFIGURATION   = $B000; // calibration coefficients, Ox00B000 -> Ox00B3FF
  //ADDR_START_SECTION_DATA_STORE      = $B400; // data segment            , Ox00B400 -> Ox00FFFF
  // commandes DistoX
  DISTOX_COMMAND_EXTINCTION             =  52;  // $34: extinction du Disto
  DISTOX_COMMAND_READ_MEMORY_AT_ADDRESS =  56;  // $38: b111000 ; conflict with measure trigging command
  DISTOX_COMMAND_CALIBRATION_ON         =  49;  // $31: b110001: Calibration ON
  DISTOX_COMMAND_CALIBRATION_OFF        =  48;  // $30: b110000: Calibration OFF
  DISTOX_COMMAND_LASER_ON               =  54;  // $36: b110110: Laser ON
  DISTOX_COMMAND_LASER_OFF              =  55;  // $37: b110111: Laser OFF
  //DISTOX_COMMAND_BEEP_ON                = $33;  // $33: Silent ON  = à ne pas utiliser
  //DISTOX_COMMAND_BEEP_OFF               = $32;  // $33: Silent OFF
  DISTOX_COMMAND_READ_DATA_SHOT         =  56;  // $38: lecture d'une visée
  DISTOX_COMMAND_BOOTLOADER_READ_DATA_AT_ADDRESS =  58;  // $38: b111010 ; conflict with measure trigging command

  ERR_MSG_NO_CONNEXION                  = '** No Connexion **';
type TMesureATraiter = record
  Longueur: double;
  Azimut  : double;
  Pente   : double;
  dx      : double;
  dy      : double;
  dp      : double;
  dz      : double;
end;
type TArrayOfMesuresATraiter = array of TMesureATraiter;

type TDistoXData8BytesBuffer = array[0..7] of Byte;
type TBootloaderModeDataBuffer = array[0..263] of Byte;

(*
type TModeAttenteDonneesRX = (mrxNONE, mrxDEPILAGE_MESURE);
type TProtoSerialError = (PROTO_OK,
                          PROTO_READ, PROTO_WRITE,
                          PROTO_COMMAND,  PROTO_ADDR,  PROTO_PACKET,
                          PROTO_CONNECT, PROTO_TIMEOUT, PROTO_MAX);

//*)
type

{ TPilotageDistoX }

 TPilotageDistoX = class(TLazSerial)
  private
    FTimeOutInMilliseconds: integer;
    // Identité du DistoX
    FDistoXSerialNumber   : integer;
    FDistoXVersionHardware: integer;
    FDistoXVersionFirmware: integer;
    FUniteDuDistoX        : double;
    // Mode 'Calibration' sinon, mode mesure
    FCalibrationModeON    : boolean;
    FLaserON              : boolean;
    //FBeepON             : boolean;
    FNbMesuresCalibration    : integer;
    FDoNonLinearCalibration  : boolean;
    // controles d'affichages
    FLabelActionsProcessing  : TStaticText;
    FGridDataGMDistoX        : TStringGrid;
    //FTimer                : TTimer;
    oldtype, oldX, oldY, oldZ: Integer;
    // listes des mesures
    FListeMesuresTopo     : TListeMesuresTopoDistoX;
    // outils de calibration (fusion de TPilotageDistoX)
    FListeMesuresDeG      : TListePoints3Df;
    FListeMesuresDeM      : TListePoints3Df;
    fMatrix_aG  : TMatrix3x3;
    fMatrix_aM  : TMatrix3x3;
    fVecteur_bG : TPoint3Df;
    fVecteur_bM : TPoint3Df;
    fDelta      : double;
    fNbIters    : integer;
    // callback de transmission de visée lors de sa réception
    FProcTransmitViseeToTableau      : TProcTransmitMesureVisee;
    FProcTransmitViseeToGHTopoCtxt2D : TProcTransmitMesureVisee;
    FProcAfficherProgression: TProcDisplayProgression;
    // modes d'attente du DistoX
    FModeAttenteDonneesRX : TModeAttenteDonneesRX;
    FNbMesuresPourViseeCheminement   : integer;

    FDistoXAccuracyLongueurs         : double;
    FDistoXAccuracyAzimuts           : double;
    FDistoXAccuracyPentes            : double;
    // série et station courante
    FCurrentSerie  : TNumeroSerie;
    FCurrentStation: integer;

    // journalisation des mesures
    FFichierJournalPrincipal         : TStringDirectoryFilename;
    FFichierJournalBackup            : TStringDirectoryFilename;





    //procedure ProcOnStatus(Sender: TObject; Reason: THookSerialReason; const Value: string);

    function CalcAzimutFromdXdY(const qdx, qdy: double): double;

    function MoyenneAzimuts(const QArr: TArrayOfMesuresATraiter): double;
    procedure SendReadCommandAtAddress(var MyBuffer: TDistoXData8BytesBuffer;const QCmd: word; const QAddress: word);
    // Buffers vierges
    function  MakeBootloaderBufferVierge(): TBootloaderModeDataBuffer;
    function  MakeBufferVierge(): TDistoXData8BytesBuffer;
    function  MakeContentBuffer(const B: TDistoXData8BytesBuffer): string;
    // Communication avec le DistoX (acquittement)
    function  Acknowledge(const B: byte): TProtoSerialError;
    // extraction des données du buffer
    function  MakeTMesureViseeDistoXFromDataBuffer(const QBlock, QSegment: integer; const MyBuffer: TDistoXData8BytesBuffer): TMesureViseeDistoX; overload;
    function  MakeTMesureViseeDistoXFromDataBuffer(const MyBuffer: TDistoXData8BytesBuffer): TMesureViseeDistoX; overload;


    // Fonctions de lecture/ecriture
    function  ReadBufferAtAddress(const QAddr: integer; var MyBuffer: TDistoXData8BytesBuffer): boolean;
    function  ReadBuffer8bytes(var MyBuffer: TDistoXData8BytesBuffer;
                               out LastSerialError: integer;
                               out QTypeData: byte;
                               out QOp      : byte): TProtoSerialError;


    function  WriteByte(const B: byte): Integer;


    function  ReadAnPacket(const QAddr: word; var QBuffer: TDistoXData8BytesBuffer): boolean;
    function  LireEtDepilerTrameDe8OctetsData(): boolean;
    // lecture du buffer 264 bits
    function  BootloaderReadBuffer264bytes(var MyBuffer: TBootloaderModeDataBuffer; const Nb: integer): integer;
    procedure BootloaderSendReadCommandAtAddress(var MyBuffer: TBootloaderModeDataBuffer;const QCmd: word; const QAddress: word);
    function  BootloaderReadAnPacket264(const QAddr: word; var QBuffer: TBootloaderModeDataBuffer): boolean;
    //-------------------------------------
    //procedure ProcTimer(Sender: TObject);
    procedure SetHeaderGrdGM;
    function  getEmptyMesure(const QTypeMesure: TTypeMesureDistoX): TMesureViseeDistoX;
    //*******************************************************************
    // calculs de calibration
    procedure setJeuDeDonneesExemple();
    procedure OptVectors(const gr, mr: TPoint3Df; const alpha: double; out gx, mx: TPoint3Df);
    procedure TurnVectors(const gxp, mxp, gr, mr: TPoint3Df; out gx, mx: TPoint3Df);
    procedure CheckOverflow(var M: TMatrix3x3; var V: TPoint3Df);
    // matrices
    function GetMatrix_aG()  : TMatrix3x3;
    function GetMatrix_aM()  : TMatrix3x3;
    function GetVecteur_bG() : TPoint3Df;
    function GetVecteur_bM() : TPoint3Df;
    function GetNbIters()    : integer;

    procedure SetCurrentNumSerie(const s: TNumeroSerie); inline;
    function  GetCurrentNumSerie(): TNumeroSerie; inline;

    procedure EvenementSerialRxData(Sender: TObject);
    procedure EvenementSerialStatus(Sender: TObject; Reason: THookSerialReason; const Value: string);

    function DescribeProtoSerialError(const E: TProtoSerialError): string;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetTimeoutDelay(const TimeOutInMilliseconds: integer);
    function  OpenDistoXConnexion(): boolean;              // ouverture du port
    procedure CloseDistoXConnexion();                      // fermeture du port
    procedure ExtinctDevice();

    // procédures d'objet rattachées
    procedure SetProcNotifySuccessConnexion(const P: TProcedureOfObject);

    // journalisation des mesures
    procedure SetFileNameJournaux(const JP, JS: TStringDirectoryFilename);
    function  GetFilenameJournalPrincipal(): TStringDirectoryFilename;
    function  GetFilenameJournalSecurite(): TStringDirectoryFilename;
    procedure ViderJournaux();
    //**************************************************************************
    //**************************************************************************
    // Fonctions à utiliser par l'appelant
    //************************************
    //  Fonctions n'ayant pas besoin de la connexion BT
    // --------------------------------------------------
    // widgets utilisés par TPilotageDistoX
    procedure SetLabelActionsProcessing(const L: TStaticText);
    // callbacks de transmission de valeur de mesure de visées
    procedure SetProcProgression(const P: TProcDisplayProgression);
    procedure SetProcTransmitViseeToTableau(const P: TProcTransmitMesureVisee);
    procedure SetProcTransmitViseeToGHTopoCtxt2D(const P: TProcTransmitMesureVisee);


    // vider les tables de données
    procedure ViderListesMesuresGM();
    procedure ViderListeMesuresTopo(const DoAddViseeBidon: boolean);  // vider liste des mesures topo

    // données de visées (mesures)
    procedure ChargerDonneesViseesBrutes(const Filename: string);
    procedure SauverDonneesViseesBrutes(const Filename: string);
    function  GetNbMesuresAcquises(): integer;
    procedure AddMesureVisee(const V : TMesureViseeDistoX; const DoRegisterInLogs: boolean);
    function  GetMesureVisee(const Idx: integer): TMesureViseeDistoX;
    procedure PutMesureVisee(const Idx: integer; const V: TMesureViseeDistoX);
    procedure RemoveMesureVisee(const Idx: integer);
    function  GetLastMesureVisee(): TMesureViseeDistoX;
    procedure RemoveLastMesureVisee;
    // nombre de mesures à considérer pour une visée
    procedure SetNbMesuresPourVisees(const n: integer);


    // données de calibration
    procedure SetNonLinearCalibration(const o: boolean);
    function  CheckMethodeCalibration(): boolean;         // contrôle de la méthode de calibration avec un dataset d'exemple
    function  GetDataGMFromGrid(const grd: TStringGrid): boolean;
    function  SetDataGMInGrid(const grd: TStringGrid): boolean;

    procedure AddMesureG(const MG: TPoint3Df);
    procedure AddMesureM(const MG: TPoint3Df);
    procedure AddMesuresGMByValues(const gx, gy, gz, mx, my, mz: double);
    function  GetMesureG(const Idx: integer): TPoint3Df;
    function  GetMesureM(const Idx: integer): TPoint3Df;
    function  GetNbMesuresG(): integer;
    function  GetNbMesuresM(): integer;
    function  GetDelta()      : double;

    function  IsReadyForCalibrate: boolean;
    procedure CalculerCalibration();
    //--------------------------------------------------------------------------
    //  Fonctions nécessitant la connextion BT
    // --------------------------------------------------
    // initialisation
    procedure SetLazSerialDevice(const D: string);   // Device: ex: COM1:
    // numéro de série du DistoX
    function  ExtractNumSerieFromDistoX(): integer;
    function  ExtractVersionFirmWareFromDistoX(): integer;
    function  ExtractVersionHardwareFromDistoX(): integer;

    // lire / écrire les paramètres de calibrage
    procedure ReadCalibrationParametersFromDistoX();
    function  SendCalibrationParametersToDistoX: integer;

    // définir les statuts
    procedure SetReadingTopoData(const B: boolean);
    procedure SetLaserOnOff(const o: boolean);
    procedure SetCalibrationOnOff(const o: boolean);
    //procedure SetBeepOnOff(const o: boolean);


    // obtenir les statuts
    function  IsLaserON()              : boolean;
    function  IsCalibrationON()        : boolean;
    function  IsAcquiringTopoData()    : boolean;
    //function  IsBeepOn()               : boolean;

    // dump de la zone de données
    function ReadDataSection(const NumBlockDebut, NumBlockFin: integer): integer;
    // envoi d'un ordre de mesure (télécommande du bouton Mesure)
    procedure DemanderUneMesure();
    procedure DemanderUneVisee();
    // modes d'attente
    function  GetModeAttenteDonneesRX(): TModeAttenteDonneesRX;
    procedure SetModeAttenteDonneesRX(const Q: TModeAttenteDonneesRX);

    function ConstruireUneViseeValide(const MyTypeVisee: TTypeDeVisee;
                                      const MySecteur: TNumeroSecteur;
                                      const MyCode: TNumeroCode;
                                      const MyExpe: TNumeroExpe;
                                      out V: TUneVisee;
                                      out Err: TErrorConstruireUneViseeValide): boolean;

    procedure SetAccuracies(const AccLong, AccAz, AccP: double);

    procedure EcrireMesureDansJournaux(const V: TMesureViseeDistoX);
end;

implementation

const
  FV = 24000.00;
  FM = 16384.00;

  MAX_IT = 500;
  EPS    = 1E-8;
  HEX_TO_RAD = 2 * PI / 65536;

  INVALID_HANDLE_VALUE = THandle(-1);

{ TPilotageDistoX }
constructor TPilotageDistoX.Create(AOwner: TComponent);
var
  QJournalName: String;
begin
  (*
  2 - Linux
To simulate  a paired serial ports on linux : socat
ex :  socat -d -d PTY: PTY:

To connect to a bluetooth GPS on Linux
echo connection to a  bluetooth GPS(/dev/rfcomm0)

sudo rfcomm bind  0 xx:xx:xx:xx:xx:xx 1
//*)
  inherited Create(AOwner);
  FNbMesuresPourViseeCheminement   := 3;
  FDistoXAccuracyLongueurs         := 0.1;
  FDistoXAccuracyAzimuts           := 1.00;
  FDistoXAccuracyPentes            := 1.00;
  //FDistoXMustBeReady     := false;
  FUniteDuDistoX         := 360.00;
  FTimeOutInMilliseconds := 30000;
  FDistoXSerialNumber    := -1;
  FDistoXVersionHardware := -1;
  FDistoXVersionFirmware := -1;
  FDoNonLinearCalibration := true;
  //FProcNotifyDistoXConnected := nil;
  FProcAfficherProgression         := nil;
  FProcTransmitViseeToTableau      := nil;
  FProcTransmitViseeToGHTopoCtxt2D := nil;
  FModeAttenteDonneesRX      := mrxNONE;
  AfficherMessageErreur(Format('%s.Create',[self.ClassName]));
  //FTimer := TTimer.Create(Application);
  FListeMesuresTopo  := TListeMesuresTopoDistoX.Create;
  FListeMesuresDeG   := TListePoints3Df.Create;
  FListeMesuresDeM   := TListePoints3Df.Create;
  FNbMesuresCalibration := 0;
  AfficherMessageErreur('-- Assign OnRxData event');
  OnRxData := EvenementSerialRxData;
  AfficherMessageErreur('-- Assign OnStatus event');
  OnStatus := EvenementSerialStatus;

  // tests de canaux
  SynSer.TestDSR := True;
  SynSer.TestCTS := True;
  SynSer.DeadlockTimeout := 10000; // 10 secondes


  // journaux
  QJournalName                     := 'Journal_' + DatePascalToDateSQL(Now()) + '.log';
  FFichierJournalPrincipal         := GetGHTopoDirectory() + QJournalName;
  {$IFDEF MSWINDOWS}
  FFichierJournalBackup            := 'D:' + PathDelim + QJournalName;
  {$ENDIF MSWINDOWS}
  {$IFDEF LINUX}
  FFichierJournalBackup            := FFichierJournalPrincipal + '.2';
  {$ENDIF LINUX}


  try
    ViderListeMesuresTopo(True);
    ViderListesMesuresGM();
  finally
  end;
end;

destructor TPilotageDistoX.Destroy;
begin
  AfficherMessage(Format('%s.Destroy',[self.ClassName]));
  try
    ViderListeMesuresTopo(false);
    ViderListesMesuresGM;
    //self.CloseLazSerial;
    AfficherMessage('TPilotageDistoX destroyed');
  finally
    // /!\ Les objets instanciés par Application ne doivent jamais être libérés explicitement
    // Laisser le parent faire ce travail
    //FListeMesuresTopo.Free;
    //FListeMesuresDeG.Free;
    //FListeMesuresDeM.Free;
    // FTimer.Free;
  end;
  inherited Destroy;
  //*)
end;
function TPilotageDistoX.CalcAzimutFromdXdY(const qdx, qdy: double): double;
var
  a: float;
  procedure miaou; inline;
  begin
    if (a < 0) then a := a + 2 * PI;
  end;
begin
  a := ArcTan2(qdy, qdx+1e-12);
  miaou;
  a := 0.50 * PI - a;
  miaou;
  Result := a * 0.50 * FUniteDuDistoX / PI;    //FUniteDuDistoX = 360.00
end;


procedure TPilotageDistoX.SetTimeoutDelay(const TimeOutInMilliseconds: integer);
begin
  FTimeOutInMilliseconds := TimeOutInMilliseconds;
  self.SynSer.DeadlockTimeout := FTimeOutInMilliseconds;
end;

// FONCTIONS DE BASE
//******************************************************************************
function TPilotageDistoX.WriteByte(const B: byte): Integer;
begin
  try
    Result := 0;
    if (SynSer.Handle =  INVALID_HANDLE_VALUE) then exit; //ComException('can not write to a closed port.');
    SynSer.SendByte(B);
    result := 1;
  except
  end;
end;


function TPilotageDistoX.ReadBufferAtAddress(const QAddr: integer; var MyBuffer: TDistoXData8BytesBuffer): boolean;
var
  t0, t1: TDateTime;
begin
  result := False;
  try
    // on boucle tant que la lecture des paquets n'a pas réussi
    while (not ReadAnPacket(QAddr, MyBuffer)) do AfficherMessageErreur(Format('%s.ReadBufferAtAddress: %s: %s', [self.ClassName, {$I %FILE%}, {$I %LINE%}]));
      ;
    Result := True;
  except
    //FDistoXSerialNumber := -1;
  end;
end;
//******************************************************************************

// INFOS SUR LE MATERIEL


// numéro de série du DistoX
function TPilotageDistoX.ExtractNumSerieFromDistoX(): integer;
var
  MyBuffer: TDistoXData8BytesBuffer;
begin
  Application.ProcessMessages;
  if (FDistoXSerialNumber = -1) then          // si FDistoXSerialNumber = - 1, on fait une tentative d'extraction;
  begin
    if (ReadBufferAtAddress($8008, MyBuffer)) then  FDistoXSerialNumber := MyBuffer[3] + MyBuffer[4] shl 8; // retourne 2395 ou 2329 pour les DistoX de JPC
  end;
  Result := FDistoXSerialNumber;
end;
// version de firmware
function TPilotageDistoX.ExtractVersionFirmWareFromDistoX(): integer;
var
  MyBuffer: TDistoXData8BytesBuffer;
begin
  Application.ProcessMessages;
  if (FDistoXVersionFirmware = -1) then     // si FDistoXVersionFirmware = - 1, on fait une tentative d'extraction;
  begin
    if (ReadBufferAtAddress($E000, MyBuffer)) then FDistoXVersionFirmware := 100 * MyBuffer[3] + MyBuffer[4];
  end;
  Result := FDistoXVersionFirmware;
end;
// version de hardware
function TPilotageDistoX.ExtractVersionHardwareFromDistoX(): integer;
var
  MyBuffer: TDistoXData8BytesBuffer;
begin
  Application.ProcessMessages;
  if (FDistoXVersionHardware = -1) then
  begin
    if (ReadBufferAtAddress($E004, MyBuffer)) then FDistoXVersionHardware := 10 * MyBuffer[3] + MyBuffer[4];     // byte 3 = major; byte 4 = minor, bytes 5 et 6 = 0
  end;
  Result := FDistoXVersionHardware;
end;

function TPilotageDistoX.Acknowledge(const B: byte): TProtoSerialError;
var
  MyByte: Byte;
begin
  MyByte := (B and $80) or $55;
  if (WriteByte(MyByte) = 1) then result := PROTO_OK else result := PROTO_WRITE;
end;



function TPilotageDistoX.LireEtDepilerTrameDe8OctetsData(): boolean;
var
  EWE: String;
  MyBuffer: TDistoXData8BytesBuffer; //array[0..7] of Byte;
  i, Nb: Integer;
  X, Y, Z: Integer;
  Dist: Integer;
  MesureG, MesureM: TPoint3Df;
  AbsG, AbsM, DipAngle: Integer;
  myMesureVisee: TMesureViseeDistoX;
  P: float;
  NbV: Integer;
  NbG: Integer;
  NbM, LastSerialError: Integer;
  MyOp, QTypeData: byte;
  function ConvFromDistoX(const valeur: integer): integer;
  begin
    Result := valeur;
    if (Result > 32767) then Result := Result - 65536;
  end;
begin
  Application.ProcessMessages;
  Result := False;
  try
     MyBuffer := MakeBufferVierge();
     if (FModeAttenteDonneesRX = mrxNONE) then Exit;
     myMesureVisee.TypeMesure := tmdxUNKNOWN;
     myMesureVisee.Longueur := 0.00;
     myMesureVisee.Azimut   := 0.00;
     myMesureVisee.Pente    := 0.00;
     myMesureVisee.IsMarked := false;
     myMesureVisee.HexaData := '';
     Nb := High(MyBuffer);
     EWE := '';
     // lire 8 octets - TODO: version à valider
     if (PROTO_OK = self.ReadBuffer8bytes(MyBuffer, LastSerialError, QTypeData, MyOp)) then
     begin
       // Ce sont des données ?
       if (MyOp < $20) then
       begin
         X := MyBuffer[1] + (MyBuffer[2] shl 8);
         Y := MyBuffer[3] + (MyBuffer[4] shl 8);
         Z := MyBuffer[5] + (MyBuffer[6] shl 8);
         if ((oldtype <> QTypeData) and
             (oldX    <> X) and
             (oldY    <> Y) and
             (oldZ    <> Z)) then
         begin
           case MyOp of
             1: // données topo
             begin
               // distances exprimées en mm si < 100 m, sinon en cm
               Dist := X + ((QTypeData and $40) shr 10);     // OK
               //myMesureVisee.TypeMesure := op;
               myMesureVisee.TypeMesure := tmdxUNKNOWN;
               myMesureVisee.Longueur := Dist * 0.001; // TODO: Cas où Dist > 100 m
               myMesureVisee.Azimut   := radtodeg(Y * HEX_TO_RAD);                   // OK
               P  := radtodeg(Z * HEX_TO_RAD);
               // if (Not InRange(P, -0.001, (90.001)) then P := P - 360.00;
               if (Not InRange(P, -0.001, 0.25 * FUniteDuDistoX + 0.001)) then P := P - FUniteDuDistoX;
               myMesureVisee.Pente := P;
               myMesureVisee.HexaData := EWE;
               myMesureVisee.TimeStamp:= Now();
               // et on ajoute la mesure
               self.AddMesureVisee(myMesureVisee, True);
               myMesureVisee := GetLastMesureVisee();
               AfficherMessage(Format('%d: qtype = %.2X; op = %.2X; L = %.3f m; Az = %.2f; P = %.2f - EWE = %s',
                                     [GetNbMesuresAcquises(),
                                      QTypeData, MyOp,
                                      myMesureVisee.Longueur,
                                      myMesureVisee.Azimut,
                                      myMesureVisee.Pente,
                                      myMesureVisee.HexaData
                                      ]));
               // on appelle le callback d'envoi vers le tableau général des mesures
               if (Assigned(FProcTransmitViseeToTableau))      then FProcTransmitViseeToTableau(myMesureVisee);
               // et celui vers le visualisateur 2D de GHTopo
               if (Assigned(FProcTransmitViseeToGHTopoCtxt2D)) then FProcTransmitViseeToGHTopoCtxt2D(myMesureVisee);

             end;
             2, 3: ; // G et M, ne fonctionne pas
           else
             ;
           end;
           oldtype := QTypeData;
           oldX    := X;
           oldY    := Y;
           oldZ    := Z;
         end;
       end;
     end; //  if (Acknowledge(qtype) = PROTO_OK)) then
   except
     ;
   end;

end;





procedure TPilotageDistoX.SetCalibrationOnOff(const o: boolean);
var
  EWE: byte;
begin
  AfficherMessage(Format('%s.SetCalibrationOn %s', [ClassName, IIF(o, 'ON', 'OFF')]));
  EWE := IIF(o, DISTOX_COMMAND_CALIBRATION_ON, DISTOX_COMMAND_CALIBRATION_OFF);
  self.WriteByte(EWE);
  FCalibrationModeON := o;
  Application.ProcessMessages;
end;
(*
procedure TPilotageDistoX.SetBeepOnOff(const o: boolean);
var
  EWE: TColor;
begin
  if (self.Active) then
  begin

    AfficherMessage(Format('%s.SetBeepOnOff %s', [ClassName, IIF(o, 'ON', 'OFF')]));
    EWE := IIF(o, DISTOX_COMMAND_BEEP_ON, DISTOX_COMMAND_BEEP_OFF);
    self.WriteByteOfData(EWE);
    FBeepON := o;
  end
  else
  begin
    AfficherMessage(Format('%s.SetBeepOnOff %s: ** NO CONNEXION **', [ClassName, IIF(o, 'ON', 'OFF')]));
  end;
end;
//*)
procedure TPilotageDistoX.SetLaserOnOff(const o: boolean);
var
  EWE: Byte;
begin
  if (self.Active) then
  begin
    AfficherMessage(Format('%s.SetLaserOn %s', [ClassName, IIF(o, 'ON', 'OFF')]));
    EWE := IIF(o, DISTOX_COMMAND_LASER_ON, DISTOX_COMMAND_LASER_OFF);
    self.WriteByte(EWE);
    FLaserON := o;
  end
  else
  begin
    AfficherMessage(Format('%s.SetLaserOn %s: ** NO CONNEXION **', [ClassName, IIF(o, 'ON', 'OFF')]));
  end;
  Application.ProcessMessages;
end;

procedure TPilotageDistoX.ExtinctDevice();
var
  EWE: byte;
begin
  Application.ProcessMessages;
  AfficherMessage(Format('%s.ExtinctDevice', [ClassName]));
  // Valeur de la commande d'extinction: 00110100
  EWE := DISTOX_COMMAND_EXTINCTION;//  $34;
  self.WriteByte(EWE);
  self.CloseDistoXConnexion();
end;

procedure TPilotageDistoX.SetProcNotifySuccessConnexion(const P: TProcedureOfObject);
begin
  //FProcNotifyDistoXConnected := P;
end;

// acquisition d'une mesure
procedure TPilotageDistoX.DemanderUneMesure(); // OK
var
  EWE: byte;
begin
  if (not self.Active) then Exit;
  AfficherMessage(Format('%s.TrigMesure', [ClassName]));
  // Valeur de la commande trig mesure: 00111000
  EWE := DISTOX_COMMAND_READ_MEMORY_AT_ADDRESS;//$38;
  self.WriteByte(EWE);
end;

procedure TPilotageDistoX.DemanderUneVisee();
var
  i: Integer;
begin
  for i := 0 to FNbMesuresPourViseeCheminement - 1 do
  begin
    Application.ProcessMessages;
    DemanderUneMesure();
    //while (not synser.Carrier) do pass;
    sleep(5000);
  end;
end;



function TPilotageDistoX.OpenDistoXConnexion(): boolean;
begin
  //Application.ProcessMessages;
  Result := false;

  FLaserON              := False;
  FModeAttenteDonneesRX := mrxNONE;
  oldtype  := 0;
  oldX     := 0;
  oldY     := 0;
  oldZ     := 0;
  //SetTimeoutDelay(10000);

  // TLazserial présumé fonctionner sur Win et Linux
  try
    self.OnRxData      := EvenementSerialRxData;
    self.OnStatus      := EvenementSerialStatus;
    //self.Active        := true;
    self.Open;
    Result             := self.Active; // ouverture de la connexion. Ne pas utiliser self.Open et consorts
    if (Result) then SetCalibrationOnOff(false); //FCalibrationModeON    := False;
  except
    on E: Exception do AfficherMessageErreur(Format('Connexion sur port "%s" impossible' + #13#10 + 'Message: %s',
          [self.Device, E.Message]));
  end;
end;

procedure TPilotageDistoX.CloseDistoXConnexion();
begin
  AfficherMessage(Format('%s.CloseLazSerial', [ClassName]));
  Application.ProcessMessages;
  oldtype  := 0;
  oldX     := 0;
  oldY     := 0;
  oldZ     := 0;
  try
    FModeAttenteDonneesRX := mrxNONE;
    SetCalibrationOnOff(false);//FCalibrationModeON := False;
    FLaserON           := False;
    FDistoXSerialNumber    := -1;
    FDistoXVersionHardware := -1;
    FDistoXVersionFirmware := -1;
    self.Close;
    //self.Active := false;
  finally
  end;
end;
//------------------------------------------------------------------------------

procedure TPilotageDistoX.ReadCalibrationParametersFromDistoX();
const
  NB_PARAMS_CALIB = 12;
var
  AddressParamCalibG: word;
  MyBuffer: TDistoXData8BytesBuffer;
  WU: Integer;
  k : integer;
  AddressParamCalibM: Integer;
begin
  if (not self.Active) then Exit;
  SetCalibrationOnOff(True);

  AfficherMessage(Format('%s.ReadCalibrationParametersFromDistoX()', [ClassName]));
  AddressParamCalibG := $8010;
  for k := 0 to NB_PARAMS_CALIB - 1 do
  begin
    MyBuffer := MakeBufferVierge();

    while (not ReadAnPacket(AddressParamCalibG + k * 2, MyBuffer)) do AfficherMessageErreur(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
      ;
    if (MyBuffer[0] = $38) then  // le premier élément du buffer doit retourner 0x38 pour une calibration
    begin
      WU := MyBuffer[3] + MyBuffer[4] shl 8; //3, 4
      AfficherMessage(Format('%d: Addr = %X  - Buff[0] = %X - Buffer = %s - Value of G = %d',
                            [k, AddressParamCalibG + k * 2, MyBuffer[0], MakeContentBuffer(MyBuffer), WU]));

    end;
    // traitement
    //+++++++++++
    //AddMesureG();
    //AddressParamCalib := AddressParamCalib + 4;
  end;
  AddressParamCalibM := $8028;
  for k := 0 to NB_PARAMS_CALIB - 1 do
  begin
    MyBuffer := MakeBufferVierge();
    while (not ReadAnPacket(AddressParamCalibM + k * 2, MyBuffer)) do AfficherMessageErreur(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
      ;
    WU := MyBuffer[3] + MyBuffer[4] shl 8;
    AfficherMessage(Format('%d: Addr = %X  - Buff[0] = %X - Buffer = %s - Value of M = %d',
                          [k, AddressParamCalibM + k * 2, MyBuffer[0], MakeContentBuffer(MyBuffer), WU]));
    // traitement
    //+++++++++++

    //AddressParamCalib := AddressParamCalib + 4;
  end;
  SetCalibrationOnOff(false);

end;



// en attente
function TPilotageDistoX.SendCalibrationParametersToDistoX: integer;
begin
  Result := -1;
  if (not self.Active) then Exit;
  AfficherMessage(Format('%s.SendCalibrationParametersToDistoX()', [ClassName]));
end;


(*
// type for DistoX buffer
type TDistoXDataBuffer = array[0..7] of Byte;

// DistoX memory map
ADDR_START_SECTION_CONFIGURATION   = $B000; // calibration coefficients, Ox00B000 -> Ox00B3FF
ADDR_START_SECTION_DATA_STORE      = $B400; // data segment            , Ox00B400 -> Ox00FFFF

// commandes DistoX
DISTOX_COMMAND_EXTINCTION             =  52;  // $34: extinction du Disto
DISTOX_COMMAND_READ_MEMORY_AT_ADDRESS =  56;  // $38: b111000 ; conflict with measure trigging command
DISTOX_COMMAND_CALIBRATION_ON         =  49;  // $31: b110001: Calibration ON
DISTOX_COMMAND_CALIBRATION_OFF        =  48;  // $30: b110000: Calibration OFF
DISTOX_COMMAND_LASER_ON               =  54;  // $36: b110110: Laser ON
DISTOX_COMMAND_LASER_OFF              =  55;  // $37: b110111: Laser OFF
DISTOX_COMMAND_READ_DATA_SHOT         =  56;  // $38: lecture d'une visée
//*)




// lecture d'un paquet (retourne 4 octets)
// TODO: A valider - Non testé
function TPilotageDistoX.ReadAnPacket(const QAddr: word; var QBuffer: TDistoXData8BytesBuffer): boolean;
var
   reply_addr, QLastSerialError: Integer;
   QTypeData, QOp: byte;
begin
  Result   := false;
  if (not self.Active) then Exit;
  QBuffer := MakeBufferVierge();
  SendReadCommandAtAddress(QBuffer, DISTOX_COMMAND_READ_MEMORY_AT_ADDRESS, QAddr);
  if (PROTO_OK = ReadBuffer8bytes(QBuffer, QLastSerialError, QTypeData, QOp)) then
  begin
    if (QBuffer[0] = DISTOX_COMMAND_READ_MEMORY_AT_ADDRESS) then
    begin
      // contrôle de l'adresse lue
      reply_addr := (QBuffer[2] shl 8) OR QBuffer[1];
      Result := (reply_addr = QAddr);
    end;
  end;
  //Application.ProcessMessages();
end;
// lecture d'un paquet (retourne 4 octets)
function TPilotageDistoX.BootloaderReadAnPacket264(const QAddr: word; var QBuffer: TBootloaderModeDataBuffer): boolean;
var
  Nb: Integer;
  reply_addr: Integer;
begin
  Result   := false;
  if (not self.Active) then Exit;
  QBuffer := MakeBootloaderBufferVierge();
  AfficherMessage(Format('BootloaderReadAnPacket264 : Adresse: %X: Commande: %X', [QAddr, DISTOX_COMMAND_BOOTLOADER_READ_DATA_AT_ADDRESS]));
  BootloaderSendReadCommandAtAddress(QBuffer, DISTOX_COMMAND_BOOTLOADER_READ_DATA_AT_ADDRESS, QAddr);
  AfficherMessage('--- 00');
  Nb := BootloaderReadBuffer264bytes(QBuffer, 264);
  AfficherMessage(Format('--- 01: %d bytes read',  [Nb]));
  if (Nb > 0) then
  begin
    if (QBuffer[0] = DISTOX_COMMAND_BOOTLOADER_READ_DATA_AT_ADDRESS) then
    begin
      // contrôle de l'adresse lue
      reply_addr := (QBuffer[2] shl 8) OR QBuffer[1];
      Result := (reply_addr = QAddr);
      AfficherMessage(Format('BootloaderReadAnPacket264 : Adresse: %X (%X): Commande: %X - Taille: %d', [QAddr, reply_addr, DISTOX_COMMAND_BOOTLOADER_READ_DATA_AT_ADDRESS, Nb]));
    end;
  end;
  Application.ProcessMessages();

end;
// TODO: A tester
function TPilotageDistoX.ReadBuffer8bytes(var MyBuffer: TDistoXData8BytesBuffer;
                                          out LastSerialError: integer;
                                          out QTypeData: byte;
                                          out QOp      : byte): TProtoSerialError;
var
  WU, Nb, n: Integer;
begin
  result := PROTO_OK;
  Nb := Max(1 + High(MyBuffer), 8);
  if (not self.Active) then Exit;
  // perte de connexion
  if (not self.SynSer.DSR) then
  begin
    AfficherMessageErreur('*** Connexion lost');
    exit;
  end;
  n := self.SynSer.RecvBuffer(@MyBuffer, Nb); //
  WU := self.SynSer.LastError;
  QTypeData := MyBuffer[0];
  QOp       := (QTypeData and $3F);
  // acquitter bonne réception et vider la première visée de la file du DistoX
  Result := Acknowledge(QTypeData);
end;

function TPilotageDistoX.BootloaderReadBuffer264bytes(var MyBuffer: TBootloaderModeDataBuffer; const Nb: integer): integer;
var
  WU: Integer;
begin
  result := -1;
  if (not self.Active) then Exit;
  Result := self.SynSer.RecvBuffer(@MyBuffer, Nb);
  WU := self.SynSer.LastError;
  AfficherMessageErreur(self.SynSer.GetErrorDesc(WU));
end;

//******************************************************************************
// lecture de la zone de données

function TPilotageDistoX.ReadDataSection(const NumBlockDebut, NumBlockFin: integer): integer;
const
  NB_BLOCKS   = 19;
  NB_SEGMENTS = 56;
var
  AdresseDataZone: Integer;
  QNumBlock: Integer;
  NbDataToRead: Integer;
  NbDataAlreadyRead: Integer;
  NbStart: Integer;
  procedure LireUneVisee(const QBlock, QSegment: integer; const QAddressDebutVisee: word);
  var
    MyTrame  : TDistoXData8BytesBuffer;
    MyBuffer1: TDistoXData8BytesBuffer;
    MyBuffer2: TDistoXData8BytesBuffer;
    i: Integer;
    EWE: String;
    myMesureVisee: TMesureViseeDistoX;
  begin
    // on boucle tant que la lecture des paquets n'a pas réussi
    while (not ReadAnPacket(QAddressDebutVisee, MyBuffer1)) do AfficherMessageErreur(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
      ;
    while (not ReadAnPacket(QAddressDebutVisee + 4, MyBuffer2)) do AfficherMessageErreur(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
      ;
    // construction des trames
    for i := 0 to 3 do MyTrame[i]   := MyBuffer1[3+i];
    for i := 0 to 3 do MyTrame[i+4] := MyBuffer2[3+i];
    EWE := MakeContentBuffer(MyTrame);
    // construction de la visée
    myMesureVisee := MakeTMesureViseeDistoXFromDataBuffer(QBlock, QSegment,MyTrame);
    AfficherMessage(Format('%d: %d:%d: L = %.3f m; Az = %.2f; P = %.2f - EWE = %s',
                                     [0,
                                      myMesureVisee.Block, myMesureVisee.Segment,
                                      myMesureVisee.Longueur,
                                      myMesureVisee.Azimut,
                                      myMesureVisee.Pente,
                                      myMesureVisee.HexaData
                                      ]));
    AddMesureVisee(myMesureVisee, false);


  end;
  // Un segment contient 18 octets mais seuls les 8 premiers contiennent une visée
  procedure LireUnSegment(const AddressBlock: word; const NumeroBlock, NumeroSegment: integer);
  var
    MyAddressSegment: Integer;
  begin
    MyAddressSegment := AddressBlock + 18 * NumeroSegment;
    // lecture de la visée
    LireUneVisee(NumeroBlock, NumeroSegment, MyAddressSegment);
    // le reste du segment est ignoré
    Application.ProcessMessages;

  end;
  // Un block a une taille de 1024 octets
  procedure LireUnBlock1024(const NumeroBlock: integer);
  var
    QNumSegment: Integer;
    T0, T1, dT: TDateTime;
    WU : String;
    EWE: String;
  begin
    for QNumSegment := 0 to NB_SEGMENTS - 1 do
    begin
      T0 := Now();
      LireUnSegment(1024 * NumeroBlock, NumeroBlock, QNumSegment);
      T1 := Now();
      dT := T1 - T0;
      if (Assigned(FProcAfficherProgression)) then
      begin
        // estimation du temps restant
        EWE := TimeToStr(dT * (NbDataToRead - NbDataAlreadyRead));
        WU  := Format('Block %d/%d  - Segment: %d/%d - Temps restant: %s', [NumeroBlock + 1, NB_BLOCKS,
                                                                            QNumSegment + 1, NB_SEGMENTS,
                                                                            EWE]);
        FProcAfficherProgression(WU, NbStart, NbDataToRead, NbDataAlreadyRead);
        NbDataAlreadyRead += 1;
      end;
    end;
  end;
begin
  Result := -1;
  if (not self.Active) then Exit;
  AdresseDataZone := $0;
  NbStart := 0;
  NbDataAlreadyRead := 0;
  NbDataToRead := (1 + NumBlockFin - NumBlockDebut) * NB_SEGMENTS;
  Application.ProcessMessages;
  AfficherMessage(Format('%s.ReadMemoryArea(%X)', [ClassName, AdresseDataZone]));
  try
    for QNumBlock := NumBlockDebut to NumBlockFin do //NB_BLOCKS - 1 do
    begin
      AfficherMessage('-----------------------------------------------');
      LireUnBlock1024(QNumBlock);
    end;
  except
    ; //AfficherMessageErreur('*** Transfert arrêté par l''utilisateur');
  end;
end;
//******************************************************************************
//******************************************************************************
//* SECTIONS COMMUNES A TOUTES LES VERSIONS                                    *
//******************************************************************************


// GESTION DES BUFFERS
//******************************************************************************
function TPilotageDistoX.MakeContentBuffer(const B: TDistoXData8BytesBuffer): string;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to High(B) do Result += Format('%.2X', [B[i]]);
end;
function TPilotageDistoX.MakeBufferVierge(): TDistoXData8BytesBuffer;
var
  q: Integer;
begin
  for q := 0 to High(Result) do Result[q] := $0;
end;
function TPilotageDistoX.MakeBootloaderBufferVierge(): TBootloaderModeDataBuffer;
var
  q: Integer;
begin
  for q := 0 to High(Result) do Result[q] := $0;
end;

// INDICATEUR DE PROGRESSION - STATUT
//******************************************************************************
procedure TPilotageDistoX.SetProcProgression(const P: TProcDisplayProgression);
begin
  FProcAfficherProgression := P;
end;

procedure TPilotageDistoX.SetProcTransmitViseeToTableau(const P: TProcTransmitMesureVisee);
begin
  FProcTransmitViseeToTableau := P;
end;

procedure TPilotageDistoX.SetProcTransmitViseeToGHTopoCtxt2D(const P: TProcTransmitMesureVisee);
begin
  FProcTransmitViseeToGHTopoCtxt2D := P;
end;

procedure TPilotageDistoX.SetLabelActionsProcessing(const L: TStaticText);
begin
  FLabelActionsProcessing := L;
end;



function TPilotageDistoX.GetDataGMFromGrid(const grd: TStringGrid): boolean;
var
  l, c, NbC, NbL: Integer;
  EWE: String;
begin
  Result := false;
  AfficherMessage(Format('%s.GetDataGMFromGrid()', [ClassName]));
  ViderListesMesuresGM;
  NbC := grd.ColCount;
  NbL := grd.RowCount;
  for l := 1 to NbL - 1 do
  begin
    EWE := '';
    for c := 1 to NbC - 1 do EWE += Trim(grd.Cells[c, l]);
    if (Length(EWE) < 6) then Break;
    AddMesuresGMByValues(ConvertirEnNombreReel(Trim(grd.Cells[1, l]), 0.00),
                         ConvertirEnNombreReel(Trim(grd.Cells[2, l]), 0.00),
                         ConvertirEnNombreReel(Trim(grd.Cells[3, l]), 0.00),
                         ConvertirEnNombreReel(Trim(grd.Cells[4, l]), 0.00),
                         ConvertirEnNombreReel(Trim(grd.Cells[5, l]), 0.00),
                         ConvertirEnNombreReel(Trim(grd.Cells[6, l]), 0.00)
                         );
  end;
  CalculerCalibration();
end;

function TPilotageDistoX.SetDataGMInGrid(const grd: TStringGrid): boolean;
  procedure SetHeaderGrdGM;
  begin
    grd.Cells[1, 0] := 'Gx';
    grd.Cells[2, 0] := 'Gy';
    grd.Cells[3, 0] := 'Gz';
    grd.Cells[4, 0] := 'Mx';
    grd.Cells[5, 0] := 'My';
    grd.Cells[6, 0] := 'Mz';
    grd.Cells[7, 0] := '|G|';
    grd.Cells[8, 0] := '|M|';
    grd.Cells[9, 0] := '/a\';
  end;
var
  i, NbG, NbM: Integer;
  mG, mM : TPoint3Df;
begin
  Result := false;
  NbG := self.GetNbMesuresG;
  NbM := self.GetNbMesuresM;
  AfficherMessage(Format('%s.SetDataGMInGrid(%d G, %d M)', [ClassName, NbG, NbM]));
  SetHeaderGrdGM;

  for i := 0 to NbG - 1 do
  begin
    mG  := GetMesureG(i);
    grd.Cells[1, i+1] := Format('%d', [Trunc(mG.X * FV)]);
    grd.Cells[2, i+1] := Format('%d', [Trunc(mG.Y * FV)]);
    grd.Cells[3, i+1] := Format('%d', [Trunc(mG.Z * FV)]);
  end;
  for i := 0 to NbM - 1 do
  begin
    mM  := GetMesureM(i);
    grd.Cells[4, i+1] := Format('%d', [Trunc(mM.X * FV)]);
    grd.Cells[5, i+1] := Format('%d', [Trunc(mM.Y * FV)]);
    grd.Cells[6, i+1] := Format('%d', [Trunc(mM.Z * FV)]);
  end;
end;

// PARTIES FACTORISABLES
//******************************************************************************
procedure TPilotageDistoX.SendReadCommandAtAddress(var MyBuffer: TDistoXData8BytesBuffer;const QCmd: word; const QAddress: word);
begin
  MyBuffer[0] :=  QCmd;                        //DISTOX_COMMAND_READ_DATA = 56; // 00111000
  MyBuffer[1] :=  QAddress and $FF;           //  Address & 0xFF
  MyBuffer[2] := (QAddress shr 8) and $FF;    // (Address >> 8) & 0xFF
  self.WriteBuffer(MyBuffer, 3); //1 + High(MyBuffer));  // WriteBuffer() command
end;

procedure TPilotageDistoX.EvenementSerialRxData(Sender: TObject);
begin
  case FModeAttenteDonneesRX of
    mrxNONE            : ;
    mrxDEPILAGE_MESURE : self.LireEtDepilerTrameDe8OctetsData();
  end;
end;



procedure TPilotageDistoX.EvenementSerialStatus(Sender: TObject; Reason: THookSerialReason; const Value: string);
begin
  AfficherMessage(Self.SynSer.Device);
  begin
    case Reason of
      HR_SerialClose : AfficherMessage('Port ' + Value + ' closed');
      HR_Connect     : AfficherMessage('Port ' + Value + ' connected');
      //HR_CanRead     : AfficherMessage('CanRead : ' + Value);
      //HR_CanWrite    : AfficherMessage('CanWrite : ' + Value);
      //HR_ReadCount   : AfficherMessage('ReadCount : ' + Value);
      //HR_WriteCount  : AfficherMessage('WriteCount : ' + Value);
      HR_Wait        : AfficherMessage('Wait : ' + Value);

    end ;
  end;
end;

function TPilotageDistoX.DescribeProtoSerialError(const E: TProtoSerialError): string;
begin
  case E of
    PROTO_OK       : Result := 'OK';
    PROTO_READ     : Result := 'read error';
    PROTO_WRITE    : Result := 'write error';
    PROTO_COMMAND  : Result := 'command error';
    PROTO_ADDR     : Result := 'address error';
    PROTO_PACKET   : Result := 'packet error';
    PROTO_CONNECT  : Result := 'connection error';
    PROTO_TIMEOUT  : Result := 'i/o timeout';
  else
    Result := 'illegal error code';
  end;
end;



procedure TPilotageDistoX.BootloaderSendReadCommandAtAddress(var MyBuffer: TBootloaderModeDataBuffer;const QCmd: word; const QAddress: word);
begin
  MyBuffer[0] :=  QCmd;
  MyBuffer[1] :=  QAddress and $FF;           //  Address & 0xFF
  MyBuffer[2] := (QAddress shr 8) and $FF;    // (Address >> 8) & 0xFF
  self.WriteBuffer(MyBuffer, 3); //1 + High(MyBuffer));  // WriteBuffer() command
end;

// interprétation des trames
function TPilotageDistoX.MakeTMesureViseeDistoXFromDataBuffer(const QBlock, QSegment: integer;
                                                              const MyBuffer: TDistoXData8BytesBuffer): TMesureViseeDistoX;
begin
  result := MakeTMesureViseeDistoXFromDataBuffer(MyBuffer);
  Result.Block   := QBlock;
  Result.Segment := QSegment;
end;

function TPilotageDistoX.MakeTMesureViseeDistoXFromDataBuffer(const MyBuffer: TDistoXData8BytesBuffer): TMesureViseeDistoX;
var
  QType, QX, QY, QZ: Integer;
  i: Integer;
  WU: String;
  Dist: Integer;
  P: Double;
begin
  Result.TypeMesure := tmdxUNKNOWN;
  Result.DistoXSerialNumber := FDistoXSerialNumber;
  Result.Block      := 0;
  Result.Segment    := 0;
  Result.Longueur := 0.00;
  Result.Azimut   := 0.00;
  Result.Pente    := 0.00;
  Result.IsMarked := false;
  Result.TimeStamp:= Now();
  WU := '';
  for i := 0 to High(MyBuffer) do WU := WU + Format('%X', [MyBuffer[i]]);
  Result.HexaData := WU;
  QType := MyBuffer[0];
  QX := MyBuffer[1] + (MyBuffer[2] shl 8);
  QY := MyBuffer[3] + (MyBuffer[4] shl 8);
  QZ := MyBuffer[5] + (MyBuffer[6] shl 8);
  Result.TypeMesure := tmdxUNKNOWN;
  // distances exprimées en mm si < 100 m, sinon en cm
  Dist := QX + ((QType and $40) shr 10);     // OK
  Result.Longueur := Dist * 0.001; // TODO: Cas où Dist > 100 m
  Result.Azimut   := radtodeg(QY * HEX_TO_RAD);                   // OK
  P  := radtodeg(QZ * HEX_TO_RAD);
  //if (Not InRange(P, -0.001, 90.001)) then P := P - 360.00;
  if (Not InRange(P, -0.001, 0.25 * FUniteDuDistoX)) then P := P - FUniteDuDistoX;

  Result.Pente := P;
end;

procedure TPilotageDistoX.SetReadingTopoData(const B: boolean);
begin
  if (B) then FModeAttenteDonneesRX := mrxDEPILAGE_MESURE
         else FModeAttenteDonneesRX := mrxNONE;
end;

function TPilotageDistoX.IsAcquiringTopoData(): boolean;
begin
  result := (FModeAttenteDonneesRX = mrxDEPILAGE_MESURE);
end;

function TPilotageDistoX.IsLaserON(): boolean;
begin
  Result := FLaserON;
end;

function TPilotageDistoX.IsCalibrationON(): boolean;
begin
  Result := FCalibrationModeON;
end;
(*
function TPilotageDistoX.IsBeepOn(): boolean;
begin
  Result := FBeepON;
end;
//*)

procedure TPilotageDistoX.SetLazSerialDevice(const D: string);
begin
  self.Device := D;
end;


procedure TPilotageDistoX.SetNonLinearCalibration(const o: boolean);
begin
  FDoNonLinearCalibration := o;
end;

// GESTION DES MESURES DE VISEES
//******************************************************************************

procedure TPilotageDistoX.AddMesureVisee(const V: TMesureViseeDistoX; const DoRegisterInLogs: boolean);
begin
  FListeMesuresTopo.AddElement(V);
  if (DoRegisterInLogs) then EcrireMesureDansJournaux(V);
end;

procedure TPilotageDistoX.RemoveMesureVisee(const Idx: integer);
begin
  FListeMesuresTopo.RemoveElement(Idx);
end;

procedure TPilotageDistoX.RemoveLastMesureVisee;
var
  Nb: Integer;
begin
  Nb := FListeMesuresTopo.GetNbElements();
  RemoveMesureVisee(Nb - 1);
end;

procedure TPilotageDistoX.SetNbMesuresPourVisees(const n: integer);
begin
  FNbMesuresPourViseeCheminement := n;
end;

function TPilotageDistoX.GetMesureVisee(const Idx: integer): TMesureViseeDistoX;
begin
  Result := FListeMesuresTopo.GetElement(Idx);
end;

function TPilotageDistoX.GetModeAttenteDonneesRX(): TModeAttenteDonneesRX;
begin
  Result := FModeAttenteDonneesRX;
end;
procedure TPilotageDistoX.SetModeAttenteDonneesRX(const Q: TModeAttenteDonneesRX);
begin
  FModeAttenteDonneesRX := Q;
end;


function TPilotageDistoX.GetLastMesureVisee(): TMesureViseeDistoX;
var
  Nb: Integer;
begin
  Nb := FListeMesuresTopo.GetNbElements();
  Result := GetMesureVisee(Nb - 1);
end;

procedure TPilotageDistoX.PutMesureVisee(const Idx: integer; const V: TMesureViseeDistoX);
begin
  FListeMesuresTopo.PutElement(Idx, V);
end;

function TPilotageDistoX.GetNbMesuresAcquises(): integer;
begin
  Result := FListeMesuresTopo.GetNbElements();
end;



procedure TPilotageDistoX.ViderListeMesuresTopo(const DoAddViseeBidon: boolean);
begin
  AfficherMessage(Format('%s.ViderListeMesuresTopo',[self.ClassName]));
  FListeMesuresTopo.ClearListe();
  // et on ajoute une mesure bidon
  if (DoAddViseeBidon) then self.AddMesureVisee(getEmptyMesure(tmdxUNKNOWN), false);
end;

// LOAD-SAVE DONNEES BRUTES
//******************************************************************************
procedure TPilotageDistoX.ChargerDonneesViseesBrutes(const Filename: string);
var
  LS: TStringList;
  i, Nb: Integer;
  EWE: TGHStringArray;
  myMesure : TMesureViseeDistoX;
begin
  //0	26/11/15 09:30	0.000	0.000	0.000
  //1	26/11/15 09:31	1.398	260.513	6.229	01760541B96E0425

  AfficherMessage(Format('%s.ChargerDonneesBrutes: %s',[self.ClassName, Filename]));
  LS := TStringList.Create;
  try
    LS.Clear;
    LS.LoadFromFile(Filename);

    Nb := LS.Count;
    // 1	3.114	329.529	32.382		012A0C55EA0717F9
    // on ajoute trois visées de signalisation en tête
    myMesure := getEmptyMesure(tmdxSIGNALISATION);
    for i := 0 to 2 do self.AddMesureVisee(myMesure, false);
    for i := 0 to Nb - 1 do
    begin
      try
        EWE := split(LS.Strings[i], #9);
        myMesure.TimeStamp  := Now();
        myMesure.TypeMesure := tmdxUNKNOWN;
        myMesure.Longueur   := ConvertirEnNombreReel(Trim(EWE[2]), 0.00);
        myMesure.Azimut     := ConvertirEnNombreReel(Trim(EWE[3]), 0.00);
        myMesure.Pente      := ConvertirEnNombreReel(Trim(EWE[4]), 0.00);
        myMesure.IsMarked   := false;
        myMesure.HexaData   := Trim(EWE[5]);
        //AfficherMessage(Format('%d: %.3f, %.3f, %.3f', [i, myMesure.Longueur, myMesure.Azimut, myMesure.Pente]));
        // Visées non nulles uniquement
        if (myMesure.Longueur > 0.0005) then self.AddMesureVisee(myMesure, false);
      except
      end;
    end;
  finally
    LS.Free;
  end;
end;

procedure TPilotageDistoX.SauverDonneesViseesBrutes(const Filename: string);
const
  QFMT = '%s' + #9 + '%.3f' + #9 + '%.3f' + #9 + '%.3f' + #9 + '%s' + #9 + '%s' + #9 + '%s';
var
  fp: TextFile;
  myMesure: TMesureViseeDistoX;
  i, Nb: Integer;
begin
  AfficherMessage(Format('%s.SauverDonneesBrutes: %s',[self.ClassName, Filename]));
  AssignFile(fp, Filename);
  Nb := self.GetNbMesuresAcquises();
  try
    ReWrite(fp);
    WriteLn(fp, 'Type' + #9 + 'Long' + #9 + 'Azimut' + #9 + 'Incl' + #9 + 'Marked' + #9 + 'Trame' + #9 + 'TimeStamp');
    for i := 0 to Nb - 1 do
    begin
      myMesure := GetMesureVisee(i);
      WriteLn(fp, Format(QFMT,
                        [getLitteralTypeMesureDistoX(myMesure.TypeMesure),
                         myMesure.Longueur, myMesure.Azimut, myMesure.Pente,
                         BoolToStr(myMesure.IsMarked, 'O', 'N'),
                         myMesure.HexaData,
                         DatePascalToDateCondensee(myMesure.TimeStamp)
                        ]));
    end;

  finally
    CloseFile(fp);
  end;
end;

// GESTION DE LA CALIBRATION
//******************************************************************************
procedure TPilotageDistoX.SetHeaderGrdGM;
begin
  FGridDataGMDistoX.Cells[1, 0] := 'Gx';
  FGridDataGMDistoX.Cells[2, 0] := 'Gy';
  FGridDataGMDistoX.Cells[3, 0] := 'Gz';
  FGridDataGMDistoX.Cells[4, 0] := 'Mx';
  FGridDataGMDistoX.Cells[5, 0] := 'My';
  FGridDataGMDistoX.Cells[6, 0] := 'Mz';
  FGridDataGMDistoX.Cells[7, 0] := '|G|';
  FGridDataGMDistoX.Cells[8, 0] := '|y|';
  FGridDataGMDistoX.Cells[9, 0] := '/a\';
end;

function TPilotageDistoX.getEmptyMesure(const QTypeMesure: TTypeMesureDistoX): TMesureViseeDistoX;
begin
  Result.TypeMesure := QTypeMesure;
  Result.DistoXSerialNumber := 0;
  Result.Block      := 0;
  Result.Segment    := 0;
  Result.Longueur := 0.00;
  Result.Azimut   := 0.00;
  Result.Pente    := 0.00;
  Result.IsMarked := false;
  Result.TimeStamp:= Now();
  Result.HexaData := '';
end;


procedure TPilotageDistoX.ViderListesMesuresGM();
var
  i, Nb: Integer;
begin
  try
    FListeMesuresDeG.ClearListe();
    FListeMesuresDeM.ClearListe();
  finally
  end;
end;
function TPilotageDistoX.IsReadyForCalibrate: boolean;
var
  NbG, NbM: Integer;
begin
  NbG := self.GetNbMesuresG;
  NbM := self.GetNbMesuresM;
  Result := (NbG = 56) and (NbM = 56);
end;
function TPilotageDistoX.GetMesureM(const Idx: integer): TPoint3Df;
begin
  Result := FListeMesuresDeM.GetElement(Idx);
end;

function TPilotageDistoX.GetNbMesuresG(): integer;
begin
  Result := FListeMesuresDeG.GetNbElements();
end;

function TPilotageDistoX.GetNbMesuresM(): integer;
begin
  Result := FListeMesuresDeM.GetNbElements();
end;
function TPilotageDistoX.GetMesureG(const Idx: integer): TPoint3Df;
begin
  Result := FListeMesuresDeG.GetElement(Idx);
end;
procedure TPilotageDistoX.AddMesureG(const MG: TPoint3Df);
begin
  FListeMesuresDeG.AddElement(MG);
end;

procedure TPilotageDistoX.AddMesureM(const MG: TPoint3Df);
begin
  FListeMesuresDeM.AddElement(MG);
end;

procedure TPilotageDistoX.AddMesuresGMByValues(const gx, gy, gz, mx, my, mz: double);
var
  M, G: TPoint3Df;
begin
  M.X := mx / FV;
  M.Y := my / FV;
  M.Z := mz / FV;
  G.X := gx / FV;
  G.Y := gy / FV;
  G.Z := gz / FV;
  AddMesureG(G);
  AddMesureM(M);
end;
// jeu de données exemple
procedure TPilotageDistoX.setJeuDeDonneesExemple();
var
  i: integer;
  G: TPoint3Df;
  M: TPoint3Df;
begin
  AfficherMessage(Format('%s.setJeuDeDonneesExemple()', [ClassName]));
  //AddMesuresMGByValues(  Gx,  Gy,   Gz,   Mx,   My,   Mz); // Azim,  Incl,  Roll,  |G|,  |M|,   a,   d
  AddMesuresGMByValues(3,109, 14158, 10445, -7506, 18917); //   36.2,    -1.8,    -4.5,   1.001,   1.002,    26.8,    0.28
  AddMesuresGMByValues(169,14774, 105, 10475, 20752, 5286); //   36.3,    -1.9,    92.1,   0.998,   1.002,    26.7,    0.34
  AddMesuresGMByValues(47,2512, -12877, 9405, 8914, -20075); //   36.0,    -1.8,   174.6,   1.001,   1.000,    26.7,    0.09
  AddMesuresGMByValues(-88,-12335, -791, 9275, -18631, -9399); //   35.9,    -1.9,   -96.1,   0.999,   1.003,    26.6,    0.34
  AddMesuresGMByValues(-862,1478, 14217, -6522, 6721, 19965); //  216.2,     1.9,     1.3,   1.002,   1.001,    26.7,    0.34
  AddMesuresGMByValues(-705,14790, 788, -7153, 20965, -5638); //  215.8,     1.8,    89.2,   0.998,   1.000,    26.8,    0.26
  AddMesuresGMByValues(-826,998, -12943, -8110, -5143, -20801); //  215.7,     1.8,  -179.0,   1.001,   1.000,    26.6,    0.21
  AddMesuresGMByValues(-966,-12411, 394, -7530, -19607, 4711); //  215.9,     1.8,   -91.1,   1.000,   1.000,    26.7,    0.12
  AddMesuresGMByValues(-2269,1578, 14051, -6838, -7320, 19180); //  127.5,     7.9,     1.7,   0.999,   1.002,    26.7,    0.19
  AddMesuresGMByValues(-2356,-12192, 2108, -7947, -19299, -6990); //  127.8,     7.8,   -83.8,   0.998,   1.003,    26.7,    0.39
  AddMesuresGMByValues(-2283,672, -12772, -7864, 8468, -20059); //  127.2,     8.0,  -177.6,   0.999,   1.001,    26.7,    0.24
  AddMesuresGMByValues(-2151,14674, 106, -6762, 20132, 7609); //  127.6,     7.9,    92.1,   0.998,   1.001,    26.7,    0.26
  AddMesuresGMByValues(1416,1979, 14067, 10762, 9360, 18618); //  308.0,    -7.7,     3.5,   1.002,   1.001,    26.7,    0.19
  AddMesuresGMByValues(1529,14658, -233, 9945, 19543, -9505); //  307.8,    -7.7,    93.6,   1.000,   0.999,    26.7,    0.19
  AddMesuresGMByValues(1426,132, -12782, 9153, -8529, -19752); //  308.1,    -7.8,  -175.4,   1.001,   0.999,    26.6,    0.25
  AddMesuresGMByValues(1301,-12242, 1929, 9910, -18115, 8792); //  307.9,    -7.8,   -84.6,   0.999,   0.999,    26.6,    0.20
  AddMesuresGMByValues(-13966,734, 935, -18632, 10043, -1414); //  196.8,    87.3,   -62.7,   1.001,   1.001,    26.8,    0.15
  AddMesuresGMByValues(-13965,1711, 1054, -19057, -1034, -9666); //  209.1,    87.6,    43.9,   1.001,   1.001,    26.7,    0.15
  AddMesuresGMByValues(-13961,1811, -78, -18921, -8598, 35); //  226.1,    86.3,   145.2,   1.002,   1.002,    26.7,    0.25
  AddMesuresGMByValues(-13932,703, 144, -18427, 2085, 9096); //  216.8,    86.7,  -129.4,   0.999,   1.000,    26.7,    0.14
  AddMesuresGMByValues(13076,1335, -430, 21856, 3865, 7470); //   31.3,   -85.4,   166.9,   0.998,   1.000,    26.7,    0.23
  AddMesuresGMByValues(13016,2738, 685, 21897, -6662, 2658); //   22.5,   -83.0,    87.7,   0.997,   1.001,    26.7,    0.35
  AddMesuresGMByValues(13050,1302, 2094, 21571, -2582, -8377); //   11.9,   -83.7,     8.4,   0.999,   1.002,    26.8,    0.23
  AddMesuresGMByValues(13034,-196, 716, 21736, 8068, -4382); //   18.1,   -84.6,   -86.0,   0.997,   1.000,    26.7,    0.30
  AddMesuresGMByValues(-9490,1217, 10687, -13615, 10039, 13038); //  251.1,    42.1,    -0.2,   0.998,   0.999,    26.6,    0.21
  AddMesuresGMByValues(-9412,11414, 922, -14300, 14112, -9325); //  250.9,    42.0,    88.3,   1.002,   1.000,    26.7,    0.21
  AddMesuresGMByValues(-9502,1287, -9425, -14847, -8669, -13613); //  251.3,    42.1,  -180.0,   1.000,   1.001,    26.7,    0.10
  AddMesuresGMByValues(-9513,-8900, 708, -14152, -12330, 9147); //  250.3,    41.7,   -89.7,   1.000,   1.000,    26.7,    0.06
  AddMesuresGMByValues(7913,9350, 7639, 11143, 20569, 4494); //  250.6,   -37.4,    49.4,   1.002,   1.000,    26.7,    0.22
  AddMesuresGMByValues(7951,8031, -7745, 9936, 5291, -21004); //  249.5,   -37.5,   140.6,   1.005,   1.000,    26.5,    0.52
  AddMesuresGMByValues(7699,-6757, -6746, 9813, -19233, -6981); //  249.1,   -37.2,  -133.2,   0.998,   1.000,    26.5,    0.30
  AddMesuresGMByValues(7645,-6359, 8576, 10708, -5566, 19444); //  248.7,   -36.8,   -43.2,   1.001,   0.999,    26.8,    0.22
  AddMesuresGMByValues(-9092,758, 11042, -7062, -9533, 17966); //   60.4,    39.8,    -2.7,   0.999,   1.000,    26.8,    0.15
  AddMesuresGMByValues(-9105,11649, 1085, -7045, 18771, 9955); //   60.6,    40.4,    87.4,   1.001,   0.998,    26.7,    0.19
  AddMesuresGMByValues(-9095,1406, -9744, -8019, 10262, -19061); //   60.6,    40.0,   179.4,   0.998,   0.999,    26.8,    0.23
  AddMesuresGMByValues(-9092,-9260, 300, -8056, -17696, -10533); //   60.4,    39.3,   -91.9,   1.000,   0.999,    26.6,    0.14
  AddMesuresGMByValues(7456,1015, 11630, 15208, -9738, 13998); //   74.1,   -35.6,    -0.5,   0.998,   0.999,    26.6,    0.28
  AddMesuresGMByValues(7445,12368, 613, 15181, 15466, 9639); //   74.8,   -34.8,    90.0,   1.003,   0.997,    26.6,    0.45
  AddMesuresGMByValues(7362,465, -10491, 14283, 9771, -16343); //   74.6,   -34.9,  -176.5,   1.002,   0.998,    26.7,    0.28
  AddMesuresGMByValues(7240,-10092, 1614, 14085, -15201, -9746); //   74.1,   -34.7,   -85.1,   1.005,   0.999,    26.7,    0.52
  AddMesuresGMByValues(-8871,8519, 8273, -18098, 5385, 8559); //  166.5,    39.0,    43.4,   0.999,   0.998,    26.7,    0.25
  AddMesuresGMByValues(-8849,8836, -6856, -18468, 9476, -5228); //  166.4,    38.8,   134.8,   1.003,   1.001,    26.5,    0.39
  AddMesuresGMByValues(-8840,-7572, -5334, -18781, -5812, -8079); //  166.2,    38.1,  -124.2,   0.998,   1.001,    26.8,    0.25
  AddMesuresGMByValues(-8816,-6548, 7995, -18351, -8290, 4604); //  165.9,    38.0,   -46.6,   0.999,   1.001,    26.7,    0.17
  AddMesuresGMByValues(8749,1689, 10586, 8788, -3017, 20940); //  154.5,   -42.6,     3.4,   0.999,   1.000,    26.3,    0.45
  AddMesuresGMByValues(8630,11393, -474, 7842, 22272, 2084); //  154.9,   -41.1,    96.0,   1.005,   0.998,    26.9,    0.58
  AddMesuresGMByValues(8315,-310, -9598, 6439, 2280, -22733); //  155.3,   -40.2,  -171.9,   0.997,   0.999,    26.6,    0.31
  AddMesuresGMByValues(8324,-9072, 2251, 6894, -21223, -2419); //  153.5,   -40.7,   -81.0,   0.999,   0.999,    26.8,    0.22
  AddMesuresGMByValues(-8692,1264, 11355, -2286, 2618, 22065); //  346.3,    37.7,     0.1,   0.999,   1.001,    26.8,    0.23
  AddMesuresGMByValues(-8663,11976, 542, -2841, 22873, -2412); //  346.7,    38.1,    90.4,   0.998,   1.000,    26.9,    0.32
  AddMesuresGMByValues(-8740,923, -10032, -3863, -1974, -22865); //  346.6,    38.0,  -178.1,   0.999,   0.998,    27.1,    0.55
  AddMesuresGMByValues(-8671,-9581, 1375, -3135, -21434, 2871); //  346.3,    37.1,   -86.2,   0.999,   0.999,    26.8,    0.13
  AddMesuresGMByValues(7555,619, 11593, 21153, -245, 9692); //    0.2,   -36.0,    -2.6,   1.000,   0.999,    26.8,    0.10
  AddMesuresGMByValues(7642,12197, 1101, 20946, 10980, -11); //    0.2,   -35.8,    87.4,   1.003,   0.998,    26.6,    0.32
  AddMesuresGMByValues(7578,581, -10323, 20446, 224, -11374); //    0.2,   -36.1,  -177.0,   1.000,   0.999,    26.6,    0.10
  AddMesuresGMByValues(7552,-9887, 596, 20676, -9827, -1094); //  359.7,   -36.3,   -90.3,   1.004,   1.000,    26.7,    0.36
  afficherMessage('   Gx     Gy     Gz     Mx     My     Mz    Azim   Incl   Roll   |G|    |M|     a      d');
  for i := 0 to GetNbMesuresG - 1 do
  begin
    G := self.GetMesureG(i);
    M := self.GetMesureM(i);
    AfficherMessage(Format('%d: %.6f, %.6f, %.6f, , %.6f, %.6f, %.6f', [i, G.X * FV, G.Y * FV, G.Z * FV, M.X * FV, M.Y * FV, M.Z * FV]));
  end;
end;

//******************************************************************************
//* FONCTIONS DE CALCUL COMMUNES TOUTES VERSIONS                               *
//******************************************************************************
function TPilotageDistoX.GetMatrix_aG(): TMatrix3x3;
begin
  result := fMatrix_aG;
end;
function TPilotageDistoX.GetMatrix_aM(): TMatrix3x3;
begin
  result := fMatrix_aM;
end;

function TPilotageDistoX.GetVecteur_bG(): TPoint3Df;
begin
  result := fVecteur_bG;
end;

function TPilotageDistoX.GetVecteur_bM(): TPoint3Df;
begin
  result := fVecteur_bM;
end;

//******************************************************************************


function TPilotageDistoX.GetDelta(): double;
begin
  result := fDelta;
end;

function TPilotageDistoX.GetNbIters(): integer;
begin
  result := fNbIters;
end;

procedure TPilotageDistoX.SetCurrentNumSerie(const s: TNumeroSerie);
begin
  FCurrentSerie := S;
end;

function TPilotageDistoX.GetCurrentNumSerie(): TNumeroSerie;
begin
  result := FCurrentSerie;
end;



procedure TPilotageDistoX.OptVectors(const gr, mr: TPoint3Df; const alpha: double; out gx, mx: TPoint3Df);
var
  no: TPoint3Df;
  s, c: double;
  s1, s2, s3, s4: TPoint3Df;
begin
  no := produitVectoriel(gr, mr, true); //Vector no = Vector.Normalized(gr % mr);  // plane normal
  sincos(alpha, s, c);
  s1 := MultVectorByScalar(mr, c); // mr * c
  s2 := MultVectorByScalar(produitVectoriel(mr, no, false), s);// (mr % no) * s
  s4 := AddVecteurs([s1, s2, gr]);
  gx := NormaliseVecteur(s4);
  // mx = gx * c + (no % gx) * s;
  s1 := MultVectorByScalar(gx, c); // gx * c +
  s2 := MultVectorByScalar(produitVectoriel(no, gx, false), s);// (no % gx) * s;
  mx := AddTwoVecteurs(s1, s2);
end;
procedure TPilotageDistoX.TurnVectors(const gxp, mxp, gr, mr: TPoint3Df; out gx, mx: TPoint3Df);
var
  s, c, a: Extended;
begin
  s  := gr.Z * gxp.Y - gr.Y * gxp.Z + mr.Z * mxp.Y - mr.Y * mxp.Z;          //float s = gr.z * gxp.y - gr.y * gxp.z + mr.z * mxp.y - mr.y * mxp.z;
  c  := gr.Y * gxp.Y + gr.Z * gxp.Z + mr.Y * mxp.Y + mr.Z * mxp.Z;          //float c = gr.y * gxp.y + gr.z * gxp.z + mr.y * mxp.y + mr.z * mxp.z;
  a  := arctan2(s, c);                                                      //float a = (float)Math.Atan2(s, c);
  gx := TurnX(gxp, a);                                                      // gx = gxp.TurnX(a);
  mx := TurnX(mxp, a);                                                      // mx = mxp.TurnX(a);
end;
// Cette fonction peut modifier les paramètres d'entrée
procedure TPilotageDistoX.CheckOverflow(var M: TMatrix3x3; var V: TPoint3Df);
const
  SHORT_MAX_VALUE = 32767;
var
  Mo: TMatrix3x3;
  Vo: TPoint3Df;
  qmax: Extended;
  qM: TMatrix3x3;
  qV: TPoint3Df;
begin
   Mo := makeMatrix3x3Nulle;
   Vo := makeTPoint3Df(0.00, 0.00, 0.00);
   qmax := max(MaxDiffMatrix(m, Mo) * FM,
               MaxDiffVector(v, Vo) * FV);
  if (qmax > SHORT_MAX_VALUE) then
  begin
     qM := MultMatrix3x3ByScalar(M, SHORT_MAX_VALUE / qmax); //           m = m * (short.MaxValue / max);
     qV := MultVectorByScalar(V, SHORT_MAX_VALUE / qmax);
  end;
end;
// Calibration non linéaire OK
procedure TPilotageDistoX.CalculerCalibration();
var
  Nb: Integer;
  gr: array of TPoint3Df;
  mr: array of TPoint3Df;
  gx: array of TPoint3Df;
  mx: array of TPoint3Df;
  InvNb: Extended;
  i: Integer;
  aG0, aM0: TMatrix3x3;
  sumG, sumM: TPoint3Df;
  sumG2, sumM2: TMatrix3x3;
  CarreKronecker: TMatrix3x3;
  sa, ca: Extended;
  G, M: TPoint3Df;
  alpha: float;
  avG, avM: TPoint3Df;
  WU: TMatrix3x3;
  invG, invM: TMatrix3x3;
  aG, aM: TMatrix3x3;
  oO, bG, bM: TPoint3Df;
  NbIter: Integer;
  delta: double;
  dg: TPoint3Df;
  dm: TPoint3Df;
  Cnt: Integer;
  grp: TPoint3Df;
  mrp: TPoint3Df;
  first: Integer;
  gt: TPoint3Df;
  mt: TPoint3Df;
  gxp: TPoint3Df;
  mxp: TPoint3Df;
  avGx: TPoint3Df;
  avMx: TPoint3Df;
  sumGxG: TMatrix3x3;
  sumMxM: TMatrix3x3;
  GG: TPoint3Df;
  MM: TPoint3Df;
  pk: TMatrix3x3;
  QQ: TMatrix3x3;
  moy: Extended;
  EWE: Double;
begin
  Nb := GetNbMesuresG;
  AfficherMessage(Format('%s.Calibrage: Nb = %d', [ClassName, Nb]));

  InvNb := 1.0/Nb;
  SetLength(gr, Nb);
  SetLength(mr, Nb);
  SetLength(gx, Nb);
  SetLength(mx, Nb);
  for i := 0 to Nb-1 do
  begin
    gr[i] := makeVecteurNul;
    mr[i] := makeVecteurNul;
    gx[i] := makeVecteurNul;
    mx[i] := makeVecteurNul;
  end;
  aG0   := makeMatrix3x3Nulle;                                                  // Matrix aG0,
  aM0   := makeMatrix3x3Nulle;                                                  //        aM0;
  sumG  := makeVecteurNul;                                                      // Vector sumG = Vector.zero;
  sumM  := makeVecteurNul;                                                      // Vector sumM = Vector.zero;
  sumG2 := makeMatrix3x3Nulle;                                                  // Matrix sumG2 = Matrix.zero;
  sumM2 := makeMatrix3x3Nulle;                                                  // Matrix sumM2 = Matrix.zero;
  CarreKronecker := makeMatrix3x3Nulle;
  sa := 0.0;                                                                    //float sa = 0, ca = 0;
  ca := 0.0;                                                                    //float sa = 0, ca = 0;
  AfficherMessage(Format('%s: %s', [{$I %FILE%}, {$I %LINE%}]));
  AfficherMessage(' -- Boucle 001');
  //Boucle 001
  for i := 0 to Nb - 1 do                                                       // for (int i = 0; i < num; i++)
  begin                                                                         // {
    // sum up g x m for initial alpha                                           //     sum up g x m for initial alpha
    G := self.GetMesureG(i);                                                    //     TPoint3Df G = this.getMesureG(i)
    M := self.GetMesureM(i);                                                    //     TPoint3Df M = this.getMesureM(i);    ;
    sa += NormeVecteur(produitVectoriel(G, M, false));                          //     sa += NormeVecteur(produitVectoriel(G, M, false)); //     Vector.Abs(g[i] % m[i]); // cross product
    ca += produitScalaire(G, M);                                                //     ca += produitScalaire(G, M);                       //     g[i] * m[i]; // dot product
    //AfficherMessage(Format('%d: G = (%.6f, %.6f, %.6f), M = (%.6f, %.6f, %.6f), sa = %.6f, ca = %.6f',
    //                       [i, G.X, G.Y, G.Z, M.X, M.Y, M.Z, sa, ca]));
    sumG := AddTwoVecteurs(sumG, G);                                            //     sumG = AddVecteurs(sumG, G);
    sumM := AddTwoVecteurs(sumM, M);                                            //     sumM := AddVecteurs(sumM, M);
    CarreKronecker := ProduitKroneckerVecteurs(G, G);                           //     CarreKronecker = ProduitKroneckerVecteurs(G, G);
    sumG2 := AddMatrix3x3(sumG2, CarreKronecker);                               //     sumG2 = AddMatrix3x3(sumG2, CarreKronecker) ;      //     sumG2 += (g[i] & g[i]); // outer product
    //afficherMatrix3x3('CarreKronecker', CarreKronecker);
    CarreKronecker := ProduitKroneckerVecteurs(M, M);                           //         CarreKronecker = ProduitKroneckerVecteurs(M, M);
    sumM2 := AddMatrix3x3(sumM2, CarreKronecker);                               //  sumM2 = AddMatrix3x3(sumM2, CarreKronecker);       //     sumM2 += (m[i] & m[i]); // outer product
    //afficherMatrix3x3('sumG2', sumG2);
    //afficherMatrix3x3('CarreKronecker', CarreKronecker);
    //afficherMatrix3x3('sumM2', sumM2);
  end;                                                                          // }
  //AfficherMessage(format('sa = %.6f, ca = %.6f', [sa, ca]));
  alpha := Math.arctan2(sa, ca);
  avG := MultVectorByScalar(sumG, invNb);                                       // TPoint3Df avG = MultVectorByScalar(sumG, invNb);         // Vector avG = sumG * invNum; // average g
  avM := MultVectorByScalar(sumM, invNb);                                       // TPoint3Df avM = MultVectorByScalar(sumM, invNb);         // Vector avM = sumM * invNum;
  // average Matrix invG = Matrix.Inverse(sumG2 - (sumG & avG));
                                                                                // TMatrix3x3 WU = new TMatrix3x3();
  CarreKronecker := ProduitKroneckerVecteurs(sumG, avG);                        // CarreKronecker = ProduitKroneckerVecteurs(sumG, avG);
  WU   := SubstractMatrix3x3(sumG2, CarreKronecker);                            // WU = SubstractMatrix3x3(sumG2, CarreKronecker);
  invG := InvMat3x3(WU);                                                        // TMatrix3x3 invG = InvMat3x3(WU);     // Matrix invG = Matrix.Inverse(sumG2 - (sumG & avG));
  // average Matrix invM = Matrix.Inverse(sumM2 - (sumM & avM));
  CarreKronecker := ProduitKroneckerVecteurs(sumM, avM);                        // CarreKronecker = ProduitKroneckerVecteurs(sumM, avM);
  WU   := SubstractMatrix3x3(sumM2, CarreKronecker);                            // WU = SubstractMatrix3x3(sumM2, CarreKronecker);
  invM := InvMat3x3(WU);                                                        // TMatrix3x3 invM = InvMat3x3(WU);     // Matrix invM = Matrix.Inverse(sumM2 - (sumM & avM));
  aG   := LoadIdentity3x3();                                                    // aG = makeIdentity(); // aG = aM = Matrix.one;
  aM   := LoadIdentity3x3();                                                    // aM = makeIdentity();
  oO   := makeVecteurNul();                                                     // TPoint3Df oO = makeVecteurNul();
  bG   := SubstractVecteurs(oO, avG);                                           // bG = SubstractVecteurs(oO, avG); //bG = Vector.zero - avG; // use negative average as initial offset
  bM   := SubstractVecteurs(oO, avM);                                           //  bM = SubstractVecteurs(oO, avM); //bM = Vector.zero - avM;
  NbIter := 0;
  // OK jusqu'ici
  afficherMatrix3x3('invG', invG);
  afficherMatrix3x3('invM', invM);
  AfficherMessage('-- Boucle principale');
  //*** Boucle principale                                                       // Boucle principale
  // Validée. Revoir la condition de sortie
  while (true) do                                                               // do
  begin                                                                         // {
    // Boucle 001: Validé
    for i := 0 to Nb -1 do                                                      //     for (int i = 0; i < Nb; i++)
    begin                                                                       //     {
      gr[i] := AddTwoVecteurs(MultMat3x3ByVector(aG, getMesureG(i)), bG);       //         fgr[i] = AddVecteurs(MultMat3x3ByVector(aG, this.getMesureG(i)), bG); //     gr[i] = (aG * g[i]) + bG;
      mr[i] := AddTwoVecteurs(MultMat3x3ByVector(aM, getMesureM(i)), bM);       //         fmr[i] = AddVecteurs(MultMat3x3ByVector(aM, this.getMesureM(i)), bM); //     mr[i] = (aM * m[i]) + bM;
    end;                                                                        //     }
    sa := 0.00; ca := 0.00;                                                     //     sa = 0F; ca = 0F;
    Cnt := 0; // Boucle à remplacer par un while                                // Boucle à remplacer par un while
    while (Cnt < Nb) do                                                         //     for (int i = 0; i < Nb; i++)
    begin                                                                       //     {
       // get optimal gx & mx from gr & mr                                      //        // get optimal gx & mx from gr & mr
       if (Cnt < 16) then // equidirectional sample groups                      //        if (i < 16)  // equidirectional sample groups
       begin                                                                    //        {
         grp := makeVecteurNul();                                               //            Vector grp = Vector.zero;
         mrp := makeVecteurNul();                                               //            Vector mrp = Vector.zero;
         first := cnt;                                                          //            int first = i;
         for i := first to (first + 4) - 1 do                                   //            for (; i < first + 4; i++)
         begin                                                                  //            {
           // match gr & mr to first gr & mr                                    //                // match gr & mr to first gr & mr
           TurnVectors(gr[Cnt], mr[Cnt], gr[first], mr[first], gt, mt);         //                TurnVectors(gr[i], mr[i], gr[first], mr[first], out gt, out mt);
           grp := AddTwoVecteurs(grp, gt);                                      //                grp += gt;
           mrp := AddTwoVecteurs(mrp, mt);                                      //                mrp += mt;
           Cnt := Cnt + 1;
         end;                                                                   //            }
         // get optimal matched gx & mx from sum of matched gr & mr             //            // get optimal matched gx & mx from sum of matched gr & mr
         OptVectors(grp, mrp, alpha, gxp, mxp);                                 //            OptVectors(grp, mrp, alpha, out gxp, out mxp);

         // alpha calculation                                                   //            alpha calculation
         sa += NormeVecteur(produitVectoriel(mrp, gxp, false));                 //            sa += Vector.Abs(mrp % gxp);
         ca += produitScalaire(mrp, gxp);                                       //            ca += mrp * gxp;
         // Boucle0004
         Cnt := first;
         for i := first to (first + 4) - 1 do                                   //            for (i = first; i < first + 4; i++)
         begin                                                                  //            { // get optimal gx & mx from matched gx & mx
           TurnVectors(gxp, mxp, gr[Cnt], mr[Cnt], gt, mt);                     //                TurnVectors(gxp, mxp, gr[i], mr[i], out gx[i], out mx[i]);
           gx[Cnt] := gt;
           mx[Cnt] := mt;
           Cnt := Cnt + 1;
         end;                                                                   //           }
         Cnt := Cnt - 1;                                                        //           i--;  // Instruction interdite en Pascal
       end                                                                      //        } //  if (i < 16) // equidirectional sample groups
       else                                                                     //        else
       begin // individual sample                                               //        { // individual sample
         ;                                                                      //             this.OutOptVectors_gx = makeVecteurNul();
         ;                                                                      //             this.OutOptVectors_mx = makeVecteurNul();
         OptVectors(gr[Cnt], mr[Cnt], alpha, gt, mt);                           //             OptVectors(gr[i], mr[i], alpha, out gx[i], out mx[i]);
         gx[Cnt] := gt;
         mx[Cnt] := mt;
         // alpha calculation                                                   //             // alpha calculation
         sa += NormeVecteur(produitVectoriel(mr[Cnt], gx[Cnt], false));         //             sa += Vector.Abs(mr[i] % gx[i]);
         ca += produitScalaire(mr[Cnt], gx[Cnt]);
       end;                                                                     //        }
       Cnt += 1;
    end;                                                                        //     }  // fin à remplacer par un while
    alpha := arctan2(sa, ca);                                                   //     alpha = Math.atan2(sa, ca);

    // get aG & aM from g, m, gx, & mx                                          //     get aG & aM from g, m, gx, & mx
    avGx := makeVecteurNul();                                                   //     Vector avGx = Vector.zero;
    avMx := makeVecteurNul();                                                   //     Vector avMx = Vector.zero;
    sumGxG := makeMatrix3x3Nulle();                                             //     Matrix sumGxG = Matrix.zero;
    sumMxM := makeMatrix3x3Nulle();                                             //     Matrix sumMxM = Matrix.zero;
    for i := 0 to Nb - 1 do                                                     //     for (int i = 0; i < num; i++)
    begin                                                                       //     {
      avGx   := AddTwoVecteurs(avGx, gx[i]);                                    //         avGx += gx[i];
      avMx   := AddTwoVecteurs(avMx, mx[i]);                                    //         avMx += mx[i];
      GG     := getMesureG(i);
      MM     := getMesureM(i);
      pk     := ProduitKroneckerVecteurs(gx[i], GG);
      sumGxG := AddMatrix3x3(sumGxG, pk);                                       //         sumGxG += (gx[i] & g[i]); // outer product
      pk     := ProduitKroneckerVecteurs(mx[i], MM);
      sumMxM := AddMatrix3x3(sumMxM, pk);                                       //         sumMxM += (mx[i] & m[i]);
    end;                                                                        //     }
    // get new aG & aM                                                          //     // get new aG & aM
    aG0 := aG;                                                                  //     aG0 = aG; aM0 = aM;
    aM0 := aM;
    //afficherMatrix3x3("-- aM", aM);
    avGx := MultVectorByScalar(avGx, invNb);                                    //     avGx = avGx * invNum; // average gx
    avMx := MultVectorByScalar(avMx, invNb);                                    //     avMx = avMx * invNum;
    CarreKronecker := ProduitKroneckerVecteurs(avGx, sumG);                     //     aG = (sumGxG - (avGx & sumG)) * invG;
    QQ := SubstractMatrix3x3(sumGxG, CarreKronecker);
    aG := ProduitMat3x3(QQ, invG);
    // aM = (sumMxM - (avMx & sumM)) * invM;                                    //     aM = (sumMxM - (avMx & sumM)) * invM;
    CarreKronecker := ProduitKroneckerVecteurs(avMx, sumM);
    //afficherMatrix3x3("Kronecker aM", CarreKronecker);
    QQ := SubstractMatrix3x3(sumMxM, CarreKronecker);
    aM := ProduitMat3x3(QQ, invM);
    // enforce symmetric aG(y,z)                                                //     // enforce symmetric aG(y,z)
    (*                                                                            //     aG.y.z = aG.z.y = (aG.y.z + aG.z.y) * 0.5F;
    if ( mNonLinear ) {
      Matrix gs = new Matrix();
      gs.x.x = g.x * g.x - 0.5f;
      gs.y.y = g.y * g.y - 0.5f;
      gs.z.z = g.z * g.z - 0.5f;
      gr = bG.plus( aG.timesV( g.plus( gs.timesV( nL ) ) ) );
    } else {
      gr = bG.plus( aG.timesV( g ) );
    }
    //*)
    // Rappel: M.getValue(i,j) = M[i][j]
    moy := (aG[2, 3] + aG[3, 2]) * 0.50;
    aG[2, 3] := moy;
    aG[3, 2] := moy;
    // get new bG & bM
    bG := SubstractVecteurs(avGx, MultMat3x3ByVector(aG, avG));                 //     bG = avGx - (aG * avG);
    bM := SubstractVecteurs(avMx, MultMat3x3ByVector(aM, avM));                 //     bM = avMx - (aM * avM);
    //-----------
    NbIter += 1;                                                                //     it++;
    EWE := max(MaxDiffMatrix(aG, aG0), MaxDiffMatrix(aM, aM0));
    AfficherMessage(Format('EWE = %.8f', [EWE]));
    if ((NbIter > MAX_IT) or (EWE < EPS)) then break;
  end;                                                                          // } while (it < MAX_IT && Math.max(MaxDiffMatrix(aG, aG0), MaxDiffMatrix(aM, aM0)) > EPS);
  //*** fin boucle principale
  CheckOverflow(aG, bG);                                                        // CheckOverflow(aG, bG);
  CheckOverflow(aM, bM);                                                        // CheckOverflow(aM, bM);
  delta := 0;                                                                   // double delta = 0;
  for i := 0 to Nb -1 do                                                        // for (int i = 0; i < Nb; i++)
  begin                                                                         // {
    dg := SubstractVecteurs(gx[i], gr[i]);                                      //     TPoint3Df dg = SubstractVecteurs(fgx[i], fgr[i]);
    dm := SubstractVecteurs(mx[i], mr[i]);                                      //     TPoint3Df dm = SubstractVecteurs(fmx[i], fmr[i]);
    delta += produitScalaire(dg, dg) + produitScalaire(dm, dm);                 //     delta += produitScalaire(dg, dg) + produitScalaire(dm, dm);
  end;                                                                          // }
  delta := sqrt(delta / Nb) * 100;                                              // delta = Math.sqrt(delta / Nb) * 100;
  afficherMessage(format('%d itérations - Delta %.5f', [NbIter, delta]));       // afficherMessageFmt("%d itérations - Delta %.5f", it, delta); // delta = 0.28
  afficherMatrix3x3('aG', aG);
  afficherVecteur('bG', bG);
  afficherMatrix3x3('aM', aM);
  afficherVecteur('bM', bM);
  //*************
  fMatrix_aG  := aG;
  fMatrix_aM  := aM;
  fVecteur_bG := bG;
  fVecteur_bM := bM;
  fDelta      := delta;
  fNbIters    := NbIter;
end;


// contrôle de la calibration
function TPilotageDistoX.CheckMethodeCalibration(): boolean;
var
  A, B, C: TMatrix3x3;
  T1, T2 : Boolean;
  //U, V, W: TPoint3Df;
  O: TMatrix3x3;
begin
  (* Valeurs attendues pour le jeu de données de Beat HEEB heeb@speleo.ch
     n: 56  it: 55  delta: 0.28

           aG              bG
    1.770 -0.016  0.001    0.031
    0.015  1.761  0.003   -0.088
    0.001  0.003  1.769   -0.047
           aM              bM
    1.092 -0.019 -0.037   -0.059
    0.008  1.053  0.024   -0.028
    0.015 -0.029  1.041    0.023

            Cg
    29001   -256     17      745
      238  28854     56    -2105
       17     56  28989    -1120
            Cm
    17889   -317   -600    -1418
      132  17255    388     -660
      247   -477  17050      550
  //*)
  Result := False;
  try
    O := makeMatrix3x3Nulle;
    ViderListesMesuresGM;
    setJeuDeDonneesExemple;
    CalculerCalibration();
    afficherMessage(format('%d itérations - Delta %.5f', [GetNbIters, GetDelta]));       // afficherMessageFmt("%d itérations - Delta %.5f", it, delta); // delta = 0.28
    afficherMatrix3x3('aG', GetMatrix_aG);
    afficherVecteur('bG', GetVecteur_bG);
    afficherMatrix3x3('aM', GetMatrix_aM);
    afficherVecteur('bM', GetVecteur_bM);
    A := GetMatrix_aG;
    B := makeMatrix3x3ByValues(1.77010915, -0.01559756, 0.00105569,
                               0.01452583,  1.76108466, 0.00341350,
                               0.00104294,  0.00341350, 1.76935746
                              );
    C := SubstractMatrix3x3(A, B);
    afficherMatrix3x3('O', O);

    afficherMatrix3x3('aG - aG0', C);

    T1 := IsSameMatrix(C, O, 10 * EPS);
    AfficherMessage(IIF(T1, 'Matrice aG identique à la matrice de référence aGo', 'Matrice aG non conforme'));
    A := GetMatrix_aM;
    B := makeMatrix3x3ByValues(1.09186956, -0.01935603, -0.03660849,
                               0.00804544,  1.05315657,  0.02369314,
                               0.01509844, -0.02908758,  1.04066124
                              );
    C := SubstractMatrix3x3(A, B);
    afficherMatrix3x3('aM - aM0', C);

    T2 := IsSameMatrix(C, O, 10 * EPS);
    AfficherMessage(IIF(T2, 'Matrice aM identique à la matrice de référence aMo', 'Matrice aM non conforme'));

    result := (T1 and T2);
  except
  end;
end;

function TPilotageDistoX.MoyenneAzimuts(const QArr: TArrayOfMesuresATraiter): double;
var
  i, n: Integer;
  dx, dy: double;
  SdX, SdY: Extended;
  dxmax, dxmin, dymin, dymax: Extended;
  EWE: TMesureATraiter;
  function CalculProj(const A: double; out qdX, qdY: double): boolean;
  var
    QA: float;
  begin
    QA := degtorad(90 - A);
    qdX := cos(QA);
    qdY := sin(QA);
  end;

begin
  n := 1 + High(QArr);
  AfficherMessage(format('%s.TrierParAzimuts(%d valeurs)', [ClassName, n]));
  if (n = 0) then exit;

  SdX := 0.00;
  SdY := 0.00;
  dxmin :=  1.0E12;
  dxmax := -1.0E12;
  dymin :=  1.0E12;
  dymax := -1.0E12;


  for i := 0 to n - 1 do
  begin
    EWE := QArr[i];
    CalculProj(EWE.Azimut, dx, dy);
    SdX += dx;
    SdY += dy;
    dxmin := Min(dx, dxmin);
    dxmax := Max(dx, dxmax);
    dymin := Min(dy, dymin);
    dymax := Max(dy, dymax);
  end;
  dx := SdX / n;
  dy := SdY / n;
  Result := CalcAzimutFromdXdY(dx, dy);

end;


// construire une visée
function TPilotageDistoX.ConstruireUneViseeValide(const MyTypeVisee: TTypeDeVisee;
                                                  const MySecteur: TNumeroSecteur;
                                                  const MyCode: TNumeroCode;
                                                  const MyExpe: TNumeroExpe;
                                                  out V: TUneVisee;
                                                  out Err: TErrorConstruireUneViseeValide): boolean;

var
  i, n: Integer;
  QArrV : TArrayOfMesuresATraiter;
  QV: TMesureViseeDistoX;
  a, p, r, a1, a2: float;
  MinL, MaxL: Extended;
  MinP, MaxP: Extended;
  MinA, MaxA: Extended;
  MinDX, MaxDX, MinDY, MaxDY: Extended;
  QLongueurOK, QPentesOK, QAzimutOK: Boolean;
begin
  Result := false;
  Err    := [];
  try
    n := GetNbMesuresAcquises();
    if (n < FNbMesuresPourViseeCheminement) then
    begin
      Err := Err + [errCVV_NOT_ENOUGHT_NB_SHOTS];
      exit(false);
    end;
    SetLength(QArrV, FNbMesuresPourViseeCheminement);
    MinL :=  10000.0;
    MaxL := -10000.0;
    MinP :=  10000.0;
    MaxP := -10000.0;
    MinA :=  10000.0;
    MaxA := -10000.0;



    for i := 0 to FNbMesuresPourViseeCheminement - 1 do
    begin
      QV := GetMesureVisee(n - 1 - i);
      QArrV[i].Longueur := QV.Longueur;
      MinL := Min(QV.Longueur, MinL);
      MaxL := Max(QV.Longueur, MaxL);
      MinP := Min(QV.Pente, MinP);
      MaxP := Max(QV.Pente, MaxP);


      QArrV[i].Azimut   := QV.Azimut;
      QArrV[i].Pente    := QV.Pente;
      a := degtorad(90 - QV.Azimut);
      p := degtorad(QV.Pente);
      sincos(p, QArrV[i].dz, QArrV[i].dp);
      sincos(a, QArrV[i].dy, QArrV[i].dx);
    end;
    for i := 0 to FNbMesuresPourViseeCheminement - 1 do
    QLongueurOK := abs(MaxL - MinL) < FDistoXAccuracyLongueurs;
    QPentesOK   := abs(MaxP - MinP) < FDistoXAccuracyPentes;



                                                                                     //Exit(false);
    MinDX :=  10000.0;
    MaxDX := -10000.0;
    MinDY :=  10000.0;
    MaxDY := -10000.0;
    for i := 0 to High(QArrV) do
    begin
      MinDX := Min(QArrV[i].dx, MinDX);
      MaxDX := Max(QArrV[i].dx, MaxDX);
      MinDY := Min(QArrV[i].dy, MinDY);
      MaxDY := Max(QArrV[i].dy, MaxDY);
    end;
    a1 := radtodeg(arctan2(MinDY, MinDX));
    a2 := radtodeg(arctan2(MaxDY, MaxDX));
    QAzimutOK   := abs(a2 - a1) < FDistoXAccuracyAzimuts;
    AfficherMessageErreur(Format('AMin = %.3f, AMax = %.3f D=%.3f (prec: %.2f)', [a1  , a2  , abs(a2   - a1)  , FDistoXAccuracyAzimuts]));
    if (not QLongueurOK) then Err := Err + [errCVV_OUTBOUNDS_LONGUEUR];
    if (not QPentesOK)   then Err := Err + [errCVV_OUTBOUNDS_PENTE];
    if (not QAzimutOK)   then Err := Err + [errCVV_OUTBOUNDS_AZIMUT];

    V.NoVisee      := 0;
    V.IDSecteur    := MySecteur;
    V.Code         := MyCode;
    V.Expe         := MyExpe;
    V.Longueur     := 0.00;
    V.Azimut       := 0.00;
    V.Pente        := 0.00;
    V.LG           := 1.00;
    V.LD           := 1.00;
    V.HZ           := 1.00;
    V.HN           := 1.00;
    V.TypeVisee    := MyTypeVisee;
    V.IDTerrainStation := '';
    V.Commentaires     := '';

    for i := 0 to High(QArrV) do
    begin
      V.Longueur += QArrV[i].Longueur;
      V.Pente    += QArrV[i].Pente;
    end;
    n := 1 + High(QArrV);
    V.Longueur /= n;
    V.Pente    /= n;
    V.Azimut   := MoyenneAzimuts(QArrV);
    AfficherMessageErreur(Format('LMin = %.3f, LMax = %.3f D=%.3f (prec: %.2f) - Value = %.3f', [MinL, MaxL, abs(MaxL - MinL), FDistoXAccuracyLongueurs, V.Longueur]));
    AfficherMessageErreur(Format('AMin = %.3f, AMax = %.3f D=%.3f (prec: %.2f) - Value = %.3f', [a1  , a2  , abs(a2   - a1)  , FDistoXAccuracyAzimuts  , V.Azimut]));
    AfficherMessageErreur(Format('PMin = %.3f, PMax = %.3f D=%.3f (prec: %.2f) - Value = %.3f', [MinP, MaxP, abs(MaxP - MinP), FDistoXAccuracyPentes   , V.Pente]));

    //Result := Result - [errCVV_OTHER];
    result := (QLongueurOK and QAzimutOK and QPentesOK);
  except
    on E: Exception do Err := Err + [errCVV_OTHER];
  end;

end;

procedure TPilotageDistoX.SetAccuracies(const AccLong, AccAz, AccP: double);
begin
  FDistoXAccuracyLongueurs  := AccLong;
  FDistoXAccuracyAzimuts    := AccAz;
  FDistoXAccuracyPentes     := AccP;

end;

procedure TPilotageDistoX.EcrireMesureDansJournaux(const V: TMesureViseeDistoX);
  procedure FP(const QF: TStringDirectoryFilename; const S: string);
  var
    QFP: TextFile;
  begin
    AssignFile(QFP, QF);
    try
      if (not FileExists(QF)) then
        ReWrite(QFP)
      else
        Append(QFP);
      WriteLn(QFP, DateTimePascalToDateTimeSQL(Now, True) + '; ' + S);
    finally
      closefile(QFP);
    end;
  end;

var
  S1: String;
begin
  try
    S1 := Format('%.3f; %.3f; %.2f', [V.Longueur, V.Azimut, V.Pente]);
    FP(FFichierJournalPrincipal, S1);
    FP(FFichierJournalBackup, S1);
  except
  end;
end;

procedure TPilotageDistoX.SetFileNameJournaux(const JP, JS: TStringDirectoryFilename);
begin
  FFichierJournalPrincipal := JP;
  FFichierJournalBackup    := JS;
end;
function  TPilotageDistoX.GetFilenameJournalPrincipal(): TStringDirectoryFilename;
begin
  result := FFichierJournalPrincipal;
end;

function  TPilotageDistoX.GetFilenameJournalSecurite(): TStringDirectoryFilename;
begin
  result := FFichierJournalBackup;
end;

procedure TPilotageDistoX.ViderJournaux();
begin
  DeleteFile(FFichierJournalBackup);
  DeleteFile(FFichierJournalPrincipal);
end;

(*
procedure TPilotageDistoX.ProcOnStatus(Sender: TObject; Reason: THookSerialReason; const Value: string);
var
  WU: String;
begin
  WU := ChooseString(Ord(Reason),
                    ['HR_SerialClose',
                    'HR_Connect',
                    'HR_CanRead',
                    'HR_CanWrite',
                    'HR_ReadCount',
                    'HR_WriteCount',
                    'HR_Wait'
                    ]);

  case (Reason) of
    HR_SerialClose:
      begin
        if (FDistoXMustBeReady) then
        begin
          AfficherMessage('*** DistoX déconnecté intempestivement - Tentative de redémarrage');
          OpenDistoXConnexion();
        end;
      end;
    HR_Connect:
      begin
        FDistoXSerialNumber    := -1;
        FDistoXVersionFirmware := -1;
        FDistoXVersionHardware := -1;
        AfficherMessage(Format('OnStatus: %s - %s', [WU, Value]));
        if (assigned(FProcNotifyDistoXConnected)) then FProcNotifyDistoXConnected;
        AfficherMessage(Format('DistoX: %d - Firmware: %d - Hardware: %d', [
                              ExtractNumSerieFromDistoX(),
                              ExtractVersionFirmWareFromDistoX(),
                              ExtractVersionHardwareFromDistoX()]));

      end;
    HR_CanRead:
      begin
        ;//pass;
      end;
    HR_CanWrite: ;
    HR_ReadCount: ;
    HR_WriteCount: ;
    HR_Wait: ;
  end;


end;
//*)


end.

