unit unitExportGIS;
{$INCLUDE CompilationParameters.inc}
// 26/02/2015: Factorisation de code
// 21/08/2017: Suppression de l'export OSM (inutile et bogué)
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  Common,
  StructuresDonnees,
  UnitEntitesExtended,
  ConvertisseurJPC,
  UnitClassPalette,
  Graphics,
  Classes, SysUtils;

// export KML: Références
const
  KML_OPENGIS_WEBSITE     = 'http://www.opengis.net/kml/2.2';
  KML_GOOGLE_KML_WEBSITE  = 'http://www.google.com/kml/ext/2.2';
  W3C_W3_WEBSITE          = 'http://www.w3.org/2005/Atom';
  W3C_XML_SCHEMA_WEBSITE  = 'http://www.w3.org/2001/XMLSchema-instance';
  GPX_TOPOGRAPHIX_WEBSITE = 'http://www.topografix.com/GPX/1/0';
const
  STYLE_CENTERLINE_PAR_DEFAUT = 'StyleCenterlineParDefault';
  STYLE_GALERIE_PAR_DEFAUT    = 'StyleGalerieParDefault';

type

{ TExportGIS }

 TExportGIS = class
  private
    Ffp               : TextFile;
    FFileName         : string;
    FOutputFormatGIS  : TOutputFormatGIS;
    FCouleurDefaut    : TColor;
    FBDDEntites       : TBDDEntites;
    FUseColorsGroupes : boolean;
    FPalette256       : TPalette256;
    FConvertisseurEPSG: TConversionSysteme;
    FCodeEPSGOrigine  : integer;
    FMapWidth         : integer;
    FMapHeight        : integer;
    FCentroideLonLat  : TProjUV;


    procedure KML_WritePolygonStyle(const IDStyle: string; const LineColor, FillColor: TColorRGBA; const LineWidth: double);
    procedure KML_WritePolyLineStyle(const IDStyle: string; const LineColor: TColorRGBA; const LineWidth: double);
    procedure WriteKMLValue(const K, V: string);
    procedure CalcCentroideEntities();



  public
    function Initialiser(const M: TOutputFormatGIS;
                         const PtrBDDEntites: TBDDEntites;
                         const C: TConversionSysteme;
                         const CodeEPSGOrigine: integer;
                         const UseColorsGroupes: boolean;
                         const CouleurDefault: TColor;
                         const QMapWidth, QMapHeight: integer;
                         const QFileName: TStringDirectoryFilename): boolean;
    procedure SetTailleMapGlissante(const W, H: integer);

    procedure WriteHeader(const DoSpecifyPoint0: boolean; const QLon, QLat: double);
    procedure WriteFooter();
    procedure Finaliser();
    procedure WriteConvertedPoint(const PolyMode: TPolyMode; const ET: TEntiteEtendue);
    procedure WriteSegmentShot(const ET: TEntiteEtendue);
    procedure WritePoint(const QLat, QLon, QAlt: double;
                         const EtiquetteTitre: string;
                         const EtiquetteDescro: string;
                         const OSM_IDNode: Int64);
//    procedure WriteOSMEntrance(const E: TEntrance; const OSM_IDNode: Int64);
    procedure WriteLigne(const S: string);
    procedure WriteCommentaire(const S: string);
    procedure WriteAEntrance(const E: TEntrance; const TagInt64: Int64);
end;


implementation
const
  MAP_NAMEREF    =  'MyMap';

procedure TExportGIS.CalcCentroideEntities;
var
  PM: TProjUV;
  P1, P2: TPoint3Df;
begin
  if (not Assigned(FBDDEntites)) then Exit;

  P1 := FBDDEntites.GetCoinBasGauche();
  P2 := FBDDEntites.GetCoinHautDroit();
  PM.U := 0.50 * (P1.X + P2.X);
  PM.V := 0.50 * (P1.Y + P2.Y);
  FCentroideLonLat := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, PM);

end;

{ TExportGIS }
// en-tête de fichiers et déf des styles
procedure TExportGIS.WriteHeader(const DoSpecifyPoint0: boolean; const QLon, QLat: double);
const
    URL_LEAFLET_JS  = 'http://cdn.leafletjs.com/leaflet/v0.7.7/leaflet.js';
    URL_LEAFLET_CSS = 'http://cdn.leafletjs.com/leaflet/v0.7.7/leaflet.css';
    URL_OSM_TILES   = 'http://{s}.tile.osm.org/{z}/{x}/{y}.png';
var
 URL_OSM_Contributors: String;
 ii    : Integer;
 QColor: TColorRGBA;
 QColorShadow: TColorRGBA;
 EWE: TExpe;
 WU: TColor;
begin
  QColor       := GetColorRGBA(FCouleurDefaut, $FF);
  QColorShadow := GetColorRGBA(clWhite, $FF);
  case FOutputFormatGIS of
   gisKML: // KML: OK pour les silhouettes
     begin // KML // On définit aussi les styles d'objets polygones =
       WriteLn(Ffp, FormatterVersionEncodageXML('1.0', 'UTF-8'));
       WriteLn(Ffp, Format('<kml xmlns="%s" xmlns:gx="%s" xmlns:kml="%s" xmlns:atom="%s">',
                   [KML_OPENGIS_WEBSITE,
                    KML_GOOGLE_KML_WEBSITE,
                    KML_OPENGIS_WEBSITE,
                    W3C_W3_WEBSITE
                   ]));
       WriteLn(Ffp, Format('<!-- Generated by GHTopo Toolbox version %s - Date: %s -->', [GetGHTopoVersion(), DateTimeToStr(Now())]));
       WriteLn(Ffp, '  <Document>');
       // style de ligne par défaut
       KML_WritePolyLineStyle(STYLE_CENTERLINE_PAR_DEFAUT, QColor, 2);
       // styles de polygones
       if (Assigned(FBDDEntites)) then
       begin

         KML_WritePolygonStyle(STYLE_GALERIE_PAR_DEFAUT, QColor, QColor, 1.0);
         //if (FUseColorGroupes) then
         begin
           for ii := 0 to FBDDEntites.GetNbExpes - 1 do
           begin
             EWE := FBDDEntites.GetExpe(ii);
             WU  := FPalette256.GetColorByIndex(EWE.Couleur);
             QColor := GetColorRGBA(WU, $FF);
             QColorShadow := QColor;
             KML_WritePolygonStyle(STYLE_GALERIE_PAR_DEFAUT +
                                   format('%.2X%.2X%.2X%.2X', [QColor.R, QColor.G, QColor.B, QColor.A]),
                                   QColorShadow,
                                   QColor,
                                   1.0);
           end;
         end;
       end;
       // dossier principal = la cavité
       WriteLn(Ffp, '    <Folder>');
       WriteLn(Ffp, Format('    <name>%s</name>', ['MyCavite001']));
       WriteLn(Ffp, Format('    <open>%d</open>', [1]));

     end;
   gisGPX:
     begin // GPX
       WriteLn(Ffp, FormatterVersionEncodageXML('1.0', 'ISO-8859-1')); //<?xml version="1.0" encoding="ISO-8859-1"?>');
       WriteLn(Ffp, Format('<gpx version="%.1f" creator="%s" xmlns:xsi="%s" xmlns="%s">',
                          [1.1, // version xml
                           rsGHTOPOEXENAME, // logiciel auteur
                           W3C_XML_SCHEMA_WEBSITE,
                           GPX_TOPOGRAPHIX_WEBSITE
                          ]));
     end;

   gisLEAFLET: // Leaflet
     begin
       CalcCentroideEntities();
       WriteLigne('<!DOCTYPE html>');
       WriteLigne(Format('<!-- %s generated by %s %s - Date: %s -->', [FFileName,
                                                                       ApplicationName,
                                                                       GetGHTopoVersion(),
                                                                       DateTimeToStr(Now())
                                                                      ]));
       WriteLigne('<HTML>');
       WriteLigne('<HEAD>');
       WriteLigne(Format('  <TITLE>%s</TITLE>', ['Report sur OpenStreetMap: ' + FFileName]));
       WriteLigne(Format('  <META charset="%s" />', ['utf-8']));
       WriteLigne(Format('  <META name="%s" content="width=%s, initial-scale=%.1f">', ['viewport', 'device-width', 1.0]));
       //<meta name="viewport" content="width=device-width, initial-scale=1.0">
       WriteLigne(Format('  <link rel="%s" href="%s" />', ['stylesheet', URL_LEAFLET_CSS]));
       WriteLigne('</HEAD>');
       WriteLigne('<BODY>');
       WriteLigne(Format('  <div id="%s" style="width: %dpx; height: %dpx"></div>', [MAP_NAMEREF, FMapWidth, FMapHeight]));
       WriteLigne(Format('  <script src="%s"></script>', [URL_LEAFLET_JS]));
       WriteLigne('    <SCRIPT>');
       WriteLigne('     // Ajouté le 01/09/2016');
       WriteLigne('     function EWE(QMap, QLat, QLon, QCaption)');
       WriteLigne('     {');
       WriteLigne('       L.marker([QLat, QLon]).addTo(QMap).bindPopup(QCaption);');
       WriteLigne('     }');
       if (DoSpecifyPoint0) then WriteLigne(Format('     var %s = L.map(''%s'').setView([%.10f, %.10f], %d);', [MAP_NAMEREF, MAP_NAMEREF, QLon, QLat, 15])) // OK
                            else WriteLigne(Format('     var %s = L.map(''%s'').setView([%.10f, %.10f], %d);', [MAP_NAMEREF, MAP_NAMEREF, FCentroideLonLat.U, FCentroideLonLat.V, 15])); // OK
       WriteLigne(Format('     L.tileLayer(''%s'',', [URL_OSM_TILES]));  // OK
       WriteLigne('       {');
       URL_OSM_Contributors := Format('<a href="%s">%s</a> %s', ['http://osm.org/copyright', 'OpenStreetMap', 'contributors']);
       WriteLigne(Format('         attribution: ''&copy; %s''', [URL_OSM_Contributors]));
       WriteLigne('       }');
       WriteLigne(Format('     ).addTo(%s);', [MAP_NAMEREF]));
       WriteLigne('      //**----- BEGIN DATA -----**');
     end;

    (*gisAUTOCAD_2D:
     begin
       WriteLigne('*** DXF EN ATTENTE - Debut fichier ***');
     end;//*)

  end;
end;

procedure TExportGIS.WriteLigne(const S: string);
begin
  writeln(Ffp, S);
end;

procedure TExportGIS.WriteCommentaire(const S: string);
begin
  case FOutputFormatGIS of
    gisKML,
    gisGPX            : WriteLigne(Format('<!-- %s -->', [XMLStrToPasStr(S)]));
    gisLEAFLET        : WriteLigne(Format('// %s', [S]));
    //gisAUTOCAD_2D     : WriteLigne(Format('*** DXF %s', [S]));
  end;
end;



procedure TExportGIS.WriteAEntrance(const E: TEntrance; const TagInt64: Int64);
var
  P1, P2: TProjUV;
begin
  P1.U := E.eXEntree;
  P1.V := E.eYEntree;
  P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
  WritePoint(P2.U, P2.V, E.eZEntree, E.eNomEntree, Format('%d.%d', [E.eRefSer, E.eRefSt]), TagInt64);
end;
procedure TExportGIS.WriteFooter();
begin
  case FOutputFormatGIS of
    gisKML:
      begin // KML
        // dossier principal = la cavité
        WriteLn(Ffp, '    </Folder>');
        WriteLn(Ffp, '  </Document>');
        WriteLn(Ffp, '</kml>');
      end;
    gisGPX:
      begin // GPX
        WriteLn(Ffp, ' </gpx>');
      end;
    gisLEAFLET: // Leaflet;
      begin
        WriteLigne('        //**----- END OF DATA -----**');
        // Ceci est indispensable pour obtenir les coordonnées du point cliqué
        WriteLigne('        // Info-bulle indiquant les coordonnées du point clique');
        WriteLigne('        var popup = L.popup();');
        WriteLigne('        function onMapClick(e) {');
        WriteLigne(Format('         popup.setLatLng(e.latlng).setContent("%s " + e.latlng.toString()).openOn(%s);', ['', MAP_NAMEREF]));
        WriteLigne('        }');
        WriteLigne(Format('        %s.on(''click'', onMapClick);', [MAP_NAMEREF]));
        WriteLigne('  </SCRIPT>');
        WriteLigne('</BODY>');
        WriteLigne('</HTML>');
      end;
    (*gisAUTOCAD_2D:
      begin
        WriteLigne('*** DXF EN ATTENTE - Fin de fichier ***');
      end;
      //*)
  end;
end;

// encodage du style
procedure TExportGIS.KML_WritePolygonStyle(const IDStyle: string;
                                           const LineColor, FillColor: TColorRGBA;
                                           const LineWidth: double);
begin
 Writeln(Ffp, Format('   <Style id="%s">', [IDStyle]));
 WriteLn(Ffp,        '     <LineStyle>');
 WriteLn(Ffp, Format('       %s', [KMLColor(LineColor.R, LineColor.G, LineColor.B, LineColor.A)]));
 WriteLn(Ffp, Format('       <width>%.1f</width>', [LineWidth]));
 WriteLn(Ffp,        '     </LineStyle>');
 WriteLn(Ffp,        '     <PolyStyle>');
 WriteLn(Ffp, Format('       %s', [KMLColor(FillColor.R, FillColor.G, FillColor.B, FillColor.A)]));
 WriteLn(Ffp,        '     </PolyStyle>');
 WriteLn(Ffp,        '   </Style>');
end;
// encodage du style
procedure TExportGIS.KML_WritePolyLineStyle(const IDStyle: string;
                                            const LineColor: TColorRGBA;
                                            const LineWidth: double);
begin
 Writeln(Ffp, Format('   <Style id="%s">', [IDStyle]));
 WriteLn(Ffp,        '     <LineStyle>');
 WriteLn(Ffp, Format('        %s', [KMLColor(LineColor.R, LineColor.G, LineColor.B, LineColor.A)]));
 WriteLn(Ffp, Format('       <width>%.1f</width>', [LineWidth]));
 WriteLn(Ffp,        '     </LineStyle>');
 WriteLn(Ffp,        '   </Style>');
end;

procedure TExportGIS.SetTailleMapGlissante(const W, H: integer);
begin
  FMapWidth  := W;
  FMapHeight := H;
end;


procedure TExportGIS.WriteKMLValue(const K, V: string);
begin
 WriteLn(Ffp, Format('     <%s>%s</%s>',[K, PasStrToXMLStr(V), K]));
end;

procedure TExportGIS.WritePoint(const QLat, QLon, QAlt: double;
                                const EtiquetteTitre: string;
                                const EtiquetteDescro: string;
                                const OSM_IDNode: Int64);
var
  WU: String;
begin
  case FOutputFormatGIS of
    gisKML:
     begin // KML; sans objet
       WriteLigne('    <Placemark>');
       WriteKMLValue('name', XMLStrToPasStr(EtiquetteTitre));
       WriteLigne('      <description>');
       WriteLigne('         <![CDATA[');
       WriteLigne(Format('         <h1>%s</h1>', [EtiquetteTitre]));
       WU := StringReplace(EtiquetteDescro, #9, '<BR>', [rfReplaceAll, rfIgnoreCase]);
       WriteLigne('         <P>' + WU + '<P>');
       //   <h1>CDATA Tags are useful!</h1>
       //   <p><font color="red">Text is <i>more readable</i> and
       //   <b>easier to write</b> when you can avoid using entity
       //   references.</font></p>
       WriteLigne('         ]]>');
       WriteLigne('      </description>');
       WriteLigne('      <Point>');

       WriteKMLValue('coordinates', Format('%s, %s, %s', [ensureConvertLongOrLatToXML(QLon), ensureConvertLongOrLatToXML(QLat), ensureConvertLongOrLatToXML(QAlt)]));
       WriteLigne('      </Point>');
       WriteLigne('    </Placemark>');
     end;
    gisGPX:
     begin
       WriteLigne(Format('    <wpt lat="%.10f" lon="%.10f">', [QLat, QLon]));
       WriteKMLValue('ele', Format('%.2f', [QAlt]));
       WriteKMLValue('name', XMLStrToPasStr(EtiquetteTitre));
       WriteLigne('    </wpt>');
     end;

    gisLEAFLET:
     begin
       WriteLigne(Format('      EWE(%s, %s, %s, "%s");', [MAP_NAMEREF,
                                                          ensureConvertLongOrLatToXML(QLat),
                                                          ensureConvertLongOrLatToXML(QLon),
                                                          EtiquetteTitre]));
     end;
    (*
    gisAUTOCAD_2D:
     begin
       WriteLigne(Format('*** DXF: %.6f, %.6f, %.3f', [QLat, QLon, QAlt]));
     end;   //*)
  end; // case
end;


procedure TExportGIS.WriteConvertedPoint(const PolyMode: TPolyMode;
                                         const ET: TEntiteEtendue);
const
 FMT_CE = 'P;%.12f;%.12f;%.0f;-9999;;0.00;00:00:00;0;0.0;-1';

var
 CPColor: TColor;
 R   : Double;
 P1, P2: TProjUV;
 PT1: TProjUV;
 PT2: TProjUV;
begin
  CPColor := FCouleurDefaut; //IIF(DoUseColors, ET.ColorEntite, );
  // éliminer visées trop courtes et trop longues
  R:=Hypot3D(ET.Une_Station_2_X - ET.Une_Station_1_X,
             ET.Une_Station_2_Y - ET.Une_Station_1_Y,
             ET.Une_Station_2_Z - ET.Une_Station_1_Z
           );
  if (R > 160.00) then Exit;
  // éliminer les entrées
  if (ET.Type_Entite = tgENTRANCE) then Exit;
  // export point
  case FOutputFormatGIS of
   gisKML:
     begin // KML; sans objet
        case PolyMode of
          tpmENTETE_POLY:
            begin
               WriteLn(Ffp, '  <Placemark>');
               WriteLn(Ffp, Format('    <styleUrl>#%s</styleUrl>', [STYLE_CENTERLINE_PAR_DEFAUT]));

            end;
          tpmSTART_POLY:
            begin

              WriteLn(Ffp, '    <LineString>');
              WriteLn(Ffp, '      <extrude>1</extrude>');
              WriteLn(Ffp, '      <tessellate>1</tessellate>');
              WriteLn(Ffp, '      <altitudeMode>relative</altitudeMode>');
              WriteLn(Ffp, '      <coordinates>');
              P1.U := ET.Une_Station_1_X;
              P1.V := ET.Une_Station_1_Y;
              P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
              WriteLn(Ffp, Format('        %s, %s, %.2f', [ensureConvertLongOrLatToXML(P2.V),
                                                           ensureConvertLongOrLatToXML(P2.U),
                                                           0.01]));
            end;
          tpmPOINT_POLY:
            begin
              P1.U := ET.Une_Station_1_X;
              P1.V := ET.Une_Station_1_Y;
              P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
              WriteLn(Ffp, Format('        %s, %s, %.2f', [ensureConvertLongOrLatToXML(P2.V),
                                                           ensureConvertLongOrLatToXML(P2.U),
                                                           0.01]));
            end;
          tpmEND_POLY:
            begin
              WriteLn(Ffp, '      </coordinates>');
              WriteLn(Ffp, '    </LineString>');
              WriteLn(Ffp, '   </Placemark>');
            end;
        end;
     end;
   gisGPX:
     begin // GPX
       // séries
       case PolyMode of
         tpmENTETE_POLY: //0
           begin  // en-tete de track/série
             WriteLn(Ffp, '    <trk> <trkseg>');
           end;
         tpmSTART_POLY: //1
           begin // début de polyligne (série)
             P1.U := ET.Une_Station_1_X;
             P1.V := ET.Une_Station_1_Y;
             P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
             WriteLn(Ffp, Format('        <trkpt lat="%s" lon="%s"></trkpt>',
                                [ensureConvertLongOrLatToXML(P2.U),
                                 ensureConvertLongOrLatToXML(P2.V)
                                ]));
            end;
         tpmPOINT_POLY: //2
           begin // point de polyligne (série)
             P1.U := ET.Une_Station_2_X;
             P1.V := ET.Une_Station_2_Y;
             P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
             WriteLn(ffp, Format('        <trkpt lat="%s" lon="%s"></trkpt>', [
                                ensureConvertLongOrLatToXML(P2.U),
                                ensureConvertLongOrLatToXML(P2.V),
                                ET.Entite_Serie, ET.Entite_Station]));
          end;
         tpmEND_POLY:   //3
           begin // GPX: balise de cloture
            WriteLn(ffp, '    </trkseg> </trk>');
           end;
       end;
     end;
   gisLEAFLET:
     begin
       ;
     end;
   (*gisAUTOCAD_2D:
     begin
       case PolyMode of
         tpmENTETE_POLY: //0
           begin  // en-tete de track/série
             WriteLn(Ffp, '    *** DXF: Header polyline');
             WriteLn(Ffp, '100');
             WriteLn(Ffp, '  AcDb2dPolyline');
             WriteLn(Ffp, '10');
             WriteLn(Ffp, ' 0');
             WriteLn(Ffp, '20');
             WriteLn(Ffp, ' 0');
             WriteLn(Ffp, '30');
             WriteLn(Ffp, ' 0.00');
           end;
         tpmSTART_POLY: //1
           begin // début de polyligne (série)
             WriteLn(Ffp, Format('        *** DXF: Start Polyline at %.3f, %.3f, %.3f',
                                  [ET.Une_Station_1_X, ET.Une_Station_1_Y, ET.Une_Station_1_Z]));

           end;
         tpmPOINT_POLY: //2
           begin // point de polyligne (série)
             WriteLn(Ffp, Format('        *** DXF: Vertex polyline at %.3f, %.3f, %.3f', [ET.Une_Station_1_X, ET.Une_Station_1_Y, ET.Une_Station_1_Z]));
           end;
         tpmEND_POLY:   //3
           begin // GPX: balise de cloture
             WriteLn(Ffp, '    *** DXF: Ending polyline');
           end;
       end;
     end;//*)
  end; // case
end;
procedure TExportGIS.WriteSegmentShot(const ET: TEntiteEtendue);
var
  P1, P2: TProjUV;
  PT1: TProjUV;
  PT2: TProjUV;
begin
  case FOutputFormatGIS of
    gisGPX:
      begin
        WriteLn(Ffp, '    <trk> <trkseg>');
        P1.U := ET.Une_Station_1_X;
        P1.V := ET.Une_Station_1_Y;
        P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
        WriteLn(Ffp, Format('        <trkpt lat="%s" lon="%s"></trkpt>', [
                               ensureConvertLongOrLatToXML(P2.U),
                               ensureConvertLongOrLatToXML(P2.V)]));
        P1.U := ET.Une_Station_2_X;
        P1.V := ET.Une_Station_2_Y;
        P2 := FConvertisseurEPSG.ConversionSyst1ToSyst2EPSG(FCodeEPSGOrigine, CODE_EPSG_WGS84, P1);
        WriteLn(Ffp, Format('        <trkpt lat="%s" lon="%s"></trkpt>', [
                               ensureConvertLongOrLatToXML(P2.U),
                               ensureConvertLongOrLatToXML(P2.V)]));
        WriteLn(Ffp, '    </trkseg> </trk>');
      end;
    (*gisAUTOCAD_2D:
      begin
        WriteLn(Ffp, Format('    *** DXF : [%d.%d] %.3f; %.3f; %.3f -> %.3f; %.3f; %.3f',
                            [ET.Entite_Serie, ET.Entite_Station,
                             ET.Une_Station_1_X, ET.Une_Station_1_Y, ET.Une_Station_1_Z,
                             ET.Une_Station_2_X, ET.Une_Station_2_Y, ET.Une_Station_2_Z]));
      end;       //*)
  otherwise
    ;
  end;
end;

//******************************************************************************

procedure TExportGIS.Finaliser;
begin
  try

  finally
    CloseFile(Ffp);
    FPalette256.Free;
  end;
end;

function TExportGIS.Initialiser(const M: TOutputFormatGIS;
                                const PtrBDDEntites: TBDDEntites;
                                const C: TConversionSysteme;
                                const CodeEPSGOrigine: integer;
                                const UseColorsGroupes: boolean;
                                const CouleurDefault: TColor;
                                const QMapWidth, QMapHeight: integer;
                                const QFileName: TStringDirectoryFilename): boolean;
begin
  Result := false;
  try
    FMapWidth         := QMapWidth;  //1200;
    FMapHeight        := QMapHeight; // 900;
    FFileName         := QFileName;

    FOutputFormatGIS  := M;
    FCouleurDefaut    := CouleurDefault;
    FUseColorsGroupes := UseColorsGroupes;
    FBDDEntites       := PtrBDDEntites;
    // convertisseur
    FConvertisseurEPSG := C;
    FCodeEPSGOrigine := CodeEPSGOrigine;
    // init palette
    FPalette256 := TPalette256.Create;
    FPalette256.GenerateTOPOROBOTPalette;
    AssignFile(FFp, QFileName);
    ReWrite(Ffp);
    Result := True;
  except
  end;
end;

end.

