unit UnitExportToODG;
// 17/02/2014: Export OK : Courbes, lignes, polygones
//             En cours  : Textes
//             A faire   : symboles, photos
//             DONE      : Conversion de coordonnées
//             A fixer   : Textes - appels OK
//             A fixer   : Le document créé nécessite une réparation (qui réussit toutefois)
//             Factorisation de QGetCoordsGCS
// 18/02/2014: Export par groupes
// 11/03/2014: Adaptation à Lazarus 1.2
//             Résolution d'un conflit de types
// 10/04/2014: Ajout des cartouches, contenant une échelle
//             Les objets polygones sont dessinés en premier
//
{$INCLUDE CompilationParameters.inc}
{$IFDEF GHTOPO}
  {$UNDEF GHCAVEDRAW}
{$ENDIF}
{$IFDEF GHCAVEDRAW}
  {$UNDEF GHTOPO}
{$ENDIF}


//{$undef GHCAVEDRAW}



interface
uses
  {$IFDEF LANGUAGE_FRENCH}
    UnitMessages_fr,
  {$ENDIF}
  Classes, SysUtils, Graphics,
  {$IFDEF GHTOPO}
     StructuresDonnees,
     Common,
     UnitEntitesExtended,
  {$ENDIF}
  {$IFDEF GHCAVEDRAW}
    UnitDocDessin,
    GeneralFunctions,
    GHCD_Types,
  {$ENDIF}
  oodrawwriter;
type

{ TExportOdgUtils }

 TExportOdgUtils = class
  private
    {$IFDEF GHCAVEDRAW}
      FMyDocDessin: TDocumentDessin;
    {$ENDIF}
    {$IFDEF GHTOPO}
      FMyTableEntites: TBDDEntites;
    {$ENDIF}

    FODGFileName: string;
    FODGContexte: Todg;
    // variables internes
    FLeGroupeCourant: string;
    // limites de dessin
    FCoordCoin1: TPoint3Df;
    FCoordCoin2: TPoint3Df;
    FHauteur   : double;
    FRapportX  : Extended;
    FEchelle   : double;

    // wrappers pour les groupes
    procedure ConversionCoordonnees(const QX, QY: double; out CX, CY: double);
    procedure DebutDeGroupe(const NomGroupe: string);
    procedure FinDeGroupe(const NomGroupe: string);
    procedure DessinCartouche;
    {$IFDEF GHCAVEDRAW}
       procedure DessinCenterLines;
       procedure DessinEntrees;

       function  QGetCoordsGCS(const IDBS: TIDBaseStation; const IDGroupe: TIDGroupeEntites; const Offset: TPoint3Df): TODGPoint2Df;
       procedure DessinerGroupe(const MyGroupe: TGroupeEntites);
       procedure DessinerLesCourbes(const LeGroupe: TGroupeEntites);
       procedure DessinerLesLignes(const LeGroupe: TGroupeEntites);
       procedure DessinerLesPolygones(const LeGroupe: TGroupeEntites);
       procedure DessinerLesTextes(const LeGroupe: TGroupeEntites);
       procedure TracerCourbe(const C: TCourbe);
       procedure TracerPolygone(const P: TPolygone);
       procedure TracerSimpleLigne(const L: TSimpleLigne);
       procedure TracerTexte(const T: TTextObject);
    {$ENDIF}


  public
    {$IFDEF GHCAVEDRAW}
      function  Initialiser(const FP: TDocumentDessin; const FileName: string): boolean;
    {$ENDIF}
    {$IFDEF GHTOPO}
      function  Initialiser(const FP: TBDDEntites; const FileName: string): boolean;
    {$ENDIF}
    procedure Finaliser;
    procedure LancerExportODG(const Echelle: double);


end;

implementation

{ TExportOdgUtils }

{$IFDEF GHTOPO}
  const GROUPE_CARTOUCHE = 'Cartouche';
  const GROUPE_ECHELLE   = 'Echelle';

  function  TExportOdgUtils.Initialiser(const FP: TBDDEntites; const FileName: string): boolean;
  begin
    FMyTableEntites := FP;
    AfficherMessage(Format('%s.Initialiser: %s', [ClassName, FileName]));//  F
  end;

{$ENDIF}
{$IFDEF GHCAVEDRAW}
function TExportOdgUtils.Initialiser(const FP: TDocumentDessin; const FileName: string): boolean;
begin
  Result := False;
  FMyDocDessin := FP;
  FODGFileName := FileName;
  AfficherMessage(Format('%s.Initialiser: %s - %s', [ClassName, FileName, FMyDocDessin.ClassName]));
  FODGContexte := Todg.Create;
  try
    // limites du dessin
    FCoordCoin1 := FMyDocDessin.GetCoordsMini;
    FCoordCoin2 := FMyDocDessin.GetCoordsMaxi;
    FHauteur    := FCoordCoin2.Y - FCoordCoin1.Y;
    FRapportX   := 1.00;
    Result := True;
  except
  end;
  // ...
end;

function TExportOdgUtils.QGetCoordsGCS(const IDBS: TIDBaseStation; const IDGroupe: TIDGroupeEntites; const Offset: TPoint3Df): TODGPoint2Df;
var
 BS : TBaseStation;
 GP : TGroupeEntites;
begin
 BS := FMyDocDessin.GetBasePointByIndex(IDBS);
 GP := FMyDocDessin.GetGroupe(IDGroupe);
 Result.X := BS.PosStation.X + GP.Decalage.X + Offset.X;
 Result.Y := BS.PosStation.Y + GP.Decalage.Y + Offset.Y;
end;

// tracé des polygones
procedure TExportOdgUtils.TracerPolygone(const P: TPolygone);
var
  i: Integer;
  PolySommets: array of TODGPoint2Df;
  PST: TVertexPolygon;
  PP, ST: TODGPoint2Df;
  EWE: String;
begin
  AfficherMessage('DessinPolygoneSVG');
  SetLength(PolySommets, 1 + High(P.Sommets));
  for i := 0 to High(P.Sommets) do
  begin
    PST := P.Sommets[i];
    PP  := QGetCoordsGCS(PST.IDStation, P.IDGroupe, PST.Offset);
    ConversionCoordonnees(PP.X, PP.Y, ST.X, ST.Y);
    PolySommets[i] := ST;
  end;
  EWE := SelectStylePolygone(P.IDStylePolygone);
  FODGContexte.DessinPolygoneSVG(EWE, PolySommets, True, True);
end;

procedure TExportOdgUtils.TracerSimpleLigne(const L: TSimpleLigne);
var
  EWE: String;
  PP1, PP2: TODGPoint2Df;
  X1, Y1, X2, Y2: TScalaire;
begin
  PP1 := QGetCoordsGCS(L.IDBaseStExt1, L.IDGroupe, L.OffsetExtr1);
  PP2 := QGetCoordsGCS(L.IDBaseStExt2, L.IDGroupe, L.OffsetExtr2);
  ConversionCoordonnees(PP1.X, PP1.Y, X1, Y1);
  ConversionCoordonnees(PP2.X, PP2.Y, X2, Y2);
  EWE := SelectStyleLigne(L.IDStyleLigne);
  FODGContexte.DessinLigne(EWE, X1, Y1, X2, Y2);
end;

// tracé des textes
procedure TExportOdgUtils.TracerTexte(const T : TTextObject);
var
  PP1: TODGPoint2Df;
  EWE: String;
  X1, Y1: double;
  BP: TBaseStation;
  WU: String;
begin
  // 7 8 9
  // 4 5 6
  // 1 2 3
  BP := FMyDocDessin.GetBasePointByIndex(T.IDBaseSt);
  PP1 := QGetCoordsGCS(T.IDBaseSt, T.IDGroupe, T.Offset);
  ConversionCoordonnees(PP1.X, PP1.Y, X1, Y1);
  EWE := SelectStyleTexte(T.IDStyleTexte);
  WU  := LLANFAIR_PG(T.Text, T.MaxLength, BP);
  FODGContexte.DessinTexte(EWE, X1, Y1, T.Alignment, WU);
 end;

// tracé des courbes
procedure TExportOdgUtils.TracerCourbe(const C : TCourbe);
var
  i, n: integer;
  QBezierArcs: array of TODGBezierArc;
  AC: TArcCourbe;
  QA : TODGBezierArc;
  QX, QY: double;
  P1, PC1, PC2, P2: TPoint2Df;
  EWE: String;
  function QGetBezierArc(const A: TArcCourbe): TODGBezierArc;
  var
    MonArc: TODGBezierArc;
  begin

    MonArc.P1 := QGetCoordsGCS(A.IDStationP1, C.IDGroupe, A.OffsetP1);
    MonArc.PC1.X  := MonArc.P1.X + A.TangP1.X;
    MonArc.PC1.Y  := MonArc.P1.Y + A.TangP1.Y;
    MonArc.P2 := QGetCoordsGCS(A.IDStationP2, C.IDGroupe, A.OffsetP2);
    MonArc.PC2.X  := MonArc.P2.X + A.TangP2.X;
    MonArc.PC2.Y  := MonArc.P2.Y + A.TangP2.Y;
    AfficherMessage(format('P1 %.2f P.2f, PC1: %.2f, %.2f, PC2: %.2f, %.2f, PT2: %.2f, %.2f',
                            [MonArc.P1.X, MonArc.P1.Y, MonArc.PC1.X, MonArc.PC1.Y,
                             MonArc.P2.X, MonArc.P2.Y, MonArc.PC2.X, MonArc.PC2.Y]));
    // conversion des coordonnées
    ConversionCoordonnees(MonArc.P1.X, MonArc.P1.Y, Result.P1.X, Result.P1.Y);
    ConversionCoordonnees(MonArc.PC1.X, MonArc.PC1.Y, Result.PC1.X, Result.PC1.Y);
    ConversionCoordonnees(MonArc.PC2.X, MonArc.PC2.Y, Result.PC2.X, Result.PC2.Y);
    ConversionCoordonnees(MonArc.P2.X, MonArc.P2.Y, Result.P2.X, Result.P2.Y);
  end;
begin
  n := 1+ high(C.Arcs);
  SetLength(QBezierArcs, n);
  for i := 0 to High(QBezierArcs) do
  begin
    AC := c.Arcs[i];
    QA := QGetBezierArc(AC);
    QBezierArcs[i] := QA;
  end;
  EWE := SelectStyleCourbe(C.IDStyleCourbe);
  FODGContexte.DessinPolyBezier(EWE, QBezierArcs, False, False);
end;

procedure TExportOdgUtils.DessinCenterLines;
var
  i: Integer;
  BP: TBaseStation;
  QX1, QY1, QX2, QY2: double;
begin
  AfficherMessage('---- DessinCenterLines');
  DebutDeGroupe(GROUPE_CENTERLINES);
  for i := 1 to FMyDocDessin.GetNbBaseStations - 1 do
  begin
    BP := FMyDocDessin.GetBaseStation(i);
    if (BP.TypeStation <> 1) then
    begin
      ConversionCoordonnees(BP.PosExtr0.X, BP.PosExtr0.Y, QX1, QY1);
      ConversionCoordonnees(BP.PosStation.X, BP.PosStation.Y, QX2, QY2);
      FODGContexte.DessinLigne(STYLES_CENTERLINES_VISEES, QX1, QY1, QX2, QY2);
      ConversionCoordonnees(BP.PosPD.X, BP.PosPD.Y, QX1, QY1);
      ConversionCoordonnees(BP.PosPG.X, BP.PosPG.Y, QX2, QY2);
      FODGContexte.DessinLigne(STYLES_CENTERLINES_SECTIONS, QX1, QY1, QX2, QY2);
    end;
  end;
  FinDeGroupe(GROUPE_CENTERLINES);
end;
procedure TExportOdgUtils.DessinEntrees;
var
  i: Integer;
  BP: TBaseStation;
  QX1: double;
  QY1: double;
begin
  AfficherMessage('---- DessinEntrees');
  DebutDeGroupe(GROUPE_CENTERLINES_ENTREES);
  for i := 1 to FMyDocDessin.GetNbBaseStations - 1 do
  begin
    BP := FMyDocDessin.GetBaseStation(i);
    if (BP.TypeStation = 1) then
    begin
      ConversionCoordonnees(BP.PosStation.X, BP.PosStation.Y, QX1, QY1);
      FODGContexte.DessinEllipseC(STYLES_CENTERLINES_ENTREES, QX1, QY1, 100, 100);
    end;
  end;
  FinDeGroupe(GROUPE_CENTERLINES);
end;
procedure TExportOdgUtils.DessinerLesCourbes(const LeGroupe: TGroupeEntites);
var
  C: TCourbe;
  i: Integer;
begin
  AfficherMessage('---- DessinCourbes');
  if (FMyDocDessin.NbCourbes = 0) then Exit;
  for i := 0 to FMyDocDessin.NbCourbes - 1 do
  begin
    C := FMyDocDessin.GetCourbe(i);
    if (C.IDGroupe = LeGroupe.IDGroupeEntites) then TracerCourbe(C);
  end;
end;
procedure TExportOdgUtils.DessinerLesPolygones(const LeGroupe: TGroupeEntites);
var
  i: Integer;
  PP: TPolygone;
begin
  AfficherMessage('---- DessinPolygones');
  if (FMyDocDessin.NbPolygones = 0) then Exit;
  for i := 0 to FMyDocDessin.NbPolygones - 1 do
  begin
    PP := FMyDocDessin.GetPolygone(i);
    if (PP.IDGroupe = LeGroupe.IDGroupeEntites) then TracerPolygone(PP);
  end;
end;

procedure TExportOdgUtils.DessinerLesTextes(const LeGroupe: TGroupeEntites);
var
  i: Integer;
  TT: TTextObject;
begin
  AfficherMessage('---- DessinTextes');
  if (FMyDocDessin.NbTextObjects = 0) then Exit;
  for i := 0 to FMyDocDessin.NbTextObjects - 1 do
  begin
    TT := FMyDocDessin.GetTexte(i);
    if (TT.IDStyleTexte = notDEBUG) then Continue; // zap des textes d'aide-mémoire
    if (TT.IDGroupe = LeGroupe.IDGroupeEntites) then TracerTexte(TT);
  end;
end;

procedure TExportOdgUtils.DessinerLesLignes(const LeGroupe: TGroupeEntites);
var
  LL: TSimpleLigne;
  i: Integer;
begin
  AfficherMessage('---- DessinLignes');
  if (FMyDocDessin.NbSimplesLignes = 0) then Exit;
  for i := 0 to FMyDocDessin.NbSimplesLignes - 1 do
  begin
    LL := FMyDocDessin.GetSimpleLigne(i);
    if (LL.IDGroupe = LeGroupe.IDGroupeEntites) then TracerSimpleLigne(LL);
  end;
end;
// dessin des groupes
procedure TExportOdgUtils.DessinerGroupe(const MyGroupe: TGroupeEntites);
begin
  AfficherMessage(Format('%s.DessinerGroupe = %d: %s',[ClassName, MyGroupe.IDGroupeEntites, MyGroupe.NomGroupe]));
  DebutDeGroupe(MyGroupe.NomGroupe);
    DessinerLesPolygones(MyGroupe);
    DessinerLesCourbes(MyGroupe);
    DessinerLesLignes(MyGroupe);
    DessinerLesTextes(MyGroupe);
  FinDeGroupe(MyGroupe.NomGroupe);
end;


// lancer l'export
procedure TExportOdgUtils.LancerExportODG(const Echelle: double);
const
  QNAME_STYLE_NONE  = 'none';
  QNAME_STYLE_SOLID = 'solid';
var
  Gr: Integer;
  QGroupe: TGroupeEntites;
begin
  AfficherMessage(Format('%s.LancerExport: %f', [ClassName, Echelle]));
  FEchelle := Echelle;
  if (FODGContexte.InitialiserFichierContent) then
  begin
    // définition des styles
    FODGContexte.AddUsuallyStyleGraphic('standard', QNAME_STYLE_SOLID, 0.00, clBlack, 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLES_CENTERLINES_VISEES  , QNAME_STYLE_SOLID, 0.00, clMaroon, 100, QNAME_STYLE_SOLID, clWhite, 100);  // style Polygone controle
    FODGContexte.AddUsuallyStyleGraphic(STYLES_CENTERLINES_SECTIONS, QNAME_STYLE_SOLID, 0.00, clSilver, 100, QNAME_STYLE_SOLID, clWhite, 100);  // style Polygone controle
    FODGContexte.AddUsuallyStyleGraphic(STYLES_CENTERLINES_ENTREES , QNAME_STYLE_SOLID, 0.00, clBlack , 100, QNAME_STYLE_SOLID, clGreen, 100);  // style Polygone controle
    // courbes
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_PAROIS         , QNAME_STYLE_SOLID, 0.045, clMaroon   , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_PAROIS_CACHEES , QNAME_STYLE_SOLID, 0.045, clGray     , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_ECOULEMENTS    , QNAME_STYLE_SOLID, 0.070, clBlue     , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_RESSAUTS       , QNAME_STYLE_SOLID, 0.01 , clPurple   , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_MINI_RESSAUTS  , QNAME_STYLE_SOLID, 0.01 , clOlive    , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_PENTES         , QNAME_STYLE_SOLID, 0.00 , clGray     , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_SURPLOMB       , QNAME_STYLE_SOLID, 0.01 , clGreen    , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_COURBE_CHENAL         , QNAME_STYLE_SOLID, 0.01 , clNavy     , 100, QNAME_STYLE_SOLID, clWhite, 100);
    // polygones
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_DEFAUT       , QNAME_STYLE_SOLID, 0.00, clBlack   , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_LAC          , QNAME_STYLE_SOLID, 0.00, clBlue    , 100, QNAME_STYLE_SOLID, clBlue, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_SILHOUETTE   , QNAME_STYLE_SOLID, 0.00, clGray    , 100, QNAME_STYLE_SOLID, clGray, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_ARGILE       , QNAME_STYLE_NONE , 0.00, clBlack   , 100, QNAME_STYLE_SOLID, clMaroon, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_SABLE        , QNAME_STYLE_NONE , 0.00, clBlack   , 100, QNAME_STYLE_SOLID, clOlive, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_BLOCS        , QNAME_STYLE_NONE , 0.00, clBlack   , 100, QNAME_STYLE_SOLID, clSilver, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_GALETS       , QNAME_STYLE_NONE , 0.00, clBlack   , 100, QNAME_STYLE_SOLID, clPurple, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_NEIGE        , QNAME_STYLE_SOLID, 0.00, clBlue    , 100, QNAME_STYLE_SOLID, clAqua, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_GROS_BLOC    , QNAME_STYLE_SOLID, 0.00, clGray    , 100, QNAME_STYLE_SOLID, clTeal, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_GOUR         , QNAME_STYLE_SOLID, 0.00, clMaroon  , 100, QNAME_STYLE_SOLID, clAqua, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_SIPHON       , QNAME_STYLE_NONE , 0.00, clNavy    , 100, QNAME_STYLE_SOLID, clNavy, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_VARVES       , QNAME_STYLE_NONE , 0.00, clMaroon  , 100, QNAME_STYLE_SOLID, clCream, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_POLYGONE_CHEMIN       , QNAME_STYLE_SOLID, 0.00, clRed     , 100, QNAME_STYLE_SOLID, clYellow, 100);
    // lignes
    FODGContexte.AddUsuallyStyleGraphic(STYLE_LIGNE_DEFAULT         , QNAME_STYLE_SOLID, 0.00, clBlack     , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_LIGNE_FLECHE          , QNAME_STYLE_SOLID, 0.00, clBlack     , 100, QNAME_STYLE_SOLID, clBlack, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_LIGNE_SUITE_RESEAU    , QNAME_STYLE_SOLID, 0.00, clNavy      , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_LIGNE_FRACTURE        , QNAME_STYLE_SOLID, 0.00, clRed       , 100, QNAME_STYLE_SOLID, clWhite, 100);
    FODGContexte.AddUsuallyStyleGraphic(STYLE_LIGNE_PENTE           , QNAME_STYLE_SOLID, 0.00, clBlack     , 100, QNAME_STYLE_SOLID, clBlack, 100);

    // textes
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_TITRES      , clWhite , DEFAULT_FONT_NAME , clBlack  , [fsBold, fsUnderline], 12, 0);
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_SOUS_TITRES , clWhite , DEFAULT_FONT_NAME , clGray   , [fsBold], 10.5, 0);
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_COTATION    , clWhite , DEFAULT_FONT_NAME , clBlue   , [fsBold], 8, 0);
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_ORDINAIRE_1 , clWhite , DEFAULT_FONT_NAME , clMaroon , [], 7, 0);
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_ORDINAIRE_1 , clWhite , DEFAULT_FONT_NAME , clMaroon , [], 6, 0);
    FODGContexte.AddUsuallyStyleText(STYLE_TEXTE_DEBUG       , clWhite , DEFAULT_FONT_NAME , clRed    , [], 6, 0);
    //--------------
    FODGContexte.AddUsuallyStyleGraphic('00_PolygoneControle', QNAME_STYLE_SOLID, 0.00, clAqua, 100, QNAME_STYLE_SOLID, clWhite, 100);  // style Polygone controle
    FODGContexte.AddUsuallyStyleGraphic('00_Rectangle', QNAME_STYLE_SOLID, 0.00, clRed, 100, QNAME_STYLE_SOLID, clBlue, 58);            // style Rectangle
    FODGContexte.AddUsuallyStyleGraphic('00_Epures', QNAME_STYLE_SOLID, 0.00, clGray, 100, QNAME_STYLE_SOLID, clWhite, 100);            // style 00_Epures

    FODGContexte.AddUsuallyStyleGraphic('00_CadresPhotos', QNAME_STYLE_SOLID, 0.08, clBlue, 100, QNAME_STYLE_SOLID, clgray, 50);      // styles 00_Ecoulements

    // styles de texte
    FODGContexte.AddUsuallyStyleText('00_Titres', clAqua, DEFAULT_FONT_NAME, clRed, [fsBold, fsUnderline], 14.5, 0);
    FODGContexte.AddUsuallyStyleText('00_TexteBleu', clYellow, DEFAULT_FONT_NAME, clBlue, [fsBold, fsUnderline], 8.4, 0);
    // styles spécifiques aux cartouches
    FODGContexte.AddUsuallyStyleGraphic('00_CartoucheRegle', QNAME_STYLE_SOLID, 0.01, clBlack, 100, QNAME_STYLE_SOLID, clGray, 100);
    FODGContexte.AddUsuallyStyleText('00_CartoucheTexte', clWhite, DEFAULT_FONT_NAME, clBlack, [fsBold, fsItalic], 5.0, 0);
    //--------------------------------------------------------------------------
    // on ouvre une nouvelle page
    FODGContexte.OuvrirNouvellePage('Page001', 'StylePagePrincipale', 'Standard');
    // LES DESSINS ICI
    //=======================================================
    DessinCartouche;
    DessinCenterLines;
    DessinEntrees;
    for Gr := 0 to FMyDocDessin.NbGroupes - 1 do
    begin
      QGroupe := FMyDocDessin.GetGroupe(Gr);
      DessinerGroupe(QGroupe);
    end;
    //=======================================================
    // FIN SECTION DESSINS
    //--------------------------------------------------------------------------
    // on ferme cette page
    FODGContexte.FermerPage('Page001');
    //--------------------------------------------------------------------------
    // Fermer le fichier CONTENT.XML
    FODGContexte.CloturerFichierContent();
    // construction du fichier ODG
    FODGContexte.GenererOdg(FODGFileName);
  end // if (testodg.InitialiserFichierContent) then
  else
  begin

  end;
end;


{$ENDIF}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// lancer l'export
procedure TExportOdgUtils.LancerExportODG(const Echelle: double);
begin
  AfficherMessage(Format('%s.LancerExport: %f', [ClassName, Echelle]));
end;

procedure TExportOdgUtils.Finaliser;
begin
  AfficherMessage(Format('%s.Finaliser', [ClassName]));
  try

  finally
    FODGContexte.Free;
  end;
end;


// dessin des entitées

procedure TExportOdgUtils.DessinCartouche;
  procedure DessinEchelle(const QX, QY, L, H: double);
  const
    SD = 10;
  var
    HL, HH: double;
    FX1, FY1, FX2, FY2: Double;
    i, EWE: Integer;
  begin
    AfficherMessage('-------- DessinEchelle');
    HH := H / 2;
    HL := L / SD;
    DebutDeGroupe(GROUPE_ECHELLE);
      AfficherMessage(Format('-----> %.0f %.0f - %.0f %.0f', [QX, QY, QX + L, QY + H]));
      ConversionCoordonnees(QX, QY, FX1, FY1);
      ConversionCoordonnees(QX + L, QY + H, FX2, FY2);
      FODGContexte.DessinRectangle('00_CartoucheRegle', FX1, FY1, FX2, FY2, False);
      for i := 0 to SD -1 do
      begin
        EWE := IIF(Odd(i), 0, 1);
        ConversionCoordonnees(QX + i     * HL, QY +       EWE * HH, FX1, FY1);
        ConversionCoordonnees(QX + (i+1) * HL, QY + (EWE + 1) * HH, FX2, FY2);
        FODGContexte.DessinRectangle('00_CartoucheRegle', FX1, FY1, FX2, FY2, True);
      end;
    FinDeGroupe(GROUPE_ECHELLE);
  end;
begin
  AfficherMessage('---- DessinCartouche');
  DebutDeGroupe(GROUPE_CARTOUCHE);
    DessinEchelle(FCoordCoin1.X + 0.50, FCoordCoin1.Y + 0.50, 50.00, 02.00);
  FinDeGroupe(GROUPE_CARTOUCHE);
end;


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// conversion de coordonnées
// TODO : Enrichir cette fonction
procedure TExportOdgUtils.ConversionCoordonnees(const QX, QY: double; out CX, CY: double);
var
  FX, FY: Extended;
begin
  FX := QX - FCoordCoin1.X;
  FY := QY - FCoordCoin1.Y;
  CX := FX * FEchelle;
  CY := (FHauteur - FY) * FEchelle;
end;

procedure TExportOdgUtils.DebutDeGroupe(const NomGroupe: string);
begin
  FLeGroupeCourant := NomGroupe;
  FODGContexte.DebutGroupe(NomGroupe);
end;

procedure TExportOdgUtils.FinDeGroupe(const NomGroupe: string);
begin
  if (FLeGroupeCourant <> NomGroupe) then
    AfficherMessageErreur(Format('Mismatch groupe: %s, must be %s', [NomGroupe, FLeGroupeCourant]));
  FODGContexte.FinGroupe(FLeGroupeCourant);
end;


end.


