unit UnitListesSimplesWithGeneriques;
// 25/10/2013: Découplage des listes simples de l'unité ToporobotClasses
// 14/04/2014: Sécurisation des accès aux réseaux, secteurs, entrées et antennes.
//             NE PAS SECURISER L'ACCES AUX EXPES ET CODES (traitement spécifique)
// 11/08/2016: Refactoring +
//             wrappers destinés à éviter les écritures de la forme
//               FTableEntrees  : TListeSimple<TEntrance>;
//             dans le code appelant. On utilise désormais
//               FTableEntrees  : TTableEntrees;
{$INCLUDE CompilationParameters.inc}

interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  Graphics,
  Classes, SysUtils;
// liste générique (fonctionne très bien)
type

{ TListeSimple }

 TListeSimple<T> = class(TFPList)
  private
  public
    procedure ClearListe();
    function  GetNbElements(): integer; inline;
    procedure InsertElement(const Idx: integer; const E: T);
    procedure AddElement(const E: T);  // et Rros Minet
    function  GetElement(const Idx: integer): T; inline;
    procedure PutElement(const Idx: integer; const E: T); inline;
    function  RemoveElement(const Idx: integer): boolean;
end;

// LISTES SIMPLES
// /!\ Laisser comme tel en raison des fonctions additionnelles
//     GetElementByIndex() et ExistsElement() qui s'appellent de la même façon

// wrappers destinés à éviter les écritures de la forme
//    FTableEntrees  : TListeSimple<TEntrance>;
// dans le code appelant. On utilise désormais
//    FTableEntrees  : TTableEntrees;

type TListOfIntegers = class(TListeSimple<integer>)
  private
  public
end;

type TTableEntites = class(TListeSimple<TEntiteEtendue>)
   private
   public
end;

type  TTableEntrees = class(TListeSimple<TEntrance>)
   private
   public
end;

type TTableReseaux = class(TListeSimple<TReseau>)
   private
   public
end;
type TTableSecteurs = class(TListeSimple<TSecteur>)
   private
   public
end;

type TTableAnnotations = class(TListeSimple<TAnnotation>)
   private
   public
end;

type TTableFiltresPersonnalises = class(TListeSimple<TFiltrePersonnalise>)
   private
   public
end;

type TTableIDTerrain = class(TListeSimple<TEtiquetteTerrain>)
   private
   public
end;

type TListeSelectedSeries = class(TListeSimple<Integer>)
  private
  public
end;
type TListePointsTopo = class(TListeSimple<TUneVisee>)
  private
  public
end;


type  TMNTTableVertex    = class(TListeSimple<TVertex>)
  private
  public
end;
type  TMNTTableTriangles = class(TListeSimple<TMNTTriangleABC>)
  private
  public
end;


type TListeAdjacenceSommet = class(TListeSimple<integer>)
  private
  public
end;


// Classe table Branches
// structure Branches (ancienne méthode, suffisante ici)
type TTableBranchesXYZ = class(TListeSimple<TBrancheXYZ>)
  private
  public
end;



// Classe pour la table des jonctions
type TTableJonctionsXYZ = class(TListeSimple<TJonctionXYZ>)
  private
  public
end;
 (*
type TTableJonctionsWithCoords = class(TListeSimple<TJonctionWithCoords>)
  private
  public
end;
//*)



type TListeJonctionsCoupe = class(TListeSimple<TJonctionCoupeDeveloppee>)
  private
  public
end;


type TTableViseesAntenne = class(TListeSimple<TViseeAntenne>)
   private
   public
     function  GetElementByIndex(const Idx: integer): TViseeAntenne;
end;
{$ifdef SUPPORT_NOTES_VERSION}
type TTableNotesVersion = class(TListeSimple<TReleaseNotes>)
   private
   public
end;
{$endif SUPPORT_NOTES_VERSION}
type THelpListeDesSections = class(TListeSimple<THelpFileSection>)
  private
  public
end;



// pour le DistoX
type TListePoints3Df = class(TListeSimple<TPoint3Df>)
  private
  public
end;
type TListeMesuresTopoDistoX = class(TListeSimple<TMesureViseeDistoX>)
  private
  public
end;


//******************************************************************************
// classes enrichies
type TTableCodes = class(TListeSimple<TCode>)
   private
   public
     function  GetElementByIndex(const Idx: TNumeroCode): TCode;
     function  ExistsElement(const Idx: TNumeroCode): boolean;
end;
type TTableExpes = class(TListeSimple<TExpe>)
   private
   public
     function  GetElementByIndex(const Idx: TNumeroExpe): TExpe;
     function  ExistsElement(const Idx: TNumeroExpe): boolean;
end;
type TListePortionsTube = class(TListeSimple<TPortionTubeVisee>)
  private
  public
    procedure SortByDepthField();

end;
type TListePointsOfInterest = class(TListeSimple<TPointOfInterest>)
  private
  public
end;








//******************************************************************************
implementation
{ TListeSimple<T> }
procedure TListeSimple<T>.ClearListe();
var
  i, n: Integer;
begin
  //AfficherMessage(Format('%s.ClearListe()', [classname]));
  n := self.Count;
  if (n > 0) then
  for i:=Count-1 downto 0 Do
  begin
    if (self.Items[i] <> Nil) then Dispose(self.Items[i]); // Libération
    self.Delete(i);                                        // Suppression de l'élément
  end;
end;

function TListeSimple<T>.GetNbElements: integer;
begin
  Result := self.Count;
end;

procedure  TListeSimple<T>.InsertElement(const Idx: integer; const E: T);
var pE: ^T;
begin
  New(pE);
  pE^ := E;
  self.Insert(Idx, pE);
end;
procedure TListeSimple<T>.AddElement(const E: T);
var pE: ^T;
begin
  New(pE);
  pE^ := E;
  self.Add(pE);
end;

function TListeSimple<T>.GetElement(const Idx: integer): T;
begin
  Result := T(Items[Idx]^);
end;



procedure TListeSimple<T>.PutElement(const Idx: integer; const E: T);
begin
  T(Items[Idx]^) := E;
end;

function TListeSimple<T>.RemoveElement(const Idx: integer): boolean;
begin
  Result := False;
  try
    Dispose(self.Items[Idx]);
    self.Delete(Idx);
    Result := True;
  except
  end;
end;



//******************************************************************************
function TTableCodes.GetElementByIndex(const Idx: TNumeroCode): TCode;
var
  ii: integer;
  E : TCode;
begin
  for ii:=0 to self.Count-1 do
  begin
    E := self.GetElement(ii);
    if (E.IDCode = Idx) then exit(E);
  end;
  // si item introuvable, on renvoie l'élément 0
  Result := self.GetElement(0);
end;
function TTableCodes.ExistsElement(const Idx: TNumeroCode): boolean;
var
  i: integer;
  EWE: TCode;
begin
  Result := False;
  for i:= 0 to self.Count - 1 do
  begin
    EWE := self.GetElement(i);
    if (EWE.IDCode = Idx) then Exit(True);
  end;
end;
//******************************************************************************
{ TTableExpes }

function TTableExpes.GetElementByIndex(const Idx: TNumeroExpe): TExpe;
var
  ii: integer;
  E : TExpe;
begin
  for ii:=0 to self.Count-1 do
  begin
    E := self.GetElement(ii);
    if (E.IDExpe = Idx) then exit(E);
  end;
  Result := self.getElement(0);
end;

function TTableExpes.ExistsElement(const Idx: TNumeroExpe): boolean;
var
  i: integer;
  EWE: TExpe;
begin
  Result := False;
  for i:= 0 to self.GetNbElements - 1 do
  begin
    EWE := self.GetElement(i);
    if (EWE.IDExpe = Idx) then Exit(True);
  end;
end;
//******************************************************************************
{ TTableViseesAntenne }

function TTableViseesAntenne.GetElementByIndex(const Idx: integer): TViseeAntenne;
var
  ii: integer;
  E : TViseeAntenne;
begin
  Result := self.GetElement(0); // sécurisation
  if (not IsInRange(Idx, 0, self.Count - 1)) then Exit;
  for ii:=0 to self.Count-1 do
  begin
    E := self.GetElement(ii);
    if (E.IDViseeAntenne = Idx) then Break;
  end;
  Result := E;
end;

//******************************************************************************

// tri des facettes
procedure TListePortionsTube.SortByDepthField();
  procedure QSortByDepth(lidx, ridx: integer);
  var
    k, e, mid: integer;
    Buffer: TPortionTubeVisee;
    bidon1,
    bidon2   : double;
  begin
    with self do
    begin
      if lidx >= ridx then Exit;
      mid := (lidx + ridx) div 2;

      Buffer := self.GetElement(lidx);
      self.PutElement(lidx, self.getElement(mid));
      self.PutElement(mid, Buffer);
      e := lidx;
      for k:=lidx+1 to ridx do
      begin
        if (GetElement(k).DepthField > GetElement(lidx).DepthField) then
        begin
          Inc(e);
          Buffer := GetElement(e);
          PutElement(e, GetElement(k));
          PutElement(k, Buffer);
        end;
      end;
      Buffer := GetElement(lidx);
      PutElement(lidx, GetElement(e));
      Putelement(e, Buffer);
      QSortByDepth(lidx, e-1);
      QSortByDepth(e+1, ridx);
    end;
  end;
begin
  QSortByDepth(1, self.getNbElements() - 1);
end;


end.
