unit UnitObjetSerie;
// Date: 24/08/2012
// Contient l'objet TObjetSerie, utilisé par:
// - ToporobotClasses2012.pas
// - CodeCalculTopo.pas
// - Le cadre Series, nouvelle version
// 06/09/2014: Ajout des membres Accroissements
// 09/05/2016: Cet objet passe aux génériques
{$INCLUDE CompilationParameters.inc}
interface

uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  {$IFDEF USE_GENERIC_TYPES}
  UnitListesSimplesWithGeneriques,
  {$ELSE}
  UnitListesSimplesWithoutGeneriques,
  {$ENDIF}
  Common,
  math,
  Classes, SysUtils, Graphics;
type

 { TObjSerie }
 {$IFDEF USE_GENERIC_TYPES}
 TObjSerie = class(TListeSimple<TUneVisee>)
 {$ELSE}
 TObjSerie = class(TList)
 {$ENDIF}
  private
    FIndexSerie  : TNumeroSerie;
    FSerieDep    : TNumeroSerie;
    FPtDep       : integer;
    FSerieArr    : TNumeroSerie;
    FPtArr       : integer;
    // NOUVEAU: Entrée rattachée à la série
    // Conventions:
    //   0 = Index d'entrée inconnu ou non renseigné
    //  -1 = Hérite du numéro d'entrée de sa série de rattachement
    //  -2 = <TODO>
    FNumeroEntrance : TNumeroEntrance;
    FReseau      : integer; // Réseau
    FChances     : integer;
    FObstacles   : integer;
    FCouleur     : TColor;
    FRaideur     : double;  // module de rigidité, égal à 1.00 par défaut
    FNomSerie    : String;
    FCommentaires: string;
    // accroissements
    FAccrX       : double;
    FAccrY       : double;
    FAccrZ       : double;
    FAccrP       : double;

    {$IFDEF USE_GENERIC_TYPES}
    {$ELSE}
    procedure ClearListe();
    function  GetNbElements(): integer;
    procedure AddElement(const E: TUneVisee);  // et Rros Minet
    function  GetElement(const Idx: integer): TUneVisee;
    procedure PutElement(const Idx: integer; const E: TUneVisee);
    function  RemoveElement(const Idx: integer): boolean;
    {$ENDIF}
  public
    procedure ClearStations();
    function  GetNbVisees(): integer;
    procedure AddVisee(const V: TUneVisee); overload;
    procedure AddVisee(const qNoVisee: integer;
                             const qNoSecteur: integer;
                             const qCode, qExpe: integer;
                             const qTypeVisee: TTypeDeVisee;
                             const qLong, qAz, qP: double;
                             const qLG, qLD, qHZ, qHN: double;
                             const qIDTerrain, qComments: string); overload;
    function  GetVisee(const Idx: integer): TUneVisee;
    function  GetLastVisee(): TUneVisee;
    procedure PutVisee(const Idx: integer; const V: TUneVisee);
    procedure DeleteVisee(const Idx: integer);


    procedure SetNomSerie(const S: string);
    procedure SetObsSerie(const S: string);
    procedure SetNoSerieDep(const N: TNumeroSerie); inline;
    procedure SetNoPointDep(const N: integer); inline;
    procedure SetNoSerieArr(const N: TNumeroSerie); inline;
    procedure SetNoPointArr(const N: integer);
    procedure SetNumeroEntrance(const N: TNumeroEntrance);
    procedure SetNumeroReseau(const N: TNumeroReseau);
    procedure SetChance(const N: integer);
    procedure SetObstacle(const N: integer);
    procedure SetCouleur(const C: TColor);
    procedure SetRaideur(const R: double);

    procedure SetChanceObstacle(const C, O: integer);
    procedure SetNomObsSerie(const N, S: string);
    procedure SetSeriePtArr(const S: TNumeroSerie; const P: integer);
    procedure SetSeriePtDep(const S: TNumeroSerie; const P: integer);
    procedure SetSeriePtExtremites(const Sd: TNumeroSerie; const Pd: integer;
                                   const Sa: TNumeroSerie; const Pa: integer);

    procedure SetNumeroSerie(const N: TNumeroSerie);
    function GetNumeroDeSerie(): TNumeroSerie;
    function GetNomSerie()  : string;
    function GetObsSerie()  : string;
    function GetNoSerieDep(): integer;
    function GetNoPointDep(): integer;
    function GetNoSerieArr(): integer;
    function GetNoPointArr(): integer;
    function GetNumeroEntrance(): TNumeroEntrance;
    function GetNumeroReseau(): TNumeroReseau;
    function GetChance()    : integer;
    function GetObstacle()  : integer;
    function GetCouleur()   : TColor;
    function GetRaideur()   : double;



    // accroissements
    procedure SetAccroissements(const x, y, z, p: Double);
    function GetAccroissementX(): double;
    function GetAccroissementY(): double;
    function GetAccroissementZ(): double;
    function GetAccroissementP(): double;

    // calcul d'erreur jusqu'à la station n
    function CalculIncertitude(const n: integer): boolean;
end;

implementation
{ TObjSerie }

{$IFDEF USE_GENERIC_TYPES}
{$ELSE}
{ TListeSimple<T> }

procedure TOBjSerie.ClearListe();
var
  i: Integer;
begin
  try
    if (self.Count > 0) then for i:= 0 to self.Count - 1 do Dispose(self.Items[i]);
  finally
    self.Clear;
  end;
end;

function TOBjSerie.GetNbElements(): integer;
begin
  Result := self.Count;
end;

procedure TOBjSerie.AddElement(const E: TUneVisee);
var
  pE: ^TUneVisee;
begin
 New(pE);
 pE^ := E;
 self.Add(pE);
end;

function TOBjSerie.GetElement(const Idx: integer): TUneVisee;
var
  pE: ^TUneVisee;
begin
  pE:= self.Items[Idx];
  Result := pE^;
end;



procedure TOBjSerie.PutElement(const Idx: integer; const E: TUneVisee);
var
  pE: ^TUneVisee;
begin
  pE := self.Items[Idx];
  pE^:= E;
  self.Items[Idx] := pE;
end;

function TOBjSerie.RemoveElement(const Idx: integer): boolean;
begin
  Result := False;
  try
    Dispose(self.Items[Idx]);
    self.Delete(Idx);
    Result := True;
  except
  end;
end;
{$ENDIF}

procedure TObjSerie.ClearStations;
var
  ii: Integer;
begin
  FAccrX := 0.00;
  FAccrY := 0.00;
  FAccrZ := 0.00;
  FAccrP := 0.00;
  self.ClearListe();
end;



function TObjSerie.GetNbVisees: integer;
begin
  Result := self.GetNbElements();
end;

procedure TObjSerie.AddVisee(const V: TUneVisee); overload;
begin
  self.AddElement(V);
end;
procedure TObjSerie.AddVisee(const qNoVisee: integer;
                             const qNoSecteur: integer;
                             const qCode, qExpe: integer;
                             const qTypeVisee: TTypeDeVisee;
                             const qLong, qAz, qP: double;
                             const qLG, qLD, qHZ, qHN: double;
                             const qIDTerrain, qComments: string); overload;
var
  WU: TUneVisee;
begin
  //WU.NoVisee     := self.Count - 1;//qNoVisee;
  WU.IDSecteur   := qNoSecteur;
  WU.Code        := qCode;
  WU.Expe        := qExpe;
  WU.TypeVisee   := qTypeVisee;
  WU.Longueur    := qLong;
  WU.Azimut      := qAz;
  WU.Pente       := qP;
  WU.LG          := qLG;
  WU.LD          := qLD;
  WU.HZ          := qHZ;
  WU.HN          := qHN;
  WU.IDTerrainStation := qIDTerrain;
  WU.Commentaires     := qComments;
  WU.DeltaX := 0.00;
  WU.DeltaY := 0.00;
  WU.DeltaZ := 0.00;
  WU.DeltaP := 0.00;
  self.AddVisee(WU);
end;


function TObjSerie.GetVisee(const Idx: integer): TUneVisee;
begin
  Result := self.GetElement(Idx);
end;

function TObjSerie.GetLastVisee: TUneVisee;
var
  n: Integer;
begin
  n := self.GetNbVisees();
  Result := self.GetVisee(n - 1);
end;

procedure TObjSerie.PutVisee(const Idx: integer; const V: TUneVisee);
begin
  self.PutElement(Idx, V);
end;

procedure TObjSerie.DeleteVisee(const Idx: integer);
var
  i, Nb: Integer;
  EWE: TUneVisee;
begin
  self.RemoveElement(Idx);
  // renuméroter les stations
  (*
  Nb := self.GetNbVisees;
  if (Nb > 0) then
  begin
    for i := 0 to Nb -1 do
    begin
      EWE := self.GetVisee(i);
      EWE.NoVisee := i;
      self.PutVisee(i, EWE);
    end;
  end;
  //*)
end;

function  TObjSerie.GetNumeroDeSerie(): TNumeroSerie;
begin
  Result := FIndexSerie;
end;
procedure TObjSerie.SetNumeroSerie(const N: TNumeroSerie);
begin
  FIndexSerie:=N;
end;

procedure TObjSerie.SetNomSerie(const S: string);
begin
  FNomSerie := S;
end;

procedure TObjSerie.SetObsSerie(const S: string);
begin
  FCommentaires := S;
end;

procedure TObjSerie.SetNomObsSerie(const N, S: string);
begin
  FNomSerie := N;
  FCommentaires := S;
end;

procedure TObjSerie.SetNoSerieDep(const N: TNumeroSerie); inline;
begin
  FSerieDep := N;
end;

procedure TObjSerie.SetNoPointDep(const N: integer); inline;
begin
  FPtDep := N;
end;

procedure TObjSerie.SetNoSerieArr(const N: TNumeroSerie); inline;
begin
  FSerieArr := N;
end;

procedure TObjSerie.SetNoPointArr(const N: integer);
begin
  if (N < 0) then FPtArr := self.GetNbVisees() - 1 else
                  FPtArr := N;
end;

procedure TObjSerie.SetNumeroEntrance(const N: TNumeroEntrance);
begin
  FNumeroEntrance := N;
end;

procedure TObjSerie.SetSeriePtDep(const S: TNumeroSerie; const P: integer);
begin
  SetNoSerieDep(S);
  SetNoPointDep(P);
end;
procedure TObjSerie.SetSeriePtArr(const S: TNumeroSerie; const P: integer);
begin
  SetNoSerieArr(S);
  SetNoPointArr(P);
end;
procedure TObjSerie.SetSeriePtExtremites(const Sd: TNumeroSerie; const Pd: integer;
                                         const Sa: TNumeroSerie; const Pa: integer);
begin
  //AfficherMessage(Format('Depart: %d.%d - Arrivee: %d.%d',[Sd, Pd, Sa, Pa]));
  SetNoSerieDep(Sd);
  SetNoPointDep(Pd);
  SetNoSerieArr(Sa);
  SetNoPointArr(Pa);
end;



procedure TObjSerie.SetNumeroReseau(const N: TNumeroReseau);
begin
  FReseau := N;
end;

procedure TObjSerie.SetChance(const N: integer);
begin
  FChances := N;
end;

procedure TObjSerie.SetObstacle(const N: integer);
begin
  FObstacles := N;
end;
procedure TObjSerie.SetChanceObstacle(const C, O: integer);
begin
  FChances := C;
  FObstacles := O;
end;

procedure TObjSerie.SetCouleur(const C: TColor);
begin
  FCouleur := C;
end;

procedure TObjSerie.SetRaideur(const R: double);
const RAIDEUR_MINI = 0.1;
begin
  FRaideur := IIF(abs(R) < RAIDEUR_MINI, RAIDEUR_MINI, abs(R));
end;




function TObjSerie.GetNomSerie: string;
begin
  Result := FNomSerie;
end;

function TObjSerie.GetObsSerie: string;
begin
  Result := FCommentaires;
end;

function TObjSerie.GetNoSerieDep: integer;
begin
  Result := FSerieDep;
end;

function TObjSerie.GetNoPointDep: integer;
begin
  Result := FPtDep;
end;

function TObjSerie.GetNoSerieArr: integer;
begin
  Result := FSerieArr;

end;

function TObjSerie.GetNoPointArr: integer;
begin
  Result := FPtArr;

end;

function TObjSerie.GetNumeroEntrance: TNumeroEntrance;
begin
  Result := FNumeroEntrance;
end;

function TObjSerie.GetNumeroReseau(): TNumeroReseau;
begin
  Result := FReseau;

end;

function TObjSerie.GetChance: integer;
begin
  Result := FChances;

end;

function TObjSerie.GetObstacle: integer;
begin
  Result := FObstacles;
end;

function TObjSerie.GetCouleur: TColor;
begin
  Result := FCouleur;

end;

function TObjSerie.GetRaideur: double;
begin
  Result := FRaideur;
end;



procedure TObjSerie.SetAccroissements(const x, y, z, p: Double);
begin
  FAccrX := x;
  FAccrY := y;
  FAccrZ := z;
  FAccrP := p;
end;

function TObjSerie.GetAccroissementX: double;
begin
  Result := FAccrX;
end;

function TObjSerie.GetAccroissementY: double;
begin
  Result := FAccrY;
end;

function TObjSerie.GetAccroissementZ: double;
begin
  Result := FAccrZ;
end;

function TObjSerie.GetAccroissementP: double;
begin
  Result := FAccrP;
end;
// calcul d'incertitude
function TObjSerie.CalculIncertitude(const n: integer): boolean;
var
  QNbSts, i: Integer;
  QSqrtNbStations: ValReal;
  QIncertitudeRelativeLongueurs : double;
  QIncertitudeRelativeAzimuts   : double;
  QIncertitudeRelativePentes    : double;
  dr , rdtheta, rdphi, dt: double;
  VV: TUneVisee;
begin
  QNbSts := IIF(n = -1, self.GetNbVisees(), n);
  AfficherMessage(format('%s.CalculIncertitude(%d):', [ClassName, QNbSts]));
  result := false;
  QSqrtNbStations := sqrt(QNbSts);
  // valeurs exemple provisoires
  QIncertitudeRelativeLongueurs := 1/1000;
  QIncertitudeRelativeAzimuts   := degtorad(1/360.00);
  QIncertitudeRelativePentes    := gradtorad(1/400.00);
  for i := 1 to QNbSts - 1 do
  begin
    VV := self.GetVisee(i);
    dr       := VV.Longueur * QIncertitudeRelativeLongueurs;
    rdtheta  := VV.Longueur * QIncertitudeRelativeAzimuts;
    rdphi    := VV.Longueur * QIncertitudeRelativePentes;
    dt       := Hypot3D(dr, rdtheta, rdphi);
  end;
end;



end.

