unit UnitSilhouetteGalerie;

{$INCLUDE CompilationParameters.inc}

interface

uses
  StructuresDonnees,
  UnitStructuresDonneesEntites,
  Common,
  {$IFDEF USE_GENERIC_TYPES}
  UnitListesSimplesWithGeneriques,
  {$ELSE}
  UnitListesSimplesWithoutGeneriques,
  {$ENDIF}

  Classes, SysUtils, Graphics;
// silhouette d'une série
type
{ TSilhouetteGalerie }
  {$IFDEF USE_GENERIC_TYPES}
  TSilhouetteGalerie = class(TListeSimple<TPtsSectionTransversale>)
  {$ELSE}
  TSilhouetteGalerie = class(TList)
  {$ENDIF}
  private
    FFillColor: TColor;
    FPenColor: TColor;
    FArrayPolygone: TArrayPoints2Df;
    FNomSilhouette: string;
    constructor Create;
    destructor  Destroy;
    {$IFDEF USE_GENERIC_TYPES}

    {$ELSE}
    procedure AddElement(const E: TPtsSectionTransversale);
    procedure ClearListe;
    function  GetElement(const Idx: integer): TPtsSectionTransversale;
    function  GetNbElements: integer;
    procedure PutElement(const Idx: integer; const E: TPtsSectionTransversale);
    function  RemoveElement(const Idx: integer): boolean;
    {$ENDIF}

  public
    procedure SetNomSilhouette(const N: string);
    function  GetNomSilhouette(): string;
    function  GetFillColor: TColorRGBA;
    procedure ClearSilhouette(const PenColor, FillColor: TColor);
    procedure AddPtsParois(const E: TPtsSectionTransversale);
    function  BuildPolygoneParois(): Boolean;
    function  GetNbPtsParois(): integer;
    function  GetPointPolygoneParois(const Idx: integer): TPoint2Df;
    function  GetNbPointsPolygoneParois(): integer;
    function  GetPtsParois(const Idx: integer): TPtsSectionTransversale;
end;

implementation
{ TSilhouetteGalerie }
{$IFDEF USE_GENERIC_TYPES}

{$ELSE}
procedure TSilhouetteGalerie.ClearListe();
var
  i: Integer;
begin
  try
    if (self.Count > 0) then for i:= 0 to self.Count - 1 do Dispose(self.Items[i]);
  finally
    self.Clear;
  end;
end;

function TSilhouetteGalerie.GetNbElements(): integer;
begin
  Result := self.Count;
end;

procedure TSilhouetteGalerie.AddElement(const E: TPtsSectionTransversale);
var
  pE: ^TPtsSectionTransversale;
begin
 New(pE);
 pE^ := E;
 self.Add(pE);
end;

function TSilhouetteGalerie.GetElement(const Idx: integer): TPtsSectionTransversale;
var
  pE: ^TPtsSectionTransversale;
begin
  pE:= self.Items[Idx];
  Result := pE^;
end;



procedure TSilhouetteGalerie.PutElement(const Idx: integer; const E: TPtsSectionTransversale);
var
  pE: ^TPtsSectionTransversale;
begin
  pE := self.Items[Idx];
  pE^:= E;
  self.Items[Idx] := pE;
end;

function TSilhouetteGalerie.RemoveElement(const Idx: integer): boolean;
begin
  Result := False;
  try
    Dispose(self.Items[Idx]);
    self.Delete(Idx);
    Result := True;
  except
  end;
end;
{$ENDIF}

procedure TSilhouetteGalerie.ClearSilhouette(const PenColor, FillColor: TColor);
var
  i: integer;
begin
  FPenColor := PenColor;
  FFillColor:= FillColor;
  SetLength(FArrayPolygone, 0);
  self.ClearListe();
end;


constructor TSilhouetteGalerie.Create;
begin
  inherited;
  FNomSilhouette := '';
end;

destructor TSilhouetteGalerie.Destroy;
begin
  inherited;
end;

procedure TSilhouetteGalerie.SetNomSilhouette(const N: string);
begin
  FNomSilhouette := N;
end;

function TSilhouetteGalerie.GetNomSilhouette: string;
begin
  Result := FNomSilhouette;
end;

procedure TSilhouetteGalerie.AddPtsParois(const E: TPtsSectionTransversale);
begin
  self.AddElement(E);
end;

function TSilhouetteGalerie.GetPtsParois(const Idx: integer): TPtsSectionTransversale;
begin
  Result := self.GetElement(Idx);
end;

function TSilhouetteGalerie.GetFillColor: TColorRGBA;
begin
  Result.R := Red(FFillColor);
  Result.G := Green(FFillColor);
  Result.B := Blue(FFillColor);
  Result.A := $FF;
end;

function TSilhouetteGalerie.GetNbPtsParois: integer;
begin
  Result := self.GetNbElements();
end;

function TSilhouetteGalerie.GetPointPolygoneParois(const Idx: integer
  ): TPoint2Df;
begin
  Result := FArrayPolygone[Idx];
end;

function TSilhouetteGalerie.GetNbPointsPolygoneParois: integer;
begin
  Result := High(FArrayPolygone) + 1;
end;
function TSilhouetteGalerie.BuildPolygoneParois: Boolean;
var
  i, Nb: integer;
  WU: TPtsSectionTransversale;
  EWE : TGHStringArray;
  TempStrings: TStringList;
begin
  Result := False;
  SetLength(FArrayPolygone, 0);
  // la classique astuce du TStringList ...
  TempStrings := TStringList.Create;
  try
    TempStrings.Clear;
    // paroi droite
    for i:= 0 to self.GetNbPtsParois - 1 do
    begin
     WU := self.GetPtsParois(i);
     TempStrings.Add(Format('%.2f|%.2f', [WU.ParoiDroiteX, WU.ParoiDroiteY]));
    end;
    // paroi gauche = parcours en arrière
    for i:= self.GetNbPtsParois - 1 downto 0 do
    begin
      WU := self.GetPtsParois(i);
      TempStrings.Add(Format('%.2f|%.2f', [WU.ParoiGaucheX, WU.ParoiGaucheY]));
    end;
    // construire le tableau ici
    if (TempStrings.Count > 0) then
    begin
      SetLength(FArrayPolygone, TempStrings.Count);
      for i:= 0 to TempStrings.Count - 1 do
      begin
        EWE := split(TempStrings.Strings[i], '|');
        FArrayPolygone[i].X := ConvertirEnNombreReel(EWE[0], 0.00);
        FArrayPolygone[i].Y := ConvertirEnNombreReel(EWE[1], 0.00);
      end;
      // résultat OK si le polygone contient au moins quatre points
      Result := (High(FArrayPolygone) > 3);
    end;
  finally
    TempStrings.Free;
  end;
end;

end.

