unit unitUtilsComposants;
// Opérations sur des contrôles (listbox, etc...)
// 06/02/2014: Remplacement de l'évaluateur d'expressions
// 25/09/2018: DrawPipistrelle est factorisé
// 11/10/2018: Pas mal de factorisations et corrections
{$INCLUDE CompilationParameters.inc}
interface
uses
  {$INCLUDE SelectionLangues.inc} // insère les unités en fonction de la langue
  StructuresDonnees,
  Common,
  math,
  StdCtrls, Grids, CheckLst,
  Classes, SysUtils, Graphics,
  Forms, Dialogs,
  ConvertisseurJPC,
  BGRABitmap,
  BGRACanvas;

  procedure RemplirCombosSystemesCoordonnees(const FC: TConversionSysteme; const Cmb: TComboBox; const DoUseEPSG: boolean; const IdxOrEPSG: integer);
  procedure SetVoyantMode(const V: TStaticText; const b: boolean); inline;
  procedure SetBtnVoyantMode(const V: TColorButton; const b: boolean); inline;

  procedure RemplirComboBoxCOMPorts(const cmbPorts: TComboBox; const qIdx: integer);

  function  getLastNumLigneDonnees(const Grd: TStringGrid): integer;
  // recherche dans une listbox
  function  FindIdxOccurrenceInListBox(const QListBox: TListBox; const FindWhat: string; out QIdx: integer): boolean;
  function  FindIdxOccurrenceInCheckListBox(const QListBox: TCheckListBox; const FindWhat: string; out QIdx: integer): boolean;


  procedure InitialiserComboBoxQuadrillage(const QCmbx: TComboBox; const QIdx: TQdrType);
  procedure InitialiserComboBoxRepresentationVisees(const QCmbx: TComboBox; const QIdx: TModeRepresentationGaleries);

  procedure PositionnerFenetre(const Wnd: TForm; const X, Y, L, H: integer);
  //procedure DrawBandeauDeuil(const Cnv: TBGRACanvas);
  procedure DrawPipistrelle(const TmpBuffer: TBGRABitmap);

implementation
procedure RemplirCombosSystemesCoordonnees(const FC: TConversionSysteme; const Cmb: TComboBox; const DoUseEPSG: boolean; const IdxOrEPSG: integer);
var
  i: integer;
  EWE: TLabelSystemesCoordsEPSG;
  WU: Integer;
begin
  Cmb.Clear;
  Cmb.DropDownCount := Min(20, FC.GetNbSystemes);
  for i := 0 to FC.GetNbSystemes - 1 do
  begin
    EWE := FC.GetCodeEPSGNomSysteme(i);
    Cmb.Items.Add(Format('EPSG:%d - %s' ,[EWE.CodeEPSG, EWE.NomEPSG]));
  end;
  if (DoUseEPSG) then
  begin
    WU := FC.GetIndexSystemeByCodeEPSG(IdxOrEPSG);
    if (WU > 0) then Cmb.ItemIndex := WU else Cmb.ItemIndex := IdxOrEPSG;
  end
  else
  begin
    Cmb.ItemIndex := IdxOrEPSG;
  end;
end;

procedure SetVoyantMode(const V: TStaticText; const b: boolean); inline;
begin
  V.Color := IIF(b, clLime, clRed);
end;

procedure SetBtnVoyantMode(const V: TColorButton; const b: boolean); inline;
begin
  V.ButtonColor := IIF(b, clLime, clRed);
end;
{$IFDEF MSWINDOWS}
procedure RemplirComboBoxCOMPorts(const cmbPorts: TComboBox; const qIdx: integer);
const
  NbPorts = 64;
var
  i: Integer;
begin
  cmbPorts.Clear;
  cmbPorts.DropDownCount := Min(NbPorts, 8);
  for i := 0 to NbPorts - 1 do cmbPorts.Items.Add(Format('COM%d', [i]));
  cmbPorts.ItemIndex := qIdx;   // pour DistoX
end;
{$ENDIF}
{$IFDEF LINUX}
procedure RemplirComboBoxCOMPorts(const cmbPorts: TComboBox; const qIdx: integer);
var
  i: Integer;
begin
  cmbPorts.Clear;
  cmbPorts.DropDownCount := 8;
  for i := 0 to 15 do cmbPorts.Items.Add(Format('/dev/ttyS%d', [i]));     // ports série
  for i := 0 to 15 do cmbPorts.Items.Add(Format('/dev/rfcomm%d', [i]));   // ports RFCOMM
  cmbPorts.ItemIndex := qIdx;   // pour DistoX
end;
{$ENDIF LINUX}

function getLastNumLigneDonnees(const Grd: TStringGrid): integer;
var
  i, NbL: Integer;
  WU: String;
  function ConcatAllCols(const r: integer): string;
  var
    j, NbC: Integer;
  begin
    Result := '';
    NbC := Grd.ColCount;
    for j := 1 to 3 do Result += Trim(Grd.Cells[j, r]);
  end;
begin
  Result := 0;
  NbL := Grd.RowCount;
  for i := 1 to NbL - 1 do
  begin
    WU  := ConcatAllCols(i);
    if (length(WU) < 3) then Exit(i);
  end;
end;
// recherche dans des TStrings
function FindIdxOccurrenceInTStrings(const QList: TStrings; const FindWhat: string): integer;
var
  S, EWE: String;
  WU, i: LongInt;
  qn: Integer;
  function QFindIndex(const QIdx: integer): integer;
  var
    j: Integer;
    QAT: TGHStringArray;
    n: integer;
  begin
    Result := -1;
    try
      for j := 0 to QList.Count - 1 do
      begin
        QAT := Split(Trim(QList[j]), ':');
        n := StrToIntDef(QAT[0], -1);
        if (n = QIdx) then Exit(j);
      end;
    except
    end;
  end;
begin
  Result := -1;
  if (QList.Count = 0) then Exit;
  S := lowercase(RemoveDiacritiques(FindWhat));
  // recherche d'un signe égal en tête => extraction de l'index
  if (S[1] = '=') then
  begin
    System.Delete(S, 1, 1);
    WU := StrToIntDef(S, 0);
    qn := QFindIndex(WU);
    if (IsInRange(qn, 0, QList.Count - 1)) then Exit(qn);
  end
  else  // recherche sur les items
  begin
    for i := 0 to QList.Count - 1 do
    begin
      EWE := lowercase(RemoveDiacritiques(Trim(QList[i])));
      if (Pos(S, EWE) > 0) then Exit(i);
    end;
  end;
end;

function FindIdxOccurrenceInListBox(const QListBox: TListBox; const FindWhat: string; out QIdx: integer): boolean;
begin
  QIdx := FindIdxOccurrenceInTStrings(QListBox.Items, FindWhat);
  Result := (QIdx >= 0);
end;
// recherche dans une checklistbox
function FindIdxOccurrenceInCheckListBox(const QListBox: TCheckListBox; const FindWhat: string; out QIdx: integer): boolean;
begin
  QIdx := FindIdxOccurrenceInTStrings(QListBox.Items, FindWhat);
  Result := (QIdx >= 0);
end;

procedure InitialiserComboBoxRepresentationVisees(const QCmbx: TComboBox; const QIdx: TModeRepresentationGaleries);
begin
  QCmbx.Style := csDropDownList;
  QCmbx.Clear;
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_NETWORKS));
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_SECTEURS));
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_EXPES));
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_GRAY));
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_DEPTH));
    QCmbx.Items.Add(GetResourceString(rsVUE2D_REPRESENTATION_TYPES));
  QCmbx.ItemIndex := ord(QIdx);
end;

procedure PositionnerFenetre(const Wnd: TForm; const X, Y, L, H: integer);
begin
  Wnd.Left := X;
  Wnd.Top  := Y;
  Wnd.Width := L;
  Wnd.Height := H;
  Wnd.WindowState := wsNormal;
end;

procedure InitialiserComboBoxQuadrillage(const QCmbx: TComboBox; const QIdx: TQdrType);
begin
  QCmbx.Style := csDropDownList;
  QCmbx.Clear;
  QCmbx.Items.add(GetResourceString(rsQDNONE));
  QCmbx.Items.add(GetResourceString(rsQDGRID));
  QCmbx.Items.add(GetResourceString(rsQDCROSS));
  QCmbx.Items.add(GetResourceString(rsQDPOINTS));
  QCmbx.ItemIndex := Ord(QIdx);
end;
(*
procedure DrawBandeauDeuil(const Cnv: TBGRACanvas);
const
  MODULE = 20;
  K1     = 2.50;
  K2     = 4.00;
var
  PP: ARRAY[0..3] of TPoint;
begin
  PP[0].X := round (K1 * MODULE);
  PP[0].Y := 0;
  PP[1].X := 0;
  PP[1].Y := round (K1 * MODULE);
  PP[2].X := 0;
  PP[2].Y := round (K2 * MODULE);
  PP[3].X := round (K2 * MODULE);
  PP[3].Y := 0;
  Cnv.Brush.Color := clBlack;
  Cnv.Brush.Style := bsSolid;
  Cnv.Pen.Color   := clBlack;
  Cnv.Polygon(PP);
end;
//*)

procedure DrawPipistrelle(const TmpBuffer: TBGRABitmap);
const
  UnitLongueur: integer = 2;
var
  i, j: integer;
  TraX, TraY: integer;
  Pipistrelle: array[0..1, 0..13] of TPoint;
begin
  //AfficherMessage(' --> DrawPipistrelle');

  TraX := TmpBuffer.Width - UnitLongueur * 16;
  TraY := TmpBuffer.Height - UnitLongueur * 16;
  for i := 0 to 13 do
  begin
    Pipistrelle[0, i] := MakeTPoint(0, 0);
    Pipistrelle[1, i] := MakeTPoint(0, 0);
  end;
  Pipistrelle[0, 0]  := MakeTPoint(0, 6 * UnitLongueur);
  Pipistrelle[0, 1]  := MakeTPoint(0, -9 * UnitLongueur);
  Pipistrelle[0, 2]  := MakeTPoint(16 * UnitLongueur, -9 * UnitLongueur);
  Pipistrelle[0, 3]  := MakeTPoint(0, -5 * UnitLongueur);
  Pipistrelle[0, 4]  := MakeTPoint(3 * UnitLongueur, -6 * UnitLongueur);
  Pipistrelle[0, 5]  := MakeTPoint(8 * UnitLongueur, -1 * UnitLongueur);
  Pipistrelle[0, 6]  := MakeTPoint(13 * UnitLongueur, -1 * UnitLongueur);
  Pipistrelle[0, 7]  := MakeTPoint(8 * UnitLongueur, 4 * UnitLongueur);
  Pipistrelle[0, 9]  := MakeTPoint(2 * UnitLongueur, 2 * UnitLongueur);
  Pipistrelle[0, 8]  := MakeTPoint(0, 1 * UnitLongueur);
  Pipistrelle[0, 10] := MakeTPoint(4 * UnitLongueur, 8 * UnitLongueur);
  Pipistrelle[0, 11] := MakeTPoint(0, 6 * UnitLongueur);
  Pipistrelle[0, 12] := MakeTPoint(4 * UnitLongueur, -8 * UnitLongueur);
  Pipistrelle[0, 13] := MakeTPoint(5 * UnitLongueur, -8 * UnitLongueur);
  for i := 0 to 13 do
  begin
    Pipistrelle[1, i] := MakeTPoint(-Pipistrelle[0, i].X, Pipistrelle[0, i].Y);
    Pipistrelle[1, i] := MakeTPoint(Pipistrelle[1, i].X + TraX, TmpBuffer.Height - (Pipistrelle[1, i].Y + TraY));
    Pipistrelle[0, i] := MakeTPoint(Pipistrelle[0, i].X + TraX, TmpBuffer.Height - (Pipistrelle[0, i].Y + TraY));
  end;
  with TmpBuffer.CanvasBGRA do
  begin
    Pen.Color := clBlue;
    Pen.Width := 0;
    for j := 0 to 1 do
    begin
      MoveTo(Pipistrelle[j, 0]);
        LineTo(Pipistrelle[j, 1]);
        LineTo(Pipistrelle[j, 2]);
      MoveTo(Pipistrelle[j, 3]);
      for i :=  4 to  8 do LineTo(Pipistrelle[j, i]);
      MoveTo(Pipistrelle[j, 9]);
      for i := 10 to 11 do LineTo(Pipistrelle[j, i]);
      MoveTo(Pipistrelle[j, 4]);
      for i := 12 to 13 do LineTo(Pipistrelle[j, i]);
    end;
  end;
end;
end.

